/*!
  @file     Converter_MapPageNoIterator.hpp
  @author   Henrik
  @ingroup  Converter
  @brief    
*/

/* ========== licence begin  GPL
  Copyright (c) 2000-2004 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end
*/


#ifndef CONVERTER_MAPPAGENOITERATOR_HPP
#define CONVERTER_MAPPAGENOITERATOR_HPP


#include "Container/Container_Vector.hpp"
#include "Converter/Converter_MapEntry.hpp"

///  buffer which contains converter map entries (== the map column)
typedef Container_Vector<Converter_MapEntry> Converter_MapVector;

class Converter_MapPageNoIterator
{

public:

    Converter_MapPageNoIterator(IOMan_DeviceNo volNo, Converter_MapVector &entryVector, SAPDB_Bool iterEnd=SAPDB_FALSE )
    :
    m_VolNo(volNo)
    {
        m_PagesIter = entryVector.Begin();
        m_EndPagesIter = entryVector.End();

        if (iterEnd)
        {
            m_PagesIter = m_EndPagesIter;
        }


        getNextEntry(SAPDB_TRUE);

    }

    Converter_MapPageNoIterator()
    {}

    /*!
      @brief Dereferences the iterator
      @return (IOMan_Blockaddress) current block
    */

    Data_PageNo operator*() const
    {
        return m_PagesIter->GetPage().GetPageNo(m_PageEntriesIter.GetPosition());
    }

    SAPDB_Bool operator==(const Converter_MapPageNoIterator &comp) const
    {
        return (m_PagesIter == comp.m_PagesIter) &&
              ((m_PagesIter->PageIsAssigned() && comp.m_PagesIter->PageIsAssigned()) ?
                         (m_PageEntriesIter == comp.m_PageEntriesIter)
                         : SAPDB_TRUE);
    }

    SAPDB_Bool operator!=(const Converter_MapPageNoIterator &comp) const
    {
        return (m_PagesIter != comp.m_PagesIter) ||
              ((m_PagesIter->PageIsAssigned() && comp.m_PagesIter->PageIsAssigned()) ?
                           (m_PageEntriesIter != comp.m_PageEntriesIter)
                          : SAPDB_FALSE);
    }

    void operator=(const Converter_MapPageNoIterator &source)
    {
        m_VolNo       = source.m_VolNo;
        m_PagesIter    = source.m_PagesIter;
        m_EndPagesIter = source.m_EndPagesIter;
        m_PageEntriesIter     = source.m_PageEntriesIter;
        m_PageEntriesEndIter  = source.m_PageEntriesEndIter;
    }

    /*!
      @brief Prefix increment; advances iterator
      @return (Converter_MapBlockAddressIterator&amp;) this instance
    */
    Converter_MapPageNoIterator& operator++()
    {
        getNextEntry();

        return *this;
    }

    SAPDB_Bool getNextEntry(SAPDB_Bool first = SAPDB_FALSE)
    {
        SAPDB_Bool pageFound = SAPDB_FALSE;

        while((m_PagesIter != m_EndPagesIter) && !pageFound)
        {
            if (m_PagesIter->PageIsAssigned())
            {
                if (first)
                {
                    m_PageEntriesIter = m_PagesIter->GetPage().Begin();
                    m_PageEntriesEndIter = m_PagesIter->GetPage().End();
                }

                pageFound =  getNextEntryOnPage(first);
                first = SAPDB_FALSE;
            }
            else
            {
                m_PageEntriesIter = Converter_PageEntryIterator();
                m_PageEntriesEndIter = Converter_PageEntryIterator();
            }
            if (!pageFound)
            {
                m_PagesIter++;
                first=SAPDB_TRUE;
            }
        }

        return pageFound;
    }

    SAPDB_Bool getNextEntryOnPage(SAPDB_Bool first)
    {
        SAPDB_Bool entryFound = SAPDB_FALSE;

        if (!first && (m_PageEntriesIter != m_PageEntriesEndIter))
        {
            ++m_PageEntriesIter;
        }

        while ((m_PageEntriesIter != m_PageEntriesEndIter) &&
               (!m_PageEntriesIter.EntryInUse() ||
                 m_PageEntriesIter.GetPackedBlockAddress().IsReserved() ||
                (m_PageEntriesIter.GetBlockAddress().GetDeviceNo() != m_VolNo)))
        {
            ++m_PageEntriesIter;
        }

        if (m_PageEntriesIter != m_PageEntriesEndIter)
        {
            entryFound = SAPDB_TRUE;
        }

        return entryFound;
    }
        

private:

    IOMan_DeviceNo m_VolNo;
    Converter_MapVector::Iterator       m_PagesIter;
    Converter_MapVector::Iterator       m_EndPagesIter;

    Converter_PageEntryIterator m_PageEntriesIter;
    Converter_PageEntryIterator m_PageEntriesEndIter;

};

#endif // CONVERTER_MAPPAGENOITERATOR_HPP
