/*!
  \file    DBMSrvCmd_SilentDispatcherUsingCommand.cpp
  \author  MarcW
  \ingroup DBMServer commands
  \brief   base class for event administration commands

    ========== licence begin  GPL
    Copyright (c) 2002-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*/

#include "DBM/Srv/Commands/DBMSrvCmd_EventAdministrationCommand.hpp"
#include "DBM/Srv/Message/DBMSrvMsg_Error.hpp"
#include "DBM/Srv/SharedMemory/DBMSrvShM_LineAccessorResourceLock.hpp"
#include "DBM/Srv/SharedMemory/DBMSrvShM_InternalResource.hpp"


DBMSrvCmd_EventAdministrationCommand::DBMSrvCmd_EventAdministrationCommand(
        CommandKey commandKey,
        bool isObsolete,
        const char* aLongHelp,
        bool isHidden)
        : DBMSrvCmd_ShmUsingCommand( commandKey, isObsolete, aLongHelp, isHidden ) {
}

tcn00_Error DBMSrvCmd_EventAdministrationCommand::runUsingSharedMemory(
                    VControlDataT* vcontrol,
                    CommandT* command,
                    DBMSrv_Reply& theReply) {
    tcn00_Error rc(OK_CN00);
    theReply.startWithOK();

    // get resource lock
    DBMSrvShM_LineAccessorResourceLock eventAdminLock(
            DBMSrvShM_InternalResource::EventAdministration);
    
    if( !eventAdminLock.lock() ) {
        DBMSrvMsg_Error errEventAdminLock(
            DBMSrvMsg_Error::DBMSrv_SHMRESLOCK, 
            Msg_Arg("DBMSRV_COMP", eventAdminLock.getResourceName()));
        return theReply.startWithMessageList(errEventAdminLock);
    }

    // get kernel parameter access
    DBMSrvPar_Parameters* kernelParameters = DBMSrvPar_Parameters::GetInstance(vcontrol->dbname, true, theReply);
    if( kernelParameters == NULL ) {
        eventAdminLock.unlock();
        // no need to provide an additional comment...
        return theReply.ReturnCode();
    }

    // let derived class do the work
    rc = runEventAdministrationCommand(vcontrol, kernelParameters, command, theReply);

    // release kernel parameter access
    delete kernelParameters;

    // release resource lock
    eventAdminLock.unlock();

    return rc;
}
