/*!
  \file    DBMSrvCmd_ShmUsingCommand.cpp
  \author  MarcW
  \ingroup DBMServer commands
  \brief   base class for commands that use shared memory

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (C) 2002-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/

#include "DBM/Logging/DBMLog_Logger.hpp"
#include "DBM/Srv/Commands/DBMSrvCmd_ShmUsingCommand.hpp"
#include "DBM/Srv/Message/DBMSrvMsg_Error.hpp"
#include "DBM/Srv/SharedMemory/DBMSrvShM_ShmAdmin.hpp"

int DBMSrvCmd_ShmUsingCommand::m_NestingLevel = 0;

DBMSrvCmd_ShmUsingCommand::DBMSrvCmd_ShmUsingCommand(
        CommandKey commandKey,
        bool isObsolete,
        const char* longHelp,
        bool isHidden )
: DBMSrv_Command( commandKey, isObsolete, longHelp, isHidden ) {
    ++m_NestingLevel;
    m_ShmAvailable = DBMSrvShM_ShmAdmin::Instance().isValid() && DBMSrvShM_ShmAdmin::Instance().shmVersionOkay();
}

DBMSrvCmd_ShmUsingCommand::~DBMSrvCmd_ShmUsingCommand() {
    --m_NestingLevel;
    if( m_NestingLevel == 0 )
        DBMSrvShM_ShmAdmin::deleteInstance();
}

tcn00_Error DBMSrvCmd_ShmUsingCommand::run(VControlDataT * vcontrol,
                CommandT      * command,
                DBMSrv_Reply& theReply) {
    if( m_ShmAvailable ) {
        return runUsingSharedMemory(vcontrol, command, theReply);
    }
    else {
        DBMSrvMsg_Error errNoShM(DBMSrvMsg_Error::DBMSrv_SHMNOTAVAILABLE);
        DBMLog_Logger::getInstance().addMessageList(errNoShM, true);
        return theReply.startWithMessageList(errNoShM);
    }
}
