/*!
  \file    DBMSrvCmd_ShmUsingCommand.hpp
  \author  MarcW
  \ingroup DBMServer commands
  \brief   base class for commands that use shared memory

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (C) 2002-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/

#include "DBM/Srv/DBMSrv_Command.hpp"

#if !defined(DBMSrvCmd_ShmUsingCommand_hpp)
#define DBMSrvCmd_ShmUsingCommand_hpp


/*! 
    @brief abstract base class for commands that use shared memory

    This class attaches to shared memory in the constuctor and detaches in the
    destructor. Subclasses don't need to care about this (and it cannot be
    forgotten).
*/
class DBMSrvCmd_ShmUsingCommand: public DBMSrv_Command {
public:
    /*!
        @brief constructor

        Attach to shared memory.
        @param commandKey [IN] this command's key
        @param isObsolete [IN] flag whether this command is obsolete
        @see DBMSrv_Command
    */
    DBMSrvCmd_ShmUsingCommand( CommandKey commandKey, bool isObsolete, const char* longHelp=0, bool isHidden=false );

    tcn00_Error run(VControlDataT * vcontrol,
                    CommandT      * command,
                    DBMSrv_Reply& theReply);

    /*!
        @brief destructor

        Detach from shared memory.
    */
    virtual ~DBMSrvCmd_ShmUsingCommand();

protected:
    // to be overwriteen by all subclasses
    virtual tcn00_Error runUsingSharedMemory(VControlDataT * vcontrol,
                    CommandT      * command,
                    DBMSrv_Reply& theReply) = 0;

    bool m_ShmAvailable;
private:
    /*! @brief if a command is called from within another command, it must not detach! */
    static int m_NestingLevel;
};
#endif //DBMSrvCmd_ShmUsingCommand_hpp

