/*!
  \file    DBMSrvCmdSQL_SqlUpdatestatPerSystemtable.cpp
  \author  marcW
  \ingroup DBMServer commands
  \brief   

    ========== licence begin  GPL
    Copyright (C) 2002-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*/

#include "DBM/Srv/Commands/SQL/DBMSrvCmdSQL_SqlUpdatestatPerSystemtable.hpp"
#include "DBM/Srv/KernelAccess/DBMSrvKnl_Session.hpp"
#include "DBM/Srv/Message/DBMSrvMsg_Error.hpp"
#include "ToolsCommon/Tools_DynamicUTF8String.hpp"

#include "hcn50.h"

DBMSrvCmdSQL_SqlUpdatestatPerSystemtable::DBMSrvCmdSQL_SqlUpdatestatPerSystemtable()
    : DBMSrv_Command(
            DBMSrv_Command::KeyUsingCommandFactory,
            false,
            DBMSrvCmdSQL_SqlUpdatestatPerSystemtable::m_LongHelp ) {
}

const char* const DBMSrvCmdSQL_SqlUpdatestatPerSystemtable::m_LongHelp=
            "@command "DBMSrvCmdSQL_SqlUpdatestatPerSystemtable_CMD" update of optimizer statistics "
                "for those tables, that need an update of their statistics"
            "@syntax "DBMSrvCmdSQL_SqlUpdatestatPerSystemtable_CMD;

const char* DBMSrvCmdSQL_SqlUpdatestatPerSystemtable::getCommandName() {
    return DBMSrvCmdSQL_SqlUpdatestatPerSystemtable_CMD;
}

const char* const DBMSrvCmdSQL_SqlUpdatestatPerSystemtable::SQL_UPDATESTAT_SMART("UPDATE STAT AS PER SYSTEM TABLE");

tcn00_Error DBMSrvCmdSQL_SqlUpdatestatPerSystemtable::run(
        VControlDataT* vcontrol,
        CommandT* command,
        DBMSrv_Reply& theReply) {

    tcn00_Error rc(OK_CN00);
    theReply.startWithOK();

    // build execute kernel command
    cn50DBMUser superUser(vcontrol->dbname, cn50DBMUser::getColdUser(vcontrol->dbname));
    DBMSrvMsg_Error errOut;
    if( !DBMSrvKnl_Session::Execute(
            vcontrol->dbname.asCharp(),
            superUser.getUserName().asCharp(),
            superUser.getClearMasterPwd().asCharp(),
            SQL_UPDATESTAT_SMART,
            errOut) )
        rc = theReply.startWithMessageList(errOut);

    return rc;
}
