/*!
    @file           DBMSrvShM_LineAccessorEventingSilent.cpp
    @author         MarcW
    @brief          

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (c) 2003-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/

#include "DBM/Srv/SharedMemory/DBMSrvShM_LineAccessorEventingSilent.hpp"

DBMSrvShM_LineAccessorEventingSilent::DBMSrvShM_LineAccessorEventingSilent()
    : DBMSrvShM_LineAccessorBase() {
    if( m_ShmAdmin.lock() ) {
        DBMSrvShM_LineLayoutEventing* le = m_ShmAdmin.getLineLayoutEventingSilent(false);
        m_Exists = (le != NULL);
        if( exists() )
            setIndex( le->getIndex() );
        m_ShmAdmin.unlock();    
    }
}

bool DBMSrvShM_LineAccessorEventingSilent::removeLineEventingSilent() {
    if( !m_ShmAdmin.lock() )
        return false;

    m_ShmAdmin.removeLineLayoutEventingSilent();
    m_Exists = false;    
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorEventingSilent::setBGSRVPid() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutEventing* le = m_ShmAdmin.getLineLayoutEventingSilent(true);
    m_Exists = (le != NULL);
    if( exists() ) {
        le->setBGSRVPid(RTESys_ProcessID());
    }
    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorEventingSilent::getBGSRVPid(RTE_OSPid& pid) {
    DBMSrvShM_LineLayoutEventing* le(NULL);

    if( !m_ShmAdmin.lock() )
       return false;

    le = m_ShmAdmin.getLineLayoutEventingSilent(false);
    m_Exists = (le != NULL);

    if( exists() ) {
        pid = le->getBGSRVPid();
    }

    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorEventingSilent::removeBGSRVPid() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutEventing* le = m_ShmAdmin.getLineLayoutEventingSilent(true);
    m_Exists = (le != NULL);
    if( exists() ) {
        le->removeBGSRVPid();
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorEventingSilent::doCancel() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutEventing* le = m_ShmAdmin.getLineLayoutEventingSilent(true);
    m_Exists = (le != NULL);    
    if( exists() ) {
        le->doCancel();
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorEventingSilent::unCancel() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutEventing* le = m_ShmAdmin.getLineLayoutEventingSilent(true);
    m_Exists = (le != NULL);    
    if( exists() ) {
        le->unCancel();
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorEventingSilent::isCancelled(bool& iscanc) {
    DBMSrvShM_LineLayoutEventing* le(NULL);

    if( !m_ShmAdmin.lock() )
        return false;

    le = m_ShmAdmin.getLineLayoutEventingSilent(false);
    m_Exists = (le != NULL);

    iscanc = !exists() || le->isCancelled();

    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorEventingSilent::setConfigurationStatus( bool isDirty ) {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutEventing* le = m_ShmAdmin.getLineLayoutEventingSilent(true);
    m_Exists = (le != NULL);    
    if( exists() )
        //le->setConfigurationStatus(isDirty);

    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorEventingSilent::getConfigurationStatus( bool& isDirty ) {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutEventing* le = m_ShmAdmin.getLineLayoutEventingSilent(true);
    m_Exists = (le != NULL);    
    if( exists() )
        //isDirty = le->getConfigurationStatus();

    m_ShmAdmin.unlock();
    return exists();
}
