/*!
    @file           DBMSrvShM_LineAccessorEventingSilent.hpp
    @author         MarcW
    @brief          line accessor class for lines of type silent eventing

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (c) 2003-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/

#include "SAPDB/DBM/Srv/SharedMemory/DBMSrvShM_LineAccessorBase.hpp"

#if !defined(DBMSrvShM_LineAccessorEventingSilent_HPP)
#define DBMSrvShM_LineAccessorEventingSilent_HPP

/*!
    @brief line accessor class for lines of type automatic eventing

    For some DBMServer commands, eventing is used "silently". This silent eventing
    has its own event dispatcher process with its own configuration, that is not
    maintained manually. Some DBMServer commands (e.g. "auto_exend ON")  use this
    silent dispatcher and modify its configuration.
*/
class DBMSrvShM_LineAccessorEventingSilent : public DBMSrvShM_LineAccessorBase {

public:
    /*!
        @brief create an accessor for the silent eventing

        At maximum, one line for silent eventing may exist. If the line does not exist,
        it is created.
    */
    DBMSrvShM_LineAccessorEventingSilent();

    bool removeLineEventingSilent();

    /*! @brief set pid of background process */
    bool setBGSRVPid();

    /*!
        @brief get background process' id from layout

        @return background process' id from layout
    */
    bool getBGSRVPid(RTE_OSPid& pid);

    /*! @brief remove any process id from layout */
    bool removeBGSRVPid();

    /*! @brief set cancel flag */
    bool doCancel();

    /*! @brief unset cancel flag */
    bool unCancel();

    /*!
        @brief check cancel flag

        @return true is cancel flag is set
    */
    bool isCancelled(bool& iscanc);

    // to make dispatcher reread configuration
    bool setConfigurationStatus( bool isDirty );
    // check the status
    bool getConfigurationStatus( bool& isDirty );
};

#endif //DBMSrvShM_LineAccessorEventingSilent_HPP
