/*!
    @file           DBMSrvShM_LineAccessorResourceLock.cpp
    @author         MarcW
    @brief          definition of class for accessing resource locks

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (c) 2003-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/

#include "SAPDB/DBM/Srv/SharedMemory/DBMSrvShM_LineAccessorResourceLock.hpp"
#include "SAPDB/DBM/Srv/SharedMemory/DBMSrvShM_LineLayoutResourceLock.hpp"
#include "SAPDBCommon/SAPDB_string.h"

DBMSrvShM_LineAccessorResourceLock::DBMSrvShM_LineAccessorResourceLock(const char* resourceName)
        : DBMSrvShM_LineAccessorBase(),
          m_MyPid(RTESys_ProcessID()),
          m_ResourceName(new char[strlen(resourceName)+1]) {

    SAPDB_strcpy(m_ResourceName, resourceName);
}

DBMSrvShM_LineAccessorResourceLock::~DBMSrvShM_LineAccessorResourceLock() {

    if( m_ShmAdmin.lock() ) {
        DBMSrvShM_LineLayoutResourceLock* lrl = m_ShmAdmin.getLineLayoutResourceLock(m_ResourceName);
        if( lrl != NULL && lrl->lockResource(m_MyPid) )
            m_ShmAdmin.removeLineLayoutResourceLock(m_ResourceName);

        m_ShmAdmin.unlock();
    }
    delete[] m_ResourceName;
}

bool DBMSrvShM_LineAccessorResourceLock::lock() {
    bool gotLock(false);

    int count(25); // 25 * 200ms = 5 seconds until we give up
    while( !gotLock && count>0 ) {
        if( m_ShmAdmin.lock() ) {
            DBMSrvShM_LineLayoutResourceLock* lrl =
                        m_ShmAdmin.getLineLayoutResourceLock(m_ResourceName);
            gotLock = (lrl != NULL) && lrl->lockResource(m_MyPid);
            m_ShmAdmin.unlock();
        }
        if( gotLock )
            break;
        else {
            RTE_ISystem::DoSleep(200);
            --count;
        }
    }
    return gotLock;
}

bool DBMSrvShM_LineAccessorResourceLock::unlock() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutResourceLock* lrl = m_ShmAdmin.getLineLayoutResourceLock(m_ResourceName);
    bool releasedLock( lrl != NULL && lrl->unlockResource(m_MyPid) );

    m_ShmAdmin.unlock();
    return releasedLock;
}
