/*!
    @file           DBMSrvShM_LineAccessorScheduler.cpp
    @author         MarcW
    @brief          

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (C) 2003 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/

#include "DBM/Srv/SharedMemory/DBMSrvShM_LineAccessorScheduler.hpp"
#include "RunTime/RTE_ISystem.hpp"
#include "SAPDBCommon/SAPDB_string.h"

DBMSrvShM_LineAccessorScheduler::DBMSrvShM_LineAccessorScheduler()
    : DBMSrvShM_LineAccessorBase() {}


bool DBMSrvShM_LineAccessorScheduler::removeLayout() {
    if( !m_ShmAdmin.lock() )
        return false;

    m_ShmAdmin.removeLineLayoutScheduler();
    m_Exists = false;    
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorScheduler::setSchedulerPid() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);
    if( exists() ) {
        le->setBGSRVPid(RTESys_ProcessID());
    }
    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::getSchedulerPid(RTE_OSPid& pid) {
    DBMSrvShM_LineLayoutScheduler* le(NULL);

    if( !m_ShmAdmin.lock() )
       return false;

    le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);

    if( exists() ) {
        pid = le->getBGSRVPid();
    }

    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::removeSchedulerPid() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);
    if( exists() ) {
        le->removeBGSRVPid();
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorScheduler::doCancel() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);    
    if( exists() ) {
        le->doCancel();
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorScheduler::unCancel() {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);    
    if( exists() ) {
        le->unCancel();
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorScheduler::isCancelled(bool& iscanc) {
    DBMSrvShM_LineLayoutScheduler* le(NULL);

    if( !m_ShmAdmin.lock() )
        return false;

    le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);

    iscanc = !exists() || le->isCancelled();

    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::clearContent() {
    bool setIT = setInfoType(IT_None);
    bool setVP = setVariablePart("");
    bool setUN = setUserName("");
    bool setR = setRecipient(Rec_None);
    return setIT && setVP && setUN && setR;
}

bool DBMSrvShM_LineAccessorScheduler::getInfoType(InfoType& aInfoType) {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);    
    if( exists() )
        aInfoType = (InfoType)le->getInfoType();
    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::setInfoType(InfoType aInfoType) {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);    
    if( exists() )
        le->setInfoType( aInfoType );
    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::getUserName(char*& aUserName) {
    DBMSrvShM_LineLayoutScheduler* le(NULL);

    if( !m_ShmAdmin.lock() )
        return false;

    le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);

    if( exists() ) {
        size_t len = le->getUserNameLen();
        aUserName = new char[len+1];
        SAPDB_memcpy(aUserName, le->getUserName(), len);
        aUserName[len] = '\0';
    }

    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::setUserName(const char* aUserName) {
    DBMSrvShM_LineLayoutScheduler* le(NULL);

    if( !m_ShmAdmin.lock() )
        return false;

    le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);
    if( exists() ) {
        SAPDB_Byte* start = le->getUserName();
        size_t oldLen = le->getUserNameLen();
        size_t newLen = strlen(aUserName);
        m_ShmAdmin.shift(DBMSrvShM_ShmAdmin::LineTypeScheduler, start, (int)newLen - (int)oldLen);
        le = m_ShmAdmin.getLineLayoutScheduler(false);
        m_Exists = (le != NULL);
        le->setUserName(aUserName);
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorScheduler::getVariablePart(char*& aVarPart) {
    DBMSrvShM_LineLayoutScheduler* le(NULL);

    if( !m_ShmAdmin.lock() )
        return false;

    le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);

    if( exists() ) {
        size_t len = le->getVariablePartLen();
        aVarPart = new char[len+1];
        SAPDB_memcpy(aVarPart, le->getVariablePart(), len);
        aVarPart[len] = '\0';
    }

    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::setVariablePart(const char* aVarPart) {
    DBMSrvShM_LineLayoutScheduler* le(NULL);

    if( !m_ShmAdmin.lock() )
        return false;

    le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);
    if( exists() ) {
        SAPDB_Byte* start = le->getVariablePart();
        size_t oldLen = le->getVariablePartLen();
        size_t newLen = strlen(aVarPart);
        m_ShmAdmin.shift(DBMSrvShM_ShmAdmin::LineTypeScheduler, start, (int)newLen - (int)oldLen);
        le = m_ShmAdmin.getLineLayoutScheduler(false);
        m_Exists = (le != NULL);
        le->setVariablePart(aVarPart);
    }
    m_ShmAdmin.unlock();
    return true;
}

bool DBMSrvShM_LineAccessorScheduler::setRecipient( Recipient aRecipient) {
    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);    
    if( exists() ) {
        le->setRecipient(aRecipient);
    }
    m_ShmAdmin.unlock();
    return exists();
}

bool DBMSrvShM_LineAccessorScheduler::recipientCheck( Recipient aRecipient) {
    Recipient currentRec(Rec_None);

    if( !m_ShmAdmin.lock() )
        return false;

    DBMSrvShM_LineLayoutScheduler* le = m_ShmAdmin.getLineLayoutScheduler(true);
    m_Exists = (le != NULL);
    if( exists() ) 
        currentRec = (Recipient)le->getRecipient();

    m_ShmAdmin.unlock();
    return (currentRec == aRecipient);
}

bool DBMSrvShM_LineAccessorScheduler::recipientWait( Recipient aRecipient) {
    int i=0;
    bool messageForRecipientAvailable(false);

    while( i<1000 && !messageForRecipientAvailable ) {
        messageForRecipientAvailable = recipientCheck( aRecipient );
        i++;
        RTE_ISystem::DoSleep(300);
    }

    return messageForRecipientAvailable;
}
