/*!
    @file           DBMSrvShM_LineAccessorSchduler.hpp
    @author         MarcW
    @brief          line accessor class for the one and only scheduler line

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (C) 2003 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/

#include "SAPDB/DBM/Srv/SharedMemory/DBMSrvShM_LineAccessorBase.hpp"

#if !defined(DBMSrvShM_LineAccessorScheduler_HPP)
#define DBMSrvShM_LineAccessorScheduler_HPP

/*! @brief accessor for lines of type scheduler */
class DBMSrvShM_LineAccessorScheduler : public DBMSrvShM_LineAccessorBase {
public:
    enum InfoType {
        IT_None,
        IT_JobID,
        IT_SerializedJobForCreation,
        IT_SerializedJobList,
        IT_SerializedJobForUpdate,
        IT_RequestJobList,
        IT_ActivateJob,
        IT_DeactivateJob,
        IT_DeleteJob,
        IT_ErrorWrongUser,
        IT_ErrorJobHasSuccessors,
        IT_ErrorJobNotExist,
        IT_ErrorPredecessorNotExist
    };

    DBMSrvShM_LineAccessorScheduler();
    ~DBMSrvShM_LineAccessorScheduler() {}
    bool removeLayout(); // called from scheduler when it stops...

    bool setSchedulerPid();
    bool getSchedulerPid(RTE_OSPid& pid);
    bool removeSchedulerPid();

    bool doCancel();
    bool unCancel();
    bool isCancelled(bool& iscanc);

    bool clearContent();
    bool setRecipientClient() {return setRecipient(Rec_Client);}
    bool setRecipientScheduler() {return setRecipient(Rec_Scheduler);}
    bool clientWait() {return recipientWait(Rec_Client);}
    bool isSchedulerCommand() {return recipientCheck(Rec_Scheduler);}


    bool getInfoType(InfoType& aInfoType);
    bool setInfoType(InfoType aInfoType);

    bool getUserName(char*& aUserName);
    bool setUserName(const char* aUserName);

    bool getVariablePart(char*& aVarPart);
    bool setVariablePart(const char* aVarPart);

private:
    enum Recipient {
        Rec_None,
        Rec_Client,
        Rec_Scheduler
    };

    bool setRecipient( Recipient aRecipient);
    bool recipientWait( Recipient aRecipient );
    bool recipientCheck( Recipient aRecipient );
};

#endif //DBMSrvShM_LineAccessorScheduler_HPP
