/*****************************************************************************/
/*!
  @file         Data_PrimTreeStatisticsCoordinator.hpp
  @author       MartinKi
  @ingroup

  @brief        Header file for class Data_PrimTreeStatisticsCoordinator.

\if EMIT_LICENCE
  ========== licence begin  GPL
  Copyright (C) 2003 SAP AG

  This program is free software; you can redisytribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end
\endif
*/
/*****************************************************************************/

#ifndef DATA_PRIMTREESTATISTICSCOORDINATOR_HPP
#define DATA_PRIMTREESTATISTICSCOORDINATOR_HPP

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "DataAccess/Data_UpdStatRequestFile.hpp"
#include "DataAccess/Data_Types.hpp"
#include "SAPDBCommon/SAPDB_Types.hpp"

#include "ggg00.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

class Msg_List;
class SrvTasks_JobCollectTableStatistics;

// the following classes are locally defined in
// Data_PrimTreeStatisticsCoordinator:
class JobQueue;
class JobInfo;

/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!
 * @class  Data_PrimTreeStatisticsCoordinator
 * @author MartinKi
 *
 * @brief Coordinates the computation of a set of table statistics
 *        given in a request file.
 *
 */
class Data_PrimTreeStatisticsCoordinator
{
public:
    /// Possible return codes.
    enum ReturnCode {
        Okay,
        Cancelled,
        SystemError
    };

    /*!
     * @brief Constructor.
     */
    Data_PrimTreeStatisticsCoordinator(
        tgg00_TransContext&      transContext,
        Data_UpdStatRequestFile& updStatRequestFile);

    /*!
     * @brief Destructor.
     */
    ~Data_PrimTreeStatisticsCoordinator();


    /*!
     * @brief Processes all requests.
     */
    ReturnCode UpdateStatistics(
        Msg_List& errMsg);

private:
    // no copy-construction of assignment
    Data_PrimTreeStatisticsCoordinator(
        const Data_PrimTreeStatisticsCoordinator&);
    Data_PrimTreeStatisticsCoordinator operator=(
        const Data_PrimTreeStatisticsCoordinator&);

    friend class SrvTasks_JobCollectTableStatistics;
    friend class JobInfo;
    friend class JobQueue;

    typedef Data_UpdStatRequestFile::TableKey TableKey;

    void BuildTempFileId(
        tgg00_FileId& fileId,
        SAPDB_Int4    currFileIndex);

    ReturnCode SelfProcessJob(
        JobQueue& jobQueue,
        JobInfo&  job,
        Msg_List& errMsg);

    ReturnCode ProcessResult(
        JobQueue&                           jobQueue,
        SrvTasks_JobCollectTableStatistics& job,
        Msg_List&                           errMsg);

    tgg00_TransContext&      m_trans;
    Data_UpdStatRequestFile& m_updStatRequestFile;
};

#endif // DATA_PRIMTREESTATISTICSCOORDINATOR_HPP
