/*!
  @file           FBM_IDataVolume.hpp
  @author         Henrik
  @ingroup        FBM

\if EMIT_LICENCE
    ========== licence begin  GPL
    Copyright (c) 2002-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end
\endif
*/

#ifndef FBM_IDATAVOLUME_H
#define FBM_IDATAVOLUME_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "FreeBlockManagement/FBM_IVolume.hpp"

/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!
  @class          FBM_IDataVolume
  @brief          representation of one data volume
 */
class FBM_IDataVolume : public FBM_IVolume
{
public:
    virtual IOMan_BlockCount GetNumBlocksToAddUntilOptIsReached() const = 0;
    virtual void SetNumBlocksToAddUntilOptIsReached(IOMan_BlockCount) = 0;

    virtual IOMan_BlockNo GetActBlockNoForBackup() const = 0;            
    virtual void SetActBlockNoForBackup(IOMan_BlockNo) = 0;            

    virtual IOMan_BlockCount GetNumBlocksMarkedForBackup() const = 0;
    virtual IOMan_BlockCount GetNumBlocksFreeAfterSVP() const = 0;
    virtual IOMan_BlockCount GetNumBlocksUsed() const = 0;
    virtual IOMan_BlockCount GetNumBlocks() const = 0;

    virtual void  PrepareForDrop() = 0;
    virtual void  RestoreForDrop() = 0;
    virtual SAPDB_Bool  IsToDrop() = 0;

    virtual FBM_BlockState GetBlockStateAfterBackup(IOMan_BlockNo BlockNo) const = 0;

    virtual void SetBlockStateAfterBackup(IOMan_BlockNo BlockNo, FBM_BlockState BlockState) = 0;

    virtual void SetBlockStateToBackUp (IOMan_BlockNo BlockNo) = 0;

    virtual void SetBlockStateToFreeAfterSVP (IOMan_BlockNo BlockNo) = 0;
    virtual void SetAllBlocksMarkedAsFreeAfterSVPToFree () = 0;

    virtual void SetBlockStateMarkedForCompressionToFree (IOMan_BlockNo BlockNo) = 0;
    virtual void SetBlockStateMarkedForCompressionToFreeAfterSVP (IOMan_BlockNo BlockNo) = 0;
    virtual void SetBlockStateMarkedForCompressionToOccupied (IOMan_BlockNo BlockNo) = 0;

    virtual void GetNextBlocksForBackUp (const IOMan_BlockCount   MaxNumBlocksWanted, 
        IOMan_BlockCount        &SuppliedNumBlocks, 
        IOMan_BlockNo        &BlockNo) = 0;

    virtual void RestoreBlockStateMarkedForBackup (
        IOMan_BlockNo  BlockNo,
        SAPDB_Bool &bRestoredStateIsFreeAfterSVP) = 0;

    virtual void RestoreAllBlockStatesMarkedForBackup (IOMan_BlockCount &NumBlocksRestoredToFreeAfterSVP) = 0;

};

#endif //FBM_IDATAVOLUME_H
