/**************************************************************************/
/*!

  @file         Kernel_MigrationMatrix.hpp
  @author       UweH
  @brief        This defines the Kernel_MigrationMatrix
  @since        2003-06-30
*/
/*!
\if EMIT_LICENCE

  copyright:    Copyright (c) 2001-2004 SAP AG


    ========== licence begin  GPL
    Copyright (c) 2001-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end


\endif
*/
#ifndef KERNEL_MIGRATION_MATRIX_HPP
#define KERNEL_MIGRATION_MATRIX_HPP

#if 0 /* IMPORTANT FOR IMKDEP */
#include "config/Buildinfo/VERSIONS"
#include "config/Persinfo/VERSIONS.persinfo"
#endif

/* --------------------------------------------------------------------------- */
/// This constant says that a migration is only necessary for livecache instances.
#define LIVECACHE_ONLY true
/* --------------------------------------------------------------------------- */
Kernel_Migration::ChangeInfo Kernel_Migration::m_ChangeInfos[] =
{
    Kernel_Migration::ChangeInfo( "HISTORY ROOT ID CHECK",
                                  Kernel_Migration::data,
                                  LIVECACHE_ONLY,
                                  Kernel_Version(<PERSINFO:historyRootIdCheck:COMMA_FORMATED>) ),
    Kernel_Migration::ChangeInfo(), // new change info can be inserted here * formally used by "omsContainerNewLockProtocol"
    Kernel_Migration::ChangeInfo( "CONVERTER VOLUME NO BIT COUNT",     
                                  Kernel_Migration::converter,
                                  ! LIVECACHE_ONLY,
                                  Kernel_Version(<PERSINFO:converterVolumeNoBitCount:COMMA_FORMATED>) ),    
    Kernel_Migration::ChangeInfo( "OPEN TRANS FILE WITH MINIMAL EOT SEQUENCE",
                                  Kernel_Migration::data,
                                  ! LIVECACHE_ONLY,
                                  Kernel_Version(<PERSINFO:openTransFileWithMinEotSeq:COMMA_FORMATED>)  ),
    Kernel_Migration::ChangeInfo( "PHYSICAL AND CONFIGURED VOLUME SIZE",
                                  Kernel_Migration::data,
                                  ! LIVECACHE_ONLY,
                                  Kernel_Version(<PERSINFO:physicalAndConfiguredVolumeSize:COMMA_FORMATED>)  ),
    Kernel_Migration::ChangeInfo( "MULTIPLE ACTIONS",
                                  Kernel_Migration::log,
                                  ! LIVECACHE_ONLY,
                                  Kernel_Version(<PERSINFO:multipleActions:COMMA_FORMATED>)  ),
    Kernel_Migration::ChangeInfo("NEW FILE DIRECTORY LAYOUT",
                                  Kernel_Migration::filedir,
                                  ! LIVECACHE_ONLY,
                                  Kernel_Version(<PERSINFO:newFileDirectoryLayout:COMMA_FORMATED>)  ),
    Kernel_Migration::ChangeInfo("FLOATING RESTART PAGE",
                                  Kernel_Migration::data,
                                  ! LIVECACHE_ONLY,
                                  Kernel_Version(<PERSINFO:floatingRestartPage:COMMA_FORMATED>)  ),
    Kernel_Migration::ChangeInfo(), // new change info can be inserted here
    // #############################################################################################
	// if you need more entries you have to update #define MAX_CHANGE_INFO 10 in Kernel_Migration.hpp
    // #############################################################################################
    Kernel_Migration::ChangeInfo()  // invalid value => DO NOT CHANGE THE LAST VALUE
};
/* --------------------------------------------------------------------------- */
#endif
