/*


    ========== licence begin  GPL
    Copyright (c) 2003-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end


*/
#ifndef __OMS_CONTAINERHANDLE_HPP
#define __OMS_CONTAINERHANDLE_HPP

#include "gsp00.h"
#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"

/*--------------------------------------------------------------------------*/
/// Definition of OMS_ContainerHandle
/*!
** OMS_ContainerHandle is the unique ID, which identifies the container within 
** the file directory.
**
** \ingroup oms
*/
class OMS_ContainerHandle {
  tsp00_C8 m_data;
public :
  OMS_ContainerHandle() {
    memset(&m_data[0], 0xFF, sizeof(m_data));
  }

  OMS_ContainerHandle(tsp00_C8 data) {
    m_data = data;
  }

  bool operator!= (const OMS_ContainerHandle &op2) const { 
    return (0 != memcmp(this,&op2,sizeof(*this))); 
  };

  bool operator== (const OMS_ContainerHandle &op2) const { 
    return (0 == memcmp(this,&op2,sizeof(*this))); 
  };

  int operator% ( const int op2 ) const;

  const tsp00_C8& operator() () const {
      return m_data; 
  };

  tsp00_Uint4 GetOldContainerHandle() const {
    return *reinterpret_cast<const tsp00_Uint4 *>(&m_data[0]);
  };

  void SetOldContainerHandle(const tsp00_Uint4 cont) {
      SAPDB_MemCopyNoCheck(&m_data[0], &cont, sizeof(cont));
      for (int i=4; i<8; i++ )
          m_data[i] = 0;
  };
};

#endif  // __OMS_CONTAINERHANDLE_HPP
