/*!========================================================================

  @file         RTETask_LegacyTaskCtrl.cpp
  @ingroup      
  @author       RaymondR

  @brief        Access old task control

                The following class provides methodes to access 
                old task control.

  @since        2003-10-28  16:48
  @sa           

  ==========================================================================

  \if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (c) 2003-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end
  \endif
 ============================================================================*/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/
#include    "RunTime/Tasking/RTETask_Task.hpp"
#include    "RunTime/Tasking/RTETask_LegacyTaskCtrl.h"
#include    "RunTime/Tasking/RTETask_LegacyTaskCtrl.hpp"
#include    "geo50_0.h"

#if defined (_WIN32)
#else
# include    "gen88.h"  /* no check */
#endif


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTION PROTOTYPES                                        *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/


RTETask_LegacyTaskCtrl*  RTETask_LegacyTaskCtrl::Self()
{
    TASK_CTRL  pTaskCtrl; 

#   if defined (_WIN32)
     PUKT_CTRL_REC  pUKT = THIS_UKT_CTRL;

     if (( pUKT ) && ( pUKT->pCTask ))
         pTaskCtrl = pUKT->pCTask;
     else
         pTaskCtrl = 0; /* called from other type of thread */
#   else
     pTaskCtrl = RTE_GetOwnTask();
#   endif

    return  (0 == pTaskCtrl) ? 0 : pTaskCtrl->pLegacyTaskCtrlObject;
}

/*---------------------------------------------------------------------------*/

RTETask_LegacyTaskCtrl*  RTETask_LegacyTaskCtrl::ByTaskID(  RTE_TaskId taskID )
{
    TASK_CTRL  pTaskCtrl = 0;

#   if defined (_WIN32)
     if ( taskID >= 1 && taskID <= (SAPDB_Long)kgs.ulNumOfTasks )
         pTaskCtrl = &(kgs.pFirstTaskCtrl[taskID - 1]) ;
#   else
     if ( taskID >= 1 && taskID <= kgs->ulNumOfTasks )
         pTaskCtrl = kgs->pFirstTaskCtrl + taskID - 1 ;
#   endif

    return  (0 == pTaskCtrl) ? 0 : pTaskCtrl->pLegacyTaskCtrlObject;
}

/*---------------------------------------------------------------------------*/

SAPDB_Bool RTETask_LegacyTaskCtrl::IsUtilityTaskConnected()
{
#       if defined (_WIN32)
    return  ( kgs.pUT != 0 
           && kgs.pUT->pUserCommCtrl != 0 
           && 0 != kgs.pUT->pUserCommCtrl->ulConnected );
#       else
    return  ( KGS != 0 
           && KGS->ut != 0 
           && KGS->ut->connection != 0 
           && 0 != KGS->ut->connection->ci_connect_time );
#       endif
}

/*---------------------------------------------------------------------------*/

externC DLQ_ELEMENT*  GetTaskDLQElement1( TASK_CTRL   pTaskCtrl )
{
    // later on, when the legacy code returns the leased queue elements
    // we should use the free list of the task object
    return pTaskCtrl->pLegacyTaskCtrlObject->CommonQueueElement1();
}

/*---------------------------------------------------------------------------*/

externC DLQ_ELEMENT*  GetTaskDLQElement2( TASK_CTRL   pTaskCtrl )
{
    // later on, when the legacy code returns the leased queue elements
    // we should use the free list of the task object
    return pTaskCtrl->pLegacyTaskCtrlObject->CommonQueueElement2();
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/