/*!*********************************************************************

    @file         SAPDBFields_VarData.hpp

    @author       DanielD

    @brief        Handling of field buffers as used for SQL communication

\if EMIT_LICENCE
    ========== licence begin  GPL
    Copyright (c) 2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end
\endif
***********************************************************************/

#ifndef SAPDBFIELDS_VARDATA_HPP
#define SAPDBFIELDS_VARDATA_HPP


/*!

  @class SAPDBFields_VarData

 */
class SAPDBFields_VarData
{
public:
    /*!

      @class Reader

     */
    class Reader
    {
    public:
        Reader (const void * buf);
        bool next (const void * & fieldptr, int & fieldlen, int & indicator);
    protected:
        int fieldsRemaining;
        const unsigned char * readPos;
    };
    /*!

      @class Writer

     */
    class Writer
    {
    public:
        Writer (void * buf, int bufsize);
        bool addField (const void * field, unsigned int fieldlen);
        bool addSpecial (int indicator);
        void close (int & fieldCount, int & buflen);

    private:
        int             fieldCount;
        unsigned char * startPos;
        unsigned char * writePos;
        unsigned char * endPos;
    };

    /*!
        @brief verify that fields don't exceed buflen
     */
    static bool Verify (const void * buf, int buflen, bool allowUnderflow = false);

private:
    SAPDBFields_VarData () {};
};

#endif
