/*!
  @file           Catalog_IndexDescriptor.cpp
  @author         ThomasA
  @ingroup        Catalog
  @brief          yields information about an index

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end


\endif
*/


#include "SQLManager/Catalog/Catalog_IndexDescriptor.hpp"
#include "SQLManager/Catalog/Catalog_TableDescriptor.hpp"
#include "SQLManager/SQLMan_Types.hpp"
#include "SQLManager/SQLMan_Context.hpp"

#include "hak24.h"

Catalog_IndexDescriptor::ColumnIterator::ColumnIterator (const Catalog_TableDescriptor& Table, const tak_multindex* Index, bool IsKey)
: m_table(Table)
, m_index(Index)
, m_curr (0)
, m_isKey(IsKey)
{
}

//------------------------------------------------------------------------------

 void Catalog_IndexDescriptor::ColumnIterator::operator++()
 {
     ++m_curr;
 }

//------------------------------------------------------------------------------

 Catalog_IndexDescriptor::ColumnIterator::operator bool() const
 {
     if (m_isKey)
     {
         return m_curr < m_table.GetKeyColumnCount();
     }
     return m_curr < m_index->icount;
 }

//------------------------------------------------------------------------------

const Catalog_ColumnDescriptor Catalog_IndexDescriptor::ColumnIterator::operator*() const
{
    if (m_isKey)
    {
        Catalog_TableDescriptor::KeyColumnIterator keyIter = m_table.BeginKey();
        for (int ix = 0; ix < m_curr; ++ix)
        {
           ++keyIter;
        }
        return *keyIter;
    }
    Catalog_TableDescriptor::ColumnIterator iter = m_table.GetColumn(m_index->icolseq[m_curr]);
    return *iter;
}

//------------------------------------------------------------------------------

Catalog_IndexDescriptor::ColumnIterator Catalog_IndexDescriptor::Begin()
{
    return ColumnIterator (*m_table, &m_pIndex->indexdef[m_index-1], m_isKey);
}

//------------------------------------------------------------------------------

Catalog_IndexDescriptor::Catalog_IndexDescriptor (SQLMan_Context& Context, 
                                                  const Catalog_TableDescriptor& TableDescriptor, 
                                                  tak_mindexrecord* pIndex, 
                                                  int               Idx,
                                                  bool              IsKey)
: m_pContext(&Context)
, m_pIndex(pIndex)
, m_index(Idx)
, m_table(&TableDescriptor)
, m_isKey(IsKey)
{
}

//------------------------------------------------------------------------------

 SQLMan_Identifier Catalog_IndexDescriptor::GetName()
{
    SQLMan_Identifier indexName;
    if (m_pIndex)
    {
        a24get_indexname (*m_pContext, reinterpret_cast<tak_sysbufferaddress>(m_pIndex), m_index, indexName); 
    }
    return indexName;
}

//------------------------------------------------------------------------------

bool Catalog_IndexDescriptor::IsKey() const
{
    return m_isKey;
}

//------------------------------------------------------------------------------

bool Catalog_IndexDescriptor::IsUniqueIndex() const
{
    if (m_isKey)
    {
        return true;
    }
    return (m_pIndex->indexdef[m_index-1].iunique != 0);
}

//------------------------------------------------------------------------------

Catalog_IndexDescriptor::operator bool() const
{
    return (m_isKey || (0 != m_pIndex));
}
