/*!***************************************************************************

  module      : Opti_PascalWrapper.cpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / QueryRewrite
  description : 

  last changed: 2004-11-17  13:45
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#include "hak664wrapper.h"

#include "SAPDB/SQLManager/SQLMan_Context.hpp"
#include "SAPDB/SQLManager/Query/Query_Types.hpp"
#include "SAPDB/SQLManager/Query/Query_Graph.hpp"
#include "SAPDB/SQLManager/Query/Query_ManipulationContext.hpp"
#include "SAPDB/SQLManager/Optimizer/QueryRewrite/QueryRewrite_Rewrite.hpp"

#include "SAPDB/SQLManager/SharedSQL/SharedSQL_Handles.hpp"

#include "SAPDB/SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
#include "SAPDB/SAPDBCommon/Tracing/SAPDBTrace_Topic.hpp"

#include "hak542.h"

// ------------------------------------------------------------------------------------------------

void a664_QueryRewrite (
    tak_all_command_glob    VAR_VALUE_REF   acv,
	tgg00_VoidPtr prepareHandle        
    )
{
    SAPDBTRACE_ROUTINE_DEBUG("a664_QueryRewrite", QueryRewrite_Trace, 5);
    SAPDBERR_ASSERT_STATE(  acv.a_ap_tree == acv.a_session_ap_tree );

	SharedSQL_PrepareHandle* pPrepareHandle = REINTERPRET_CAST(SharedSQL_PrepareHandle*, prepareHandle);

    SQLMan_Context&             SQLManContext = SQLMan_Context::AcvToContext(acv);
	// ManipulationContext MUST be created before QueryGraph !!!
    Query_ManipulationContext   QRWContext(SQLManContext, pPrepareHandle);
    
    if ( QRWContext.IsOk() )
    {
        Query_QueryGraph QueryGraph(QRWContext);

        bool NeedOptimization = false;
        if ( QueryGraph.Initialize(NeedOptimization) )
        {
			//if ( NeedOptimization )
			//{
			//	// DDT : NO OPTIMIZATION DURING FIRST TEST PHASE
			//	//Opti_QueryRewriter QueryRewriter(QRWContext, QueryGraph);
			//	//QueryRewriter.Optimize();
			//}
			//else 
			{	// Keep the changes in any case ...
				QRWContext.KeepChanges();
			}
        }
    }
}

void a664_FreeQueryRewriteCaches(
    tak_all_command_glob    VAR_VALUE_REF   acv
                                )
{
    SAPDBTRACE_ROUTINE_DEBUG("a664_FreeQueryRewriteCaches", QueryRewrite_Trace, 5);
	if ( acv.a_queryrewrite_modified )
	{
		SAPDB_Int2 oldReturnCode = acv.a_returncode;
		acv.a_returncode = 0;
		if ( acv.a_cmd_segm )
			if ( acv.a_cmd_segm->sp1c_producer() == sp1pr_queryrewrite )
				a542pop_packet(acv);	// a542reset_packet (acv);

		SQLMan_Context::AcvToContext(acv).DeleteSyntaxTree();
		acv.a_ap_tree = acv.a_session_ap_tree; 
		acv.a_queryrewrite_modified = false;
		acv.a_returncode = oldReturnCode;
	}
}

// ------------------------------------------------------------------------------------------------

