/*!***************************************************************************

  module      : QueryRewrite_Rewrite.cpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / QueryRewrite
  description : 

  last changed: 2004-11-17  13:45
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#include "SAPDB/SQLManager/Optimizer/QueryRewrite/QueryRewrite_Rewrite.hpp"
#include "SAPDB/SQLManager/Query/Query_Graph.hpp"
#include "SAPDB/SQLManager/Query/Query_Quantifier.hpp"

// ------------------------------------------------------------------------------------------------

bool Opti_QueryRewriter::Optimize( void )
{
    SAPDBTRACE_METHOD_DEBUG ("Opti_QueryRewriter::Optimize", QueryRewrite_Trace, 5);

    if ( mQueryGraph.IsOk() )
    {
        if ( QueryRewrite_Trace.TracesLevel(1) )
            mRewriteContext.VTrace() << "-- QueryRewrite: Optimize START -----------------" << FlushLine;

        bool Optimized = false;
        //bool applied = false;
        //int k = 0;
        //do
        //{
        //    if ( QueryRewrite_Trace.TracesLevel(3) )
        //        mRewriteContext.VTrace() << "   Optimize Step " << k+1 << FlushLine;
        //    if ( applied = ApplyRules(/*Level=*/1, mQueryGraph.RootNode(), /*ChildrenFirst=*/true) )
        //        Optimized |= true;
        //}while ( applied && ++k<11 );

/*DDT*/
        //if ( ApplyRules(/*Level=*/2, mQueryGraph.RootNode(), /*ChildrenFirst=*/false) )
        //    Optimized |= true;

        if ( QueryRewrite_Trace.TracesLevel(1) )
            mRewriteContext.VTrace() << "-- QueryRewrite: Optimize END -------------------" << FlushLine;

        if ( Optimized )
            mQueryGraph.SetProgress(PG_Optimized);
    
        if ( QueryRewrite_Trace.TracesLevel(1) )
            mRewriteContext.VTrace() << "-------------------------------------------------" << FlushLine;

        if ( mQueryGraph.IsOk() && 
             mRewriteContext.IsOk() )
        {
            bool rc = mRewriteContext.KeepChanges();

			if ( rc && (Optimized || mQueryGraph.ViewReplaced()) && mRewriteContext.SQLManContext().a_queryrewrite_modified )
			{
				mRewriteContext.SQLManContext().SetTransInfoWarning(warn9_stmt_optimized, /*on=*/true);
				mRewriteContext.SQLManContext().a_queryrewrite_done.becomes(Operator_Rewrite);
			}

			return rc;
        }
    }
    mRewriteContext.DiscardChanges();
    return false;
}

bool Opti_QueryRewriter::ApplyRules( SAPDB_Int2 Level, Query_Node* Node, bool ChildrenFirst )
{
    SAPDBTRACE_METHOD_DEBUG ("Opti_QueryRewriter::ApplyRules", QueryRewrite_Trace, 5);

    Container_Vector<Query_Quantifier*>& mQuantifier = Node->GetQuantifier();
    bool applied = false;

	if ( ChildrenFirst )
		for ( unsigned cb=0; cb < mQuantifier.GetSize(); cb++ )
		{
			applied |= ApplyRules(Level, mQuantifier[cb]->NodeRef(), ChildrenFirst);
		}

    // Apply rules to the given box
    unsigned r=0;
    while ( r < mRules.GetSize() )
    {
        if ( mRules[r]->GetRuleLevel() == Level )
        {
            if ( mRules[r]->GetParamType() == Optim_Rule::NodeOnly )
            {
                if ( mRules[r]->IsApplicable(Node) )
                {
                    if ( mRules[r]->Apply(Node) )
                        applied |= true;
                }
            }
            if ( mRules[r]->GetParamType() == Optim_Rule::NodeAndQuantifier )
            {
                unsigned n = Node->GetQuantifier().GetSize();  // DDT 
                boolean a = false;
                for ( int q = (int)n-1; q >= 0; q-- )  // DDT : sizw may vary if SELMERGE or UNIONMERGE is applied
                {
                    if ( !Node->GetQuantifier()[q]->Disabled() )
                    {
                        if ( mRules[r]->IsApplicable(Node, q) )  // Quatifiers may be added/deletd only AFTER q
                        {
                            if ( mRules[r]->Apply(Node, q) )
                                a |= true;
                        }
                    }
                }
                if ( a )
                {
                    Node->DiscardDisabledQuantifiers();
                    applied |= true;
                }
            }
        }
        ++r;
    }

	if ( !ChildrenFirst )
		for ( unsigned cb=0; cb < mQuantifier.GetSize(); cb++ )
		{
			applied |= ApplyRules(Level, mQuantifier[cb]->NodeRef(), ChildrenFirst);
		}

    return applied;
}

// ------------------------------------------------------------------------------------------------
