/*!***************************************************************************

  module      : QueryRewrite_Rules.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / QueryRewrite
  description : 

  last changed: 2004-11-17  13:45
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef QUERYREWRITE_REWRITE
#define QUERYREWRITE_REWRITE

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SQLManager/Query/Query_ManipulationContext.hpp"
#include "SAPDB/SQLManager/Optimizer/QueryRewrite/QueryRewrite_Rules.hpp"
#include "SAPDB/SQLManager/Query/Query_Types.hpp"

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDB/Container/Container_Vector.hpp"

// ------------------------------------------------------------------------------------------------

class Query_QueryGraph;

// ------------------------------------------------------------------------------------------------

class Opti_QueryRewriter
{
public:
    Opti_QueryRewriter( Query_ManipulationContext& Context, Query_QueryGraph& QueryGraph )
        : mRewriteContext(Context)
        , mQueryGraph(QueryGraph)
        , mRules(Context.GetAllocator())
    {
        mRules.Reserve(10);

        InsertRule(new (mRewriteContext.GetAllocator()) Rule_MergeUnions(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_MergeSelects(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_EtoF_SelMerge(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_PushDownConditions(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_PullUpDistinct(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_EXEC2NEXIST(QueryGraph));
        //InsertRule(new (mRewriteContext.GetAllocator()) Rule_AddKeys(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_INT2EXIST(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_DISTPDTO(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_EorAPDFR(QueryGraph));
        InsertRule(new (mRewriteContext.GetAllocator()) Rule_DISTPDFR(QueryGraph));
        //InsertRule(new (mRewriteContext.GetAllocator()) Rule_CopyNodes(QueryGraph));
    }

    ~Opti_QueryRewriter( void ) 
    {
        for ( unsigned i = 0; i<mRules.GetSize(); i++ )
        {
            if ( mRules[i] )
                destroy(mRules[i], mRewriteContext.GetAllocator());
        }
    }

    bool Optimize( void );

protected:

    bool ApplyRules( SAPDB_Int2 Level, Query_Node* Node, bool ChildrenFirst );

    inline void InsertRule( Optim_Rule* R )
    {
        if (R)
        {   // DDT : Insert should be ordered in some way !!!
            mRules.InsertEnd(R);
        }
    }

private:
    Query_ManipulationContext&      mRewriteContext;
    Query_QueryGraph&               mQueryGraph;
    Container_Vector<Optim_Rule*>   mRules;
};

// ------------------------------------------------------------------------------------------------

#endif