/*!***************************************************************************

  module      : QueryRewrite_Rewrite.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / QueryRewrite
  description : 

  last changed: 2004-11-17  13:45
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef QUERYREWRITE_RULES
#define QUERYREWRITE_RULES

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"

//#include "SAPDB/SQLManager/Query/Query_Node.hpp"
//#include "SAPDB/SQLManager/Query/Query_Graph.hpp"

// ------------------------------------------------------------------------------------------------

class Query_Node;
class Query_SelectNode;
class Query_QueryGraph;

// ------------------------------------------------------------------------------------------------

class Optim_Rule
{
public:
    enum ParamType{
                            NodeOnly,
                            NodeAndQuantifier
                  };

    enum RuleID   { 
                            R_MergeUnions,			// Prio 1
                            R_MergeSelects,			// Prio 1
                            R_PushDownConditions,	// Prio 1
                            R_PullUpDistinct,		// Prio 2
                            R_EXEC2NEXIST,			// Prio 2
                            R_EtoF,					// Prio 2
                            R_AddKeys,				// Prio 3
                            R_INT2EXIST,			// Prio 3
                            R_DISTPDTO,				// Prio 4
                            R_EorAPDFR,				// Prio 5
                            R_DISTPDFR,				// Prio 5
                            R_CopyNodes				// Prio 5
                  };
    
    Optim_Rule( Query_QueryGraph& Graph ) : mGraph(Graph) {};

    /*!
        return TRUE if the rule can be applied, else FALSE
    */
    virtual bool  IsApplicable( Query_Node* Node ) const = 0;
    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const = 0;

    /*!
        return  TRUE if there was no error while applying the rule, else FALSE 
    */
    virtual bool  Apply( Query_Node* Node ) const  = 0;
    // Quantifiers with n >= QuantifierNo may be added/deleted for those with n<QuantifierNo must be set to Disabled!
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const = 0;

    virtual SAPDB_Int2  GetRulePrio( void ) const = 0;
    virtual SAPDB_Int2  GetRuleLevel( void ) const { return 1; };

    virtual Optim_Rule::RuleID GetRuleID( void ) const = 0;
    virtual const char* const  GetRuleName( void ) const = 0;

    virtual Optim_Rule::ParamType GetParamType( void ) const = 0;

protected:
    Query_QueryGraph&   mGraph;
};

// ------------------------------------------------------------------------------------------------

// ------------------------------------------------------------------------------------------------
class Rule_MergeSelects : public Optim_Rule                     // SelectMerge
{
public:
    Rule_MergeSelects( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 1; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_MergeSelects; }
    virtual const char* const  GetRuleName( void ) const { return "MergeSelects"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeAndQuantifier; }

private:
    bool  Merge( Query_SelectNode* Node, SAPDB_UInt2 QuantifierNo ) const;
    bool  CopyBodyQuery( Query_SelectNode* UpperNode, Query_SelectNode* LowerNode ) const;
    bool  CopyFromPartSyntaxTree( Query_SelectNode* UpperNode, Query_SelectNode* LowerNode ) const;
    bool  AdjustSelectListSyntaxTree( Query_SelectNode* UpperNode, Query_SelectNode* LowerNode ) const;
    bool  DeleteLowerNodeSyntaxTree( Query_SelectNode* UpperNode, Query_SelectNode* LowerNode ) const;
    bool  DeleteLowerNodeQuery( Query_SelectNode* UpperNode, SAPDB_UInt2 QuantifierNo, Query_SelectNode* LowerNode ) const;

	// the next 2 lines are only to avoid compilter wornings on SunOS
	virtual bool  IsApplicable( Query_Node* Node ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_MergeUnions : public Optim_Rule                   // UnionMerge
{
public:
    Rule_MergeUnions( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 1; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_MergeUnions; }
    virtual const char* const  GetRuleName( void ) const { return "MergeUnions"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeAndQuantifier; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
	virtual bool  IsApplicable( Query_Node* Node ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_EtoF_SelMerge : public Rule_MergeSelects             // SelectMerge&EtoF
{
public:
    Rule_EtoF_SelMerge( Query_QueryGraph& Graph ) : Rule_MergeSelects(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
//    virtual SAPDB_Int2  GetRulePrio( void ) const { return 1; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_EtoF; }
    virtual const char* const  GetRuleName( void ) const { return "EToF"; }
//    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeAndQuantifier; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
	virtual bool  IsApplicable( Query_Node* Node ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_PushDownConditions : public Optim_Rule                       // ConditionPushDown
{
public:
    Rule_PushDownConditions( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node ) const;
    virtual bool  Apply( Query_Node* Node ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 1; }
    virtual SAPDB_Int2  GetRuleLevel( void ) const { return 2; };
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_PushDownConditions; }
    virtual const char* const  GetRuleName( void ) const { return "PushDownConditions"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeOnly; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_PullUpDistinct : public Optim_Rule                       // DistinctPullUp   
{
public:
    Rule_PullUpDistinct( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node ) const;
    virtual bool  Apply( Query_Node* Node ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 2; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_PullUpDistinct; }
    virtual const char* const  GetRuleName( void ) const { return "PullUpDistinct"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeOnly; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_EXEC2NEXIST : public Optim_Rule                  // 
{
public:
    Rule_EXEC2NEXIST( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node ) const;
    virtual bool  Apply( Query_Node* Node ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 2; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_EXEC2NEXIST; }
    virtual const char* const  GetRuleName( void ) const { return "EXEC2NEXIST"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeOnly; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
//class Rule_AddKeys : public Optim_Rule
//{
//public:
//    Rule_AddKeys( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};
//
//    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
//    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
//    virtual SAPDB_Int2  GetRulePrio( void ) const { return 3; }
//    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_AddKeys; }
//    virtual const char* const  GetRuleName( void ) const { return "AddKeys"; }
//    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeAndQuantifier; }
//
//private:
//    // the next 2 lines are only to avoid compilter wornings on SunOS
//	  virtual bool  IsApplicable( Query_Node* Node ) const { return false; } ;
//    virtual bool  Apply( Query_Node* Node ) const { return false; } ;
//};

// ------------------------------------------------------------------------------------------------
class Rule_INT2EXIST : public Optim_Rule                    // IntersectToExist
{
public:
    Rule_INT2EXIST( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node ) const;
    virtual bool  Apply( Query_Node* Node ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 3; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_INT2EXIST; }
    virtual const char* const  GetRuleName( void ) const { return "INT2EXIST"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeOnly; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_DISTPDTO : public Optim_Rule
{
public:
    Rule_DISTPDTO( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node ) const;
    virtual bool  Apply( Query_Node* Node ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 4; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_DISTPDTO; }
    virtual const char* const  GetRuleName( void ) const { return "DISTPDTO"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeOnly; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_EorAPDFR : public Optim_Rule
{
public:
    Rule_EorAPDFR( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 5; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_EorAPDFR; }
    virtual const char* const  GetRuleName( void ) const { return "EorAPDFR"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeAndQuantifier; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
	virtual bool  IsApplicable( Query_Node* Node ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
class Rule_DISTPDFR : public Optim_Rule
{
public:
    Rule_DISTPDFR( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};

    virtual bool  IsApplicable( Query_Node* Node ) const;
    virtual bool  Apply( Query_Node* Node ) const;
    virtual SAPDB_Int2  GetRulePrio( void ) const { return 5; }
    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_DISTPDFR; }
    virtual const char* const  GetRuleName( void ) const { return "DISTPDFR"; }
    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeOnly; }

private:
	// the next 2 lines are only to avoid compilter wornings on SunOS
    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
};

// ------------------------------------------------------------------------------------------------
//class Rule_CopyNodes  : public Optim_Rule
//{
//public:
//    Rule_CopyNodes( Query_QueryGraph& Graph ) : Optim_Rule(Graph) {};
//
//    virtual bool  IsApplicable( Query_Node* Node ) const;
//    virtual bool  Apply( Query_Node* Node ) const;
//    virtual SAPDB_Int2  GetRulePrio( void ) const { return 5; }
//    virtual Optim_Rule::RuleID GetRuleID( void ) const { return R_CopyNodes; }
//    virtual const char* const  GetRuleName( void ) const { return "CopyNodes"; }
//    virtual Optim_Rule::ParamType GetParamType( void ) const { return NodeOnly; }
//
//private:
//    // the next 2 lines are only to avoid compilter wornings on SunOS
//    virtual bool  IsApplicable( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
//    virtual bool  Apply( Query_Node* Node, SAPDB_UInt2 QuantifierNo ) const { return false; } ;
//};

// ------------------------------------------------------------------------------------------------

#endif