/*!***************************************************************************

  module      : Query_Column.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef Query_Column_H
#define Query_Column_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SQLManager/SQLMan_Types.hpp"

#include "SAPDB/SQLManager/Query/Query_Types.hpp"
#include "SAPDB/SQLManager/Query/Query_SyntaxTree.hpp"

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"

// ------------------------------------------------------------------------------------------------

class Query_Column
{
private:    // private copyconstuctor used by CreateDuplicate
    Query_Column( const Query_Column& C, Query_SyntaxTree& SyntaxTree, Query_NodeMapper* mpMap );
public:

	enum ColumnState  { Unidentified, MayBeCorrelated, Identified };

    Query_Column( void );
    Query_Column( SyntaxTreeNodeNo Node, bool SelCol );
	Query_Column( const Query_Column& C );

    inline SyntaxTreeNodeNo GetColumnSyntaxNode( void ) { return mSyntaxNode;   }

    inline SQLMan_Identifier GetColumnName( void )      { return mColName;      }

    inline SQLMan_Identifier GetReferenceName( void )   { return mRefName;      }

    inline SQLMan_Identifier GetIdentificationName( void ) { return (mRefName!=SQLMan_Identifier()?mRefName:mColName); }

    inline SAPDB_Int2 GetQuantifierSyntaxNode( void )   { return mQuantifierNode;   }
    
    inline SAPDB_Int2 GetNodeID( void )                   { return mNodeID;   }

    inline bool IsOutputColumn( void )                  { return mOutputColumn; }

    inline bool IsAsterix( void )                       { return mAsterix;      }

    inline SAPDB_Int2 GetColumnCount( void )            { return mColumnCount;  }

    inline SAPDB_Int2 GetColumnIndex( void )            { return mColumnIndex;  }

	inline ColumnState GetStatus( void )				{ return mStatus; }

	inline bool IsUnidentified( void )					{ return mStatus != Identified; }

	inline void SetStatus( ColumnState Status )
	{
		mStatus = Status;
	}

    inline void SetColumnInfo( SAPDB_Int2 NodeID, SAPDB_Int2 QNode, bool Asterix, SQLMan_Identifier CName, SQLMan_Identifier RName, SAPDB_Int2 ColCnt, SAPDB_Int2 ColIdx )
    {
        SAPDBERR_ASSERT_ARGUMENT (  (!Asterix && ColCnt==1) ||
                                    ( Asterix && ColCnt>=1) );
        if ( QNode==cak_qrewrite_ref_multistar )
        {
            SAPDBERR_ASSERT_ARGUMENT(Asterix);
        }

        mColName        = CName;
        mRefName        = RName;
        mNodeID         = NodeID;	// QueryNode
        mQuantifierNode = QNode;    // Quantifier->SyntaxNode() within the QueryNode
        mColumnCount    = ColCnt;
        mColumnIndex    = ColIdx;
        mAsterix        = Asterix;
		SetStatus(Identified);
    }

    inline void SetColumnInfo( SAPDB_Int2 NodeID, SAPDB_Int2 QNode )
    {
        SAPDBERR_ASSERT_ARGUMENT (  (mNodeID == cak_qrewrite_ref_init) &&
                                    (mQuantifierNode == cak_qrewrite_ref_init) );

        mNodeID         = NodeID;
        mQuantifierNode = QNode;
		SetStatus(Identified);
    }

	inline void SetColumnSyntaxNode( SyntaxTreeNodeNo SyntaxNode )
	{
		mSyntaxNode = SyntaxNode;
	}

    inline void ChangeColumnInfo( SAPDB_Int2 NodeID, SQLMan_Identifier& CName, SQLMan_Identifier& RName, SAPDB_Int2 QNode, SAPDB_Int2 ColIdx )
    {
       
		mNodeID         = NodeID;		// QueryNode
        mColName        = CName;
        mRefName        = RName;
        mQuantifierNode = QNode;    // Quantifier->SyntaxNode() within the QueryNode
        ChangeColumnIndex(ColIdx);
		SetStatus(Identified);
    }

	inline void ChangeColumnIndex( SAPDB_Int2 NewColumnIndex )
	{
		mColumnIndex = NewColumnIndex;
		if ( NewColumnIndex != cak_qrewrite_ref_init )
			mAsterix = false;
		else
			mAsterix = true;
	}

	inline void ChangeColumnIndexDelta( SAPDB_Int2 DeltaColumnIndex )
	{
		SAPDBERR_ASSERT_STATE( mColumnIndex != cak_qrewrite_ref_init);
		mColumnIndex += DeltaColumnIndex;
	}

	inline void SetReferenceName( SQLMan_Identifier RefName )
	{
		mRefName = RefName;
	}


private:

    SAPDB_Int2          mNodeID;        // the QueryNode this column belongs to. indicates if it is a correlation
    SyntaxTreeNodeNo    mQuantifierNode;// the quantifier this column belongs to
                                        // may be cak_qrewrite_ref_multistar or
                                        // cak_qrewrite_ref_function/cak_qrewrite_ref_aggregate
    SAPDB_Int2          mColumnIndex;   // Column number within the Quantifier           
    SAPDB_Int2          mColumnCount;   // number of columns: 1 if column >=1 if '*'
    SQLMan_Identifier   mColName;       // 
    SQLMan_Identifier   mRefName;       // the reference name of this column
    SyntaxTreeNodeNo    mSyntaxNode;    // syntax node : a60/2 for OutputColumns, else 0/s_columnname /s_columnid
    bool                mOutputColumn;  // Output column ?
    bool                mAsterix; 
	ColumnState			mStatus;						
};

// ------------------------------------------------------------------------------------------------

#endif