/*!***************************************************************************

  module      : Query_Node.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef Query_Node_H
#define Query_Node_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SQLManager/SQLMan_Types.hpp"
#include "SAPDB/SQLManager/SQLMan_Context.hpp"
#include "SAPDB/SQLManager/SQLMan_ContextSessionMembers.hpp" // DDT why ???

#include "SAPDB/SQLManager/Query/Query_Types.hpp"
#include "SAPDB/SQLManager/Query/Query_Column.hpp"

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDB/Container/Container_Vector.hpp"

// ------------------------------------------------------------------------------------------------

class	Query_ManipulationContext;
class   Query_Quantifier;
class   Query_QueryGraph;
class   Query_SelectNode;
class   SAPDBMem_IRawAllocator;

// ------------------------------------------------------------------------------------------------

class Query_Node
{
protected:    // private copyconstuctor used by CreateDuplicate
    Query_Node( const Query_Node& N, Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap );
public:
    virtual Query_Node* CreateDuplicate( Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap ) = 0;

    Query_Node( Query_ManipulationContext& Context, Query_QueryGraph& QueryGraph, 
                Query_Position Position, SyntaxTreeNodeNo APNode,
                bool HeadDistinct, Query_DistinctType BodyDistinctType ); 

	virtual ~Query_Node( void );

	bool IllegalUseOfMethod( char* MethodName );

    virtual Query_NodeType GetNodeType( void ) = 0;
    virtual Query_JoinType GetJoinType( void )		{ return JT_NoJoin; }
    virtual bool OuterJoin( void )					{ return IllegalUseOfMethod("OuterJoin");	}
    virtual bool Aggregates( void )					{ return IllegalUseOfMethod("Aggregates");	} 
    virtual void SetOuterJoin( Query_JoinType JT )	{ IllegalUseOfMethod("SetOuterJoin");		}
    virtual void SetForReuse( void )				{ IllegalUseOfMethod("SetForReuse");		}
	virtual void SetGivenSequence( void )			{ IllegalUseOfMethod("SetGivenSequence");	}
	virtual bool HasSysKey( void )					{ return false; }

    virtual SyntaxTreeNodeNo	SelectListStart( void )							{ return cak_qrewrite_ref_init; }
    virtual void				SetSelectListStart( SyntaxTreeNodeNo Start )	{ IllegalUseOfMethod("SetSelectListStart"); }
    virtual SyntaxTreeNodeNo	FromPartStart( void )							{ return cak_qrewrite_ref_init; }
    virtual void				SetFromPartStart( SyntaxTreeNodeNo Start )		{ IllegalUseOfMethod("SetFromPartStart"); }

    virtual SyntaxTreeNodeNo	SearchConditionStart( void )					{ return cak_qrewrite_ref_init; }
    virtual void				SetSearchConditionStart( SyntaxTreeNodeNo Start ){ IllegalUseOfMethod("SetSearchConditionStart"); }
    virtual bool				HasSearchCondition( void )						{ return false; }
    virtual SyntaxTreeNodeNo	OrderByStart( void )							{ return cak_qrewrite_ref_init; }
    virtual void				SetOrderByStart( SyntaxTreeNodeNo Start )       { IllegalUseOfMethod("SetOrderByStart"); }
    virtual bool				HasOrderBy( void )								{ return false; }
    virtual SyntaxTreeNodeNo	GroupByStart( void )							{ return cak_qrewrite_ref_init; }
    virtual void				SetGroupByStart( SyntaxTreeNodeNo Start )       { IllegalUseOfMethod("SetGroupByStart"); }
    virtual bool				HasGroupBy( void )								{ return false; }
    virtual SyntaxTreeNodeNo	HavingStart( void )								{ return cak_qrewrite_ref_init; }
    virtual void				SetHavingStart( SyntaxTreeNodeNo Start )        { IllegalUseOfMethod("SetHavingStart"); }
    virtual bool				HasHaving( void )								{ return false; }
    virtual SyntaxTreeNodeNo	UpdateColumnsStart( void )						{ return cak_qrewrite_ref_init; }
    virtual void				SetUpdateColumnsStart( SyntaxTreeNodeNo Start ) { IllegalUseOfMethod("SetUpdateColumnsStart"); }
    virtual bool				HasForUpdate( void )							{ return false; }

    virtual void PrintToTrace( Kernel_VTrace& VTrace );
    void PrintChildNodesToTrace( Kernel_VTrace& VTrace );

	virtual bool SelectPrivilegesForAllColumns( void )	{ return true; }

    virtual bool ReplaceComplexViews( void );

	virtual bool IsColumnOfTypeLong( SAPDB_Int2 ColumnIndex ) { return false; }
	virtual bool IsColumnOfTypeLong( SAPDB_Int2 QuantifierSyntaxNode, SAPDB_Int2 ColumnIndex ) { return IsColumnOfTypeLong(ColumnIndex); }

	virtual Query_Node* GetFirstChildNode( SAPDB_Int2& ChildCount );

    bool AddQuantifier ( Query_Quantifier* Q );
    bool AddQuantifier ( Query_Node* Node, Query_DistinctType DistinctType, Query_Position Position, Query_QuantifierType QuantifierType );
    void DiscardDisabledQuantifiers( void );

    Container_Vector<Query_Quantifier*>& GetQuantifier( void )
    {
        return mQuantifier;
    }

    inline       void               SetNodeID( SAPDB_UInt2 ID ) { mID = ID; }
    inline const SAPDB_UInt2&       NodeID( void )          { return mID; }
    inline const SAPDB_UInt2&       FQuantifierCnt( void )  { return mFQuantifierCnt; }
    inline const SAPDB_UInt2&       WQuantifierCnt( void )  { return mWQuantifierCnt; }
    inline const SyntaxTreeNodeNo&  SyntaxNode( void )      { return mAPNode;   };
    inline const Query_Position&    Position( void )        { return mPosition; };
    inline bool&                    HeadDistinct( void )    { return mHeadDistinct; }
    inline Query_DistinctType&      BodyDistinctType( void ){ return mBodyDistinctType; }
    inline Query_Node*&         FatherNode( void )  { return mFatherNode; }
    inline Query_Quantifier*&       FatherQuantifier( void ){ return mFatherQuantifier; }
    inline Query_QueryGraph&        QueryGraph( void )      { return mQueryGraph; }
    inline SAPDBMem_IRawAllocator&  GetAllocator( void )    { return mAllocator; }
    inline Query_ManipulationContext& Context( void )       { return mContext; }
    
    virtual bool Semantics( bool UniqueOutputColumnNames, bool ComplexViewReplacement, bool& UnidentifiedColumns );
    virtual bool Semantics_Correlation( Query_SelectNode* TopNode, bool& UnidentifiedColumns );

    virtual bool PullDownFQuantifierBegin( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier ) { return false; }
    virtual bool PullDownFQuantifierEnd( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier ) { return false; }
    virtual bool PullDownSearchCond( Container_Vector<Query_Column>*& Columns, SyntaxTreeNodeNo SearchCond ) { return false; }

    virtual bool RemoveSubSelectNode( SyntaxTreeNodeNo SubSelNode, Query_Position PosType, bool ReplaceByChildren )
    {
        return mFatherNode->RemoveSubSelectNode(SubSelNode, PosType, ReplaceByChildren);
    }
    virtual bool AddSubSelectNode( SyntaxTreeNodeNo SubSelNode, SyntaxTreeNodeNo FatherNode, Query_Position FatherPosType, SAPDB_Int2 Pos )
    {
        return mFatherNode->AddSubSelectNode(SubSelNode, FatherNode, FatherPosType, Pos);
    }

	virtual bool CreateColRefNamesAndResolveAsterix( SyntaxTreeNodeNo& RefNameNodes ) { return false; }

    virtual bool AppendColumn( Container_Vector<Query_Column>*& pColumnList, Query_Column Col ) { return IllegalUseOfMethod("AppendColumn"); }
    virtual bool AppendOutputColumn ( SyntaxTreeNodeNo Node ) { return IllegalUseOfMethod("AppendOutputColumn"); }
    virtual SAPDB_Int2 OutputColumns( void ) { return 0; }
    virtual SAPDB_Int2 FunctionOutputColumns( void ) { return 0; }
    virtual bool GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& ColName )	{ return false; }
    virtual bool GetOutputColumnIndex( SAPDB_Int2 i, SAPDB_Int2& ColumnIndex )	{ return false; }
    virtual bool CheckForColumn( SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex ) { return false; }
    virtual bool CheckForColumn( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex ) { return false; }
    virtual bool CheckQuantifierName( SQLMan_Identifier Schema, SQLMan_Identifier Table ) { return false; }
    virtual void GetIdentificationNames(SQLMan_Identifier& Schema, SQLMan_Identifier& Table) { Schema = SQLMan_Identifier(); Table = SQLMan_Identifier(); };
	virtual bool HasReferenceName()			{ return false; }
    virtual bool IsCorrelated( void )		{ return false; }

protected:

	bool GetQuantifierNoFromSyntaxNode( SyntaxTreeNodeNo Node, SAPDB_Int2& QNo );

    Query_ManipulationContext&          mContext;
    SAPDBMem_IRawAllocator&             mAllocator;

    Query_QueryGraph&                   mQueryGraph;
    Query_Node*                     mFatherNode;
    Container_Vector<Query_Quantifier*> mQuantifier;  
    Query_Quantifier*                   mFatherQuantifier;
    SAPDB_UInt2                         mID;

    SyntaxTreeNodeNo                    mAPNode;
    SAPDB_UInt2                         mFQuantifierCnt; 
    SAPDB_UInt2                         mWQuantifierCnt; 
    Query_Position                      mPosition;

    Query_DistinctType                  mBodyDistinctType;
    bool                                mHeadDistinct;
	bool                                mUnidentifiedColumnsInChildNodes;
    bool                                mSemanticsDone;

};

// ------------------------------------------------------------------------------------------------

#endif