/*!***************************************************************************

  module      : Query_NodeRoot.cpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#include "SAPDB/SQLManager/Query/Query_NodeRoot.hpp"
#include "SAPDB/SQLManager/Query/Query_Graph.hpp"

// ------------------------------------------------------------------------------------------------
    
Query_RootNode::Query_RootNode( Query_ManipulationContext& Context, Query_QueryGraph& QueryGraph ) 
    : Query_Node(Context, QueryGraph, P_Start, 0, false, DT_Permit)
{
}

Query_RootNode::Query_RootNode( Query_Node* AnyOtherNode )
	: Query_Node(AnyOtherNode->Context(), AnyOtherNode->QueryGraph(), P_Start, 0, false, DT_Permit)
{
}

Query_RootNode::Query_RootNode( const Query_RootNode& N, Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap )
    : Query_Node(N, Father, Q, mpMap) 
{
}

Query_Node* Query_RootNode::CreateDuplicate( Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap )
{
    return new(mAllocator) Query_RootNode(*this, Father, Q, mpMap);
}

void Query_RootNode::PrintToTrace( Kernel_VTrace& VTrace )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_RootNode::PrintToTrace", QueryRewrite_Trace, 8);

    Query_Node::PrintToTrace(VTrace);
    if ( QueryRewrite_Trace.TracesLevel(2) )
    {
        VTrace << "NodeType  : Top" << FlushLine;
    }
    PrintChildNodesToTrace(VTrace);
}

bool Query_RootNode::Semantics( bool UniqueOutputColumnNames, bool ComplexViewReplacement)
{
	bool UnidentifiedColumns = false;
	bool erg = Query_Node::Semantics(UniqueOutputColumnNames, ComplexViewReplacement, UnidentifiedColumns);
	if ( UnidentifiedColumns )
		mQueryGraph.SetError("Unidentified Columns");
	return (erg && (!UnidentifiedColumns));
}


// ------------------------------------------------------------------------------------------------
