/*!***************************************************************************

  module      : Query_NodeSelect.cpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#include "SAPDB/SQLManager/Query/Query_NodeSelect.hpp"
#include "SAPDB/SQLManager/Query/Query_Graph.hpp"
#include "SAPDB/SQLManager/Query/Query_Quantifier.hpp"

#include "SAPDB/RunTime/RTE_CurrentStack.hpp"

// ------------------------------------------------------------------------------------------------

Query_SelectNode::Query_SelectNode(  Query_ManipulationContext& Context, Query_QueryGraph& QueryGraph, Query_Position Position, SyntaxTreeNodeNo APNode,
                bool All ) 
    : Query_Node(Context, QueryGraph, Position, APNode, (All?false:true), (All?DT_Preserve:DT_Enforce))
    , mReferenceName()
    , mpSelectListColumns(0)
    , mpSearchConditionColumns(0)
    , mpHavingColumns(0)
    , mpGroupByColumns(0)
    , mpOrderByColumns(0)
	, mpUpdateColumnsColumns(0)
    , mOutputColumnCnt(0)
    , mFunctionOutputColumnCnt(0)
    , mSelectListStartNo(cak_qrewrite_ref_init)
    , mFromPartStartNo(cak_qrewrite_ref_init)
    , mSearchConditionStartNo(cak_qrewrite_ref_init)
    , mOrderByStartNo(cak_qrewrite_ref_init)
    , mGroupByStartNo(cak_qrewrite_ref_init)
    , mHavingStartNo(cak_qrewrite_ref_init)
    , mForUpdateColumnStartNo(cak_qrewrite_ref_init)
    , mJoinType(JT_Unknown)
	, mExprCnt(0)
    , mAllFlg(All)
	, mForReuse(false)
	, mGivenSequence(false)
    , mIsCorrelated(false)
	, mUnidentifiedColumns(false)
    , mCountFlg(false)
    , mSumFlg(false)
    , mAvgFlg(false)
    , mMinFlg(false)
    , mMaxFlg(false)
{
}

// copy constructor
Query_SelectNode::Query_SelectNode( const Query_SelectNode& N, Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap )
    : Query_Node(N, Father, Q, mpMap) 
{
}

Query_SelectNode::~Query_SelectNode( void )
{
    if ( mpSelectListColumns ) 
        destroy(mpSelectListColumns, mAllocator);
    if ( mpSearchConditionColumns ) 
        destroy(mpSearchConditionColumns, mAllocator);
    if ( mpHavingColumns ) 
        destroy(mpHavingColumns, mAllocator);
    if ( mpGroupByColumns ) 
        destroy(mpGroupByColumns, mAllocator);
    if ( mpOrderByColumns ) 
        destroy(mpOrderByColumns, mAllocator);
    if ( mpUpdateColumnsColumns ) 
        destroy(mpUpdateColumnsColumns, mAllocator);
}

Query_Node* Query_SelectNode::CreateDuplicate( Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap )
{
    return new(mAllocator) Query_SelectNode(*this, Father, Q, mpMap);
}

// ------------------------------------------------------------------------------------------------

void Query_SelectNode::PrintToTrace( Kernel_VTrace& VTrace )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::PrintToTrace", QueryRewrite_Trace, 8);

    Query_Node::PrintToTrace(VTrace);
    if ( QueryRewrite_Trace.TracesLevel(2) )
    {
        VTrace << "NodeType  : Select" << FlushLine;
        VTrace << "SelectList : " << mSelectListStartNo << FlushLine;
        VTrace << "FromPart   : " << mFromPartStartNo << FlushLine;
        VTrace << "SearchCond : " << mSearchConditionStartNo << FlushLine;
        VTrace << "OrderBy    : " << mOrderByStartNo << FlushLine;
        VTrace << "GroupBy    : " << mGroupByStartNo << FlushLine;
        VTrace << "Having     : " << mHavingStartNo << FlushLine;
        VTrace << "ForUpdate  : " << mForUpdateColumnStartNo << FlushLine;

        VTrace << "InnerJoin  : " << ((mFQuantifierCnt && !OuterJoin())?"yes":"no") << FlushLine;
        VTrace << "FullOuter  : " << ((mJoinType & JT_FullOuter )?"yes":"no") << FlushLine;
        VTrace << "RightOuter : " << ((mJoinType & JT_RightOuter)?"yes":"no") << FlushLine;
        VTrace << "LeftOuter  : " << ((mJoinType & JT_LeftOuter )?"yes":"no") << FlushLine;

        VTrace << "Quantifier : " << mQuantifier.GetSize() << FlushLine;
        VTrace << "FQuantifier: " << mFQuantifierCnt << FlushLine;
        if ( mSemanticsDone && QueryRewrite_Trace.TracesLevel(3) )
        {
            VTrace << "FQuantifier: " << mFQuantifierCnt << FlushLine;
            for ( SAPDB_UInt2 f=0; f<mFQuantifierCnt; f++ )
            {
                VTrace  << "FQ" << f << " Node    : " 
                        << mQuantifier[f]->NodeRef()->SyntaxNode();
            }

            VTrace << "OutColumns : " << mOutputColumnCnt << FlushLine;
            if ( mpSelectListColumns )
            {
                for ( SAPDB_UInt2 c=0; c<mpSelectListColumns->GetSize(); c++ )
                {
                    if ( (*mpSelectListColumns)[c].IsOutputColumn() )
                    {
                        SAPDB_Int2 FromTabNode = (*mpSelectListColumns)[c].GetQuantifierSyntaxNode();
                        if ( FromTabNode > 0 )
						{
							SAPDB_Int2 QNo=0;
							GetQuantifierNoFromSyntaxNode(FromTabNode, QNo);
                            VTrace  << "Col" << c << " Node   : " 
                                    << (*mpSelectListColumns)[c].GetColumnSyntaxNode() 
                                    << " Quantifier Node " 
                                    << mQuantifier[QNo]->NodeRef()->SyntaxNode();
						}
                    }
                }
            }
        }
    }
    if ( QueryRewrite_Trace.TracesLevel(1) && !QueryRewrite_Trace.TracesLevel(2) )
    {
        SAPDB_Int2 n = mQuantifier.GetSize();

        VTrace << "[" << mID << "] SELECT ";
        
        bool found = false;
        SAPDB_Int2 i;
        for (i=0; i<n; i++)
        {
            Query_Quantifier* Q = mQuantifier[i];
            if ( Q->QuantifierType() == QT_L )
            {
                if ( !found )
                {
                    VTrace << "<" << Q->NodeID() << ">";
                    found = true;
                }
                else
                    VTrace << ",<" << Q->NodeID() << ">";
            }
        }
        if (!found)
            VTrace << "...";
        VTrace << FlushLine; 

        VTrace << "               FROM ";
        found = false;
        for (i=0; i<n; i++)
        {
            Query_Quantifier* Q = mQuantifier[i];
            if ( Q->QuantifierType() == QT_F )
            {
                if ( !found )
                {
                    VTrace << "<" << Q->NodeID() << ">";
                    found = true;
                }
                else
                    VTrace << ",<" << Q->NodeID() << ">";
            }
        }
        VTrace << FlushLine; 

        found = false;
        for (i=0; i<n; i++)
        {
            Query_Quantifier* Q = mQuantifier[i];
            if ( Q->QuantifierType() == QT_A || Q->QuantifierType() == QT_E )
            {
                if ( !found )
                {
                    VTrace << "               WHERE <" << Q->NodeID() << ">";
                    found = true;
                }
                else
                    VTrace << ",<" << Q->NodeID() << ">";
            }
        }
        VTrace << FlushLine;
    }
    PrintChildNodesToTrace(VTrace);
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::IsFromSelect( void )
{
	if ( mFatherNode && mFatherQuantifier )
	{
		if ( mFatherNode->GetNodeType() == NT_Select &&
			mFatherQuantifier->QuantifierType() == QT_F )
			return true;
	}
	return false;
}

bool Query_SelectNode::Semantics( bool UniqueOutputColumnNames, bool ComplexViewReplacement, bool& UnidentifiedColumns )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics", QueryRewrite_Trace, 5);

    if ( !mQueryGraph.IsOk() )
        return false;

	// must be called FIRST !!!
    if ( !Query_Node::Semantics(UniqueOutputColumnNames, ComplexViewReplacement, UnidentifiedColumns) ) 
        return false;

	if ( !Semantics_FromPart() )
        return false;
    if ( !Semantics_SelectList(ComplexViewReplacement) )
        return false;
    if ( !Semantics_WherePart() )
        return false;
    if ( !Semantics_Having() )
        return false;
    if ( !Semantics_GroupBy() )
        return false;
    if ( !Semantics_OrderBy() )
        return false;
	if ( !Semantics_UpdateColumns() )
		return false;

    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
    SyntaxTree[mAPNode].n_querycheck.addElement(qcn_Semantic);
    
    if ( mJoinType == JT_Unknown )
    {
        if ( mFQuantifierCnt == 1 )
            mJoinType = JT_NoJoin;
        else
            mJoinType = JT_Inner;
    }

	mSemanticsDone = true;

	if ( UniqueOutputColumnNames )
        if ( !Semantics_CheckUniqueOutputColumns() )
            return false;

	if ( UnidentifiedColumns )
	{
		mQueryGraph.SetNotYetImplemented("Correlation");
		//UnidentifiedColumns = false;
		//if ( !Semantics_Correlation(this, UnidentifiedColumns) )
			return false;
	}

	UnidentifiedColumns |= mUnidentifiedColumns;

	return (true);
}

bool Query_SelectNode::Semantics_FromPart( void ) 
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_FromPart", QueryRewrite_Trace, 5);
	
	// check for duplicate names 
    SAPDB_Int2 n = mQuantifier.GetSize();
    for ( SAPDB_Int2 i=0; i<n; i++ )
    {
        Query_Quantifier* Q1 = mQuantifier[i];
        if ( Q1->QuantifierType() == QT_F )
        {
            for ( SAPDB_Int2 k=i+1; k<n; k++ )
            {
                Query_Quantifier* Q2 = mQuantifier[k];
                if ( Q2->QuantifierType() == QT_F )
                {
                    if ( Q1->IqualIdentificationNames(Q2) )
                        return false;
                }
            }
        }
    }
    return true;
}

bool Query_SelectNode::Semantics_SelectList( bool ComplexViewReplacement ) 
{ 
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_SelectList", QueryRewrite_Trace, 5);

	// 1. check if each select-colum occurs in exactly one quantifier
    // 2. check if the output column names are unique if it is a fromselect  
    //    or the left select of an union/intersect/difference
    // exactly one time within one of the from-quantifiers and add 
    // information about the quantifier to the column-node

    mOutputColumnCnt = 0;
    if ( mSelectListStartNo > 0 )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

        SQLMan_Identifier RefName, Schema, TableName, ColumnName;

        SAPDB_Int2 n = mpSelectListColumns->GetSize();
        for ( SAPDB_Int2 i=0; i<n; i++ )
        {
            Query_Column* Col = &(*mpSelectListColumns)[i];
            if ( Col->IsOutputColumn() )
            {
                bool function				= true;
                SyntaxTreeNodeNo Node       = Col->GetColumnSyntaxNode();
                RefName = Schema = TableName = ColumnName = SQLMan_Identifier();

                Node = SyntaxTree[Node].n_lo_level;

                // Reference name 
                if ( (SyntaxTree[Node].n_proc == 0) &&
                    (SyntaxTree[Node].n_symb == s_reference_name) )
                {
                    RefName = mQueryGraph.GetIdentifier(Node);
                    Node = SyntaxTree[Node].n_sa_level;
                }
                // Schema
                if ( (SyntaxTree[Node].n_proc == 0) && 
                    (SyntaxTree[Node].n_symb == s_authid) )
                {
                    Schema = mQueryGraph.GetSchemaName(Node);
                }
                // Tablename
                if ( (SyntaxTree[Node].n_proc == 0) &&  
                    (SyntaxTree[Node].n_symb == s_tablename) )
                {
                    TableName = mQueryGraph.GetIdentifier(Node);
                    Node = SyntaxTree[Node].n_sa_level;
                }
                // Columnname
                if ( (SyntaxTree[Node].n_proc == 0) &&  
                        (SyntaxTree[Node].n_symb == s_columnname) )
                {
                    SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);
                    function = false;
                    ColumnName = mQueryGraph.GetIdentifier(Node);

                    SAPDB_Int2 FromTabNode=cak_qrewrite_ref_init, ColumnIndex=cak_qrewrite_ref_init;
                    if ( 1!=Semantics_FindColumnInFromPart(Schema, TableName, ColumnName, FromTabNode, ColumnIndex) )
                        return false;

					if ( IsFromSelect() )
					{
						// DDT : CHECK for column of type LONG as this cannot be handled by the kernel so far

						if ( IsColumnOfTypeLong(FromTabNode, ColumnIndex) )
						{
							mQueryGraph.SetNotYetImplemented("Column of type LONG in FromSelect");
							return false;
						}
					}

					AdjustFromTabNode(FromTabNode);
/* DDT */           if ( !SyntaxTree.ChangeColumnNameToColumnId(Node, FromTabNode, ColumnIndex) )
                        return false;
                    Col->SetColumnInfo(mID, FromTabNode, false, ColumnName, RefName, 1, ColumnIndex);
                    mOutputColumnCnt += 1;
                }
				else	// this 'else' is necessary as ColumnName may be changed to ColumnID before !
				{	
					// ColumnId
					if ( (SyntaxTree[Node].n_proc == 0) &&  
							(SyntaxTree[Node].n_symb == s_columnid) )
					{
						SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);
						function = false;
						SAPDB_Int2 FromTabNode	= SyntaxTree[Node].n_fromtabnode();
						SAPDB_Int2 ColumnIndex	= SyntaxTree[Node].n_columnindex();
						Col->SetColumnInfo(mID, FromTabNode, false, ColumnName, RefName, 1, ColumnIndex);
						mOutputColumnCnt += 1;
					}
				}
                // Asterix
                if ( (SyntaxTree[Node].n_proc == 0) &&  
                    (SyntaxTree[Node].n_symb == s_asterisk) )
                {
                    SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);
                    function = false;
                    SAPDBERR_ASSERT_STATE( RefName == SQLMan_Identifier() );

                    if ( TableName != SQLMan_Identifier() )
                    {   // '(s).t.*'
                        bool found = false;
                        SAPDB_Int2 n = mQuantifier.GetSize();
                        for ( SAPDB_Int2 i=0; i<n; i++ )
                        {
                            if ( mQuantifier[i]->QuantifierType() == QT_F )
                            {
                                Query_Quantifier* Q = mQuantifier[i];
                                if ( Q->CheckQuantifierName(Schema, TableName) )
                                {
									SAPDB_Int2 Cnt = Q->OutputColumns();
									SyntaxTreeNodeNo FromTabNode = Q->NodeRef()->SyntaxNode();

									AdjustFromTabNode(FromTabNode);
/* DDT */							if ( !SyntaxTree.ChangeColumnNameToColumnId(Node, FromTabNode, cak_qrewrite_ref_star) )
										return false;									
									Col->SetColumnInfo(mID, FromTabNode, true, ColumnName, RefName, Cnt, cak_qrewrite_ref_star);
									mOutputColumnCnt += Cnt;
									found = true;
									break;
                                }
                            }
                        }
                        if ( ! found )
                        {
                            // DDT : unknown table name
                            return false;
                        }
                    }
                    else 
                    {   // '*'
                        SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);
                        function = false;
                        SAPDB_Int2 Cnt = 0;
                        SAPDB_Int2 Pos = 0;
                        SyntaxTreeNodeNo FromTabNode = 0;
                        SAPDB_Int2 n = mQuantifier.GetSize();
						for ( SAPDB_Int2 i=0; i<n; i++ )
						{
							Query_Quantifier* Q = mQuantifier[i];
							if ( Q->QuantifierType() == QT_F )
							{
								Pos			= i;
								FromTabNode = Q->NodeRef()->SyntaxNode();
								Cnt		   += Q->OutputColumns();
							}
						}

						AdjustFromTabNode(FromTabNode);
						Col->SetColumnInfo(mID, (mFQuantifierCnt>1?cak_qrewrite_ref_multistar:FromTabNode), true, ColumnName, RefName, Cnt, cak_qrewrite_ref_multistar);
						mOutputColumnCnt += Cnt;
                    }
                }

                // SubQuery within selectlist
                if ( SyntaxTree[Node].n_proc == a63query_spec )
                {
                    function = false;
                    SAPDB_Int2 k = mQuantifier.GetSize();
                    SAPDB_Int2 i = 0;
                    for ( ; i<k; i++ ) // find quantifier
                    {
                        Query_Quantifier* Q = mQuantifier[i];
                        if ( Q->NodeRef()->SyntaxNode() == Node )
                        {
                            SAPDB_Int2 C = Q->OutputColumns();
                            if ( C != 1 )
                            {
                                // DDT: -2013 Select column list too long
                                // is there any reason for this ?
                                return false;
                            }
                            SQLMan_Identifier SColName;
                            if ( Q->GetOutputColumn(0, SColName) )  // 0 or 1 ?
                            {   /* DDT ??? */
//                                SyntaxTreeNodeNo QNo = Q->NodeRef()->SyntaxNode();

								AdjustFromTabNode(Node);
                                Col->SetColumnInfo(mID, /*FromTabNode*/ Node, false, SColName, RefName, 1, cak_qrewrite_ref_init);
                                mOutputColumnCnt += 1;
                                break;
                            }
                            else
                            {
                                return false;
                            }
                        }
                    }
                    if ( i==k )
                    {
                        // ERROR: subquery not found !
                        return false;
                    }
                }

                // function
                if ( function )
                {   
                    // without reference name
                    if ( ComplexViewReplacement && RefName == SQLMan_Identifier() )
                    {
						if ( !mQueryGraph.AddExpressionRefName(Node, mExprCnt) )
							return false;
                    }
                    ++mFunctionOutputColumnCnt;
					++mOutputColumnCnt;
					bool Aggregates = Semantics_CheckSubtreeForAggregates(Node);
					Col->SetColumnInfo(mID, (Aggregates?cak_qrewrite_ref_aggregate:cak_qrewrite_ref_function), false, SQLMan_Identifier(), RefName, 1, mOutputColumnCnt);
                    Semantics_IdentifyColumns(mpSelectListColumns, false, Node, /*ForOrderBy*/false);
                }
            }
        }
    }
	return (mOutputColumnCnt > 0);
}

bool Query_SelectNode::Semantics_WherePart( void ) 
{   
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_WherePart", QueryRewrite_Trace, 5);

	// determine to which quantifier the column belongs
    // only those columns may be changes where no informations has been determiend before 
    // in case of correlation for some of the columns the information will 
    // be added when handling the outer select 

    if ( mSearchConditionStartNo > 0 )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
		SyntaxTreeNodeNo StartNode = SyntaxTree[mSearchConditionStartNo].n_lo_level;
        return Semantics_IdentifyColumns(mpSearchConditionColumns, /*MayBeCorrelated*/true, StartNode, /*ForOrderBy*/false);
    }
	return true;
}

bool Query_SelectNode::Semantics_Having( void ) 
{   
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_Having", QueryRewrite_Trace, 5);

	if ( mHavingStartNo > 0 )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
		SyntaxTreeNodeNo StartNode = SyntaxTree[mHavingStartNo].n_lo_level;
        return Semantics_IdentifyColumns(mpHavingColumns, /*MayBeCorrelated*/true, StartNode, /*ForOrderBy*/false);
    }
    return true; 
}

bool Query_SelectNode::Semantics_Correlation( Query_SelectNode* TopNode, bool& UnidentifiedColumns ) 
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_Correlation", QueryRewrite_Trace, 5);

	// check if one of the subqueries is correlated with this SelectNode

	bool ok = Query_Node::Semantics_Correlation(TopNode, UnidentifiedColumns);

	if ( TopNode != this && mUnidentifiedColumns )
	{
		mUnidentifiedColumns = false;
		if ( mpSearchConditionColumns ) 
			ok &= Semantics_FindCorrelatedColumn(TopNode, mpSearchConditionColumns, mUnidentifiedColumns);
		if ( mpHavingColumns ) 
			ok &= Semantics_FindCorrelatedColumn(TopNode, mpHavingColumns, mUnidentifiedColumns);
		UnidentifiedColumns = mUnidentifiedColumns;
	}
	return ok;
}

bool Query_SelectNode::Semantics_FindCorrelatedColumn( Query_SelectNode* TopNode, Container_Vector<Query_Column>*& pColumnList, bool& Unidentified ) 
{
	SAPDB_Int2 n = pColumnList->GetSize();
	for (SAPDB_Int2 i=0; i<n; i++)
	{
		Query_Column* Col = &(*pColumnList)[i];
		if ( Col->IsUnidentified() )
		{
			SyntaxTreeNodeNo Node = Col->GetColumnSyntaxNode();
		    if ( Node > 0 )
			{
				Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
				SQLMan_Identifier Schema, TableName, ColumnName;

				if ( (SyntaxTree[Node].n_proc == 0) && 
					(SyntaxTree[Node].n_symb == s_authid) )
				{
					Schema = mQueryGraph.GetSchemaName(Node);
				}
				if ( (SyntaxTree[Node].n_proc == 0) &&  
					(SyntaxTree[Node].n_symb == s_tablename) )
				{
					TableName = mQueryGraph.GetIdentifier(Node);
					Node = SyntaxTree[Node].n_sa_level;
				}
				if ( (SyntaxTree[Node].n_proc == 0) &&  
					(SyntaxTree[Node].n_symb == s_columnname) ) 
				{
					ColumnName = mQueryGraph.GetIdentifier(Node);
					SAPDB_Int2 FromTabNode=cak_qrewrite_ref_init, ColumnIndex=cak_qrewrite_ref_init;

					if ( 1==TopNode->Semantics_FindColumnInFromPart(Schema, TableName, ColumnName, FromTabNode, ColumnIndex) )
					{
						if ( FromTabNode > 0 )
						{
							AdjustFromTabNode(FromTabNode);
							if ( !SyntaxTree.ChangeColumnNameToColumnId(Node, FromTabNode, ColumnIndex) )
								return false;
						}
						Col->SetColumnInfo( TopNode->NodeID(), FromTabNode, false, ColumnName, SQLMan_Identifier(), 1, ColumnIndex);
						Col->SetColumnSyntaxNode(Node);
						mIsCorrelated = true;
					}
					else
						Unidentified |= true;
				}
			}
		}
	}
	return true;
}

bool Query_SelectNode::Semantics_GroupBy( void ) 
{   
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_GroupBy", QueryRewrite_Trace, 5);

	if ( mGroupByStartNo > 0 )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
		SyntaxTreeNodeNo StartNode = SyntaxTree[mGroupByStartNo].n_lo_level;
        return Semantics_IdentifyColumns(mpGroupByColumns, false, StartNode, /*ForOrderBy*/false);
    }
    return true; 
}

bool Query_SelectNode::Semantics_OrderBy( void ) 
{   
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_OrderBy", QueryRewrite_Trace, 5);

	if ( mOrderByStartNo > 0 )
    {
/*DDT*/
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
		SyntaxTreeNodeNo StartNode = SyntaxTree[mOrderByStartNo].n_lo_level;
        return Semantics_IdentifyColumns(mpOrderByColumns, false, StartNode, /*ForOrderBy*/true);
    }
    return true; 
}

bool Query_SelectNode::Semantics_UpdateColumns( void ) 
{   
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_UpdateColumns", QueryRewrite_Trace, 5);

	if ( mForUpdateColumnStartNo > 0 )
    {
/*DDT*/
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
		SyntaxTreeNodeNo StartNode = SyntaxTree[mForUpdateColumnStartNo].n_lo_level;
        return Semantics_IdentifyColumns(mpUpdateColumnsColumns, false, StartNode, /*ForOrderBy*/false);
    }
    return true; 
}


// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::Semantics_IdentifyColumns( Container_Vector<Query_Column>*& pColumnList, bool MayBeCorrelated, SyntaxTreeNodeNo Node, bool ForOrderBy)
{	// DDT : RECURSIVE

    //{
    //    SAPDB_Int4 StackFree = RTE_CurrentStackFree();

    //    if ( StackFree < 4096 )
    //        SetError("Stack exhausted", /*PrintToTrace=*/false);
    //}

    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

	while ( Node > 0 )
	{
		if ( (SyntaxTree[Node].n_proc == 0) && 
			((SyntaxTree[Node].n_symb == s_authid) ||
			(SyntaxTree[Node].n_symb == s_tablename) ||
			(SyntaxTree[Node].n_symb == s_columnname) ||
			(SyntaxTree[Node].n_symb == s_columnid)) )
		{
			if ( !Semantics_IdentifySingleColumn(pColumnList, MayBeCorrelated, Node, ForOrderBy) )
				return false;
		}

		if ( SyntaxTree[Node].n_proc == a63query_spec )	// stop at subselects as they define their own operator
			return true;								// DDT : the same for UNIONS ???

		if ( !Semantics_IdentifyColumns(pColumnList, MayBeCorrelated, SyntaxTree[Node].n_lo_level, ForOrderBy) )
			return false;
		Node = SyntaxTree[Node].n_sa_level;
	}
	return true;
}

bool Query_SelectNode::Semantics_IdentifySingleColumn( Container_Vector<Query_Column>*& pColumnList, bool MayBeCorrelated, SyntaxTreeNodeNo& Node, bool ForOrderBy )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_IdentifySingleColumn", QueryRewrite_Trace, 5);

	SQLMan_Identifier Schema, TableName, ColumnName;
	bool SchemaOrTableName = false;

    if ( Node > 0 )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
		SyntaxTreeNodeNo StartNodeForCorrelation = Node;

        if ( (SyntaxTree[Node].n_proc == 0) && 
             (SyntaxTree[Node].n_symb == s_authid) )
        {
            SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);
            Schema = mQueryGraph.GetSchemaName(Node);
			SchemaOrTableName = true;
        }
        if ( (SyntaxTree[Node].n_proc == 0) &&  
             (SyntaxTree[Node].n_symb == s_tablename) )
        {
            SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);
            TableName = mQueryGraph.GetIdentifier(Node);
            Node = SyntaxTree[Node].n_sa_level;
			SchemaOrTableName = true;
        }
        if ( (SyntaxTree[Node].n_proc == 0) &&  
             (SyntaxTree[Node].n_symb == s_columnname) &&
             !SyntaxTree[Node].n_querycheck.includes(qcn_Semantic) ) // not checked before
        {
			if ( ForOrderBy )	// we check the order by subtree
			{					// columnnames are only allowed if we are the top level select 
								// and not the first select of some union
				if ( mFatherNode->GetNodeType() != NT_Top )
				{
					mQueryGraph.SetError("Illegal columnname in order by");
					return false;
				}
			}

            SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);
            ColumnName = mQueryGraph.GetIdentifier(Node);
            SAPDB_Int2 FromTabNode=cak_qrewrite_ref_init, ColumnIndex=cak_qrewrite_ref_init;
			SAPDB_Int2 found = 0;
			if ( ForOrderBy && !SchemaOrTableName )
			{
				found = Semantics_FindOrderColumnInSelectList(ColumnName, FromTabNode, ColumnIndex);
				//if ( found = Semantics_FindOrderColumnInSelectList(ColumnName, FromTabNode, ColumnIndex) )
				//{
				//	if ( FromTabNode < 0 )
				//	{
				//		 we have someting like "select hex(a) as b from t order by b"
				//		 we must change this to "... order by ColumnIndex"
				//		if ( !SyntaxTree.ChangeOrderColumnNameToRefName(Node, Pos, Len) )
				//			return false;
				//	}
				//}
			}	
			if ( !found )
			{
				found += Semantics_FindColumnInFromPart(Schema, TableName, ColumnName, FromTabNode, ColumnIndex);
				if ( !found && MayBeCorrelated )    
				{
					mUnidentifiedColumns = true;

					Query_Column Col = Query_Column(StartNodeForCorrelation, false);
					Col.SetStatus(Query_Column::Unidentified);

					if ( !AppendColumn(pColumnList, Col) )
						return false;

					return true;
				}
			}

			if ( found!=1 )
				return false;

			if ( FromTabNode > 0 )
			{
				AdjustFromTabNode(FromTabNode);
				if ( !SyntaxTree.ChangeColumnNameToColumnId(Node, FromTabNode, ColumnIndex) )
					return false;
			}
            Query_Column Col = Query_Column(Node, false);
            Col.SetColumnInfo( (FromTabNode>=0?mID:cak_qrewrite_ref_init), (FromTabNode>=0?FromTabNode:cak_qrewrite_ref_init), false, ColumnName, SQLMan_Identifier(), 1, ColumnIndex);

            if ( !AppendColumn(pColumnList, Col) )
                return false;
        }
		else
		{
			if ( (SyntaxTree[Node].n_proc == 0) &&  
				(SyntaxTree[Node].n_symb == s_columnid) &&
				!SyntaxTree[Node].n_querycheck.includes(qcn_Semantic) )
			{
				SyntaxTree[Node].n_querycheck.addElement(qcn_Semantic);

				Query_Column Col = Query_Column(Node, false);
				SAPDB_Int2 FromTabNode	= SyntaxTree[Node].n_fromtabnode();
				SAPDB_Int2 QNo			= FromTabNode;
				SAPDB_Int2 ColumnIndex	= SyntaxTree[Node].n_columnindex();
				Col.SetColumnInfo( (QNo>=0?mID:cak_qrewrite_ref_init), (QNo>=0?QNo:cak_qrewrite_ref_init), false, ColumnName, SQLMan_Identifier(), 1, ColumnIndex);

				if ( !AppendColumn(pColumnList, Col) )
	                return false;
			}
			else
			{
				mQueryGraph.SetNotYetImplemented("Unknown n_symb");
				return false;
			}
		}
    }
    return true;
}

SAPDB_Int2 Query_SelectNode::Semantics_FindOrderColumnInSelectList( SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_FindOrderColumnInSelectList", QueryRewrite_Trace, 5);

	SAPDB_Int2 found = 0;
    if ( mpSelectListColumns )
    {
        SAPDB_Int2 cn = mpSelectListColumns->GetSize();
        for ( SAPDB_Int2 i=0; i<cn; i++ )
        {
            Query_Column Col = (*mpSelectListColumns)[i];
			if ( Col.IsOutputColumn() )
			{
				if ( !Col.IsAsterix() )
				{
 					if ( Col.GetReferenceName()  == Column ||
 						 (/*Col.GetReferenceName() == SQLMan_Identifier() &&*/	// DDT : Eigentlich falsch...
 					      Col.GetColumnName()    == Column ) )
					{
						SAPDB_Int2 FromTabNode = Col.GetQuantifierSyntaxNode();
						QuantifierSyntaxNode = FromTabNode;
						ColumnIndex = Col.GetColumnIndex();
						++found;
					}
				}
				else
				{	
				    SAPDB_Int2 FromTabNode = Col.GetQuantifierSyntaxNode();
				    if ( FromTabNode != cak_qrewrite_ref_multistar )
					{
						SAPDB_Int2 QNo=0;
						if ( GetQuantifierNoFromSyntaxNode(FromTabNode, QNo) )
						{
							Query_Quantifier* Q = mQuantifier[QNo];
							if ( Q->QuantifierType() == QT_F )
							{
								if ( Q->CheckForColumn(SQLMan_Identifier(), SQLMan_Identifier(), Column, QuantifierSyntaxNode, ColumnIndex ) )
									++found;
							}
							else
							{
								mQueryGraph.SetError("");
							}
						}
						else
						{
							mQueryGraph.SetError("");
						}
					}
					else
					{
						if ( 1==Semantics_FindColumnInFromPart(SQLMan_Identifier(), SQLMan_Identifier(), Column, QuantifierSyntaxNode, ColumnIndex) )
							++found;
					}
				}
			}
		}
		if ( found > 1 )
		{
			QuantifierSyntaxNode = cak_qrewrite_ref_init;
			ColumnIndex = cak_qrewrite_ref_init;
			// DDT : Column not unique
			mQueryGraph.SetError("Order by : Duplicate Columname");
		}
	}
	return found;
}

SAPDB_Int2 Query_SelectNode::Semantics_FindColumnInFromPart( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_FindColumnInFromPart", QueryRewrite_Trace, 5);

	SAPDB_Int2 QSyntaxNode = cak_qrewrite_ref_init, ColIndex = cak_qrewrite_ref_init;
	SAPDB_Int2 found = 0;
    SAPDB_Int2 n = mQuantifier.GetSize();
    for ( SAPDB_Int2 i=0; i<n; i++ )
    {
        Query_Quantifier* Q = mQuantifier[i];
        if ( Q->QuantifierType() == QT_F )
        {
            if ( Q->CheckForColumn(Schema, Table, Column, QSyntaxNode, ColIndex) )
            {
				QuantifierSyntaxNode = QSyntaxNode;
				ColumnIndex = ColIndex;
				//if ( IsFromSelect() && mFQuantifierCnt>1 ) 
				//	++ColumnIndex;
                ++found;
            }
        }
    }
    if ( found>1 )
    {
        // DDT : Column not unique
		QuantifierSyntaxNode = cak_qrewrite_ref_init;
		ColumnIndex = cak_qrewrite_ref_init;
    }
	return found;
}

void Query_SelectNode::AdjustFromTabNode( SyntaxTreeNodeNo& FromTabNode )
{
	if ( FromTabNode > 0 )
	{
		Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
		if ( SyntaxTree[FromTabNode].n_proc == a63query_spec || 
			(SyntaxTree[FromTabNode].n_proc == a63 && SyntaxTree[FromTabNode].n_subproc() == cak_x_union)		||
			(SyntaxTree[FromTabNode].n_proc == a63 && SyntaxTree[FromTabNode].n_subproc() == cak_x_union_all)	||
			(SyntaxTree[FromTabNode].n_proc == a63 && SyntaxTree[FromTabNode].n_subproc() == cak_x_except)		||
			(SyntaxTree[FromTabNode].n_proc == a63 && SyntaxTree[FromTabNode].n_subproc() == cak_x_except_all)	||
			(SyntaxTree[FromTabNode].n_proc == a63 && SyntaxTree[FromTabNode].n_subproc() == cak_x_intersect) 	||
			(SyntaxTree[FromTabNode].n_proc == a63 && SyntaxTree[FromTabNode].n_subproc() == cak_x_intersect_all) )
			FromTabNode = SyntaxTree[FromTabNode].n_refback;

		if ( SyntaxTree[FromTabNode].n_proc == a63 && SyntaxTree[FromTabNode].n_subproc() == cak_x_start_union )
			FromTabNode = SyntaxTree[FromTabNode].n_refback;
	}
}

bool Query_SelectNode::Semantics_CheckUniqueOutputColumns( void ) 
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::Semantics_CheckUniqueOutputColumns", QueryRewrite_Trace, 5);

	SAPDB_Int2 n = OutputColumns();
    for ( SAPDB_Int2 i=1; i<n; i++ )
    {
        SQLMan_Identifier Col1; 
		if ( GetOutputColumn(i, Col1) )
		{
			for ( SAPDB_Int2 k=i+1; k<=n; k++ )
			{
				SQLMan_Identifier Col2; 
				if ( GetOutputColumn(k, Col2) )
				{
					if ( Col1 == Col2 )
						return false;
				}
				else
				{
					return false;
				}
			}
		}
		else
		{
			return false;
		}
    }
    return true;
}

bool Query_SelectNode::Semantics_CheckSubtreeForAggregates( SyntaxTreeNodeNo StartNode )
{
	return false; /* DDT */
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::IsColumnOfTypeLong( SAPDB_Int2 QuantifierSyntaxNode, SAPDB_Int2 ColumnIndex )
{
	SAPDB_Int2 QNo=0;
	if ( GetQuantifierNoFromSyntaxNode(QuantifierSyntaxNode, QNo) )
	{
		Query_Quantifier* Q = mQuantifier[QNo];
		return Q->NodeRef()->IsColumnOfTypeLong(ColumnIndex);
	}
	return true;
}

bool Query_SelectNode::IsColumnOfTypeLong( SAPDB_Int2 ColumnIndex )
{
	SQLMan_Identifier ColName, RefName;
	SAPDB_Int2 QNode = cak_qrewrite_ref_init, ColIndex = cak_qrewrite_ref_init;
	if ( GetOutputColumn( ColumnIndex, ColName, RefName, QNode, ColIndex ) )
	{
		if ( QNode > 0 )
			return IsColumnOfTypeLong(QNode, ColIndex);
	}
	return true;
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::AppendColumn( Container_Vector<Query_Column>*& pColumnList, Query_Column Col ) 
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::AppendColumn", QueryRewrite_Trace, 5);

	if ( !pColumnList )
    {
        pColumnList = new(mAllocator)  Container_Vector<Query_Column>(mAllocator);
        pColumnList->Reserve(10);
    }
    if ( pColumnList )
    {
        pColumnList->InsertEnd(Col);
        return true;
    }
    else
    {
        mQueryGraph.SetError("Query_SelectNode::AppendColumn");
        return false;
    }
}

bool Query_SelectNode::AppendOutputColumn ( SyntaxTreeNodeNo Node )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_Node::AppendOutputColumn", QueryRewrite_Trace, 5);

    return AppendColumn(mpSelectListColumns, Query_Column(Node, true));
}

SAPDB_Int2 Query_SelectNode::OutputColumns( void )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    // mOutputColumnCnt may vary from mpSelectListColumns->GetSize() when there are '*' within the
    // select list!
    return mOutputColumnCnt;
}

SAPDB_Int2 Query_SelectNode::FunctionOutputColumns( void )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );
    return mFunctionOutputColumnCnt;
}

bool Query_SelectNode::GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& Col )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    SQLMan_Identifier RefName;
    SAPDB_Int2 QuantifierSyntaxNode=0;
    SAPDB_Int2 ColumnIndex=0;

    if ( GetOutputColumn(i, Col, RefName, QuantifierSyntaxNode, ColumnIndex) )
    {
        if ( RefName != SQLMan_Identifier() )
            Col = RefName;
        return true;
    }
    return false;
}

bool Query_SelectNode::GetOutputColumnIndex( SAPDB_Int2 i, SAPDB_Int2& ColumnIndex )
{
    ColumnIndex = i;
    return ( i>0 && i<=mOutputColumnCnt );
}

bool Query_SelectNode::GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& ColName, SQLMan_Identifier& RefName, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    if ( i>0 && i<=mOutputColumnCnt )
    {
        SAPDB_Int2 pos = 1;
        SAPDB_Int2 n = mpSelectListColumns->GetSize();
        for ( SAPDB_Int2 k=0; k<n; k++ )
        {
            Query_Column C = (*mpSelectListColumns)[k];
            if ( C.IsOutputColumn() )
            {
                SAPDB_Int2 ColCnt = C.GetColumnCount();
                if ( pos+ColCnt > i )
                {

					SyntaxTreeNodeNo FromTabNode = C.GetQuantifierSyntaxNode();
					if ( !C.IsAsterix() )
					{
						QuantifierSyntaxNode    = FromTabNode;
						ColumnIndex             = C.GetColumnIndex();
						ColName					= C.GetColumnName();
						RefName					= C.GetReferenceName();
						return true;
					}
					else
					{
						if ( FromTabNode > 0 )
						{
							SAPDB_Int2 QNo=0;
							if ( GetQuantifierNoFromSyntaxNode(FromTabNode, QNo) )
							{
								Query_Quantifier* Q     = mQuantifier[QNo];
								QuantifierSyntaxNode    = Q->NodeRef()->SyntaxNode(); 
								RefName                 = SQLMan_Identifier();
								if ( Q->GetOutputColumn(i-pos+1, ColName) )
									return GetOutputColumnIndex(i-pos+1, ColumnIndex);
								return false;
							}
							else
							{
								// DDT : ERROR - how can this happen ???
								return false; 
							}
						}
						else	// cak_qrewrite_ref_multistar
						{
							SAPDB_Int2 n = mQuantifier.GetSize();
							for ( SAPDB_Int2 i=0; i<n; i++ )
							{
								Query_Quantifier* Q = mQuantifier[i];
								if ( Q->QuantifierType() == QT_F )
								{
									SAPDB_Int2 cn = Q->OutputColumns();
									if ( pos + cn > i )
									{
										QuantifierSyntaxNode    = Q->NodeRef()->SyntaxNode(); 
										RefName                 = SQLMan_Identifier();
										if ( Q->GetOutputColumn(i-pos+1, ColName) )
											return GetOutputColumnIndex(i-pos+1, ColumnIndex);
									}
									pos += cn;
								}
							}
							return false;
						}
					}

					//SyntaxTreeNodeNo FromTabNode = C.GetQuantifierSyntaxNode();
					//if ( FromTabNode > 0 )
					//{
					//	if ( !C.IsAsterix() )
					//	{
					//		QuantifierSyntaxNode    = FromTabNode;
					//		ColumnIndex             = C.GetColumnIndex();
					//		ColName = C.GetColumnName();
					//		RefName = C.GetReferenceName();
					//		return true;
					//	}
					//	else
					//	{
					//		SAPDB_Int2 QNo=0;
					//		if ( GetQuantifierNoFromSyntaxNode(FromTabNode, QNo) )
					//		{
					//			Query_Quantifier* Q     = mQuantifier[QNo];
					//			QuantifierSyntaxNode    = Q->NodeRef()->SyntaxNode(); 
					//			RefName                 = SQLMan_Identifier();
					//			if ( Q->GetOutputColumn(i-pos+1, ColName) )
					//				return GetOutputColumnIndex(i-pos+1, ColumnIndex);
					//			return false;
					//		}
					//		else
					//		{
					//			// DDT : ERROR - how can this happen ???
					//			return false; 
					//		}
					//	}
					//}
					//else
					//{
					//	switch ( FromTabNode )	// DDT
					//	{
					//	case cak_qrewrite_ref_multistar:	// TODO : s. CheckForColumn

					//		break;

					//	case cak_qrewrite_ref_function:
					//	case cak_qrewrite_ref_aggregate:
					//		break;

					//	case cak_qrewrite_ref_init:
					//		return false;
					//		break;
					//	}
					//	return false;
					//}
                }
                pos += ColCnt;
            }
        }
    }
    return false;
}

bool Query_SelectNode::CheckForColumn( SQLMan_Identifier Column, SyntaxTreeNodeNo& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex )
{
    // check if this select has an output column "Column"

	// DDT: TODO : only if the result must have unique output column names there may be only one matching column !
	// select * from t1,t2 may have duplicate column names and therefore
	// select * from t1, t2 where a=7 must through an error if t1 AND t2 contain a column 'a'
	// ERROR !!!

    SAPDBERR_ASSERT_STATE( mSemanticsDone );
    QuantifierSyntaxNode = mAPNode;
    ColumnIndex = 0;

    SAPDB_Int2 n = mpSelectListColumns->GetSize();
    for ( SAPDB_Int2 i=0; i<n; i++ )
    {
        Query_Column Col = (*mpSelectListColumns)[i];
        if ( Col.IsOutputColumn() )
        {
            SAPDB_Int2 FromTabNode = Col.GetQuantifierSyntaxNode();
            if ( FromTabNode != cak_qrewrite_ref_multistar )
            {
                if ( !Col.IsAsterix() )
                {
                    ++ColumnIndex;
                    SQLMan_Identifier RefName = Col.GetReferenceName();
                    if ( RefName == SQLMan_Identifier() )
                    {
                        if ( Column == Col.GetColumnName() )
                            return true;
                    }
                    else
                    {
                        if ( Column == RefName )
                            return true;
                    }
                }
                else
                {
                    SAPDB_Int2 N, Idx;
					SAPDB_Int2 QNo=0;
					if ( GetQuantifierNoFromSyntaxNode(FromTabNode, QNo) )
					{
						if ( mQuantifier[QNo]->CheckForColumn(Column, N, Idx) )
						{
							ColumnIndex += Idx;
							return true;
						}
						else
						{
							ColumnIndex += mQuantifier[QNo]->OutputColumns();
						}
					}
					else
					{
						// DDT : ERROR - how can this happen ???
						return false;
					}
                }
            }
            else
            {
                SAPDB_Int2 m = mQuantifier.GetSize();
                for ( SAPDB_Int2 k=0; k<m; k++ )
                {
                    if ( mQuantifier[k]->QuantifierType() == QT_F )
                    {
                        SAPDB_Int2 N, Idx;
                        if ( mQuantifier[k]->CheckForColumn(Column, N, Idx) )
                        {
                            ColumnIndex += Idx;
                            return true;
                        }
                        else
                        {
                            ColumnIndex += mQuantifier[k]->OutputColumns();
                        }
                    }

                }
            }
        }
    }
    return false;
}

bool Query_SelectNode::CheckForColumn( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    if ( CheckQuantifierName(Schema, Table) )
        return CheckForColumn(Column, QuantifierSyntaxNode, ColumnIndex);
    return false;
}

bool Query_SelectNode::CheckQuantifierName( SQLMan_Identifier Schema, SQLMan_Identifier Table )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    if ( Schema == SQLMan_Identifier() )
    {
		if ( Table != SQLMan_Identifier() )
		{
			Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

			SyntaxTreeNodeNo Node = SyntaxTree[mAPNode].n_sa_level;
			if ( Node>0 )
			{
				if ( (SyntaxTree[Node].n_proc == no_proc) && 
					 (SyntaxTree[Node].n_symb == s_identifier) )
				{
					SQLMan_Identifier IName = mQueryGraph.GetIdentifier(Node);
					if ( IName == Table )
						return true;
				}
			}
			return false;
		}
		return true;
    }
    return false;
}

void Query_SelectNode::GetIdentificationNames(SQLMan_Identifier& Schema, SQLMan_Identifier& Table)
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    Schema = SQLMan_Identifier();
    Table  = mReferenceName;
}

bool Query_SelectNode::HasReferenceName( void )
{
	return ( mReferenceName != SQLMan_Identifier() );
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::SubstituteOneColForAsterix( 
	SyntaxTreeNodeNo& SelColNo,
	SyntaxTreeNodeNo& RefNameNodes,
//	SyntaxTreeNodeNo NewColIDNode,
	SyntaxTreeNodeNo ColQuantNode,
	SAPDB_Int2 ColIndexQuant,
	SAPDB_Int2 ColIndexOwn,
	bool CreateNewNode )
{
	Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
	
	SyntaxTreeNodeNo RefNames = RefNameNodes;
	RefNameNodes = SyntaxTree[RefNameNodes].n_sa_level;

	SyntaxTreeNodeNo NewColIDNode = cak_qrewrite_ref_init;
	SyntaxTreeNodeNo NewA60Node   = cak_qrewrite_ref_init;
	if ( CreateNewNode ) // if not the first one
	{
		NewA60Node	 = SyntaxTree.GetFreeNode();
		NewColIDNode = SyntaxTree.GetFreeNode();
		if ( NewA60Node > 0 )
		{
			SyntaxTree[NewA60Node].n_querycheck.addElement(qcn_Syntax);
			SyntaxTree[NewA60Node].n_refback  = SelColNo;
			SyntaxTree[NewA60Node].n_sa_level = SyntaxTree[SelColNo].n_sa_level;

			if ( SyntaxTree[NewA60Node].n_sa_level > 0 )
				SyntaxTree[SyntaxTree[NewA60Node].n_sa_level].n_refback = NewA60Node;
			SyntaxTree[SelColNo].n_sa_level = NewA60Node;
			SelColNo = NewA60Node;
		}
	}
	else
	{
		NewA60Node   = SelColNo;
		NewColIDNode = SyntaxTree[SelColNo].n_lo_level;
	}

	if ( (NewA60Node > 0) && (NewColIDNode > 0 ) )
	{
		SyntaxTree[NewA60Node  ].n_lo_level  = RefNames;
		SyntaxTree[RefNames    ].n_refback   = NewA60Node;

		SyntaxTree[RefNames    ].n_sa_level = NewColIDNode;
		SyntaxTree[NewColIDNode].n_refback  = RefNames;

		SyntaxTree[NewA60Node].n_proc.becomes(a60);
		SyntaxTree[NewA60Node].n_subproc() = cak_x_select_column;

		SyntaxTree[RefNames    ].n_symb.becomes(s_reference_name);

		SyntaxTree[NewColIDNode].n_querycheck.addElement(qcn_Syntax);
		SyntaxTree[NewColIDNode].n_proc.becomes(no_proc);
		SyntaxTree[NewColIDNode].n_symb.becomes(s_columnid);
		SyntaxTree[NewColIDNode].n_fromtabnode() = ColQuantNode;
		SyntaxTree[NewColIDNode].n_columnindex() = ColIndexQuant + 1; // DDT : +1 ???
		SyntaxTree[NewColIDNode].n_pos()    = SyntaxTree[RefNames].n_pos();
	}
	else
	{
		mQueryGraph.SetError("GetFreeNode");
		return false;
	}

	Query_Column NewCol = Query_Column(NewColIDNode, false);
	SQLMan_Identifier RefName    = mQueryGraph.GetIdentifier(RefNames);
	NewCol.SetColumnInfo( mID, ColQuantNode, false, SQLMan_Identifier(), RefName, 1, ColIndexQuant);
	mpSelectListColumns->Insert(ColIndexOwn, NewCol);

	return true;
}


bool Query_SelectNode::CreateColRefNamesAndResolveAsterix( SyntaxTreeNodeNo& RefNameNodes )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );
	SAPDBERR_ASSERT_ARGUMENT( RefNameNodes > 0 );

	if ( RefNameNodes <= 0 )
		return false;

	Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

	SAPDB_Int2 k = 0;

	while ( RefNameNodes > 0 )
	{
		SAPDBERR_ASSERT_ARGUMENT( k<mOutputColumnCnt );
		if ( k==mOutputColumnCnt )
		{
			// ERROR: more columns within RefNameNodes than OutputColumns ...
			return false;
		}

		if ( (*mpSelectListColumns)[k].IsAsterix() )
		{
			SAPDB_Int2 AsterixColCnt = (*mpSelectListColumns)[k].GetColumnCount();
			// it's an asterix that stands for AxterixColCnt columns
			// so we have to substitute the current columns by the next AsterixColCnt columns from RefNameNodes

			SyntaxTreeNodeNo ColQuantNode = (*mpSelectListColumns)[k].GetQuantifierSyntaxNode();
			if ( ColQuantNode != cak_qrewrite_ref_multistar )
			{
				// the asterix stands for all columns from ONE quantifier

				SyntaxTreeNodeNo SelColNo = (*mpSelectListColumns)[k].GetColumnSyntaxNode();
				mpSelectListColumns->Delete(k);

				for ( SAPDB_Int2 i=0; i<AsterixColCnt; i++ )
				{
					if ( !SubstituteOneColForAsterix( SelColNo, 
												RefNameNodes,
												ColQuantNode,
												i+1,
												k+i,	
												(i?true:false)) )
					{
						return false;
					}
				}
			}
			else
			{
				// the asterix stands for all columns from MORE THAN ONE quantifier

				SyntaxTreeNodeNo SelColNo = (*mpSelectListColumns)[k].GetColumnSyntaxNode();
				mpSelectListColumns->Delete(k);

				SAPDB_Int2 c = 0;
		        SAPDB_Int2 n = mQuantifier.GetSize();

				for ( SAPDB_Int2 q=0; q<n; q++ )
				{
		            Query_Quantifier* Q = mQuantifier[q];
					if ( Q->QuantifierType() == QT_F )
		            {
						SyntaxTreeNodeNo ColQuantNode	= Q->NodeRef()->SyntaxNode();
						SAPDB_Int2 qc = Q->OutputColumns();
						for ( SAPDB_Int2 j=0; j<qc; j++ )
						{
							if ( SubstituteOneColForAsterix( SelColNo,
														RefNameNodes,
														ColQuantNode,
														j+1,
														k+c,	
														(c?true:false)) )
							{
								++c;
							}
							else
							{
								return false;
							}
						}
					}
				}

				if ( c != AsterixColCnt )
				{
					mQueryGraph.SetError("ColumnMismatch");
					return false;
				}
			}
//			ColumnCnt += AsterixColCnt-1; // one was already there...
			k += AsterixColCnt;
		}
		else
		{
			SyntaxTreeNodeNo ColQuantNode	= (*mpSelectListColumns)[k].GetQuantifierSyntaxNode();
			SyntaxTreeNodeNo SelColNo		= (*mpSelectListColumns)[k].GetColumnSyntaxNode();
			
			SyntaxTreeNodeNo Help			= SyntaxTree[SelColNo].n_lo_level;

			if ( (SyntaxTree[Help].n_proc == no_proc) && 
				 (SyntaxTree[Help].n_symb == s_reference_name) )
			{	// we already have a reference name

				// just change the name
				SyntaxTree[Help].n_pos()    = SyntaxTree[RefNameNodes].n_pos();
				SyntaxTree[Help].n_length() = SyntaxTree[RefNameNodes].n_length();

				// and discard the other node
				Help = RefNameNodes;
				RefNameNodes = SyntaxTree[RefNameNodes].n_sa_level;
				mQueryGraph.SyntaxTree().SetUnusedNode(Help, /*Recursive=*/false);
			}
			else	// were is no reference name yet
			{
				SyntaxTreeNodeNo Next = SyntaxTree[RefNameNodes].n_sa_level;

				// link the RefNameNode
				SyntaxTree[RefNameNodes].n_querycheck.addElement(qcn_Syntax);
				SyntaxTree[RefNameNodes].n_proc.becomes(no_proc);
				SyntaxTree[RefNameNodes].n_symb.becomes(s_reference_name);

				SyntaxTree[SelColNo    ].n_lo_level = RefNameNodes;
				SyntaxTree[RefNameNodes].n_refback  = SelColNo;
				SyntaxTree[RefNameNodes].n_sa_level = Help;
				SyntaxTree[Help        ].n_refback  = RefNameNodes;

				SQLMan_Identifier NewRefName = mQueryGraph.GetIdentifier(RefNameNodes);
				(*mpSelectListColumns)[k].SetReferenceName(NewRefName);

				RefNameNodes = Next;
			}
			++k;
		}
	}  // while

	if ( (k!=mOutputColumnCnt) || (RefNameNodes>0) )
	{
		// ERROR
		return false;
	}

	return true;
}

SyntaxTreeNodeNo Query_SelectNode::GetRefColNo(SyntaxTreeNodeNo ColNodeStart, SAPDB_Int2 ColIdx)
{
	Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
	SAPDB_Int2 cnt = 0;
    while ( (ColNodeStart = SyntaxTree[ColNodeStart].n_sa_level) > 0 )
    {
        if ( ++cnt == ColIdx )
			return ColNodeStart;
    }
	return cak_qrewrite_ref_init;
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::RemoveSubSelectNode( SyntaxTreeNodeNo SubSelNode, Query_Position PosType, bool ReplaceByChildren )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::RemoveSubSelectNode", QueryRewrite_Trace, 6);

    return mQueryGraph.SyntaxTree().RemoveSubSelectNode(SubSelNode);
}

bool Query_SelectNode::AddSubSelectNode( SyntaxTreeNodeNo SelNode, SyntaxTreeNodeNo FatherNode, Query_Position FatherPosType, SAPDB_Int2 Pos )
{
    // Adds the already existing SubSelect node identified by SubSelNode as a subselect to FatherNode as children number Pos !

    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::AddSubSelectNode", QueryRewrite_Trace, 6);

    if ( mFatherNode->GetNodeType() == NT_Select )
        return mFatherNode->AddSubSelectNode(SelNode, FatherNode, FatherPosType, Pos);

    return mQueryGraph.SyntaxTree().AddSubSelectNode( mAPNode, SelNode, FatherNode, Pos );
}

bool Query_SelectNode::RemoveDistinctNode( void )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::RemoveDistinctNode", QueryRewrite_Trace, 6);

    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
    SyntaxTreeNodeNo Dist = SyntaxTree[mAPNode].n_lo_level;
    if ( (SyntaxTree[Dist].n_proc == a63) &&
         (SyntaxTree[Dist].n_subproc() == cak_x_distinct) )
    {
        SyntaxTree[SyntaxTree[Dist].n_lo_level].n_refback = mAPNode;
        SyntaxTree[mAPNode].n_lo_level = SyntaxTree[Dist].n_lo_level;
        SyntaxTree[Dist].n_sa_level = 0;
        SyntaxTree[Dist].n_lo_level = 0;
        SyntaxTree[Dist].n_refback = 0;
        SyntaxTree.SetUnusedNode(Dist);
    }
    mAllFlg = true;
    return true;
}

bool Query_SelectNode::AddDistinctNode( void )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::AddDistinctNode", QueryRewrite_Trace, 6);

    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
    if ( SyntaxTreeNodeNo Dist = SyntaxTree.GetFreeNode() ) 
    {
		SyntaxTree[Dist].n_querycheck.addElement(qcn_Syntax);
        SyntaxTree[Dist].n_proc.becomes(a63);
        SyntaxTree[Dist].n_subproc()    = cak_x_distinct;
        SyntaxTree[Dist].n_querycheck.addElement(qcn_Semantic);

        SyntaxTree[Dist].n_lo_level     = SyntaxTree[mAPNode].n_lo_level;
        SyntaxTree[SyntaxTree[Dist].n_lo_level].n_refback = Dist;
        SyntaxTree[mAPNode].n_lo_level  = Dist;
        SyntaxTree[Dist].n_refback      = mAPNode;

        mAllFlg = false;
        return true;
    }
    return false;
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::PullDownFQuantifierBegin( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier )
{
    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
    SyntaxTreeNodeNo Node = SyntaxTree[FromPartStart()].n_lo_level;

    SyntaxTree[FromPartStart()].n_lo_level  = FromQuantifier;
    SyntaxTree[FromQuantifier ].n_refback   = FromPartStart();

    SyntaxTreeNodeNo N2 = FromQuantifier;
    while ( SyntaxTree[N2].n_sa_level > 0 )
    {
        N2 = SyntaxTree[N2].n_sa_level;
    }

    SyntaxTree[N2   ].n_sa_level  = Node;
    SyntaxTree[Node ].n_refback   = N2;

    return mQuantifier.InsertFront(Q);
}

bool Query_SelectNode::PullDownFQuantifierEnd( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier )
{
    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
    SyntaxTreeNodeNo Node = SyntaxTree[FromPartStart()].n_lo_level;

    while ( SyntaxTree[Node].n_sa_level > 0 )
    {
        Node = SyntaxTree[Node].n_sa_level;
    }
    SyntaxTree[Node          ].n_sa_level    = FromQuantifier;
    SyntaxTree[FromQuantifier].n_refback     = Node;

    return mQuantifier.InsertEnd(Q);
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::PushDownSearchCond( void )
{   // push down my search condition to child
    // TODO: Split search condition and push it down to those fquantifiers the parts belong to

    if ( mFQuantifierCnt==1 )
    {

        SAPDB_Int2 pos=-1, n = mQuantifier.GetSize();
        for ( SAPDB_Int2 i=0; i<n; i++ )
        {
            Query_Quantifier* Q = mQuantifier[i];
            if ( Q->QuantifierType() == QT_F && Q->NodeTypeSelectOrSet() )
            {
                pos = i;
                break;
            }
        }

        if ( pos >= 0 )
        {
            SyntaxTreeNodeNo SearchCond = UnlinkSearchCond();
			bool rc = mQuantifier[pos]->PullDownSearchCond(mpSearchConditionColumns, SearchCond);
			if ( rc )
				mpSearchConditionColumns = 0;
			return rc;
        }
        // do not return false here !!! this is needed by PullDownSearchCond !
    }
    return true;
}

bool Query_SelectNode::PullDownSearchCond( Container_Vector<Query_Column>*& Columns, SyntaxTreeNodeNo SearchCond  )
{   
    return AddSearchCond(Columns, SearchCond);
}

// ------------------------------------------------------------------------------------------------

bool Query_SelectNode::PullUpSearchCond( Query_SelectNode* Lower  )
{
    SyntaxTreeNodeNo SearchCond = Lower->UnlinkSearchCond();

    // adjust column and tables names within qualification part !
    //SQLMan_Identifier RefName = Lower->GetReferenceName();
    //bool RefNameFlg = false;
    //if ( RefName != SQLMan_Identifier() )
    //{
    //    RefNameFlg = true;
    //}

    Container_Vector<Query_Column>* Cols = 0;
    // 
    return AddSearchCond(Cols, SearchCond); // DDT 
}

// ------------------------------------------------------------------------------------------------

SyntaxTreeNodeNo Query_SelectNode::UnlinkSearchCond( void )
{
    SyntaxTreeNodeNo SearchStart = SearchConditionStart();
    if ( SearchStart > 0 )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
        SyntaxTree.UnlinkNode(SearchStart, /*SetUnsused*/ false);
        SetSearchConditionStart(cak_qrewrite_ref_init);
    }
    return SearchStart;
}

bool Query_SelectNode::AddSearchCond( Container_Vector<Query_Column>*& Columns, SyntaxTreeNodeNo AddSearchCond )
{
    if ( AddSearchCond > 0 )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

        if ( Columns )
        {
            SAPDB_Int2 n = Columns->GetSize();
            for ( SAPDB_Int2 i=0; i<n; i++ )
            {
                if ( !MapColumnFromOuterToInnerSelect((*Columns)[i]) )
                {
					mQueryGraph.SetError("MapColumns");
                    return false;
                }
				if ( mpSearchConditionColumns )
					mpSearchConditionColumns->InsertEnd((*Columns)[i]);
            }
			if ( !mpSearchConditionColumns )
				mpSearchConditionColumns = Columns;
        }

        SyntaxTreeNodeNo SearchCond = SearchConditionStart();
        if ( SearchCond > 0 )
        {
            // create AND-Node
            SyntaxTreeNodeNo AndNode = SyntaxTree.GetFreeNode();
            if ( AndNode > 0 )
            { 
				SyntaxTree[AndNode].n_querycheck.addElement(qcn_Syntax);
                SyntaxTree[AndNode].n_proc.becomes(no_proc);
                SyntaxTree[AndNode].n_subproc()     = 0; 
                SyntaxTree[AndNode].n_symb.becomes(s_and);
                SyntaxTree[AndNode].n_sa_level      = 0; 
                SyntaxTree[AndNode].n_querycheck.addElement(qcn_Syntax);

                SyntaxTreeNodeNo Predicates    = SyntaxTree[   SearchCond].n_lo_level;
                SyntaxTreeNodeNo AddPredicates = SyntaxTree[AddSearchCond].n_lo_level;
            
                SyntaxTree.UnlinkNode(Predicates, /*SetUnused*/ false);
                SyntaxTree.UnlinkNode(AddSearchCond, /*SetUnused*/ false);
                SyntaxTree.UnlinkNode(AddPredicates, /*SetUnused*/ false);
                SyntaxTree.SetUnusedNode(AddSearchCond);

                SyntaxTree.SetSucessorOnLowLevel(SearchCond, AndNode);
                SyntaxTree.SetSucessorOnLowLevel(AndNode, Predicates);
                SyntaxTree.SetSucessorOnLowLevel(AndNode, AddPredicates);
                SyntaxTree.Changes() = true;

                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
            SyntaxTree.SetSucessorOnSameLevel(FromPartStart(), AddSearchCond);
            SetSearchConditionStart(AddSearchCond);
            SyntaxTree.Changes() = true;

            return true;
        }
    }
    return true;
}

bool Query_SelectNode::MapColumnFromOuterToInnerSelect( Query_Column& Column )
{
    // Column comes from the outer select and is pushed down to 'this'
    // Column.ColumnIndex  > No of outputcolum of 'this'

    SQLMan_Identifier ColName, RefName;
    SAPDB_Int2 QuantifierSyntaxNode=0; 
    SAPDB_Int2 ColumnIndex=0;

    if ( GetOutputColumn(Column.GetColumnIndex(), ColName, RefName, QuantifierSyntaxNode, ColumnIndex) )
    {
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

        // adjust the Column attributes
        Column.ChangeColumnInfo(mID, ColName, RefName, QuantifierSyntaxNode, ColumnIndex);

		if ( QuantifierSyntaxNode != cak_qrewrite_ref_function &&
			 QuantifierSyntaxNode != cak_qrewrite_ref_aggregate )
		{
			// adjust the SyntaxNode attributes
			SyntaxTreeNodeNo H              = Column.GetColumnSyntaxNode();
			SyntaxTree[H].n_fromtabnode()   = QuantifierSyntaxNode;
			SyntaxTree[H].n_columnindex()   = ColumnIndex;
	        return true;
		}
		else
		{
			if ( QuantifierSyntaxNode == cak_qrewrite_ref_function )
			{
			}
			else	// cak_qrewrite_ref_aggregate
			{
			}
		}
    }
    return false;
}

bool Query_SelectNode::IsCorrelatedSubquery( SyntaxTreeNodeNo QuantifierSyntaxNode )
{
    SAPDB_Int2 n = mQuantifier.GetSize();
    for ( SAPDB_Int2 i=0; i<n; i++ )
    {
        if ( mQuantifier[i]->NodeRef()->SyntaxNode() == QuantifierSyntaxNode )
            return mQuantifier[i]->IsCorrelated();
    }
    return true;
}

bool Query_SelectNode::CheckForLocalPredicate( SyntaxTreeNodeNo StartNode, SyntaxTreeNodeNo& QuantifierSyntaxNode )
{
    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

    if ( SyntaxTree[StartNode].n_proc == a63query_spec )
    {
        // DDT : stop at subqueries - return false if the subquery could be correlated else true...
        return !IsCorrelatedSubquery(StartNode);
    }

    if ( (SyntaxTree[StartNode].n_proc == no_proc) &&
         (SyntaxTree[StartNode].n_symb == s_columnname) )
         return false;  // this should not happen if all s_columnname were replaced by s_columnid

    if ( (SyntaxTree[StartNode].n_proc == no_proc) &&
         (SyntaxTree[StartNode].n_symb == s_columnid) )
    {
        if ( QuantifierSyntaxNode > 0 )
        {
            if ( QuantifierSyntaxNode != SyntaxTree[StartNode].n_fromtabnode() )
                return false;
        }
        else
        {
            QuantifierSyntaxNode = SyntaxTree[StartNode].n_fromtabnode();
        }
    }

    if ( SyntaxTree[StartNode].n_lo_level > 0 )
        if ( !CheckForLocalPredicate(SyntaxTree[StartNode].n_lo_level, QuantifierSyntaxNode) )
            return false;

    StartNode = SyntaxTree[StartNode].n_sa_level;
    while ( StartNode > 0 )
    {
        if ( !CheckForLocalPredicate(StartNode, QuantifierSyntaxNode) )
            return false;
    }
    return true;
}

// ------------------------------------------------------------------------------------------------
