/*!***************************************************************************

  module      : Query_NodeSelect.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef Query_NodeSelect_H
#define Query_NodeSelect_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SQLManager/Query/Query_Node.hpp"

// ------------------------------------------------------------------------------------------------

class Query_SelectNode : public Query_Node
{
protected:    // private copyconstuctor used by CreateDuplicate
    Query_SelectNode( const Query_SelectNode& N, Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap );
public:
    virtual Query_Node* CreateDuplicate( Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap );

    Query_SelectNode( Query_ManipulationContext& Context, Query_QueryGraph& QueryGraph, 
                      Query_Position Position, SyntaxTreeNodeNo APNode, bool All ); 

    virtual ~Query_SelectNode( void );

    virtual Query_NodeType GetNodeType( void )	{ return NT_Select; }
    virtual Query_JoinType GetJoinType( void )		{ return mJoinType; }
    virtual bool OuterJoin( void )					{ return  ((mJoinType==JT_Unknown) || (mJoinType&JT_Outer)); }
    virtual void SetOuterJoin( Query_JoinType JT )	{ mJoinType |= JT; }
    virtual void SetForReuse( void )				{ mForReuse = true; }
	virtual void SetGivenSequence( void )			{ mGivenSequence = true; }

    virtual bool Aggregates( void )					{ return ( mCountFlg || mSumFlg || mAvgFlg || mMinFlg ||mMaxFlg ); }

    virtual void PrintToTrace( Kernel_VTrace& VTrace );
    virtual bool Semantics( bool UniqueOutputColumnNames, bool ComplexViewReplacement, bool& UnidentifiedColumns );  
    virtual bool Semantics_Correlation( Query_SelectNode* TopNode, bool& UnidentifiedColumns );

    virtual bool AppendColumn( Container_Vector<Query_Column>*& pColumnList, Query_Column Col );
    virtual bool AppendOutputColumn ( SyntaxTreeNodeNo Node );
    virtual SAPDB_Int2 OutputColumns( void );
    virtual SAPDB_Int2 FunctionOutputColumns( void );
    virtual bool GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& ColName );
    virtual bool GetOutputColumnIndex( SAPDB_Int2 i, SAPDB_Int2& ColumnIndex );
    virtual bool CheckForColumn( SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
    virtual bool CheckForColumn( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
    virtual bool CheckQuantifierName( SQLMan_Identifier Schema, SQLMan_Identifier Table );
    virtual void GetIdentificationNames(SQLMan_Identifier& Schema, SQLMan_Identifier& Table);
	virtual bool HasReferenceName( void );
    
	virtual bool IsCorrelated( void ) { return mIsCorrelated; }
    virtual bool HasUnidentifiedColumns( void ) { return mUnidentifiedColumns; }

	virtual bool IsColumnOfTypeLong( SAPDB_Int2 ColumnIndex );

    virtual bool PullDownFQuantifierBegin( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier );
    virtual bool PullDownFQuantifierEnd( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier );
    virtual bool PullDownSearchCond( Container_Vector<Query_Column>*& Columns, SyntaxTreeNodeNo SearchCond );
    bool         PushDownSearchCond( void );
    bool         PullUpSearchCond( Query_SelectNode* Lower );
    SyntaxTreeNodeNo UnlinkSearchCond( void );
    bool         AddSearchCond( Container_Vector<Query_Column>*& Columns, SyntaxTreeNodeNo SearchCond );

	virtual bool CreateColRefNamesAndResolveAsterix( SyntaxTreeNodeNo& RefNameNodes ) ;

    virtual bool RemoveSubSelectNode( SyntaxTreeNodeNo SubSelNode, Query_Position PosType, bool ReplaceByChildren );
    virtual bool AddSubSelectNode( SyntaxTreeNodeNo SubSelNode, SyntaxTreeNodeNo FatherNode, Query_Position FatherPosType, SAPDB_Int2 Pos );
    bool RemoveDistinctNode( void );
    bool AddDistinctNode( void );

    virtual SyntaxTreeNodeNo	SelectListStart( void )							{ return mSelectListStartNo; }
    virtual void				SetSelectListStart( SyntaxTreeNodeNo Start )	{ mSelectListStartNo = Start; }
    virtual SyntaxTreeNodeNo	FromPartStart( void )							{ return mFromPartStartNo; }
    virtual void				SetFromPartStart( SyntaxTreeNodeNo Start )		{ mFromPartStartNo = Start; }

    virtual SyntaxTreeNodeNo	SearchConditionStart( void )					{ return mSearchConditionStartNo; }
    virtual void				SetSearchConditionStart( SyntaxTreeNodeNo Start ){ mSearchConditionStartNo = Start; }
    virtual bool				HasSearchCondition( void )						{ return (mSearchConditionStartNo > 0); }
    virtual SyntaxTreeNodeNo	OrderByStart( void )							{ return mOrderByStartNo; }
    virtual void				SetOrderByStart( SyntaxTreeNodeNo Start )       { mOrderByStartNo = Start; }
    virtual bool				HasOrderBy( void )								{ return (mOrderByStartNo > 0); }
    virtual SyntaxTreeNodeNo	GroupByStart( void )							{ return mGroupByStartNo; }
    virtual void				SetGroupByStart( SyntaxTreeNodeNo Start )       { mGroupByStartNo = Start; }
    virtual bool				HasGroupBy( void )								{ return (mGroupByStartNo > 0); }
    virtual SyntaxTreeNodeNo	HavingStart( void )								{ return mHavingStartNo; }
    virtual void				SetHavingStart( SyntaxTreeNodeNo Start )        { mHavingStartNo = Start; }
    virtual bool				HasHaving( void )								{ return (mHavingStartNo > 0); }
    virtual SyntaxTreeNodeNo	UpdateColumnsStart( void )						{ return mForUpdateColumnStartNo; }
    virtual void				SetUpdateColumnsStart( SyntaxTreeNodeNo Start ) { mForUpdateColumnStartNo = Start; }
    virtual bool				HasForUpdate( void )							{ return (mForUpdateColumnStartNo > 0); }

    bool& All( void )     { return mAllFlg; }

    bool& Count( void )   { return mCountFlg; }
    bool& Sum( void )     { return mSumFlg; }
    bool& Avg( void )     { return mAvgFlg; }
    bool& Min( void )     { return mMinFlg; }
    bool& Max( void )     { return mMaxFlg; }

    Container_Vector<Query_Column>*& GetColumns( void ) { return mpSelectListColumns; }

protected:

	bool IsFromSelect( void );

	bool Semantics_FromPart( void );
    bool Semantics_SelectList( bool ComplexViewReplacement );
    bool Semantics_WherePart( void );
    bool Semantics_Having( void );
    bool Semantics_GroupBy( void );
    bool Semantics_OrderBy( void );
	bool Semantics_UpdateColumns( void );
    bool Semantics_CheckUniqueOutputColumns( void );
	bool Semantics_CheckSubtreeForAggregates( SyntaxTreeNodeNo StartNode );

    bool Semantics_IdentifyColumns( Container_Vector<Query_Column>*& pColumnList, bool MayBeCorrelated, SyntaxTreeNodeNo Node, bool ForOrderBy );
    bool Semantics_IdentifySingleColumn( Container_Vector<Query_Column>*& pColumnList, bool MayBeCorrelated, SyntaxTreeNodeNo& Node, bool ForOrderBy );
    SAPDB_Int2 Semantics_FindColumnInFromPart( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
	SAPDB_Int2 Semantics_FindOrderColumnInSelectList( SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
	bool Semantics_FindCorrelatedColumn( Query_SelectNode* TopNode, Container_Vector<Query_Column>*& pColumnList, bool& Unidentified );

	bool IsColumnOfTypeLong( SAPDB_Int2 QuantifierSyntaxNode, SAPDB_Int2 ColumnIndex );

	bool GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& ColName, SQLMan_Identifier& RefName, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
    bool MapColumnFromOuterToInnerSelect( Query_Column& Column );
    bool CheckForLocalPredicate( SyntaxTreeNodeNo StartNode, SyntaxTreeNodeNo& QuantifierSyntaxNode );
    bool IsCorrelatedSubquery( SyntaxTreeNodeNo QuantifierSyntaxNode );

	bool SubstituteOneColForAsterix( SyntaxTreeNodeNo& SelColNo, SyntaxTreeNodeNo& RefNameNodes, /*SyntaxTreeNodeNo NewColIDNode,*/ SyntaxTreeNodeNo ColQuantNode, SAPDB_Int2 ColIndexQuant, SAPDB_Int2 ColIndexOwn, bool CreateNewNode );

	void AdjustFromTabNode( SyntaxTreeNodeNo& FromTabNode );

	SyntaxTreeNodeNo GetRefColNo( SyntaxTreeNodeNo ColNodeStart, SAPDB_Int2 ColIdx );

    // Attributes

    SQLMan_Identifier                   mReferenceName;

    Container_Vector<Query_Column>*     mpSelectListColumns;  
    SAPDB_Int2                          mOutputColumnCnt;
    SAPDB_Int2                          mFunctionOutputColumnCnt;
    Container_Vector<Query_Column>*     mpSearchConditionColumns;  
    Container_Vector<Query_Column>*     mpHavingColumns;  
    Container_Vector<Query_Column>*     mpGroupByColumns;  
    Container_Vector<Query_Column>*     mpOrderByColumns;  
	Container_Vector<Query_Column>*     mpUpdateColumnsColumns;

    SyntaxTreeNodeNo    mSelectListStartNo;
    SyntaxTreeNodeNo    mFromPartStartNo;
    SyntaxTreeNodeNo    mSearchConditionStartNo;
    SyntaxTreeNodeNo    mOrderByStartNo;
    SyntaxTreeNodeNo    mGroupByStartNo;
    SyntaxTreeNodeNo    mHavingStartNo;
	SyntaxTreeNodeNo    mForUpdateColumnStartNo;

    Query_JoinType  mJoinType;
	SAPDB_Int2 mExprCnt;
	bool      mAllFlg;
	bool      mForReuse;
	bool      mGivenSequence;

    bool      mIsCorrelated;
	bool      mUnidentifiedColumns;

    bool      mCountFlg;
    bool      mSumFlg;
    bool      mAvgFlg;
    bool      mMinFlg;
    bool      mMaxFlg;
};

// ------------------------------------------------------------------------------------------------

#endif