/*!***************************************************************************

  module      : Query_NodeSet.cpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#include "SAPDB/SQLManager/Query/Query_NodeSet.hpp"
#include "SAPDB/SQLManager/Query/Query_Graph.hpp"
#include "SAPDB/SQLManager/Query/Query_Quantifier.hpp"

// ------------------------------------------------------------------------------------------------

Query_SetNode::Query_SetNode( Query_ManipulationContext& Context, Query_QueryGraph& QueryGraph, Query_Position Position, SyntaxTreeNodeNo APNode, bool All )
    : Query_Node(Context, QueryGraph, Position, APNode, (All?false:true), (All?DT_Preserve:DT_Enforce))
    , mAllFlg(All)
    , mReferenceName()
{
}

// copy constructor
Query_SetNode::Query_SetNode( const Query_SetNode& N, Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap )
    : Query_Node(N, Father, Q, mpMap) 
{
}

// ------------------------------------------------------------------------------------------------

bool Query_SetNode::OuterJoin( void )
{
    Query_Quantifier*& Q1 = GetQuantifier()[0];
    Query_Quantifier*& Q2 = GetQuantifier()[1];

    return (Q1->OuterJoin() || Q2->OuterJoin());
}

bool Query_SetNode::Aggregates( void )
{
    Query_Quantifier*& Q1 = GetQuantifier()[0];
    Query_Quantifier*& Q2 = GetQuantifier()[1];

    return (Q1->Aggregates() || Q2->Aggregates());
}

// ------------------------------------------------------------------------------------------------

bool Query_SetNode::PullDownFQuantifierBegin( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier )
{
    if ( FromQuantifier > 0 )
    {   
        SyntaxTreeNodeNo FQ[2];
        Query_Quantifier* Quantifier[2];
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
        FQ[0] = FromQuantifier;
        FQ[1] = SyntaxTree.DoubleTreePart(FromQuantifier);
        Quantifier[0] = Q;
        Quantifier[1] = Q; /* DDT : ERROR */

        if ( FQ[1] > 0 )
        {
            bool rc = true;
            for (int i=0; i<2; i++)
            {
                Query_Node* Node = GetQuantifier()[i]->NodeRef();
                switch ( Node->GetNodeType() )
                {
                case NT_Select:
                case NT_Union:
                case NT_Intersect:
                case NT_Difference:
                    rc &= Node->PullDownFQuantifierBegin(Quantifier[i], FQ[i]);
                    break;
                default:
                    mQueryGraph.SetError("Query_SetNode::PushDownFQuantifierBegin");
                    rc = false;
                    break;
                }
            }
            return rc;
        }
        return false;
    }
    return true;
}

bool Query_SetNode::PullDownFQuantifierEnd( Query_Quantifier* Q, SyntaxTreeNodeNo FromQuantifier )
{
    if ( FromQuantifier > 0 )
    {   
        SyntaxTreeNodeNo FQ[2];
        Query_Quantifier* Quantifier[2];
        Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
        FQ[0] = FromQuantifier;
        FQ[1] = SyntaxTree.DoubleTreePart(FromQuantifier);
        Quantifier[0] = Q;
        Quantifier[1] = Q; /* DDT : ERROR */
        
        if ( FQ[1] > 0 )
        {
            bool rc = true;
            for (int i=0; i<2; i++)
            {
                Query_Node* Node = GetQuantifier()[i]->NodeRef();
                switch ( Node->GetNodeType() )
                {
                case NT_Select:
                case NT_Union:
                case NT_Intersect:
                case NT_Difference:
                    rc &= Node->PullDownFQuantifierEnd(Quantifier[i], FQ[i]);
                    break;
                default:
                    mQueryGraph.SetError("Query_SetNode::PushDownFQuantifierEnd");
                    rc = false;
                    break;
                }
            }
            return rc;
        }
        return false;
    }
    return true;
}

// ------------------------------------------------------------------------------------------------

bool Query_SetNode::PullDownSearchCond( Container_Vector<Query_Column>*& Columns, SyntaxTreeNodeNo SearchCond )
{
	if ( SearchCond > 0 )
    {   
		bool rc = true;

		Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
        SyntaxTreeNodeNo SearchConditions[2];
        Container_Vector<Query_Column>* Cols[2];

        SearchConditions[0] = SearchCond;
        Cols[0] = Columns;

		Query_NodeMapper* mpMapper = new(mContext.GetAllocator()) Query_NodeMapper(mContext.GetAllocator());
		if ( mpMapper )
		{
			SAPDB_Int2 n = Columns->GetSize();

			for ( SAPDB_Int2 i=0; i<n; i++ )
			{
				// Set adustment flag for all columnnodes
				SyntaxTree[(*Columns)[i].GetColumnSyntaxNode()].n_querycheck.addElement(qcn_NeedAdjustment);
			}

			SearchConditions[1] = SyntaxTree.DoubleTreePart(SearchCond, mpMapper);
			if ( SearchConditions[1] > 0 )
			{
				Cols[1] = new(mAllocator)  Container_Vector<Query_Column>(mAllocator);
				if ( Cols[1] )
				{
					Cols[1]->Reserve(n);
					for ( SAPDB_Int2 i=0; i<n; i++ )
					{
						Query_Column Col = (*(Cols[0]))[i];

						// adjust the columnnodenumber 
						SyntaxTreeNodeNo Source = Col.GetColumnSyntaxNode();
						SyntaxTreeNodeNo Target = mpMapper->GetDestNodeFromSrcNode(Source);
						Col.SetColumnSyntaxNode(Target);
						/// reset the adjustmentflag
						SyntaxTree[Source].n_querycheck.delElement(qcn_NeedAdjustment);
						SyntaxTree[Target].n_querycheck.delElement(qcn_NeedAdjustment);
						Cols[1]->InsertEnd(Col);
					}
					
					for (int k=0; (k<2) && rc; k++)
					{
						Query_Quantifier* Q = GetQuantifier()[k];
						if ( Q->NodeTypeSelectOrSet() )
						{
							rc &= Q->PullDownSearchCond(Cols[k], SearchConditions[k]);
						}
						else
						{
							mQueryGraph.SetError("Query_SetNode::PullDownSearchCond");
							rc = false;
						}
					}
				}
				else
				{
					rc = false;
				}
			}
			else
			{
				rc = false;
			}
			destroy(mpMapper, mContext.GetAllocator());
		}
		else
		{
			rc = false;
		}
		return rc;
    }
	return true;
}

// ------------------------------------------------------------------------------------------------

bool Query_SetNode::CreateColRefNamesAndResolveAsterix( SyntaxTreeNodeNo& RefNameNodes )
{
    if ( !mQueryGraph.IsOk() )
        return false;

	Query_Quantifier* Q1 = GetQuantifier()[0];
	return Q1->NodeRef()->CreateColRefNamesAndResolveAsterix(RefNameNodes);
}

// ------------------------------------------------------------------------------------------------

bool Query_SetNode::Semantics( bool UniqueOutputColumnNames, bool ComplexViewReplacement, bool& UnidentifiedColumns )
{
    if ( !mQueryGraph.IsOk() )
        return false;

    Query_Quantifier* Q1 = GetQuantifier()[0];
    if ( !Q1->NodeRef()->Semantics(UniqueOutputColumnNames, ComplexViewReplacement, UnidentifiedColumns) )
        return false;
	SAPDB_Int2 OutCols1 = Q1->OutputColumns();

    Query_Quantifier* Q2 = GetQuantifier()[1];
    if ( !Q2->NodeRef()->Semantics(false, ComplexViewReplacement, UnidentifiedColumns) )
        return false;
	SAPDB_Int2 OutCols2 = Q2->OutputColumns();

    Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();
    SyntaxTree[mAPNode].n_querycheck.addElement(qcn_Semantic);

	if ( OutCols1 != OutCols2 )     // DDT: different number of output columns within the two selects !!!
    {   
        mQueryGraph.SetError("SetSemantics");
        return false;
    }
    mSemanticsDone = true;
    return mSemanticsDone;
}

// ------------------------------------------------------------------------------------------------

bool Query_SetNode::IsColumnOfTypeLong( SAPDB_Int2 ColumnIndex )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

	Query_Quantifier* Q1 = GetQuantifier()[0];
	Query_Quantifier* Q2 = GetQuantifier()[1];
	if ( Q1 && Q2 )
	{
		return	Q1->NodeRef()->IsColumnOfTypeLong(ColumnIndex) ||
				Q2->NodeRef()->IsColumnOfTypeLong(ColumnIndex);
	}
	return true; 
}

// ------------------------------------------------------------------------------------------------

SAPDB_Int2 Query_SetNode::OutputColumns( void )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    Query_Quantifier* Q1 = GetQuantifier()[0];
    return Q1->OutputColumns();
}

bool Query_SetNode::GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& ColName )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    Query_Quantifier* Q1 = GetQuantifier()[0];
    return Q1->GetOutputColumn(i, ColName);
}

bool Query_SetNode::GetOutputColumnIndex( SAPDB_Int2 i, SAPDB_Int2& ColumnIndex )
{
    Query_Quantifier* Q1 = GetQuantifier()[0];
    ColumnIndex = i;
    return ( i>=0 && i<Q1->OutputColumns() );
}

bool Query_SetNode::CheckForColumn( SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    QuantifierSyntaxNode = mAPNode;
    Query_Quantifier* Q1 = GetQuantifier()[0];
    SAPDB_Int2 Node;
    return Q1->CheckForColumn(Column, Node, ColumnIndex);
}

bool Query_SetNode::CheckForColumn( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    QuantifierSyntaxNode = mAPNode;
    if ( CheckQuantifierName(Schema, Table) )
    {
        Query_Quantifier* Q1 = GetQuantifier()[0];
        SAPDB_Int2 Node;
        return Q1->CheckForColumn(Column, Node, ColumnIndex);
    }
    return false;
}

bool Query_SetNode::CheckQuantifierName( SQLMan_Identifier Schema, SQLMan_Identifier Table )
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    if ( Schema != SQLMan_Identifier() )
    {
		if ( Table != SQLMan_Identifier() )
		{
			Query_SyntaxTree& SyntaxTree = mQueryGraph.SyntaxTree();

			SyntaxTreeNodeNo Node = SyntaxTree[mAPNode].n_sa_level;
			if ( Node>0 )
			{
				if ( (SyntaxTree[Node].n_proc == 0) && 
					(SyntaxTree[Node].n_subproc() == s_identifier) )
				{
					SQLMan_Identifier IName = mQueryGraph.GetIdentifier(Node);
					if ( IName == Table )
						return true;
				}
			}
			return false;
		}
		else
		{
			mQueryGraph.SetError("Query_SetNode::CheckQuantifierName");
			// DDT: Schema given but no table - this should never happen...
			return false;
		}
    }
    return true;
}

void Query_SetNode::GetIdentificationNames(SQLMan_Identifier& Schema, SQLMan_Identifier& Table)
{
    SAPDBERR_ASSERT_STATE( mSemanticsDone );

    Schema = SQLMan_Identifier();
    Table  = mReferenceName;
}

bool Query_SetNode::HasReferenceName( void )
{
	return ( mReferenceName != SQLMan_Identifier() );
}

// ------------------------------------------------------------------------------------------------
