/*!***************************************************************************

  module      : Query_NodeTable.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef Query_NodeTable_H
#define Query_NodeTable_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SQLManager/Query/Query_Node.hpp"

// ------------------------------------------------------------------------------------------------

class Query_TableNode : public Query_Node
{
protected:    // private copyconstuctor used by CreateDuplicate
    Query_TableNode( const Query_TableNode& N, Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap );
public:
    virtual Query_Node* CreateDuplicate( Query_Node* Father, Query_Quantifier* Q, Query_NodeMapper* mpMap );

    Query_TableNode( Query_ManipulationContext& Context, Query_QueryGraph& QueryGraph, Query_Position Position, SyntaxTreeNodeNo APNode );

    virtual Query_NodeType GetNodeType( void ) { return NT_TableAccess; }
    virtual void PrintToTrace( Kernel_VTrace& VTrace );
    virtual bool Semantics( bool UniqueOutputColumnNames, bool ComplexViewReplacement, bool& UnidentifiedColumns );  

	virtual bool HasSysKey( void );

	virtual bool SelectPrivilegesForAllColumns( void );

	virtual bool ReplaceComplexViews( void );

	virtual bool IsColumnOfTypeLong( SAPDB_Int2 ColumnIndex );

    virtual SAPDB_Int2 OutputColumns( void );
    virtual bool GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& ColName );
    virtual bool GetOutputColumnIndex( SAPDB_Int2 i, SAPDB_Int2& ColumnIndex );
    virtual bool CheckForColumn( SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
    virtual bool CheckForColumn( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
    virtual bool CheckQuantifierName( SQLMan_Identifier Schema, SQLMan_Identifier Table );
    virtual void GetIdentificationNames(SQLMan_Identifier& Schema, SQLMan_Identifier& Table);
	virtual bool HasReferenceName( void );

    inline Query_TableType TableType( void )
    {
        return mTableType; 
    }

    inline bool BasicTableName( SQLMan_Identifier& SchemaName, SQLMan_Identifier& TableName )
    {
        SAPDBTRACE_METHOD_DEBUG ("Query_TableNode::TableName", QueryRewrite_Trace, 6);

        if ( mTableDescriptor )
        {
            mTableDescriptor.GetTableName(TableName);
            mTableDescriptor.GetSchemaName(SchemaName);
            return true;
        }
        return false;
    }

    inline bool TableName( SQLMan_Identifier& SchemaName, SQLMan_Identifier& TableName )
    {
        SAPDBTRACE_METHOD_DEBUG ("Query_TableNode::TableName", QueryRewrite_Trace, 6);

        SchemaName = mSchemaName;
        TableName  = mTableName;
        return true;
    }

    inline Catalog_ObjectId TabID( void )
    {
        return mTableDescriptor.GetTableId();
    }

private:

    Catalog_TableDescriptor mTableDescriptor;
    SAPDB_Int2              mOutputColumnCnt;
    Query_TableType         mTableType;
	SQLMan_Identifier       mSchemaName;
	SQLMan_Identifier       mTableName;
    SQLMan_Identifier       mReferenceName;
	bool					mSelectPrivForAllColumns;
    bool                    mExplicitSchema;
	bool					mReferencedViaSynonym;
};

// ------------------------------------------------------------------------------------------------

#endif