/*!***************************************************************************

  module      : Query_Quantifier.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef Query_Quantifier_H
#define Query_Quantifier_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SQLManager/SQLMan_Types.hpp"

#include "SAPDB/SQLManager/Query/Query_Types.hpp"
#include "SAPDB/SQLManager/Query/Query_SyntaxTree.hpp"
#include "SAPDB/SQLManager/Query/Query_Column.hpp"

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDB/Container/Container_Vector.hpp"

// ------------------------------------------------------------------------------------------------

class   Query_Node;

// ------------------------------------------------------------------------------------------------

class Query_Quantifier
{
private:    // private copyconstuctor used by CreateDuplicate
    Query_Quantifier( const Query_Quantifier& Q, Query_Node* Father, Query_NodeMapper* mpMap );
public:
    Query_Quantifier* CreateDuplicate( Query_Node* Father, Query_NodeMapper* mpMap );
    Query_Quantifier( void );
    Query_Quantifier( Query_Node* Ref, Query_DistinctType DistinctType, Query_QuantifierType QuantifierType );
    ~Query_Quantifier( void );

    bool ReplaceComplexViews( void );

    SAPDB_Int2 OutputColumns( void );
    SAPDB_Int2 FunctionOutputColumns( void );
    bool GetOutputColumn( SAPDB_Int2 i, SQLMan_Identifier& ColName );
    bool GetOutputColumnIndex( SAPDB_Int2 i, SAPDB_Int2& ColumnIndex );
    bool CheckForColumn( SQLMan_Identifier Column, SyntaxTreeNodeNo& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
    bool CheckForColumn( SQLMan_Identifier Schema, SQLMan_Identifier Table, SQLMan_Identifier Column, SAPDB_Int2& QuantifierSyntaxNode, SAPDB_Int2& ColumnIndex );
    bool CheckQuantifierName( SQLMan_Identifier Schema, SQLMan_Identifier Table );
    void GetIdentificationNames(SQLMan_Identifier& Schema, SQLMan_Identifier& Table);
	bool HasReferenceName( void );
    bool IqualIdentificationNames(Query_Quantifier*& Q);

    bool IsCorrelated( void );
	
	bool SelectPrivilegesForAllColumns( void );
	bool HasSysKey( void );

	void AdjustColumnIndexForPullDown( Container_Vector<Query_Column>*& Columns );
	void AdjustColumnIndexForPullUp( Container_Vector<Query_Column>*& Columns );
    bool PullDownSearchCond( Container_Vector<Query_Column>*& Columns, SyntaxTreeNodeNo SearchCond );

	bool OneTupleCondition( void );
    bool NoDuplicateCondition( void ); 

    Query_Node*&			NodeRef( void )						{ return mRef; }
	void                    SetNodeRef( Query_Node* Ref )		{ mRef = Ref; }
    Query_DistinctType&     DistinctType( void )				{ return mDistinctType; }
    Query_QuantifierType&   QuantifierType( void )				{ return mQuantifierType; }
    SAPDB_UInt2             NodeID( void );
    Query_NodeType			NodeType( void );
    bool                    NodeTypeTable( void );
    bool                    NodeTypeSelect( void );
    bool                    NodeTypeSelectOrSet( void );
    bool                    NodeTypeSet( void );
	bool                    FromTableSpec( void );
	Query_JoinType			JoinType( void );
    bool                    OuterJoin( void );
    bool                    Aggregates( void );

    bool                    Disabled( void )        { return mDisabled; }
    void                    SetDisabled( void )     { mDisabled = true; }

private:
    Query_Node*         mRef;
    Query_DistinctType      mDistinctType;
    Query_QuantifierType    mQuantifierType;
    bool                    mDisabled;
};

// ------------------------------------------------------------------------------------------------

#endif