/*!***************************************************************************

  module      : Query_SyntaxTree.cpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#include "SAPDB/SQLManager/Query/Query_SyntaxTree.hpp"

#include "SAPDB/SQLManager/SQLMan_Context.hpp"
#include "SAPDB/SQLManager/SQLMan_ContextSessionMembers.hpp"  // DDT 

#include "SAPDB/RunTime/RTE_CurrentStack.hpp"

#include "hak01.h"
#include "hak99.h"

// ------------------------------------------------------------------------------------------------

Query_SyntaxTree::Query_SyntaxTree( SQLMan_Context& Context )
    : mContext(Context) 
    , mFirstUnusedNode(cak_qrewrite_ref_init)
    , mFreeNodeCount(0)
    , mChanges(false)
    , mError(false)
{
    mAPTreePtr = (tak_ap_node*) mContext.a_ap_tree;
}

Query_SyntaxTree& Query_SyntaxTree::operator= ( const Query_SyntaxTree& T )
{
    SAPDB_MemCopyNoCheck(this, &T, sizeof(*this));
    return *this;    
}

void Query_SyntaxTree::PrintToTrace( void )
{
#if defined(SAPDB_SLOW)
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::PrintToTrace", QueryRewrite_Trace, 8);
    if ( QueryRewrite_Trace.TracesLevel(2) )
        a99print_nodes(*mContext.a_ap_tree, mContext.a_scv_index, mContext.a_first_hint_node);
#endif
}

void Query_SyntaxTree::SetRefBack( SyntaxTreeNodeNo StartNodeNo )
{	
    if ( mAPTreePtr[StartNodeNo].n_lo_level > 0 )
    {
        mAPTreePtr[mAPTreePtr[StartNodeNo].n_lo_level ].n_refback = StartNodeNo;
        SetRefBack(mAPTreePtr[StartNodeNo].n_lo_level );
    }
    if ( mAPTreePtr[StartNodeNo].n_sa_level > 0 )
    {
        mAPTreePtr[mAPTreePtr[StartNodeNo].n_sa_level].n_refback = StartNodeNo;
        SetRefBack(mAPTreePtr[StartNodeNo].n_sa_level );
    }
    return;


 //   // Don't do this with recursion but use n_refback we just set !
 //   SyntaxTreeNodeNo Act  = StartNodeNo;
 //   while ( Act>0 )
 //   {
 //       SyntaxTreeNodeNo Next;
 //       if ( (Next=mAPTreePtr[Act].n_sa_level) > 0 )
 //       {
 //           mAPTreePtr[Next].n_refback = Act;
 //           Act = Next;
 //       }
 //       else
 //       {
 //           while ( !(mAPTreePtr[Act].n_lo_level > 0) && (Act>0) );
 //           {
 //               Act = mAPTreePtr[Act].n_refback;
 //           }
 //           if ( Act > 0 )
 //           {
 //               Next = mAPTreePtr[Act].n_lo_level;
 //               mAPTreePtr[Next].n_refback = Act;
 //               Act = Next;
 //           }
 //       }
 //   }
	//// DDT : does this work if StartNodeNo is not the real root of the tree but some subnode ???

}

bool Query_SyntaxTree::GetFirstTreeNode( SyntaxTreeNodeNo& Node )
{
    SyntaxTreeNodeNo Act  = Node;
    if ( Act>=0 )
    {
        while ( mAPTreePtr[Act].n_lo_level > 0 )
        {
            Act = mAPTreePtr[Act].n_lo_level;
        }
        Node = Act;
        return true;
    }
	Node = cak_qrewrite_ref_init;
    return false;
}

bool Query_SyntaxTree::GetNextTreeNode( SyntaxTreeNodeNo& Node )
{
    if ( Node >= 0 )
    {
        SyntaxTreeNodeNo Next = mAPTreePtr[Node].n_sa_level;
        if ( Next > 0 )
        {
            Node = Next;
            return GetFirstTreeNode(Node);
        }
        else
        {
            do
            {
                Next = mAPTreePtr[Node].n_refback;
                if ( Next == Node ) //
				{
					Node = cak_qrewrite_ref_init;
                    return false;
				}
                if ( mAPTreePtr[Next].n_lo_level == Node )
                {
                    Node = Next;
                    return true;
                }
                Node = Next;
            } while ( Node>=0 );
        }
    }
	Node = cak_qrewrite_ref_init;
    return false;
}

bool Query_SyntaxTree::GetFirstArrayNode( SyntaxTreeNodeNo& Node )
{
    Node = 0;
    return true;
}

bool Query_SyntaxTree::GetNextArrayNode( SyntaxTreeNodeNo& Node )
{
    if ( Node >= 0 )
    {
        if ( ! mAPTreePtr[Node].n_special.includes(ni_skip_node) )
        {
            ++Node;
        }
        else
        {
            if ( !(Node = mAPTreePtr[Node].n_pos()) )
                return false;
        }
        if ( Node <= mContext.a_scv_index )
            return true;
    }
    return false;
}

SyntaxTreeNodeNo Query_SyntaxTree::GetFreeNode( bool ForcePutNode /*=false*/ )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::GetFreeNode", QueryRewrite_Trace, 6);

    SyntaxTreeNodeNo NodeNo = mFirstUnusedNode;
    if ( NodeNo > 0 && !ForcePutNode )
    {   // there are unused nodes that can be used
        mFirstUnusedNode = mAPTreePtr[NodeNo].n_sa_level;
        mAPTreePtr[NodeNo].n_sa_level = 0;
        --mFreeNodeCount;
        // DDT : initialize node !!!
    }
    else
    {   
        a01_call_put_queryrewrite(mContext, NodeNo);
        if ( mContext.ReturnCode() )
        {
            NodeNo = cak_qrewrite_ref_init;
            mContext.ResetReturnCode();
        }
        if ( (tak_ap_node*)mContext.a_ap_tree != mAPTreePtr )
            mAPTreePtr = (tak_ap_node*)mContext.a_ap_tree;
    }
    return NodeNo;
}

void Query_SyntaxTree::SetUnusedNode( SyntaxTreeNodeNo NodeNo, bool Recursive /*=false*/ )
{   
    if ( NodeNo > 0 )
    {
        if ( Recursive )
        {	// DDT : RECURSIVE
            SetUnusedNode(mAPTreePtr[NodeNo].n_lo_level, true);
            SetUnusedNode(mAPTreePtr[NodeNo].n_sa_level, true);
        }
        mAPTreePtr[NodeNo].n_lo_level   = 0;
        mAPTreePtr[NodeNo].n_sa_level   = mFirstUnusedNode;
        mAPTreePtr[NodeNo].n_refback    = cak_qrewrite_ref_init;
        mAPTreePtr[NodeNo].n_querycheck.clear();
        mFirstUnusedNode = NodeNo;
        ++mFreeNodeCount;
    }
}

// -------------------

void Query_SyntaxTree::UnlinkNode( SyntaxTreeNodeNo NodeNo, bool SetUnused )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::UnlinkNode", QueryRewrite_Trace, 6);

    if ( NodeNo > 0 )
    {
        SyntaxTreeNodeNo predecessor    = mAPTreePtr[NodeNo].n_refback;
        SyntaxTreeNodeNo successor      = mAPTreePtr[NodeNo].n_sa_level;

        if ( predecessor >= 0 )
        {
            if ( mAPTreePtr[predecessor].n_sa_level == NodeNo )
                mAPTreePtr[predecessor].n_sa_level = successor;
            else
                mAPTreePtr[predecessor].n_lo_level = successor;
        }
        if ( successor > 0 )
            mAPTreePtr[successor].n_refback = predecessor;

        mAPTreePtr[NodeNo].n_refback  = cak_qrewrite_ref_init;
        mAPTreePtr[NodeNo].n_sa_level = cak_qrewrite_ref_init;

        if ( SetUnused )
            SetUnusedNode(NodeNo, true);
    }
}

void Query_SyntaxTree::SubstituteNodeBySubtree( SyntaxTreeNodeNo OldNodeNo, SyntaxTreeNodeNo StartSubtreeNodeNo )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::SubstituteNodeBySubtree", QueryRewrite_Trace, 6);

    if ( OldNodeNo > 0 && StartSubtreeNodeNo > 0)
    {
        // DDT: was, wenn StarSubTreeNodeNO.n_refback =OldNodeNo ???????????????

        SyntaxTreeNodeNo OldSuccessor   = mAPTreePtr[OldNodeNo].n_sa_level;
        SyntaxTreeNodeNo predecessor    = mAPTreePtr[OldNodeNo].n_refback;

        if ( predecessor >= 0 )
            if ( mAPTreePtr[predecessor].n_sa_level == OldNodeNo )
            {
                mAPTreePtr[predecessor].n_sa_level = StartSubtreeNodeNo;
            }
            else
            {
                mAPTreePtr[predecessor].n_lo_level = StartSubtreeNodeNo;
            }

        mAPTreePtr[StartSubtreeNodeNo].n_refback = predecessor;

        if ( OldSuccessor >= 0 )
        {
            SyntaxTreeNodeNo i = StartSubtreeNodeNo;
            while ( mAPTreePtr[i].n_sa_level > 0 ) 
            {
                i = mAPTreePtr[i].n_sa_level;
            }
            mAPTreePtr[i].n_sa_level = OldSuccessor;
            mAPTreePtr[OldSuccessor].n_refback = i;
        }
        mAPTreePtr[OldNodeNo].n_refback     = cak_qrewrite_ref_init;
        mAPTreePtr[OldNodeNo].n_sa_level    = cak_qrewrite_ref_init;
        mAPTreePtr[OldNodeNo].n_lo_level    = cak_qrewrite_ref_init;
        SetUnusedNode(OldNodeNo);
    }
}

void Query_SyntaxTree::SetSucessorOnSameLevel( SyntaxTreeNodeNo NodeNo, SyntaxTreeNodeNo NewSuccessor )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::SetSucessorOnSameLevel", QueryRewrite_Trace, 6);

    if ( NodeNo >= 0 )
    {
        SyntaxTreeNodeNo OldSuccessor = mAPTreePtr[NodeNo].n_sa_level;

        mAPTreePtr[NewSuccessor].n_sa_level = OldSuccessor;
        mAPTreePtr[NewSuccessor].n_refback  = NodeNo;
        mAPTreePtr[NodeNo      ].n_sa_level = NewSuccessor;

        if ( OldSuccessor >= 0 )
            mAPTreePtr[OldSuccessor].n_refback = NewSuccessor;
    }
}

void Query_SyntaxTree::SetSucessorOnLowLevel( SyntaxTreeNodeNo NodeNo, SyntaxTreeNodeNo NewSuccessor )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::SetSucessorOnLowLevel", QueryRewrite_Trace, 6);

    if ( NodeNo >= 0 )
    {
        mAPTreePtr[NewSuccessor].n_sa_level = mAPTreePtr[NodeNo].n_lo_level;
        mAPTreePtr[NewSuccessor].n_refback  = NodeNo;
        mAPTreePtr[NodeNo      ].n_lo_level = NewSuccessor;
    }
}

// -------------------

bool Query_SyntaxTree::SubtreeContainsNode( SyntaxTreeNodeNo SubtreeStart, SyntaxTreeNodeNo Node )
{	// DDT : RECURSIVE
    if ( SubtreeStart > 0 && Node > 0 )
    {
        if ( SubtreeStart == Node )
            return true;
        if ( SubtreeContainsNode(mAPTreePtr[SubtreeStart].n_lo_level, Node) )
            return true;
        return SubtreeContainsNode(mAPTreePtr[SubtreeStart].n_sa_level, Node);
    }
    return false;
}

// -------------------

SyntaxTreeNodeNo Query_SyntaxTree::CheckNodes( void  )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::CheckNodes", QueryRewrite_Trace, 6);

    SyntaxTreeNodeNo Node=0;
    GetFirstTreeNode(Node);
    do
    {
        if ( !CheckSingleNode(Node) )
            return Node;

        // DDT: don't check infonodes for the moment ! should be changed in future !
//        if ( mAPTreePtr[Node].n_special.includes(ni_info_node) )
//            GetNextTreeNode(Node); // DDT : This is ok ! We really need the next node within the array!

    }while(GetNextTreeNode(Node)); // DDT : GetNextTreeNode ? The info_node is not in the tree

    return cak_qrewrite_ref_init;
}

bool Query_SyntaxTree::CheckSingleNode( SyntaxTreeNodeNo NodeNo )
{
    if ( !(mAPTreePtr[NodeNo].n_querycheck.includes(qcn_Syntax)) && 
          (mAPTreePtr[NodeNo].n_proc != a80) &&
          (NodeNo != 0) )
         return false;

    return true;
}

SyntaxTreeNodeNo Query_SyntaxTree::DoubleTreePart( tak_ap_node* SourceSyntaxTree, SyntaxTreeNodeNo StartNode, SAPDB_Int2 PosOffset, Query_NodeMapper* mpMap )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::DoubleTreePart", QueryRewrite_Trace, 7);

    if ( !mpMap )
        mpMapper = new(mContext.GetAllocator()) Query_NodeMapper(mContext.GetAllocator());
    else
        mpMapper = mpMap;

    SyntaxTreeNodeNo DestRoot = DoubleTreePartInternal(SourceSyntaxTree, StartNode, PosOffset );
    MapSubSelectNodes();

    if ( !mpMap && mpMapper )
        destroy(mpMapper, mContext.GetAllocator());
	else
		mpMapper = 0;
    
    return DestRoot;
}

void Query_SyntaxTree::MapSubSelectNodes( void )
{
    if ( mpMapper )
    {
        SAPDB_Int2 n = mpMapper->GetSize();
        for (SAPDB_Int2 i = 0; i<n; i++)
        {
            SyntaxTreeNodeNo node = mpMapper->GetDestNodeFromPos(i);
            if ( ( mAPTreePtr[node].n_proc == a92fromsel ) ||
                 ( mAPTreePtr[node].n_proc == a63 && mAPTreePtr[node].n_subproc() == cak_x_subquery )            ||
                 ( mAPTreePtr[node].n_proc == a63 && mAPTreePtr[node].n_subproc() == cak_x_one_val_subquery )    ||
                 ( mAPTreePtr[node].n_proc == a63 && mAPTreePtr[node].n_subproc() == cak_x_arith_with_subquery ) ||
                 ( mAPTreePtr[node].n_proc == a80 && mAPTreePtr[node].n_subproc() == cak_x_select_hint ))
				 // this if-statement must match with Query_SyntaxTree::NeedsMapping
            {
                SyntaxTreeNodeNo Target   = mpMapper->GetDestNodeFromSrcNode(mAPTreePtr[node].n_pos());
                mAPTreePtr[node].n_pos()  = Target;
                if ( mAPTreePtr[node].n_proc == a92fromsel )
                    mAPTreePtr[node].n_subproc() = Target;
            }
        }
    }
}

SyntaxTreeNodeNo Query_SyntaxTree::DoubleTreePartInternal( tak_ap_node* SourceSyntaxTree, SyntaxTreeNodeNo StartNode, SAPDB_Int2 PosOffset /*=0*/ )
{	// DDT : RECURSIVE
    if ( mError )
       return cak_qrewrite_ref_init;

    SyntaxTreeNodeNo NewRootNode = GetFreeNode(true);

    tak_ap_node* SourceAPTreePtr = SourceSyntaxTree;

    if ( NewRootNode > 0 )
    {
        mAPTreePtr[NewRootNode] = SourceAPTreePtr[StartNode];
		if ( PosOffset != 0 && !WithOutPositionOffset(NewRootNode) )
	        mAPTreePtr[NewRootNode].n_pos() += PosOffset;

		if ( mpMapper )
        {
            if ( NeedsMapping(NewRootNode) )
                mpMapper->AddMappingNode(StartNode, NewRootNode);
        }

        if ( SourceAPTreePtr[StartNode].n_special.includes(ni_info_node) )
        {
            SyntaxTreeNodeNo NewInfoNode = GetFreeNode(true);
            if ( NewInfoNode > 0 )
            {
                SyntaxTreeNodeNo OldInfoNode=StartNode+1;
                if ( SourceAPTreePtr[OldInfoNode].n_special.includes(ni_skip_node) )
                    OldInfoNode = SourceAPTreePtr[OldInfoNode].n_pos();
                mAPTreePtr[NewInfoNode] = SourceAPTreePtr[OldInfoNode];
            }
        }

        if ( SourceAPTreePtr[StartNode].n_lo_level > 0 )
        {
            SyntaxTreeNodeNo Lo = DoubleTreePartInternal(SourceSyntaxTree, SourceAPTreePtr[StartNode].n_lo_level, PosOffset);
            if ( Lo > 0 )
            {
                mAPTreePtr[NewRootNode].n_lo_level = Lo;
                mAPTreePtr[Lo         ].n_refback  = NewRootNode;
            }
            else
            {
                mAPTreePtr[NewRootNode].n_lo_level = 0;
                mError = true;
                return cak_qrewrite_ref_init;
            }
        }

        if ( SourceAPTreePtr[StartNode].n_sa_level > 0 )
        {
            SyntaxTreeNodeNo Sa = DoubleTreePartInternal(SourceSyntaxTree, SourceAPTreePtr[StartNode].n_sa_level, PosOffset);
            if ( Sa > 0 )
            {
                mAPTreePtr[NewRootNode].n_sa_level = Sa;
                mAPTreePtr[Sa         ].n_refback  = NewRootNode;
            }
            else
            {
                mAPTreePtr[NewRootNode].n_sa_level = 0;
                mError = true;
                return cak_qrewrite_ref_init;
            }
        }

    }
    return NewRootNode;
}

// ------------------------------------------------------------------------------------------------

SyntaxTreeNodeNo Query_SyntaxTree::FindSubSelectNode( SyntaxTreeNodeNo Node )
{
    SyntaxTreeNodeNo i=0;
    GetFirstArrayNode(i);
    while ( GetNextArrayNode(i) ) // DDT : GetNextTreeNode ?
    {
        if ( mAPTreePtr[i].n_pos()  == Node )
        {
            if ( ((mAPTreePtr[i].n_proc == a63) &&
                  (mAPTreePtr[i].n_subproc() == 27) &&
                  (mAPTreePtr[i].n_symb == s_select)) ||

                 ((mAPTreePtr[i].n_proc == a92fromsel)) ||

                 ((mAPTreePtr[i].n_proc == a63) &&
                  (mAPTreePtr[i].n_subproc() == 27) &&
                  (mAPTreePtr[i].n_symb == s_where)) ||

				  ((mAPTreePtr[i].n_proc == a63) &&
                  (mAPTreePtr[i].n_subproc() == 27) &&
                  (mAPTreePtr[i].n_symb == s_having)) )
            {
                return i;
            }
        }
    }
    return cak_qrewrite_ref_init;
}

bool Query_SyntaxTree::AddSubSelectNode( SyntaxTreeNodeNo SelNode, SyntaxTreeNodeNo SubSelNode, SyntaxTreeNodeNo FatherOfSubSelNode, SAPDB_Int2 Pos )
{
    // Adds an already existing SubSelect node identified by SubSelNode as a subselect to FatherNode as children number Pos !

    SAPDBTRACE_METHOD_DEBUG ("Query_SelectNode::AddSubSelectNode", QueryRewrite_Trace, 6);

	SyntaxTreeNodeNo TopNode = SelNode;
    if ( SelNode == FatherOfSubSelNode )
	{
        mAPTreePtr[SubSelNode].n_sa_level   = mAPTreePtr[TopNode].n_sa_level;
        if ( mAPTreePtr[SubSelNode].n_sa_level > 0 )
            mAPTreePtr[mAPTreePtr[SubSelNode].n_sa_level].n_refback = SubSelNode;
        mAPTreePtr[TopNode].n_sa_level      = SubSelNode;
        mAPTreePtr[SubSelNode].n_refback    = TopNode;	
		return true;
	}
	else
	{
        SyntaxTreeNodeNo TopNode = FindSubSelectNode(FatherOfSubSelNode);
		if ( TopNode > 0 )
		{
			mAPTreePtr[SubSelNode].n_sa_level   = mAPTreePtr[TopNode].n_lo_level;
			if ( mAPTreePtr[SubSelNode].n_sa_level > 0 )
				mAPTreePtr[mAPTreePtr[SubSelNode].n_sa_level].n_refback = SubSelNode;
			mAPTreePtr[TopNode].n_lo_level      = SubSelNode;
			mAPTreePtr[SubSelNode].n_refback    = TopNode;
			return true;
		}
	}
    return false;
}

bool Query_SyntaxTree::RemoveSubSelectNode( SyntaxTreeNodeNo NodeNo )
{
    SAPDBTRACE_METHOD_DEBUG ("Query_SyntaxTree::RemoveFromSelNode", QueryRewrite_Trace, 6);
    SAPDBERR_ASSERT_ARGUMENT( NodeNo > 0 );

    SyntaxTreeNodeNo ToBeRemoved = FindSubSelectNode(NodeNo);
    if ( ToBeRemoved > 0 )
    {
        SyntaxTreeNodeNo successor      = mAPTreePtr[ToBeRemoved].n_sa_level;
        SyntaxTreeNodeNo predecessor    = mAPTreePtr[ToBeRemoved].n_refback;

        if ( mAPTreePtr[predecessor].n_lo_level == ToBeRemoved )
        {
            mAPTreePtr[predecessor].n_lo_level = successor;
            mAPTreePtr[successor].n_lo_level = predecessor;
        }
        else
        {
            mAPTreePtr[predecessor].n_sa_level = successor;
            mAPTreePtr[successor].n_sa_level = predecessor;
        }
        SetUnusedNode(ToBeRemoved);
        return true;
    }
    return false;
}

// ------------------------------------------------------------------------------------------------

bool Query_SyntaxTree::ChangeColumnNameToColumnId( SyntaxTreeNodeNo Node, SyntaxTreeNodeNo FromTabNode, SAPDB_Int2 ColumnIndex )
{
	SAPDBERR_ASSERT_ARGUMENT( (FromTabNode>0) && ((ColumnIndex>0) || ColumnIndex==cak_qrewrite_ref_star) );

	if ( (FromTabNode>0) && ((ColumnIndex>0) || ColumnIndex==cak_qrewrite_ref_star))
	{
		if ( (mAPTreePtr[Node].n_proc == 0) &&  
				(mAPTreePtr[Node].n_symb == s_columnname) &&
				(mAPTreePtr[Node].n_querycheck.includes(qcn_Semantic)) )
		{
			mAPTreePtr[Node].n_fromtabnode() = FromTabNode;
			mAPTreePtr[Node].n_columnindex() = ColumnIndex;
//			mAPTreePtr[Node].n_filler() = 0;	// do not clear as this is n_pos !
			mAPTreePtr[Node].n_symb.becomes(s_columnid);

			Node = mAPTreePtr[Node].n_refback;
			if ( (mAPTreePtr[Node].n_proc == 0) &&  
				(mAPTreePtr[Node].n_symb == s_tablename) )
			{
				SyntaxTreeNodeNo Help = Node;
				Node = mAPTreePtr[Node].n_refback;
				UnlinkNode(Help, /*SetUnused*/ true);
			}

			if ( (mAPTreePtr[Node].n_proc == 0) && 
				(mAPTreePtr[Node].n_symb == s_authid) )
			{
				UnlinkNode(Node, /*SetUnused*/ true);
			}
			return true;
		}
	}
    return false;
}

//bool Query_SyntaxTree::ChangeOrderColumnNameToRefName( SyntaxTreeNodeNo Node, SAPDB_Int4& Pos, SAPDB_Int2& Len  )
//{
//	//if ( (mAPTreePtr[Node].n_proc == 0) &&  
//	//		(mAPTreePtr[Node].n_symb == s_columnname) &&
//	//		(mAPTreePtr[Node].n_querycheck.includes(qcn_Semantic)) )
//	//{
//	//	mAPTreePtr[Node].n_pos() = Len;
//	//	mAPTreePtr[Node].n_length() = Len;
//	//	mAPTreePtr[Node].n_symb.becomes(s_columnname);
//
//	//	Node = mAPTreePtr[Node].n_refback;
//	//	if ( (mAPTreePtr[Node].n_proc == 0) &&  
//	//		(mAPTreePtr[Node].n_symb == s_tablename) )
//	//	{
//	//		SyntaxTreeNodeNo Help = Node;
//	//		Node = mAPTreePtr[Node].n_refback;
//	//		UnlinkNode(Help, /*SetUnused*/ true);
//	//	}
//
//	//	if ( (mAPTreePtr[Node].n_proc == 0) && 
//	//		(mAPTreePtr[Node].n_symb == s_authid) )
//	//	{
//	//		UnlinkNode(Node, /*SetUnused*/ true);
//	//	}
//	//	return true;
//	//}
//	return false;
//}

// ------------------------------------------------------------------------------------------------