/*!***************************************************************************

  module      : Query_Types.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef QUERY_TYPES_H
#define QUERY_TYPES_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SQLManager/SQLMan_Context.hpp"

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDB/SAPDBCommon/SAPDB_MemCopyMove.hpp"
#include "SAPDB/SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
#include "SAPDB/SAPDBCommon/Tracing/SAPDBTrace_Topic.hpp"

#include "SAPDB/KernelCommon/Kernel_VTrace.hpp"
#include "SAPDB/SAPDBCommon/SAPDB_OStream.hpp"

#include "vak001.h"

// ------------------------------------------------------------------------------------------------

extern SAPDBTrace_Topic QueryRewrite_Trace;

// ------------------------------------------------------------------------------------------------

#define QRW_PRINT2TRACE(Level, X) \
    if (QueryRewrite_Trace.TracesLevel(Level)) \
        mVTrace << X << FlushLine;

typedef tak_querycheck_node Query_NodeState;

typedef SAPDB_Int2 Query_JoinType;
#define JT_Unknown           0
#define JT_NoJoin            1
#define JT_Inner             2
#define JT_Cross             4
#define JT_FullOuter         8
#define JT_RightOuter       16 
#define JT_LeftOuter        32
#define JT_Outer            (JT_FullOuter|JT_RightOuter|JT_LeftOuter)

// ------------------------------------------------------------------------------------------------

enum Query_RC           {   Q_Ok, Q_Unknown, Q_Error };

enum Query_Progress     { 
                            PG_Error, 
                            PG_Init, 
                            PG_Nodes, 
                            PG_SubstituteViews, 
                            PG_Semantics, 
                            PG_Normalized, 
                            PG_Optimized 
                        };

enum Query_SelColType   {
                            CT_Column,
                            CT_Asterix,
                            CT_Function,
							CT_Aggregate,
                            CT_Select
                        };

enum Query_NodeType {   
							NT_Unknown,
                            NT_Top, 
                            NT_TableAccess, 
                            NT_Select,
                            NT_Union, NT_Difference, NT_Intersect,
                            NT_Insert, NT_Update, NT_Delete 
                        };

enum Query_QuantifierType {
                            QT_F, // From Part
                            QT_A, // Where Part ALL quantifier
                            QT_E, // Where Part EXIST quantifier
                            QT_L, // Select list
                            QT_Unknown
                          };

enum Query_TableType      { 
                            TT_BaseTable,
                            TT_Synonym,
                            TT_Cursor,
                            TT_View, 
                            TT_JoinView, 
                            TT_ComplexView,
                            TT_Unknown
                        };

enum Query_DistinctType   { 
                            DT_Enforce, 
                            DT_Preserve, 
                            DT_Permit,
                            DT_Unknown
                        };

/// Beschreibt, wo in dem darberliegenden Node ein Node einzuordnen ist bzw. wo bei der Abarbeitung man sich gerade befindet.
enum Query_Position       { 
                            P_Start, 
                            P_Union, P_Difference, P_Intersect, 
                            P_Select, P_SelectList, P_FromPart, 
                            P_Where, P_WhereAll, P_Having, P_HavingAll, P_GroupBy, P_OrderBy, P_UpdateColumns,
                            P_Unknown 
                        };

#define Query_MaxSubqueries   cak00_maxsubqueries
#define Query_MaxFQuantifier  cak00_maxsources

// ------------------------------------------------------------------------------------------------

typedef SAPDB_Int2              SyntaxTreeNodeNo;
typedef SQLMan_Identifier /*tsp00_KnlIdentifier*/     Query_Identifier;

//inline SAPDB_OStream & operator<< (SAPDB_OStream & ostream, const Query_Identifier & Ident)
//{
//    char s[2+sizeof(tsp00_KnlIdentifier)];
//    *((Query_Identifier*)s) = Ident;
//    s[sizeof(tsp00_KnlIdentifier)+1] = s[sizeof(tsp00_KnlIdentifier)] = '\0';
//    return (ostream << s);
//}

//inline SAPDB_OStream & operator<< (SAPDB_OStream & ostream, const SharedSQL_ParseID & parseId)
//{
//    SAPDBTrace_Hex hex(&parseId, sizeof(parseId));
//    return hex.Print(ostream);
//}

// ------------------------------------------------------------------------------------------------

#endif