/****************************************************************************/
/*!

  @file        SQLMan_SessionIterator.cpp
  @author      MartinKi

  @ingroup
  @brief       Implementation of class SQLMan_SessionIterator.

  @see

*/
/*-------------------------------------------------------------------------

  copyright:    Copyright (c) 2005 SAP AG


    ========== licence begin  GPL
    Copyright (c) 2002-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SQLManager/SQLMan_SessionIterator.hpp"

#include "SQLManager/SQLMan_Context.hpp"
#include "SQLManager/SQLMan_Types.hpp"

#include "hak51.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/

/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/

/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

SQLMan_SessionIterator::~SQLMan_SessionIterator()
{
    // make sure lock is released
    this->Reset();
}

/**************************************************************************/

SAPDB_Bool SQLMan_SessionIterator::Next(SQLMan_Context*& context)
{
    pasbool isFirst = m_isFirst;
    tak_all_command_glob* sessionAcv = a51NextSession( isFirst );
    if ( sessionAcv == 0 ) {
        m_isFirst = true;
        return false;
    }

    context   = &SQLMan_Context::AcvToContext( *sessionAcv );
    m_isFirst = false;
    return true;
}

/**************************************************************************/

void SQLMan_SessionIterator::Reset()
{
    if ( m_isFirst ) {
        return;
    }

    pasbool isFirst = m_isFirst;
    while ( a51NextSession( isFirst ) != 0 ) {
        ;
    }

    m_isFirst = true;
}

/**************************************************************************/
