/****************************************************************************

  module      : vbd490.cpp

  -------------------------------------------------------------------------

  author      : TorstenS
  responsible : TorstenS

  special area: CreateIndexCoordinator
  description : 


  last changed: 1999-07-21  14:00
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 1999-2004 SAP AG



    ========== licence begin  GPL
    Copyright (c) 1999-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

// Content of include files
#include "gbd490.h"
/* */
#include "gbd491.h"   // CPP   : CreateIndexParticipant
#include "ggg11.h"    // PASCAL: GG_vtrace_types
#include "hbd01.h"    // PASCAL: filesysteminterface_1
#include "hbd03.h"    // CPP   : filesysteminterface_3
#include "hbd50.h"    // PASCAL: Indexhandling
#include "hkb53.h"    // PASCAL: KB_transaction
#include "hgg01.h"    // PASCAL: Configuration_Parameter
#include "hgg01_1.h"  // PASCAL: Configuration_Parameter
#include "hgg01_3.h"  // PASCAL: Configuration_Parameter
#include "heo56.h"    // RTE   : Vsleep

#include "Trace/Trace_Entry.hpp"
#include "SAPDB/SAPDBCommon/SAPDB_RangeCode.hpp" // Kernel_move_and_fill

#include "ServerTasks/SrvTasks_JobCreateIndex.hpp"
#include "ServerTasks/SrvTasks_WaitingTask.hpp"
#include "ServerTasks/SrvTasks_JobList.hpp"

#if COMPILEMODE_MEO00 >= SLOW_MEO00 
#include "hta99.h"
#endif

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define IN_REGION_BD490         true

#define MAX_RETRIES_BD490       10

#define CANCEL_MSG_1_BD490      "Index Coordinator Canceled              "
#define SEND_MSG_1_BD490        "All ServerTasks Are Busy                "
#define PROCESS_MSG_1_BD490     "Start Process Keyrange Without Server   "
#define PROCESS_MSG_2_BD490     "Stop Process Keyrange Without Server    "


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/

/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/



/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL VARIABLES                                                          *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL FUNCTIONS (PROTOTYPES)                                             *
 *===========================================================================*/

inline void
bd490_AddIndexCounter (tgg00_IndexCounter       &SourceCounter,
                       const tgg00_IndexCounter &AddCounter);

/*---------------------------------------------------------------------------*/

inline tsp00_Int4
bd490_GetEstimatedRecordsPerLeaf (
    const tsp00_Int4    NumPrimLeafs,
    const tsp00_Int4    NumPrimRecords,
    tsp00_Int4          AvgRecordsPerLeaf);

/*===========================================================================*
 *  GLOBAL FUNCTIONS (CODE)                                                  *
 *===========================================================================*/

void
bd490InitIndexCounter (tgg00_IndexCounter  &IndexCounter)
{
    IndexCounter.idc_prim_leafnodes  = 0;
    IndexCounter.idc_prim_indexnodes = 0;
    IndexCounter.idc_sec_leafnodes   = 0;
    IndexCounter.idc_prim_keycount   = 0;
    IndexCounter.idc_sec_keycount    = 0;
    IndexCounter.idc_nullcount       = 0;
    IndexCounter.idc_prim_mismatch   = 0;
}

/*===========================================================================*
 *  LOCAL FUNCTIONS (CODE)                                                   *
 *===========================================================================*/

inline void
bd490_AddIndexCounter (
    tgg00_IndexCounter       &SourceCounter,
    const tgg00_IndexCounter &AddCounter)
{
    bd490Inc (SourceCounter.idc_prim_leafnodes,  AddCounter.idc_prim_leafnodes);
    bd490Inc (SourceCounter.idc_prim_indexnodes, AddCounter.idc_prim_indexnodes);
    bd490Inc (SourceCounter.idc_sec_leafnodes,   AddCounter.idc_sec_leafnodes);
    bd490Inc (SourceCounter.idc_prim_keycount,   AddCounter.idc_prim_keycount);
    bd490Inc (SourceCounter.idc_sec_keycount,    AddCounter.idc_sec_keycount);
    bd490Inc (SourceCounter.idc_nullcount,       AddCounter.idc_nullcount);
    bd490Inc (SourceCounter.idc_prim_mismatch,   AddCounter.idc_prim_mismatch);
}

/*---------------------------------------------------------------------------*/

inline tsp00_Int4
bd490_GetEstimatedRecordsPerLeaf (
    const tsp00_Int4    NumPrimLeafs,
    const tsp00_Int4    NumPrimRecords,
    tsp00_Int4          AvgRecordsPerLeaf)
{
    if ((0 != NumPrimLeafs) && (0 != NumPrimRecords))
        AvgRecordsPerLeaf = (NumPrimRecords / NumPrimLeafs);

    return (AvgRecordsPerLeaf + (AvgRecordsPerLeaf * 3 / 10) + 5); // +30 % security
}

/*===========================================================================*
 *  DEFINITION OF METHODS DECLARED IN gbd490.h (CODE)                        * 
 *===========================================================================*/


cbd490_CreateIndexParallel::cbd490_CreateIndexParallel(
    cbd494_SortableInvTree  &InvTree,
    tbd_current_tree        &PrimCurrent,
    tgg00_StackDesc         &StackDesc,
    tgg00_IndexCounter      &IndexCounter,
    tgg00_IndexMode         IndexMode
)
        :
        m_TrError               (PrimCurrent.curr_trans->trError_gg00),
        m_Trans                 (*PrimCurrent.curr_trans),
        m_PrimCurrent           (PrimCurrent),
        m_IndexCounter          (IndexCounter),
        m_IndexMode             (IndexMode),
        m_NumUsableServerTasks  (0), // PTS 1104880 TS 1999-12-06
        m_Stack                 (PrimCurrent.curr_trans->trError_gg00, StackDesc),
        m_FileList              (*PrimCurrent.curr_trans, InvTree, m_Stack.bd497IsInvUnique())
{
    ROUTINE_DBG_MEO00 ("cbd490_CreateIndexParallel");


    if (e_ok != m_TrError) return;

    bd490InitIndexCounter (m_IndexCounter);

    m_NumUsableServerTasks = bd01UsableServerForParallelExecution();
}

/*---------------------------------------------------------------------------*/

cbd490_CreateIndexParallel::~cbd490_CreateIndexParallel ()
{
    ROUTINE_DBG_MEO00 ("~cbd490_CreateIndexParallel");

    // Call Destructor for FileList
}

/*---------------------------------------------------------------------------*/

void
cbd490_CreateIndexParallel::bd490BuildIndex()
{
    ROUTINE_DBG_MEO00 ("bd490BuildIndex");


    tsp00_Bool    bNoMorePrimData   = false;
    /* */
    tsp00_Int4    CurrFileIndex     = 0;
    tsp00_Int4    AvgRecordsPerLeaf = 0;
    tsp00_Int4    NumLeavesToScan   = 0;
    tsp00_Int4    NumSendOrders     = 0;
    tsp00_Int4    RetryCount        = 0;
    tsp00_Int4    StartPrimKeyLen   = 0;
    tsp00_Int4    StopPrimKeyLen    = 0;
    /* */
    tsp00_PageNo  PrimQueuePno;
    tsp00_PageNo  SecQueuePno;
    /* */
    tgg00_FileId  QueueId;
    /* */
    tsp00_Key     StartPrimKey;
    tsp00_Key	  StopPrimKey;
    /* */
    tsp00_BytePtr pStartPrimKey = REINTERPRET_CAST (tsp00_BytePtr, &StartPrimKey);
    tsp00_BytePtr pStopPrimKey  = REINTERPRET_CAST (tsp00_BytePtr, &StopPrimKey);

    if (0>= m_NumUsableServerTasks)
    {
        m_TrError = e_no_more_memory;
        return;
    }

	SrvTasks_WaitingTask                      waitingTask (*RTETask_ITask::Self());
	SrvTasks_JobList<SrvTasks_JobCreateIndex> jobs        (*(reinterpret_cast<SAPDBMem_IRawAllocator*>(m_Trans.trAllocator_gg00)));

    while
    (
        ((e_ok == m_TrError) && (! bNoMorePrimData)) || ((e_no_server_task_available == m_TrError))
    )
    {
        if ((e_ok == m_TrError) && (create_index == m_IndexMode))
        {
            m_FileList.bd496CreateFile (CurrFileIndex, QueueId,	PrimQueuePno, SecQueuePno);
        }

        if (e_ok == m_TrError)
        {
            bd50DetermineKeyRangeForServer (m_PrimCurrent,
                                            pStartPrimKey, StartPrimKeyLen, pStopPrimKey, StopPrimKeyLen,
                                            NumLeavesToScan, AvgRecordsPerLeaf, bNoMorePrimData);
            if( e_ok == m_TrError ){ // file directory statistic count migration
                bd490Inc( m_IndexCounter.idc_prim_indexnodes, 1);
            }

            AvgRecordsPerLeaf = bd490_GetEstimatedRecordsPerLeaf( m_IndexCounter.idc_prim_leafnodes, 
                                    m_IndexCounter.idc_prim_keycount, AvgRecordsPerLeaf);
        }

        if ((e_ok == m_TrError) || (e_no_server_task_available == m_TrError))
        {
            bd490_SendKeyRangeToServer (CurrFileIndex, NumLeavesToScan, AvgRecordsPerLeaf,
                                        pStartPrimKey, StartPrimKeyLen, pStopPrimKey, StopPrimKeyLen,
                                        NumSendOrders, RetryCount, QueueId, PrimQueuePno, SecQueuePno,
                                        waitingTask, jobs);
        }

        if  (
            (
                (e_ok == m_TrError) &&
                ((NumSendOrders == m_NumUsableServerTasks) || (bNoMorePrimData))
            )
            ||
            ((e_file_limit == m_TrError) && (NumSendOrders > 0))
            ||
            ((e_no_server_task_available == m_TrError) && (NumSendOrders > 0))
        )
        {
            bd490_WaitForReply (bNoMorePrimData, NumSendOrders, waitingTask, jobs);
        }

        if ((e_no_server_task_available == m_TrError ) &&
                (0 == NumSendOrders                   ) &&
                (MAX_RETRIES_BD490 <= RetryCount      ))
        {
            bd490_ProcessKeyRangeByCoordinator (CurrFileIndex, NumLeavesToScan, AvgRecordsPerLeaf,
                                                pStartPrimKey, StartPrimKeyLen, pStopPrimKey, StopPrimKeyLen, RetryCount, QueueId,
                                                PrimQueuePno, SecQueuePno);
        }

        if (
            (create_index == m_IndexMode)              &&
            (
                ((e_ok == m_TrError) && (bNoMorePrimData)) ||
                (e_file_limit == m_TrError               )
            )
        )
        {
            m_FileList.bd496MergeFiles (bNoMorePrimData, m_Stack, m_IndexCounter);
        }

        if  (
            (e_ok == m_TrError                   ) &&
            (! bNoMorePrimData                   ) &&
            (m_Trans.trRteCommPtr_gg00->to_cancel)
        )
        {
            m_TrError = e_cancelled;
            g01optextmsg (sp3p_knldiag, sp3m_info, BD490_CANCEL_1_SP03,
                          csp3_n_index, CANCEL_MSG_1_BD490);
        }
    }

    if (e_ok != m_TrError)
    {
        jobs.AbortAll(waitingTask);

        if (create_index == m_IndexMode){
            m_FileList.bd496DeleteAllFiles();
        }
    }
}

/*---------------------------------------------------------------------------*/

void
cbd490_CreateIndexParallel::bd490_ProcessKeyRangeByCoordinator (
    tsp00_Int4       CurrFileIndex,
    tsp00_Int4       NumLeavesToScan,
    tsp00_Int4       AvgRecordsPerLeaf,
    tsp00_BytePtr    pStartPrimKey,
    tsp00_Int4       StartPrimKeyLen,
    tsp00_BytePtr    pStopPrimKey,
    tsp00_Int4       StopPrimKeyLen,
    tsp00_Int4      &RetryCount,
    tgg00_FileId    &QueueId,
    tsp00_PageNo     PrimQueuePno,
    tsp00_PageNo     SecQueuePno
)
{
    ROUTINE_DBG_MEO00 ("bd490_ProcessKeyRangeByCoordinator");


    if (e_no_server_task_available == m_TrError) m_TrError = e_ok;

    if (e_ok != m_TrError) return;

    tsp00_Bool          bEntriesExist;
    tsp00_PageNo        FirstLeafPno = NIL_PAGE_NO_GG00;
    tgg00_IndexCounter  AuxCounter;
    tgg00_StackDesc     AuxStackDesc =  m_Stack.bd497GetStackDesc(); // reduce code check errors

    RetryCount = 0;

    g01optextmsg (sp3p_knldiag, sp3m_warning,
                  BD490_PROCESS_KEY_RANGE_SP03, csp3_n_index, PROCESS_MSG_1_BD490);

    bd03ParticipantCreateIndex (m_Trans, m_PrimCurrent.curr_tree_id, QueueId,
                                AuxStackDesc, pStartPrimKey, pStopPrimKey, StartPrimKeyLen, StopPrimKeyLen,
                                NumLeavesToScan, AvgRecordsPerLeaf, PrimQueuePno, SecQueuePno,
                                (create_index == m_IndexMode), m_IndexMode, AuxCounter, FirstLeafPno, bEntriesExist);

    if (create_index == m_IndexMode)
    {
        if (NIL_PAGE_NO_GG00 != FirstLeafPno)
            m_FileList.bd496ConvertFile (CurrFileIndex, FirstLeafPno);

        if (! bEntriesExist)
            m_FileList.bd496DeleteFile (CurrFileIndex);

        if (e_ok == m_TrError)
            bd490_AddIndexCounter (m_IndexCounter, AuxCounter);

        if (g01vtrace.vtrAll_gg00 || g01vtrace.vtrBdIndex_gg00)
        {
            tgg11_BdRootTrace RootTrace;

            RootTrace.bdrTrType_gg11.becomes(bdtrRoot_egg11);
            RootTrace.bdrError_gg11  = m_TrError;
            RootTrace.bdrRoot_gg11   = m_IndexCounter.idc_prim_keycount;

            Trace_CommonEntry entry( tgg00_Debug::fromConst(bd), tgg00_VtraceType::fromConst(b03create),
                               REINTERPRET_CAST( tsp00_BytePtr, &RootTrace ), sizeof (RootTrace) );
        }
    }
    g01optextmsg (sp3p_knldiag, sp3m_warning,
                  BD490_PROCESS_KEY_RANGE_SP03, csp3_n_index, PROCESS_MSG_2_BD490);
}

/*---------------------------------------------------------------------------*/
externPascal void gg999Offline (
    tgg00_BasisError                     error);
/*---------------------------------------------------------------------------*/

void
cbd490_CreateIndexParallel::bd490_SendKeyRangeToServer (
    tsp00_Int4       CurrFileIndex,
    tsp00_Int4       NumLeavesToScan,
    tsp00_Int4       AvgRecordsPerLeaf,
    tsp00_BytePtr    pStartPrimKey,
    tsp00_Int4       StartPrimKeyLen,
    tsp00_BytePtr    pStopPrimKey,
    tsp00_Int4       StopPrimKeyLen,
    tsp00_Int4      &NumSendOrders,
    tsp00_Int4      &RetryCount,
    tgg00_FileId    &TempFileId,
    tsp00_PageNo     PrimQueuePno,
    tsp00_PageNo     SecQueuePno,
	SrvTasks_WaitingTask                       &waitingTask,
	SrvTasks_JobList<SrvTasks_JobCreateIndex>  &jobs
)
{
    ROUTINE_DBG_MEO00 ("bd490_SendKeyRangeToServer");


    tgg00_BasisError AuxError = (e_ok == m_TrError) ? e_ok : m_TrError;

    if (e_ok != m_TrError)
        m_TrError = e_ok; // m_TrError could be only e_no_server_task_available

    Msg_List                       &errlist = *(reinterpret_cast<Msg_List*>(m_Trans.trErrorList_gg00));
    SrvTasks_JobCreateIndex::Input  input;
    SrvTasks_JobCreateIndex        *pNewJob = new(jobs.GetAllocator()) SrvTasks_JobCreateIndex(CurrFileIndex);
    if ( pNewJob == 0 )
    {
        m_TrError = e_no_more_memory;
        return;
    }
    input.PrimQueuePno      = PrimQueuePno;
    input.SecQueuePno       = SecQueuePno;
    input.NumLeavesToScan   = NumLeavesToScan; 
    input.AvgRecordsPerLeaf = AvgRecordsPerLeaf; 
    input.StartPrimKeyLen   = StartPrimKeyLen;
    input.StopPrimKeyLen    = StopPrimKeyLen;
    input.IndexMode         = m_IndexMode;
    input.UseMemorySort     = (create_index == m_IndexMode);
    input.PrimFileId        = m_PrimCurrent.curr_tree_id;
    input.InvFileId         = TempFileId;
    input.StackDescLen      = m_Stack.bd497GetStackDescSize();
    input.pStartPrimKey     = pStartPrimKey;
    input.pStopPrimKey      = pStopPrimKey;
    input.pStackDesc        = &m_Stack.bd497GetStackDesc();
    if ( ! pNewJob->Initialize(jobs.GetAllocator(), input, errlist) )
    {
        m_TrError = e_no_more_memory;
        return;
    }
    if (e_ok == m_TrError)
    {
        Msg_List &errlist = *(reinterpret_cast<Msg_List*>(m_Trans.trErrorList_gg00));
        switch ( pNewJob->ExecuteByServer(waitingTask.Task(),errlist,&waitingTask) )
        {
            case SrvTasks_Job::executeOK:
                if ( ! jobs.InsertEnd(pNewJob) )
                    m_TrError = e_no_more_memory;
                break;
            case SrvTasks_Job::executeNoServerAvailable:
                m_TrError = e_no_server_task_available;
                break;
            case SrvTasks_Job::executeError:
                m_TrError = e_start_server_task_error;
                break;
            default:
                m_TrError = e_not_implemented;
                break;
        }
        // k90send (SendMBlock, SendTrans);

        if (e_ok == m_TrError)  ++NumSendOrders;
        else if (e_no_server_task_available == m_TrError)
        {
            g01optextmsg (sp3p_knldiag, sp3m_warning, BD490_SEND_SP03,
                          csp3_n_index, SEND_MSG_1_BD490);
            ++RetryCount;
            // vsleep (m_Trans.trTaskId_gg00, 1);
        }
    }

    if ((e_ok == m_TrError) && (e_no_server_task_available == AuxError))
    {
        RetryCount = 0;
        m_TrError  = e_ok;
    }
}

/*---------------------------------------------------------------------------*/

void
cbd490_CreateIndexParallel::bd490_WaitForReply (
    tsp00_Bool   bNoMorePrimData,
    tsp00_Int4  &NumSendOrders,
    SrvTasks_WaitingTask                      &waitingTask,
    SrvTasks_JobList<SrvTasks_JobCreateIndex> &jobs
)
{
    ROUTINE_DBG_MEO00 ("bd490_WaitForReply");


    tgg00_BasisError    AuxError = (e_ok == m_TrError) ? e_ok : m_TrError;
    tsp00_Int4          OldNumSendOrders = NumSendOrders;

    SrvTasks_JobCreateIndex::Output output;
    tgg00_BasisError                jobError;
    SAPDB_UInt                      jobId;

    if (e_ok != m_TrError) m_TrError = e_ok; // Real error is stored within AuxError

#   if COMPILEMODE_MEO00 >= SLOW_MEO00 
    t01bool (bd_idx_create, "Finish      ", bNoMorePrimData);
    t01int4 (bd_idx_create, "NumSendOrder", NumSendOrders);
#   endif
    
    if ( NumSendOrders != jobs.GetSize() )
        m_TrError = e_internal_error;
        
    while
    (
        (
            (e_ok == m_TrError) &&
            (
                ((0 < NumSendOrders) && (bNoMorePrimData)                        ) ||
                ((NumSendOrders == m_NumUsableServerTasks) && (! bNoMorePrimData))
            )
        )
        ||
        (
            (e_file_limit == AuxError) && (0 < NumSendOrders)
        )
        ||
        (
            (e_no_server_task_available == AuxError) && (NumSendOrders == OldNumSendOrders)
        )
    )
    {
        SrvTasks_JobList<SrvTasks_JobCreateIndex>::Iterator finishedJobIter;
        if ( jobs.WaitForAnyJobFinished( waitingTask, finishedJobIter ) )
        {
            output = (*finishedJobIter)->GetOutput();

            Msg_List errlist;
            jobError = (*finishedJobIter)->GetResult(errlist);
            jobId    = (*finishedJobIter)->GetId();

            if ( jobError != e_ok )
            {
                Msg_List &globalerrlist = *(reinterpret_cast<Msg_List*>(m_Trans.trErrorList_gg00));
                globalerrlist.AppendNewMessage(errlist);
                RTE_Message(globalerrlist);
                globalerrlist.ClearMessageList();
            }
            
            jobs.RemoveJob(finishedJobIter);
        }
        else
            m_TrError = e_no_more_jobs_to_wait_for; // because the other counters should ensure not to wait on empty list

        if (e_ok != m_TrError) return;
        --NumSendOrders;

        if (e_ok != jobError)
        {
            m_TrError = jobError;
            return;
        }

        bd490_AddIndexCounter (m_IndexCounter, output.IndexCounter);

        if (create_index == m_IndexMode)
        {
            if (NIL_PAGE_NO_GG00 != output.FirstLeafPno)
                m_FileList.bd496ConvertFile (jobId, output.FirstLeafPno);

            if ( output.EntriesExist == 0 )
            {
                // Result file from servertask is empty
                m_FileList.bd496DeleteFile (jobId);

                if (e_file_limit == AuxError)
                {
                    // Don't merge if a ServerTask returns an empty file and
                    // the original error says that no empty file is available
                    AuxError = e_ok;
                }
            }
            if (g01vtrace.vtrAll_gg00 || g01vtrace.vtrBdIndex_gg00)
            {
                tgg11_BdRootTrace RootTrace;

                RootTrace.bdrTrType_gg11.becomes(bdtrRoot_egg11);
                RootTrace.bdrError_gg11  = m_TrError;
                RootTrace.bdrRoot_gg11   = m_IndexCounter.idc_prim_keycount;

                Trace_CommonEntry entry( tgg00_Debug::fromConst(bd), tgg00_VtraceType::fromConst(b03create),
                                   REINTERPRET_CAST( tsp00_BytePtr, &RootTrace ), sizeof (RootTrace) );
            }
        }
    }
    if ((e_ok == m_TrError) && (e_ok != AuxError))  m_TrError = AuxError;

#   if COMPILEMODE_MEO00 >= SLOW_MEO00 
    t01int4 (bd_idx_create, "PrimLeafNode",m_IndexCounter.idc_prim_leafnodes);
#   endif
}


/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

cbd490_CreateIndexSequential::cbd490_CreateIndexSequential (
    cbd300_InvCurrent       &InvCurrent,
    tbd_current_tree        &PrimCurrent,
    tgg00_StackDesc         &StackDesc,
    tgg00_IndexCounter      &IndexCounter,
    tgg00_IndexMode			IndexMode
)
        :
        m_TrError		(PrimCurrent.curr_trans->trError_gg00),
        m_Trans			(*PrimCurrent.curr_trans),
        m_InvCurrent    (InvCurrent),
        m_PrimCurrent	(PrimCurrent),
        m_IndexCounter	(IndexCounter),
        m_IndexMode		(IndexMode),
        m_Stack         (PrimCurrent.curr_trans->trError_gg00, StackDesc)
{
    ROUTINE_DBG_MEO00 ("cbd490_CreateIndexSequential");


    if (e_ok != m_TrError) return;

    bd490InitIndexCounter (m_IndexCounter);
}

/*---------------------------------------------------------------------------*/

cbd490_CreateIndexSequential::~cbd490_CreateIndexSequential()
{
    ROUTINE_DBG_MEO00 ("~cbd490_CreateIndexSequential");

    if (e_ok == m_TrError)
        m_IndexCounter.idc_sec_leafnodes = m_InvCurrent.bd300GetRootLeafCount();
}

/*---------------------------------------------------------------------------*/

void
cbd490_CreateIndexSequential::bd490BuildIndex ()
{
    ROUTINE_DBG_MEO00 ("bd490BuildIndex");


    tsp00_Bool         bEntriesExist;

    tsp00_Int4         StartPrimKeyLen = 0;;
    tsp00_Int4         StopPrimKeyLen  = MAX_KEYLEN_GG00;

    tsp00_BytePtr      pStartPrimKey = NULL;
    tsp00_BytePtr      pStopPrimKey =  REINTERPRET_CAST (tsp00_BytePtr, &b01fullkey);

    tgg00_StackDesc    StackDesc =  m_Stack.bd497GetStackDesc(); // reduce code check errors


    cbd491_CreateIndexParticipant Participant (m_PrimCurrent, StackDesc, m_IndexCounter, false);

    Participant.bd491InsertionSort (m_InvCurrent, m_IndexMode, pStartPrimKey, pStopPrimKey,
                                    StartPrimKeyLen, StopPrimKeyLen, bEntriesExist);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/

