/*!***************************************************************************

  module      : vgg942.cpp
  responsible : UweH
  special area: ComponentTests
  created     : 2000-11-24
  last changed: 2000-11-30  9:36
  Copyright (c) 2000-2004 SAP AG
  description : component tests
                DIAGNOSE CHECK INSTANCE <number>

    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"
#include "ggg00.h"
#include "Logging/Log_Types.hpp" // needed for vgg999 PTS 1124727 mb 2003-10-30

#if COMPILEMODE_MEO00 >= SLOW_MEO00
#include "hta99.h" /* nocheck */
#endif

#include "hgg942.h" // check of PASCAL header vgg942
#include "hgg999.h" // KernelAdministration_PascalInterface
#include "hbd03.h"  // bd03TestTree
#include "ggg92.h"  // in dependence of hgg06
#include "hgg06.h"  // gg06buildCompressedKey,gg06CalculateHash

#include "DataAccess/Data_Test.hpp"
#include "KernelCommon/Kernel_Test.hpp"
#include "Logging/Log_Test.hpp"
#include "Logging/Log_Transaction.hpp"
#include "RunTime/RTE_Test.hpp"
#include "Table/Table_Test.hpp"
#include "SAPDBCommon/Fields/SAPDBFields_FieldList.hpp"
#include "SAPDBCommon/Algorithms/SAPDBAlgo_Test.hpp"
#include "SQLManager/SQLMan_Context.hpp"
#include "SQLManager/SQLMan_ContextSessionMembers.hpp"
#include "SQLManager/SQLMan_Test.hpp"
#include "SQLManager/Catalog/Catalog_Test.hpp"
#include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
#include "SAPDBCommon/SAPDB_RangeCode.hpp"
#include "SAPDBCommon/SAPDB_UTF8Basis.hpp"
#include "KernelCommon/Kernel_VTrace.hpp"
#include "KernelCommon/Kernel_Trace.hpp"
#include "KernelCommon/Kernel_Version.hpp"
#include "KernelCommon/Kernel_Exception.hpp"
#include "RunTime/System/RTESys_MicroTime.h"
#include "RunTime/RTE_MySQLProxy.hpp"
#include "ConsistentRead/CnsRead_Test.hpp"
#include "Transaction/Trans_Iterator.hpp"
#include "Transaction/Trans_Context.hpp"
#include "RunTime/Tasking/RTETask_ITask.hpp"
#include "KernelCommon/Kernel_OpMsg.hpp"
#include "Restart/Rst_RedoFile.hpp"
#include "ServerTasks/SrvTasks_Test.hpp"
#include "ggg91.h"
#include "ggg90.h"
#include "hkb560.h"
#include "hkb50.h"
#include "hkb51.h"
#include "hkb52.h"
#include "hkb53.h"
#include "hbd01.h"
#include "hgg01.h"

/*----------------------------------------------------------------------------*/
bool ReserveRedoFile (Rst_RedoFile &file, SAPDB_UInt size, SAPDB_UInt minsize, SAPDB_UInt seq, SAPDB_UInt ioseq, Msg_List &errlist)
{
    Data_SplitSpaceWriter writer = file.ReserveSpace (size, minsize, seq, ioseq, errlist);
    return errlist.IsEmpty();
}

#define UNDO_CHANGES true

/*----------------------------------------------------------------------------*/
bool testRedoFile(Trans_Context &trans)
{
    Msg_List     &errlist = trans.ErrorList();

    if ( trans.OldTrans().trWriteTransId_gg00.gg90IsNil() )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Must run within transaction."));
        return false;
    }

    Rst_RedoFile  file (trans.OldTrans(), Data_PageNo(), Data_PageNo());

    if ( ! file.Create(errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: file.Create()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        return false;
    }

    // reserve successfull in one page 

    if ( ! ReserveRedoFile (file, 8000, 10, 1, 10, errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull in one page; Reserve()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }
        
    if ( ! file.ReleaseSpace( ! UNDO_CHANGES, errlist ) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull in one page; Release()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }

    if ( ! file.Truncate (0, errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull in one page; Truncate()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }

    // reserve UNsuccessfull in one page 

    if ( ! ReserveRedoFile (file, 8000, 10, 1, 10, errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve UNsuccessfull in one page; Reserve()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }
        
    if ( ! file.ReleaseSpace( UNDO_CHANGES, errlist ) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve UNsuccessfull in one page; Release()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }

    // reserve successfull in one page and then reserve as much that it needs to write into a new page

    if ( ! ReserveRedoFile (file, 8000, 10, 2, 20, errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull in one page + one new page; Reserve1()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }
        
    if ( ! file.ReleaseSpace( ! UNDO_CHANGES, errlist ) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull in one page + one new page; Release1()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }

    if ( ! ReserveRedoFile (file, 8000, 2000, 3, 30, errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull in one page + one new page; Reserve2()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }
        
    if ( ! file.ReleaseSpace( UNDO_CHANGES, errlist ) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull in one page + one new page; Release2()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }

    // reserve successfull more then one page

    if ( ! ReserveRedoFile (file, 8000*4, 40, 4, 40, errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull more then one page; Reserve()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }
        
    if ( ! file.ReleaseSpace( ! UNDO_CHANGES, errlist ) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve successfull more then one page; Release()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }

    // reserve UNsuccessfull more then one page

    if ( ! ReserveRedoFile (file, 8000*4, 40, 5, 50, errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve UNsuccessfull more then one page; Reserve()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }
        
    if ( ! file.ReleaseSpace( UNDO_CHANGES, errlist ) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"Test: reserve UNsuccessfull more then one page; Release()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        Msg_List auxerrlist;
        if ( ! file.Drop(auxerrlist) )
        {
            auxerrlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
            errlist.AppendNewMessage(auxerrlist);
            return false;
        }
        return false;
    }

    // drop the complete file

    if ( ! file.Drop(errlist) )
    {
        errlist.AppendNewMessage(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"file.Drop()"));
        trans.OldTrans().trWriteTransId_gg00.gg90SetNil();
        return false;
    }
    return true;
}

/*----------------------------------------------------------------------------*/
void randomKeyAssign ( SAPDB_Byte                     *s,
                       SAPDB_UInt                      l,
                       RTE_MySQLPseudoRandomGenerator &rand,
                       bool                            onlyAscii )
{
    const SAPDB_UInt variableLength = l/10;
    const SAPDB_UInt firstOffset    = l/2 - variableLength/2;
    const SAPDB_UInt lastOffset     = firstOffset+variableLength;
    SAPDB_MemFillNoCheck (s,0x20,l-1);
    for ( SAPDB_UInt i = firstOffset; i < lastOffset; i++ )
    {
        if ( onlyAscii )
        {
            SAPDB_Byte b = static_cast<SAPDB_Byte>(rand.RandomNumber() * 127);
            *(s+i) = b < 32 ? 32 : b;
        }
        else
            *(s+i) = static_cast<SAPDB_Byte>(rand.RandomNumber() * SAPDB_MAX_UINT1);
    }
    // Kernel_TraceBuffer (s,l,"randomKey", 80);
}

#include "FileDirectory/FileDir_Test.hpp"

/*----------------------------------------------------------------------------*/
void testCompression(tgg00_TransContext& trans)
{
    const SAPDB_Byte key1[] = { 0xFF,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,
                                0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,
                                0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xFF,0xDD,0xDD,
                                0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,
                                0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,0xDD,
                                0xDD,0xDD,0xFF,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,
                                0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,
                                0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xFF
                              };
    const SAPDB_UInt   key1length = 80;
    const SAPDB_Byte key2[] = { 0xFF,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,
                                0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,
                                0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xFF,0xFF,0xBB,
                                0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,
                                0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,0xBB,
                                0xBB,0xBB,0xBB,0xBB,0xBB,0xFF
                              };
    const SAPDB_UInt   key2length = 56;

	const tsp00_Uint4   COMPRESSEDKEYSIZE = 64;
          tsp00_Byte    compressedkey [COMPRESSEDKEYSIZE];
    
    (void)gg06buildCompressedKey (key1,key1length,1,key1length,COMPRESSEDKEYSIZE,compressedkey);

    if ( ! memcmp (key2,compressedkey,key2length) )
        Kernel_VTrace() << "Keycompression passed.";
	else
        Kernel_TraceBuffer (compressedkey,COMPRESSEDKEYSIZE,"BAD compressedkey", COMPRESSEDKEYSIZE);
}

/*----------------------------------------------------------------------------*/
void testKnownKeys(tgg00_TransContext& trans)
{
    const SAPDB_Byte key1[] = { 0x20,0x30,0x30,0x30,0x20,0x46,0x4f,0x52,0x4d,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x48,0x52,0x5f,0x49,0x44,0x5f,0x54,0x41,0x58,0x31,0x37,0x32,0x31,0x41,0x42,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x54,0x58,0x54,0x20,0x20,0x45 };
    const SAPDB_UInt   key1length = 93;
    const SAPDB_Byte key2[] = { 0x20,0x30,0x30,0x30,0x20,0x46,0x4f,0x52,0x4d,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x4f,0x44,0x5f,0x56,0x42,0x4c,0x5f,0x41,0x4e,0x4d,0x45,0x4c,0x20,0x20,0x20,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
                                0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x54,0x58,0x54,0x20,0x20,0x45 };
    const SAPDB_UInt   key2length = 93;

    const tsp00_Uint4   MAXHASHTYPE       = 3;
	const tsp00_Uint4   COMPRESSEDKEYSIZE = 64;
          tsp00_Byte    compressedkey1 [COMPRESSEDKEYSIZE];
          tsp00_Byte    compressedkey2 [COMPRESSEDKEYSIZE];
          tsp00_Uint4   key1hashvalue;
          tsp00_Uint4   key2hashvalue;
    
    for ( tsp00_Byte hashtype = 0; hashtype < MAXHASHTYPE; hashtype++ )
    {
        (void)gg06buildCompressedKey (key1,key1length,1,key1length,COMPRESSEDKEYSIZE,compressedkey1);
        (void)gg06buildCompressedKey (key2,key2length,1,key2length,COMPRESSEDKEYSIZE,compressedkey2);

        Kernel_TraceBuffer (compressedkey1,COMPRESSEDKEYSIZE,"compressedkey1", COMPRESSEDKEYSIZE);
        Kernel_TraceBuffer (compressedkey2,COMPRESSEDKEYSIZE,"compressedkey2", COMPRESSEDKEYSIZE);

        if ( memcmp (compressedkey1,compressedkey2,COMPRESSEDKEYSIZE) )
            Kernel_VTrace() << "Hash Type: " << hashtype << " (0=old,1=fnv,2=simple) PASSED.";
        else
        {
            Kernel_VTrace() << "LockKey collision for type (0=old,1=fnv,2=simple): " << hashtype;
            gg06CalculateHash(key1,key1length,hashtype,key1hashvalue);
            gg06CalculateHash(key2,key2length,hashtype,key2hashvalue);
            if ( key1hashvalue == key2hashvalue )
                Kernel_VTrace() << "Hash collision for type (0=old,1=fnv,2=simple): " << hashtype;
            else
                Kernel_VTrace() << "NO Hash collision ?";
        }
    }
}

/*----------------------------------------------------------------------------*/
void testHashSpreading(tgg00_TransContext& trans,
                       bool                onlyAscii)
{
    const tsp00_Uint4   MAXLOOPS = 100000;
    const tsp00_Uint4   MAXHASHTYPE = 3;

    typedef tsp00_Uint2 CollisionList [MAXLOOPS];

    CollisionList       spreading     [MAXHASHTYPE];
    tsp00_Uint4         collisions    [MAXHASHTYPE];
    
    const tsp00_Uint4   KEYLENGTH         = 1024;
    tsp00_Byte          key           [KEYLENGTH];

    tsp00_Uint4         hashvalue;
    tsp00_Uint4         loop;
    tsp00_Byte          hashtype;

    Kernel_VTrace() << "TEST SPREADING: loops=" << MAXLOOPS << " keylength=" << KEYLENGTH
                    << (onlyAscii?" only ascii keys":" keys 0-255");

    for ( hashtype = 0; hashtype < MAXHASHTYPE; hashtype++ )
    {
        collisions[hashtype] = 0;
        for ( loop = 0; loop < MAXLOOPS; loop++ )
            spreading [hashtype][loop] = 0;
    }
    
    RTE_MySQLPseudoRandomGenerator rand(11);

    for ( loop = 0; loop < MAXLOOPS; loop++ )
    {
        randomKeyAssign (key, KEYLENGTH, rand, onlyAscii);
        for ( hashtype = 0; hashtype < MAXHASHTYPE; hashtype++ )
        {
            gg06CalculateHash(key,KEYLENGTH,hashtype,hashvalue);
            if ( spreading [hashtype][hashvalue%MAXLOOPS] > 0 )
                collisions[hashtype]++;
            (spreading[hashtype][hashvalue%MAXLOOPS])++;
        }
    }
    for ( hashtype = 0; hashtype < MAXHASHTYPE; hashtype++ )
        Kernel_VTrace() << "HASHTYPE (0=old,1=fnv,2=simple) " << hashtype
                        << ": collisions=" << collisions[hashtype];
}
/*----------------------------------------------------------------------------*/
void testHashSpeed (tgg00_TransContext& trans)
{
    tgg00_BasisError   &berr              = trans.trError_gg00;
    const tsp00_Uint4   KEYLENGTH         = 1024;
	const tsp00_Uint4   COMPRESSEDKEYSIZE = 64;
    const tsp00_Uint4   MAXLOOPS          = 100000;
    tsp00_Byte          key           [KEYLENGTH];
    tsp00_Byte          compressedkey [COMPRESSEDKEYSIZE];
    tsp00_Uint4         loop;
    SAPDB_UInt8         timeBegin;
    SAPDB_UInt8         timeEnd;

    SAPDB_MemFillNoCheck (key, 0xFF, KEYLENGTH);
    SAPDB_MemFillNoCheck (compressedkey, 0x00, COMPRESSEDKEYSIZE);

    Kernel_VTrace() << "TEST HASHES:";
    Kernel_VTrace() << "KEYLENGTH = " << KEYLENGTH << ", COMPRESSEDKEYSIZE = " << COMPRESSEDKEYSIZE;
    Kernel_VTrace() << "MAXLOOPS  = " << MAXLOOPS;

    timeBegin = RTESys_MicroSecTimer();
    for ( loop = 0; loop < MAXLOOPS; loop++ )
    {
        (void)gg06buildCompressedKey (key,KEYLENGTH,1,KEYLENGTH,COMPRESSEDKEYSIZE,compressedkey);
    }
    timeEnd = RTESys_MicroSecTimer();
    Kernel_VTrace() << "time = " << timeEnd-timeBegin;
}

/*----------------------------------------------------------------------------*/

externPascal void gg942ComponentTestsWithStringarg ( tgg00_TransContext& trans,
                                                     tsp00_Int4          TestNo,
                                                     tsp00_MoveObjPtr    StringPtr,
                                                     tsp00_Bool          isUnicode,
                                                     tsp00_Int4          StringLen )
{
    Trans_Context          &context            = Trans_Context::GetContext(trans);
    RTETask_ITask          &task               = context.Task();
    Msg_List               &errlist            = context.ErrorList();
    SAPDBMem_IRawAllocator &allocator          = context.Allocator();
    
    Kernel_Version::ComponentType    componentType;
    Kernel_Version::BuildKind        buildInfo;
    Kernel_Version::DevelopmentPhase developPhase;
    Kernel_Version::InstanceType     builtForInstance;
    
    Kernel_Version::GetExtraInfoFromSoftware (componentType, buildInfo, developPhase, builtForInstance);
    if ( developPhase != Kernel_Version::DevelopPhase ) 
    {
        trans.trError_gg00 = e_invalid_command;
        return;
    }
    else
    {
        SAPDB_UTF8             *utf8Argument       = 0;
        SAPDB_Int4              utf8ArgumentLength = 0;
        
        if ( StringLen > 0 )
        {
            if ( isUnicode )
            {
                utf8Argument = (SAPDB_UTF8 *)allocator.Allocate(3*StringLen + 1);
                if ( utf8Argument )
                {
                    tsp81_UCS2Char const * srcAt;
                    SAPDB_UTF8 * destAt;
                    if ( SAPDB_UTF8Basis::Success 
                        != SAPDB_UTF8Basis::KernelConvertFromUTF16(
                        (tsp81_UCS2Char *)StringPtr,
                        (tsp81_UCS2Char *)(((SAPDB_Byte *)StringPtr)+StringLen),
                        srcAt,
                        0, // command part always High byte / Low byte -> not swapped
                        utf8Argument,
                        utf8Argument+3*StringLen,
                        destAt ) )
                    {
                        allocator.Deallocate(utf8Argument);
                        errlist =  SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                            "DIAGNOSE CHECK INSTANCE string conversion" );
                        trans.trError_gg00 = e_internal_error;
                        return;
                    }
                    utf8ArgumentLength = destAt - utf8Argument;
                    *destAt = 0;
                }
            }
            else
            {
                utf8Argument = (SAPDB_UTF8 *)allocator.Allocate(StringLen + 1);
                if ( utf8Argument )
                {
                    SAPDB_MemCopyNoCheck(utf8Argument, StringPtr, StringLen); // source not zero terminated
                    utf8Argument[StringLen] = 0;
                    utf8ArgumentLength = StringLen;
                }
            }
            
            if ( !utf8Argument )
            {
                errlist =  SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                    "DIAGNOSE CHECK INSTANCE string allocation" );
                trans.trError_gg00 = e_internal_error;
                return;
            }
        }
        
        switch (TestNo)
        {
        case 1:
            if ( ! Data_Test (trans, errlist) )
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                "DIAGNOSE CHECK INSTANCE 1 (Data)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 2:
            if ( ! Kernel_Test (task, errlist) )
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                "DIAGNOSE CHECK INSTANCE 2 (Kernel: Version + Migration)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 3:
            {
                Log_Test testRunner;
                testRunner.PerformTest(trans, Log_Test::Log_ResetStatistics);
                // testRunner.PerformTest(trans, Log_Test::Log_TestIOSequences);
                // testRunner.PerformTest(trans, Log_Test::Log_TestActions);       
                break;
            }
        case 4:
            RTE_Test (trans, utf8Argument, utf8ArgumentLength);
            break;
        case 5:
            if ( ! Table_Test (context) )
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                "DIAGNOSE CHECK INSTANCE 5 (Table_Test)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 7:
            if ( ! SQLMan_Test(context) )
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                "DIAGNOSE CHECK INSTANCE 7 (SQLMan_Test)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 8:
            if ( ! SAPDBAlgo_Test(context) )
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                "DIAGNOSE CHECK INSTANCE 8 (SAPDBAlgo_Test)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 13: case 14: case 15: case 16: case 17: case 18: case 19: case 20: case 21: case 22:
            FileDir_Test::Test ( trans, TestNo - 12 );
            break;
        case 9:
            bd03TestTree(trans);
            break;
        case 10:
            testHashSpeed(trans);
            testHashSpreading(trans,false);
            testHashSpreading(trans,true);
            testKnownKeys(trans);
            testCompression(trans);
            break;
        case 11:
            if ( ! SAPDBFields_FieldList::Test(errlist) )
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                "DIAGNOSE CHECK INSTANCE 11 (FieldList)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 12:
            if (!testRedoFile(Trans_Context::GetContext(trans)))
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                                                       "DIAGNOSE CHECK INSTANCE 12 (RedoFile)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 25: case 26: case 27: case 28: case 29:
            Catalog_Test::Execute(TestNo - 24);
            break;
        case 30: case 31: case 32: case 33: case 34:
            CnsRead_Test::Execute(TestNo - 29);
            break;
        case 35:
            if (!Trans_Iterator::Test(Trans_Context::GetContext(trans)))
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                                                       "DIAGNOSE CHECK INSTANCE 35 (Trans_Iterator)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        case 36:
            if (!SrvTasks_Test(Trans_Context::GetContext(trans)))
            {
                errlist = errlist + SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                                                       "DIAGNOSE CHECK INSTANCE 36 (ServerTasks)" );
                trans.trError_gg00 = e_internal_error;
            }
            break;
        default:
            break;
        }
        
        allocator.Deallocate(utf8Argument);
    }
}

externPascal void gg942ComponentTests (tgg00_TransContext& trans,
                                       tsp00_Int4          TestNo)
{
    ROUTINE_DBG_MSP00 ("gg942ComponentTests");
    gg942ComponentTestsWithStringarg( trans, TestNo, (tsp00_MoveObjPtr)0, 0, 0);
}
