#!/usr/bin/perl
#
# $SRC/install/perl/Makefile.PL
#
# $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/Makefile.PL#1 $
# $DateTime: 2005/03/21 16:21:00 $
# $Change: 93297 $
#

use Config;
use Symbol;

sub main {
	print "Generating Makefiles\n";

	my %Makedefs = (
	'PERL_PERL' => 'perl',
	'PERL_COREDIR' => get_coredir (),
	'PERL_CHG2PM' => get_chg2pm (),
	'PERL_TXT2PM' => get_txt2pm (),
	'PERL_TGZCREATE' => get_tgzcreate (),
	'PERL_WXTGZCREATE' => get_wxtgzcreate (),
	'PERL_MKDIR' => get_mkdir (),
	'PERL_RM' => get_rm (),
	'PERL_CP' => get_cp (),
	'PERL_TOOLLIB' => get_toollib (),
	'PERL_INSTPREFIX' => get_instprefix (),
	'PERL_INSTALL_PERLDLL' => get_install_perldll (),
	'PERL_INSTALL_IMAGESTGZ' => get_install_imagestgz (),
	'PERL_INSTALL_WXTGZ' => get_install_wxtgz (),
	'PERL_INSTALL_SDBINSTTGZ' => get_install_sdbinsttgz (),
	'PERL_DEEPCOPY' => get_deepcopy (),
	'PERL_EXEEXT' => $Config{'exe_ext'},
	'PERL_MYMAKE' => $^O =~ /aix/i ? 'gmake' : '$(MAKE)',
	'PERL_INSTANCE_PACKAGEDATA' => get_instance_packagedata(),
	'PERL_INSTANCE_PACKAGESCRIPT' => get_instance_packagescript()
	);

	write_makefile (%Makedefs);

	#
	# setup include path for cross compilation environment
	#
	if ($^O =~ /mswin/i && $Config{'archname'} =~ /IA64/) {
		push @INC, "../win64/libx86", "../win64/lib";
	}

	chdir ("lib"); do ("Makefile.PL"); chdir ("..");
	chdir ("tool/lib"); do ("Makefile.PL"); chdir ("../..");
	chdir ("sdbrun"); do ("Makefile.PL"); chdir ("..");
	chdir ("helpers"); do ("Makefile.PL"); chdir ("..");
}

sub write_makefile {
	my %Makedefs = @_;
	my $fh = gensym ();
	open ($fh, "> Makefile");
	print ($fh
	"#\n".
	"# Makefile\n".
	"# this file is generated from Makefile.PL\n".
	"# do not edit this file, any changes will be lost\n".
	"#\n".
	"\n".	
	".SUFFIXES:\n".
	"\n");

	if ($^O =~ /^aix/) {
		print ($fh
		"#\n".
		"# make sure that we are in 32 bit mode on aix\n".
		"#\n".
		"OBJECT_MODE = 32\n".
		"\n");
	}

	while (my ($key, $value) = each %Makedefs) {
		next unless (defined $value);
		print ($fh $key." = ".$value."\n");
	}
	print $fh ("\n");

	write_body ($fh);
	close ($fh);
}

sub write_body {
	my $fh = shift;
	while (<DATA>) {
		chomp;
		print ($fh $_."\n");
	}
}

sub get_install_perldll {
	($^O =~ /mswin/i) ?
		"\$(CP) ".($Config{'libperl'} =~ /(.+)\.lib/ )[0].".dll \$(INST_PREFIX)" :
	($^O =~ /hpux/i && $Config{'archname'} !~ /ia64/i) ?
		"\$(CP) \$(COREDIR)/libperl.sl \$(INST_PREFIX) && ".
		"chmod u+w \$(INST_PREFIX)/libperl.sl" :
	($^O =~ /aix/i) ?
		"\$(CP) \$(COREDIR)/libperl.a \$(INST_PREFIX) && ".
		"chmod u+w \$(INST_PREFIX)/libperl.a" :
		"\$(CP) \$(COREDIR)/libperl.so \$(INST_PREFIX) && ".
		"chmod u+w \$(INST_PREFIX)/libperl.so"; 
}

sub get_coredir {
	my $dir =
	defined ($Config{'archlibexp'}) ? $Config{'archlibexp'}.'/CORE' :
	$Config{'privlibexp'}.'/CORE';

	$dir =~ s/\//\\/ if $^O =~ /^mswin/i;
	return ($dir);
}

sub get_install_sdbinsttgz {
	"\$(CP) SDBINST.TGZ \$(INST_PREFIX)";
}

sub get_install_wxtgz {
	if($^O =~ /mswin/i and exists $ENV{'XCOMP'}){
		return undef;
	}
	"\$(CP) WXPERL.TGZ \$(INST_PREFIX)";
}

sub get_install_imagestgz {
	if($^O =~ /mswin/i and exists $ENV{'XCOMP'}){
		return undef;
	}
	"\$(CP) RESOURCES.TGZ \$(INST_PREFIX)";
}

sub get_mkdir {
	$^O =~ /^mswin/i ? "mkdir" : "-mkdir -p";
}

sub get_rm {
	$^O =~ /^mswin/i ? "del" : "-rm";
}

sub get_cp {
	$^O =~ /^mswin/i ? "copy" : "cp -p";
}

sub get_instprefix {
	$^O =~ /^mswin/i ?
	$ENV{'OWN'}."\\sys\\wrk\\fast\\install" :
	$ENV{'OWN'}."/sys/wrk/fast/install";
}

sub get_toollib {
	$^O =~ /^mswin/i ? "tool\\lib" : "tool/lib";
}

sub get_tgzcreate {
	$^O =~ /^mswin/i ?
	"\$(PERL) ..\\tool\\tgzcreate" :
	"\$(PERL) ../tool/tgzcreate";
}

sub get_wxtgzcreate {
	if($^O =~ /mswin/i and exists $ENV{'XCOMP'}){
		return undef;
	}
	$^O =~ /^mswin/i ?
	"cd \$(INST_PREFIX)\\local\\lib && \$(PERL) ..\\..\\tmp\\tool\\wxar ".
	"../../tmp/WXPERL.TGZ hello.pm Wx Wx.pm" :
	"cd \$(INST_PREFIX)/local/lib && \$(PERL) ../../tmp/tool/wxar ".
	"../../tmp/WXPERL.TGZ Wx hello.pm Wx.pm";
}

sub get_deepcopy {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) tool\\deepcopy" :
	"\$(PERL) tool/deepcopy";
}

sub get_chg2pm {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ..\\tool\\chg2pm" :
	"\$(PERL) ../tool/chg2pm";
}

sub get_txt2pm {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ..\\tool\\txt2pm" :
	"\$(PERL) ../tool/txt2pm";
}

sub get_instance_packagedata{
	my $file = get_instprefix().'/packages/instance/PACKAGEDATA';
	if($^O =~ /mswin/i){
		$file =~ s/\//\\\\/g;
	}
	return $file;	
}

sub get_instance_packagescript{
	my $packagedata = get_instance_packagedata();
	my $script;
	open(FH,$packagedata) or die("cannot open PACKAGEDATA:$!\n");
	while(<FH>){
		chomp;
		($script) = (/^\s*SCRIPT\s*=\s*\"(.+)\"/);
		if($script =~ /\S/){
				last;
		}
	}	
	close(FH);
	$script =  get_instprefix().'/packages/instance/'.$script;
	if($^O =~ /mswin/i){
		$script =~ s/\//\\\\/g;
	}
	return $script;	
}

main (@ARGV);

__DATA__
PERL = $(PERL_PERL)
RM = $(PERL_RM)
CP = $(PERL_CP)
MKDIR = $(PERL_MKDIR)
COREDIR = $(PERL_COREDIR)
MYMAKE = $(PERL_MYMAKE)
INST_PREFIX = $(PERL_INSTPREFIX)
INSTALL_PERLDLL = $(PERL_INSTALL_PERLDLL)
INSTALL_WXTGZ = $(PERL_INSTALL_WXTGZ)
INSTALL_IMAGESTGZ = $(PERL_INSTALL_IMAGESTGZ)
INSTALL_SDBINSTTGZ = $(PERL_INSTALL_SDBINSTTGZ)
TOOLLIB = $(PERL_TOOLLIB)
CHG2PM = $(PERL_CHG2PM)
TXT2PM = $(PERL_TXT2PM)
DEEPCOPY = $(PERL_DEEPCOPY)
TGZCREATE = $(PERL_TGZCREATE)
WXTGZCREATE = $(PERL_WXTGZCREATE)
EXEEXT = $(PERL_EXEEXT)
INSTANCE_PACKAGEDATA = $(PERL_INSTANCE_PACKAGEDATA)
INSTANCE_PACKAGESCRIPT = $(PERL_INSTANCE_PACKAGESCRIPT)

all: 
	cd lib && $(MAKE)
	cd $(TOOLLIB) && $(MAKE)
	cd sdbrun && $(MAKE)
	cd helpers && $(MAKE)
	cd buildroot && $(TGZCREATE) ../SDBINST.TGZ .
	cd images && $(TGZCREATE) ../RESOURCES.TGZ .
	cd buildroot && $(TXT2PM) $(INSTANCE_PACKAGEDATA) . SAPDB::Install::BuildPackage::packages::instance::packagedata  
	cd buildroot && $(TXT2PM) $(INSTANCE_PACKAGESCRIPT) . SAPDB::Install::BuildPackage::packages::instance::script -append 

install: all
	cd helpers && $(MAKE) install
	cd sdbrun && $(MAKE) install
#	$(DEEPCOPY) SAPDB $(INST_PREFIX)
	$(INSTALL_SDBINSTTGZ)
	$(INSTALL_PERLDLL)
	$(INSTALL_WXDLLS)
	$(WXTGZCREATE)
	$(INSTALL_WXTGZ)
	$(INSTALL_IMAGESTGZ)

clean:
	cd lib && $(MAKE) clean
	cd $(TOOLLIB) && $(MAKE) clean
	cd helpers && $(MAKE) clean
	cd sdbrun && $(MAKE) clean
	$(RM) SDBINST.TGZ
	$(RM) WXPERL.TGZ
	$(RM) RESOURCES.TGZ
	$(RM) Makefile
