#!/usr/bin/perl
#
# $Header: //sapdb/I00/develop/sys/src/install/perl/SAPDB/Install/App.pm#7 $
# $DateTime: 2003/08/08 16:56:44 $
# $Change: 49684 $
#
# Desc: Resource module of SDBINST / SDBUPD / sdbunist / sdbregview / sdbverify
#

package SAPDB::Install::Resources;

use Wx;
use SAPDB::Install::StdIO;	

our %resource_cache;

sub new {
	my ($type,$archive) = @_;
	return $resource_cache{$archive} if (exists $resource_cache{$archive});

	-f $archive or
	print2stderr("cannot open archive $archive: $!\n")
	and return;

	my %members;
	$members{'archive'} = $archive;
	$members{'tar'} = SDB::Common::Archive::Tar::new ($archive);
	unless (defined $members{'tar'}) {
		print2stderr("cannot open archive: $archive\n");
		return;
	}

	$resource_cache{$archive} = bless(\%members,$type);
	return $resource_cache{$archive};
}

sub readAll {
	my ($self) = @_;

	my $ah = $self->{'tar'};
	$ah->rewind ();

	my $bmp_hnd;
	my $png_hnd;
	my $jpg_hnd;

	for(;;){
		my $name = $ah->read ();
		last unless (defined $name && $name ne '');

		if($name =~ /\.bmp$|\.png$|\.jpg$|\.jepg$/i) {
			my $fh = $ah->open ($name);
			my $img = Wx::Image->new ();

			if ($name =~ /\.png$/i){
				unless(defined $png_hnd){
					$png_hnd = Wx::PNGHandler->new(); 
				}
				$png_hnd->LoadFile ($img, $fh);
			} elsif ($name =~ /\.bmp$/i) {
				unless(defined $bmp_hnd){
					$bmp_hnd = Wx::BMPHandler->new();
				}
				$bmp_hnd->LoadFile ($img, $fh);
			} elsif ($name =~ /\.jpg$/i || $name =~ /\.jepg$/i) {
				unless(defined $jpg_hnd){
					$jpg_hnd = Wx::JPEGHandler->new();
				}
				$jpg_hnd->LoadFile ($img, $fh);
			}
			$self->{$name} = $img;			
		}
	}
}

sub getImages{
	my ($self,@list) = @_;
	my %return_value;
	my @missing;

	foreach my $image (@list){
		if( defined $self->{$image}){
			$return_value{$image} = $self->{$image};
		} else {
			push @missing, $image;
		}
	}

	if ($#missing >= 0) {
		my $ah = $self->{'tar'};
		$ah->rewind ();

		my $bmp_hnd;
		my $png_hnd;
		my $jpg_hnd;

		for (;;) {
			my $name = $ah->read ();
			last unless (defined $name && $name ne '');

			foreach my $mem (@missing){
				next unless ($name eq $mem);
				my $fh = $ah->open ($name);
				my $img = Wx::Image->new ();
				if($name =~ /\.png$/i){
					unless(defined $png_hnd){
						$png_hnd = Wx::PNGHandler->new(); 
					}
					$png_hnd->LoadFile ($img, $fh);
				} elsif ($name =~ /\.jpg$/i || $name =~ /\.jepg$/i) {
					unless(defined $jpg_hnd){
						$jpg_hnd = Wx::JPEGHandler->new();
					}
					$jpg_hnd->LoadFile ($img, $fh);
				} 
				else{
					unless(defined $bmp_hnd){
						$bmp_hnd = Wx::BMPHandler->new();
					}
					$bmp_hnd->LoadFile ($img, $fh);
				}
				$self->{$name} = $img;
				$return_value{$name} = $img;				
			}
		}
	}

	return \%return_value;
}

sub getData {
	my ($self,@list) = @_;
	$self->{'tar'}->rewind();
	my %return_value;

	for(;;){
		my $name = $self->{'tar'}->read ();
		last unless (defined $name && $name ne '');

		foreach my $mem (@list){
			next unless ($name eq $mem);

			my $fh = $ah->open ($name);
			my $img = Wx::Image->new ();

			my $imgdata = $img->GetData ();
			my $height = $img->GetHeight ();
			my $width = $img->GetWidth ();
			$return_value{$mem} = [$imgdata,$width,$height];
		}
	}
	return \%return_value;
}

1;
__END__

package SAPDB::Install::Resources;

use Wx qw (wxBLACK wxSOLID wxWHITE);

use SAPDB::Install::Untgz;
use SAPDB::Install::StdIO;	

local %resource_cache;

sub new{
	my ($type,$archive) = @_;
	if (exists $resource_cache{$archive}){
		return $resource_cache{$archive};
	}
	-f $archive or print2stderr("cannot open archive $archive: $!\n") and return;
	my %members;
	$members{'archive'} = $archive;
	$members{'untgz'} = SAPDB::Install::Untgz::new();
	$members{'untgz'}->Open($archive);
	if($members{'untgz'}->GetErr){
		print2stderr("cannot open archive: ".join(': ',$untgz->GetErr)."\n");
		return;
	}
	$members{'brush'} = Wx::Brush->new(wxWHITE,wxSOLID);
    $members{'pen'} = Wx::Pen->new(wxBLACK,1,wxSOLID);
	$resource_cache{$archive} = bless(\%members,$type);
	return $resource_cache{$archive};
}


sub DIB2DDB{
	my ($data) = @_;
	my $file_header = {};
	my $bmih = {};
	my ($fh,$bmi_header) = ($data =~ /^(.{14})(.{40})/);
	$rest = $data;
	$data = undef;
	 $rest =~ s/^.{54}//;
	($file_header->{'type'},$file_header->{'size'},
		$file_header->{'reserverd1'},$file_header->{'reserverd2'},
		$file_header->{'offbits'}) = unpack('vVvvV',$fh);
	if($file_header->{'type'} != 0x4d42){
		print2stderr("ERR: DIB2DD(): invalid file header".sprintf('%x',$file_header->{'type'})."\n");
	}
	
	($bmih->{'size'},$bmih->{'width'},$bmih->{'height'},$bmih->{'planes'},
	$bmih->{'bitcount'},$bmih->{'compression'},$bmih->{'sizeimage'},
	$bmih->{'xperls_per_meter'},$bmih->{'yperls_per_meter'},
	$bmih->{'clr_used'},$bmih->{'clr_important'}) = unpack('VVVvvVVVVVV',$bmi_header);
	my $mapsize = (2 ** $bmih->{'bitcount'}) * 4 ;
	my ($colourmap) = ($rest =~ /^(.{$mapsize})/);
	$rest =~ s/^.{$mapsize}//;
	my @colourmap;
	my $index = 0;
	while(1){
		last if (length($colourmap) == 0);
		(my $colour,$colourmap) = ($colourmap =~ /^(.{4})(.*)$/);
		my ($b,$g,$r,$x) = unpack('CCCC',$colour);
		$index++;
		push @colourmap, pack('CCC',$r,$g,$b);
	
	}
	
	my $img_data;
	
	my $y = $bmih->{'width'};
	
	$line_count = 0;
	
	while(1){
		$line_count ++;
		last if $line_count > $bmih->{'height'} + 1 ;
		last if (length($rest) == 0);
		($line) = ($rest =~ /(.{$y})$/);
		unless ($line){
			last;
			(my $tmp) = ($rest =~ s/(.){100}$//);
			($line) = ($rest =~ s/(.{$y - 100})$//);
			$line = $tmp.$line;
		}
	
		$rest = $rest;
		$rest =~ s/.{$y}$//;
		while(1){
			last if (length($line) == 0);
			(my $point) = ($line =~ /^(.)/);
			$line =~ s/^.//;
			$point = unpack('C',$point);
			$img_data .= $colourmap[$point];
			my ($r,$g,$b) = unpack('CCC',$colourmap[$point]);
		}
	}
	return ($bmih,$img_data);
}

sub readAll{
	my ($self) = @_;
	$self->{'untgz'}->Rewind();
	while(1){
		my $info = $self->{'untgz'}->Next;
		last unless($info) || $self->{'untgz'}->GetErr;
		next if $info =~ /^d/;
		my ($file) = ($info =~ /\s(\S+)$/);
		if($file =~ /\.bmp$/i){
			my $imgdata,$height,$width; 				
			$self->{'untgz'}->ExtractDIB($width,$height,$imgdata);
			my $img = Wx::Image->new($width,$height);
			$img->SetData($imgdata);
			$self->{$file} = $img;				
		}
	}
}

sub getImages{
	my ($self,@list) = @_;
	my %return_value;
	my @missing;
	foreach my $image (@list){
		if( defined $self->{$image}){
			$return_value{$image} = $self->{$image};
		}
		else{
			push @missing,$image;
		}
	}
	
	if($#missing > -1){
		$self->{'untgz'}->Rewind();
		while(1){
			my $info = $self->{'untgz'}->Next;
			last unless($info) || $self->{'untgz'}->GetErr;
			next if $info =~ /^d/;
			my ($file) = ($info =~ /\s(\S+)$/);
			foreach my $mem (@missing){
				if($file eq $mem){
					my $imgdata,$height,$width; 				
					#$self->{'untgz'}->ExtractScalar($data);
					#my ($bmih,$imgdata) = DIB2DDB($data);
					#my $img = Wx::Image->new($bmih->{'width'},$bmih->{'height'});
					$self->{'untgz'}->ExtractDIB($width,$height,$imgdata);
					my $img = Wx::Image->new($width,$height);
					$img->SetData($imgdata);
					$self->{$mem} = $img;				
					$return_value{$mem} = $img;
				}
			}
		}
	}
	return \%return_value;
}

sub getData{
	my ($self,@list) = @_;
	$self->{'untgz'}->Rewind();
	my %return_value;
	while(1){
		my $info = $self->{'untgz'}->Next;
		last unless($info) || $self->{'untgz'}->GetErr;
		next if $info =~ /^d/;
		my ($file) = ($info =~ /\s(\S+)$/);
		foreach my $mem (@list){
			if($file eq $mem){
				my $imgdata,$height,$width; 				
				$self->{'untgz'}->ExtractDIB($width,$height,$imgdata);
				$return_value{$mem} = [$imgdata,$width,$height];
			}
		}
	}
	return \%return_value;
}

sub getMDC{
	my ($self,@list) = @_;
	my $img_data = $self->getData(@list);
	my %returnvalue;
	foreach my $id (keys(%$img_data)){
		$returnvalue{$id} = Wx::MemoryDC->new();
		$returnvalue{$id}->SelectObject(Wx::Bitmap->new($img_data->{$id}->[1],$img_data->{$id}->[2]));
		$returnvalue{$id}->BeginDrawing() if $^O =~ /mswin/i;
		$returnvalue{$id}->SetBackground($self->{'brush'});
  		$returnvalue{$id}->Clear;
		my $data = $img_data->{$id}->[0];
		my $row_length = $img_data->{$id}->[1] * 3;;
		my $complete_size =  $row_length * $img_data->{$id}->[2];
		my ($y,$x) = (0,0);
		my $index = 0;
		my $row = 0;
		while(1){
			last if $index >= $complete_size;
			#print ">>> $index\n";
			if($index != 0 && $index % $row_length == 0){
				$row++;	
			}
			#my ($r,$g,$b) = ($data[$index++],$data[$index++],$data[$index++]);
			(my $bcolour,$data) = (($data =~ /^(.{3})(.*$)/)); 
			my ($r,$g,$b) = unpack ('CCC',$bcolour);
			#print ">>>>> RGB: $r|$g|$b\n";
			$index += 3;
			if (($r == 165 && $g == 134 && $b == 179)){ # transparent pixel
				next;
			}
			my @point = (($index % $row_length) / 3,$row);
			#print "\$r,\$g,\$b = $r,$g,$b\n";
			$self->{'pen'}->SetColour($r,$g,$b);
			$returnvalue{$id}->SetPen($self->{'pen'});
			$returnvalue{$id}->DrawPoint(@point);
		}
		$returnvalue{$id}->EndDrawing() if $^O =~ /mswin/i;
	}
	return \%returnvalue;
}


1;
__END__
