package SAPDB::Install::Templates2::InstTemplate;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::InstUnit;
use SAPDB::Install::Registry;
use SAPDB::Install::System;

 our @ISA;
 if($SCRIPTING){
   require Exporter;
   push @ISA,qw(Exporter);
 }
 else{
   require SDB::Common::Exporter;
   push @ISA,qw(SDB::Common::Exporter);
 }
push  @ISA, qw( SAPDB::Install::Templates2::Properties);

sub new{
  my $id = shift;
  my $this = { 'KEYS' =>{
                         'packages'  =>
                         {
                          'VALIDATOR' => 0,
                          'PARSER' => \&packageParser,
                          'FORMATTER' =>\&packageFormatter 
                         },
                         'removeList'  =>
                         {
                          'VALIDATOR' => 0,
                          'PARSER' => \&removeListParser,
                          'FORMATTER' =>\&removeListFormatter 
                         },
                         'dBAUser'  =>
                         {
                          'DEFAULT' => 'DBA,DBA',
                          'INITIAL' => 'DBA,DBA'
                         },
                         'dBMUser'  =>
                         {
                          'DEFAULT' => 'DBM,DBM',
                          'INITIAL' => 'DBM,DBM'
                         },
                         'lCAUser'  =>
                         {
                          'DEFAULT' => 'lcaadm,sap'
                         },
                         'lCAEncoding'  =>
                         {
                          'DEFAULT' => 'ascii'
                         },
                         'archiveDir'  => { 'DEFAULT' => '.' ,
                                            'TYPE' => 'PATH'
                                          },
                         'indepData'  => {
                                          'DEFAULT' =>  '${applicationDataPath}\sdb\data',
                                          'TYPE' =>  'PATH'
                                         },
                         'indepProg'  => {
                                          'DEFAULT' =>  '${systemProgPath}\sdb\programs',
                                          'TYPE' =>  'PATH'
                                         },
                         'dependendPath'  => {
                                          'DEFAULT' =>  '${systemProgPath}\sdb\${dBName}',
                                          'TYPE' =>  'PATH'
                                         },
                         'registryFile'  => {
                                             'DEFAULT' =>  '${indepData}\config\install\INSTREG',
                                             'TYPE' =>  'PATH'
                                            },
                         'ownerName'  => {
                                          'DEFAULT' =>  'sdb'
                                         },
                         'ownerGroup'  => {
                                           'DEFAULT' =>  'sdba'
                                          },
                         'systemProgPath'  => {
                                             'TYPE' =>  'PATH'
                                           },
                         'applicationDataPath'  => {
                                             'TYPE' =>  'PATH'
                                           },
                         'dBName'  => {
                                       'DEFAULT' =>  'MAXDB1'
                                      },
                         'dBParams'  => {
                                         'FORMATTER' => \&dbParamsFormatter
                                        } ,
                         'dBVolumes'  => {
                                          'FORMATTER' => \&dbVolumesFormatter
                                         } ,
                         'instance'  => {
                                        },
                         'upgradeFile'  => {
                                            'TYPE' =>  'PATH',
                                            'DEFAULT' =>  'upgrade_${dbname}.config',
                                            'INITIAL' =>  'upgrade_${dbname}.config'
                                           },
                         'logFile'  => {
                                        'TYPE' =>  'PATH'
                                       },
                         'templateID'  => { },
                         'dBInstance'  => { }
                        }};
  bless $this ;
  $this->SUPER::initData();
  $this->setTemplateID($id);
  return  $this;
}


my $xmlDataRaw = << "XMLDATA";
<Template id="templ_desktop" instanceType="SAPDB::Install::Templates2::InstanceDesktop">
 <Wizard  purpose="InstanceCreation" >
   <Frame previous="" id="DBNamePasswdDesktop" next="VolumesMinimal"/>
   <Frame previous="DBNamePasswdDesktop" id="VolumesMinimal" next=""/>
 </Wizard>
</Template>
<Template id="templ_server_advanced" instanceType="SAPDB::Install::Templates2::InstanceCustom">
 <Wizard  purpose="InstanceCreation"  >
   <Frame previous="" id="DBNamePasswdSimple" next="VolumesMinimal"/>
   <Frame previous="DBNamePasswdSimple" id="VolumesAdvanced" next="SizeMemoryAdvanced"/>
   <Frame previous="VolumesAdvanced" id="SizeMemoryAdvanced" next="GlobalDBSettings"/>
   <Frame previous="SizeMemoryAdvanced" id="GlobalDBSettings" next=""/>
 </Wizard>
</Template>
  <FrameDefinitions>
   <Frame id="DBNamePasswdSimple"  >
    <Title>Specify installation prefix and provide database authorization data</Title>
     <ControlSequence>
       <Control parameter="loadSampleSchema"  priority="2" controlType="booleanType" >
          <Label>Load Tutorial Data</Label>
       </Control>
       <Control parameter="adminUserStruct"  priority="3" controlType="adminusersType" >
          <Label>Database Users</Label>
       </Control>
       <Control parameter="dBName"  controlType="stringType" priority="1" >
          <Label>Database Name</Label>
       </Control>
       <Control parameter="softwarePath"  controlType="pathType" priority="0" >
          <Label>Installation Prefix</Label>
       </Control>
     </ControlSequence>
   </Frame>
   <Frame id="VolumesAdvanced"  >
    <Title>Specify locations of database volumes</Title>
     <ControlSequence>
       <Control parameter="volstructLog"  controlType="logVolumesType" priority="2" >
          <Label>Log Volumes</Label>
       </Control>
       <Control parameter="volstructData"  controlType="dataVolumesType" priority="2" >
          <Label>Data Volumes</Label>
       </Control>
     </ControlSequence>
   </Frame>
   <Frame id="SizeMemoryAdvanced"  >
    <Title>Specify database size and assign system resources</Title>
    <ControlSequence >
      <Control parameter="dBSize"  priority="0" Disabled="1" controlType="pagesType">
        <Label>Database Size</Label>
      </Control>
      <Control parameter="dBMemory"  priority="1"  controlType="sliderType" >
        <Label>Memory</Label>
      </Control>
      <Control parameter="cPUCount"  priority="2"  controlType="spinnerType" >
        <Label>CPUs</Label>
      </Control>
      <Control parameter="parallelConnections"  priority="3"  controlType="spinnerType" >
        <Label>Parallel Database Sessions</Label>
      </Control>
    </ControlSequence>
   </Frame>
   <Frame id="GlobalDBSettings"  >
     <Title>Specify global database settings</Title>
      <ControlSequence>
        <Control parameter="autoRestart"  priority="1"  controlType="booleanType">
         <Label>Automatic Start On System Reboot</Label>
        </Control>
        <Control parameter="autoExtend"  priority="2"  controlType="booleanType">
         <Label>Automatic Extension of Data Areas</Label>
        </Control>
        <Control parameter="loggingStruct"  priority="4"  controlType="loggingType">
         <Label>Log Settings</Label>
        </Control>
        <Control parameter="autoUpdateStatistics"  priority="3"  controlType="booleanType">
         <Label>Automatic Statistics Update</Label>
        </Control>
     </ControlSequence>
   </Frame>
   <Frame id="DBNamePasswdDesktop"  >
    <Title>Specify database name and provide authorization data</Title>
    <ControlSequence>
       <Control parameter="dBName"  controlType="stringType" priority="0" Disabled="">
          <Label>Database Name</Label>
       </Control>
       <Control parameter="dBSize"  priority="1" Disabled="0" controlType="pagesType">
          <Label>Database Size</Label>
       </Control>
       <Control parameter="loadSampleSchema"  priority="2" controlType="booleanType" >
          <Label>Load Tutorial Data</Label>
          <Description>Load Tutorial Data</Description>
          <LongDescription>Load Tutorial Data</LongDescription>
       </Control>
       <Control parameter="globalUserStruct"  priority="3" controlType="">
          <Label></Label>
          <Description></Description>
          <LongDescription></LongDescription>
       </Control>
    </ControlSequence>
   </Frame>
   <Frame id="VolumesMinimal"  >
    <Title>Specify disk drives for database software and database areas</Title>
    <ControlSequence>
       <Control parameter="sWDrive"  priority="0"  controlType="driveType">
          <Label>Database Software</Label>
          <Description></Description>
       </Control>
       <Control parameter="volStructSimple"  priority="1" controlType="simpleVolumesType" >
          <Label>Database Areas</Label>
          <Description>Database Areas</Description>
       </Control>
    </ControlSequence>
   </Frame>
</FrameDefinitions>
XMLDATA

my @xmlData = split(/\n/,$xmlDataRaw);


sub nextFrame{
  my ($this,$frame,$wizard) = @_;
  $wizard or $wizard = 'InstanceCreation';
  my $retVal = 0;
  my $templateId = $this->getTemplateID();
  my $frameid ='';
  $frame  and $frameid = $frame->{'FrameId'};
  my @red = filterXMLTag(\@xmlData,'Template', "id=\"$templateId\"");
  @red = filterXMLTag(\@red,'Wizard', "purpose=\"$wizard\"");
  my $nextid;
  $retVal = parseXML(\@red,$this,
           { 'beginFrame'=>
             sub { my  ($this,$atts,$stack) = @_;
                   $atts->{'previous'} eq $frameid and $nextid = $atts->{'id'};}
           });

  $retVal and return Logit("Error getting next frame: ".$retVal,undef);
  $nextid or return undef;
  return $this->createFrame($nextid);
}

sub createFrame{
  my($this,$id) = @_;
  $id or return undef;
  my @red = filterXMLTag(\@xmlData,'FrameDefinitions');
  @red = filterXMLTag(\@red,'Frame',"id=\"$id\"");
  my $frame = { 'FrameId' => $id, 'Data' => {} };
  my $txt = "";
  my $retVal;
  $retVal = parseXML(\@red,$this,
           { 'endTitle'=> sub { $frame->{'Title'}=$txt;$txt='';},
             'endDescription'=> sub { $frame->{'Description'}=$txt;$txt='';},
             'endLongDescription'=> sub { $frame->{'LongDescription'}=$txt;$txt='';},
             'cdata'=> sub { $txt.=$_[1];}
   }) and return Logit("Parse error creating frame: ".$retVal,undef);

  @red = filterXMLTag(\@red,'ControlSequence');
  my @controls;
  $txt='';
  my $control; 
  $retVal = parseXML(\@red,$this,
           { 'beginControl'=> sub {
               my ($this,$atts,$stack) = @_;
               $control = {};
               $control->{'parameter'} = $atts->{'parameter'};
               $control->{'priority'} = $atts->{'priority'};
               $control->{'Disabled'} = $atts->{'Disabled'};
               $control->{'id'} = $atts->{'controlType'};
             },
             'endControl'=> sub { push @controls, ($control); undef $control; },
             'endLabel'=> sub { $control->{'Label'}=$txt;$txt='';},
             'endDescription'=> sub { $control->{'Description'}=$txt;$txt='';},
             'endLongDescription'=> sub { $control->{'LongDescription'}=$txt;$txt='';},
             'cdata'=> sub { $txt.=$_[1];}
           })
    and return Logit("Parse error creating frame: ".$retVal,undef);
  foreach(@controls){
    my $ac = $_;
    my $param = $this->createParameterDescriptionById($ac->{'parameter'});
    $frame->{'Data'}->{$param->{'id'}}=$param;
    $param->{'ControlType'}=$ac;
    $param->{'Description'}=$ac->{'Description'};
    $param->{'LongDescription'}=$ac->{'LongDescription'};
    $param->{'Label'}=$ac->{'Label'};
    $ac->{'dependencies'}=$param->{'dependencies'};
  }
  return $frame;
}

sub createParameterDescriptionById{
  my ($this,$id) = @_;
  my $instance = $this->getDBInstance();
# Debugit(4,"Creating parameter $id for instance $instance");
  $instance or return $this->Logit("Cannot find instance object in InstTemplate::createParameterDescriptionById",undef);
  return $instance->createParameterDescriptionById($id);
}

sub setTemplateID{
  my ($this,$id) = @_;
  $this->Set('templateID',$id);
  $id or $this->setDBInstance(undef);
  $id or return;
  my @red = filterXMLTag(\@xmlData,'Template',"id=\"$id\"");
  my $data = {'PACKAGE_NAME' =>'DBInstance[' . $id.']'};
  my $retVal = parseXML
    (\@red,$this,
     { 'beginTemplate'=> sub {$data->{'CLASS'}=$_[1]->{'instanceType'}}});
  $retVal and  return Logit("Parse error setting template id $id:".$retVal,undef);
  $data->{'CLASS'} or return Logit("Instance type not found for $id.",undef);
  my $instance =  SAPDB::Install::Templates2::InstUnit::new($data,$this);
#  Debugit(4,"Created database instance object $instance");
  $instance or  Logit("Could not get create instance object for Template $id");
  $this->setDBInstance($instance);
}




sub createObject{
  my ($file,$id) =@_;
  my $obj = new($id);
  my $retVal = 0;
  $file and
    $retVal = $obj->parseFile($file)
      and return ("Configuration file '$file' could not be read:\n".$retVal,undef);
  my ($aindep_data,$aindep_prog)=readIndepPath(); 

  my $sysProgPath=getSysProgPath(); 
  my $appDataPath=getAppDataPath(); 
  $obj->getSystemProgPath() or $obj->setSystemProgPath($sysProgPath);
  $obj->getApplicationDataPath() or $obj->setApplicationDataPath($appDataPath);

  
  if (defined $aindep_data and not $aindep_data =~ /^\s*$/) {
    $obj->setIndepData($aindep_data);
  }
  if (defined $aindep_prog and not $aindep_prog =~ /^\s*$/) {
    $obj->setIndepProg($aindep_prog);
  }
  return ($retVal,$obj);
}

sub packageParser{
  my ($this,$key,$value ) = @_;
  my @packages = split/,/,$value;
  return \@packages;
}

sub packageFormatter{
  my ($this,$key,$value ) = @_;
  my $packages = join ',',@$value;
  return (uc $key) . " " . $packages . "\n";
}

sub removeListParser{
  my ($this,$key,$value ) = @_;
  my @retVal;
  if ($value) {
    my @packages = split(/,/,$value);
    foreach (@packages) {
      if (/^(.*)\s*\[(.*)\]$/) {
        my $name = $1;
        my $path = $2;
        $name =~ s/\s+$//;
        $name =~ s/^\s+//;
        $path =~ s/\s+$//;
        $path =~ s/^\s+//;
        push @retVal,([$name,$path]);
      }
    }
  }
  return \@retVal;
}





sub getRemoveList{
  my $this = shift;
  my $l = $this->Get('removeList');
  my @outlist;
  foreach(@$l){
    my $name = $_->[0];
    my $path = $_->[1];
    $path = $this->recursiveSubstitute($path);
    push @outlist, ([$name,$path]);
  }
  return \@outlist;
}

sub removeListFormatter{
  my ($this,$key,$value ) = @_;
  my $retVal = '';
  foreach(@$value){
    $retVal .= ',' . $_->[0] . '[' . $_->[1] . ']';
  }
  $retVal =~ s/^,//;
  $retVal = (uc $key) . " " . $retVal. "\n";
#  print  "retVal: $retVal\n";
  return $retVal;
}


sub dbParamsFormatter{
  my ($this,$key,$value ) = @_;
  my $retVal = '';
  if($value){
    foreach(keys %$value){
      $retVal .=  'DBPARAM ' . $_ . ' ' . $value->{$_}  . "\n";
    }
  }
  return  $retVal;
}

sub dbVolumesFormatter{
  my ($this,$key,$value ) = @_;
  my $retVal = '';
  if($value){
    foreach(@$value){
      $retVal .=  'DBVOLUME '. $_ . "\n";
    }
  }
  return  $retVal;
}

sub getDBVolumes{
  my $this = shift;
  my $l = $this->Get('dBVolumes');
  my @outlist;
  foreach(@$l){
    my $val = $this->recursiveSubstitute($_);
    push @outlist, ($val);
  }
  return \@outlist;
}
sub parseLine{
  my ($this,$aLine) = @_;
  if ( $aLine =~ /^\s*DBPARAM\s+(.*)$/) {
    my $rest = $1;
    if ( $rest =~ m/^\s*([\w\d]+)\s+(.+)\s*$/) {
      my $val = $2;
      my $key = $1;
      my $dbps = $this->getDBParams();
      $dbps = $dbps ?  $dbps : {};
      $dbps->{$key}=$val;
      $this->setDBParams($dbps);
    }
  } elsif ( $aLine =~ /^\s*DBVOLUME\s+(.*)$/) {
    my $vol = $1;
    my $vols = $this->Get('dBVolumes');
    $vols = $vols ? $vols : [];
    push @$vols,($vol);
    $this->setDBVolumes($vols);
  } else {
    $this->SUPER::parseLine($aLine);
  }
}

package SAPDB::Install::Templates2::TemplateList;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;

my ($templateList,$templateSection,$template,$description);
sub beginTemplateList{
  my  ($atts,$stack) = @_;
  $templateList = [];
}
sub endTemplateList{
  my  ($name,$stack) = @_;
}
sub beginTemplateSection{
  my  ($atts,$stack) = @_;
  $templateSection = {'label' => $atts->{'label'},'data'=>[]};
}
sub endTemplateSection{
  push @$templateList,($templateSection);
  undef $templateSection;
}
sub beginTemplate{
  my  ($atts,$stack) = @_;
  $template = {'TemplateCode'=>$atts->{'id'},
               'ShortDescription'=>$atts->{'label'}
              };
}
sub endTemplate{
  push @{$templateSection->{'data'}},($template);
  undef $template;
}
sub beginDescription{
  $description='';
}
sub endDescription{
  $template->{'LongDescription'}=$description;
  undef $description;
}
sub cdata{
  my ($line,$stack) = @_;
  defined $description or return;
  chomp $line;
  $line =~ s/^\s+//;
  $line =~ s/\s+$/ /;
  $description .= $line;
}



my $listData = << "LISTDATA";
<TemplateList>
  <TemplateSection label="General">
    <Template id="templ_desktop" label="Desktop PC / Laptop" >
      <Description>
       Typical installation for a single user environment. 
       Can be used during development of simple database applications, for 
       the MaxDB tutorial or for testing purposes.
      </Description>
    </Template>
  </TemplateSection>
  <TemplateSection label="Custom">
    <Template id="templ_server_advanced" label="Custom" >
      <Description>
       Customizable installation of a databases instance. 
       Default values for memory usage, cpu usage and other parameters are calculated and can be adapted.
       Any number of data volumes on multiple hard disks can be specified. 
      </Description>
    </Template>
  </TemplateSection>
</TemplateList>
LISTDATA


sub createTemplateList{
  my @lines = split(/\n/,$listData);
  my $retVal = parseXML(\@lines);
  bless $templateList;
  return ($retVal,$templateList);
}


sub descById{
  my ($this,$id) = @_;
  foreach(@$this){
    my $ts = $_->{'data'};
    foreach (@$ts) {
      $_->{'TemplateCode'} eq $id and return $_;
    }
  }
}



1;
__DATA__
<TemplateList>
  <TemplateSection label="General">
    <Template id="templ_desktop" label="Desktop PC / Laptop" >
      <Description>
       Typical installation for a single user environment. 
       Can be used during development of simple database applications, for 
       the MaxDB tutorial or for testing purposes.
      </Description>
    </Template>
  </TemplateSection>
  <TemplateSection label="Custom">
    <Template id="templ_server_advanced" label="Custom" >
      <Description>
       Customizable installation of a databases instance. 
       Default values for memory usage, cpu usage and other parameters are calculated and can be adapted.
       Any number of data volumes on multiple hard disks can be specified. 
      </Description>
    </Template>
  </TemplateSection>
</TemplateList>



# current element name from stack:  $stack->[$#{@{$stack}}][0];

# Local Variables: ***
# eval:  (setq perl-dbg-flags (format "-w -Ic:\\SapDevelop\\aw\\sys\\wrk\\script -I%s\\sys\\src\\install\\perl " (sdb-get-current-own))) ***
# End: ***
