#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/SAPDB/Install/Window/DBDrop.pm#1 $
# $DateTime: 2005/03/21 16:21:00 $
# $Change: 93297 $
#
# Desc: Installation dialog - extract files and do register calls
#

package SAPDB::Install::Window::DBDrop;
use Wx qw (
	wxEXPAND wxTOP wxALIGN_CENTER wxALL wxHORIZONTAL
	wxLEFT wxVERTICAL wxSUNKEN_BORDER wxTE_PASSWORD
	wxALIGN_CENTER_VERTICAL wxALIGN_RIGHT wxBOTTOM
	wxICON_ERROR wxCENTRE wxRIGHT);

use Wx::Event qw (EVT_BUTTON EVT_TEXT);

our @ISA = qw(Wx::Panel);

use SAPDB::Install::DBMCmd;
use SAPDB::Install::Registry;
use SAPDB::Install::Tools;
use SAPDB::Install::System;
use SAPDB::Install::Resources;

our $img_keys = SAPDB::Install::Resources->new('RESOURCES.TGZ')->
						getImages('icon/keys.png')->{'icon/keys.png'};

our ($indep_data,$indep_prog) = readIndepPath();

#
#	$my_args = {
#				'dbname' => <database name>,
#				'log' => <a object of type SAPDB::Install::Log or with similar interface>,
#			};
#



sub new{
	my ($class,$my_args,@args) = @_;
	my $self = $class->SUPER::new(@args);
	$self->{'dbname'} = $my_args->{'dbname'};
	$self->{'log'} = $my_args->{'log'};
	my $bitmap = Wx::StaticBitmap->new($self,-1,Wx::Bitmap->new($img_keys));
	my $topsizer = Wx::BoxSizer->new(wxVERTICAL);
	my $static_box = Wx::StaticBox->new($self,-1,'Database System Administrator Login');
	my $boxsizer = Wx::StaticBoxSizer->new($static_box,wxHORIZONTAL);
	my $edit_sizer = Wx::GridSizer->new(2,2);
	$edit_sizer->SetVGap(10);
	$edit_sizer->SetHGap(10);
	my $label_dbm = Wx::StaticText->new($self,-1,'User Name',[0,0],[-1,-1]);
	my $label_dbm_passwd = Wx::StaticText->new($self,-1,'Password',[0,0],[-1,-1]);
	$self->{'edit_dbm'} = Wx::TextCtrl->new($self,-1,'',[0,0],[70,-1]);
	$self->{'edit_dbm_pass'} = Wx::TextCtrl->new($self,-1,'',[0,0],[70,-1],wxTE_PASSWORD);
	$self->{'button_drop'} = Wx::Button->new($self,-1,'&Drop Database');
	$self->{'button_cancel'} = Wx::Button->new($self,-1,'&Cancel');
	EVT_BUTTON($self,$self->{'button_cancel'}->GetId,\&OnCancel);
	EVT_BUTTON($self,$self->{'button_drop'}->GetId,\&OnDrop);
	EVT_TEXT($self,$self->{'edit_dbm'}->GetId,\&OnTextChange);
	EVT_TEXT($self,$self->{'edit_dbm_pass'}->GetId,\&OnTextChange);
	$self->{'button_drop'}->Enable(0);
	my $button_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$edit_sizer->Add($label_dbm,0,wxALIGN_CENTER_VERTICAL | wxLEFT , 10);
	$edit_sizer->Add($self->{'edit_dbm'},0,wxALIGN_RIGHT);
	$button_sizer->Add($self->{'button_drop'},0);
	$button_sizer->Add($self->{'button_cancel'},0,wxLEFT,20);
	$edit_sizer->Add($label_dbm_passwd,0,wxALIGN_CENTER_VERTICAL | wxLEFT , 10);
	$edit_sizer->Add($self->{'edit_dbm_pass'},0,wxALIGN_RIGHT);
	$edit_sizer->CalcMin;
	$boxsizer->Add($bitmap,0);
	$boxsizer->Add($edit_sizer,0, wxTOP | wxRIGHT | wxBOTTOM , 20);
	$boxsizer->CalcMin;
	$topsizer->Add($boxsizer,1,wxEXPAND | wxALL, 10 );
	$topsizer->Add($button_sizer,0,((wxALL) & ~(wxTOP)) | wxALIGN_RIGHT, 10);	
	$self->Fit;
	$self->SetSizer($topsizer);
	$self->SetAutoLayout(1);
	$self->Layout();
	$self->Fit;
	return $self;
}

sub OnTextChange{
	my ($self,$evt) = @_;
	my $text = $self->FindWindow($evt->GetId)->GetValue;
	if($text =~ /\S/){
		if ($self->{'edit_dbm'}->GetValue =~ /\S/ and 
			$self->{'edit_dbm_pass'}->GetValue =~ /\S/){
			$self->{'button_drop'}->Enable(1);
		}
	
	}
	else{
		$self->{'button_drop'}->Enable(0);
	}
	
	$evt->Skip(1);
}


sub OnCancel{
	my ($self,$evt) = @_;
	$self->GetParent->EndModal(0);
	#$self->Close(1);
	
}


sub OnDrop{
	my ($self,$evt) = @_;
	unless($self->drop()){
		my $msgbox = Wx::MessageDialog->new($self,$self->{'last_err'},
					'Error', wxICON_ERROR | wxCENTRE); 
		$msgbox->ShowModal;
		$msgbox->Destroy;
		return undef;
	}
	
}



sub getstate{
	my ($self) = @_;
	my $prog  = "$indep_prog/bin/x_cons";
	if( $^O =~ /mswin/i){
		$prog .= '.exe';
		$prog =~ s/\//\\/g;
	}
	
	unless(-f $prog){
		return undef;
	}

	my $cmd  = '"'.$prog.'" '.$self->{'dbname'}.' state';
	 
	my $db_state;
	callsystem($cmd,1,sub{
		my ($text) = @_;
		foreach my $line (split("\n",$text)){
			if($line =~ /state/i){
				($db_state) = ($line =~ /\s(\S+)\s*$/);				
				last;
			}
		}
		return 1;	
	});
	unless($db_state =~ /\S/){
		return undef;
	}
	
	return $db_state;		
	
}

sub connect{
	my ($self) = @_;
	
	my $info = xserverInfo();
	unless($info->{'is_running'}){
		
		if(-f "$indep_prog/bin/x_server"){
			callsystem("$indep_prog/bin/x_server start");		
		}
		elsif(-f "$indep_prog/bin/x_server.exe"){
			callsystem("\"$indep_prog/bin/x_server.exe\" start" );
		}
		else{
			$self->{'last_err'} = 'Communication Error - cannot find x_server executable';
			return undef;
		}
		$info = xserverInfo();
		unless($info->{'is_running'}){
			$self->{'last_err'}  = 'Communication Error - cannot start communication server';
			return undef;			
		}
	}
	
	$self->{'dbm'} = SAPDB::Install::DBMCmd->new('',$self->{'dbname'});

	unless(defined $self->{'dbm'}){
			$self->{'last_err'} = 'Connect failed - cannot connet to '.$db.'@localhost';
			return undef;
	}
	
	my $value = $self->{'edit_dbm'}->GetValue().
				','.$self->{'edit_dbm_pass'}->GetValue();
	
	my $rc = $self->{'dbm'}->user_logon($self->{'edit_dbm'}->GetValue().
				','.$self->{'edit_dbm_pass'}->GetValue());		
	
	unless(defined $rc){
		$self->{'last_err'} = 'user logon failed: '."\n".
							  $self->{'dbm'}->lastmsg;
		return undef;
	}	
	$self->{'connected'} = 1;
	
	$self->{'log'}->SetMsg("MSG: connection to database ".$self->{'dbname'}." successfully established\n");
	
	my $state = $self->{'dbm'}->db_state;
	unless(defined $state){
		$self->{'last_err'} = 'cannot get database state: '."\n".
							  $self->{'dbm'}->lastmsg;
		return undef;
	}
	$state =~ tr/[A-Z]/[a-z]/;
	$self->{'log'}->SetMsg("MSG: database ".$self->{'dbname'}." is $state\n");	
	
	if($state =~ /offline/i){
		$self->{'stopped'} = 1;
	}

	return 1;
}


sub stop{
	my ($self) = @_;
	unless($self->{'connected'}){
		unless($self->connect()){
			return undef;
		}	
	}
	my $rc = $self->{'dbm'}->db_offline;
	unless(defined $rc){
		$self->{'last_err'} = 'cannot stop database'."\n".
								$self->{'dbm'}->lastmsg;		
		return undef;
	}
	$self->{'log'}->SetMsg("MSG: database ".$self->{'dbname'}." stopped\n");
	$self->{'stopped'} = 1;
	return 1;
				
}


sub drop{
	my ($self) = @_;
	
	unless($self->{'connected'}){
		unless($self->connect()){
			return undef;
		}	
	}
	unless($self->{'stopped'}){
		unless($self->stop()){
			return undef;
		}
	}
	unless($self->{'dbm'}->db_drop){
		$self->{'last_err'} = 'drop database failed'."\n".
								$self->{'dbm'}->lastmsg;					
		return undef;
	}
	$self->{'dropped'} = 1;
	$self->{'log'}->SetMsg("MSG: database ".$self->{'dbname'}." dropped\n");
	$self->GetParent->EndModal(1);				
	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	
	if(defined $self->{'edit_dbm'}){
		$self->{'edit_dbm'}->Destroy;
		delete  $self->{'edit_dbm'};
	}
	if(defined $self->{'edit_dbm_pass'}){
		$self->{'edit_dbm_pass'}->Destroy;
		delete  $self->{'edit_dbm_pass'};
	}
	if(defined $self->{'button_drop'}){
		$self->{'button_drop'}->Destroy;
		delete $self->{'button_drop'};
	}
	if(defined $self->{'button_cancel'}){
		$self->{'button_cancel'}->Destroy;
		delete $self->{'button_cancel'};
	}
	$self->SUPER::Destroy;
}


sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;
