package SAPDB::Install::Window::ItemList;

use Wx::Event qw(EVT_GRID_SELECT_CELL EVT_GRID_EDITOR_HIDDEN EVT_GRID_EDITOR_SHOWN);
use Wx qw(wxEXPAND wxLIGHT_GREY wxVERTICAL
		  wxSUNKEN_BORDER wxEVT_GRID_CELL_LEFT_DCLICK
		  wxLEFT wxCENTRE wxWHITE wxTAB_TRAVERSAL);
use Wx::Grid;


our @ISA = qw(Wx::Panel);

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->SetWindowStyleFlag($this->GetWindowStyleFlag | wxTAB_TRAVERSAL);
	$this->{'grid'} = Wx::Grid->new($this,-1);
	$this->{'grid'}->EnableCellEditControl(1);
	$this->{'grid'}->EnableGridLines(1);
	$this->{'grid'}->SetGridLineColour(wxWHITE);
	$this->{'grid'}->SetColLabelSize(20);
	$this->{'grid'}->SetColLabelAlignment(wxLEFT,wxCENTRE);
	$this->{'grid'}->SetRowLabelSize(0);
	$this->{'grid'}->SetCellHighlightROPenWidth(0);
	$this->{'grid'}->EnableEditing(0);
	$this->{'grid'}->SetCellHighlightPenWidth(0);
	$this->{'grid'}->DisableDragGridSize;
	$this->{'bright_colour'} = Wx::Colour->new(240,240,240);
	$this->{'dark_colour'} = Wx::Colour->new(200,200,200);
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	$this->SetAutoLayout(1);
	$this->SetSizer($sizer);
	return $this;
}

sub create{
	my ($self,$items) = @_;
	return undef unless ref($items) eq 'ARRAY';
	my @cols = @{ref($self).'::cols'};
	my @items = @$items;
	$self->{'grid'}->CreateGrid($#items + 1,$#cols + 1);
	foreach my $col (0..$#cols){
		my $row = 0;
		$self->{'grid'}->SetColLabelValue($col,$cols[$col]->{'label'});
		foreach my $item (@items){
			if($row % 2){
				$self->{'grid'}->SetCellBackgroundColour($row,$col,$self->{'dark_colour'});
			}
			else{
				$self->{'grid'}->SetCellBackgroundColour($row,$col,$self->{'bright_colour'});
			}
			if($cols[$col]->{'alignment'}){
				$self->{'grid'}->SetCellAlignment($row,$col,$cols[$col]->{'alignment'},-1);
			}
			$self->{'grid'}->SetCellValue($row++,$col,$item->{$cols[$col]->{'key'}});
		}		
	}
	$self->{'grid'}->Fit;
	$self->GetSizer->Add($self->{'grid'},1,wxEXPAND);
	$self->Layout;
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'grid'}){
		$self->{'grid'}->Destroy;
		delete $self->{'grid'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}

1;