package SAPDB::Install::Window::Parameter;

use Wx::Event qw(EVT_BUTTON);

use Wx qw(wxALIGN_LEFT wxEXPAND 
		wxHORIZONTAL wxRIGHT wxVERTICAL
		wxDD_NEW_DIR_BUTTON wxBOTTOM);

use SAPDB::Install::Control::StaticText;
use SAPDB::Install::System;

our @ISA = qw(Wx::Panel);

our $path_desc = {
	'Independent Data Path' => 'Path for storing the data, configurations, and run directories of database instances and database applications.',
	'Independent Program Path' => 'Path for storing the programs and libraries shared by the database instances and database applications.',
	'Dependent Path' => 'Path for storing the server software that depends on the database version. This path must be unique. Multiple directories with different versions can exist alongside each other.'
};

our $label_width = 140;



sub new{
	my($this ) = shift->SUPER::new(@_);
	return $this;
}

sub init{
	my ($self,$params) = @_;
	if($self->{'initialized'}){
		$self->killParms;
	}
	my @params = keys(%$params);
	
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	$self->{'params'} = $params;
	foreach my $param (sort {$params->{$a}->{'seq'} <=>  $params->{$b}->{'seq'}} keys(%$params)){
		my $sizer_line = Wx::BoxSizer->new(wxHORIZONTAL);
		if($params->{$param}->{'type'} eq 'path'){
			my $text = 	Wx::StaticText->new($self,-1,$params->{$param}->{'label'},[0,0],[$label_width,-1]);
			if(exists $path_desc->{$params->{$param}->{'label'}}){
				my $desc_text = SAPDB::Install::Control::StaticText->new($self,-1,[0,0],[500,20]);
				$desc_text->{'auto_split'} = 1;
				$desc_text->{'auto_resize'} = 0;
				$desc_text->SetLabel($path_desc->{$params->{$param}->{'label'}});
				$sizer->Add($desc_text, 0,wxEXPAND | wxBOTTOM,5);
			} 
			my $path = $params->{$param}->{'default'};
			$path =~ s/\//\\/g if $^O =~ /mswin/i;
			my $edit = Wx::TextCtrl->new($self,-1,$path,[0,0],[270,-1]);
			my $button = Wx::Button->new($self,-1,'&Browse...',[0,0],[-1,-1]);
			
			EVT_BUTTON($self,$button->GetId,
				sub {
					my ($self,$evt) = @_;
					my $dirdlg = Wx::DirDialog->new($self,'Choose a directory',$edit->GetValue,wxDD_NEW_DIR_BUTTON);
					my $dir = $edit->GetValue,
					$dirdlg->SetPath($edit->GetValue);
					$dirdlg->ShowModal;
					my $result = $dirdlg->GetPath;
					$edit->SetValue($result) if $result =~ /\S/;
				}
			);
			$params->{$param}->{'getvalue'} = 
				sub {
					my $path = $edit->GetValue;
					normalizePath(\$path); 
					return $path;
				};
			$sizer_line->Add($text, 0, wxALIGN_LEFT);
			$sizer_line->Add($edit,0,wxRIGHT, 10);
			$sizer_line->Add($button,0);
		}
		elsif($params->{$param}->{'type'} eq 'list'){
			my $rbox = Wx::RadioBox->new($self,-1,)
		}	
		elsif($params->{$param}->{'type'} eq 'string'){
			my $label = Wx::StaticText->new($self,-1,$params->{$param}->{'label'},[0,0],[$label_width,-1]);
			$sizer_line->Add($label,1);
			my $ctrl = Wx::TextCtrl->new($self,-1,'');
			$params->{$param}->{'getvalue'} = sub {return $ctrl->GetValue};
			$ctrl->SetValue($params->{$param}->{'default'});
			$sizer_line->Add($ctrl,1);
			 
		}
		else{
			$sizer_line = Wx::StaticText->new($self,-1,"$param has unkown type");
		}
		$sizer->Add($sizer_line,0, wxBOTTOM, 5);
	}
	$self->SetSizer($sizer);
	$self->SetAutoLayout(1);
	$self->{initialized} = 1;
	$self->Layout;
};

sub killParams{
	my ($self) = @_;
	foreach my $param (keys(%$params)){
		
	}
	$self->{initialized} = undef;
	
}

sub SetValues{
	my ($self) = @_;
	return unless $self->{'initialized'};
	foreach my $param (keys(%{$self->{'params'}})){
		$self->{'params'}->{$param}->{'value'} = &{$self->{'params'}->{$param}->{'getvalue'}};
	}
	return $self->{'params'};
};

sub Destroy{
	my ($self) = @_;
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;