#!/usr/bin/perl
#
# $Header: //sapdb/V76/develop/sys/src/install/perl/SAPDB/Install/Dialog/Prepare.pm#1 $
# $DateTime: 2004/03/11 10:29:37 $
# $Change: 66919 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade,
#       called by SDBINST/SDBUPD

package SAPDB::Install::Wizard::Dialog::CreateInstance;

use Wx qw (
	wxEXPAND wxALL wxTE_MULTILINE wxTE_READONLY
	wxALIGN_CENTER_HORIZONTAL wxTOP wxVERTICAL
	wxALIGN_LEFT wxLEFT);
	
use SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::StdIO;
use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Templates::Setup;
use SAPDB::Install::Window::ExecPlan;

our @ISA = qw (SAPDB::Install::Wizard::Dialog::Base);

our $is_wxx11 = Wx::wxX11;

sub new{
	my($this ) = shift->SUPER::new(@_);
	return $this;
}



sub init{
	my ($self,$config) = @_;
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	#$self->{'caption'} = SAPDB::Install::Control::StaticText->new($self,-1,[0,0],[350,30]);
	my $text = "Creating database instance ".$config->{'container'}->Template->getDBInstance()->getDBName();
	$self->{'caption'} = Wx::StaticText->new($self,-1,$text,[0,0],[350,15]);
	#my ($width,$height) = $self->{'caption'}->getTextExtend($text);
	#if($^O =~ /mswin/i){
	#	$self->{'caption'}->SetSize([$width,$height]);	
	#}
	#else{
	#	$self->{'caption'}->SetSize([350,30]);
	#}
	#$self->{'caption'}->setText($text);
	$sizer->Add($self->{'caption'},0,wxALIGN_LEFT | wxLEFT | wxTOP , 20);
	$self->{'status'} = SAPDB::Install::Window::ExecPlan->new($self,-1);
	$sizer->Add($self->{'status'},1,wxEXPAND | wxALL, 20);
	#$self->{'progress_text'} = Wx::TextCtrl->new($self,-1,'',[0,10],[180,200],wxTE_MULTILINE | wxTE_READONLY);
	#$sizer->Add($self->{'progress_text'},1,wxEXPAND | wxALL, 20);
	$self->SetSizer($sizer);
	$self->SetAutoLayout(1);
	$self->Layout;
	&{$SAPDB::Install::Values::RefreshGui}();
	return {'auto_next' => 1};
}

sub Process{
	my ($self,$config) = @_;
	
	$self->{'status'}->init($config->{'container'}->Template->
							getDBInstance->getStatusList);
	
	my $step_id = 0;
	
	$config->{'container'}->Template->setStatusListCallback(
		sub{
			my ($statusList) =  @_;
			if($step_id == 0 and $statusList->[$step_id]->{'Status'} eq 'RUNNING'){
				$self->{'status'}->Next($statusList->[$step_id]->{'Status'});
			}
			elsif(($statusList->[$step_id]->{'Status'} ne 'RUNNING') and
			   $statusList->[$step_id]->{'Status'} ne 'PENDING'){
				$self->{'status'}->Next($statusList->[$step_id++]->{'Status'});
			}
			&{$SAPDB::Install::Values::RefreshGui}();
		}
	);

	$config->{'container'}->Template->setStatusCallback(
		sub { 
			my ($text) = @_;
			#$self->{'progress_text'}->AppendText($text);
			&{$SAPDB::Install::Values::RefreshGui}();
		} 
	);
	
	$config->{'container'}->Template->setAsyncCallback(
		sub { 
			my ($elapsed,$text) = @_;
			&{$SAPDB::Install::Values::RefreshGui}();
		} 
	);


	my ($rc,$short_text,$long_text);
	
	eval{
		($rc,$short_text,$long_text) = $config->{'container'}->Template->configure();
	};
	
	die($@) if($@ =~ /^SDB_closing/);
	
	if($@){
		$self->SetError("Runtime error",$@);
		return {'errstate' => 1};
	}
	
	unless($rc){
		$self->SetError("Error checking parameters",$short_text.":\n".$long_text);
		return {'errstate' => 1};
	}	

	($rc,$short_text,$long_text) = $config->{'container'}->Template->install;
	
	if($rc == 0){
		return {'errstate' => 1,'errmsg' => $short_text,'errdetail' => $long_text};

	}

	return {};
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}		
	}
	
	if(defined $self->{'caption'}){
		$self->{'caption'}->Destroy;
		delete $self->{'caption'};
	}
	if(defined $self->{'status'}){
		$self->{'status'}->Destroy;
		delete $self->{'status'};
	}
	$self->SUPER::Destroy;

}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
