#!/usr/bin/perl
#
# $SRC/install/perl/Makefile.PL
#
# $Header$
# $DateTime$
# $Change$
#

use Config;
use Symbol;

sub main {
	print 
	"Generating Makefile for ".
	"SDBINST, SDBUPD, sdbregview, sdbuninst, sdbverify, sdbpack\n";

	my %Makedefs = (
	'PERL_RM' => get_rm (),
	'PERL_CP' => get_cp (),
	'PERL_STRIP' => get_strip (),
	'PERL_INST_PREFIX' => get_inst_prefix (),
	'PERL_CCFLAGS' => $Config{'ccflags'}." ".$Config{'cccdlflags'},
	'PERL_CFLAGS' => "\$(PERL_CCFLAGS)",
	'PERL_LD' => $^O =~ /^mswin/i ? "link" : "\$(CC)",
	'PERL_LD_FLAGS' => get_ld_flags (),
	'PERL_OBJEXT' => $Config{'obj_ext'},
	'PERL_EXEEXT' => $Config{'exe_ext'},
	'PERL_SDBInst_RES' => ($^O =~ /mswin/i ? 'SDBInst.res' : ''),
	'PERL_LIBOVERFLOW' => get_liboverflow (),
	'PERL_GEN_MAPLIBNAME' => ($^O !~ /mswin/i ?  get_genmaplibname () : ''),
	'PERL_FINDLIBS' => ($^O !~ /mswin/i ?  get_findlibs () : ''),
	'PERL_EXEC_MAPLIBNAME' => ($^O !~ /mswin/i ?  get_execmaplibname () : '')
	);
	
	write_makefile (%Makedefs);
}

sub write_makefile {
	my %Makedefs = @_;
	my $fh = gensym ();
	open ($fh, "> Makefile");
	print ($fh
	"#\n".
	"# Makefile\n".
	"# this file is generated from Makefile.PL\n".
	"# do not edit this file, any changes will be lost\n".
	"#\n".
	"\n".	
	".SUFFIXES:\n".
	".SUFFIXES: .o .c\n".
	"\n");

	if ($^O =~ /^aix/) {
		print ($fh
		"#\n".
		"# make sure that we are in 32 bit mode on aix\n".
		"#\n".
		"OBJECT_MODE = 32\n".
		"\n");
	}

	while (my ($key, $value) = each %Makedefs) {
		next unless (defined $value);
		print ($fh $key." = ".$value."\n");
	}
	print $fh ("\n");

	write_body ($fh);
	close ($fh);
}

sub write_body {
	my $fh = shift;
	while (<DATA>) {
		chomp;
		if ($^O =~ /^mswin/i) {
			if ($_ =~ /-o \w+\$\(EXEEXT\)/) {
				$_ =~ s/-o\s+/-out:/;
			}
		} 
		print ($fh $_."\n");
	}
}

sub get_strip {
	$^O =~ /^mswin/i ? "touch" : "strip";
}

sub get_rm {
	$^O =~ /^mswin/i ? "del" : "-rm";
}

sub get_cp {
	$^O =~ /^mswin/i ? "copy" : "cp";
}

sub get_inst_prefix {
	$^O =~ /^mswin/i ? $ENV{'OWN'}."\\sys\\wrk\\fast\\install" :
	                   $ENV{'OWN'}."/sys/wrk/fast/install";
}

sub get_ld_flags{
	$^O =~ /mswin/i && $Config{'archname'} =~ /AMD64/ ?
		'/ENTRY:mainCRTStartup' :
	$^O =~ /mswin/i ?
		'/SUBSYSTEM:WINDOWS /ENTRY:mainCRTStartup' :
		'';
}

sub get_liboverflow {
	$^O =~ /mswin/i && $ENV{'HAS_MSC_SECURITY_CHECKS'} ? 'bufferoverflowu.lib' : '';
}

sub get_genmaplibname{
	my $tool =$ENV{'WRK'}.'/fast/install/tmp/tool/genmaplibname';
	$tool =~ s/\//\\/g if $^O =~ /mswin/i;
	return $tool;
}

sub get_findlibs{
	my $tool =$ENV{'WRK'}.'/fast/install/tmp/tool/findlibs';
	$tool =~ s/\//\\/g if $^O =~ /mswin/i;
	return $tool;
}

sub get_execmaplibname{
	return 'perl '.get_genmaplibname();
}



main (@ARGV);

__DATA__
RC = rc
RM = $(PERL_RM)
LD = $(PERL_LD)
LD_FLAGS = $(PERL_LD_FLAGS)
CP = $(PERL_CP)
STRIP = $(PERL_STRIP)
INST_PREFIX = $(PERL_INST_PREFIX)
CFLAGS = $(PERL_CFLAGS)
OBJEXT = $(PERL_OBJEXT)
EXEEXT = $(PERL_EXEEXT)
LIBOVERFLOW = $(PERL_LIBOVERFLOW)
SDBInst_RES = $(PERL_SDBInst_RES)
GEN_MAPLIBNAME = $(PERL_GEN_MAPLIBNAME)
FINDLIBS = $(PERL_FINDLIBS)
EXEC_MAPLIBNAME = $(PERL_EXEC_MAPLIBNAME)

.c$(OBJEXT):
	$(CC) -c $(CFLAGS) $<

SDBRun_OBJ = SDBRun$(OBJEXT)
SDBInst_OBJ = SDBInst$(OBJEXT)
SDBSetup_OBJ = SDBSetup$(OBJEXT)
SDBUpd_OBJ = SDBUpd$(OBJEXT)
SDBUninst_OBJ = SDBUninst$(OBJEXT)
SDBRegView_OBJ = SDBRegView$(OBJEXT)
SDBVerify_OBJ = SDBVerify$(OBJEXT)
SDBPack_OBJ = SDBPack$(OBJEXT)

ALL_HELPERS = SDBINST$(EXEEXT) SDBSETUP$(EXEEXT) SDBUPD$(EXEEXT) sdbuninst$(EXEEXT) sdbregview$(EXEEXT) sdbverify$(EXEEXT) sdbpack$(EXEEXT)


ALL_HELPERS_OBJ = $(SDBUpd_OBJ) $(SDBInst_OBJ) $(SDBSetup_OBJ) $(SDBInst_RES) $(SDBUninst_OBJ) $(SDBRegView_OBJ) $(SDBVerify_OBJ) $(SDBPack_OBJ) helpers$(OBJEXT)

all: $(ALL_HELPERS)

SDBInst.res: setup.ico SDBInst.rc
	$(RC) SDBInst.rc

helpers$(OBJEXT): helpers.c helpers.h maplibname.h 
	$(CC) -c $(CFLAGS) helpers.c

maplibname.h: $(GEN_MAPLIBNAME) $(FINDLIBS)
	$(EXEC_MAPLIBNAME)

SDBUpd$(OBJEXT): SDBUpd.c helpers.h
	$(CC) -c $(CFLAGS) SDBUpd.c

SDBUPD$(EXEEXT): $(SDBUpd_OBJ) helpers$(OBJEXT)
	$(LD) -o SDBUPD$(EXEEXT) $(SDBUpd_OBJ) helpers$(OBJEXT) $(LIBOVERFLOW)
	$(STRIP) SDBUPD$(EXEEXT)

SDBInst$(OBJEXT): SDBInst.c helpers.h
	$(CC) -c $(CFLAGS) SDBInst.c

SDBSetup$(OBJEXT): SDBSetup.c helpers.h
	$(CC) -c $(CFLAGS) SDBSetup.c

SDBINST$(EXEEXT): $(SDBInst_OBJ) helpers$(OBJEXT)
	$(LD) $(LD_FLAGS) -o SDBINST$(EXEEXT) $(SDBInst_OBJ) helpers$(OBJEXT) $(LIBOVERFLOW)
	$(STRIP) SDBINST$(EXEEXT)

SDBSETUP$(EXEEXT): $(SDBSetup_OBJ) helpers$(OBJEXT) $(SDBInst_RES)
	$(LD) $(LD_FLAGS) -o SDBSETUP$(EXEEXT) $(SDBSetup_OBJ) helpers$(OBJEXT) $(LIBOVERFLOW) $(SDBInst_RES)
	$(STRIP) SDBSETUP$(EXEEXT)

SDBUninst$(OBJEXT): SDBUninst.c helpers.h
	$(CC) -c $(CFLAGS) SDBUninst.c

sdbuninst$(EXEEXT): $(SDBUninst_OBJ) helpers$(OBJEXT)
	$(LD) $(LD_FLAGS) -o sdbuninst$(EXEEXT) $(SDBUninst_OBJ) helpers$(OBJEXT) $(LIBOVERFLOW)
	$(STRIP) sdbuninst$(EXEEXT)

SDBRegView$(OBJEXT): SDBRegView.c helpers.h
	$(CC) -c $(CFLAGS) SDBRegView.c

SDBVerify$(OBJEXT): SDBVerify.c helpers.h
	$(CC) -c $(CFLAGS) SDBVerify.c

SDBPack$(OBJEXT): SDBPack.c helpers.h
	$(CC) -c $(CFLAGS) SDBPack.c

sdbregview$(EXEEXT): $(SDBRegView_OBJ) helpers$(OBJEXT)
	$(LD) -o sdbregview$(EXEEXT) $(SDBRegView_OBJ) helpers$(OBJEXT) $(LIBOVERFLOW)
	$(STRIP) sdbregview$(EXEEXT)

sdbverify$(EXEEXT): $(SDBVerify_OBJ) helpers$(OBJEXT)
	$(LD) -o sdbverify$(EXEEXT) $(SDBVerify_OBJ) helpers$(OBJEXT) $(LIBOVERFLOW)
	$(STRIP) sdbverify$(EXEEXT)

sdbpack$(EXEEXT): $(SDBPack_OBJ) helpers$(OBJEXT)
	$(LD) -o sdbpack$(EXEEXT) $(SDBPack_OBJ) helpers$(OBJEXT) $(LIBOVERFLOW)
	$(STRIP) sdbpack$(EXEEXT)

install: all
	$(CP) SDBINST$(EXEEXT) $(INST_PREFIX)
	$(CP) SDBSETUP$(EXEEXT) $(INST_PREFIX)
	$(CP) SDBUPD$(EXEEXT) $(INST_PREFIX)
	$(CP) sdbuninst$(EXEEXT) $(INST_PREFIX)
	$(CP) sdbregview$(EXEEXT) $(INST_PREFIX)
	$(CP) sdbverify$(EXEEXT) $(INST_PREFIX)
	$(CP) sdbpack$(EXEEXT) $(INST_PREFIX)

clean:
	$(RM) $(ALL_HELPERS) $(ALL_HELPERS_OBJ)
	$(RM) Makefile
