/*
 * helpers.c
 *
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/helpers/helpers.c#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 *
 */

static char szWhatHeaderSDBInstHelper[] =
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/helpers/helpers.c#1 $";

/*
 * general perl preprocessor code
 */
#ifndef WIN32
#define UNIX
#endif

#include "helpers.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/*
 * preprocessor code for getcwd () and system calls
 */
#ifdef UNIX
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#endif

#ifdef WIN32 
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <direct.h>
#endif

#ifdef UNIX
#ifdef __hpux
#define LIBPATHVARNAME "SHLIB_PATH"
#elif defined _AIX
#define LIBPATHVARNAME "LIBPATH"
#else
#define LIBPATHVARNAME "LD_LIBRARY_PATH"
#endif
#endif

#ifndef MAX_PATH
#ifdef _MAX_PATH
#define MAX_PATH _MAX_PATH
#else
#define MAX_PATH 1024
#endif
#endif

/*
 * real code starts here
 */
char *
getAbsPath (char *Arg0) {
	char *OwnPath;
	char *EnvPath;
	char *Path;
	char *Ptr;
	char *Prog;
	char *Executable;

	if (Arg0 == 0)
		return 0;

	/*
	 * if arg0 contains slashes, that means no path was used
	 */
	Ptr = strrchr (Arg0, '/');
	if (Ptr == 0)
		Ptr = strrchr (Arg0, '\\');

	if (Ptr != 0) {
		/* use path from arg0 */
		int Len;
		char *Cwd;

#ifdef UNIX
		Len = Ptr - Arg0;

		if (Arg0[0] == '/') {
			/* it is already relative to '/' */
			OwnPath = (char *) malloc (Len + 1);
			if (OwnPath == 0)
				return 0;

			memcpy (OwnPath, Arg0, Len);
			OwnPath[Len] = '\0';
			return (OwnPath);
		}

		/* it is relative to the current working directory */
		Cwd = malloc (MAX_PATH + 1);
		Cwd = getcwd (Cwd, MAX_PATH);
		if (Cwd == 0) {
			free (Cwd);
			return 0;
		}

		OwnPath = (char *) malloc (strlen (Cwd) + 1 + Len + 1);
		if (OwnPath == 0) {
			free (Cwd);
			return 0;
		}

		memset (OwnPath, 0, strlen (Cwd) + 1 + Len + 1);
		strcpy (OwnPath, Cwd);
		strcat (OwnPath, "/");
		memcpy (OwnPath + strlen (OwnPath), Arg0, Len);

		free (Cwd);
		return (OwnPath);

#endif
#ifdef WIN32
		Len = Ptr - Arg0;
		OwnPath = (char *) malloc (Len + 1);
		if (OwnPath == 0)
			return 0;

		memcpy (OwnPath, Arg0, Len);
		OwnPath[Len] = '\0';
		return (OwnPath); 
#endif
	}

	/*
	 * normalize program name
	 */
#ifdef UNIX
	Executable = Arg0;
#endif
#ifdef WIN32
	Executable = alloca (strlen (Arg0) + 4 + 1);
	strcpy (Executable, Arg0);
	Ptr = strrchr (Executable, '.');
	if (Ptr != 0)
		*Ptr = '\0';

	strcat (Executable, ".exe");
#endif

	/*
	 * look in current working directory first if we are windows
	 */
#ifdef WIN32
	Ptr = alloca (MAX_PATH + 1);
	Ptr = getcwd (Ptr, MAX_PATH);
	if (Ptr != 0) {
		char *Prog;

		Prog = alloca (strlen (Ptr) + 1 + strlen (Executable) + 1);
		strcpy (Prog, Ptr);
		strcat (Prog, PATH_SEPARATOR);
		strcat (Prog, Executable);

		if (access (Prog, R_OK | X_OK) == 0) {
			OwnPath = (char *) malloc (strlen (Ptr) + 1);
			if (OwnPath == 0)
				return 0;

			strcpy (OwnPath, Ptr);
			return (OwnPath);
		}
	}
#endif

	/*
	 * path from environment was used 
	 */
	Ptr = getenv ("PATH");
	if (Ptr == 0)
		return 0;

	/*
	 * make working copy of path
	 */
	EnvPath = alloca (strlen (Ptr) + 1);
	strcpy (EnvPath, Ptr);

	for (;;) {
		if (EnvPath == 0)
			return 0;

		Path = EnvPath;

		/*
		 * split first part if ther is one move to next part
		 */
		Ptr = strchr (Path, PATH_SPLIT_CHAR);
		if (Ptr != 0) {
			*Ptr = '\0';
			EnvPath = Ptr + 1;
		} else {
			EnvPath = 0;
		}

		/*
		 * skip empty string
		 */
		if (Path[0] == '\0')
			continue;

		/*
		 * setup probable program name
		 */
		Prog = alloca (strlen (Path) + 1 + strlen (Executable) + 1);
		strcpy (Prog, Path);
		strcat (Prog, PATH_SEPARATOR);
		strcat (Prog, Executable);

		/*
		 * check for the file
		 */
		if (access (Prog, R_OK | X_OK) == 0) {
			OwnPath = (char *) malloc (strlen (Path) + 1);

			if (OwnPath == 0)
				return 0;

			strcpy (OwnPath, Path);
			return (OwnPath);
		}
	}

	return 0;
}

/*
 * return path of executeable form arg0
 */
char *
getRelPath (char *Arg0) {
	char *ptr;
	char *RelPath;

	if (Arg0 == 0)
		return 0;

	/*
	 * separate program name
	 */
	RelPath = (char *) malloc (strlen (Arg0) + 1);
	if (RelPath == 0)
		return 0;

	strcpy (RelPath, Arg0);
	ptr = strrchr (RelPath, '/');
	if (ptr == 0)
		ptr = strrchr (RelPath, '\\');

	if (ptr == 0) {
		free (RelPath);
		return 0;
	}

	*ptr = '\0';
	return (RelPath);
}

/*
 * get basename of program from arg0 
 * (without extention and without path)
 */
char *
getProgName (char *Arg0) {
	char *ptr;
	char *ProgramName;

	if (Arg0 == 0)
		return 0;

	/*
	 * first get the program name
	 */
	ptr = strrchr (Arg0, '/');
	if (ptr == 0)
		ptr = strrchr (Arg0, '\\');

	if (ptr == 0) {
		ptr = Arg0;
	} else {
		ptr++;
	}

	ProgramName = (char *) malloc (strlen (ptr) + 1);
	if (ProgramName == 0)
		return 0;

	strcpy (ProgramName, ptr);

	/*
	 * put away extention from program name
	 */
	ptr = strrchr (ProgramName, '.');
	if (ptr != 0)
		*ptr = '\0';

	return (ProgramName);
}

int 
hasMainSymbolInArgv (int argc, char **argv) {
	int i;

	for (i = 1; i < argc - 1; i++) {
		if (strcmp (argv[i], "-main") == 0)
			return 1;
	}

	for (i = 1; i < argc - 1; i++) {
		if (strcmp (argv[i], "--main") == 0)
			return 1;
	}

	for (i = 1; i < argc; i++) {
		const char *MainSwitch = "-main=";
		if (memcmp (argv[i], MainSwitch, sizeof (MainSwitch)) == 0)
			return 1;
	}

	for (i = 1; i < argc; i++) {
		const char *MainSwitch = "--main=";
		if (memcmp (argv[i], MainSwitch, sizeof (MainSwitch)) == 0)
			return 1;
	}

	return 0;
}



#ifdef UNIX

#include <sys/stat.h>
#include "maplibname.h"

int
PrepareLibs(char * szRuntimeDir,char ** libsv,char ** szTmpLDPath){
	int i = -1;
	for(;;){
		struct stat statbuf;
		char * szFullPath;
		i++;
		if(libsv[i] == 0)
			break;
		szFullPath = (char *) malloc(strlen(szRuntimeDir) + strlen(libsv[i]) + 2);
		memset(szFullPath,0,strlen(szRuntimeDir) + strlen(libsv[i]) + 2);
		strcat(szFullPath,szRuntimeDir);
		strcat(szFullPath,"/");
		strcat(szFullPath,libsv[i]);
		if(stat(szFullPath,&statbuf) != 0){
			char * szMapFullPath;
			char * szLibMap;
			char * szLinkName;
			if(strcmp(libsv[i],SDB_LIBPERL) == 0){
				szLibMap = SDB_LIBPERL_MAP;
			}else
			if(strcmp(libsv[i],SDB_LIBWXPERL) == 0){
				szLibMap = SDB_LIBWXPERL_MAP;	
			}
			else
			if(strcmp(libsv[i],SDB_LIBWXWIN) == 0){
			 	szLibMap = SDB_LIBWXWIN_MAP;
			}
			else
			if(strcmp(libsv[i],SDB_LIBGRID) == 0){
			 	szLibMap = SDB_LIBGRID_MAP;
			}
			else{
				fprintf(stderr,"PrepareLibs(): unkown lib: %s\n",libsv[i]);
				free(szFullPath);
				return -1;
			}
			szMapFullPath = (char *) malloc(strlen(szRuntimeDir) + strlen(szLibMap) + 2);
			memset(szMapFullPath,0,strlen(szRuntimeDir) + strlen(szLibMap) + 2);
			strcat(szMapFullPath,szRuntimeDir);
			strcat(szMapFullPath,"/");
			strcat(szMapFullPath,szLibMap);
			if(stat(szMapFullPath,&statbuf) != 0){
				/*fprintf(stderr,"PrepareLibs(): lib %s not found\n",libsv[i]);*/
				free(szFullPath);
				free(szMapFullPath);
				continue;
			}
			if(*szTmpLDPath == 0){
				char * szTemplate;
				char * szTmplFile = "SDBLDXXXXXX";
				char * szTmpFS = getenv("TMP");
				if(szTmpFS == 0){
					szTmpFS = "/var/tmp";
					if(access(szTmpFS,W_OK) != 0){
						szTmpFS = "/tmp";
						if(access(szTmpFS,W_OK) != 0){
							fprintf(stderr,"PrepareLibs(): no tmp dir found\n");
							free(szFullPath);
							free(szMapFullPath);
							return -1;
						}	
					}
				}
				szTemplate = (char *) malloc(strlen(szTmpFS) + strlen(szTmplFile) + 2);
				memset(szTemplate,0,strlen(szTmpFS) + strlen(szTmplFile) + 2);
				strcat(szTemplate,szTmpFS);
				strcat(szTemplate,"/");
				strcat(szTemplate,szTmplFile);
#ifdef HAS_MKDTEMP
				*szTmpLDPath = mkdtemp(szTemplate);
#else
				*szTmpLDPath = mktemp(szTemplate);
				mkdir(*szTmpLDPath,0700);
#endif
			}
			szLinkName = (char *) malloc(strlen(*szTmpLDPath) + strlen(libsv[i]) + 2);
			memset(szLinkName,0,strlen(*szTmpLDPath) + strlen(libsv[i]) + 2);
			strcat(szLinkName,*szTmpLDPath);
			strcat(szLinkName,"/");
			strcat(szLinkName,libsv[i]); 
			symlink(szMapFullPath,szLinkName);
			free(szMapFullPath);
			free(szLinkName);
		}
		free(szFullPath);
	}
	return 0;
}




#endif


#ifdef UNIX
/*
 * UNIX uses 'execv ("SDBRUN", ...)' to start runtime
 */
int
LoadAndExec (char *exename, int argc, char **argv, char **env) {
	int rc;
	char *szMyLdLibPath;
	char *szLdLibPath;
	char *szEnvLdLibPath;
	char *ptr;
	char * libsv[5];
	char * szTmpLDPath = 0;	
	libsv[0] = SDB_LIBWXWIN;
	libsv[1] = SDB_LIBPERL;
	libsv[2] = SDB_LIBWXPERL;
	libsv[3] = SDB_LIBGRID;
	libsv[4] = 0;
	
	

#if defined __hpux && defined __hppa
	putenv("LD_PRELOAD=/usr/lib/libpthread.sl:/usr/lib/libcl.sl");
#endif

	szMyLdLibPath = (char *) malloc (strlen (exename) + 1);
	if (szMyLdLibPath == 0) {
		fprintf (stderr, "cannot allocate memory\n");
		return -1;
	}

	memset (szMyLdLibPath, 0, strlen (exename) + 1);
	strcpy (szMyLdLibPath, exename);
	ptr = strrchr (szMyLdLibPath, PATH_SEPARATOR_CHAR);
	if (ptr != 0) {
		*ptr = '\0';
	} else {
		char *szEnvPath, *szPath;

		ptr = getenv ("PATH");
		if (ptr == 0)
			return -1;

		/*
		 * make working copy of path
		 */
		szEnvPath = malloc (strlen (ptr) + 1);
		strcpy (szEnvPath, ptr);

		for (;;) {
			if (szEnvPath == 0) {
				/* nothing found */
				return -1;
			}

			szPath = szEnvPath;

			/*
			 * split first part, if there is one move to next part
			 */
			ptr = strchr (szPath, PATH_SPLIT_CHAR);
			if (ptr != 0) {
				*ptr = '\0';
				szEnvPath = ptr + 1;
			} else {
				szEnvPath = 0;
			}

			/*
			 * skip empty string
			 */
			if (szPath[0] == '\0')
				continue;

			/*
			 * setup probable program name
			 */
			ptr = malloc (strlen (szPath) + 1 + strlen ("SDBRUN") + 1);
			strcpy (ptr, szPath);
			strcat (ptr, PATH_SEPARATOR);
			strcat (ptr, "SDBRUN");

			/*
			 * check for the file
			 */
			if (access (ptr, R_OK | X_OK) == 0) {
				free(szMyLdLibPath);
				szMyLdLibPath = (char *) malloc (strlen (szPath) + 1);

				if (szMyLdLibPath == 0) {
					free (ptr);
					return -1;
				}

				strcpy (szMyLdLibPath, szPath);
				free (ptr);
				break;
			}
		}
	}

	ptr = strstr(exename,"./");
	
	if(ptr == exename){
		char * cwd = malloc (MAX_PATH + 1);
		char * szAbs;
		cwd = getcwd (cwd, MAX_PATH);
		szAbs = malloc(strlen(cwd) + strlen(szMyLdLibPath));
		memset(szAbs,0,strlen(cwd) + strlen(szMyLdLibPath));
		strcat(szAbs,cwd);
		if(strlen(szMyLdLibPath) > 1){
			strcat(szAbs,szMyLdLibPath + 1);
		}
		free(szMyLdLibPath);
		free(cwd);
		szMyLdLibPath = szAbs;
	}
	
	ptr = strstr(exename,"../");
	
	if(ptr == exename){
		char * cwd = malloc (MAX_PATH + 1);
		char * szAbs;
		int len;
		cwd = getcwd (cwd, MAX_PATH);
		ptr = strrchr(cwd,'/');
		if(ptr == 0)
			len = strlen(cwd);
		else
			len = ptr - cwd;
				
		szAbs = malloc( len  + strlen(szMyLdLibPath) - 1);
		memset(szAbs,0, len  + strlen(szMyLdLibPath) - 1);
		strncpy(szAbs,cwd,len);
		if(strlen(szMyLdLibPath) > 2){
			strcat(szAbs,szMyLdLibPath + 2);
		}
		free(szMyLdLibPath);
		free(cwd);
		szMyLdLibPath = szAbs;
	}

	PrepareLibs(szMyLdLibPath,libsv,&szTmpLDPath);
	
	if(szTmpLDPath != 0 && strlen(szTmpLDPath) != 0){
		char ** newargv;
		int count;
		szMyLdLibPath =	(char *) realloc (szMyLdLibPath,
		strlen (szMyLdLibPath) + 1 + strlen (szTmpLDPath) + 1);
		if (szMyLdLibPath == 0) {
			fprintf (stderr, "cannot allocate memory\n");
			return -1;	
		}
		memset(szMyLdLibPath + strlen(szMyLdLibPath),0,
			strlen (szTmpLDPath) + 2);
		strcat (szMyLdLibPath, ":");
		strcat (szMyLdLibPath, szTmpLDPath);
		
		newargv = (char **) malloc(sizeof(void*) * (argc + 3));
		memset(newargv,0,sizeof(void*) * (argc + 3));
				
		for(count=0;count<argc;count++){
			newargv[count] = argv[count];
		}
		newargv[argc] = "--TmpLDPath";
		newargv[argc + 1] = szTmpLDPath;
		argv = newargv;
	}
	szLdLibPath = getenv (LIBPATHVARNAME);
	if (szLdLibPath != 0 && strlen (szLdLibPath) != 0) {
		szMyLdLibPath =	(char *) realloc (szMyLdLibPath,
		strlen (szMyLdLibPath) + 1 + strlen (szLdLibPath) + 1);
		if (szMyLdLibPath == 0) {
			fprintf (stderr, "cannot allocate memory\n");
			return -1;	
		}
		memset(szMyLdLibPath + strlen(szMyLdLibPath),0,
			strlen (szLdLibPath) + 2);
		
		strcat (szMyLdLibPath, ":");
		strcat (szMyLdLibPath, szLdLibPath);
	}
	
	szEnvLdLibPath = (char *)
		malloc (strlen (LIBPATHVARNAME) + 1 + strlen (szMyLdLibPath) + 1);

	if (szEnvLdLibPath == 0) {
		fprintf (stderr, "cannot allocate memory\n");
		return -1;	
	}

	strcpy (szEnvLdLibPath, LIBPATHVARNAME);
	strcat (szEnvLdLibPath, "=");
	strcat (szEnvLdLibPath, szMyLdLibPath);
	putenv (szEnvLdLibPath);
	
	free (szMyLdLibPath);

	rc = -1;
	argv[0] = exename;
	rc = execv (exename, argv);
	fprintf (stderr, "cannot exec %s: %s\n", exename, strerror(errno));  

	return (rc);
}
#endif

#ifdef WIN32
/*
 * windows uses 'LoadLibraryEx ("sdbrun.dll", ...)' to start runtime
 */
typedef int (*sdbrunmain_t) (int, char **, char **);

int
LoadAndExec (char *libname, int argc, char **argv, char **env) {
	int rc;
	HANDLE hLibSDBRUN;
	sdbrunmain_t SDBRUNMain;

	char *symname = "sdbrun_main";
	char *our;
	char *their;
	char *dir;
	char *ptr;

	/*
	 * modify PATH here to find depending perl58.dll,
	 * it is located in the same directory like sdbrun.dll
	 */
	our = malloc (strlen ("PATH=") + 1);
	if (our == 0) {
		fprintf (stderr, "cannot allocate memory\n");
		return -1;
	}

	strcpy (our, "PATH=");
	dir = (char *) malloc (strlen (libname) + 1);
	if (dir == 0) {
		fprintf (stderr, "cannot allocate memory\n");
		return -1;
	}

	strcpy (dir, libname);
	ptr = strrchr (dir, '\\');
	if (ptr != 0)
		*ptr = '\0';
	
	if (strlen (dir) == 0) {
		free (dir);
		dir = 0;
	}

	if (dir != 0) {
		our = realloc (our, strlen (our) + strlen (dir) + 1);
		if (our == 0) {
			fprintf (stderr, "cannot allocate memory\n");
			free (dir);
			return -1;
		}

		strcat (our, dir);
		free (dir);
	}

	their = getenv ("PATH");
	if (their != 0 && dir != 0) {
		our = realloc (our, strlen (our) + 1 + 1);
		if (our == 0) {
			fprintf (stderr, "cannot allocate memory\n");
			return -1;
		}

		strcat (our, ";");
	}

	if (their != 0) {
		our = realloc (our, strlen (our) + strlen (their) + 1);
		if (our == 0) {
			fprintf (stderr, "cannot allocate memory\n");
			return -1;
		}

		strcat (our, their);
	}

	putenv (our);
	
	hLibSDBRUN = LoadLibraryEx (libname, 0, 0);
	if (hLibSDBRUN == 0) {
		DWORD errcode;
		char *errtext ;

		errcode = GetLastError ();

		rc = FormatMessage (
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 0,
		errcode, MAKELANGID (LANG_ENGLISH, SUBLANG_ENGLISH_US),
		(LPTSTR) &errtext, 0, 0);

		if (rc == 0) {
			fprintf (stderr, "cannot load lib %s, error %d\n", libname, errcode);
		} else {
			fprintf (stderr, "cannot load lib %s, error (%d) %s\n", libname, errcode, errtext);
			LocalFree (errtext);
		}

		free (our);
		free (dir);
		return -1;
	}

	SDBRUNMain = (sdbrunmain_t) GetProcAddress (hLibSDBRUN, symname);
	if (SDBRUNMain == 0) {
		free (our);
		FreeLibrary (hLibSDBRUN);
		fprintf (stderr, "cannot find entry point %s\n", symname);
		return -1;
	}

	rc = SDBRUNMain (argc, argv, env);

	free (our);
	FreeLibrary (hLibSDBRUN);
	return (rc);
}
#endif
