/*
 * ArchiveZip.xs
 */

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#ifdef UNIX
#include <unistd.h>
#endif

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <io.h>
#include <malloc.h>
#endif

#ifdef UNIX
#include <malloc.h>
#endif

#include <sys/stat.h>
typedef struct stat stat_t;

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#ifdef PERL_OBJECT
#define FILE struct _PerlIO
#endif

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#if defined _WIN64
typedef __int64 ptr_t;
#else
typedef long ptr_t;
#endif

#include "unzip.h"

typedef struct {
	char *fn;
	void *fp;
	int idx;
	int cnt;
	int eof;
	int usecache;
	int size;
} ziparch_t;

typedef struct {
	ziparch_t *arch;
	int size;
	int pos;
} zipfile_t;

static int
find (ziparch_t *arch, char *my_filename) {
	char *their_filename;
	unz_file_info fi[1];

	if (unzGoToFirstFile (arch->fp) != 0)
		return -1;

	arch->idx = -1;

	their_filename = 0;
	for (;;) {
		if (arch->idx >= 0)
			if (unzGoToNextFile (arch->fp) != 0) {
				if (their_filename != 0)
					free (their_filename);
				return -1;
			}

		arch->idx++;
	
		if (unzGetCurrentFileInfo (arch->fp, fi, 0, 0, 0, 0, 0, 0) != 0) {
			if (their_filename != 0)
				free (their_filename);

			return -1;
		}

		their_filename =
		(char *) realloc (their_filename, fi->size_filename + 1);
		
		if (
		unzGetCurrentFileInfo (arch->fp, fi, their_filename,
		fi->size_filename + 1, 0, 0, 0, 0) != 0) {
			if (their_filename != 0)
				free (their_filename);

			return -1;
		}

		if (strcmp (my_filename, their_filename) == 0) {
			if (their_filename != 0)
				free (their_filename);

			return 0;
		}
		
		if (arch->idx >= arch->cnt) {
			if (their_filename != 0)
				free (their_filename);

			return -1;
		}
	}

	if (their_filename != 0)
		free (their_filename);

	return -1;
}

/*
 * helper functions to access member variables
 */
static ziparch_t *
getarch (HV *self) {
	char *key = "arch";
	SV **psv;

	psv = hv_fetch (self, key, strlen (key), 0);
	if (psv == 0)
        	return 0;

	return ((ziparch_t *) SvIV (*psv));
}

static zipfile_t *
getfile (HV *self) {
	char *key = "file";
	SV **psv;

	psv = hv_fetch (self, key, strlen (key), 0);
	if (psv == 0)
        	return 0;

	return ((zipfile_t *) SvIV (*psv));
}

/*
 * implementaion of sysseek() is really poor
 * only set to current position is implemented
 */
MODULE = SDB::Common::Archive::Zip	PACKAGE = SDB::Common::Archive::Zip

PROTOTYPES: DISABLE

void
new (...)
PREINIT:
	int rc;
	HV *self;
	char *fn;
	ziparch_t *arch;
	char *CLASS = 0;
	unz_global_info gi[1];
	const char *key = "arch";
	Stat_t sb[1];
CODE:
	if (items < 1 || items > 2)
		XSRETURN_UNDEF;

	if (items == 2 && (strlen (SvPV_nolen (ST(0))) > 0))
		CLASS = SvPV_nolen (ST(0));

	if (CLASS == 0)
		CLASS = "SDB::Common::Archive::Zip";

	fn = (char *) SvPV_nolen (ST (items - 1));
	if (strlen (fn) == 0)
		XSRETURN_UNDEF;

	if (stat (fn, sb) != 0)
		XSRETURN_UNDEF;

	if (S_ISREG (sb->st_mode) == 0)
		XSRETURN_UNDEF;

	arch = (ziparch_t *) malloc (sizeof (ziparch_t));
	if (arch == 0)
		XSRETURN_UNDEF;

	memset (arch, 0, sizeof (ziparch_t));

	arch->fn = (char *) malloc (strlen (fn) + 1);
	if (arch->fn == 0) {
		free (arch);
		XSRETURN_UNDEF;
	}

	strcpy (arch->fn, fn);

	arch->fp = (void *) unzOpen (fn);
	if (arch->fp == 0) {
		free (arch->fn);
		free (arch);
		XSRETURN_UNDEF;
	}

	rc = unzGetGlobalInfo (arch->fp, gi);
	if (rc != 0) {
		free (arch->fp);
		free (arch->fn);
		free (arch);
		XSRETURN_UNDEF;
	}

	arch->cnt = gi->number_entry;
	arch->idx = -1;

	self = newHV ();
	ST(0) = sv_bless (newRV ((SV*) self), gv_stashpv (CLASS, 1));

	hv_store (self, key, strlen (key), newSViv ((ptr_t) arch), 0);

	sv_2mortal (ST(0));
	SvREFCNT_dec (self);

void
DESTROY (...)
PREINIT:
	ziparch_t *arch;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	arch = getarch ((HV*) SvRV (ST(0)));
	if (arch == 0)
		XSRETURN_UNDEF;

	if (arch->fp != 0) {
		unzClose (arch->fp);
		arch->fp = 0;
	}

	if (arch->fn != 0) {
		free (arch->fn);
		arch->fn = 0;
	}

	if (arch != 0)
		free (arch);

	XSRETURN (0);

void
open (...)
PREINIT:
	int rc;
	SV *sv;
	HV *self;
	char *filename = 0;
	ziparch_t *arch;
	zipfile_t *file;
	const char *key = "file";
	const char *CLASS = "SDB::Common::Archive::Zip::File";
PPCODE:
	if (items < 1 || items > 2)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	arch = getarch ((HV*) SvRV (ST(0)));
	if (arch == 0)
		XSRETURN_UNDEF;

	if (arch->fp == 0)
		XSRETURN_UNDEF;
	
	if (items == 2) {
		filename = SvPV_nolen (ST(1));
		if (filename == 0 || strlen (filename) == 0)
			XSRETURN_UNDEF;

		if (find (arch, filename) != 0)
			XSRETURN_UNDEF;
	}

	arch->eof = 0;

	rc = unzOpenCurrentFile (arch->fp);
	if (rc != 0)
		XSRETURN_UNDEF;

	file = (zipfile_t *) malloc (sizeof (zipfile_t));
	if (file == 0)
		XSRETURN_UNDEF;

	memset (file, 0, sizeof (zipfile_t));
	file->arch = arch;
	file->size = arch->size;

	self = newHV ();
	hv_store (self, key, strlen (key), newSViv ((ptr_t) file), 0);
	sv = sv_bless (newRV ((SV*) self), gv_stashpv (CLASS, 1));

	XPUSHs (sv_2mortal (sv));
	XSRETURN (1);

void
usecache (...)
PREINIT:
	ziparch_t *arch;
PPCODE:
	if (items != 2)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	arch = getarch ((HV*) SvRV (ST(0)));
	if (arch == 0)
		XSRETURN_UNDEF;

	arch->usecache = SvIV (ST(1));
	if (arch->usecache != 0)
		arch->usecache = 1;

	XPUSHs (sv_2mortal (newSViv (0)));
	XSRETURN (1);

MODULE = SDB::Common::Archive::Zip	PACKAGE = SDB::Common::Archive::Zip::File

PROTOTYPES: DISABLE

void
DESTROY (...)
PREINIT:
	ziparch_t *arch;
	zipfile_t *file;
PPCODE:
	if (items != 1)
		XSRETURN (0);

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN (0);

	file = getfile ((HV*) SvRV (ST(0)));
	if (file == 0)
		XSRETURN (0);

	arch = file->arch;
	if (arch == 0)
		XSRETURN (0);

	if (arch->fp == 0)
		XSRETURN (0);

	unzCloseCurrentFile (arch->fp);
	free (file);

	XSRETURN (0);

void
close (...)
PREINIT:
	int rc;
	HV *self;
	ziparch_t *arch;
	zipfile_t *file;
	const char *key = "file";
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	file = getfile (self);
	if (file == 0)
		XSRETURN_UNDEF;

	arch = file->arch;
	if (arch == 0)
		XSRETURN_UNDEF;

	if (arch->fp == 0) {
		XPUSHs (sv_2mortal (newSViv (0)));
		XSRETURN (1);
	}

	rc = unzCloseCurrentFile (arch->fp);
	if (rc != 0)
		XSRETURN_UNDEF;

	free (file);
	hv_store (self, key, strlen (key), newSViv (0), 0);

	XPUSHs (sv_2mortal (newSViv (0)));
	XSRETURN (1);

void
sysread (...)
PREINIT:
	HV *self;
	zipfile_t *file;
	ziparch_t *arch;
	int want, got, off;
	char *buff;
PPCODE:
	if (items != 3)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	file = getfile (self);
	if (file == 0)
		XSRETURN_UNDEF;

	arch = file->arch;
	if (arch == 0)
		XSRETURN_UNDEF;

	if (arch->fp == 0) {
		XPUSHs (sv_2mortal (newSViv (-1)));
		XSRETURN (1);
	}

	if (SvREADONLY (ST(1)))
		XSRETURN_UNDEF;

	want = SvIV (ST (2));
	buff = (char *) malloc (want);
	if (buff == 0)
		XSRETURN_UNDEF;

	got = unzReadCurrentFile (arch->fp, buff, want);
	if (got < 0) {
		free (buff);
		XSRETURN_UNDEF;
	}

	file->pos += got;

	sv_setpvn (ST(1), (const char *) buff, got);
	free (buff);

	XPUSHs (sv_2mortal (newSViv (got)));
	XSRETURN (1);

void
read (...)
PREINIT:
	SV *sv;
	HV *self;
	zipfile_t *file;
	ziparch_t *arch;
	int want, got, off;
	char *buff;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	file = getfile (self);
	if (file == 0)
		XSRETURN_UNDEF;

	arch = file->arch;
	if (arch == 0)
		XSRETURN_UNDEF;

	if (arch->fp == 0)
		XSRETURN_UNDEF;

	off = 0;
	buff = 0;

	for (;;) {
		want = off + 8192;
		buff = (char *) realloc (buff, want);
		if (buff == 0)
			XSRETURN_UNDEF;

		got = unzReadCurrentFile (arch->fp, buff + off, 8192);
		if (got < 0) {
			free (buff);
			XSRETURN_UNDEF;
		}

		off += got;
		if (got == 0) {
			got = off;
			break;
		}
	}

	sv = newSVpvn ("", 0);
	sv_setpvn (sv, (const char *) buff, got);
	free (buff);

	XPUSHs (sv_2mortal (sv));
	XSRETURN (1);

void
sysseek (...)
PREINIT:
	int rc;
	HV *self;
	ziparch_t *arch;
	zipfile_t *file;
	const char *key = "file";
	int off;
	int mode;
PPCODE:
	if (items != 3)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	file = getfile (self);
	if (file == 0)
		XSRETURN_UNDEF;

	arch = file->arch;
	if (arch == 0)
		XSRETURN_UNDEF;

	off = SvIV (ST(1));
	mode = SvIV (ST(2));

	if (mode == SEEK_CUR && off == 0) {
		XPUSHs (sv_2mortal (newSViv (file->pos)));
		XSRETURN (1);
	}

	if (mode == SEEK_SET && off == file->pos) {
		XPUSHs (sv_2mortal (newSViv (file->pos)));
		XSRETURN (1);
	}

	XPUSHs (sv_2mortal (newSViv (-1)));
	XSRETURN (1);

