/*
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/BackupPipe.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/BackupPipe.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#ifdef PERL_OBJECT
#define FILE struct _PerlIO
#endif

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif
#include <errno.h>
#ifdef UNIX

#endif

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#if defined _WIN64
typedef __int64 ptr_t;
#else
typedef long ptr_t;
#endif

#if defined WIN32
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#define NEED_TYPEDEF_UNIT16_T NEED_TYPEDEF_UNIT16_T
#define NEED_TYPEDEF_UNIT32_T NEED_TYPEDEF_UNIT32_T
#define NEED_TYPEDEF_UNIT64_T NEED_TYPEDEF_UNIT64_T
#elif defined linux
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __hpux && defined __hppa
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __hpux && defined __ia64 && ! defined HAS_STRTOLL
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __osf__
#define NEED_TYPEDEF_UNIT16_T NEED_TYPEDEF_UNIT16_T
#define NEED_TYPEDEF_UNIT32_T NEED_TYPEDEF_UNIT32_T
#define NEED_TYPEDEF_UNIT64_T NEED_TYPEDEF_UNIT64_T
#define NEED_SNPRINTF NEED_SNPRINTF
#define HAS_SPRINTF_INTRINSIC HAS_SPRINTF_INTRINSIC
#endif

#ifdef NEED_TYPEDEF_UCHAR_T
typedef unsigned char  uchar_t;
#endif
#ifdef NEED_TYPEDEF_UNIT16_T
typedef unsigned short uint16_t;
#endif
#ifdef NEED_TYPEDEF_UNIT32_T
typedef unsigned int   uint32_t;
#endif
#ifdef NEED_TYPEDEF_UNIT64_T
typedef unsigned long  uint64_t;
#endif


#ifndef WIN32
#define UNIX UNIX
#endif

typedef struct {
	char *name;
	char *mode;
#ifdef UNIX
	int fd;
#endif
#ifdef WIN32
	HANDLE hnd;
#endif
} pipe_t;

#ifdef WIN32
static pipe_t *
createpipe (const char *filename, const char *mode) {
	pipe_t *pipe = 0;

	/* only write pipe is implemented */
	if (strcmp (mode, "w") != 0)
		return 0;

	/* pipe must have a name */
	if (filename == 0 || filename[0] == '\0')
		return 0;

	pipe = (pipe_t *) malloc (sizeof (pipe_t));
	if (pipe == 0)
		return 0;

	memset (pipe, 0, sizeof (pipe_t));
	pipe->hnd = INVALID_HANDLE_VALUE;

	pipe->name = (char *) malloc (strlen (filename) + 1);
	if (pipe->name == 0) {
		free (pipe);
		return 0;
	}

	strcpy (pipe->name, filename);

	pipe->mode = (char *) malloc (strlen (mode) + 1);
	if (pipe->mode == 0) {
		unlink (filename);
		free (pipe->name);
		free (pipe);
		return 0;
	}

	strcpy (pipe->mode, mode);
	return (pipe);
}

static int
openpipe (pipe_t *pipe) {
	DWORD rc = 0;
	ULONG size = 64 * 1024;

	/* only write pipe is implemented */
	if (strcmp (pipe->mode, "w") != 0)
		return 0;

	pipe->hnd =
	CreateNamedPipe (pipe->name, PIPE_ACCESS_OUTBOUND,
	PIPE_TYPE_BYTE | PIPE_READMODE_BYTE | PIPE_WAIT, 1,
	size, size, INFINITE, 0);

	if (pipe->hnd != INVALID_HANDLE_VALUE) {
		if (ConnectNamedPipe (pipe->hnd, 0) == 0) {
			CloseHandle (pipe->hnd);
			pipe->hnd = INVALID_HANDLE_VALUE;

			return 0;
		}

		return 1;
	}
	
	if (GetLastError () != ERROR_PIPE_BUSY)
		return 0;

	pipe->hnd =
	CreateFile (pipe->name, GENERIC_WRITE,
	FILE_SHARE_READ, 0, OPEN_EXISTING,
	FILE_ATTRIBUTE_NORMAL, 0);

	if (pipe->hnd == INVALID_HANDLE_VALUE);
		return 0;

	return 1;
}

static int
closepipe (pipe_t *pipe) {
	if (pipe->hnd == INVALID_HANDLE_VALUE)
		return 0;

	CloseHandle (pipe->hnd);
	return 0;
}

static int
writepipe (const pipe_t *pipe, const void *buff, const int size) {
	DWORD want, got;

	if (pipe->hnd == INVALID_HANDLE_VALUE)
		return -1;

	want = (DWORD) size;

	if (WriteFile (pipe->hnd, buff, want, &got, 0) == 0)
		return -1;

	return (got);
}
#endif

#ifdef UNIX

#include <fcntl.h>

static pipe_t *
createpipe (const char *filename, const char *mode) {
	int rc;
	pipe_t *pipe;

	signal (SIGPIPE, SIG_IGN);

	rc = mknod (filename, 0666 | S_IFIFO, 0);
	if (rc != 0) {
		if (errno != EEXIST)
			return 0;

		unlink (filename);
		rc = mknod (filename, 0666 | S_IFIFO, 0);
	}

	if (rc != 0)
		return 0;

	rc = chmod (filename, 0666);
	if (rc != 0) {
		unlink (filename);
		return 0;
	}

	pipe = (pipe_t *) malloc (sizeof (pipe_t));
	if (pipe == 0) {
		unlink (filename);
		return 0;
	}

	memset (pipe, 0, sizeof (pipe_t));

	pipe->fd = -1;

	pipe->name = (char *) malloc (strlen (filename) + 1);
	if (pipe->name == 0) {
		unlink (filename);
		free (pipe);
		return 0;
	}

	strcpy (pipe->name, filename);

	pipe->mode = (char *) malloc (strlen (mode) + 1);
	if (pipe->mode == 0) {
		unlink (filename);
		free (pipe->name);
		free (pipe);
		return 0;
	}

	strcpy (pipe->mode, mode);

	return (pipe);
}

static int
openpipe (pipe_t *pipe) {
	if (pipe == 0)
		return 0;

	if (pipe->fd >= 0)
		return 0;

	/* only write pipe is implemented */
	if (strcmp (pipe->mode, "w") == 0)
		pipe->fd = open (pipe->name, O_WRONLY);

	if (pipe->fd < 0)
		return 0;

	return 1;
}

static int
closepipe (pipe_t *pipe) {
	if (pipe == 0)
		return 0;

	if (pipe->fd >= 0)
		close (pipe->fd);

	if (pipe->name != 0) {
		unlink (pipe->name);
		free (pipe->name);
	}

	if (pipe->mode != 0)
		free (pipe->mode);
	
	memset (pipe, 0, sizeof (pipe_t));
	pipe->fd = -1;

	return 0;
}

static int
writepipe (const pipe_t *pipe, const void *buff, const int size) {
	int got, want;

	if (pipe == 0)
		return -1;

	if (pipe->fd < 0)
		return -1;

	want = (int) size; 
	got = write (pipe->fd, buff, want);
	if (got < 0)
		return -1;

	return (got);
}
#endif


MODULE = SAPDB::Install::BackupPipe PACKAGE = SAPDB::Install::BackupPipe

PROTOTYPES: DISABLE

void
new (...)
PREINIT:
	char *CLASS;
	HV *self;
	SV *sv;
	pipe_t *obj;
	char *keyobj = "obj";
	char *name;
	char *mode;
CODE:
	if (items != 3)	
		XSRETURN_UNDEF;

	CLASS = SvPV (ST(0), PL_na);
	name =  SvPV (ST(1), PL_na);
	mode =  SvPV (ST(2), PL_na);

	obj = createpipe (name, mode);
	if (obj == 0)
		XSRETURN_UNDEF;

	self = newHV ();
	ST(0) = sv_bless (newRV ((SV*) self), gv_stashpv (CLASS, 1));
	sv = newSViv ((ptr_t) obj);
	hv_store (self, keyobj, strlen (keyobj), sv, 0);

	sv_2mortal (ST(0));
	SvREFCNT_dec (self);

void
DESTROY (...)
PREINIT:
	HV *self;
	SV **psv;
	pipe_t *obj;
	char *keyobj = "obj";
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (sv_isobject (ST(0)) == 0)
		XSRETURN_UNDEF;

	if (SvTYPE (SvRV (ST(0))) != SVt_PVHV)
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	psv = hv_fetch (self, keyobj, strlen (keyobj), 0);
	if (psv == 0)
		XSRETURN_UNDEF;

	obj = (pipe_t *) SvIV (*psv);
	if (obj == 0)
		XSRETURN_UNDEF;

	closepipe (obj);
	free (obj);

	XSRETURN_UNDEF;

void
Open (...)
PREINIT:
	int rc;
	HV *self;
	SV **psv;
	pipe_t *obj;
	char *keyobj = "obj";
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (sv_isobject (ST(0)) == 0)
		XSRETURN_UNDEF;

	if (SvTYPE (SvRV (ST(0))) != SVt_PVHV)
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	psv = hv_fetch (self, keyobj, strlen (keyobj), 0);
	if (psv == 0)
		XSRETURN_UNDEF;

	obj = (pipe_t *) SvIV (*psv);
	if (obj == 0)
		XSRETURN_UNDEF;

	rc = openpipe (obj);

	XPUSHs (sv_2mortal (newSViv (rc)));
	XSRETURN (1);

void
Close (...)
PREINIT:
	HV *self;
	SV **psv;
	pipe_t *obj;
	char *keyobj = "obj";
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (sv_isobject (ST(0)) == 0)
		XSRETURN_UNDEF;

	if (SvTYPE (SvRV (ST(0))) != SVt_PVHV)
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	psv = hv_fetch (self, keyobj, strlen (keyobj), 0);
	if (psv == 0)
		XSRETURN_UNDEF;

	obj = (pipe_t *) SvIV (*psv);
	if (obj == 0)
		XSRETURN_UNDEF;

	closepipe (obj);

	XSRETURN_UNDEF;

void
Write (...)
PREINIT:
	HV *self;
	SV **psv;
	int got;
	int size;
	char *buff;
	pipe_t *obj;
	char *keyobj = "obj";
PPCODE:
	if (items != 3)
		XSRETURN_UNDEF;

	if (sv_isobject (ST(0)) == 0)
		XSRETURN_UNDEF;

	if (SvTYPE (SvRV (ST(0))) != SVt_PVHV)
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	psv = hv_fetch (self, keyobj, strlen (keyobj), 0);
	if (psv == 0)
		XSRETURN_UNDEF;

	obj = (pipe_t *) SvIV (*psv);
	if (obj == 0)
		XSRETURN_UNDEF;

	buff =  SvPV (ST(1), PL_na);
	size =  SvIV (ST(2));

	got = writepipe (obj, buff, size);

	XPUSHs (sv_2mortal (newSViv (got)));
	XSRETURN (1);

