/*
 * $SRC/install/perl/Comm.xs
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/Comm.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/Comm.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#ifdef __hpux
#define DONT_DECLARE_STD 1
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#ifdef UNIX
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#endif

#if defined _WIN64
typedef __int64 ptr_t;
#else
typedef long ptr_t;
#endif

#if defined WIN32
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#define NEED_TYPEDEF_UNIT16_T NEED_TYPEDEF_UNIT16_T
#define NEED_TYPEDEF_UNIT32_T NEED_TYPEDEF_UNIT32_T
#define NEED_TYPEDEF_UNIT64_T NEED_TYPEDEF_UNIT64_T
#elif defined linux
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __hpux  && defined __hppa
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __hpux && defined __ia64 && ! defined HAS_STRTOLL
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __osf__
#define NEED_TYPEDEF_UNIT16_T NEED_TYPEDEF_UNIT16_T
#define NEED_TYPEDEF_UNIT32_T NEED_TYPEDEF_UNIT32_T
#define NEED_TYPEDEF_UNIT64_T NEED_TYPEDEF_UNIT64_T
#define NEED_SNPRINTF NEED_SNPRINTF
#define HAS_SPRINTF_INTRINSIC HAS_SPRINTF_INTRINSIC
#endif

#ifdef NEED_TYPEDEF_UCHAR_T
typedef unsigned char  uchar_t;
#endif
#ifdef NEED_TYPEDEF_UNIT16_T
typedef unsigned short uint16_t;
#endif
#ifdef NEED_TYPEDEF_UNIT32_T
typedef unsigned int   uint32_t;
#endif
#ifdef NEED_TYPEDEF_UNIT64_T
typedef unsigned long  uint64_t;
#endif

/*
 * our internal comm structure
 */
typedef struct commobj {
	int fd;
	uint32_t theirref;
} commobj_t;

/*
 * rte header
 * total 24 byte
 */
typedef struct {
	uint32_t actlen;
	uchar_t  protocol;
	uchar_t  messclass;
	uchar_t  flags;
	uchar_t  trailpkg;
	uint32_t sendref;
	uint32_t recvref;
	uint16_t rc;
	uchar_t  swaptype;
	uchar_t  reserved;
	uint32_t maxlen;
} sapdbcomm_rtehdr_t;

/*
 * minimal part of connect packet,
 * additional len type value encoded data can follow
 * total 40 + options
 */
typedef struct {
	uchar_t  charcoding;
	uchar_t  swapmode;
	uint16_t len;
	uchar_t  service;
	uchar_t  ostype;
	uint16_t padding;
	uint32_t max_segment_size;
	uint32_t max_data_size;
	uint32_t packet_size;
	uint32_t min_reply_size;
	uchar_t  dbname[8];
	uchar_t  reserved[8];
} sapdbcomm_conhdr_t;

#define SAPDBCOMM_RTE_PROTO_TCP         3

#define SAPDBCOMM_RTE_SWAPTYPE_BE	3
#define SAPDBCOMM_RTE_SWAPTYPE_LE	4

#define SAPDBCOMM_RTE_MESS_PINGREQ     53
#define SAPDBCOMM_RTE_MESS_PINGREP     54
#define SAPDBCOMM_RTE_MESS_USERDATAREQ 63
#define SAPDBCOMM_RTE_MESS_USERDATAREP 64
#define SAPDBCOMM_RTE_MESS_DBMCONREQ   91
#define SAPDBCOMM_RTE_MESS_DBMCONREP   92

#define SAPDBCOMM_CON_OPTION_VERSION  'V'
#define SAPDBCOMM_CON_OPTION_PID      'I'
#define SAPDBCOMM_CON_OPTION_DIR      'd'
#define SAPDBCOMM_CON_OPTION_EXE      'p'

#define SAPDBCOMM_CON_CHAR_ASCII        0

#define SAPDBCOMM_CON_SWAPTYPE_BE       1
#define SAPDBCOMM_CON_SWAPTYPE_LE       2

#define SAPDBCOMM_CON_SERVICE_DBM       4

#define SAPDBCOMM_CON_OSTYPE_UNIX      11 
#define SAPDBCOMM_CON_OSTYPE_WIN32     13 

#define SAPDBCOMM_DBM_PACKET_SIZE (16 * 1024)
#define SAPDBCOMM_DBM_MINREP_SIZE       (112) 
#define SAPDBCOMM_DBM_MAXDATA_SIZE \
	(SAPDBCOMM_DBM_PACKET_SIZE - 2 * sizeof (sapdbcomm_rtehdr_t))

#if HAS_BYTEORDER == 4321 || HAS_BYTEORDER == 87654321
static const int my_rtehdr_swaptype = SAPDBCOMM_RTE_SWAPTYPE_BE;
static const int my_conhdr_swapmode = SAPDBCOMM_CON_SWAPTYPE_BE;
#else
static const int my_rtehdr_swaptype = SAPDBCOMM_RTE_SWAPTYPE_LE;
static const int my_conhdr_swapmode = SAPDBCOMM_CON_SWAPTYPE_LE;
#endif

#ifdef UNIX
static const int my_conhdr_ostype = SAPDBCOMM_CON_OSTYPE_UNIX;
#endif
#ifdef WIN32
static const int my_conhdr_ostype = SAPDBCOMM_CON_OSTYPE_WIN32;
#endif

static int tcp_connect (char *, char *, char *);
static int tcp_release (int, char *);
static int comm_conopt_addstring (uchar_t *, int, char *);
static int comm_conopt_getstring (uchar_t *, int, int *, char *);
static void swap_rtehdr (sapdbcomm_rtehdr_t *);
static void swap_uint32 (void *);
static void swap_uint16 (void *);

/*
 * you can use DEBUG_RTEHDR to dump your rtehdr
 * if you completely do not know what's 
 * wrong in sapdb communication
 */

#ifdef DEBUG_RTEHDR
static void printf_rtehdr (sapdbcomm_rtehdr_t * rtehdr) {
	printf ("rtehdr->actlen=%d\n", rtehdr->actlen);
	printf ("rtehdr->maxlen=%d\n", rtehdr->maxlen);
	printf ("rtehdr->sendref=%d\n", rtehdr->sendref);
	printf ("rtehdr->recvref=%d\n", rtehdr->recvref);
	printf ("rtehdr->swaptype=%d\n", rtehdr->swaptype);
	printf ("rtehdr->protocol=%d\n", rtehdr->protocol);
	printf ("rtehdr->messclass=%d\n", rtehdr->messclass);
}
#endif

/*
 * some tcp special communiation code
 */
static int
tcp_release (int fd, char *sz_err) {
	int rc;

	rc = close (fd);
	if (rc < 0) {
		sprintf (sz_err, "close failed");
	}
	return 0;
}

static int
tcp_connect (char *sz_host, char *sz_service, char *sz_err) {
	int fd;
	int rc;
	struct sockaddr sa[1];
	struct sockaddr_in *sa_in = (struct sockaddr_in *) sa;
	struct servent *se;
	struct hostent *he;

	fd = socket (PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fd < 0) {
		sprintf (sz_err, "socket failed");
		return -1;
	}

	se = getservbyname (sz_service, "tcp");
	if (se == 0) {
		sprintf (sz_err, "getservbyname failed");
		close (fd);
		return -1;
	}

	he = gethostbyname (sz_host);
	if (he == 0) {
		sprintf (sz_err, "gethostbyname failed");
		close (fd);
		return -1;
	}

	sa->sa_family = AF_INET;
	sa_in->sin_port = se->s_port;
	memcpy (&sa_in->sin_addr, &he->h_addr[0], sizeof (sa_in->sin_addr));

	rc = connect (fd, sa, sizeof (struct sockaddr));
	if (rc < 0) {
#if defined WIN32
		if (WSAGetLastError () == WSAECONNREFUSED) {
#else
		if (errno == ECONNREFUSED) {
#endif
			sprintf (sz_err, 
			"connect failed: Connection refused, "
			"probably no vserver running\n"); 
		} else {
			sprintf (sz_err, 
#ifdef WIN32
			"connect failed\n");
#else
			"connect failed: %s\n",Strerror(errno));
#endif
		}
		close (fd);
		return -1;
	}
	
	return fd;
}

/*
 * some sapdb session communication code
 */
static commobj_t *
comm_new (char *sz_host, char *sz_err) {
	commobj_t *obj;

	obj = (commobj_t *) malloc (sizeof (commobj_t));
	if (obj == 0) {
		strcpy (sz_err, "malloc for comm session object failed");
		return 0;
	}

	memset (obj, 0, sizeof (commobj_t));

	/*
	 * establish tcp connection
	 */
	obj->fd = tcp_connect (sz_host, "sql6", sz_err);
	if (obj->fd < 0) {
		obj->fd = -1;
		return (obj);
	}

	return (obj);
}

static int
comm_ping (commobj_t *obj, char *sz_version, char *sz_err) {
	int rc;
	int idx;
	int type;
	int len;
	fd_set fds;
	int fd_max;
	struct timeval tv;

	sapdbcomm_rtehdr_t *rtehdr;
	sapdbcomm_conhdr_t *conhdr;
	uchar_t buff[16 * 1024];

	if (sz_version != 0) {
		strcpy (sz_version, "");
	}

	if (sz_err != 0) {
		strcpy (sz_err, "");
	}

	if (obj == 0) {
		return -1;
	}

	if (obj->fd == -1) {
		return -1;
	}

	/*
	 * make layout of sapdb-comm-connect-packet
	 */
	rtehdr = (sapdbcomm_rtehdr_t *) buff;
	conhdr = (sapdbcomm_conhdr_t *) (rtehdr + 1);

	/*
 	 * fill in rte-header
	 */
	rtehdr->actlen    = sizeof (sapdbcomm_rtehdr_t);
	rtehdr->protocol  = SAPDBCOMM_RTE_PROTO_TCP;
	rtehdr->messclass = SAPDBCOMM_RTE_MESS_PINGREQ;
	rtehdr->flags     = 0;
	rtehdr->trailpkg  = 0;
	rtehdr->sendref   = 1;
	rtehdr->recvref   = 0;
	rtehdr->rc        = 0;
	rtehdr->swaptype  = my_rtehdr_swaptype;
	rtehdr->reserved  = 0;
	rtehdr->maxlen    = sizeof (sapdbcomm_rtehdr_t);
	
	/*
	 * fill in connect-packet-header for dbmsrv
	 */
	conhdr->charcoding = SAPDBCOMM_CON_CHAR_ASCII;
	conhdr->swapmode = my_conhdr_swapmode;
	conhdr->len = sizeof (sapdbcomm_conhdr_t);
	conhdr->service = 0;
	conhdr->ostype = my_conhdr_ostype;
	conhdr->padding = 0;
	conhdr->packet_size = SAPDBCOMM_DBM_PACKET_SIZE;
	conhdr->max_data_size = SAPDBCOMM_DBM_MAXDATA_SIZE;
	conhdr->max_segment_size = SAPDBCOMM_DBM_MAXDATA_SIZE;
	conhdr->min_reply_size = SAPDBCOMM_DBM_MINREP_SIZE;
	memset (conhdr->dbname, ' ', sizeof (conhdr->dbname));
	memset (conhdr->reserved, ' ', sizeof (conhdr->reserved));
	rtehdr->actlen += sizeof (sapdbcomm_conhdr_t);
	rtehdr->maxlen += sizeof (sapdbcomm_conhdr_t);

#ifdef DEBUG_RTEHDR
	printf ("send comm pkt\n");
	printf_rtehdr (rtehdr);
	printf ("\n");
#endif
	/*
	 * send it to server
	 */	
	rc = write (obj->fd, buff, rtehdr->actlen);
	if (rc != rtehdr->actlen) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "write () failed");
		obj->fd = -1;
		return (-1);
	}	

	/*
	 * wait 180 sec for reply
	 */
	for (;;) {
		int timeout = 180;

		FD_ZERO (&fds);
		FD_SET (obj->fd, &fds);	
		fd_max = obj->fd;
	
		tv.tv_sec = timeout;
		tv.tv_usec = 0;
		rc = select (fd_max + 1, &fds, 0, 0, &tv);

		if (rc == 0) {
			tcp_release (obj->fd, sz_err);
			sprintf (sz_err, "no ping reply after %d sec", timeout);
			obj->fd = -1;
			return (-1);
		}
		
		if (rc < 0) {
			tcp_release (obj->fd, sz_err);
			sprintf (sz_err, "error during select()");
			obj->fd = -1;
			return (-1);
		}
		
		if (FD_ISSET (obj->fd, &fds)) {
			break;
		}
	}

	rc = read (obj->fd, buff, 16 * 1024);
	if (rc < 0) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "read () failed");
		obj->fd = -1;
		return (-1);
	}

	if (rc == 0) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "connection closed by counterpart");
		obj->fd = -1;
		return (-1);
	}

	/*
	 * make sure that binary values are readable
	 */
	if (rtehdr->swaptype != my_rtehdr_swaptype) {
		swap_rtehdr (rtehdr);
	}

#ifdef DEBUG_RTEHDR
	printf ("received comm pkt\n");
	printf_rtehdr (rtehdr);
	printf ("\n");
#endif
	if (rtehdr->messclass != SAPDBCOMM_RTE_MESS_PINGREP) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "ping reply expected, got %d",
		         rtehdr->messclass);
		obj->fd = -1;
		return (-1);
	}

	if (rtehdr->rc != 0) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "ping rtehdr->rc=%d", rtehdr->rc);
		obj->fd = -1;
		return (-1);
	}

	obj->theirref = rtehdr->sendref;

	for (idx = 0;; idx++) {
		len = comm_conopt_getstring (buff, idx, &type, 0);

		if (len <= 0) {
			return 0;
		}

		if (type == SAPDBCOMM_CON_OPTION_VERSION) {
			comm_conopt_getstring (buff, idx, &type, sz_version);
			return 0;
		}
	}

	return 0;
}

static int
sess_release (commobj_t *obj, char *sz_err) {
	int rc = 0;
	
	if (obj == 0)
		return (rc);

	/*
	 * free own connection
	 */
	if (obj->fd != -1) {
		rc = tcp_release (obj->fd, sz_err); 
	}

	free (obj);

	return (rc);	
}

static int
comm_conopt_getstring (uchar_t *pkt, int idx, int *p_type, char *sz_text) {
	unsigned int pos;
	int i;
	int len;
	sapdbcomm_rtehdr_t *rtehdr;
	sapdbcomm_conhdr_t *conhdr;

	/*
	 * get layout of conpkt
	 */
	rtehdr = (sapdbcomm_rtehdr_t *) pkt;
	conhdr = (sapdbcomm_conhdr_t *) (rtehdr + 1);
 
	if (rtehdr->actlen == 
	sizeof (sapdbcomm_rtehdr_t) + sizeof (sapdbcomm_conhdr_t)) {
		*p_type = 0;
		return 0;
	}

	pos = sizeof (sapdbcomm_rtehdr_t) + sizeof (sapdbcomm_conhdr_t);
	
	for (i = 0; i < idx; i++) {
		pos += pkt[pos];
		if (pos > rtehdr->actlen) {
			*p_type = 0;
			return 0;
		}
	}

	*p_type = pkt[pos + 1];
	len = pkt[pos];

	if (sz_text != 0) {
		memcpy (sz_text, pkt + pos + 2, len);
		sz_text[len] = '\0';
	}
	return (len);
}

static int
comm_conopt_addstring (uchar_t *pkt, int type, char *sz_text) {
	int pos;
	int len;
	sapdbcomm_rtehdr_t *rtehdr;
	sapdbcomm_conhdr_t *conhdr;

	/*
	 * get layout of conpkt
	 */
	rtehdr = (sapdbcomm_rtehdr_t *) pkt;
	conhdr = (sapdbcomm_conhdr_t *) (rtehdr + 1);
 
	/*
	 * add string on the end of packet
	 * a length, type, data encoding is used
	 * strings are represented as ascii with terminating zero
	 * length includes the terminating zero
	 */
	pos = rtehdr->actlen;
	len = strlen (sz_text) + 3;
	pkt[pos] = (uchar_t) len;	
	pkt[pos + 1] = (uchar_t) type;	
	memcpy (pkt + pos + 2, sz_text, strlen (sz_text) + 1); 

	/*
	 * fix len fields of packet
	 */
	rtehdr->actlen += len;
	rtehdr->maxlen += len; 
	conhdr->len += len; 
	return 0;
}

/*
 * helper functions for swapping binary int's in rtehdr
 */
static void 
swap_rtehdr (sapdbcomm_rtehdr_t *rtehdr) {
	swap_uint32 (&rtehdr->actlen);	
	swap_uint32 (&rtehdr->sendref);	
	swap_uint32 (&rtehdr->recvref);	
	swap_uint32 (&rtehdr->maxlen);	
	swap_uint16 (&rtehdr->rc);	
	rtehdr->swaptype = my_rtehdr_swaptype;
}

static void
swap_uint16 (void *val) {
	uchar_t *byte = (uchar_t *) val;
	uchar_t tmp;

	tmp = byte[1];
	byte[1] = byte[0];
	byte[0] = tmp;
}

static void
swap_uint32 (void *val) {
	uchar_t *byte = (uchar_t *) val;
	uchar_t tmp;

	tmp = byte[3];
	byte[3] = byte[0];
	byte[0] = tmp;
	
	tmp = byte[2];
	byte[2] = byte[1];
	byte[1] = tmp;
}

/*
 * helper functions to access member variables
 */
static commobj_t *
self_getobj (HV *self) {
	char *key_obj = "obj";
	SV **psv_obj;
	commobj_t *obj;

	psv_obj = hv_fetch (self, key_obj, strlen (key_obj), 0);
	if (psv_obj == 0) {
		return 0;
	}	

	obj = (commobj_t *) SvIV (*psv_obj);
	return (obj);
}

static SV *
self_getsverr (HV *self) {
	char *key_err = "err";
	SV **psv_err;

	psv_err = hv_fetch (self, key_err, strlen (key_err), 0);
	if (psv_err == 0) {
		return 0;
	}	
	return (*psv_err);
}

MODULE = SAPDB::Install::Comm       PACKAGE = SAPDB::Install::Comm

PROTOTYPES: DISABLE

void
new (...)
PREINIT:
	char *CLASS;
	HV *self;
	SV *sv_obj;
	SV *sv_err;
	char *key_obj = "obj";
	char *key_err = "err";
	commobj_t *obj;
	char *sz_host = "localhost";
	char sz_err[1024];
CODE:
	if (items >= 1) {
		if (strlen (SvPV (ST(0), PL_na)) > 0)
			CLASS = SvPV (ST(0), PL_na);
	}
	if (items >= 2) {
		if (strlen (SvPV (ST(1), PL_na)) > 0)
			sz_host = SvPV (ST(1), PL_na);
	}
	if (items >= 3 || items == 0) {
		XSRETURN_UNDEF;
	}
	
	self = newHV ();
	ST(0) = sv_bless (newRV ((SV*) self), gv_stashpv (CLASS, 1));
	obj = comm_new (sz_host, sz_err);

	if (obj == 0 || obj->fd == -1) {
		sv_err = newSVpv (sz_err, strlen (sz_err));
	} else {
		sv_err = newSVsv (&PL_sv_undef);
	}
	hv_store (self, key_err, strlen (key_err), sv_err, 0);

	sv_obj = newSViv ((ptr_t) obj);
	hv_store (self, key_obj, strlen (key_obj), sv_obj, 0);

	sv_2mortal (ST(0));
	SvREFCNT_dec (self);

void
DESTROY (...)
PREINIT:
	HV *self;
	commobj_t *obj;
	char sz_err[1024];
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
		XSRETURN_UNDEF;
	}

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	sess_release (obj, sz_err);
	XSRETURN_UNDEF;

void
ping (...)
PREINIT:
	HV *self;
	commobj_t *obj;
	SV *sv_err;
	char sz_err[1024];
	char sz_version[1024];
	int rc;
	int argc;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
		XSRETURN_UNDEF;
	}

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	sv_err = self_getsverr (self);

	rc = comm_ping (obj, sz_version, sz_err);

	if (rc != 0) {
		sv_setpv (sv_err, sz_err);
		XSRETURN_UNDEF;
	}

	sv_setsv (sv_err, &PL_sv_undef);
	XPUSHs (sv_2mortal (newSViv (0)));
	argc = 1;
	
	if (strlen (sz_version) > 0) {
		char *sz_version_name = "version";

		argc +=2;
		XPUSHs (sv_2mortal (newSVpvn (sz_version_name,
		        strlen (sz_version_name))));
		XPUSHs (sv_2mortal (newSVpvn (sz_version,
		        strlen (sz_version))));
	}

	XSRETURN (argc);

void
lasterr (...)
PREINIT:
	HV *self;
	SV *sverr;
PPCODE:
	if (items > 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
		XSRETURN_UNDEF;
	}

	self = (HV*) SvRV (ST(0));
	sverr = self_getsverr (self);
	if (sverr == 0)
		XSRETURN_UNDEF;

	XPUSHs (sv_2mortal (newSVsv (sverr)));
	XSRETURN (1);

