#!/usr/bin/perl
#
# DBMCmd.pl
# perl source for package SAPDB::Install::DBMCmd
#
# $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/DBMCmd.pl#1 $
# $DateTime: 2005/03/21 16:21:00 $
# $Change: 93297 $
#

package SAPDB::Install::DBMCmd;

#
# return last complete dbm dialog
# contains cmd, msg if there is any and err if there is any
# 
# this is no dbmsrv command, 
# but an internal helper function to read our local communication buffers
#
sub lastdialog {
	my ($dbm, $id) = @_;

	$id = 1 unless (defined $id);
	my $cmd = $dbm->lastcmd ($id);
	my $msg = $dbm->lastmsg ($id);
	my $err = $dbm->lasterr ($id);

	my $txt = '';
	$txt .= ($cmd."\n") if (defined $cmd);
	$txt .= ($msg."\n") if (defined $msg);
	$txt .= ($err."\n") if (defined $err);

	return ($txt);
}

#
# executes a dbm command with force_ascii == 0
#
# this is no dbmsrv command, 
# but an internal helper function used by almost any dbm command
#
sub exec {
	my ($dbm, @args) = @_;

	return (exec_ex ($dbm, 0, @args));
}

#
# do a dbm command 'param_startsession'
# return the dbm message
#
sub param_startsession {
	my ($dbm) = @_;
	my ($rc) = split ('\n', $dbm->exec ("param_startsession"));
	return undef
		unless ($rc =~ /^OK/);

	return $rc;
}

#
# do a dbm command 'param_abortsession'
# return the dbm message
#
sub param_abortsession {
	my ($dbm) = @_;
	my ($rc) = split ('\n', $dbm->exec ("param_abortsession"));
	return undef
		unless ($rc =~ /^OK/);

	return $rc;
}

#
# do a dbm command 'param_commitsession'
# return the dbm message
#
sub param_commitsession {
	my ($dbm) = @_;
	my ($rc) = split ('\n', $dbm->exec ("param_commitsession"));
	return undef
		unless ($rc =~ /^OK/);

	return $rc;
}

#
# do a dbm command 'param_getfull'
# return a key value hash
#
sub param_getfull {
	my ($dbm, $name) = @_;
	my ($rc, $type, $default, $value, @data) = 
		split ('\n', $dbm->exec ("param_getfull ".$name));
	return undef
		unless ($rc =~ /^OK/);

	my %param;
	
	#
	# saparate handling for TYPE, DEAFAULT and VALUE,
	# no names are given, we get only the value
	#
	$param{'TYPE'} = $type; 
	$param{'DEFAULT'} = $default; 
	$param{'VALUE'} = $value; 

	while ($_ = shift @data) {
		# we cannot use split,
		# because some parameter are containing spaces
		# there are paramters without a value too
		my ($key, $val) = ($_ =~ /^(\S+)\s+(\S.*)/);
		unless (defined $key && defined $val) {
			($key) = ($_ =~ /^(\S+)/);
			$val = "" if (defined $key);
		}
		if ($key eq 'HELP' || $key eq 'EXPLAIN') {
			unshift @data, $_;
			last;
		}
		$param{$key} = $val if (defined $key && defined $val);
	}

	#
	# speparate handling for HELP and EXPLAIN
	# these are multiline values at the end of the message
	#
	my ($key, $val);
	foreach $_ (@data) {
		$key = $val = undef;	
		if ($_ eq 'HELP' || $_ eq 'EXPLAIN') {
			if (defined $key) {
				chomp $val;
				$param{$key} = $val;
			}
			$key = $_;
			$val = "";
			next;
		}
		$val .= $key."\n"; 
	}
	if (defined $key) {
		chomp $val;
		$param{$key} = $val;
	}
	return %param;
}

#
# do a dbm command 'param_directgetall'
# return a key value hash,
# where key is paramter name and value is parameter value
#
sub param_directgetall {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ("param_directgetall"));
	return undef
		unless ($rc =~ /^OK/);

	my $param = {};
	foreach (@data) {
		# we cannot use split,
		# because some parameter are containing spaces
		# there are paramters without value
		my ($key, $val) = ($_ =~ /^(\S+)\s+(\S.*)/);
		unless (defined $key && defined $val) {
			($key) = ($_ =~ /^(\S+)/);
			$val = "" if (defined $key);
		}
		$param->{$key} = $val if (defined $key && defined $val);
	}
	return $param;
}

#
# do a dbm command 'param_directget'
# return the value
#
sub param_directget {
	my ($dbm, $name) = @_;
	my ($rc, $data) = split ('\n', $dbm->exec ("param_directget ".$name));
	return undef
		unless ($rc =~ /^OK/);

	$_ = $data;
	# we cannot use split,
	# because some parameter are containing spaces
	# there are paramters without value
	my ($key, $val) = ($_ =~ /^(\S+)\s+(\S.*)/);
	unless (defined $key && defined $val) {
		($key) = ($_ =~ /^(\S+)/);
		$val = "" if (defined $key);
	}
	return undef
		unless (defined $key && defined $val && $key eq $name);

	return $val;
}

#
# do a dbm command 'param_directput'
# return 'OK' when command was successfully or undef in case of error
#
sub param_directput {
	my ($dbm, $key, $val) = @_;
	return undef unless (defined $key && defined $val);

	my ($rc) = split ('\n', $dbm->exec ("param_directput ".$key." ".$val));
	return undef unless ($rc =~ /^OK/);

	return $rc;
}

#
# do a dbm command 'param_getvolsall'
# return a hash reference
#
sub param_getvolsall {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ("param_getvolsall"));
	return undef
		unless ($rc =~ /^OK/);

	my $param = {};
	foreach (@data) {
		# we cannot use split,
		# because some parameter are containing spaces
		# there are paramters without value
		my ($key, $val, $size, $type, $name);

		# may be volume definition,
		# looks like 'LOG_VOLUME_NAME_001 5000 F LOG_001'
		($key, $size, $type, $name) =
		($_ =~ /^(\S+)\s+(\S.+)\s+(\S+)\s+(\S.*)/);
		if (defined $key && $key =~ /LOG_VOLUME_|ARCHIVE_LOG_|DATA_VOLUME_|DATADEV_/ &&
		    defined $size && defined $type && defined $name) {
			$val = {};
			$val->{size} = $size;
			$val->{type} = $type;
			$val->{name} = $name;
			$param->{$key} = $val;
			next;
		}

		# looks like 'M_LOG_VOLUME_NAME_001 F LOG_001'
		($key, $type, $name) =
		($_ =~ /^(\S+)\s+(\S+)\s+(\S+)/);
		if (defined $key && $key =~ /M_LOG_VOLUME_|M_ARCHIVE_LOG_/ &&
		    defined $type && defined $name) {
			$val = {};
			$val->{size} = '';
			$val->{type} = $type;
			$val->{name} = $name;
			$param->{$key} = $val;
			next;
		}

		# may be simple value, looks like 'MAXLOGVOLUMES 2'
		($key, $val) = ($_ =~ /^(\S+)\s+(\S.*)/);
		if (defined $key && defined $val) {
			$param->{$key} = $val;
			next;
		}

		# may be empty value, should not happen 
		($key) = ($_ =~ /^(\S+)/);
		if (defined $key) {
			$param->{$key} = "";
			next;
		}
	}

	return $param;
}

#
# do a dbm command 'param_gethistory'
# return a hash of arrays,
# where keys are column name for dbm message
# and values are references to arrays
# each array is a list of values according to the column name
#
sub param_gethistory {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ("param_gethistory"));
	return undef
		unless ($rc =~ /^OK/);

	# get column names and positions form header line
	# build initial return hash
	my %history;
	my @pos = ();
	my @names = ();

	my $header = shift @data;
	foreach my $columname (split (' ', $header)) {
		my @columnvalues = ();
		$history{$columname} = \@columnvalues;
		$header =~ /$columname/;
		push @pos, length ($`);
		push @names, $columname;
	}

	# transform each raw
	foreach $raw (@data) {
		# get each value from the raw
		for (my $i = 0; $i <= $#pos; $i++) {
			my $val =
			($i + 1 <= $#pos) ?
			substr ($raw, $pos[$i], $pos[$i + 1] - $pos[$i]) :
			substr ($raw, $pos[$i]);
		
			# get rid of trailing spaces
			($val, my $space) = ($val =~ /^(.*\S)(\s+)$/);
			
			# fill it into return hash
			push @{$history{$names[$i]}}, $val;
		}
	}
	return (%history);
}

#
# do a dbm command 'version'
# return the a hash,
# where key is value name and value is value
# the dbm message:
#
# "version,os,dbroot,logon,code,swap\n"
# "\"7.2.4\", \"UNIX\", \"/usr/sapdb/depend\",False,ASCII,2"
#
# will be the hash
#
# (VERSION => "7.2.4", OS => "UNIX", INSTROOT => "/usr/sapdb/depend", 
#  LOGON => False, CODE => ASCII, SWAP => 2)
#
# keys are converted to look like dbm_version output
#
sub version {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ("version"));
	return undef
		unless ($rc =~ /^OK/);

	my %version;
	my @line0 = split (',', $data[0]);
	my @line1 = split (',', $data[1]);
	
	my ($key, $val);

	while ($key = shift (@line0)) {
		$val = shift (@line1);
		# unquote values
		($val) = $val =~ /^\"(.*)\"$/ if ($val =~ /^\".*\"$/);

		# uppercase keys
		$key = ($key =~ /^DBROOT$/i) ? 'INSTROOT' : uc ($key);
		$version{$key} = $val;
	}
	return %version;
}

#
# do a dbm command 'dbm_version'
# return the a hash,
# where key is value name and value is value
# the dbm message:
#
# VERSION    = 7.3.0
# BUILD      = DBMServer 7.3.0    Build 014-000-067-507
# OS         = UNIX
# INSTROOT   = /ls8801_d1/port/V7300/usr
# LOGON      = False
# CODE       = UTF8
# SWAP       = full
# UNICODE    = YES
# INSTANCE   = OLTP
# SYSNAME    = Linux
#
# will be the hash
#
# VERSION => 7.3.0
# BUILD => DBMServer 7.3.0    Build 014-000-067-507
# OS => UNIX
# INSTROOT => /ls8801_d1/port/V7300/usr
# LOGON => False
# CODE => UTF8
# SWAP => full
# UNICODE => YES
# INSTANCE => OLTP
# SYSNAME => Linux
#
sub dbm_version {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ("dbm_version"));
	return undef
		unless ($rc =~ /^OK/);

	my %dbm_version;
	
	foreach (@data) {
		my ($key, $val) = ($_ =~ /(\w+)\s+= (.+)/);
		$dbm_version{$key} = $val;
	}
	return %dbm_version;
}

#
#
# do a dbm command 'db_restartinfo'
# return the a hash,
# where key is value name and value is value
# the dbm message:
#
# Used LOG Page           36
# First LOG Page          0
# Restartable             1
# Id Restart Record       ls8801:M7300_20010629_131133
# Id LOG Info             ls8801:M7300_20010629_131133
# Consistent              1
# 
# will be the hash
#
# ('Used LOG Page'     => 36, 
#  'First LOG Page'    => 0, 
#  'Restartable'       => 1,
#  'Id Restart Record' => ls8801:M7300_20010629_131133
#  'Id LOG Info'       => ls8801:M7300_20010629_131133
#  'Consistent'        => 1)
#
sub db_restartinfo {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ("db_restartinfo"));

	return undef
		unless ($rc =~ /^OK/);

	my %info;
	foreach (@data) {
		my @line = split;
		my $val = pop @line;
		my $key = join (" ", @line);
		$info{$key} = $val;
	}
	return %info;
}

#
# do a dbm command 'help'
# a parameter is optional
# return the dbm message in array format
#
sub help {
	my ($dbm, $name) = @_;
	
	my $cmd = "help";
	$cmd .= " ".$name if (defined $name);

	my ($rc, @data) = split ('\n', $dbm->exec ($cmd));

	return undef
		unless ($rc =~ /^OK/);
	
	return @data;
}

#
# do a dbm command 'user_logon'
# return the complete dbm message
#
sub user_logon {
	my ($dbm, $name) = @_;

	my ($rc) = split ('\n', $dbm->exec ("user_logon ".$name));
	return undef
		unless ($rc =~ /^OK/);

	return $rc;
}

#
# do a dbm command 'dbm_getpath'
# sometimes they return two lines, second line contains 
# 'INFO: Path extracted from $DBROOT'
# put these grabage away and return only first line
#
sub dbm_getpath {
	my ($dbm, $name) = @_;
	
	my $cmd = 'dbm_getpath';
	$cmd .= ' '.$name if (defined $name);
	my ($rc, $data, @garbage) = split ('\n', $dbm->exec ($cmd));

	return undef
		unless ($rc =~ /^OK/);
	
	return $data;
}

#
# do a dbm command 'db_state'
# returns current state or undef on error
# 
sub db_state {
	my ($dbm)=@_;
	my ($rc, $padding, $state) = split ('\n', $dbm->exec ("db_state"));
	return undef
		unless ($rc =~ /^OK/);
	return $state;
}

#
# do a dbm command 'db_cold'
# returns current state or undef on error
# 
sub db_cold {
	my ($dbm)=@_;
	my ($rc, $padding, $state);

	($rc) = split ('\n', $dbm->exec ("db_cold"));
	return undef
		unless ($rc =~ /^OK/);
	
	($rc, $padding, $state) = split ('\n', $dbm->exec ("db_state"));
	return undef
		unless ($rc =~ /^OK/);
	return $state;
}

#
# do a dbm command 'db_admin'
# returns current state or undef on error
# 
sub db_admin {
	my ($dbm)=@_;
	my ($rc, $padding, $state);

	($rc) = split ('\n', $dbm->exec ("db_admin"));
	return undef
		unless ($rc =~ /^OK/);
	
	($rc, $padding, $state) = split ('\n', $dbm->exec ("db_state"));
	return undef
		unless ($rc =~ /^OK/);
	return $state;
}

#
# do a dbm command 'db_warm'
# returns current state or undef on error
# 
sub db_warm {
	my ($dbm)=@_;
	my ($rc, $padding, $state);

	($rc) = split ('\n', $dbm->exec ("db_warm"));
	return undef
		unless ($rc =~ /^OK/);
	
	($rc, $padding, $state) = split ('\n', $dbm->exec ("db_state"));
	return undef
		unless ($rc =~ /^OK/);
	return $state;
}

#
# do a dbm command 'db_online'
# returns current state or undef on error
# 
sub db_online {
	my ($dbm)=@_;
	my ($rc, $padding, $state);

	($rc) = split ('\n', $dbm->exec ("db_online"));
	return undef
		unless ($rc =~ /^OK/);
	
	($rc, $padding, $state) = split ('\n', $dbm->exec ("db_state"));
	return undef
		unless ($rc =~ /^OK/);
	return $state;
}

#
# do a dbm command 'db_offline'
# returns current state or undef on error
# 
sub db_offline {
	my ($dbm)=@_;
	my ($rc, $padding, $state);

	($rc) = split ('\n', $dbm->exec ("db_offline"));
	return undef
		unless ($rc =~ /^OK/);
	
	($rc, $padding, $state) = split ('\n', $dbm->exec ("db_state"));
	return undef
		unless ($rc =~ /^OK/);
	return $state;
}

#
# do a dbm command 'db_drop'
# returns success
# 
sub db_drop {
	my ($dbm)=@_;
	my ($rc, $padding, $state);

	($rc) = split ('\n', $dbm->exec ("db_drop"));
	return undef
		unless ($rc =~ /^OK/);
	return 1;
}




#
# do a dbm command 'db_clear'
# returns current state or undef on error
# 
sub db_clear {
	my ($dbm)=@_;
	my ($rc, $padding, $state);

	($rc) = split ('\n', $dbm->exec ("db_clear"));
	return undef
		unless ($rc =~ /^OK/);
	
	($rc, $padding, $state) = split ('\n', $dbm->exec ("db_state"));
	return undef
		unless ($rc =~ /^OK/);
	return $state;
}

sub show_version {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ('show version'));
	return undef
		unless ($rc =~ /^OK/);

	return @data;
}

sub param_checkall {
	my ($dbm) = @_;
	my ($rc) = split ('\n', $dbm->exec ("param_checkall"));
	return undef
		unless ($rc =~ /^OK/);
	return $rc;	
}

sub load_systab {
	my ($dbm, $domain_passwd) = @_;

	my $cmd = (defined $domain_passwd and $domain_passwd ne '') ?
	"load_systab -ud ".$domain_passwd : "load_systab";

	my ($rc) = split ('\n', $dbm->exec ($cmd));
	return undef
		unless ($rc =~ /^OK/);
	return $rc;	
}

#
# backup_history_open
# does a dbm command backup_history_open,
# any options especially '-e' to get external media info are ignored
#
sub backup_history_open {
	my ($dbm) = @_; 
	my ($rc) = split ('\n', $dbm->exec ('backup_history_open'));
	return undef
		unless ($rc =~ /^OK/);
	return $rc;
}

#
# backup_history_close
# does a dbm command backup_history_close
#
sub backup_history_close {
	my ($dbm) = @_; 
	my ($rc) = split ('\n', $dbm->exec ('backup_history_close'));
	return undef
		unless ($rc =~ /^OK/);
	return $rc;
}

#
# backup_history_list
# get all requested backup history data
# options '-e' for external media info 
# and '-m' for backup media info are ignored 
#
sub backup_history_list {
	my ($dbm, $options) = @_;
	my $cmd = 'backup_history_list';
	my $state = '';
	my @cols = (); 

	#
	# pass options like '-k', ...
	# ignore options '-e' and '-m'
	# save '-c' option, will be handled separtely
	#
	foreach (split (' ', $options)) {
		if ($_ =~ /^-e|^-m/) {
			# ignore them
			next;
		} elsif ($state =~ /^-c/) {
			# save requested col names
			@cols = split (',', $_);
			$state = '';
			next;
		} elsif ($_ =~ /^-c/) {
			# found '-c' option
			$state = '-c';
			next;
		}

		# pass option to dbmsrv
		$cmd .= ' '.$_;	
	}

	#
	# setup default colums if no colums are requested
	# and pass columns to dbmsrv
	#
	if ($#cols < 0) {
		@cols = (
			'KEY', 'LABEL', 'ACTION', 'STAMP1', 'STAMP2', 
			'START', 'STOP', 'FIRSTLOG', 'LASTLOG', 'LOG', 
			'MEDIA', 'PAGES', 'VOLUMES', 'RC', 'ERROR');
	}
	$cmd .= ' -c '.join (',', @cols);

	#
	# read first chunk of data
	# use 'backup_history_list' to pass options
	#
	my ($rc, $cont, @history) = 
		split ('\n', $dbm->exec ($cmd));

	return undef unless ($rc =~ /^OK/);

	#	
	# read following chunks of data
	# use 'backup_history_listnext' till we come to end
	#
	for (;;) {
		last if ($cont =~ /^END/);

		my @val;
		($rc, $cont, @val) = 
			split ('\n', $dbm->exec ('backup_history_listnext'));

		return undef unless ($rc =~ /^OK/);
		push @history, @val;
	}
	
	#
	# return empty string in case of empty history
	#
	@history = ('') if ($#history < 0);	

	#
	# make a hash output,
	# this looks like a table, where keys are columnames
	# and colums are arrays
	#
	my %out;
	foreach my $key (@cols) {
		my @tmp = ();
		$out{$key} = \@tmp;
	}

	foreach my $line (@history) {
		my @array = split (/\|/, $line);
		for (my $i = 0; defined $cols[$i]; $i++) {
			my $val = $array[$i];
			#
			# remove trailing and heading spaces for every value
			# expect LABEL or LOG in HISTLOST
			#
			unless ($cols[$i] =~ /^LABEL$|^LOG$/ &&
			$line =~ /HISTLOST/) {
				$val =~ s/^(\s+)//;
				$val =~ s/(\s+)$//;
			}
			push @{$out{$cols[$i]}}, $val;
		}
	}
	return (\%out);
}

#
# open a file which is readable by dbmgetf
# returns an dbmgetf object
#
# lines can be read by $dbmgetf_obj->readline ()
#
sub dbmgetf {
	my ($dbm, $fileid, $options) = @_;

	my $dbmgetf = {};
	bless ($dbmgetf, 'SAPDB::Install::DBMCmd::Getf');

	#
	# set force_ascii to 1 for UTLPRT cause
	# dbm.utl may be polluted by non ascii characters
	# e.g. in case of failed restore
	#
	my $force_ascii = ($fileid =~ /UTLPRT/) ? 1 : 0;

	#
	# open file and get first chunk of data
	#
	my $cmd = 'file_getfirst '.$fileid;
	$cmd .= ' '.$options if (defined $options);

	my $raw = $dbm->exec_ex ($force_ascii, $cmd);

	#
	# first split rc, must be 'OK'
	#	
	my $rc = ($raw =~ s/^(.+)\n//) ? $1 : undef;
	return undef unless ($rc =~ /^OK/);

	#
	# split other lines of data
	#
	$dbmgetf->{'handle'} = ($raw =~ s/^(.+)\n//) ? $1 : undef;
	$dbmgetf->{'cont'} = ($raw =~ s/^(.+)\n//) ? $1 : undef;
	$raw =~ s/^(.+)\n//;

	#
	# store text, dbmobj and fileid
	#
	$dbmgetf->{'txt'} = $raw;
	$dbmgetf->{'dbm'} = $dbm;
	$dbmgetf->{'fileid'} = $fileid;
	$dbmgetf->{'force_ascii'} = $force_ascii;
	$dbmgetf->{'goteof'} = 0;

	return $dbmgetf;
}

package SAPDB::Install::DBMCmd::Getf;

sub readline {
	my ($dbmgetf) = @_;
	my $fileid = $dbmgetf->{'fileid'};

	#
	# try to read line from buffer
	#
	my $line = ($dbmgetf->{'txt'} =~ s/^(.+)\n//) ? $1 : undef;
	if (defined $line) {
		#
		# remove any 'cr', I've found them even on UNIX in 'dbm.utl'
		#
		$line =~ s/\r//g;

		if ($fileid =~ /UTLPRT/) {
			#
			# find eof mark,
			# for UTLPRT this is '--- current write position --'
			#
			if ($line =~ /^\-+\scurrent\swrite\sposition\s\-+/) {
				$dbmgetf->{'goteof'} = 1;
				return $line;
			}

			#
			# stop reading if  we got eof mark
			# and buffer contains only whitespace
			#
			if ($dbmgetf->{'goteof'} == 1 && $line =~ /^\s+$/) {
				return undef;
			}
		} elsif ($fileid =~ /KNLDIAG/) {
			#
			# stop reading if we got something other then
			# nonwhitespace normaly a line starts with
			# date and time or
			# timestamp in seconds sine 1970 or
			# separators like '---' or '==='
			#
			# we stop now if there is an empty line
			# because sometimes unformatted text will be found
			# in 7.4 and move error causes sqlabort (),
			# which is code from the client runtime and does not case
			# about knldiag output format
			#
			unless ($line =~ /\S/) {
				return undef;
			}
		}
		return $line;
	}

	#
	# are still data available from dbmsrv
	# 
	unless ($dbmgetf->{'cont'} =~ /^CONTINUE/) {
		return undef;
	}

	#
	# try to get next chunk of data from dbmsrv 
	#
	my $cmd = 'file_getnext';
	$cmd .= ' '.$dbmgetf->{'fileid'}.' '.$dbmgetf->{'handle'};

	my $raw = $dbmgetf->{'dbm'}->exec_ex ($dbmgetf->{'force_ascii'}, $cmd);
	
	#
	# split return code
	# 
	my $rc = ($raw =~ s/^(.+)\n//) ? $1 : undef;
	return undef unless ($rc =~ /^OK/);
	
	#
	# split other lines of data
	#
	$dbmgetf->{'handle'} = ($raw =~ s/^(.+)\n//) ? $1 : undef;
	$dbmgetf->{'cont'} = ($raw =~ s/^(.+)\n//) ? $1 : undef;
	$raw =~ s/^(.+)\n//;
		
	#
	# append txet
	#
	$dbmgetf->{'txt'} .= $raw;

	#
	# try to read line from buffer again
	#
	$line= ($dbmgetf->{'txt'} =~ s/^(.+)\n//) ? $1 : undef;
	if (defined $line) {
		#
		# remove any 'cr', I've found them even on UNIX in 'dbm.utl'
		#
		$line =~ s/\r//g;	

		if ($fileid =~ /UTLPRT/) {
			#
			# find eof mark,
			# for UTLPRT this is '--- current write position --'
			#
			if ($line =~ /^\-+\scurrent\swrite\sposition\s\-+/) {
				$dbmgetf->{'goteof'} = 1;
				return $line;
			}

			#
			# stop reading if  we got eof mark
			# and buffer contains only whitespace
			#
			if ($dbmgetf->{'goteof'} == 1 && $line =~ /^\s+$/) {
				return undef;
			}
		} elsif ($fileid =~ /KNLDIAG/) {
			#
			# stop reading if we got something other then
			# nonwhitespace normaly a line starts with
			# date and time or
			# timestamp in seconds sine 1970 or
			# separators like '---' or '==='
			#
			unless ($line =~ /^\S/) {
				return undef;
			}
		}
		return $line;
	}
	return undef;
}

sub close {
	my ($dbmgetf) = @_;
	
	delete $dbmgetf->{'handle'} if (defined $dbmgetf->{'handle'});
	delete $dbmgetf->{'cont'} if (defined $dbmgetf->{'cont'});
	delete $dbmgetf->{'txt'} if (defined $dbmgetf->{'txt'});
	delete $dbmgetf->{'dbm'} if (defined $dbmgetf->{'dbm'});
	delete $dbmgetf->{'fileid'} if (defined $dbmgetf->{'fileid'});
	delete $dbmgetf->{'goteof'} if (defined $dbmgetf->{'goteof'});
	return;
}

sub DESTROY {
	my ($self) = @_;
	$self->close ();
}

package SAPDB::Install::DBMCmd;

#
# do a dbm command 'db_enum'
# return hash reference or undef in case of error
# 
sub db_enum {
	my ($dbm) = @_;
	my ($rc, @data) = split ('\n', $dbm->exec ('db_enum'));
	return undef
		unless ($rc =~ /^OK/);

	my %out;
	my @colnames = ('DBNAME', 'PATH', 'VERSION', 'SPEED', 'STATE');
	foreach my $col (@colnames) {
		$out{$col} = [];
	}
	
	foreach my $line (@data) {
		my (@values) = split ('\t', $line);

		my $i = 0;
		foreach my $col (@colnames) {
			push @{$out{$col}}, $values[$i];
			$i++;
		}
	}

	return (\%out);
}

#
# do a dbm command 'db_register'
# 
sub db_register {
	my ($dbm) = @_;
	my $rc;

	($rc) = split ('\n', $dbm->exec ("db_register"));
	return undef
		unless ($rc =~ /^OK/);
	
	return ($rc);
}

#
# do a dbm command 'util_execute'
# 
sub util_execute {
	my ($dbm, $cmd) = @_;
	my $rc;

	$cmd = '' unless (defined $cmd);
	($rc) = split ('\n', $dbm->exec ("util_execute ".$cmd));
	return undef
		unless ($rc =~ /^OK/);
	
	return ($rc);
}

#
# do a dbm command 'user_getall'
# return reference to list of users
# or empty array in case of error
#
sub user_getall {
	my ($dbm) = @_;

	my $users = [];
	my ($rc, @data) = split ('\n', $dbm->exec ("user_getall"));
	return undef
		unless ($rc =~ /^OK/);
	
	push @{$users}, @data;
	
	return ($users);
}

#
# do a dbm command 'user_get'
# return reference to hash
# or undef in case of error
#
sub user_get {
	my ($dbm, $user) = @_;

	my $attrib = ();

	my ($rc, @data) = split ('\n', $dbm->exec ("user_get ".$user));
	return undef
		unless ($rc =~ /^OK/);
	
	foreach my $line (@data) {
		my ($key, $val) = ($line =~ /^(\w+)=(\w*)/);

		if (defined $val && $val ne "") {
			$attrib->{$key} = $val;
		} else {
			$attrib->{$key} = "";
		}
	}
	return ($attrib);
}

#
# do a dbm command sql_execute
#
sub sql_execute {
	my ($dbm, $cmd) = @_;
	my $sql = ();
	
	my ($rc, $err, @data) = split ('\n', $dbm->exec ("sql_execute ".$cmd));

	if ($rc eq 'ERR' && $err =~ ERR_SQL) {
		my @errc = split (',', $data[0]);
		$sql->{error} = \@errc;
		$sql->{result} = undef;
		return ($sql);
	}

	if ($rc eq 'OK') {
		$sql->{error} = undef;
		$sql->{result} = \@data;
		return ($sql);
	}

	return undef; 
}

#
# do a dbm command exec_lcinit
#
sub exec_lcinit {
	my ($dbm, $cmd) = @_;
	my $sql = ();
	
	my ($rc, @data) = split ('\n', $dbm->exec ("exec_lcinit ".$cmd));

	return undef
		unless ($rc =~ /^OK/);

	return ($rc); 
}

#
# do a dbm command db_migratecatalog
#
sub db_migratecatalog {
	my ($dbm) = @_;

	my ($rc, @data) = split ('\n', $dbm->exec ("db_migratecatalog"));

	return undef
		unless ($rc =~ /^OK/);

	return ($rc);
}
