/*
 * $SRC/install/perl/DBMCmd.xs
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/DBMCmd.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/DBMCmd.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#ifdef __hpux
#define DONT_DECLARE_STD 1
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#ifdef UNIX
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#endif

#if defined _WIN64
typedef __int64 ptr_t;
#else
typedef long ptr_t;
#endif

#if defined WIN32
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#define NEED_TYPEDEF_UNIT16_T NEED_TYPEDEF_UNIT16_T
#define NEED_TYPEDEF_UNIT32_T NEED_TYPEDEF_UNIT32_T
#define NEED_TYPEDEF_UNIT64_T NEED_TYPEDEF_UNIT64_T
#elif defined linux
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __hpux && defined __hppa
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __hpux && defined __ia64 && ! defined HAS_STRTOLL
#define NEED_TYPEDEF_UCHAR_T NEED_TYPEDEF_UCHAR_T
#elif defined __osf__
#define NEED_TYPEDEF_UNIT16_T NEED_TYPEDEF_UNIT16_T
#define NEED_TYPEDEF_UNIT32_T NEED_TYPEDEF_UNIT32_T
#define NEED_TYPEDEF_UNIT64_T NEED_TYPEDEF_UNIT64_T
#define NEED_SNPRINTF NEED_SNPRINTF
#define HAS_SPRINTF_INTRINSIC HAS_SPRINTF_INTRINSIC
#endif

#ifdef NEED_TYPEDEF_UCHAR_T
typedef unsigned char  uchar_t;
#endif
#ifdef NEED_TYPEDEF_UNIT16_T
typedef unsigned short uint16_t;
#endif
#ifdef NEED_TYPEDEF_UNIT32_T
typedef unsigned int   uint32_t;
#endif
#ifdef NEED_TYPEDEF_UNIT64_T
typedef unsigned long  uint64_t;
#endif

/*
 * our internal dbm session structure
 */
typedef struct dbmobj {
	int fd;
	uint32_t theirref;
	int id;
	int inuse;
	char *sz_host;
	char *sz_dbname;
	char *sz_dir;
	char *sz_logoncmd;
	struct dbmobj *next;
	uchar_t inbuff[16 * 1024];
	uchar_t outbuff[16 * 1024];
} dbmobj_t;

/*
 * rte header
 * total 24 byte
 */
typedef struct {
	uint32_t actlen;
	uchar_t  protocol;
	uchar_t  messclass;
	uchar_t  flags;
	uchar_t  trailpkg;
	uint32_t sendref;
	uint32_t recvref;
	uint16_t rc;
	uchar_t  swaptype;
	uchar_t  reserved;
	uint32_t maxlen;
} sapdbcomm_rtehdr_t;

/*
 * minimal part of connect packet,
 * additional len type value encoded data can follow
 * total 40 + options
 */
typedef struct {
	uchar_t  charcoding;
	uchar_t  swapmode;
	uint16_t len;
	uchar_t  service;
	uchar_t  ostype;
	uint16_t padding;
	uint32_t max_segment_size;
	uint32_t max_data_size;
	uint32_t packet_size;
	uint32_t min_reply_size;
	uchar_t  dbname[8];
	uchar_t  reserved[8];
} sapdbcomm_conhdr_t;

#define SAPDBCOMM_RTE_PROTO_TCP         3

#define SAPDBCOMM_RTE_SWAPTYPE_BE       3
#define SAPDBCOMM_RTE_SWAPTYPE_LE       4

#define SAPDBCOMM_RTE_MESS_PINGREQ     53
#define SAPDBCOMM_RTE_MESS_PINGREP     54
#define SAPDBCOMM_RTE_MESS_USERDATAREQ 63
#define SAPDBCOMM_RTE_MESS_USERDATAREP 64
#define SAPDBCOMM_RTE_MESS_DBMCONREQ   91
#define SAPDBCOMM_RTE_MESS_DBMCONREP   92

#define SAPDBCOMM_CON_OPTION_VERSION  'V'
#define SAPDBCOMM_CON_OPTION_PID      'I'
#define SAPDBCOMM_CON_OPTION_DIR      'd'
#define SAPDBCOMM_CON_OPTION_EXE      'p'

#define SAPDBCOMM_CON_CHAR_ASCII        0

#define SAPDBCOMM_CON_SWAPTYPE_BE       1
#define SAPDBCOMM_CON_SWAPTYPE_LE       2

#define SAPDBCOMM_CON_SERVICE_DBM       4

#define SAPDBCOMM_CON_OSTYPE_UNIX      11 
#define SAPDBCOMM_CON_OSTYPE_WIN32     13 

#define SAPDBCOMM_DBM_PACKET_SIZE (16 * 1024)
#define SAPDBCOMM_DBM_MINREP_SIZE       (112) 
#define SAPDBCOMM_DBM_MAXDATA_SIZE \
	(SAPDBCOMM_DBM_PACKET_SIZE - 2 * sizeof (sapdbcomm_rtehdr_t))

#if HAS_BYTEORDER == 4321 || HAS_BYTEORDER == 87654321
static const int my_rtehdr_swaptype = SAPDBCOMM_RTE_SWAPTYPE_BE;
static const int my_conhdr_swapmode = SAPDBCOMM_CON_SWAPTYPE_BE;
#else
static const int my_rtehdr_swaptype = SAPDBCOMM_RTE_SWAPTYPE_LE;
static const int my_conhdr_swapmode = SAPDBCOMM_CON_SWAPTYPE_LE;
#endif

#ifdef UNIX
static const int my_conhdr_ostype = SAPDBCOMM_CON_OSTYPE_UNIX;
#endif
#ifdef WIN32
static const int my_conhdr_ostype = SAPDBCOMM_CON_OSTYPE_WIN32;
#endif

static int tcp_connect (char *, char *, char *);
static int tcp_release (int, char *);
static dbmobj_t *sess_connect (dbmobj_t*, char *, char *, char *, char *);
static int sess_release (dbmobj_t *, char *);
static int sess_exec (dbmobj_t *, char *, char *, char *);
static int sess_exec_ex (dbmobj_t *, int, char *, char *, char *);
static int sess_conopt_addstring (uchar_t *, int, char *);
static void swap_rtehdr (sapdbcomm_rtehdr_t *);
static void swap_uint32 (void *);
static void swap_uint16 (void *);

/*
 * you can use DEBUG_RTEHDR to dump your rtehdr
 * if you completely do not know what's 
 * wrong in sapdb communication
 */
#ifdef DEBUG_RTEHDR
static void printf_rtehdr (sapdbcomm_rtehdr_t * rtehdr) {
	printf ("rtehdr->actlen=%d\n", rtehdr->actlen);
	printf ("rtehdr->maxlen=%d\n", rtehdr->maxlen);
	printf ("rtehdr->sendref=%d\n", rtehdr->sendref);
	printf ("rtehdr->recvref=%d\n", rtehdr->recvref);
	printf ("rtehdr->swaptype=%d\n", rtehdr->swaptype);
	printf ("rtehdr->protocol=%d\n", rtehdr->protocol);
	printf ("rtehdr->messclass=%d\n", rtehdr->messclass);
}
#endif

/*
 * some tcp special communiation code
 */
static int
tcp_release (int fd, char *sz_err) {
	int rc;

	rc = close (fd);
	if (rc < 0) {
		sprintf (sz_err, "close failed");
	}
	return 0;
}

static int
tcp_connect (char *sz_host, char *sz_service, char *sz_err) {
	int fd;
	int rc;
	struct sockaddr sa[1];
	struct sockaddr_in *sa_in = (struct sockaddr_in *) sa;
	struct servent *se;
	struct hostent *he;

	fd = socket (PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fd < 0) {
		sprintf (sz_err, "socket failed");
		return -1;
	}

	se = getservbyname (sz_service, "tcp");
	if (se == 0) {
		sprintf (sz_err, "getservbyname failed");
		close (fd);
		return -1;
	}

	he = gethostbyname (sz_host);
	if (he == 0) {
		sprintf (sz_err, "gethostbyname failed");
		close (fd);
		return -1;
	}

	sa->sa_family = AF_INET;
	sa_in->sin_port = se->s_port;
	memcpy (&sa_in->sin_addr, &he->h_addr[0], sizeof (sa_in->sin_addr));

	rc = connect (fd, sa, sizeof (struct sockaddr));
	if (rc < 0) {
#if defined WIN32
		if (WSAGetLastError () == WSAECONNREFUSED) {
#else
		if (errno == ECONNREFUSED) {
#endif
			sprintf (sz_err, 
			"connect failed: Connection refused, "
			"probably no vserver running\n"); 
		} else {
			sprintf (sz_err, 
#ifdef WIN32
			"connect failed\n");
#else
			"connect failed: %s\n",Strerror(errno));
#endif
		}
		close (fd);
		return -1;
	}
	
	return fd;
}

/*
 * some sapdb session communication code
 */
static dbmobj_t *
sess_connect (
dbmobj_t *obj, char *sz_host, char *sz_dbname, char * sz_dir, char *sz_err) {
	unsigned int rc;
	uchar_t pidbuff[64];
	uchar_t buff[1024];
	sapdbcomm_rtehdr_t *rtehdr;
	sapdbcomm_conhdr_t *conhdr;

	if (obj == 0) {
		obj = (dbmobj_t *) malloc (sizeof (dbmobj_t));
		memset (obj, 0, sizeof (dbmobj_t));
	}

	if (obj == 0) {
		strcpy (sz_err, "malloc for dbm session object failed");
		return 0;
	}

	/*
	 * no next pointer in new struct, not in use
	 */
	obj->next = 0;
	obj->inuse = 0;

	/*
	 * establish tcp connection
	 */
	obj->fd = tcp_connect (sz_host, "sql6", sz_err);
	if (obj->fd < 0) {
		obj->fd = -1;
		return (obj);
	}

	/*
	 * make layout of sapdb-comm-connect-packet
	 */
	rtehdr = (sapdbcomm_rtehdr_t *) buff;
	conhdr = (sapdbcomm_conhdr_t *) (rtehdr + 1);

	/*
 	 * fill in rte-header
	 */
	rtehdr->actlen    = sizeof (sapdbcomm_rtehdr_t);
	rtehdr->protocol  = SAPDBCOMM_RTE_PROTO_TCP;
	rtehdr->messclass = SAPDBCOMM_RTE_MESS_DBMCONREQ;
	rtehdr->flags     = 0;
	rtehdr->trailpkg  = 0;
	rtehdr->sendref   = 1;
	rtehdr->recvref   = 0;
	rtehdr->rc        = 0;
	rtehdr->swaptype  = my_rtehdr_swaptype;
	rtehdr->reserved  = 0;
	rtehdr->maxlen    = sizeof (sapdbcomm_rtehdr_t);
	
	/*
	 * fill in connect-packet-header for dbmsrv
	 */
	conhdr->charcoding = SAPDBCOMM_CON_CHAR_ASCII;
	conhdr->swapmode = my_conhdr_swapmode;
	conhdr->len = sizeof (sapdbcomm_conhdr_t);
	conhdr->service = SAPDBCOMM_CON_SERVICE_DBM;
	conhdr->ostype = my_conhdr_ostype;
	conhdr->padding = 0;
	conhdr->packet_size = SAPDBCOMM_DBM_PACKET_SIZE;
	conhdr->max_data_size = SAPDBCOMM_DBM_MAXDATA_SIZE;
	conhdr->max_segment_size = SAPDBCOMM_DBM_MAXDATA_SIZE;
	conhdr->min_reply_size = SAPDBCOMM_DBM_MINREP_SIZE;
	memset (conhdr->dbname, ' ', sizeof (conhdr->dbname));
	memset (conhdr->reserved, ' ', sizeof (conhdr->reserved));
	if (sz_dbname != 0 && strlen (sz_dbname) != 0) {
		memcpy (conhdr->dbname, sz_dbname, strlen (sz_dbname));
	}
	rtehdr->actlen += sizeof (sapdbcomm_conhdr_t);
	rtehdr->maxlen += sizeof (sapdbcomm_conhdr_t);

	/*
	 * fill in options for dbmsrv
	 */
	sprintf ((char *) pidbuff, "%d", getpid());	
	sess_conopt_addstring (
		buff, SAPDBCOMM_CON_OPTION_PID, (char *) pidbuff);
	sess_conopt_addstring (
		buff, SAPDBCOMM_CON_OPTION_EXE, "dbmsrv");
	if (sz_dir != 0 && strlen (sz_dir) != 0) {
		sess_conopt_addstring (
			buff, SAPDBCOMM_CON_OPTION_DIR, (char *) sz_dir);
	}

#ifdef DEBUG_RTEHDR
	printf ("send connect pkt\n");
	printf_rtehdr (rtehdr);
	printf ("\n");
#endif
	/*
	 * send it to server
	 */	
	rc = write (obj->fd, buff, rtehdr->actlen);
	if (rc != rtehdr->actlen) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "write () failed");
		obj->fd = -1;
		return (obj);
	}	

	/*
	 * wait for reply
	 */
	rc = read (obj->fd, buff, 1024);
	if (rc < 0) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "read () failed");
		obj->fd = -1;
		return (obj);
	}

	/*
	 * make sure that binary values are readable
	 */
	if (rtehdr->swaptype != my_rtehdr_swaptype) {
		swap_rtehdr (rtehdr);
	}

#ifdef DEBUG_RTEHDR
	printf ("received connect pkt\n");
	printf_rtehdr (rtehdr);
	printf ("\n");
#endif
	if (rtehdr->messclass != SAPDBCOMM_RTE_MESS_DBMCONREP) {
		tcp_release (obj->fd, sz_err);
		sprintf (sz_err, "connect reply expected, got %d",
		         rtehdr->messclass);
		obj->fd = -1;
		return (obj);
	}

	if (rtehdr->rc != 0) {
		tcp_release (obj->fd, sz_err);
		if (rtehdr->rc == 13) {
			sprintf (sz_err, 
			"connect: cannot start dbmsrv for this database");
		} else {
			sprintf (sz_err, "connect rtehdr->rc=%d", rtehdr->rc);
		}
		obj->fd = -1;
		return (obj);
	}

	obj->theirref = rtehdr->sendref;

	/*
	 * save session parameter for additional sessions
	 */
	obj->sz_host = (char *) malloc (strlen (sz_host) + 1);
	strcpy (obj->sz_host, sz_host);

	obj->sz_dbname = (char *) malloc (strlen (sz_dbname) + 1);
	strcpy (obj->sz_dbname, sz_dbname);

	obj->sz_dir = (char *) malloc (strlen (sz_dir) + 1);
	strcpy (obj->sz_dir, sz_dir);

	/*
	 * not yet loged on
	 */
	obj->sz_logoncmd = 0;

	return (obj);
}

static int
sess_release (dbmobj_t *obj, char *sz_err) {
	int rc = 0;
	
	if (obj == 0)
		return (rc);

	/*
	 * free other connections
	 */
	if (obj->next != 0) {
		sess_release (obj->next, sz_err);
		obj->next = 0;
	}
	
	/*
	 * free own connection
	 */
	if (obj->fd != -1) {
		sess_exec (obj, "release", 0, sz_err);
		rc = tcp_release (obj->fd, sz_err); 
	}

	if (obj->sz_host != 0) {
		free (obj->sz_host);
	}

	if (obj->sz_dbname != 0) {
		free (obj->sz_dbname);
	}

	if (obj->sz_dir != 0) {
		free (obj->sz_dir);
	}

	if (obj->sz_logoncmd != 0) {
		free (obj->sz_logoncmd);
	}

	free (obj);

	return (rc);	
}

static int
sess_send (dbmobj_t *obj, char *data_in, char *sz_err) {
	sapdbcomm_rtehdr_t *rtehdr;
	uchar_t *container;
	int len_in;
	int len_in_aligned;
	unsigned int rc;

	/*
	 * make layout of sapdb-comm-packet
	 */
	rtehdr = (sapdbcomm_rtehdr_t *) obj->inbuff;
	container = (uchar_t *)(rtehdr + 1);

	/*
 	 * fill in rte-header
	 */
	rtehdr->actlen    = sizeof (sapdbcomm_rtehdr_t);
	rtehdr->protocol  = SAPDBCOMM_RTE_PROTO_TCP;
	rtehdr->messclass = SAPDBCOMM_RTE_MESS_USERDATAREQ;
	rtehdr->flags     = 0;
	rtehdr->trailpkg  = 0;
	rtehdr->sendref   = 1;
	rtehdr->recvref   = obj->theirref;
	rtehdr->rc        = 0;
	rtehdr->swaptype  = my_rtehdr_swaptype;
	rtehdr->reserved  = 0;
	rtehdr->maxlen    = sizeof (sapdbcomm_rtehdr_t);

	/*
	 * calculate aligned len
	 */
	len_in = strlen (data_in) + 1;
	len_in_aligned = len_in;
	if ((len_in_aligned & 7) != 0) {
		len_in_aligned += 8;
		len_in_aligned &= ~7;
	}
	
	/*
	 * fill in data into container
	 * content must be an 8-byte aligned amount of data
	 */
	memcpy (container, data_in, len_in);
	if (len_in < len_in_aligned) {
		memset (container + len_in, '\0', len_in_aligned - len_in);
	}
	
	rtehdr->actlen    += len_in_aligned;
	rtehdr->maxlen    += len_in_aligned;

#ifdef DEBUG_RTEHDR
	printf ("send data pkt\n");
	printf_rtehdr (rtehdr);
	printf ("\n");
#endif

	rc = write (obj->fd, obj->inbuff, rtehdr->actlen);
	if (rc != rtehdr->actlen) {
		sprintf (sz_err, "write() failed\n");
		return -1;
	}

	return 0;
}

static int
sess_recv (dbmobj_t *obj, char *data_out, char *sz_err) {
	sapdbcomm_rtehdr_t *rtehdr;
	uchar_t *container;
	uchar_t *ptr; 
	int len_got;
	int len_out;
	int rc;

	/*
	 * make layout of sapdb-comm-packet
	 */
	rtehdr = (sapdbcomm_rtehdr_t *) obj->outbuff;
	container = (uchar_t *)(rtehdr + 1);

	/*
	 * wait for reply
	 */
	rc = read (obj->fd, obj->outbuff, sizeof (sapdbcomm_rtehdr_t));
	if (rc < 0) {
		close (obj->fd);
		obj->fd = -1;
	}

	if (rc != sizeof (sapdbcomm_rtehdr_t)) {
		sprintf (sz_err, "read () for rtehdr failed");
		return -1;
	}
	len_got = rc - sizeof (sapdbcomm_rtehdr_t);
	ptr = container + len_got;

#ifdef DEBUG_RTEHDR
	printf ("received data pkt\n");
	printf_rtehdr (rtehdr);
	printf ("\n");
#endif
	/*
	 * make sure that binary values are readable on our system
	 */
	if (rtehdr->swaptype != my_rtehdr_swaptype) {
		swap_rtehdr (rtehdr);
	}
	len_out = rtehdr->actlen - sizeof (sapdbcomm_rtehdr_t);

	/*
	 * there may be futher tcp-fragments waiting
	 */
	container[0] = '\0';
	while (len_got < len_out) {
		rc = read (obj->fd, ptr, len_out - len_got);
		if (rc < 0) {
			sprintf (sz_err, "read () for data failed");
			close (obj->fd);
			obj->fd = -1;
			return -1;
		}
		
		len_got += rc;
		ptr += rc;
		container[len_got] = '\0';
	}

	/*
	 * nobody wants to know the answer 
	 */
	if (data_out == 0)
		return 0;

	/*
	 * return a zero terminated string
	 */
	memcpy (data_out, (char *) container, len_got + 1);
	return (len_got + 1);
}

static int
sess_exec (dbmobj_t *obj, char *data_in, char *data_out, char *sz_err) {
	return (sess_exec_ex (obj, 0, data_in, data_out, sz_err));
}

static int
sess_exec_ex (dbmobj_t *obj, int force_ascii, char *data_in, char *data_out, char *sz_err) {
	int rc;
	int i;
	const char cmd_userlogon[] = "user_logon";

	/*
	 * save logon cmd for setup additional sessions
	 */
	if (memcmp (data_in, cmd_userlogon, strlen (cmd_userlogon)) == 0) {
		if (obj->sz_logoncmd != 0)
			free (obj->sz_logoncmd);
		
		obj->sz_logoncmd = (char *) malloc (strlen (data_in) + 1);
		strcpy (obj->sz_logoncmd, data_in);
	}

	rc = sess_send (obj, data_in, sz_err);
	if (rc < 0)
		return (rc);

	rc = sess_recv (obj, data_out, sz_err);
	if (rc < 0)
		return (rc);

	if (force_ascii == 0)
		return 0;

	for (i = 0; i < rc - 1; i++) {
		if (data_out[i] >= ' ')
			continue;

		if (data_out[i] == '\n')
			continue;

		if (data_out[i] == '\r')
			continue;

		if (data_out[i] == '\t')
			continue;

		data_out[i] = ' ';
	}

	return 0;
}

static int
sess_conopt_addstring (uchar_t *pkt, int type, char *sz_text) {
	int pos;
	int len;
	sapdbcomm_rtehdr_t *rtehdr;
	sapdbcomm_conhdr_t *conhdr;

	/*
	 * get layout of conpkt
	 */
	rtehdr = (sapdbcomm_rtehdr_t *) pkt;
	conhdr = (sapdbcomm_conhdr_t *) (rtehdr + 1);
 
	/*
	 * add string on the end of packet
	 * a length, type, data encoding is used
	 * strings are represented as ascii with terminating zero
	 * length includes the terminating zero
	 */
	pos = rtehdr->actlen;
	len = strlen (sz_text) + 3;
	pkt[pos] = (uchar_t) len;	
	pkt[pos + 1] = (uchar_t) type;	
	memcpy (pkt + pos + 2, sz_text, strlen (sz_text) + 1); 

	/*
	 * fix len fields of packet
	 */
	rtehdr->actlen += len;
	rtehdr->maxlen += len; 
	conhdr->len += len; 
	return 0;
}

/*
 * helper functions for swapping binary int's in rtehdr
 */
static void 
swap_rtehdr (sapdbcomm_rtehdr_t *rtehdr) {
	swap_uint32 (&rtehdr->actlen);	
	swap_uint32 (&rtehdr->sendref);	
	swap_uint32 (&rtehdr->recvref);	
	swap_uint32 (&rtehdr->maxlen);	
	swap_uint16 (&rtehdr->rc);	
	rtehdr->swaptype = my_rtehdr_swaptype;
}

static void
swap_uint16 (void *val) {
	uchar_t *byte = (uchar_t *) val;
	uchar_t tmp;

	tmp = byte[1];
	byte[1] = byte[0];
	byte[0] = tmp;
}

static void
swap_uint32 (void *val) {
	uchar_t *byte = (uchar_t *) val;
	uchar_t tmp;

	tmp = byte[3];
	byte[3] = byte[0];
	byte[0] = tmp;
	
	tmp = byte[2];
	byte[2] = byte[1];
	byte[1] = tmp;
}

/*
 * helper functions to access member variables
 */
static dbmobj_t *
self_getobj (HV *self) {
	char *key_obj = "obj";
	SV **psv_obj;
	dbmobj_t *obj;

	psv_obj = hv_fetch (self, key_obj, strlen (key_obj), 0);
	if (psv_obj == 0)
		return 0;

	obj = (dbmobj_t *) SvIV (*psv_obj);
	return (obj);
}

static SV *
self_getsverr (HV *self) {
	char *key_err = "err";
	SV **psv_err;

	psv_err = hv_fetch (self, key_err, strlen (key_err), 0);
	if (psv_err == 0)
		return 0;

	return (*psv_err);
}

/*
 * now we add the perl source as as inline code
 * symbol name of the string containing the code
 * is sz_text
 * this code will be evaled in the BOOT method of the module 
 */
#include "DBMCmd.h"

MODULE = SAPDB::Install::DBMCmd       PACKAGE = SAPDB::Install::DBMCmd

PROTOTYPES: DISABLE

BOOT:
	perl_eval_pv ((char *) sz_text, 1);

void
new (...)
PREINIT:
	char *CLASS;
	HV *self;
	SV *sv_obj;
	SV *sv_err;
	char *key_obj = "obj";
	char *key_err = "err";
	dbmobj_t *obj;
	char *sz_host = "localhost";
	char *sz_dbname = "";
	char *sz_dir = "";
	char sz_err[1024];
CODE:
	if (items >= 1) {
		if (strlen (SvPV (ST(0), PL_na)) > 0)
			CLASS = SvPV (ST(0), PL_na);
	}
	if (items >= 2) {
		if (strlen (SvPV (ST(1), PL_na)) > 0)
			sz_host = SvPV (ST(1), PL_na);
	}
	if (items >= 3) {
		if (strlen (SvPV (ST(2), PL_na)) > 0)
			sz_dbname = SvPV (ST(2), PL_na);
	}
	if (items == 4) {
		if (strlen (SvPV (ST(3), PL_na)) > 0)
			sz_dir = SvPV (ST(3), PL_na);
	}
	if (items >= 5 || items == 0) {
		XSRETURN_UNDEF;
	}
	
	self = newHV ();
	ST(0) = sv_bless (newRV ((SV*) self), gv_stashpv (CLASS, 1));
	obj = sess_connect (0, sz_host, sz_dbname, sz_dir, sz_err);
	obj->id = 1;

	if (obj == 0 || obj->fd == -1) {
		sv_err = newSVpv (sz_err, strlen (sz_err));
	} else {
		sv_err = newSVsv (&PL_sv_undef);
	}
	hv_store (self, key_err, strlen (key_err), sv_err, 0);

	sv_obj = newSViv ((ptr_t) obj);
	hv_store (self, key_obj, strlen (key_obj), sv_obj, 0);

	sv_2mortal (ST(0));
	SvREFCNT_dec (self);

void
DESTROY (...)
PREINIT:
	HV *self;
	dbmobj_t *obj;
	char sz_err[1024];
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	sess_release (obj, sz_err);
	XSRETURN_UNDEF;

void
release (...)
PREINIT:
	HV *self;
	dbmobj_t *obj;
	SV **psv;
	char *key = "obj";
	char sz_err[1024];
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));

	psv = hv_fetch (self, key, strlen (key), 0);
	if (psv == 0)
		XSRETURN_UNDEF;

	obj = (dbmobj_t *) SvIV (*psv);
	if (obj == 0)
		XSRETURN_UNDEF;

	sess_release (obj, sz_err);
	sv_setiv (*psv, 0);

	XPUSHs (sv_2mortal (newSViv (0)));
	XSRETURN (1);

void
send (...)
PREINIT:
	HV *self;
	dbmobj_t *root;
	dbmobj_t *obj;
	int rc;
	char *cmd;
	char sz_err[1024];
	SV *sv_err;
PPCODE:
	if (items != 2)
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));

	root = self_getobj (self);
	if (root == 0)
		XSRETURN_UNDEF;

	sv_err = self_getsverr (self);
	cmd = SvPV (ST(1), PL_na);

	obj = root;
	for (;;) {
		if (obj->inuse == 0)
			break;
		
		if (obj->next == 0) {
			char msg[16 * 1024];		
			const char okmsg[] = "OK\n";			

			obj->next = sess_connect (0,
			root->sz_host, root->sz_dbname, root->sz_dir, sz_err);
			obj->next->id = obj->id + 1;
			obj = obj->next;

			if (root->sz_logoncmd == 0)
				break;

			rc = sess_exec (obj, root->sz_logoncmd, msg, sz_err);
			if (rc != 0) {
				XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
				XSRETURN (1);
			}

			if (memcmp (msg, okmsg, strlen (okmsg)) != 0) {
				strcpy (sz_err, "user_logon failed");
				XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
				XSRETURN (1);
			}

			break;
		}

		obj = obj->next;
	}

	obj->inuse = 1;

	*((char *)(obj->outbuff) + sizeof (sapdbcomm_rtehdr_t)) = '\0';
	rc = sess_send (obj, cmd, sz_err);
	if (rc != 0) {
		XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
	} else {
		XPUSHs (sv_2mortal (newSViv (obj->id)));
	}
	XSRETURN (1);

void
recv (...)
PREINIT:
	HV *self;
	dbmobj_t *root;
	dbmobj_t *obj;
	int rc;
	int id;
	char sz_err[1024];
	char msg[16 * 1024];		
	SV *sv_err;
PPCODE:
	if (items != 2)
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));

	root = self_getobj (self);
	if (root == 0)
		XSRETURN_UNDEF;

	sv_err = self_getsverr (self);
	id = SvIV (ST(1));

	for (obj = root; obj != 0; obj = obj->next) {
		if (obj->id == id) break;
	}

	if (obj->id != id) {
		sv_setpv (sv_err, "invalid session id");
		XSRETURN_UNDEF;
	}

	if (obj->inuse == 0) {
		sv_setpv (sv_err, "no request pending");
		XSRETURN_UNDEF;
	}

	rc = sess_recv (obj, msg, sz_err);
	obj->inuse = 0;

	if (rc < 0) {
		XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
	} else {
		XPUSHs (sv_2mortal (newSVpv (msg, strlen (msg))));
	}
	XSRETURN (1);

void
select (...)
PREINIT:
	HV *self;
	dbmobj_t *root;
	dbmobj_t *obj;
	int rc;
	int timeout;
	int forever;
	fd_set fds;
	int fd_max;
	SV *sv_err;
PPCODE:
	if (items < 2)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));

	root = self_getobj (self);
	if (root == 0)
		XSRETURN_UNDEF;

	sv_err = self_getsverr (self);

	if (items == 1) {
		timeout = 0;
		forever = 1;
	}

	if (items == 2) {
		timeout =  SvIV (ST(1));
		forever = 0;
	}

	FD_ZERO (&fds);
	fd_max = 0;

	for (obj = root; obj != 0; obj = obj->next) {
		if (obj->inuse != 0) {
			FD_SET (obj->fd, &fds);
			fd_max = (obj->fd > fd_max) ? obj->fd : fd_max;
		}
	}

	if (fd_max == 0) {
		XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
		XSRETURN (1);
	}

	if (forever != 0) {
		rc = select (fd_max + 1, &fds, 0, 0, 0);
	} else {
		struct timeval tv;

		tv.tv_sec = timeout;
		tv.tv_usec = 0;
		rc = select (fd_max + 1, &fds, 0, 0, &tv);
	}

	switch (rc) {
	case 0:
		XPUSHs (sv_2mortal (newSViv (0)));
		XSRETURN (1);
		break;
	case (-1):
		XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
		XSRETURN (1);
		break;
	}	
	
	for (obj = root; obj != 0; obj = obj->next) {
		if (FD_ISSET (obj->fd, &fds)) {
			XPUSHs (sv_2mortal (newSViv (obj->id)));
			XSRETURN (1);
		}
	}

	XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
	XSRETURN (1);

void
cancel (...)
PREINIT:
	HV *self;
	dbmobj_t *root;
	dbmobj_t *obj;
	int rc;
	int id;
	char sz_err[1024];
	char msg[16 * 1024];		
	SV *sv_err;
	char *sz_logoncmd;
	char *sz_cmd;
	char *cmd;
PPCODE:
	if (items != 2)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));

	root = self_getobj (self);
	if (root == 0)
		XSRETURN_UNDEF;

	sv_err = self_getsverr (self);
	id = SvIV (ST(1));

	for (obj = root; obj != 0; obj = obj->next) {
		if (obj->id == id)
			break;
	}

	if (obj->id != id) {
		sv_setpv (sv_err, "invalid session id");
		XSRETURN_UNDEF;
	}

	if (obj->inuse == 0) {
		sv_setpv (sv_err, "no request pending");
		XSRETURN_UNDEF;
	}

	sz_logoncmd = 0;
	if (obj->sz_logoncmd != 0) {
		sz_logoncmd = (char *) malloc (strlen (obj->sz_logoncmd) + 1);
		strcpy (sz_logoncmd, obj->sz_logoncmd);
	}

	cmd = (char *)(obj->inbuff) + sizeof (sapdbcomm_rtehdr_t);
	sz_cmd = (char *) malloc (strlen (cmd) + 1);
	strcpy (sz_cmd, cmd);
	
	tcp_release (obj->fd, sz_err);
	obj->fd = -1;

	obj = 
	sess_connect (obj, obj->sz_host, obj->sz_dbname, obj->sz_dir, sz_err);

	obj->id = id;
	obj->inuse = 0;

	if (sz_logoncmd != 0) {
		const char okmsg[] = "OK\n";			

		rc = sess_exec (obj, sz_logoncmd, msg, sz_err);
		free (sz_logoncmd);
		sz_logoncmd = 0;

		if (rc != 0) {
			XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
			XSRETURN (1);
		}

		if (memcmp (msg, okmsg, strlen (okmsg)) != 0) {
			strcpy (sz_err, "user_logon failed");
			XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
			XSRETURN (1);
		}
	}

	strcpy (cmd, sz_cmd);
	free (sz_cmd);

	*((char *)(obj->outbuff) + sizeof (sapdbcomm_rtehdr_t)) = '\0';
	sv_setpv (sv_err, "command canceled");

	XPUSHs (sv_2mortal (newSViv (obj->id)));
	XSRETURN (1);

void
exec_ex (...)
PREINIT:
	HV *self;
	dbmobj_t *obj;
	int rc;
	char *cmd;
	char msg[16 * 1024];		
	char sz_err[1024];
	SV *sv_err;
	int cmd_num;
	int cmd_idx;
	int force_ascii;
PPCODE:
	if (items < 3)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	force_ascii = SvIV (ST(1));
	cmd_num = items - 2;

	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	if (obj->fd == -1)
		XSRETURN_UNDEF;

	sv_err = self_getsverr (self);

	if (obj->inuse != 0) {
		sv_setpv (sv_err, "session already in use");
		XSRETURN_UNDEF;
	}

	for (cmd_idx = 0; cmd_idx < cmd_num; cmd_idx++) {
		cmd = SvPV (ST(cmd_idx + 2), PL_na);
		rc = sess_exec_ex (obj, force_ascii, cmd, msg, sz_err);
		if (rc == 0) {
			sv_setsv (sv_err, &PL_sv_undef);
			if (memcmp (msg, "OK\n", 3) != 0)
				break;
		} else {
			sv_setpv (sv_err, sz_err);
			break;
		}
	}

	if (rc != 0) {	
		XPUSHs (sv_2mortal (newSVsv (&PL_sv_undef)));
	} else if (cmd_num == 1) {
		XPUSHs (sv_2mortal (newSVpv (msg, strlen (msg))));
	} else if (memcmp (msg, "OK\n", 3) == 0) {
		XPUSHs (sv_2mortal (newSVpv ("OK\n", 3)));
	} else {
		XPUSHs (sv_2mortal (newSVpv ("ERR\n", 4)));
	}
	XSRETURN (1);

void
lasterr (...)
PREINIT:
	HV *self;
	dbmobj_t *obj;
	SV *sverr;
	int id;
PPCODE:
	if (items < 1 && items > 2)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0) {
		XPUSHs (sv_2mortal (newSVpv ("dbm session not established", 27)));
		XSRETURN (1);
	}

	sverr = self_getsverr (self);
	if (sverr == 0)
		XSRETURN_UNDEF;

	id = 1;
	if (items == 2)
		id = SvIV (ST(1));

	XPUSHs (sv_2mortal (newSVsv (sverr)));
	XSRETURN (1);

void
lastcmd (...)
PREINIT:
	HV *self;
	dbmobj_t *obj;
	dbmobj_t *root;
	char *cmd;
	int id;
PPCODE:
	if (items < 1 && items > 2)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	root = self_getobj (self);
	if (root == 0)
		XSRETURN_UNDEF;

	id = 1;
	if (items == 2)
		id = SvIV (ST(1));

	for (obj = root; obj != 0; obj = obj->next)
		if (obj->id == id)
			break;

	if (obj->id != id)
		XSRETURN_UNDEF;

	cmd = (char *)(obj->inbuff) + sizeof (sapdbcomm_rtehdr_t);
	XPUSHs (sv_2mortal (newSVpvn (cmd, strlen (cmd))));
	XSRETURN (1);

void
lastmsg (...)
PREINIT:
	HV *self;
	dbmobj_t *root;
	dbmobj_t *obj;
	char *msg;
	int id;
PPCODE:
	if (items < 1 && items > 2)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	root = self_getobj (self);
	if (root == 0)
		XSRETURN_UNDEF;

	id = 1;
	if (items == 2)
		id = SvIV (ST(1));

	for (obj = root; obj != 0; obj = obj->next)
		if (obj->id == id)
			break;

	if (obj->id != id)
		XSRETURN_UNDEF;

	msg = (char *)(obj->outbuff) + sizeof (sapdbcomm_rtehdr_t);
	XPUSHs (sv_2mortal (newSVpvn (msg, strlen (msg))));
	XSRETURN (1);
