/*
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/FileLock.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] =
        "@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/FileLock.xs#1 $";

static char *CLASS = "SAPDB::Install::FileLock";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#if defined _WIN64
typedef __int64 ptr_t;
#else
typedef long ptr_t;
#endif

#ifdef UNIX
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <utime.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/wait.h>

#ifdef _AIX
#define NEED_PRAGMA_ALLOCA NEED_PRAGMA_ALLOCA
#endif
#if defined UNIX && !defined _AIX && !defined SNI
#define NEED_ALLOCA_H NEED_ALLOCA_H
#endif

#ifdef NEED_ALLOCA_H
#include <alloca.h>
#endif

#ifdef NEED_PRAGMA_ALLOCA
#pragma alloca
#endif

#endif

#define LOCK_MODE_SHARED 0
#define LOCK_MODE_EXCLUSIVE 1

typedef struct{
	int fd;		/* UNIX -> file descriptor ; WIN -> file handle */
	int pid;	/* UNIX -> process id ; WIN -> 1 */
	char mode;     	/* LOCK_MODE_SHARED or LOCK_MODE_EXCLUSIVE */
	char locked;   	/* 1 -> file locked by this process ; 0 -> no own lock */
	char *errtext; 	/* error message of last error */
} lock_obj_t; 




int
get_pid (void){
	#ifdef UNIX
	return (int) getpid();
	#else
	return 0;
	#endif
} 


void
free_buf (void * ptr){
        if(ptr != 0){
                free(ptr);
        }
}


int
setErrMsg ( char * msg , int sys_errno , lock_obj_t *obj ) {
	char * errbuf;
	
	if (sys_errno == 0){
		/* no system error occured */
		free_buf(obj->errtext);
		obj->errtext = (char *) malloc (strlen (msg) + 1);
       	strcpy (obj->errtext,msg);
		return 0;
	}

#ifdef UNIX
	errbuf = malloc (strlen(Strerror(sys_errno)) + 1);
	strcpy(errbuf,Strerror(sys_errno));
#else
    
	/* WINDOWS */  
	FormatMessage (
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        (DWORD) sys_errno,
        0, // Default language
        (LPTSTR) &errbuf,
        0,
        NULL
	);
			
	{	/* this block remove all in errbuf included carriage return / line feeds */
		/* reallocate buffer with malloc -> now can use same free like unix */
		char * result = 0;
		char * next = errbuf;
		char * remove = "\r\n\0";
		char * replace = " \0";
		int len_remove = strlen (remove);
		int len_replace = strlen (replace);
		int orig_buf_len = strlen (errbuf);
		
		while (1) {
				char * temp_result = result; 
				char * start = next;
				if(next > (orig_buf_len + errbuf))
					break; /* next is out of bound */
				next = strstr (next,remove);
				if (next)
					*next = '\0';
				if (result) {
					result =
					(char *) malloc (strlen (result) + len_replace + strlen (start) + 1);
					strcpy (result,temp_result);
					strcat (result,replace);
					strcat (result,start);
					free_buf (temp_result);
				}
				else {
					result =
					(char *) malloc (strlen(start) + 1); /* first loop run */
					strcpy (result,start);
				}
				if (!next)
					break; /* no more appearances found */
				next += len_remove;	/* start search in next loop after last found string */
		}
		if (result) {
			LocalFree (errbuf);
		}
		else {
			result = (char *) malloc (strlen (errbuf) + 1);
			strcpy (result,errbuf);
			LocalFree (errbuf);
		}
		errbuf=result;
	}
	
#endif        

	free_buf (obj->errtext);
	
	obj->errtext = (char *) malloc
		(strlen (msg) + strlen (errbuf) + 1);
	
	strcpy (obj->errtext,msg);
	strcat (obj->errtext, errbuf);
	free_buf (errbuf);
	return 0;
}



/*
 * helper function for XS funtion to find
 * session structure from perl object
 */

static lock_obj_t *
self_getobj (HV *self) {
        char *key_obj = "obj";
        SV **psv_obj;
        lock_obj_t *obj;

        psv_obj = hv_fetch (self, key_obj, strlen (key_obj), 0);
        if (psv_obj == 0) {
                return 0;
        }

        obj = (lock_obj_t *) SvIV (*psv_obj);
        return (obj);
}

static void
self_putobj (HV *self,lock_obj_t *obj) {
        char *key_obj = "obj";
        SV *sv_obj;
        sv_obj = newSViv ((ptr_t) obj);
        hv_store (self, key_obj, strlen (key_obj), sv_obj, 0);
        return;
}





#ifdef UNIX

int
setLock (lock_obj_t *obj){
	int rc;
	struct flock lock[1];
	if(obj->mode == LOCK_MODE_EXCLUSIVE){
		lock->l_type=F_WRLCK;
	}
	else{
		lock->l_type=F_RDLCK;
	}
	lock->l_whence=SEEK_SET;
	lock->l_start=0;
	lock->l_len=0;		/* whole file */
	lock->l_pid = (pid_t) obj->pid; 	
	rc = fcntl(obj->fd,F_SETLK,lock);
	if(rc != 0){
		char * msg = "set lock failed: ";
		setErrMsg(msg,errno,obj);
		return -1;	
	}
	obj->locked=1;
	return 0;
} 



int
testLock (lock_obj_t *obj){
	/* 
        	file unlocked:  return 0 
        	file locked:    return pid
        	error:          return -1
	*/
	int rc;
	struct flock lock[1];
	
	if (obj->mode == LOCK_MODE_EXCLUSIVE){
                lock->l_type=F_WRLCK;
        }
        else{
                lock->l_type=F_RDLCK;
        }
	lock->l_whence=SEEK_SET;
	lock->l_start=0;
        lock->l_len=0;		/* whole file */
	lock->l_pid=obj->pid;
	rc=fcntl(obj->fd,F_GETLK,lock);
	if ( rc != 0 ){
		char * msg = "get lock failed: ";
                setErrMsg(msg,errno,obj);
		return -1;
	}
	if ( lock->l_type == F_UNLCK ){
		return 0;
	}
	
        return (int) lock->l_pid;
}

int
unlock (lock_obj_t *obj){
	int rc;
	struct flock lock[1];
	lock->l_type=F_UNLCK;
        lock->l_whence=SEEK_SET;
        lock->l_start=0;
        lock->l_len=0;		/* whole file */
        lock->l_pid=obj->pid;
	rc=fcntl(obj->fd,F_SETLK,lock);
	if ( rc != 0 ){
              	char * msg = "unlock failed: ";
                setErrMsg(msg,errno,obj);
		return -1;
        }
	obj->locked=0;
	return 0;
}


#else
/* WINDOWS*/

int
setLock (lock_obj_t *obj){
    BOOL rc;
	DWORD mode;
	OVERLAPPED ovlpd[1];
	ovlpd->Offset=0;
	ovlpd->OffsetHigh=0;
	ovlpd->hEvent=NULL;
        if (obj->mode == LOCK_MODE_EXCLUSIVE){
                mode = LOCKFILE_EXCLUSIVE_LOCK;
        }
        else{
                mode=0;
        }	
 	rc = LockFileEx ( 
		(HANDLE) ((ptr_t) win32_get_osfhandle(obj->fd)),
		LOCKFILE_FAIL_IMMEDIATELY | 
		mode,
		0,
		0xffffffff,   
		0xffffffff,		/* whole file */
		ovlpd
	);		
        if (!rc){
		char * msg = "set lock failed: ";
                setErrMsg(msg,GetLastError(),obj);
		return -1;
        }
	obj->locked=1;
	return 0;
}

int
unlock (lock_obj_t *obj){
        BOOL rc;
        OVERLAPPED ovlpd[1];
        ovlpd->Offset=0;
        ovlpd->OffsetHigh=0;
        ovlpd->hEvent=NULL;

        rc = UnlockFileEx(
                (HANDLE) ((ptr_t) win32_get_osfhandle(obj->fd)),
                0,
                0xffffffff, 
                0xffffffff,		/* whole file */
                ovlpd
        );

        if (!rc){
                char * msg = "unlock failed: ";
                setErrMsg(msg,GetLastError(),obj);
                return -1;
        }
        obj->locked=0;
	return 0;
}



int
testLock (lock_obj_t *obj){
    /* 
          file unlocked:  return 0 
          file locked:    return 1 
          error:          return -1
    */
	BOOL rc;
	DWORD mode;
	OVERLAPPED ovlpd[1];
    if(obj->locked)
		return 0;  /* no other process have a competed lock -> same behaviour like unix */
	ovlpd->Offset=0;
    ovlpd->OffsetHigh=0;
    ovlpd->hEvent=NULL;
	if (obj->mode == LOCK_MODE_EXCLUSIVE){
		mode = LOCKFILE_EXCLUSIVE_LOCK; 
	}
	else{
		mode=0;
	}
	
	rc = LockFileEx(
		(HANDLE) ((ptr_t) win32_get_osfhandle(obj->fd)),
		LOCKFILE_FAIL_IMMEDIATELY |
		mode,
		0,
		0xffffffff, 
		0xffffffff,	/* whole file */
		ovlpd
	);
        if(!rc){
                if ( GetLastError() == ERROR_LOCK_VIOLATION || GetLastError() == ERROR_SHARING_VIOLATION){
			return 1;	
		}
		else{
			char * msg = "test lock failed: ";
                	setErrMsg(msg,GetLastError(),obj);
                	return -1;
		}
        }
	if(!obj->locked){
		if(unlock(obj) != 0)
			return -1;
	}
    return 0;

}


#endif


int getFD (I32 perl_fd){
        int fd;
        dSP;
        ENTER;
        SAVETMPS;
        PUSHMARK (sp);
        XPUSHs(sv_2mortal(newSViv(perl_fd)));
        PUTBACK;
        printf("enter getFD(%d) now\n",perl_fd);
        perl_call_pv ("fileno",G_SCALAR);
        printf("noch da\n");
        SPAGAIN;
        printf("noch da\n");
        fd=POPi;
        printf("sys fd = %d\n",fd);
        PUTBACK;
        FREETMPS;
        LEAVE;
        return fd;
}




MODULE = SAPDB::Install::FileLock	PACKAGE = SAPDB::Install::FileLock

PROTOTYPES: DISABLE


void
new (...)
PREINIT:
	lock_obj_t *obj;
	HV * self;
	int fd;
	int pid;
	char mode;
CODE:
	if (items != 1 && items != 2) {
                printf("%s::new(): item error\n",CLASS);
		XSRETURN_UNDEF;
	}
	if ( SvOK(ST(0)) == 0 ){
		printf("%s::new(): undefined fd\n",CLASS);
		XSRETURN_UNDEF;
	}

	if (items == 2 && SvIV (ST(1)) == LOCK_MODE_EXCLUSIVE){
		mode = LOCK_MODE_EXCLUSIVE;
	}
	else{
		mode = LOCK_MODE_SHARED;
	}

	fd = (int) SvIV (ST(0));	
	#perl_fd =  (I32) SvIV (ST(0));
	#fd=getFD(perl_fd);
	#if( fd == -1 ){
	#	printf("%s::new(): cannot get sys fd\n",CLASS);
	#	XSRETURN_UNDEF;
	#}
	pid=get_pid();
	obj = (lock_obj_t *) malloc (sizeof (lock_obj_t));
        if (obj == 0) {
                printf("%s::new(): malloc failed\n",CLASS);
                XSRETURN_UNDEF;
        }
        memset (obj, 0, sizeof (lock_obj_t));	
        obj->fd = fd;
        obj->pid = pid;
	obj->errtext = 0;
	obj->locked = 0;
	obj->mode = mode;
	self = newHV ();
	ST(0) = sv_bless (newRV ((SV*) self), gv_stashpv (CLASS, 1));
	self_putobj(self,obj);
	sv_2mortal (ST(0));
	SvREFCNT_dec (self);


void 
DESTROY (...)
PREINIT:
        HV * self;
        lock_obj_t * obj;
CODE:
        if (items != 1) {
                printf("%s::DESTROY(): item error\n",CLASS);
                XSRETURN_UNDEF;
        }
        if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
                printf("%s::DESTROY(): first arg is no %s object\n",CLASS,CLASS);
                XSRETURN_UNDEF;
        }
        self = (HV*) SvRV (ST(0));
        obj = self_getobj (self);
        if (obj == 0){
                printf("%s::DESTROY(): cannot get object\n",CLASS);
                XSRETURN_UNDEF;
        }
	free_buf(obj->errtext);
	free_buf(obj);




void
Lock (...)
PREINIT:
	HV * self;
	lock_obj_t * obj;	
        int rc;
CODE:
	if (items != 1) {
                printf("%s::Lock(): item error\n",CLASS);
		XSRETURN_UNDEF;
        }
	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
		printf("%s::Lock(): first arg is no FileLock object\n",CLASS);
		XSRETURN_UNDEF;
	}
	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0){
		printf("%s::Lock(): cannot get object\n",CLASS);
		XSRETURN_UNDEF;
        }
	rc=setLock(obj);	
	ST(0)=newSViv((I32) rc);

void
Unlock (...)
PREINIT:
        HV * self;
	lock_obj_t * obj;
	int rc;
CODE:
        if (items != 1) {
                XSRETURN_UNDEF;
        }
        if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
                printf("%s::Unlock(): first arg is no FileLock object\n",CLASS);
                XSRETURN_UNDEF;
        }
        self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0) {
		printf("%s::Unlock(): cannot get object\n",CLASS);
                XSRETURN_UNDEF;
        }
        rc=unlock(obj);
        ST(0)=newSViv((I32) rc);

void
Test (...)
PREINIT:
	HV * self;	
        lock_obj_t * obj;
        int rc;
CODE:
	if (items != 1) {
                printf("%s::Test(): item error\n",CLASS);
		XSRETURN_UNDEF;
        }
	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
                printf("%s::Test(): first arg is no FileLock object\n",CLASS);
                XSRETURN_UNDEF;
        }
        self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0) {
                printf("%s::Test(): cannot get object\n",CLASS);
		XSRETURN_UNDEF;
        }
	rc=testLock(obj);
	ST(0)=newSViv((I32) rc);

void
GetError (...)
PREINIT:
	HV * self;
	lock_obj_t * obj;
PPCODE:
	if (items != 1) {
		XSRETURN_UNDEF;
	}
	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV))) {
                printf("%s::GetError(): first arg is no FileLock object\n",CLASS);
                XSRETURN_UNDEF;
        }
        self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0) {
		printf("%s::GetError(): cannot get object\n",CLASS);
		XSRETURN_UNDEF;
	}

	if (obj->errtext == 0) {
		XSRETURN_UNDEF;
	}
	
	XPUSHs (sv_2mortal (newSVpv (obj->errtext, strlen (obj->errtext))));
	XSRETURN (1);

