/*
 * GnuTar.xs
 *
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/GnuTar.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/GnuTar.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#include "zlib.h"


#ifdef UNIX
#include <stdio.h>
static int myfread (void *ptr, int size, int nmemb, void *fp) {
    return (fread (ptr, size, nmemb, fp));
}
#endif

#ifdef WIN32
#define NEED_TYPEDEF_INT64_T NEED_TYPEDEF_INT64_T
#define myfread(a,b,c,d) fread (a, b, c, d)
#endif


#ifdef _WIN64
typedef __int64 ptr_t;
#else
typedef long ptr_t;
#endif

#ifdef UNIX
#include <pwd.h>
typedef struct passwd passwd_t;

#include <grp.h>
typedef struct group grp_t;
#endif

#ifdef WIN32
typedef struct {
	char *pw_name;
} passwd_t;

typedef struct {
	char *gr_name;
} grp_t;
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include <time.h>

#ifdef PERL_OBJECT
#define FILE struct _PerlIO
#endif

/*
 * alloca specific defines
 */
#ifdef _AIX
#define NEED_PRAGMA_ALLOCA NEED_PRAGMA_ALLOCA
#endif
#if defined UNIX && !defined _AIX && !defined SNI
#define NEED_ALLOCA_H NEED_ALLOCA_H
#endif

#ifdef NEED_ALLOCA_H
#include <alloca.h>
#endif

#ifdef NEED_PRAGMA_ALLOCA
#pragma alloca
#endif

#ifdef NEED_TYPEDEF_INT64_T
#ifdef WIN32
typedef __int64 int64_t;
#else
#ifdef _LP64
typedef long int64_t;
#else
typedef long long int64_t;
#endif
#endif
#endif


#ifdef WIN32
#ifndef alloca
#define alloca(size) _alloca(size)
#endif
#endif

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
/*#pragma warning(disable : 4113)*/
#endif

/* 
 * tar typeflag types
 */
#define TAR_AREGTYPE '\0'
#define TAR_REGTYPE  '0'
#define TAR_LNKTYPE  '1'
#define TAR_SYMTYPE  '2'
#define TAR_CHRTYPE  '3'
#define TAR_BLKTYPE  '4'
#define TAR_DIRTYPE  '5'
#define TAR_FIFOTYPE '6'

#define DEFAULT_MODE 0600
#define DEFAULT_UID 501
#define DEFAULT_GID 501


/*
 * tar magic
 */
static char TAR_MAGIC[5] = {'u', 's', 't', 'a', 'r'};

/*
 * this is known as the tar header
 */
typedef struct tarhdr {
	char name[100];
	char mode[8];
	char uid[8];
	char gid[8];
	char size[12];
	char mtime[12];
	char chksum[8];
	char typeflag[1];
	char linkname[100];
	char magic[6];
	char version[2];
	char uname[32];
	char gname[32];
	char devmajor[8];
	char devminor[8];
	char prefix[155];
} tarhdr_t;

/*
 * session structure
 */
typedef struct tarobj {
	gzFile fp;
	int    mode;
	char   *fn;
} tarobj_t;

#define TAR_MODE_RO 0
#define TAR_MODE_WR 1

/*
 * this BLOCKSIZE read from an file must be 512 bytes
 * because the tar header and gzip are using 512 byte
 * blocks
 */
#define BLOCKSIZE 512

#ifdef WIN32
static grp_t *
getgrgid (int gid) {
	return 0;
}

static passwd_t *
getpwuid (int uid) {
	return 0;
}
#endif

static int
calc_chksum (void *buff, int cnt) {
	char *ptr;
	int i = 0;
	int sum = 0;

	ptr = (char *) buff;
	for (i = 0; i < cnt; i++)
		sum += 0xff & *ptr++;

	return (sum);
}

static char *
itooctn (char *buff, int64_t val, int cnt) {
	char *ptr;

	if (buff == 0 || cnt <= 0)
		return 0;

	for (ptr = buff + cnt - 1; ptr >= buff; ptr--) {
		*ptr = (char) (val & 7) + '0';
		val = val >> 3;
	}

	return (buff);
}

static tarobj_t *
self_getobj (HV *self) {
	char *key_obj = "obj";
	SV **psv_obj;
	tarobj_t *obj;

	psv_obj = hv_fetch (self, key_obj, strlen (key_obj), 0);
	if (psv_obj == 0)
		return 0;

	obj = (tarobj_t *) SvIV (*psv_obj);
	return (obj);
}

static int
close_archive (tarobj_t * obj) {
	int rc = 0;

	/*
	 * we have to write an empty block as eof mark
	 * before closing archive
	 */
	if (obj->fp != 0) {
		char buff[BLOCKSIZE];

		memset (buff, 0, BLOCKSIZE);
		rc |= gzwrite (obj->fp, buff, BLOCKSIZE);
		rc |= gzclose (obj->fp);
		obj->fp = 0;
	}

	if (obj->fn != 0) {
		free (obj->fn);
		obj->fn = 0;
	}

	memset (obj, 0, sizeof (tarobj_t));
	return (rc);
}

MODULE = SAPDB::Install::GnuTar            PACKAGE = SAPDB::Install::GnuTar

PROTOTYPES: DISABLE

void
new (...)
PREINIT:
	char *CLASS = 0;
	char *sz_class = "SAPDB::Install::GnuTar";
	char *key_obj = "obj";
	HV *self;
	SV *sv_obj;
	tarobj_t *obj;
CODE:
	if (items > 1)
		XSRETURN_UNDEF;

	if (items == 1 && (strlen (SvPV (ST(0), PL_na)) > 0))
		CLASS = SvPV (ST(0), PL_na);

	if (items == 0)
		EXTEND (sp, 1);

	if (CLASS == 0)
		CLASS = sz_class;

	obj = (tarobj_t *) malloc (sizeof (tarobj_t));
	if (obj == 0)
		XSRETURN_UNDEF;

	memset (obj, 0, sizeof (tarobj_t));

	self = newHV ();
	ST(0) = sv_bless (newRV ((SV*) self), gv_stashpv (CLASS, 1));

	sv_obj = newSViv ((ptr_t) obj);
	hv_store (self, key_obj, strlen (key_obj), sv_obj, 0);

	sv_2mortal (ST(0));
	SvREFCNT_dec (self);

void
DESTROY (...)
PREINIT:
	HV *self;
	tarobj_t *obj;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);

	if (obj != 0) {
		close_archive (obj);
		free (obj);
	}

	XSRETURN_UNDEF;

void
OpenArchive (...)
PREINIT:
	HV *self;
	tarobj_t *obj;
	char *tar_fn = 0;
	char *tar_mode = 0;
PPCODE:
	if (items < 2 || items > 3) 
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	tar_fn = (char *) SvPV (ST (1), PL_na);
	if (strlen (tar_fn) == 0)
		XSRETURN_UNDEF;

	if (items == 3)
		tar_mode = (char *) SvPV (ST (2), PL_na);	

	if (tar_mode == 0 || strlen (tar_mode) == 0 || tar_mode[0] == 'r') {
		obj->mode = TAR_MODE_RO;
	} else if (tar_mode[0] == 'w') {
		obj->mode = TAR_MODE_WR;
	} else {
		XSRETURN_UNDEF;
	}

	obj->fn = (char *) malloc (strlen (tar_fn) + 1);
	if (obj->fn == 0)
		XSRETURN_UNDEF;

	strcpy (obj->fn, tar_fn);

	switch (obj->mode) {
	case TAR_MODE_RO:
		obj->fp = (void *) gzopen (obj->fn, "rb");
		break;
	case TAR_MODE_WR:
		obj->fp = (void *) gzopen (obj->fn, "wb9");
		break;
	default:
		obj->fp = 0;
		break;
	}

	if (obj->fp == 0) {
		free (obj->fn);
		obj->fn = 0;
		XSRETURN_UNDEF;
	}

	XPUSHs (sv_2mortal (newSViv (0)));
	XSRETURN (1);

void
CloseArchive (...)
PREINIT:
	HV *self;
	tarobj_t *obj;
	int rc;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	rc = close_archive (obj);
	if (rc != 0)
		XSRETURN_UNDEF;

	XPUSHs (sv_2mortal (newSViv (0)));
	XSRETURN (1);

void
Add (...)
PREINIT:
	HV *self;
	tarobj_t *obj;
	int rc;
	char *filename = 0;
	char *alias = 0;
	FILE *fp;
	char *ptr;
	char buff[BLOCKSIZE];
	tarhdr_t *hdr;
	Stat_t statbuf[1];
	passwd_t *pwd;
	grp_t *grp;
PPCODE:
	if (items < 2 && items > 3)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	if (obj->fp == 0)
		XSRETURN_UNDEF;

	if (obj->mode != TAR_MODE_WR)
		XSRETURN_UNDEF;

	filename = (char *) SvPV (ST(1), PL_na);
	if (filename == 0 || strlen (filename) == 0)
		XSRETURN_UNDEF;
	
	if(items == 3){
		alias = (char *) SvPV (ST(2), PL_na);
		if (alias == 0 || strlen (alias) == 0)
			XSRETURN_UNDEF;
	
	}


	if (strlen (filename) >= 100)
		XSRETURN_UNDEF;

	rc = stat (filename, statbuf);
	if (rc != 0)
		XSRETURN_UNDEF;

	if (S_ISREG (statbuf->st_mode) == 0 &&
	    S_ISDIR (statbuf->st_mode) == 0) {
		XPUSHs (sv_2mortal (newSViv (0)));
		XSRETURN (1);
	}
		
	memset (buff, 0, BLOCKSIZE);
	hdr = (tarhdr_t *) buff;

	memcpy (hdr->magic, TAR_MAGIC, sizeof (TAR_MAGIC));

	if(alias)
		strcpy (hdr->name, alias);
	else
		strcpy (hdr->name, filename);
	for (ptr = hdr->name; *ptr != '\0'; ptr++)
		if (*ptr == '\\')
			*ptr = '/';

	itooctn (hdr->mode, statbuf->st_mode, sizeof (hdr->mode) - 1);
	itooctn (hdr->uid, statbuf->st_uid, sizeof (hdr->uid) - 1);
	itooctn (hdr->gid, statbuf->st_gid, sizeof (hdr->gid) - 1);
	itooctn (hdr->size, statbuf->st_size, sizeof (hdr->size) - 1);
	itooctn (hdr->mtime, statbuf->st_mtime, sizeof (hdr->mtime) - 1);
	memset (hdr->chksum, ' ', sizeof (hdr->chksum));
	if (S_ISREG (statbuf->st_mode))
		itooctn (hdr->typeflag, TAR_REGTYPE, 1);
	if (S_ISDIR (statbuf->st_mode))
		itooctn (hdr->typeflag, TAR_DIRTYPE, 1);
	itooctn (hdr->version, 0, sizeof (hdr->version));

	pwd = getpwuid ((statbuf->st_uid));
	if (pwd == 0) {
		pwd = (passwd_t *) alloca (sizeof (passwd_t));
		pwd->pw_name = (char *) alloca (32);
		strcpy (pwd->pw_name, "tar");
	}

	grp = getgrgid (statbuf->st_gid);
	if (grp == 0) {
		grp = (grp_t *) alloca (sizeof (grp_t));
		grp->gr_name = (char *) alloca (32);
		strcpy (grp->gr_name, "tar");
	}

	strncpy (hdr->uname, pwd->pw_name, sizeof (hdr->uname) - 1);
	strncpy (hdr->gname, grp->gr_name, sizeof (hdr->gname) - 1);

	itooctn (hdr->chksum,
		calc_chksum (buff, BLOCKSIZE),
		sizeof (hdr->chksum) - 1);
	hdr->chksum[sizeof (hdr->chksum) - 1] = '\0';

	if (S_ISREG (statbuf->st_mode)) {
		fp = fopen (filename, "rb");
		if (fp == 0)
			XSRETURN_UNDEF;
	}

	rc = gzwrite (obj->fp, buff, BLOCKSIZE);
	if (rc != BLOCKSIZE) {
		fclose (fp);
		XSRETURN_UNDEF;
	}

	if (S_ISREG (statbuf->st_mode)) {
		for (;;) {
			int got, want;

			memset (buff, 0, BLOCKSIZE);
			got = myfread (buff, 1, BLOCKSIZE, fp);
			if (got < 0) {
				fclose (fp);
				XSRETURN_UNDEF;
			}

			if (got == 0)
				break;

			want = got;
			got =  gzwrite (obj->fp, buff, BLOCKSIZE);
			if (got < 0) {
				fclose (fp);
				XSRETURN_UNDEF;
			}

			if (want < BLOCKSIZE)
				break;
		}

		fclose (fp);
	}

	XPUSHs (sv_2mortal (newSViv (0)));
	XSRETURN (1);

void
AddScalar (...)
PREINIT:
	HV *self;
	tarobj_t *obj;
	int rc;
	int blox = 0;
	int len = 0;
	time_t ltime;
	char *filename = 0;
	char *content = 0;
	char *ptr;
	char buff[BLOCKSIZE];
	tarhdr_t *hdr;
	passwd_t *pwd;
	grp_t *grp;
PPCODE:
	if (items != 3)
		XSRETURN_UNDEF;

	if (!(sv_isobject (ST(0)) && (SvTYPE (SvRV (ST(0))) == SVt_PVHV)))
		XSRETURN_UNDEF;

	self = (HV*) SvRV (ST(0));
	obj = self_getobj (self);
	if (obj == 0)
		XSRETURN_UNDEF;

	if (obj->fp == 0)
		XSRETURN_UNDEF;

	if (obj->mode != TAR_MODE_WR)
		XSRETURN_UNDEF;

	filename = (char *) SvPV (ST(1), PL_na);
	if (filename == 0 || strlen (filename) == 0)
		XSRETURN_UNDEF;

	content = (char *) SvPV (ST(2), PL_na);
	len = strlen(content);
	if (content == 0 || strlen (content) == 0)
		XSRETURN_UNDEF;

	if (strlen (filename) >= 100)
		XSRETURN_UNDEF;

	memset (buff, 0, BLOCKSIZE);
	hdr = (tarhdr_t *) buff;

	memcpy (hdr->magic, TAR_MAGIC, sizeof (TAR_MAGIC));

	strcpy (hdr->name, filename);
	for (ptr = hdr->name; *ptr != '\0'; ptr++)
		if (*ptr == '\\')
			*ptr = '/';

	time(&ltime);
	itooctn (hdr->mode, DEFAULT_MODE, sizeof (hdr->mode) - 1);
	itooctn (hdr->uid, DEFAULT_UID, sizeof (hdr->uid) - 1);
	itooctn (hdr->gid, DEFAULT_GID, sizeof (hdr->gid) - 1);
	itooctn (hdr->size, strlen(content), sizeof (hdr->size) - 1);
	itooctn (hdr->mtime, ltime, sizeof (hdr->mtime) - 1);
	memset (hdr->chksum, ' ', sizeof (hdr->chksum));
	itooctn (hdr->typeflag, TAR_REGTYPE, 1);
	itooctn (hdr->version, 0, sizeof (hdr->version));

	pwd = (passwd_t *) alloca (sizeof (passwd_t));
	pwd->pw_name = (char *) alloca (32);
	strcpy (pwd->pw_name, "tar");
	
	grp = (grp_t *) alloca (sizeof (grp_t));
	grp->gr_name = (char *) alloca (32);
	strcpy (grp->gr_name, "tar");
	
	strncpy (hdr->uname, pwd->pw_name, sizeof (hdr->uname) - 1);
	strncpy (hdr->gname, grp->gr_name, sizeof (hdr->gname) - 1);

	itooctn (hdr->chksum,
		calc_chksum (buff, BLOCKSIZE),
		sizeof (hdr->chksum) - 1);
	hdr->chksum[sizeof (hdr->chksum) - 1] = '\0';

	rc = gzwrite (obj->fp, buff, BLOCKSIZE);

	if (rc != BLOCKSIZE) {
		XSRETURN_UNDEF;
	}

	for (;;) {
		int got, want;
		memset (buff, 0, BLOCKSIZE);
		if(blox * BLOCKSIZE > len){
			break;
		}
		else if(len - blox * BLOCKSIZE < BLOCKSIZE){
			memcpy (buff,content + (blox++ * BLOCKSIZE), len % BLOCKSIZE);	
			got = len % BLOCKSIZE;
		}
		else{
			memcpy (buff,content + (blox++ * BLOCKSIZE) ,BLOCKSIZE);
			got = BLOCKSIZE;
		}
		
		if (got == 0)
			
		want = got;
		got =  gzwrite (obj->fp, buff, BLOCKSIZE);
		if (got < 0) {
			XSRETURN_UNDEF; 
		}
		if (want < BLOCKSIZE)
			break;
	}

	XPUSHs (sv_2mortal (newSViv (0)));
	XSRETURN (1);



