/*
 * MachInfo.xs
 *
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/MachInfo.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/MachInfo.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#if defined  _AIX || defined __osf__ || defined sun
#include <stdio.h>
static char *
myfgets (char *str, int size, void *fp) {
	return (fgets (str, size, (FILE *) fp));
}
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#ifdef PERL_OBJECT
#define FILE struct _PerlIO
#endif

#ifdef WIN32
#define NEED_INT64_T NEED_INT64_T
#endif

#ifdef NEED_INT64_T
#ifdef WIN32
typedef __int64 int64_t;
#elif defined UNIX && defined _LP64
typedef long int64_t;
#elif defined UNIX && ! defined _LP64
typedef long long int64_t;
#endif
#endif

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#ifdef UNIX
#include <unistd.h>
#endif

static const unit        = 1;
static const kilo_unit   = 1024;
static const mega_unit   = 1024 * 1024;
static const giga_unit   = 1024 * 1024 * 1024;

typedef struct {
	int num_of_cpu;
	unsigned long phys_mem_total_value;
	unsigned long phys_mem_total_unit;
	unsigned long phys_mem_avail_value;
	unsigned long phys_mem_avail_unit;
} machinfo_t;

#ifdef UNIX

static void
normalize_values (unsigned long *value, unsigned long *unit) {
	int64_t largevalue;

	largevalue = (int64_t)(*value) * (int64_t)(*unit);
	if (largevalue == 0) {
		*unit = 1;
	} else if (largevalue < ((int64_t) 128 * (int64_t) mega_unit)) {
		*unit = kilo_unit;
	} else if (largevalue < ((int64_t) 128 * (int64_t) giga_unit)) {
		*unit = mega_unit;
	} else {
		*unit = giga_unit;
	}

	*value = (int)(largevalue / (int64_t)(*unit));
	return;
}
#endif

#ifdef WIN32
static void
normalize_values (unsigned long *outval, unsigned long *unit, int64_t inval) {
	if (inval == 0) {
		*unit = 1;
	} else if (inval < ((int64_t) 128 * (int64_t) mega_unit)) {
		*unit = kilo_unit;
	} else if (inval < ((int64_t) 128 * (int64_t) giga_unit)) {
		*unit = mega_unit;
	} else {
		*unit = giga_unit;
	}

	*outval = (int)(inval / (int64_t)(*unit));
	return;
}
#endif

#if defined _AIX || defined __osf__ || defined sun
static char *
strstrip (char *txt) {
	int i;

	if (txt == 0)
		return 0;

	for (;;) {
		i = strlen (txt) - 1;
		if (i < 0)
			break;

		if (txt[i] > ' ')
			break;

		txt[i] = '\0';
	}

	return (txt);
}

static int
get_phys_mem_avail_vmstat (int *memvalue, int *memunit) {
	int fdout[2];
	pid_t pid;
	int state;
	char buff[4096 + 1];
	int pos;
	FILE *fp;

	*memvalue = 0;
	*memunit = 0;

	if (pipe (fdout) < 0)
		return -1;

	pid = fork ();
	if (pid < 0) {
		/* fork failed */
		return -1;
	} else if (pid == 0) {
		/*
		 * child,
		 * do a vmstat command,
		 * redirect STDOUT and STDERR to parent
		 */
		close (fdout[0]);
		dup2 (fdout[1], 1);
		dup2 (fdout[1], 2);

		execlp ("vmstat", "vmstat", 0);
		exit (-1);
	}

	/* parent,  get a file pointer to STDOUT and STDERR of child */
	close (fdout[1]);
	fp = fdopen (fdout[0], "r");
	if (fp == 0) {
		close (fdout[1]);
		waitpid (pid, &state, 0);
		return -1;
	}

	/*
	 * read every line of command output,
	 */
	pos = -1;
	while (myfgets (buff, 4096, fp) != 0) {
		int i;
		char *beg, *end;
#ifdef _AIX
		const char key[] = "fre"; 
#elif defined sun || defined __osf__
		const char key[] = "free";
#else
#error unknown keyword for free pages
#endif
		strstrip (buff);

		/*
		 * header and pos already found,
		 * get value and calc units
		 */
		if (pos >= 0) {
			long val;
			long multiplier;

			beg = buff;
			for (i = 0; i <= pos; i++) {
				val = strtol (beg, &end, 10);

#ifdef _AIX
				multiplier = 4096;

#elif defined __osf__
				switch (*end) {
				case 'G':
					multiplier = 4096 * giga_unit;
					end++;
					break;
				case 'M':
					multiplier = 4096 * mega_unit;
					end++;
					break;
				case 'K':
					multiplier = 4096 * kilo_unit;
					end++;
					break;
				default:
					multiplier = 4096;
					break;
				}

#elif defined sun
				multiplier = 1024;

#else
#error unknown multipiler for vmstat free
#endif

				beg = end + 1;
			}

			*memvalue = val;
			*memunit = multiplier;

			break;
		}

		/* skip any line until header */
		if (strstr (buff, key) == 0)
			continue;

		/* find key and calc position in header */
		beg = buff;
		end = buff;
		for (i = 0;; i++) {
			for (beg = end; *beg != '\0'; beg++)
				if (*beg > ' ')
					break;

			for (end = beg; *end != '\0'; end++)
				if (*end <=  ' ')
					break;

			if (*end == '\0') {
				end = 0;
			} else {
				*end = '\0';
				end++;
			}

			if (strcmp (beg, key) == 0) {
				pos = i;
				break;
			}

			if (end == 0)
				break; 
		}
	}

	fclose (fp);
	close (fdout[1]);
	waitpid (pid, &state, 0);

	return 0;
}

#endif

#if defined __linux

#include <sys/sysinfo.h>

/*
 * The field mem_unit in the sysinfo structure had been introduced with linux 2.4
 * It gives a multiplier for the elements in the structure that describe amounts
 * of memory. To make the code compilable under all releases of the linux kernel,
 * the structure is definde here and not taken from /usr/include/linux/kernel.h
 */

typedef struct {
	long uptime;                    /* seconds since boot               */
	unsigned long  loads[3];        /* 1, 5 and 15 minute load averages */
	unsigned long  totalram;        /* total usable main memory size    */
	unsigned long  freeram;         /* available memory size            */
	unsigned long  sharedram;       /* amount of shared memory          */
	unsigned long  bufferram;       /* memory used by buffers           */
	unsigned long  totalswap;       /* total swap space size            */
	unsigned long  freeswap;        /* swap space still available       */
	unsigned short procs;           /* number of current processes      */
	unsigned long  totalhigh;       /* total high memory size           */
	unsigned long  freehigh;        /* available high memory size       */
	unsigned int   memunit;         /* memory unit size in bytes        */
	char padding[20 - 2 * sizeof (long) - sizeof (int)];
} my_sysinfo_t;

static machinfo_t *
get_machinfo () {
	machinfo_t *mi;
    my_sysinfo_t si[1];
	int rc;

	mi = (machinfo_t *) malloc (sizeof (machinfo_t));
	if (mi == 0)
		return 0;

	memset (mi, 0, sizeof (machinfo_t));

	mi->num_of_cpu = (int) sysconf (_SC_NPROCESSORS_ONLN);

	memset (si, 0, sizeof (struct sysinfo));
	rc = sysinfo ((struct sysinfo *) si);
	if (rc == -1) {
		free (mi);
		return 0;
	}

	mi->phys_mem_total_value = si->totalram;
	mi->phys_mem_total_unit = si->memunit;

	mi->phys_mem_avail_value = si->freeram;
	mi->phys_mem_avail_unit = si->memunit;

	normalize_values (&mi->phys_mem_total_value, &mi->phys_mem_total_unit);
	normalize_values (&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	return (mi);
}

#elif defined _AIX

static int
get_phys_mem_total_aix (int *memvalue, int *memunit) {
	int fdout[2];
	pid_t pid;
	int state;
	char buff[4096 + 1];
	FILE *fp;

	*memvalue = 0;
	*memunit = 0;

	if (pipe (fdout) < 0)
		return -1;

	pid = fork ();
	if (pid < 0) {
		/* fork failed */
		return -1;
	} else if (pid == 0) {
		/*
		 * child,
		 * do a lsattr -E -l sys0 -a realmem command,
		 * redirect STDOUT and STDERR to parent
		 */
		close (fdout[0]);
		dup2 (fdout[1], 1);
		dup2 (fdout[1], 2);

		execlp ("lsattr", "lsattr", "-E", "-l", "sys0", "-a", "realmem", 0);
		exit (-1);
	}

	/* parent,  get a file pointer to STDOUT and STDERR of child */
	close (fdout[1]);
	fp = fdopen (fdout[0], "r");
	if (fp == 0) {
		close (fdout[1]);
		waitpid (pid, &state, 0);
		return -1;
	}

	/*
	 * read every line of command output,
	 * search for 'realmem', this is size of physical memory in kbytes
	 */
	while (myfgets (buff, 4096, fp) != 0) {
		const char key[] = "realmem"; 
		char *end;

		strstrip (buff);
		if (memcmp (buff, key, strlen (key) == 0))
			continue;

		*memvalue = (int)(strtol (buff + strlen (key), &end, 10));
		*memunit = kilo_unit;
	}

	fclose (fp);
	close (fdout[1]);
	waitpid (pid, &state, 0);

	return 0;
}

static machinfo_t *
get_machinfo () {
	machinfo_t *mi;
	int rc;

	mi = (machinfo_t *) malloc (sizeof (machinfo_t));
	if (mi == 0)
		return 0;

	memset (mi, 0, sizeof (machinfo_t));

	mi->num_of_cpu = (int) sysconf (_SC_NPROCESSORS_ONLN);

	get_phys_mem_total_aix (
	&mi->phys_mem_total_value, &mi->phys_mem_total_unit);

	get_phys_mem_avail_vmstat (
	&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	normalize_values (&mi->phys_mem_total_value, &mi->phys_mem_total_unit);
	normalize_values (&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	return (mi);
}

#elif defined __hpux

#include <sys/pstat.h>
#include <sys/param.h>

typedef struct pst_dynamic   pst_dynamic_t;
typedef struct pst_processor pst_processor_t;
typedef struct pst_static    pst_static_t;

static machinfo_t *
get_machinfo () {
	machinfo_t *mi;
	int i, rc;
	pst_processor_t cpu[1];
	pst_static_t    fix[1];
	pst_dynamic_t   dyn[1];

	mi = (machinfo_t *) malloc (sizeof (machinfo_t));
	if (mi == 0)
		return 0;

	memset (mi, 0, sizeof (machinfo_t));

	rc = pstat_getstatic (fix, sizeof (pst_static_t), 0, 0);
	if (rc <= 0) {
		free (mi);
		return 0;
	}

	for (i = 0; i < fix->cpu_states; i++) {
		rc = pstat_getprocessor (cpu, sizeof (pst_processor_t), 1, i);
		if (rc <= 0)
			break;
	}

	rc = pstat_getdynamic (dyn, sizeof (pst_dynamic_t), 0, 0);
	if (rc <= 0) {
		free (mi);
		return 0;
	}

	mi->num_of_cpu = i;

	mi->phys_mem_total_value = fix->physical_memory;
	mi->phys_mem_total_unit  = fix->page_size;

	mi->phys_mem_avail_value = dyn->psd_free;
	mi->phys_mem_avail_unit  = fix->page_size;

	normalize_values
	(&mi->phys_mem_total_value, &mi->phys_mem_total_unit);

	normalize_values
	(&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	return (mi);
}

#elif defined sun

static machinfo_t *
get_machinfo () {
	machinfo_t *mi;

	mi = (machinfo_t *) malloc (sizeof (machinfo_t));
	if (mi == 0)
		return 0;

	memset (mi, 0, sizeof (machinfo_t));

	mi->num_of_cpu = (int) sysconf (_SC_NPROCESSORS_ONLN);

	mi->phys_mem_total_value = sysconf (_SC_PHYS_PAGES);
	mi->phys_mem_total_unit = sysconf (_SC_PAGE_SIZE);
	
	get_phys_mem_avail_vmstat (
	&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	normalize_values (&mi->phys_mem_total_value, &mi->phys_mem_total_unit);
	normalize_values (&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	return (mi);
}

#elif defined __osf__

#include <sys/sysinfo.h>
#include <machine/hal_sysinfo.h>

static machinfo_t *
get_machinfo () {
	machinfo_t *mi;
	int intval;
	long longval;

	mi = (machinfo_t *) malloc (sizeof (machinfo_t));
	if (mi == 0)
		return 0;

	memset (mi, 0, sizeof (machinfo_t));

	getsysinfo (GSI_CPUS_IN_BOX, (char *) &intval, sizeof (intval), 0, 0, 0);
	mi->num_of_cpu = intval;

	getsysinfo (GSI_PHYSMEM, (char *) &longval, sizeof (longval), 0, 0, 0);

	mi->phys_mem_total_value = longval;
	mi->phys_mem_total_unit = kilo_unit;
	
	get_phys_mem_avail_vmstat (
	&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	normalize_values (&mi->phys_mem_total_value, &mi->phys_mem_total_unit);
	normalize_values (&mi->phys_mem_avail_value, &mi->phys_mem_avail_unit);

	return (mi);
}

#elif defined WIN32

static machinfo_t *
get_machinfo () {
	machinfo_t *mi;
	SYSTEM_INFO si[1];
	MEMORYSTATUSEX ms[1];

	mi = (machinfo_t *) malloc (sizeof (machinfo_t));
	if (mi == 0)
		return 0;

	memset (mi, 0, sizeof (machinfo_t));

	GetSystemInfo (si);
	mi->num_of_cpu = si->dwNumberOfProcessors;

	ms->dwLength = sizeof (MEMORYSTATUSEX);

	GlobalMemoryStatusEx (ms);

	normalize_values (&mi->phys_mem_total_value,
	&mi->phys_mem_total_unit, (int64_t)(ms->ullTotalPhys));

	normalize_values (&mi->phys_mem_avail_value,
	&mi->phys_mem_avail_unit, (int64_t)(ms->ullAvailPhys));

	return (mi);
}

#else

static machinfo_t *
get_machinfo () {
	return 0;
}

#endif

MODULE = SAPDB::Install::MachInfo	PACKAGE = SAPDB::Install::MachInfo

PROTOTYPES: DISABLE

SV *
GetMachInfo (...)
PREINIT:
	machinfo_t *machinfo;
	char *key_cpu    = "NumOfCPU";
	char *key_memtot = "PhysicalMemoryTotal";
	AV *av_memtot;
	char *key_memavl = "PhysicalMemoryAvailable";
	AV *av_memavl;
	HV *hv;
	SV *rv;
PPCODE:
	machinfo = get_machinfo ();

	if (machinfo == 0)
		XSRETURN_UNDEF;

	hv = newHV ();

	hv_store (hv, key_cpu, strlen (key_cpu), newSViv (machinfo->num_of_cpu), 0);

	av_memtot = newAV ();
	av_push (av_memtot, newSViv (machinfo->phys_mem_total_value));
	av_push (av_memtot, newSViv (machinfo->phys_mem_total_unit));
	hv_store (hv, key_memtot, strlen (key_memtot), newRV ((SV *) av_memtot), 0);

	av_memavl = newAV ();
	av_push (av_memavl, newSViv (machinfo->phys_mem_avail_value));
	av_push (av_memavl, newSViv (machinfo->phys_mem_avail_unit));

	hv_store (hv, key_memavl, strlen (key_memavl), newRV ((SV *) av_memavl), 0);

	free (machinfo);

	rv = sv_2mortal (newRV ((SV *) hv));
	SvREFCNT_dec (hv);

	XPUSHs (rv);
	XSRETURN (1);
