#!/usr/bin/perl
#
# $SRC/install/perl/Makefile.PL
#
# $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/Makefile.PL#1 $
# $DateTime: 2005/03/21 16:21:00 $
# $Change: 93297 $
#

use Config;
use Symbol;

sub main {
	print "Generating Makefile for SDBRUN\n";

	my %Makedefs = (
	'PERL_PERL' => 'perl',
	'PERL_PERLLIB' => get_perllib (),
	'PERL_COREDIR' => get_coredir (),
	'PERL_INC' => get_coredir (),
	'PERL_XSUBPP' => get_xsubpp (),
	'PERL_TXT2INC' => get_txt2inc (),
	'PERL_CHG2INC' => get_chg2inc (),
	'PERL_CHG2PM' => get_chg2pm (),
	'PERL_MAKESYM' => get_makesym (),
	'PERL_MKDIR' => get_mkdir (),
	'PERL_RM' => get_rm (),
	'PERL_CP' => get_cp (),
	'PERL_DYNALOADER' => get_dynaloader (),
	'PERL_DEEPCOPY' => get_deepcopy (),
	'PERL_STRIP' => get_strip (),
	'PERL_CCFLAGS' => get_ccflags (),
	'PERL_DEFS' => get_defs (),
	'PERL_INSTPREFIX' => get_instprefix (),
	'PERL_CC' => $Config{'cc'},
	'PERL_CFLAGS' => "\$(PERL_CCFLAGS) -I\$(PERL_COREDIR) \$(PERL_DEFS)",
	'PERL_DLD' => $^O =~ /^mswin/i ? "link" : $Config{'ld'},
	'PERL_DLDFLAGS' => get_dldflags (),
	'PERL_LD' => get_ld (),
	'PERL_LDFLAGS' => get_ldflags (),
	'PERL_DLEXT' => $Config{'dlext'},
	'PERL_OBJEXT' => $Config{'obj_ext'},
	'PERL_EXEEXT' => $Config{'exe_ext'},
	'PERL_EXTLIBS' => get_extlibs (),
	'PERL_LIBSDBRUN' => get_libsdbrun (),
	'PERL_LIBWXWIN' => get_libwxwin (),
	'PERL_LIBWXPERL' => get_libwxperl (),
	'PERL_LIBWXPERLGRID' => get_libwxperlgrid (),
	'PERL_CHMODLIBWXPERL' => get_chmodlibwxperl (),
	'PERL_CHMODLIBWXPERLGRID' => get_chmodlibwxperlgrid (),
	'PERL_INSTALLWX' => get_installwx (),	
	'PERL_MAKELIB' => get_makelib (),
	'PERL_LIBOVERFLOW' => get_liboverflow ());

	write_makefile (%Makedefs);

	chdir ("readkey");
	do ("Makefile.PL");
	chdir ("..");
}

sub write_makefile {
	my %Makedefs = @_;
	my $fh = gensym ();
	open ($fh, "> Makefile");
	print ($fh
	"#\n".
	"# Makefile\n".
	"# this file is generated from Makefile.PL\n".
	"# do not edit this file, any changes will be lost\n".
	"#\n".
	"\n".	
	".SUFFIXES:\n".
	".SUFFIXES: .o .c .xs\n".
	"\n");

	if ($^O =~ /^aix/) {
		print ($fh
		"#\n".
		"# make sure that we are in 32 bit mode on aix\n".
		"#\n".
		"OBJECT_MODE = 32\n".
		"\n");
	}

	while (my ($key, $value) = each %Makedefs) {
		next unless (defined $value);
		print ($fh $key." = ".$value."\n");
	}
	print $fh ("\n");

	write_body ($fh);
	close ($fh);
}

sub write_body {
	my $fh = shift;
	while (<DATA>) {
		chomp;
		if ($^O =~ /^aix/ && $_ =~ /-o (\w+)\.\$\(DLEXT\)/) {
			$_ .= " -bE:".$1.".exp";
		}

		if ($^O =~ /^mswin/i) {
			if ($_ =~ /-o (\w+)\.\$\(DLEXT\)/) {
				$_ .= " -def:".$1.".def";
				$_ =~ s/-o\s+/-out:/;
			} elsif ($_ =~ /-o \w+\$\(EXEEXT\)/) {
				$_ =~ s/-o\s+/-out:/;
			} elsif ($_ =~ /\$\(MAKELIB\)\s+-o\s+/) {
				$_ =~ s/-o\s+/-out:/;
				$_ .= ' $(LIBS)';
			}
		} else {
			if ($_ =~ /\$\(MAKELIB\)\s+-o\s+/) {
				$_ =~ s/-o\s+//;
			} elsif ($_ =~ /\$\(LD\)\s+-o\s+SDBRUN/) {
				$_ .= ' $(LIBSDBRUN) $(LD_FLAGS) $(LIBS)';
			}
		}

		print ($fh $_."\n");
	}
}

sub get_ccflags {
	my $ccflags = $Config{'ccflags'}." ".$Config{'cccdlflags'};
	$ccflags .= ' /Zm200' if ($^O =~ /mswin/i);
	return ($ccflags);
}

sub get_perllib {
	# UNIX
	return "libperl.a" unless ($^O =~ /mswin/i);

	# WIN32 Active State Perl
	return "PerlCRT.lib" if $Config{'archname'} =~ /object/;

	return $Config{'libperl'};
}

sub get_ld {
	$^O =~ /^mswin/i ? "link" :
	$^O =~ /^solaris/ ? "CC" :
	$^O =~ /aix/i ? "xlC_r" :
	$^O =~ /hpux/i ? "aCC" :
	"\$(CC)";
}

sub get_dldflags {
	$^O =~ /aix/    ? get_dldflags_aix () :
	$^O =~ /mswin/i ? 
	$Config{'lddlflags'}." -libpath:\$(PERL_COREDIR) \$(PERL_PERLLIB)" :
	$Config{'lddlflags'};
}

sub get_dldflags_aix {
	my @in = split (' ', $Config{'lddlflags'});
	my @out = ();
	foreach $_ (@in) {
		next if ($_ =~ /^-bE\:/);
		push @out, $_;
	}

	return (join " ", @out);
}

sub get_extlibs {
	$^O =~ /mswin/i   ? $Config{'libs'} :
	$^O =~ /^aix/     ? "-b32 -lc -ldl -lld -lbind -lnsl -lm -lbsd" :
	$^O =~ /^dec_osf/ ? "-lm -lsecurity" :
	$^O =~ /^hpux/ && $Config{'archname'} =~ /^pa-risc/i ?  "-lm -lsec" :
	$^O =~ /^hpux/ && $Config{'archname'} =~ /^ia64/i ?  "-lm -lsec" :
	$^O =~ /^linux/ && $Config{'archname'} =~ /^i\d86/ ? "-lm -lcrypt -ldl" :
	$^O =~ /^linux/ && $Config{'archname'} =~ /^ia64/ ? "-lm -lcrypt" :
	$^O =~ /^linux/ && $Config{'archname'} =~ /^s390x/ ? "-lm -lcrypt" :
	$^O =~ /^solaris/ ? "-lm -lsocket -lnsl" :
	"";
}

sub get_liboverflow {
	$^O =~ /mswin/i && $ENV{'HAS_MSC_SECURITY_CHECKS'} ? 'bufferoverflowu.lib' : '';
}

sub get_strip {
	$^O =~ /^mswin/i ? "touch" : "strip";
}

sub get_mkdir {
	$^O =~ /^mswin/i ? "mkdir" : "-mkdir -p";
}

sub get_rm {
	$^O =~ /^mswin/i ? "del" : "-rm";
}

sub get_cp {
	$^O =~ /^mswin/i ? "copy" : "cp";
}

sub get_instprefix {
	$^O =~ /^mswin/i ?
	$ENV{'OWN'}."\\sys\\wrk\\fast\\install" :
	$ENV{'OWN'}."/sys/wrk/fast/install";
}

sub get_deepcopy {
	$^O =~ /^mswin/i ?
	"\$(PERL) ..\\tool\\deepcopy" :
	"\$(PERL) ../tool/deepcopy";
}

sub get_coredir {
	my $dir =
	defined ($Config{'archlibexp'}) ? $Config{'archlibexp'}.'/CORE' :
	$Config{'privlibexp'}.'/CORE';

	$dir =~ s/\//\\/ if $^O =~ /^mswin/i;
	return ($dir);
}

sub get_xsubpp {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ".$Config{'privlibexp'}."\\ExtUtils\\xsubpp" :
	"\$(PERL) ".$Config{'privlibexp'}."/ExtUtils/xsubpp";
}

sub get_txt2inc {
	$^O =~ /^mswin/i ?
	"\$(PERL) ..\\tool\\txt2inc" :
	"\$(PERL) ../tool/txt2inc";
}

sub get_chg2inc {
	$^O =~ /^mswin/i ? 
	"\$(PERL) ..\\tool\\chg2inc" :
	"\$(PERL) ../tool/chg2inc";
}

sub get_chg2pm {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ..\\tool\\chg2pm" :
	"\$(PERL) ../tool/chg2pm";
}

sub get_makesym {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ..\\tool\\makesym" :
	"\$(PERL) ../tool/makesym";
}

sub get_dynaloader {
	$^O !~ /mswin/i ?
	"../../local/lib/auto/DynaLoader/DynaLoader.a" :
	""
}

sub get_ldflags {
	$^O =~ /mswin/i ? 
		"-nodefaultlib -libpath:\$(PERL_COREDIR) \$(PERL_PERLLIB)" :
	$^O =~ /hpux/ && $Config{'archname'} =~ /^pa-risc/i ?
		"+DS2.0 -L\$(PERL_COREDIR) -Wl,+s -lperl \$(PERL_DYNALOADER)" :
	$^O =~ /hpux/ && $Config{'archname'} =~ /^ia64/i ?
		"-L\$(PERL_COREDIR) -Wl,+s -lperl \$(PERL_DYNALOADER)" :
	$^O =~ /linux/ ?
		"-lpthread -L\$(PERL_COREDIR) -Wl,-lperl -Wl,\$(PERL_DYNALOADER)" : 
	$^O =~ /solaris/i ?
		"-L\$(PERL_COREDIR) -lperl \$(PERL_DYNALOADER) -ldl" : 
	$^O =~ /aix/i ?
		"-L\$(PERL_COREDIR) -lperl \$(PERL_DYNALOADER) -brtl -L/usr/local/lib -b32 -bE:\$(PERL_COREDIR)/perl.exp" : 
	$^O =~ /osf/i ?
		"-pthread -L\$(PERL_COREDIR) -lperl \$(PERL_DYNALOADER)" :
	"-lperl ";
}

sub get_defs {
	"-DHAS_BYTEORDER=".$Config{'byteorder'};
}

sub get_libsdbrun {
	$^O =~ /mswin/i ? "sdbrun.dll" : "libsdbrun.a";
}

sub get_libwxperl {
	my $lib = `perl $ENV{WRK}/fast/install/tmp/tool/findlibs --wxperl`;
	return $lib;
}

sub get_libwxperlgrid {
	my $lib = `perl $ENV{WRK}/fast/install/tmp/tool/findlibs --wxperl_grid`;	
	return $lib;
}

sub get_chmodlibwxperl {
		$^O =~ /mswin/i ? "&& attrib -R \$(INSTPREFIX)\\Wx.\$(DLEXT)" :
		"&& chmod u+w \$(INSTPREFIX)/Wx.\$(DLEXT)";
}

sub get_chmodlibwxperlgrid {
		$^O =~ /mswin/i ? "&& attrib -R \$(INSTPREFIX)\\Grid.\$(DLEXT)" :
		"&& chmod u+w \$(INSTPREFIX)/Grid.\$(DLEXT)";
}

sub get_libwxwin {
	my $lib = `perl $ENV{WRK}/fast/install/tmp/tool/findlibs --wxlib`;	
	return $lib;
}

sub get_makelib {
	$^O =~ /mswin/i ?
	"\$(DLD) \$(DLD_FLAGS) -subsystem:console" :
	"ar -r";
}

sub get_installwx {
	my $command = '';
	my $wxperl = get_libwxperl();
	if($wxperl =~ /\S/){
		$command .= '$(CP) '.$wxperl.' $(INSTPREFIX) '.get_chmodlibwxperl(); 
	}
	my $wxperlgrid = get_libwxperlgrid();
	if($wxperlgrid =~ /\S/){
		$command .= ($command =~ /\S/ ? ' && ' : '').'$(CP) '.$wxperlgrid.' $(INSTPREFIX) '.get_chmodlibwxperlgrid (); 
	}
	my $libwxwin = get_libwxwin();
	if($libwxwin =~ /\S/){
		$command .= ($command =~ /\S/ ? ' && ' : '').'$(CP) '.$libwxwin.' $(INSTPREFIX)'; 
	}
	return $command;
}


main (@ARGV);

__DATA__
PERL = $(PERL_PERL)
RM = $(PERL_RM)
LD = $(PERL_LD)
CP = $(PERL_CP)
CC = $(PERL_CC)
MKDIR = $(PERL_MKDIR)
STRIP = $(PERL_STRIP)
INSTPREFIX = $(PERL_INSTPREFIX)
TXT2INC = $(PERL_TXT2INC)
CHG2INC = $(PERL_CHG2INC)
CHG2PM = $(PERL_CHG2PM)
MAKESYM = $(PERL_MAKESYM)
DEEPCOPY = $(PERL_DEEPCOPY)
XSUBPP = $(PERL_XSUBPP)
CFLAGS = $(PERL_CFLAGS)
DLD = $(PERL_DLD)
DLD_FLAGS = $(PERL_DLDFLAGS)
LIBS = $(PERL_EXTLIBS)
LD_FLAGS = $(PERL_LDFLAGS)
DLEXT = $(PERL_DLEXT)
OBJEXT = $(PERL_OBJEXT)
EXEEXT = $(PERL_EXEEXT)
LIBSDBRUN = $(PERL_LIBSDBRUN)
LIBOVERFLOW = $(PERL_LIBOVERFLOW)
LIBWXWIN = $(PERL_LIBWXWIN)
LIBWXPERL = $(PERL_LIBWXPERL)
LIBWXPERLGRID = $(PERL_LIBWXPERLGRID)
CHMODLIBWXPERL=$(PERL_CHMODLIBWXPERL)
CHMODLIBWXPERLGRID=$(PERL_CHMODLIBWXPERLGRID)
INSTALLWX=$(PERL_INSTALLWX)
MAKELIB = $(PERL_MAKELIB)

.xs.c:
	$(XSUBPP) $*.xs > $*.c

.c$(OBJEXT):
	$(CC) -c $(CFLAGS) $<

ReadKey_OBJ = readkey/ReadKey$(OBJEXT)
Comm_OBJ = Comm$(OBJEXT)
Jar_OBJ = Jar$(OBJEXT)
GetParam_OBJ = GetParam$(OBJEXT)
GnuTar_OBJ = GnuTar$(OBJEXT)
InstInfo_OBJ = InstInfo$(OBJEXT)
DBMCmd_OBJ = DBMCmd$(OBJEXT)
MachInfo_OBJ = MachInfo$(OBJEXT)
MD5Sum_OBJ = MD5Sum$(OBJEXT)
BackupPipe_OBJ = BackupPipe$(OBJEXT)
BuildInfo_OBJ = BuildInfo$(OBJEXT)
SetUser_OBJ = SetUser$(OBJEXT)
SystemUnix_OBJ = SystemUnix$(OBJEXT)
SystemUnixDir_OBJ = SystemUnixDir$(OBJEXT)
SystemWin32Registry_OBJ = SystemWin32Registry$(OBJEXT)
SystemWin32API_OBJ = SystemWin32API$(OBJEXT)
SystemWin32Console_OBJ = SystemWin32Console$(OBJEXT)
Untgz_OBJ = Untgz$(OBJEXT)
Common_OBJ = Common$(OBJEXT)
ArchiveTar_OBJ = ArchiveTar$(OBJEXT)
ArchiveZip_OBJ = ArchiveZip$(OBJEXT)
PipeExec_OBJ =PipeExec$(OBJEXT)
FileLock_OBJ = FileLock$(OBJEXT)
FSUsage_OBJ = FSUsage$(OBJEXT)
SysInfo_OBJ = SysInfo$(OBJEXT)
ProcState_OBJ = ProcState$(OBJEXT)
VersionInfo_OBJ = VersionInfo$(OBJEXT)

SDBRun_OBJ = SDBRun$(OBJEXT)
libSDBRun_OBJ = libSDBRun$(OBJEXT)

ZLIB_OBJ = ../lib/zlib/zlib.ro
ZIPLIB_OBJ = ../lib/zip/ziplib.ro

ReadKey_DLL = readkey/ReadKey.$(DLEXT)

ALL_PL_H = SDBRun.h DBMCmd.h Untgz.h FileMagic.h Getopt.h Exporter.h PipeExec.h GetParam.h Jar.h SysInfo.h

ALL_XS_C = GnuTar.c BackupPipe.c BuildInfo.c MachInfo.c MD5Sum.c SetUser.c SystemUnix.c SystemUnixDir.c SystemWin32Registry.c SystemWin32API.c SystemWin32Console.c Untgz.c DBMCmd.c Common.c ArchiveTar.c ArchiveZip.c PipeExec.c FileLock.c FSUsage.c SysInfo.c Comm.c GetParam.c Jar.c ProcState.c VersionInfo.c InstInfo.c

ALL_XS_OBJ = $(GnuTar_OBJ) $(BackupPipe_OBJ) $(BuildInfo_OBJ) $(MachInfo_OBJ) $(MD5Sum_OBJ) $(SetUser_OBJ) $(SystemUnix_OBJ) $(SystemUnixDir_OBJ) $(SystemWin32Registry_OBJ) $(SystemWin32API_OBJ) $(SystemWin32Console_OBJ) $(Untgz_OBJ) $(DBMCmd_OBJ) $(Common_OBJ) $(ArchiveTar_OBJ) $(ArchiveZip_OBJ) $(PipeExec_OBJ) $(FSUsage_OBJ) $(SysInfo_OBJ) $(Comm_OBJ) $(GetParam_OBJ) $(Jar_OBJ) $(FileLock_OBJ) $(ProcState_OBJ) $(VersionInfo_OBJ) $(InstInfo_OBJ)

ALL_XS_DLL = GnuTar.$(DLEXT) BackupPipe.$(DLEXT) BuildInfo.$(DLEXT) MachInfo.$(DLEXT) MD5Sum.$(DLEXT) SetUser.$(DLEXT) SystemUnix.$(DLEXT) SystemUnixDir.$(DLEXT) SystemWin32Registry.$(DLEXT) SystemWin32API.$(DLEXT) SystemWin32Console.$(DLEXT) Untgz.$(DLEXT) DBMCmd.$(DLEXT) Common.$(DLEXT) ArchiveTar.$(DLEXT) ArchiveZip.$(DLEXT) PipeExec.$(DLEXT) FSUsage.$(DLEXT) SysInfo.$(DLEXT) Comm.$(DLEXT) GetParam.$(DLEXT) Jar.$(DLEXT) FileLock.$(DLEXT) ProcState.$(DLEXT) VersionInfo.$(DLEXT) InstInfo.$(DLEXT) 

all: $(ALL_XS_DLL) $(LIBSDBRUN) SDBRUN$(EXEEXT)

SDBRUN$(EXEEXT): $(SDBRun_OBJ) $(LIBSDBRUN)
	$(LD) -o SDBRUN$(EXEEXT) $(SDBRun_OBJ) $(LIBOVERFLOW)

SDBRun$(OBJEXT): SDBRun.c
	$(CC) -c $(CFLAGS) SDBRun.c

$(LIBSDBRUN): $(libSDBRun_OBJ) $(ALL_XS_OBJ) $(ReadKey_OBJ) $(ZLIB_OBJ) $(ZIPLIB_OBJ)
	$(MAKELIB) -o $(LIBSDBRUN) $(libSDBRun_OBJ) $(ALL_XS_OBJ) $(ReadKey_OBJ) $(ZLIB_OBJ) $(ZIPLIB_OBJ)

libSDBRun$(OBJEXT): SDBRun.c SDBRun.h version.h
	$(CC) -c $(CFLAGS) libSDBRun.c

SDBRun.h: SDBRun.pl
	$(TXT2INC) SDBRun.pl SDBRun.h sz_text

$(ReadKey_DLL): $(ReadKey_OBJ)

$(ReadKey_OBJ): readkey/ReadKey.xs readkey/Makefile
	cd readkey && $(MAKE)

InstInfo.$(DLEXT): $(InstInfo_OBJ)
	$(MAKESYM) InstInfo.xs
	$(DLD) $(DLD_FLAGS) -o InstInfo.$(DLEXT) $(InstInfo_OBJ) $(LIBS)

VersionInfo.$(DLEXT): $(VersionInfo_OBJ)
	$(MAKESYM) VersionInfo.xs
	$(DLD) $(DLD_FLAGS) -o VersionInfo.$(DLEXT) $(VersionInfo_OBJ) $(LIBS)

FSUsage.$(DLEXT): $(FSUsage_OBJ)
	$(MAKESYM) FSUsage.xs
	$(DLD) $(DLD_FLAGS) -o FSUsage.$(DLEXT) $(FSUsage_OBJ) $(LIBS)

PipeExec.$(DLEXT): $(PipeExec_OBJ)
	$(MAKESYM) PipeExec.xs
	$(DLD) $(DLD_FLAGS) -o PipeExec.$(DLEXT) $(PipeExec_OBJ) $(LIBS)

FileLock.$(DLEXT): $(FileLock_OBJ)
	$(MAKESYM) FileLock.xs
	$(DLD) $(DLD_FLAGS) -o FileLock.$(DLEXT) $(FileLock_OBJ) $(LIBS)

Common.$(DLEXT): $(Common_OBJ)
	$(MAKESYM) Common.xs
	$(DLD) $(DLD_FLAGS) -o Common.$(DLEXT) $(Common_OBJ) $(LIBS)

ArchiveTar.$(DLEXT): $(ArchiveTar_OBJ) $(ZLIB_OBJ)
	$(MAKESYM) ArchiveTar.xs
	$(DLD) $(DLD_FLAGS) -o ArchiveTar.$(DLEXT) $(ArchiveTar_OBJ) $(ZLIB_OBJ) $(LIBS) 

ArchiveZip.$(DLEXT): $(ArchiveZip_OBJ) $(ZIPLIB_OBJ) $(ZLIB_OBJ)
	$(MAKESYM) ArchiveZip.xs
	$(DLD) $(DLD_FLAGS) -o ArchiveZip.$(DLEXT) $(ArchiveZip_OBJ) $(ZIPLIB_OBJ) $(ZLIB_OBJ) $(LIBS)

Comm.$(DLEXT): $(Comm_OBJ)
	$(MAKESYM) Comm.xs
	$(DLD) $(DLD_FLAGS) -o Comm.$(DLEXT) $(Comm_OBJ) $(LIBS)

GetParam.$(DLEXT): $(GetParam_OBJ)
	$(MAKESYM) GetParam.xs
	$(DLD) $(DLD_FLAGS) -o GetParam.$(DLEXT) $(GetParam_OBJ) $(LIBS)

GnuTar.$(DLEXT): $(GnuTar_OBJ) $(ZLIB_OBJ)
	$(MAKESYM) GnuTar.xs
	$(DLD) $(DLD_FLAGS) -o GnuTar.$(DLEXT) $(GnuTar_OBJ) $(ZLIB_OBJ) $(LIBS)

DBMCmd.$(DLEXT): $(DBMCmd_OBJ)
	$(MAKESYM) DBMCmd.xs
	$(DLD) $(DLD_FLAGS) -o DBMCmd.$(DLEXT) $(DBMCmd_OBJ) $(LIBS)

SystemUnix.$(DLEXT): $(SystemUnix_OBJ)
	$(MAKESYM) SystemUnix.xs
	$(DLD) $(DLD_FLAGS) -o SystemUnix.$(DLEXT) $(SystemUnix_OBJ) $(LIBS)

SystemUnixDir.$(DLEXT): $(SystemUnixDir_OBJ)
	$(MAKESYM) SystemUnixDir.xs
	$(DLD) $(DLD_FLAGS) -o SystemUnixDir.$(DLEXT) $(SystemUnixDir_OBJ) $(LIBS)

SystemWin32Registry.$(DLEXT): $(SystemWin32Registry_OBJ)
	$(MAKESYM) SystemWin32Registry.xs
	$(DLD) $(DLD_FLAGS) -o SystemWin32Registry.$(DLEXT) $(SystemWin32Registry_OBJ) $(LIBS)

SystemWin32API.$(DLEXT): $(SystemWin32API_OBJ)
	$(MAKESYM) SystemWin32API.xs
	$(DLD) $(DLD_FLAGS) -o SystemWin32API.$(DLEXT) $(SystemWin32API_OBJ) $(LIBS)

SystemWin32Console.$(DLEXT): $(SystemWin32Console_OBJ)
	$(MAKESYM) SystemWin32Console.xs
	$(DLD) $(DLD_FLAGS) -o SystemWin32Console.$(DLEXT) $(SystemWin32Console_OBJ) $(LIBS)

Untgz.$(DLEXT): $(Untgz_OBJ) $(ZLIB_OBJ)
	$(MAKESYM) Untgz.xs
	$(DLD) $(DLD_FLAGS) -o Untgz.$(DLEXT) $(Untgz_OBJ) $(LIBS) $(ZLIB_OBJ)

Jar.$(DLEXT): $(Jar_OBJ) $(ZIPLIB_OBJ) $(ZLIB_OBJ)
	$(MAKESYM) Jar.xs
	$(DLD) $(DLD_FLAGS) -o Jar.$(DLEXT) $(Jar_OBJ) $(ZIPLIB_OBJ) $(ZLIB_OBJ) $(LIBS)

MachInfo.$(DLEXT): $(MachInfo_OBJ)
	$(MAKESYM) MachInfo.xs
	$(DLD) $(DLD_FLAGS) -o MachInfo.$(DLEXT) $(MachInfo_OBJ) $(LIBS)

MD5Sum.$(DLEXT): $(MD5Sum_OBJ)
	$(MAKESYM) MD5Sum.xs
	$(DLD) $(DLD_FLAGS) -o MD5Sum.$(DLEXT) $(MD5Sum_OBJ) $(LIBS)

BackupPipe.$(DLEXT): $(BackupPipe_OBJ)
	$(MAKESYM) BackupPipe.xs
	$(DLD) $(DLD_FLAGS) -o BackupPipe.$(DLEXT) $(BackupPipe_OBJ) $(LIBS)

BuildInfo.$(DLEXT): $(BuildInfo_OBJ)
	$(MAKESYM) BuildInfo.xs
	$(DLD) $(DLD_FLAGS) -o BuildInfo.$(DLEXT) $(BuildInfo_OBJ) $(LIBS)

SysInfo.$(DLEXT): $(SysInfo_OBJ)
	$(MAKESYM) SysInfo.xs
	$(DLD) $(DLD_FLAGS) -o SysInfo.$(DLEXT) $(SysInfo_OBJ) $(LIBS)

ProcState.$(DLEXT): $(ProcState_OBJ)
	$(MAKESYM) ProcState.xs
	$(DLD) $(DLD_FLAGS) -o ProcState.$(DLEXT) $(ProcState_OBJ) $(LIBS)

SetUser.$(DLEXT): $(SetUser_OBJ)
	$(MAKESYM) SetUser.xs
	$(DLD) $(DLD_FLAGS) -o SetUser.$(DLEXT) $(SetUser_OBJ) $(LIBS)

InstInfo$(OBJEXT): InstInfo.c
	$(CC) -c $(CFLAGS) InstInfo.c

VersionInfo$(OBJEXT): VersionInfo.c
	$(CC) -c $(CFLAGS) VersionInfo.c

FSUsage$(OBJEXT): FSUsage.c
	$(CC) -c $(CFLAGS) FSUsage.c

PipeExec$(OBJEXT): PipeExec.c PipeExec.h
	$(CC) -c $(CFLAGS) PipeExec.c

FileLock$(OBJEXT): FileLock.c
	$(CC) -c $(CFLAGS) FileLock.c

Common$(OBJEXT): Common.c Getopt.h Exporter.h Require.h Archive.h
	$(CC) -c $(CFLAGS) Common.c

ArchiveTar$(OBJEXT): ArchiveTar.c
	$(CC) -c $(CFLAGS) -I../lib/zlib ArchiveTar.c

ArchiveZip$(OBJEXT): ArchiveZip.c
	$(CC) -c $(CFLAGS) -I../lib/zip ArchiveZip.c

Jar$(OBJEXT): Jar.c Jar.h
	$(CC) -c $(CFLAGS) -I../lib/zip Jar.c

GetParam$(OBJEXT): GetParam.c
	$(CC) -c $(CFLAGS) GetParam.c

GnuTar$(OBJEXT): GnuTar.c
	$(CC) -c $(CFLAGS) -I../lib/zlib GnuTar.c
  
Comm$(OBJEXT): Comm.c
	$(CC) -c $(CFLAGS) Comm.c

DBMCmd$(OBJEXT): DBMCmd.c DBMCmd.h
	$(CC) -c $(CFLAGS) DBMCmd.c

SetUser$(OBJEXT): SetUser.c
	$(CC) -c $(CFLAGS) SetUser.c

Untgz$(OBJEXT): Untgz.c Untgz.h
	$(CC) -c $(CFLAGS) -I../lib/zlib Untgz.c

BackupPipe$(OBJEXT): BackupPipe.c
	$(CC) -c $(CFLAGS) BackupPipe.c

BuildInfo$(OBJEXT): BuildInfo.c FileMagic.h
	$(CC) -c $(CFLAGS) BuildInfo.c

SysInfo$(OBJEXT): SysInfo.c SysInfo.h
	$(CC) -c $(CFLAGS) SysInfo.c

ProcState$(OBJEXT): ProcState.c
	$(CC) -c $(CFLAGS) ProcState.c

SystemUnix$(OBJEXT): SystemUnix.c
	$(CC) -c $(CFLAGS) SystemUnix.c

SystemUnixDir$(OBJEXT): SystemUnixDir.c
	$(CC) -c $(CFLAGS) SystemUnixDir.c

SystemWin32Registry$(OBJEXT): SystemWin32Registry.c
	$(CC) -c $(CFLAGS) SystemWin32Registry.c

SystemWin32API$(OBJEXT): SystemWin32API.c
	$(CC) -c $(CFLAGS) SystemWin32API.c

SystemWin32Console$(OBJEXT): SystemWin32Console.c
	$(CC) -c $(CFLAGS) SystemWin32Console.c

MachInfo$(OBJEXT): MachInfo.c
	$(CC) -c $(CFLAGS) MachInfo.c

MD5Sum$(OBJEXT): MD5Sum.c
	$(CC) -c $(CFLAGS) MD5Sum.c

InstInfo.c: InstInfo.xs
	$(XSUBPP) $*.xs > $*.c 

VersionInfo.c: VersionInfo.xs
	$(XSUBPP) $*.xs > $*.c 

FSUsage.c: FSUsage.xs
	$(XSUBPP) $*.xs > $*.c 

PipeExec.h: PipeExec.pl
	$(TXT2INC) PipeExec.pl PipeExec.h sz_text

PipeExec.c: PipeExec.xs
	$(XSUBPP) $*.xs > $*.c 

FileLock.c: FileLock.xs
	$(XSUBPP) $*.xs > $*.c 

Exporter.h: Exporter.pl
	$(TXT2INC) Exporter.pl Exporter.h sz_exporter

Archive.h: Archive.pl
	$(TXT2INC) Archive.pl Archive.h sz_archive

Require.h: Require.pl
	$(TXT2INC) Require.pl Require.h sz_require

Getopt.h: Getopt.pl
	$(TXT2INC) Getopt.pl Getopt.h sz_getopt

Common.c: Common.xs Getopt.h Exporter.h Require.h Archive.h
	$(XSUBPP) $*.xs > $*.c 

GetParam.h: GetParam.pl
	$(TXT2INC) GetParam.pl GetParam.h sz_text

GetParam.c: GetParam.xs GetParam.h
	$(XSUBPP) $*.xs > $*.c 

GnuTar.c: GnuTar.xs
	$(XSUBPP) $*.xs > $*.c

ArchiveTar.c: ArchiveTar.xs
	$(XSUBPP) $*.xs > $*.c 

ArchiveZip.c: ArchiveZip.xs
	$(XSUBPP) $*.xs > $*.c 

Jar.h: Jar.pl
	$(TXT2INC) Jar.pl Jar.h sz_text

Jar.c: Jar.xs
	$(XSUBPP) $*.xs > $*.c 

Comm.c: Comm.xs
	$(XSUBPP) $*.xs > $*.c 

DBMCmd.h: DBMCmd.pl
	$(TXT2INC) DBMCmd.pl DBMCmd.h sz_text

DBMCmd.c: DBMCmd.xs DBMCmd.h
	$(XSUBPP) $*.xs > $*.c 

Untgz.h: Untgz.pl
	$(TXT2INC) Untgz.pl Untgz.h sz_text

Untgz.c: Untgz.xs Untgz.h
	$(XSUBPP) $*.xs > $*.c 

SetUser.c: SetUser.xs
	$(XSUBPP) $*.xs > $*.c 

FileMagic.h: FileMagic.pl
	$(TXT2INC) FileMagic.pl FileMagic.h sz_text

BackupPipe.c: BackupPipe.xs
	$(XSUBPP) $*.xs > $*.c

BuildInfo.c: BuildInfo.xs
	$(XSUBPP) $*.xs > $*.c

SysInfo.h: SysInfo.pl
	$(TXT2INC) SysInfo.pl SysInfo.h sz_text

SysInfo.c: SysInfo.xs SysInfo.h
	$(XSUBPP) $*.xs > $*.c

ProcState.c: ProcState.xs
	$(XSUBPP) $*.xs > $*.c

MachInfo.c: MachInfo.xs
	$(XSUBPP) $*.xs > $*.c

MD5Sum.c: MD5Sum.xs
	$(XSUBPP) $*.xs > $*.c

SystemUnix.c: SystemUnix.xs
	$(XSUBPP) $*.xs > $*.c

SystemUnixDir.c: SystemUnixDir.xs
	$(XSUBPP) $*.xs > $*.c 

SystemWin32Registry.c: SystemWin32Registry.xs
	$(XSUBPP) $*.xs > $*.c

SystemWin32API.c: SystemWin32API.xs
	$(XSUBPP) $*.xs > $*.c

SystemWin32Console.c: SystemWin32Console.xs
	$(XSUBPP) $*.xs > $*.c  

install: all
	$(CP) $(LIBSDBRUN) $(INSTPREFIX)
	$(INSTALLWX)
	$(CP) SDBRUN$(EXEEXT) $(INSTPREFIX)

clean:
	cd readkey && $(MAKE) clean
	$(RM) $(ALL_PL_H)
	$(RM) $(ALL_XS_C) 
	$(RM) $(ALL_XS_OBJ) 
	$(RM) $(ALL_XS_DLL)
	$(RM) $(LIBSDBRUN)
	$(RM) SDBRUN$(EXEEXT)
	$(RM) SDBRun$(OBJEXT)
	$(RM) Makefile
