#!/usr/bin/perl
#
# SDB::Common::Require.pm
#

#line 7 SDB::Common::Require.pm

package SDB::Common::Require;

our $modules = {};

sub requireex {
	my ($loc, $mod) = @_;

	# already loaded
	return 1 if (exists $modules->{$mod}->{loaded});

	unless (exists ($modules->{$mod}->{txt})) {
		if (-f $loc) {
			# load from archive

			my $ah = SDB::Common::Archive::new ($loc);
			return undef unless (defined $ah);

			for (;;) {
				my $fn = $ah->read ();
				last unless (defined $fn);

				my $fh = $ah->open ($fn);
				$modules->{$fn}->{txt} = $fh->read ();
			}
		} else {
			# load from filesystem

			my $file = $loc.'/'.$mod;
			return undef unless (-r $file);

			open (IN, $file);
			$modules->{$mod}->{txt} = join ("\n", <IN>);
			close (IN);
		}
	}

	return undef unless (exists ($modules->{$mod}->{txt}));

	$modules->{$mod}->{loaded} = 1;
	my @lines = (split '\n', $modules->{$mod}->{txt});

	# set filename for compilation
	unshift @lines, '#line 1 '.$mod;

	# move @EXPORTER and @ISA assignments into BEGIN blocks
	my $inbegin = 0;
	foreach my $line (@lines) {
		chomp $line;
		last if ($line =~ /^__END__/);

		if ($inbegin && $line =~ /;\s*$/) {
			$line .= "}";
			$inbegin = 0;
			next;
		}

		next unless ($line =~ /EXPORT/ || $line =~ /ISA/);

		next unless (
		$line =~ /^\s*our\s+\@EXPORT\s*=/ ||
		$line =~ /^\s*\@EXPORT\s*=/ ||
		$line =~ /^\s*our\s+\@ISA\s*=/ ||
		$line =~ /^\s*\@ISA\s*=/);

		if ($line =~ /;\s*$/) {
			$line = "sub BEGIN {".$line."}";
			next;
		}

		$line = "sub BEGIN {".$line;
		$inbegin = 1;
	}

	# prevent eval form print to STDERR and die ('__WARN__')
	local $SIG{'__WARN__'} = sub {};

	my $rc = eval (join ("\n", @lines));

	if ($@) {
		print $@;
		$rc = undef;

		#
		# this will cause a top env error,
		# has to be fixed later
		#
		die $@;
	} elsif (!$rc) {
		my $err = "require: $mod does not return a true value\n";
		print $err;

		#
		# this will cause a top env error,
		# has to be fixed later
		#
		die $err;
	}

	$modules->{$mod}->{loaded} = 0;
	$modules->{$mod}->{txt} = undef;
	return ($rc);
}

sub require {
	my ($mod) = @_;

	# version format like 5.005 or 5.005003
	# causes error in CORE::require
	return 1 if ($mod =~ /^\d[\.,]\d+$/);

	# get normaized filename
	$mod =~ s/::/\//g;
	$mod =~ s/\\/\//g;
	$mod .= '.pm' unless ($mod =~ /\.pm$/);

	# if absolute path does not exist
	unless ($mod =~ /^\/|^\\\\|^[A-Z]:\\|^[a-z]:\\/) {
		# and module is not in search path
		foreach my $loc (@INC) {
			# then try require extention
			return 1 if (requireex ($loc, $mod));
		}
	}

	# otherwise use perl buildin require
	# return (CORE::require ($mod));

	return undef;
}

1;
