/*
 * SetUser.xs
 * 
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SetUser.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SetUser.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#ifdef UNIX
#include <unistd.h>
#include <grp.h>
#include <pwd.h>
#endif

#ifdef __GNUC__
#define GCC_INLINE inline
#else
#define GCC_INLINE static
#endif

typedef struct {
	uid_t uid;
	uid_t euid;
	gid_t gid;
	gid_t egid;
} ids_t;

#define ID_TYPE_UID 1
#define ID_TYPE_GID 2

#ifndef HAS_SETEUID
#ifdef HAS_SETRESUID
static
int seteuid (uid_t euid) {
	return setresuid (-1, euid, -1);
}
#endif
#endif

#ifndef HAS_SETEGID
#ifdef HAS_SETRESGID
static
int setegid (gid_t egid) {
	return setresgid (-1, egid, -1);
}
#endif
#endif

#ifdef UNIX
GCC_INLINE void 
get_ids (ids_t *ids) {
	ids->uid = getuid ();
	ids->euid = geteuid ();
	ids->gid = getgid ();
	ids->egid = getegid ();
}

GCC_INLINE int
sv_2id (SV *sv, uid_t *p_id, int id_type) {
	int len;
	char *name;
	struct passwd *pwd;
	struct group *grp;
	
	if (SvIOKp (sv)) {
		*p_id = SvIV (sv);
		return 0;
	} else if (SvPOKp (sv)) {
		name = (char *) SvPV (sv, PL_na);	
		if (strlen (name) == 0) {
			return (-1);
		}
		switch (id_type) {
		case ID_TYPE_UID:
			pwd = getpwnam (name);
			if (pwd == 0) {
				return (-1);
			}
			*p_id = pwd->pw_uid;
			break;
		case ID_TYPE_GID:
			grp = getgrnam (name);
			if (grp == 0) {
				return (-1);
			}
			*p_id = grp->gr_gid;
			break;
		default:
			return (-1);
		}		
		return 0;
	} else {
		return (-1);
	}
}

GCC_INLINE int
get_primary_gid (uid_t uid, gid_t *p_gid) {
	struct passwd *pwd;
	pwd = getpwuid (uid);
	if (pwd == 0) {
		return (-1);
	}
	
	*p_gid = pwd->pw_gid;
	return 0;
}

GCC_INLINE int
SetUser (int items, SV* sv0, SV* sv1, char *err) {
	ids_t start;
	uid_t euid;
	gid_t egid;
	
	/* save our current ids */
	get_ids (&start);

	/* only root can do this */
	if (start.uid != 0) {
		strcpy (err,"only root can do this");
		return -1;
	}

	switch (items) {
	case 0:
		/* set to real uid and gid if no parameter are given */
		euid = start.uid;
		egid = start.gid;
		break;
	case 1:
		if (sv_2id (sv0, &euid, ID_TYPE_UID) != 0) {
			strcpy (err, "cannot get euid");
			return -1;
		}
		if (get_primary_gid (euid, &egid) != 0) {
			strcpy (err, "primary group found");
			return -1;
		}
		break;
	case 2:
		if (sv_2id (sv0, &euid, ID_TYPE_UID) != 0) {
			strcpy (err, "cannot get euid");
			return -1;
		}
		if (sv_2id (sv1, &egid, ID_TYPE_GID) != 0) {
			strcpy (err, "cannot get egid");
			return -1;
		}
		break;
	default:
		strcpy (err, "invalid argument count");
		return -1;
	}
	
	if (setegid (egid) != 0) {
		sprintf (err, "setegid (%d) failed\n", egid); 
		return -1;
	}

	if (seteuid (euid) != 0) {
		setegid (start.egid);
		sprintf (err, "setegid (%d) failed\n", egid); 
		return -1;
	}
	strcpy (err, "");
	return 0;	
}
#endif

#ifdef WIN32
GCC_INLINE int
SetUser (int items, SV* sv0, SV* sv1, char *err) {
	strcpy (err, "not implemented on WIN32");
	return -1;
}
#endif

/*
 * SAPDB::Install::SetUser
 * 
 * sets euid and egid to do things temporarily as a nonpreveleged user
 * your real uid must be 0, only root can do this
 *
 * if no arguments are given, euid and egid are set to uid and gid
 *
 * if one argument is given,
 * this is the uid or the name of the user,
 * the primary gid of this user will be used as egid
 * 
 * if two arguments are given,
 * the first argument is the uid or username
 * and the second argument is the gid or groupname
 * 
 * returns (-1) on error or 0 on success 
 */

MODULE = SAPDB::Install::SetUser	PACKAGE = SAPDB::Install::SetUser

PROTOTYPES: DISABLE

void
SetUser (...)
PREINIT:
	int retcode;
	char err[256];
PPCODE:

	retcode = SetUser (items, ST(0), ST(1), err);
	if (retcode != 0 && GIMME == G_ARRAY) {
		PUSHs (sv_2mortal (newSViv (retcode)));
		PUSHs (sv_2mortal (newSVpv (err, strlen (err))));
		XSRETURN (2);
	}

	PUSHs (sv_2mortal (newSViv (retcode)));
	XSRETURN (1);
