/* 
 * SystemUnix.xs
 * 
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SystemUnix.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SystemUnix.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#ifndef HAS_SETEUID
#ifdef HAS_SETRESUID
static
int seteuid (uid_t euid) {
	return setresuid (-1, euid, -1);
}
#endif
#endif

#ifndef HAS_SETEGID
#ifdef HAS_SETRESGID
static
int setegid (gid_t egid) {
	return setresgid (-1, egid, -1);
}
#endif
#endif

#if defined linux || defined sun || defined SNI
#define HAS_SHADOW HAS_SHADOW
#endif

#if defined _AIX 
#define HAS_USERPW HAS_USERPW
#include <errno.h>
#endif

#if defined __hpux || defined __osf__
#define HAS_PRPWNAM HAS_PRPWNAM
#endif

typedef struct {
	char *name;
	char *pwd;
	int locked;
	int lastchg;
	int min;
	int max;
	int warn;
	int inact;
	int expire;
} myspent_t;

typedef struct {
	char * sysname;
	char * nodename;
	char * release;
	char * version;
	char * machine;
} myutsname_t;

#ifdef UNIX
#include <stdlib.h>
#include <pwd.h>
#include <sys/utsname.h>

static int
is_locked_password (char *password) {
	int i, len;

	if (password == 0)
		return 0;

	len = strlen (password);

	/*
	 * login without password means unlocked
	 */
	if (len == 0)
		return 0;

	/*
	 * account is locked by too short encrypted password
	 * crypt () expects 13 characters
	 */
	if (len < 13)
		return 1;

	/* 
	 * account is locked by invalid character
	 * valid character are [A-Za-z0-9/.]
	 */
	for (i = 0; i < len; i++) {
		if (password[i] >= 'A' && password[i] <= 'Z')
			continue;

		if (password[i] >= 'a' && password[i] <= 'z')
			continue;

		if (password[i] >= '0' && password[i] <= '9')
			continue;

		if (password[i] == '.')
			continue;

		if (password[i] == '/')
			continue;

		return 1;
	}

	return 0;
}

#endif

#if defined HAS_SHADOW
#include <shadow.h>

static myspent_t *
mygetspnam (char *name) {
	myspent_t *ent;
	struct spwd *spwd;
	struct passwd *pwd;

	pwd = getpwnam (name);
	
	/* give up if user does not exists */
	if (pwd == 0)
		return 0;

	ent = (myspent_t *) malloc (sizeof (myspent_t));
	if (ent == 0)
		return 0;

	memset (ent, 0, sizeof (myspent_t));

	/* first use shadow password */
	spwd = getspnam (name);
	if  (spwd != 0) {
		ent->name = (char *) malloc (strlen (spwd->sp_namp) + 1);
		if (ent->name == 0) {
			free (ent);
			return 0;
		}

		strcpy (ent->name, spwd->sp_namp);

		ent->pwd = (char *) malloc (strlen (spwd->sp_pwdp) + 1);
		if (ent->pwd == 0) {
			free (ent->pwd);
			free (ent);
			return 0;
		}
	
		strcpy (ent->pwd, spwd->sp_pwdp);

		ent->lastchg = spwd->sp_lstchg;
		ent->min = spwd->sp_min;
		ent->max = spwd->sp_max;
		ent->warn = spwd->sp_warn;
		ent->inact = spwd->sp_inact;
		ent->expire = spwd->sp_expire;
	} else {
		ent->name = (char *) malloc (strlen (pwd->pw_name) + 1);
		if (ent->name == 0) {
			free (ent);
			return 0;
		}

		strcpy (ent->name, pwd->pw_name);

		ent->pwd = (char *) malloc (strlen (pwd->pw_passwd) + 1);
		if (ent->pwd == 0) {
			free (ent->pwd);
			free (ent);
			return 0;
		}

		strcpy (ent->pwd, pwd->pw_passwd);
	}

	ent->locked = is_locked_password (ent->pwd) ? 1 : 0;

	return (ent);
}

#elif defined HAS_USERPW
#include <userpw.h>
#include <usersec.h>
#include <errno.h>

static myspent_t *
mygetspnam (char *name) {
	myspent_t *ent;
	struct userpw *upw;
	int rc, intval;

	upw = getuserpw (name);
	if (upw == 0){
		if(errno == ENOENT){
		
		   /*
		   *	if there is no user entry in /etc/security/passwd,
		   *    user may have no valid password or there were a synchronization problem
		   *	while creation of user account.
		   *	thus getpwnam is used to verify password in /etc/passwd
		   */
		
			struct passwd *pwd;
			pwd = getpwnam(name);
			if(pwd == 0){
				return 0;
			}
			ent = (myspent_t *) malloc (sizeof (myspent_t));
			if (ent == 0)
				return 0;

			memset (ent, 0, sizeof (myspent_t));
			ent->name = (char *) malloc (strlen (pwd->pw_name) + 1);
			if (ent->name == 0) {
				free (ent);
				return 0;
			}

			ent->pwd = (char *) malloc (strlen (pwd->pw_passwd) + 1);
			if (ent->pwd == 0) {
				free (ent->name);
				free (ent);
				return 0;
			}
			strcpy (ent->name, pwd->pw_name);
			strcpy (ent->pwd, pwd->pw_passwd);							
		}
		else
			return 0;	
	}
	else
	{
		ent = (myspent_t *) malloc (sizeof (myspent_t));
		if (ent == 0)
			return 0;

		memset (ent, 0, sizeof (myspent_t));

		ent->name = (char *) malloc (strlen (upw->upw_name) + 1);
		if (ent->name == 0) {
			free (ent);
			return 0;
		}

		ent->pwd = (char *) malloc (strlen (upw->upw_passwd) + 1);
		if (ent->pwd == 0) {
			free (ent->name);
			free (ent);
			return 0;
		}
		strcpy (ent->name, upw->upw_name);
		strcpy (ent->pwd, upw->upw_passwd);
		ent->lastchg = upw->upw_lastupdate;
	}

	intval = 0;

	rc = getuserattr (name, S_LOCKED, &intval, SEC_BOOL);
	

	/* account is also locked if password is invalid for NIS */
	ent->locked = intval;
	ent->locked |= is_locked_password (ent->pwd) ? 1 : 0;

	return (ent);
}

#elif defined HAS_PRPWNAM

#include <sys/types.h>
#include <prot.h>
#ifdef __osf__
#include <sys/security.h>
#elif defined  __hpux
#include <hpsecurity.h>
#endif

static myspent_t *
mygetspnam (char *name) {
	myspent_t *ent;
	struct passwd *pwd;

	ent = (myspent_t *) malloc (sizeof (myspent_t));
	if (ent == 0)
		return 0;

	memset (ent, 0, sizeof (myspent_t));

	pwd = getpwnam (name);
	if (pwd == 0) {
		free (ent);
		return 0;
	}

	if (pwd->pw_passwd[0], '*' == 0) {
		/* protected password is used */

		struct pr_passwd *spwd;

		spwd = getprpwnam (name);
		if (spwd != 0) {

			ent->name = (char *) malloc (strlen (spwd->ufld.fd_name) + 1);
			if (ent->name == 0) {
				free (ent);
				return 0;
			}

			ent->pwd =
			(char *) malloc (strlen (spwd->ufld.fd_encrypt) + 1);

			if (ent->pwd == 0) {
				free (ent->pwd);
				free (ent);
				return 0;
			}

			strcpy (ent->name, spwd->ufld.fd_name);
			strcpy (ent->pwd, spwd->ufld.fd_encrypt);
			ent->lastchg = spwd->ufld.fd_schange;
			ent->min = spwd->ufld.fd_min;
			ent->locked = (spwd->ufld.fd_lock == 0) ? 0 : 1;
#ifdef __hpux
			ent->expire = spwd->ufld.fd_acct_expire;
#endif

			return (ent);
		}
	}

	ent->name = (char *) malloc (strlen (pwd->pw_name) + 1);
	if (ent->name == 0) {
		free (ent);
		return 0;
	}

	ent->pwd = (char *) malloc (strlen (pwd->pw_passwd) + 1);
	if (ent->pwd == 0) {
		free (ent->pwd);
		free (ent);
		return 0;
	}

	strcpy (ent->name, pwd->pw_name);
	strcpy (ent->pwd, pwd->pw_passwd);

	ent->locked = is_locked_password (ent->pwd) ? 1 : 0;

#ifdef __osf__
	if (strcmp (ent->pwd, "Nologin") == 0)
		ent->locked = 1;
#endif

	return (ent);
}

#else

static myspent_t *
mygetspnam (char *name) {
	return 0;
}

#endif

#ifdef UNIX

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#endif

typedef struct {
	uid_t uid;
	gid_t gid;
	int count;
	gid_t *groups;
} ids_t;

#ifdef UNIX

static int
mylchown (char *filename, uid_t uid, gid_t gid) {
	return (lchown (filename, uid, gid));	
}

typedef struct {
	int count;
	gid_t *groups;
} groups_t;

static int
myuaccess (char *username, char *filename) {
	int i, rc, num;
	struct stat sb[1];

	gid_t mygid;
	groups_t sav[1];
	struct passwd *pw;

	if (getuid () != 0)
		return -1;

	mygid = getgid ();

	pw = getpwnam (username);
	if (pw == 0)
		return -1;

	memset (sav, 0, sizeof (groups_t));
	sav->count = getgroups (0, 0);
	if (sav->count != 0) {
		sav->groups =
		(gid_t *) malloc (sav->count * sizeof (gid_t));
		if (sav->groups == 0)
			return -1;

		getgroups (sav->count, sav->groups);
	}

	initgroups (username, pw->pw_gid);
	setegid (pw->pw_gid);
	seteuid (pw->pw_uid);

	rc = stat (filename, sb);

	seteuid (0);
	setegid (mygid);
	setgroups (sav->count, sav->groups);

	if (sav->groups != 0)
		free (sav->groups);

	return (rc);
}

static ids_t *
myugetids (char *username) {
	ids_t *ids;
	gid_t mygid;
	groups_t sav[1];
	struct passwd *pw;

	if (getuid () != 0)
		return 0;

	mygid = getgid ();

	pw = getpwnam (username);
	if (pw == 0)
		return 0;

	memset (sav, 0, sizeof (groups_t));
	sav->count = getgroups (0, 0);
	if (sav->count != 0) {
		sav->groups =
		(gid_t *) malloc (sav->count * sizeof (gid_t));
		if (sav->groups == 0)
			return 0;

		getgroups (sav->count, sav->groups);
	}

	ids = (ids_t *) malloc (sizeof (ids_t));
	if (ids == 0) {
		if (sav->groups != 0)
			free (sav->groups);

		return 0;
	}

	memset (ids, 0, sizeof (ids_t));
	
	initgroups (username, pw->pw_gid);
	setegid (pw->pw_gid);
	seteuid (pw->pw_uid);

	ids->uid = pw->pw_uid;
	ids->gid = pw->pw_gid;
	ids->count = getgroups (0, 0);
	if (ids->count != 0) {
		ids->groups =
		(gid_t *) malloc (ids->count * sizeof (gid_t));
		if (ids->groups == 0) {
			seteuid (0);
			setegid (mygid);

			setgroups (sav->count, sav->groups);
			if (sav->groups != 0)
				free (sav->groups);

			free (ids);
			return 0;
		}

		getgroups (ids->count, ids->groups);
	}

	seteuid (0);
	setegid (mygid);

	setgroups (sav->count, sav->groups);
	if (sav->groups != 0)
		free (sav->groups);

	return (ids);
}

static int
myuname(myutsname_t * name){
	int rc;
	struct utsname utsname;	
	rc = uname(&utsname);
	if(rc < 0)
		return rc;
	name->nodename = (char * ) malloc(strlen(utsname.nodename) + 1);
	strcpy(name->nodename,utsname.nodename);
	name->sysname = (char * ) malloc(strlen(utsname.sysname) + 1);
	strcpy(name->sysname,utsname.sysname);
	name->release = (char * ) malloc(strlen(utsname.release) + 1);
	strcpy(name->release,utsname.release);
	name->version = (char * ) malloc(strlen(utsname.version) + 1);
	strcpy(name->version,utsname.version);
	name->machine = (char * ) malloc(strlen(utsname.machine) + 1);
	strcpy(name->machine,utsname.machine);
	return rc;
}


#endif

#ifdef WIN32

static int
mylchown (char *filename, uid_t uid, gid_t gid) {
	return -1;
}

static int
myuaccess (char *username, char *filename) {
	return -1;
}

static int
myuname (myutsname_t * name) {
	return -1;
}

static ids_t *
myugetids (char *username) {
	return 0;
}

#endif

static void
myfreeids (ids_t *ids) {
	if (ids == 0)
		return;

	if (ids->groups != 0)
		free (ids->groups);

	free (ids);
	return;
}

static char *
mysprintmode (mode_t mode) {
	mode_t mask;
	char *buff, *ptr;

	const char *all = "rwxrwxrwx";

	buff = (char *) malloc (strlen (all) + 1);
	if (buff == 0)
		return 0;

	strcpy (buff, all);
	ptr = buff;
	mode &= 07777;

	for (mask = 0400; mask != 0; mask >>= 1) {
		if ((mode & mask) == 0)
			*ptr = '-';

		if (
		(mask == 0100) && (mode & 04000) ||
		(mask == 0010) && (mode & 02000)) {
			if ((mask & mode) == 0) {
				*ptr = 'S';
			} else {
				*ptr = 's';
			}
		}

		if ((mask == 0001) && (mode & 01000)) {
			if ((mask & mode) == 0) {
				*ptr = 'T';
			} else {
				*ptr = 't';
			}
		}

		ptr++;
	}

	*ptr = '\0';
	return (buff);
}

MODULE = SAPDB::Install::System::Unix	PACKAGE = SAPDB::Install::System::Unix

PROTOTYPES: DISABLE

void
getspnam (...)
PREINIT:
	char *name;
	myspent_t *ent;
	int cnt;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	name = (char*) SvPV(ST(0), PL_na);

	ent = mygetspnam (name);
	if (ent == 0)
		XSRETURN_UNDEF;

	if (GIMME == G_ARRAY) {
		cnt = 9;
		XPUSHs (sv_2mortal (newSVpv (ent->name, strlen (ent->name))));
		XPUSHs (sv_2mortal (newSVpv (ent->pwd, strlen (ent->pwd))));
		XPUSHs (sv_2mortal (newSViv (ent->locked)));
		XPUSHs (sv_2mortal (newSViv (ent->lastchg)));
		XPUSHs (sv_2mortal (newSViv (ent->min)));
		XPUSHs (sv_2mortal (newSViv (ent->max)));
		XPUSHs (sv_2mortal (newSViv (ent->warn)));
		XPUSHs (sv_2mortal (newSViv (ent->inact)));
		XPUSHs (sv_2mortal (newSViv (ent->expire)));
	} else {
		cnt = 1;	
		XPUSHs (sv_2mortal (newSVpv (ent->pwd, strlen (ent->pwd))));
	}
 
	free (ent->name);
	free (ent->pwd);
	free (ent);

	XSRETURN (cnt);


void
lchown (...)
PREINIT:
	uid_t uid;
	gid_t gid;
	char *filename;
	int i, rc;
PPCODE:
	if (items <= 2)
		XSRETURN_UNDEF;

	uid = (uid_t) SvIV(ST(0));
	gid = (gid_t) SvIV(ST(1));

	for (i = 2; i < items; i++) {
		filename = (char *) SvPV(ST(i), PL_na);
		rc = mylchown (filename, uid, gid);
		if (rc != 0)
			break;
	}

	XPUSHs (sv_2mortal (newSViv (i - 2)));
	XSRETURN (1);

void
uaccess (...)
PREINIT:
	char *username;
	char *filename;
	int rc;
PPCODE:
	if (items != 2)
		XSRETURN_UNDEF;

	username = (char *) SvPV(ST(0), PL_na);
	if (username == 0 || username[0] == '\0')
		XSRETURN_UNDEF;

	filename = (char *) SvPV(ST(1), PL_na);
	if (filename == 0 || filename[0] == '\0')
		XSRETURN_UNDEF;

	rc = myuaccess (username, filename);

	XPUSHs (sv_2mortal (newSViv (rc)));
	XSRETURN (1);

void
ugetids (...)
PREINIT:
	char *username;
	ids_t *ids;
	int i;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	username = (char *) SvPV(ST(0), PL_na);
	if (username == 0 || username[0] == '\0')
		XSRETURN_UNDEF;

	ids = myugetids (username);

	if (ids == 0)
		XSRETURN_UNDEF;

	XPUSHs (sv_2mortal (newSViv (ids->uid)));
	XPUSHs (sv_2mortal (newSViv (ids->gid)));
	for (i = 0; i < ids->count; i++) {
		XPUSHs (sv_2mortal (newSViv (ids->groups[i])));
	}

	free (ids);

	XSRETURN (i + 2);

void
sprintmode (...)
PREINIT:
	mode_t mode;
	char *buff;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	mode = (mode_t) SvIV(ST(0));
	buff = mysprintmode (mode);
	if (buff == 0 || *buff == '\0')
		XSRETURN_UNDEF;

	XPUSHs (sv_2mortal (newSVpv (buff, strlen (buff))));
	free (buff);

	XSRETURN (1);

void
uname (...)
PREINIT:
	int rc;
	SV * rv;
	HV * hv;
	myutsname_t name;
	char * sznode_key = "nodename";
	char * szsysname_key = "sysname";
	char * szrelease_key = "release";
	char * szversion_key = "version";
	char * szmachine_key = "machine";
PPCODE:
	if (items != 0)
		XSRETURN_UNDEF;
	
	rc = myuname (&name);
	
	if (rc < 0)
		XSRETURN_UNDEF;
	
	hv = newHV();

	hv_store(hv,sznode_key,strlen(sznode_key),
			 newSVpv (name.nodename, strlen (name.nodename)), 0);
	hv_store(hv,szsysname_key,strlen(szsysname_key),
			 newSVpv (name.sysname, strlen (name.sysname)), 0);
	hv_store(hv,szrelease_key,strlen(szrelease_key),
			 newSVpv (name.release, strlen (name.release)), 0);
	hv_store(hv,szversion_key,strlen(szversion_key),
			 newSVpv (name.version, strlen (name.version)), 0);
	hv_store(hv,szmachine_key,strlen(szmachine_key),
			 newSVpv (name.machine, strlen (name.machine)), 0);

	free (name.nodename);
	free (name.sysname);
	free (name.release);
	free (name.version);
	free (name.machine);
	
	rv = sv_2mortal (newRV ((SV *) hv));
	SvREFCNT_dec (hv);

	XPUSHs (rv);
	
	XSRETURN (1);


