/*
 * SystemUnix.xs
 * 
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SystemUnixDir.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SystemUnixDir.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

typedef struct fnode {
	char *name;
	mode_t mode;
	struct fnode *child;
	struct fnode *parent;
	struct fnode *sibling;
	struct fnode *next;
} fnode_t;

#ifdef UNIX

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <sys/stat.h>
#include <sys/types.h>

static fnode_t *getdirnodes (char *);
static fnode_t *addnode (fnode_t *, fnode_t *, char *);
static fnode_t *iteratenode (fnode_t *, fnode_t *);
static fnode_t *unifynodes (fnode_t *);
static void freenodes (fnode_t *);
static char *getparentdir (char *);

#if 0
static void
printall (fnode_t *root) {
	fnode_t *curr;

	printf (">>>\n");
	for (curr = root; curr != 0; curr = curr->next) {
		printf (">>>%s\n", curr->name);
	}
	printf (">>>\n");

	return;
}
#endif

static fnode_t *
getdirnodes (char *filename) {
	fnode_t *root, *curr;

	/* create initial node */
	root = addnode (0, 0, filename);
	if (root == 0)
		return 0;

	/* find all parent directories of root node */
	for (curr = root; curr != 0; curr = iteratenode (root, curr)) {
		char *name;

		name = getparentdir (curr->name);
		if (name == 0) {
			freenodes (root);
			return 0;
		}

		if (addnode (root, curr, name) == 0) {
			freenodes (root);
			return 0;
		}

		free (name);
	}

	root = unifynodes (root);

	return (root);
}

static fnode_t *
unifynodes (fnode_t *root) {
	int removed, sorted;
	fnode_t *curr, *last, *tmp;

	/* remove navigation information */
	for (curr = root; curr != 0; curr = curr->next) {
		curr->child = 0;
		curr->parent = 0;
		curr->sibling = 0;
	}

	/* remove any other files then directories */
	last = 0;
	for (curr = root; curr != 0;) {
		if ((curr->mode & S_IFDIR) == S_IFDIR) {
			last = curr;
			curr = curr->next;
			continue;
		}

		if (last == 0)
			root = curr->next;
		else
			last->next = curr->next;

		free (curr->name);
		free (curr);

		if (last == 0)
			curr = root;
		else
			curr = last->next;
	}

	/* remove duplicated names */
	last = 0;
	for (curr = root; curr != 0;) {
		removed = 0;
		for (tmp = root; tmp != 0 && tmp != curr; tmp = tmp->next) {
			if (strcmp (curr->name, tmp->name) == 0) {
				removed  = 1;
				if (last == 0)
					root = curr->next;
				else
					last->next = curr->next;

				free (curr->name);
				free (curr);
			}
		}

		if (removed == 0) {
			last = curr;
			curr = curr->next;
		} else {
			if (last == 0)
				curr = root;
			else
				curr = last->next;
		}
	}

	/* sort by name */
	for (;;) {
		sorted = 0;
		for (curr = root; curr->next != 0; curr = curr->next) {
			if (strcmp (curr->name, curr->next->name) >= 0) {
				char *name;

				name = curr->name;
				curr->name = curr->next->name;
				curr->next->name = name;

				sorted++;
			}
		}

		if (sorted == 0)
			break;
	}

	return (root);
}

static fnode_t *
addnode (fnode_t *root, fnode_t *last, char *filename) {
	int rc;
	fnode_t *curr, *tmp;
	char *name, *ptr;
	int i, linkcnt;
	char **links;
	struct stat sb[1];
	mode_t mode;

	/* initialize */
	linkcnt = 0;
	links = malloc ((linkcnt + 1) * sizeof (char *));
	if (links == 0)
		return 0;

	links[linkcnt] = 0;

	name = (char *) malloc (strlen (filename) + 1);
	if (name == 0)
		return 0;

	strcpy (name, filename);

	for (;;) {
		int buffsize = 4096;

		/* check if file exists */
		rc = lstat (name, sb);
		if (rc != 0) {
			free (name);
			return 0;
		}

		/* check if file is a symlink */
		if ((sb->st_mode & S_IFLNK) != S_IFLNK)
			break;

		linkcnt++;
		links = (char **) realloc (links, (linkcnt + 1) * sizeof (char *));
		if (links == 0) {
			return 0;
		}

		links[linkcnt - 1] = name;
		links[linkcnt] = 0;

		name = (char *) malloc (buffsize + 1);
		if (name == 0) {
			for (i = 0; i < linkcnt; i++) {
				free (links[i]);
				free (links);
			}
			return 0;
		}

		memset (name, 0, buffsize + 1);
		rc = readlink (links[linkcnt - 1], name, buffsize);
		if (rc < 0) {
			free (name);
			for (i = 0; i < linkcnt; i++) {
				free (links[i]);
				free (links);
			}
			return 0;
		}

		/* check for rel path */
		if (*name != '/') {
			strcpy (name, links[linkcnt - 1]);
			ptr = strrchr (name, '/');
			ptr++;
			*ptr = '\0';

			buffsize -= ptr - name;
			rc = readlink (links[linkcnt - 1], ptr, buffsize);
			if (rc < 0) {
				free (name);
				for (i = 0; i < linkcnt; i++) {
					free (links[i]);
					free (links);
				}
				return 0;
			}
		}

		name = realloc (name, strlen (name) + 1);
		if (name == 0) {
			for (i = 0; i < linkcnt; i++) {
				free (links[i]);
				free (links);
			}
			return 0;
		}
	}

	/* get mode of original file */
	rc = stat (name, sb);
	if (rc != 0) {
		free (name);
		for (i = 0; i < linkcnt; i++) {
			free (links[i]);
			free (links);
		}
		return 0;
	}

	mode = sb->st_mode;
	
	/* add node for original file */
	curr = (fnode_t *) malloc (sizeof(fnode_t));
	if (curr == 0) {
		free (name);
		for (i = 0; i < linkcnt; i++) {
			free (links[i]);
			free (links);
		}
		return 0;
	}

	memset (curr, 0, sizeof (fnode_t));
	curr->name = name;
	curr->mode = mode;

	if (root == 0) {
		root = curr;
	} else {
		for (tmp = root; tmp->next != 0; tmp = tmp->next);
		tmp->next = curr;
		curr->child = last;
		last->parent = curr;
	}

	if (linkcnt == 0)
		return (curr);

	/* add node for sibling */
	for (i = 0; i < linkcnt; i++) {
		curr->sibling = (fnode_t *) malloc (sizeof(fnode_t));
		if (curr->sibling == 0) {
			int j;

			for (j = i; j < linkcnt; j++) {
				free (links[j]);
				free (links);
			}
			return 0;
		}

		memset (curr->sibling, 0, sizeof (fnode_t));
		curr->sibling->name = links[i];
		curr->sibling->mode = mode;

		for (tmp = root; tmp->next != 0; tmp = tmp->next);
		tmp->next = curr->sibling;

		curr->sibling->child = last;
	}
	free (links);

	return (curr);
}

static fnode_t *
iteratenode (fnode_t *root, fnode_t *curr) {
	fnode_t *ptr;

	/* first try parent */
	ptr = curr->parent;
	if (ptr != 0 && strcmp (ptr->name, "/") != 0)
		return (ptr);

	/* go back until symlink */
	for (ptr = curr; ptr != 0; ptr = ptr->child) {
		if (ptr->sibling != 0 && strcmp (ptr->sibling->name, "/") != 0)
			return (ptr->sibling);
	}

	return 0;
}

static char *
getparentdir (char *name) {
	char *parent, *ptr;

	parent = (char *) malloc (strlen (name) + 1);
	if (parent == 0)
		return 0;

	strcpy (parent, name);

	ptr = strrchr (parent, '/');
	if (ptr == 0) {
		free (parent);
		return 0;
	}

	/* keep slash in root */
	if (ptr == parent)
		ptr++;

	*ptr = '\0';

	parent = realloc (parent, strlen (parent) + 1);
	if (parent == 0)
		return 0;

	return (parent);
}
#endif

#ifdef WIN32

static fnode_t *
getdirnodes (char *filename) {
	return 0;
}

#endif

static void
freenodes (fnode_t *root) {
	fnode_t *curr;

	if (root == 0)
		return;

	for (curr = root; curr != 0;) {
		fnode_t *ptr;

		ptr = curr;

		curr = curr->next;
		if (ptr->name != 0)
			free (ptr->name);

		free (ptr);
	}
}

MODULE = SAPDB::Install::System::Unix::Dir PACKAGE = SAPDB::Install::System::Unix::Dir

PROTOTYPES: DISABLE

void
getancestors (...)
PREINIT:
	int i;
	char *filename;
	fnode_t *root, *curr;
PPCODE:
	if (items != 1)
		XSRETURN_UNDEF;

	filename = (char *) SvPV(ST(0), PL_na);
	if (filename == 0 || filename[0] == '\0')
		XSRETURN_UNDEF;

	root = getdirnodes (filename);

	i = 0;
	for (curr = root; curr != 0; curr = curr->next) {
		i++;
		XPUSHs (sv_2mortal (newSVpv (curr->name, strlen (curr->name))));
	}

	freenodes (root);
	XSRETURN (i);
