/* 
 * System::Win32::Registry.xs
 * 
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SystemWin32Registry.xs#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/SystemWin32Registry.xs#1 $";

#ifndef __cplusplus
#ifndef HAS_BOOL
typedef char bool;
#endif
#endif

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <malloc.h>
static const int IsWindows = 1;
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/*
 * expect compiler warning 4113 when using Microsoft C
 * in c code generated by xs preprocessor
 */
#if _MSC_VER >= 1200
#pragma warning(disable : 4113)
#endif

/*
 * cast from HKEY to causes compiler warning on 64-bit windows
 * this should be no problem, beause HKEY is not greater then 2G
 */
#if defined _WIN64 && _MSC_VER >= 1200
#pragma warning(disable : 4311)
#endif

#ifndef WIN32
#define UNIX UNIX
#endif

#ifdef UNIX
static const int HKEY_CLASSES_ROOT = 0; 
static const int HKEY_CURRENT_CONFIG = 0;
static const int HKEY_CURRENT_USER = 0;
static const int HKEY_LOCAL_MACHINE = 0;
static const int HKEY_USERS = 0;
static const int HKEY_PERFORMANCE_DATA = 0;
static const int HKEY_DYN_DATA = 0;
static const int KEY_CREATE_LINK = 0;
static const int KEY_CREATE_SUB_KEY = 0;
static const int KEY_ENUMERATE_SUB_KEYS = 0;
static const int KEY_EXECUTE = 0;
static const int KEY_NOTIFY = 0;
static const int KEY_QUERY_VALUE = 0;
static const int KEY_SET_VALUE = 0;
static const int KEY_ALL_ACCESS = 0;
static const int KEY_READ = 0;
static const int KEY_WRITE = 0;
static const int REG_DWORD = 0;
static const int REG_SZ = 0;
static const int REG_EXPAND_SZ = 0;
static const int REG_MULTI_SZ = 0;
static const int REG_BINARY = 0;
static const int REG_OPTION_NON_VOLATILE = 0;
static const int REG_OPTION_VOLATILE = 0;
static const int REG_OPTION_BACKUP_RESTORE = 0;
static const int REG_CREATED_NEW_KEY = 0;
static const int REG_OPENED_EXISTING_KEY = 0;

static const int IsWindows = 0;
#endif

#ifdef WIN32

static int
myRegOpenKeyEx (SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4) {
	char *subkey;
	DWORD options;
	HKEY hkey0, hkey1;
	REGSAM sam;

	hkey0 = (HKEY) SvIV (sv0);
	subkey = (char *) SvPV (sv1, PL_na);	
	options = SvIV (sv2);
	sam = (REGSAM) SvIV (sv3);	

	if (RegOpenKeyEx (hkey0, subkey, options, sam, &hkey1) != ERROR_SUCCESS)
		return -1;

	sv_setiv (sv4, (long) hkey1);
	return 0;
}

static int 
myRegCloseKey (SV *sv0) {
	HKEY hkey;

	hkey = (HKEY) SvIV (sv0);
	if (RegCloseKey (hkey) != ERROR_SUCCESS)
		return -1;

	return 0;
}

static int
myRegQueryValueEx (SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5) {
	int rc;
	char *key;
	char *value;
	long len;
	HKEY hkey;
	DWORD type;
	
	/*
	 * sv0: hkey
	 * sv1: key
	 * sv2: reserved, will be ignored
	 * sv3: type
	 * sv4: value
	 * sv5: len
	 */

	hkey = (HKEY) SvIV (sv0);
	key = (char *) SvPV (sv1, PL_na);	

	if (RegQueryValueEx (hkey, key, 0, &type, 0, (unsigned long *) &len) != ERROR_SUCCESS)
		return -1;

	value = (char *) _alloca (len + 1);
	if (value == 0)
		return -1;

	if (RegQueryValueEx (hkey, key, 0 , &type, (unsigned char *) value, (unsigned long *) &len) != ERROR_SUCCESS)
		return -1;
	
	rc = 0;
	switch (type) {
	case REG_DWORD:
		sv_setiv (sv4, *(DWORD *) value);
		break;
	case REG_SZ:
	case REG_EXPAND_SZ:
		len = strlen (value);
		sv_setpvn (sv4, value, len);
		break;
	case REG_BINARY:
		sv_setpvn (sv4, value, len);
		break;
	case REG_MULTI_SZ:
		{
			char *ptr;
			AV *av;

			av = newAV ();
			for (ptr = value;; ptr += strlen (ptr) + 1) {
				if (strlen (ptr) == 0)
					break;
				
				av_push (av, newSVpv (ptr, strlen (ptr)));
			}

			sv_setsv (sv4, sv_2mortal (newRV ((SV *) av)));
			SvREFCNT_dec (av);
		}		
		break;
	default:
		rc = -1;
	}

	if (rc == 0 && !SvREADONLY (sv3))
		sv_setiv (sv3, type);

	if (rc == 0 && !SvREADONLY (sv5))
		sv_setiv (sv5, len);

	return rc;
}

static int
myRegQueryInfoKey (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5,
SV *sv6, SV *sv7, SV *sv8, SV *sv9, SV *sv10, SV *sv11) {
	void *hkey;
	DWORD val_count;
	DWORD key_count;
	int rc;
	/*
	 * sv0:    hkey
	 * sv4:    key count
	 * sv7:    value count
	 * others: ignored
	 */

	hkey = (HKEY) SvIV (sv0);

	rc = RegQueryInfoKey (
	     hkey, 0, 0, 0, &key_count, 0, 0, &val_count, 0, 0, 0, 0
		 );
	if (rc != ERROR_SUCCESS)
		return -1;

	if (!SvREADONLY(sv4))
		sv_setiv (sv4, key_count);

	if (!SvREADONLY(sv7))
		sv_setiv (sv7, val_count);
	
	return 0;
}


static int
myRegEnumValue (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5, SV *sv6, SV *sv7) {
	int rc;
	void *hkey;
	DWORD index;
	char *name;
	DWORD name_len;
	DWORD type;
	char *data;
	DWORD data_len;
	DWORD name_len_max;
	/*
	 * sv0: hkey
	 * sv1: index
	 * sv2: name
	 * sv3: name_len
	 * sv4: reserved
	 * sv5: type
	 * sv6: data
	 * sv7: data_len
	 */

	hkey = (HKEY) SvIV (sv0);
	index = (DWORD) SvIV (sv1);
	
	rc = RegQueryInfoKey (hkey, 0, 0, 0, 0, 0, 0, 0, 0, 0, &name_len_max, 0);
	if (rc != ERROR_SUCCESS)
		return -1;

	name = (char *) _alloca (name_len_max + 1);
	if (name == 0)
		return -1;

	name_len = name_len_max + 1;

	rc = RegEnumValue (hkey, index, (LPTSTR) name, &name_len, 0, 0, 0, &data_len);
	if (rc != ERROR_SUCCESS) {
		return -1;
	}

	data = (char *) _alloca (data_len + 1);
	if (data == 0)
		return -1;

	name_len = name_len_max + 1;
	rc = RegEnumValue (hkey, index, (LPTSTR) name, &name_len, 0, &type, (unsigned char *) data, &data_len);

	if (rc != ERROR_SUCCESS)
		return -1;

	rc = 0;
	switch (type) {
	case REG_DWORD:
		sv_setiv (sv6, *(DWORD *) data);
		break;
	case REG_SZ:
	case REG_EXPAND_SZ:
		data_len = strlen (data);
		sv_setpvn (sv6, data, data_len);
		break;
	case REG_BINARY:
		sv_setpvn (sv6, data, data_len);
		break;
	case REG_MULTI_SZ:
		{
			char *ptr;
			AV *av;

			av = newAV ();
			for (ptr = data;; ptr += strlen (ptr) + 1) {
				if (strlen (ptr) == 0)
					break;
				
				av_push (av, newSVpv (ptr, strlen (ptr)));
			}

			sv_setsv (sv6, sv_2mortal (newRV ((SV *) av)));
			SvREFCNT_dec (av);
		}		
		break;
	default:
		rc = -1;
	}

	if (rc == 0)
		sv_setpvn (sv2, name, name_len);	

	if (rc == 0 && !SvREADONLY (sv5))
		sv_setiv (sv5, type);

	if (rc == 0 && !SvREADONLY (sv7))
		sv_setiv (sv7, data_len);

	return rc;
}

static int
myRegEnumKeyEx (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5, SV *sv6, SV *sv7) {
	int rc;
	void *hkey;
	DWORD index;
	char *name;
	DWORD name_len;
	DWORD name_len_max;
	FILETIME ft;
	/*
	 * sv0: hkey
	 * sv1: index
	 * sv2: name
	 * sv3: name_len
	 * sv4: reserved,     ignored
	 * sv5: class,        ignored
	 * sv6: class_len,    ignored
	 * sv7: p_write_time, ignored
	 */
	
	hkey = (HKEY) SvIV (sv0);
	index = (DWORD) SvIV (sv1);
	
	rc = RegQueryInfoKey (hkey, 0, 0, 0, 0, &name_len_max, 0, 0, 0, 0, 0, 0);
	if (rc != ERROR_SUCCESS)
		return -1;

	name = (char *) _alloca (name_len_max + 1);
	if (name == 0)
		return -1;
	
	name_len = name_len_max + 1; 

	rc = RegEnumKeyEx (hkey, index, (LPTSTR) name, &name_len, 0, 0, 0, &ft);
	if (rc != ERROR_SUCCESS) 
		return -1;

	sv_setpvn (sv2, name, name_len);
	if (!SvREADONLY (sv3))
		sv_setiv (sv3, name_len);

	return 0;
}

static int
myExpandEnvironmentStrings (
SV *sv0, SV *sv1, SV *sv2) {
	char *from;
	char *to;
	int len;
	/*
	 * sv0: string with environment variables
	 * sv1: expandet string
	 * sv2: max len of expandet string, ignored
	 */

	from = (char *) SvPV (sv0, PL_na);
	len = 32 * 1024;
	to = (char *) _alloca (len);
	ExpandEnvironmentStrings (from, to, len);

	if (!SvREADONLY (sv1))
		sv_setpvn (sv1, to, strlen (to));

	return 0;
}

static int
myRegSetValueEx (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5) {
	void *hkey;
	char *value_name;
	DWORD value_type;
	char *sz_data;
	int cb_data;
	int rc;

	/*
	 * sv0: hkey
	 * sv1: value name
	 * sv2: reserved and ignored
	 * sv3: type
	 * sv4: data ptr
	 * sv5: data length, including terminating zero  
	 */
	hkey = (HKEY) SvIV (sv0);
	value_name = SvPV (sv1, PL_na);
	value_type = (DWORD) SvIV (sv3);

	if (value_type == REG_MULTI_SZ) {
		AV *av;
		SV **p_sv;
		int i, i_max, len;
		char *ptr;
			
		if (SvTYPE (sv4) != SVt_RV)
			return -1;

		av = (AV *) SvRV (sv4);
		if (SvTYPE (av) != SVt_PVAV)
			return -1;

		len = 0;
		i_max = av_len (av) + 1;
		for (i = 0; i < i_max; i++) {
			p_sv = av_fetch (av, i, 0);
			len += strlen (SvPV (*p_sv, PL_na)) + 1;
		}
		ptr = (char *) _alloca (len + 1);

		len = 0;
		for (i = 0; i < i_max; i++) {
			p_sv = av_fetch (av, i, 0);
			strcpy (ptr + len, SvPV (*p_sv, PL_na));
			len += strlen (SvPV (*p_sv, PL_na)) + 1;
		}
		strcpy (ptr + len, "");

		rc = RegSetValueEx (hkey, value_name, 0, value_type, (unsigned char *) ptr, len + 1);
	} else {
		sz_data = SvPV (sv4, PL_na);
		cb_data = SvIV (sv5);

		if (cb_data == 0) {
			switch (value_type) {
			case REG_SZ:
			case REG_EXPAND_SZ:
				cb_data = strlen (sz_data) + 1;
				break;
			}
		}
		rc = RegSetValueEx (hkey, value_name, 0, value_type, (unsigned char *) sz_data, cb_data);
	}

	if (rc != ERROR_SUCCESS) 
		return -1;

	return 0;
}

static int
myRegCreateKeyEx (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4,
SV *sv5, SV *sv6, SV *sv7, SV *sv8) {
	HKEY hkey0, hkey1;
	char *subkey_name;
	DWORD options;
	REGSAM sam;
	DWORD disposition;
	int rc;

	/*
	 * sv0: hkey
	 * sv1: subkey name
	 * sv2: reserved and ignored
	 * sv3: class and ignored
	 * sv4: options
	 * sv5: sam
	 * sv6: ptr to security attributes, ignored
	 * sv7: hkey result
	 * sv8: disposition
	 */

	hkey0 = (HKEY) SvIV (sv0);
	subkey_name = SvPV (sv1, PL_na);
	options = (DWORD) SvIV (sv4);
	sam = (REGSAM) SvIV (sv5);

	if (SvREADONLY (sv7) || SvREADONLY (sv8)) {
		/* cannot put back return values */
		rc = -1;
	}

	rc = RegCreateKeyEx (
		 hkey0, subkey_name, 0, 0, options, sam, 0, &hkey1, &disposition);

	if (rc != ERROR_SUCCESS) {
		return -1;
	}

	sv_setiv (sv7, (long) hkey1);
	sv_setiv (sv8, disposition);

	return 0;
}

static int
myRegDeleteValue (SV *sv0, SV *sv1) {
	int rc;
	HKEY hkey;
	char *value_name;

	/*
	 * sv0: hkey
	 * sv1: value name
	 */
	hkey = (HKEY) SvIV (sv0);
	value_name = SvPV (sv1, PL_na);

	rc = RegDeleteValue (hkey, value_name);
	return rc;
}

static int
myRegDeleteKey (SV *sv0, SV *sv1) {
	int rc;
	HKEY hkey;
	char *key_name;

	/*
	 * sv0: hkey
	 * sv1: key name
	 */
	hkey = (HKEY) SvIV (sv0);
	key_name = SvPV (sv1, PL_na);

	rc = RegDeleteKey (hkey, key_name);
	return rc;
}
#endif

#ifdef UNIX
static int 
myRegOpenKeyEx (SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4) {
	return -1;
}

static int
myRegQueryInfoKey (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5,
SV *sv6, SV *sv7, SV *sv8, SV *sv9, SV *sv10, SV *sv11) {
	return -1;
}

static int 
myRegCloseKey (SV *sv0) {
	return -1;
}

static int
myRegQueryValueEx (SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5) {
	return -1;
}

static int
myRegEnumValue (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5, SV *sv6, SV *sv7) {
	return -1;
}

static int
myRegEnumKeyEx (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5, SV *sv6, SV *sv7) {
	return -1;
}

static int
myExpandEnvironmentStrings (
SV *sv0, SV *sv1, SV *sv2) {
	return -1;
}

static int
myRegSetValueEx (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5) {
	return -1;
}

static int
myRegCreateKeyEx (
SV *sv0, SV *sv1, SV *sv2, SV *sv3, SV *sv4, SV *sv5, SV *sv6, SV *sv7, SV *sv8) {
	return -1;
}

static int
myRegDeleteValue (SV *sv0, SV *sv1) {
	return -1;
}

static int
myRegDeleteKey (SV *sv0, SV *sv1) {
	return -1;
}
#endif

MODULE = SAPDB::Install::System::Win32::Registry	PACKAGE = SAPDB::Install::System::Win32::Registry		

PROTOTYPES: DISABLE

void
REG_DOWRD (...)
PREINIT:
PPCODE:
	if (IsWindows) 
		XSRETURN_IV ((long) REG_DWORD);
	XSRETURN_UNDEF;		

void
REG_SZ (...)
PREINIT:
PPCODE:
	if (IsWindows) 
		XSRETURN_IV ((long) REG_SZ);
	XSRETURN_UNDEF;		

void
REG_MULTI_SZ (...)
PREINIT:
PPCODE:
	if (IsWindows) 
		XSRETURN_IV ((long) REG_MULTI_SZ);
	XSRETURN_UNDEF;		

void
REG_EXPAND_SZ (...)
PREINIT:
PPCODE:
	if (IsWindows) 
		XSRETURN_IV ((long) REG_EXPAND_SZ);
	XSRETURN_UNDEF;		

void
REG_BINARY (...)
PREINIT:
PPCODE:
	if (IsWindows) 
		XSRETURN_IV ((long) REG_BINARY);
	XSRETURN_UNDEF;		

void
HKEY_CLASSES_ROOT (...)
PREINIT:
PPCODE:
	if (IsWindows) 
		XSRETURN_IV ((long) HKEY_CLASSES_ROOT);
	XSRETURN_UNDEF;		

void
HKEY_CURRENT_CONFIG (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) HKEY_CURRENT_CONFIG);
	XSRETURN_UNDEF;		

void
HKEY_CURRENT_USER (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) HKEY_CURRENT_USER);
	XSRETURN_UNDEF;		

void
HKEY_LOCAL_MACHINE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) HKEY_LOCAL_MACHINE);
	XSRETURN_UNDEF;

void
HKEY_USERS (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) HKEY_USERS);
	XSRETURN_UNDEF;

void
HKEY_PERFORMANCE_DATA (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) HKEY_PERFORMANCE_DATA);
	XSRETURN_UNDEF;

void
HKEY_DYN_DATA (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) HKEY_PERFORMANCE_DATA);
	XSRETURN_UNDEF;

void
KEY_CREATE_LINK (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_CREATE_LINK);
	XSRETURN_UNDEF;

void
KEY_CREATE_SUB_KEY (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_CREATE_SUB_KEY);
	XSRETURN_UNDEF;

void
KEY_ENUMERATE_SUB_KEYS (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_ENUMERATE_SUB_KEYS);
	XSRETURN_UNDEF;

void
KEY_EXECUTE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_EXECUTE);
	XSRETURN_UNDEF;

void
KEY_NOTIFY (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_NOTIFY);
	XSRETURN_UNDEF;

void
KEY_QUERY_VALUE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_QUERY_VALUE);
	XSRETURN_UNDEF;

void
KEY_SET_VALUE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_SET_VALUE);
	XSRETURN_UNDEF;

void
KEY_ALL_ACCESS (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_ALL_ACCESS);
	XSRETURN_UNDEF;

void
KEY_READ (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_READ);
	XSRETURN_UNDEF;

void
KEY_WRITE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) KEY_WRITE);
	XSRETURN_UNDEF;

void 
REG_OPTION_NON_VOLATILE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) REG_OPTION_NON_VOLATILE);
	XSRETURN_UNDEF;

void 
REG_OPTION_VOLATILE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) REG_OPTION_VOLATILE);
	XSRETURN_UNDEF;

void 
REG_OPTION_BACKUP_RESTORE (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) REG_OPTION_BACKUP_RESTORE);
	XSRETURN_UNDEF;

void 
REG_CREATED_NEW_KEY (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) REG_CREATED_NEW_KEY);
	XSRETURN_UNDEF;

void 
REG_OPENED_EXISTING_KEY (...)
PREINIT:
PPCODE:
	if (IsWindows)
		XSRETURN_IV ((long) REG_OPENED_EXISTING_KEY);
	XSRETURN_UNDEF;

void
RegOpenKeyEx (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 5) {
		rc = myRegOpenKeyEx (ST(0), ST(1), ST(2), ST(3), ST(4));
	}
	XSRETURN_IV (rc);

void
RegCloseKey (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 1) {
		rc = myRegCloseKey (ST(0));
	}
	XSRETURN_IV (rc);

void
RegQueryValueEx (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 6) {
		rc = myRegQueryValueEx (ST(0), ST(1), ST(2), ST(3), ST(4), ST(5));
	}
	XSRETURN_IV (rc);

void
RegQueryInfoKey (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 12) {
		rc = myRegQueryInfoKey (
		ST(0), ST(1), ST(2), ST(3), ST(4), ST(5),
		ST(6), ST(7), ST(8), ST(9), ST(10), ST(11)
		);
	}
	XSRETURN_IV (rc);

void
RegEnumValue (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 8) {
		rc = myRegEnumValue (
		ST(0), ST(1), ST(2), ST(3), ST(4), ST(5), ST(6), ST(7)
		);
	}
	XSRETURN_IV (rc);

void
RegEnumKeyEx (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 8) {
		rc = myRegEnumKeyEx (
		ST(0), ST(1), ST(2), ST(3), ST(4), ST(5), ST(6), ST(7)
		);
		
	}
	XSRETURN_IV (rc);

void
RegSetValueEx (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 6) {
		rc = myRegSetValueEx (
		ST(0), ST(1), ST(2), ST(3), ST(4), ST(5)
		);
		
	}
	XSRETURN_IV (rc);

void
RegCreateKeyEx (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 9) {
	rc = myRegCreateKeyEx (
		ST(0), ST(1), ST(2), ST(3), ST(4), ST(5), ST(6), ST(7), ST(8)
	);
	}
	XSRETURN_IV (rc);

void
RegDeleteValue (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 2) {
		rc = myRegDeleteValue (ST(0), ST(1));
	}
	XSRETURN_IV (rc);

void
RegDeleteKey (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 2) {
		rc = myRegDeleteKey (ST(0), ST(1));
	}
	XSRETURN_IV (rc);

void
ExpandEnvironmentStrings (...)
PREINIT:
	int rc;
PPCODE:
	rc = -1;
	if (items == 3) {
		rc = myExpandEnvironmentStrings (ST(0), ST(1), ST(2));
	}
	XSRETURN_IV (rc);

