/*
 * SDBRun.c 
 *
 * $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/libSDBRun.c#1 $
 * $DateTime: 2005/03/21 16:21:00 $
 * $Change: 93297 $
 *
 */

static char szWhatHeader[] = 
	"@(#) $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/libSDBRun.c#1 $";
 
/*
 * general perl preprocessor code
 */
#ifndef HAS_BOOL
typedef char bool;
#endif

#ifdef __cplusplus
#define externC extern "C"
#else
#define externC extern
#endif

#ifdef __hpux
#define DONT_DECLARE_STD
#endif

#ifdef __cplusplus
externC {
#endif
#include <EXTERN.h>
#include <perl.h>
#include <XSUB.h>
#ifdef __cplusplus
}
#endif

#ifndef WIN32
#define UNIX
#endif

/*
 * preprocessor code for stat ()
 */
#include <sys/types.h>

/*
 * preprocessor code for getcwd ()
 */
#ifdef UNIX
#include <unistd.h>
#endif

/*
 * preprocessor code for alloca ()
 */
#ifdef _AIX
#define NEED_PRAGMA_ALLOCA NEED_PRAGMA_ALLOCA
#endif
#if defined UNIX && !defined _AIX
#define NEED_ALLOCA_H  NEED_ALLOCA_H
#endif
#if defined NEED_ALLOCA_H
#include <alloca.h>
#elif defined NEED_PRAGMA_ALLOCA
#pragma alloca
#endif
#ifdef WIN32
#define alloca(size) _alloca (size)
#endif

/*
 * include some version info
 * these are global symbols named szInstVersion and szChangeList
 */ 
#include "version.h"

#ifndef PERL_EXIT_DESTRUCT_END
#define PERL_EXIT_DESTRUCT_END 0
#endif

/*
 * load initial script
 */
#include "SDBRun.h"

/*
 * real code starts here
 */
static void xs_init (pTHX);

#ifdef UNIX
/*
 * return path of executeable form arg0
 */
static char *
getSDBRunDir (char *Arg0) {
	char *SDBRunDir;
	char *ptr;

	if (Arg0 == 0)
		return 0;

	SDBRunDir = malloc (strlen (Arg0) + 1);
	if (SDBRunDir == 0)
		return 0;

	strcpy (SDBRunDir, Arg0);

	/*
	 * separate program name
	 */
	ptr = strrchr (SDBRunDir, '/');
	if (ptr == 0)
		ptr = strrchr (SDBRunDir, '\\');

	if (ptr == 0) {
		*SDBRunDir = '\0';
	} else {
		*ptr = '\0';
	}

	return (SDBRunDir);
}
#endif

#ifdef WIN32
static char *
getSDBRunDir (char *Arg0) {
	char *SDBRunDir;
	HANDLE hSDBRun;
	char *ptr;
	int want;
	int got;

	want = 0;
	SDBRunDir = 0;

	hSDBRun = GetModuleHandle ("sdbrun.dll");
	if (hSDBRun == 0)
		return 0;

	for (;;) {
		want += 1024;
		SDBRunDir = realloc (SDBRunDir, want);
		if (SDBRunDir == 0)
			return 0;

		got = GetModuleFileName (hSDBRun, SDBRunDir, want);
		if (got == 0) {
			free (SDBRunDir);
			return 0;
		}

		if (got < want - 1)
			break;
	}

	/*
	 * separate program name
	 */
	ptr = strrchr (SDBRunDir, '/');
	if (ptr == 0)
		ptr = strrchr (SDBRunDir, '\\');

	if (ptr == 0) {
		free (SDBRunDir);
		return 0;
	}

	*ptr = '\0';
	return (SDBRunDir);
}
#endif

/*
 * setup an initialized empty perl interpreter
 */
static PerlInterpreter *
init_perl (char **env) { 
	char *PerlEmbedArgv[] = {"", "-e", "0"};
	PerlInterpreter *p_perl;
	int rc;

	/*
	 * create perl interpreter
	 */
	p_perl = perl_alloc ();
	if (p_perl == 0) {
		printf ("cannot alloc space for perl interpreter\n");
		return 0;
	}

#ifdef WIN32
	PL_exit_flags |= PERL_EXIT_DESTRUCT_END;
#endif
	
	perl_construct (p_perl);
	PL_perl_destruct_level = 0;

	/*
	 *  load empty script into perl interpreter
	 */
	rc = perl_parse (p_perl, xs_init, 3, PerlEmbedArgv, env);
	if (rc != 0) {
		printf ("initial load of perl failed\n");
		perl_free (p_perl);
		return 0;
	}

	/*
	 * start perl interpreter
	 */
	rc = perl_run (p_perl);
	if (rc != 0) {
		printf ("initial start of perl failed\n");
		perl_free (p_perl);
		return 0;
	}

	return (p_perl);
}

/*
 * call method SAPDB::Install::Start::Start in perl interpreter
 */
static int
enter_perl (int argc, char **argv) {
	dSP;
	int i;
	int rc,count;
	ENTER;
	SAVETMPS;
	PUSHMARK(SP);

	/*
	 * push argument list
	 */
	for (i = 1; i < argc; i++)
		XPUSHs (sv_2mortal (newSVpv (argv[i], strlen (argv[i]))));

	PUTBACK;
	count = perl_call_pv ("SAPDB::Install::Start::Start", G_SCALAR);
	SPAGAIN;
	rc = POPi;
	PUTBACK;
	FREETMPS;
	LEAVE;
	
	return (rc);
}

/*
 * set a variable in a hash 
 */
static int
set_perlhashvar (char *name, char *key, char *val) {
	HV *hv;
	SV *sv;

	hv = perl_get_hv (name, TRUE);
	sv = newSVpv (val, strlen (val));
	hv_store (hv, key, strlen (key), sv, 0);
	return 0;	
}

#ifdef WIN32
#define DLLEXPORT __declspec (dllexport)
#endif

#ifdef UNIX
#define DLLEXPORT
#endif

/*
 * the main entry of sdbrun
 */
DLLEXPORT int
sdbrun_main (int argc, char **argv, char **env) {
	PerlInterpreter *pPerl;
	int rc;
	int exitcode;
	char *SDBRunDir;
	char *CallerDir;

#ifdef PERL_GLOBAL_STRUCT
#define PERLVAR(var,type)
#define PERLVARA(var,type)
#define PERLVARI(var,type,init) PL_Vars.var = init;
#define PERLVARIC(var,type,init) PL_Vars.var = init;
#include "perlvars.h"
#undef PERLVAR
#undef PERLVARA
#undef PERLVARI
#undef PERLVARIC
#endif

#ifdef PERL_GPROF_MONCONTROL
	PERL_GPROF_MONCONTROL (0);
#endif

	PERL_SYS_INIT3 (&argc, &argv, &env);

#if defined (USE_5005THREADS) || defined (USE_ITHREADS)
	PTHREAD_ATFORK (Perl_atfork_lock, Perl_atfork_unlock, Perl_atfork_unlock);
#endif

	pPerl = init_perl (env);
	if (pPerl == 0)
		return -1;

	/*
	 * load inital script
	 */
	perl_eval_pv ((char *) sz_text, 1);

	SDBRunDir = getSDBRunDir (argv[0]);

	CallerDir = getcwd (alloca (1024), 1024);
	if (CallerDir == 0) {
		printf ("cannot get caller current directory\n");
		free (SDBRunDir);
		return -1;
	}

	/*
	 * change to directory where the sdbinst runtime is located
	 */
	if (SDBRunDir != 0 && strlen (SDBRunDir) > 0) {
		rc = chdir (SDBRunDir);
		if (rc != 0) {
			printf ("cannot change to directory %s\n", SDBRunDir);
			free (SDBRunDir);
			return -1;
		}
	}
	
	if (set_perlhashvar (
	"SAPDB::Install::Config", "CallerDir", CallerDir) != 0) {
		free (SDBRunDir);
		return -1;
	}

	if (set_perlhashvar (
	"SAPDB::Install::Config", "RuntimeDir", SDBRunDir) != 0) {
		free (SDBRunDir);
		return -1;
	}

	free(SDBRunDir);

	if (set_perlhashvar (
	"SAPDB::Install::Config", "BinaryInstVersion", szInstVersion) != 0)
		return -1;

	if (set_perlhashvar (
	"SAPDB::Install::Config", "BinaryChangeList", szChangeList) != 0)
		return -1;

	if (set_perlhashvar (
	"SAPDB::Install::Config", "BinaryBuildString", szBuildString) != 0)
		return -1;

	if (set_perlhashvar (
	"SAPDB::Install::Config", "BinaryMakeId", szMakeId) != 0)
		return -1;


		
	/*
	 * run sub main
	 */
	exitcode = enter_perl (argc, argv);
	PL_perl_destruct_level = 0;
	perl_destruct(pPerl);
	perl_free (pPerl);

	PERL_SYS_TERM();

	return (exitcode);
}

/*
 * entry points of used extentions
 */
externC XS (boot_DynaLoader);
externC XS (boot_SAPDB__Install__InstInfo);
externC XS (boot_SAPDB__Install__BackupPipe);
externC XS (boot_SAPDB__Install__BuildInfo);
externC XS (boot_SAPDB__Install__SysInfo);
externC XS (boot_SAPDB__Install__PipeExec);
externC XS (boot_SAPDB__Install__ProcState);
externC XS (boot_SAPDB__Install__FileLock);
externC XS (boot_SAPDB__Install__FSUsage);
externC XS (boot_SAPDB__Install__GetParam);
externC XS (boot_SAPDB__Install__Comm);
externC XS (boot_SAPDB__Install__DBMCmd);
externC XS (boot_SAPDB__Install__MachInfo);
externC XS (boot_SAPDB__Install__MD5Sum);
externC XS (boot_SAPDB__Install__SetUser);
externC XS (boot_SAPDB__Install__Untgz);
externC XS (boot_SAPDB__Install__Jar);
externC XS (boot_SAPDB__Install__GnuTar);
externC XS (boot_SAPDB__Install__System__Unix);
externC XS (boot_SAPDB__Install__System__Unix__Dir);
externC XS (boot_SAPDB__Install__System__Win32__API);
externC XS (boot_SAPDB__Install__System__Win32__Console);
externC XS (boot_SAPDB__Install__System__Win32__Registry);
externC XS (boot_SAPDB__Install__System__Win32__VersionInfo);
externC XS (boot_SAPDB__Install__Term__ReadKey);
externC XS (boot_SDB__Common);
externC XS (boot_SDB__Common__Archive__Tar);
externC XS (boot_SDB__Common__Archive__Zip);

/*
 * export entry points for extentions
 */
static void 
xs_init (pTHX) {
	char *file = __FILE__;
	dXSUB_SYS;

	newXS("DynaLoader::boot_DynaLoader", 
	       boot_DynaLoader, file);
	newXS ("SAPDB::Install::InstInfo::bootstrap",
	       boot_SAPDB__Install__InstInfo, file);
	newXS ("SAPDB::Install::SysInfo::bootstrap",
	       boot_SAPDB__Install__SysInfo, file);
	newXS ("SAPDB::Install::ProcState::bootstrap",
	       boot_SAPDB__Install__ProcState, file);
	newXS ("SAPDB::Install::PipeExec::bootstrap",
	       boot_SAPDB__Install__PipeExec, file);
	newXS ("SAPDB::Install::FileLock::bootstrap",
	       boot_SAPDB__Install__FileLock, file);
	newXS ("SAPDB::Install::FSUsage::bootstrap",
	       boot_SAPDB__Install__FSUsage, file);
	newXS ("SAPDB::Install::GetParam::bootstrap",
	       boot_SAPDB__Install__GetParam, file);
	newXS ("SAPDB::Install::Comm::bootstrap",
	       boot_SAPDB__Install__Comm, file);
	newXS ("SAPDB::Install::DBMCmd::bootstrap",
	       boot_SAPDB__Install__DBMCmd, file);
	newXS ("SAPDB::Install::BackupPipe::bootstrap",
	       boot_SAPDB__Install__BackupPipe, file);
	newXS ("SAPDB::Install::BuildInfo::bootstrap",
	       boot_SAPDB__Install__BuildInfo, file);
	newXS ("SAPDB::Install::MachInfo::bootstrap",
	       boot_SAPDB__Install__MachInfo, file);
	newXS ("SAPDB::Install::MD5Sum::bootstrap",
	       boot_SAPDB__Install__MD5Sum, file);
	newXS ("SAPDB::Install::SetUser::bootstrap",
	       boot_SAPDB__Install__SetUser, file);
	newXS ("SAPDB::Install::Untgz::bootstrap",
	       boot_SAPDB__Install__Untgz, file);
	newXS ("SAPDB::Install::Jar::bootstrap",
	       boot_SAPDB__Install__Jar, file);
	newXS ("SAPDB::Install::GnuTar::bootstrap",
	       boot_SAPDB__Install__GnuTar, file);
	newXS ("SAPDB::Install::System::Unix::Dir::bootstrap",
	       boot_SAPDB__Install__System__Unix__Dir, file);
	newXS ("SAPDB::Install::System::Unix::bootstrap",
	       boot_SAPDB__Install__System__Unix, file);
	newXS ("SAPDB::Install::System::Win32::VersionInfo::bootstrap",
	       boot_SAPDB__Install__System__Win32__VersionInfo, file);
	newXS ("SAPDB::Install::System::Win32::Registry::bootstrap",
	       boot_SAPDB__Install__System__Win32__Registry, file);
	newXS ("SAPDB::Install::System::Win32::API::bootstrap",
	       boot_SAPDB__Install__System__Win32__API, file);
	newXS ("SAPDB::Install::System::Win32::Console::bootstrap",
	       boot_SAPDB__Install__System__Win32__Console, file);
	newXS ("SAPDB::Install::Term::ReadKey::bootstrap",
	       boot_SAPDB__Install__Term__ReadKey, file);
	newXS ("SDB::Common::bootstrap",
	       boot_SDB__Common, file);
	newXS ("SDB::Common::Archive::Tar::bootstrap",
	       boot_SDB__Common__Archive__Tar, file);
	newXS ("SDB::Common::Archive::Zip::bootstrap",
	       boot_SDB__Common__Archive__Zip, file);
}
