#!/usr/bin/perl
#
# $SRC/install/perl/Makefile.PL
#
# $Header: //sapdb/V76/c_00/b_09/sys/src/install/perl/sdbrun/readkey/Makefile.PL#1 $
# $DateTime: 2005/03/21 16:21:00 $
# $Change: 93297 $
#

use Config;
use Symbol;

sub main {
	print "Generating Makefile for ReadKey\n";	

	my %Makedefs = (
	'PERL_PERL' => 'perl',
	'PERL_PERLLIB' => get_perllib (),
	'PERL_COREDIR' => get_coredir (),
	'PERL_XSUBPP' => get_xsubpp (),
	'PERL_TXT2INC' => get_txt2inc (),
	'PERL_MAKESYM' => get_makesym (),
	'PERL_CC' => $Config{'cc'},
	'PERL_CCFLAGS' => $Config{'ccflags'}." ".$Config{'cccdlflags'},
	'PERL_DEFS' => get_defs (),
	'PERL_INC' => "\$(PERL_COREDIR)",
	'PERL_INCLUDE' => "-I\$(PERL_COREDIR)",
	'PERL_CFLAGS' => "\$(PERL_CCFLAGS) \$(PERL_INCLUDE) \$(PERL_DEFS)",
	'PERL_DLD' => $^O =~ /^mswin/i ? "link" : $Config{'ld'},
	'PERL_DLDFLAGS' => get_dldflags (),
	'PERL_LD' => $^O =~ /^mswin/i ? "link" : $Config{'cc'},
	'PERL_LDFLAGS' => get_ldflags (),
	'PERL_DLEXT' => $Config{'dlext'},
	'PERL_OBJEXT' => $Config{'obj_ext'},
	'PERL_EXEEXT' => $Config{'exe_ext'},
	'PERL_EXTLIBS' => get_extlibs ());
	
	write_makefile (%Makedefs);
}

sub write_makefile {
	my %Makedefs = @_;
	my $fh = gensym ();
	open ($fh, "> Makefile");
	print ($fh
	"#\n".
	"# Makefile\n".
	"# this file is generated from Makefile.PL\n".
	"# do not edit this file, any changes will be lost\n".
	"#\n".
	"\n".	
	".SUFFIXES:\n".
	".SUFFIXES: .o .c .xs\n".
	"\n");

	while (my ($key, $value) = each %Makedefs) {
		next unless (defined $value);
		print ($fh $key." = ".$value."\n");
	}
	print $fh ("\n");

	write_body ($fh);
	close ($fh);
}

sub write_body {
	my $fh = shift;
	while (<DATA>) {
		chomp;
		if ($^O =~ /^aix/ && $_ =~ /-o (\w+)\.\$\(DLEXT\)/) {
			$_ .= " -bE:".$1.".exp";
		}
		if ($^O =~ /^mswin/i) {
			if ($_ =~ /-o (\w+)\.\$\(DLEXT\)/) {
				$_ .= " -def:".$1.".def";
				$_ =~ s/-o\s+/-out:/;
			} elsif ($_ =~ /-o \w+\$\(EXEEXT\)/) {
				$_ =~ s/-o\s+/-out:/;
			}
		}
		print ($fh $_."\n");
	}
}

sub get_perllib {
	$^O =~ /mswin/i ? $Config{'libperl'} :
	"libperl.a";
}

sub get_dldflags {
	$^O =~ /aix/    ? get_dldflags_aix () :
	$^O =~ /mswin/i ?
	$Config{'lddlflags'}." -libpath:\$(PERL_COREDIR) \$(PERL_PERLLIB)" :
	$Config{'lddlflags'};
}

sub get_dldflags_aix {
	my @in = split (' ', $Config{'lddlflags'});
	my @out = ();
	foreach $_ (@in) {
		next if ($_ =~ /^-bE\:/);
		push @out, $_;
	}
	return (join " ", @out);
}

sub get_extlibs {
	$^O =~ /mswin/i   ? $Config{'libs'} :
	$^O =~ /^aix/     ? "-lm -lbsd" :
	$^O =~ /^dec_osf/ ? "-lm" :
	$^O =~ /^hpux/    ? "-lm" :
	$^O =~ /^linux/ && $Config{'archname'} =~ /^i\d86/ ? "-lm -lcrypt" :
	$^O =~ /^solaris/ ? "-lm -lsocket -lnsl" :
	"";	
}

sub get_xsubpp {
	$^O =~ /^mswin/i ?
	"\$(PERL) ".$Config{'privlibexp'}."\\ExtUtils\\xsubpp" :
	"\$(PERL) ".$Config{'privlibexp'}."/ExtUtils/xsubpp";
}

sub get_txt2inc {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ..\\..\\tool\\txt2inc" :
	"\$(PERL) ../../tool/txt2inc";
}

sub get_makesym {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ..\\..\\tool\\makesym" :
	"\$(PERL) ../../tool/makesym";
}

sub get_coredir {
	my $dir = defined ($Config{'archlibexp'}) ?
	$Config{'archlibexp'}.'/CORE' :
	$Config{'privlibexp'}.'/CORE';

	$dir =~ s/\//\\/ if $^O =~ /^mswin/i;
	return ($dir);
}

sub get_ldflags {
	$^O =~ /mswin/i ? "-nodefaultlib -libpath:\$(PERL_COREDIR) \$(PERL_PERLLIB)" :
	$^O =~ /hpux/ ? "+DA1.1 +DS2.0 -L\$(PERL_COREDIR) -lperl" :
	"-L\$(PERL_COREDIR) -lperl";
}

sub get_defs {
	my $defs = "-DHAS_BYTEORDER=".$Config{'byteorder'};
}

main (@ARGV);
do ('genchars.pl');


__DATA__
PERL = $(PERL_PERL)
LD = $(PERL_LD)
TXT2INC=$(PERL_TXT2INC)
XSUBPP = $(PERL_XSUBPP)
MAKESYM = $(PERL_MAKESYM)
CC = $(PERL_CC)
CFLAGS = $(PERL_CFLAGS)
DLD = $(PERL_DLD)
DLD_FLAGS = $(PERL_DLDFLAGS)
LIBS = $(PERL_EXTLIBS)
LD_FLAGS = $(PERL_LDFLAGS)
DLEXT = $(PERL_DLEXT)
OBJEXT = $(PERL_OBJEXT)
EXEEXT = $(PERL_EXEEXT)

.xs.c:
	$(MAKESYM) $*.xs
	$(XSUBPP) $*.xs > $*.c
 
.c$(OBJEXT):
	$(CC) -c $(CFLAGS) $<

ReadKey_OBJ = ReadKey$(OBJEXT)

ALL_XS_OBJ = $(ReadKey_OBJ)
ALL_XS = ReadKey.$(DLEXT)

all: $(ALL_XS)

ReadKey.$(DLEXT): $(ReadKey_OBJ)
	$(MAKESYM) ReadKey.xs
	$(DLD) $(DLD_FLAGS) -o ReadKey.$(DLEXT) $(ReadKey_OBJ) $(LIBS)

ReadKey$(OBJEXT): ReadKey.c ReadKey.h
	$(CC) -c $(CFLAGS) ReadKey.c

ReadKey.h: ReadKey.pl
	$(TXT2INC) ReadKey.pl ReadKey.h sz_text

ReadKey.c: ReadKey.xs
	$(XSUBPP) ReadKey.xs > ReadKey.c

clean:
	-rm ReadKey.c ReadKey$(OBJEXT) ReadKey.$(DLEXT) Makefile cchars.h

