#!/usr/bin/perl
#
# $SRC/install/perl/tool/xs/Makefile.PL
#
# $Header: //sapdb/V74/develop/sys/src/install/perl/Makefile.PL#26 $
# $DateTime: 2002/10/02 13:10:30 $
# $Change: 28183 $
#

use Config;
use Symbol;

sub main {
	print "Generating Makefile for tool extentions\n";
	my %Makedefs = (
	'PERL_PERL' => 'perl',
	'PERL_PERLLIB' => get_perllib (),
	'PERL_COREDIR' => get_coredir (),
	'PERL_XSUBPP' => get_xsubpp (),
	'PERL_MAKESYM' => get_makesym (),
	'PERL_MKDIR' => get_mkdir (),
	'PERL_RM' => get_rm (),
	'PERL_CP' => get_cp (),
	'PERL_STRIP' => get_strip (),
	'PERL_CC' => $Config{'cc'},
	'PERL_CCFLAGS' => $Config{'ccflags'}." ".$Config{'cccdlflags'},
	'PERL_DEFS' => get_defs (),
	'PERL_INC' => "\$(PERL_COREDIR)",
	'PERL_INCLUDE' => "-I\$(PERL_COREDIR)",
	'PERL_CFLAGS' => "\$(PERL_CCFLAGS) \$(PERL_INCLUDE) \$(PERL_DEFS)",
	'PERL_DLD' => $^O =~ /^mswin/i ? "link" : $Config{'ld'},
	'PERL_DLDFLAGS' => get_dldflags (),
	'PERL_LD' => $^O =~ /^mswin/i ? "link" : $Config{'cc'},
	'PERL_LDFLAGS' => get_ldflags (),
	'PERL_DLEXT' => $Config{'dlext'},
	'PERL_OBJEXT' => $Config{'obj_ext'},
	'PERL_EXTLIBS' => get_extlibs ());
	write_makefile (%Makedefs);
}

sub write_makefile {
	my %Makedefs = @_;
	my $fh = gensym ();
	open ($fh, "> Makefile");

	if ($^O =~ /mswin/i && $Config{'archname'} =~ /IA64/) {
		# just copy Makefile.w64 if we are windows itanium
		my $filename = "Makefile.w64";

		my $fh_in = gensym ();
		open ($fh_in, $filename);
		while (<$fh_in>) {
			chomp;
			print $fh $_."\n";
		}
		close ($fh_in);
	} else {
		print ($fh
		"#\n".
		"# Makefile\n".
		"# this file is generated from Makefile.PL\n".
		"# do not edit this file, any changes will be lost\n".
		"#\n".
		"\n".	
		".SUFFIXES:\n".
		".SUFFIXES: .o .c .xs\n".
		"\n");

		if ($^O =~ /^aix/) {
			print ($fh
			"#\n".
			"# make sure that we are in 32 bit mode on aix\n".
			"#\n".
			"OBJECT_MODE = 32\n".
			"\n");
		}

		while (my ($key, $value) = each %Makedefs) {
			next unless (defined $value);
			print ($fh $key." = ".$value."\n");
		}
		print $fh ("\n");

		write_body ($fh);
	}

	close ($fh);
}

sub write_body {
	my $fh = shift;
	while (<DATA>) {
		chomp;
		if ($^O =~ /^aix/ && $_ =~ /-o (\w+)\.\$\(DLEXT\)/) {
			$_ .= " -bE:".$1.".exp";
		}
		if ($^O =~ /^mswin/i) {
			if ($_ =~ /-o (\w+)\.\$\(DLEXT\)/) {
				$_ .= " -def:".$1.".def";
				$_ =~ s/-o\s+/-out:/;
			} elsif ($_ =~ /-o \w+\$\(EXEEXT\)/) {
				$_ =~ s/-o\s+/-out:/;
			}
		} 
		print ($fh $_."\n");
	}
}

sub get_perllib {

	# UNIX
	return "libperl.a" unless ($^O =~ /mswin/i);

	# WIN32 Active State Perl
	return "PerlCRT.lib" if $Config{'archname'} =~ /object/;

	return $Config{'libperl'};
}

sub get_dldflags {
	$^O =~ /aix/    ? get_dldflags_aix () :
	$^O =~ /mswin/i ? 
	$Config{'lddlflags'}." -libpath:\$(PERL_COREDIR) \$(PERL_PERLLIB)" :
	$Config{'lddlflags'};
}

sub get_dldflags_aix {
	my @in = split (' ', $Config{'lddlflags'});
	my @out = ();
	foreach $_ (@in) {
		next if ($_ =~ /^-bE\:/);
		push @out, $_;
	}
	return (join " ", @out);
}

sub get_extlibs {
	$^O =~ /mswin/i   ? $Config{'libs'} :
	$^O =~ /^aix/     ? "-lm -lbsd" :
	$^O =~ /^dec_osf/ ? "-lm" :
	$^O =~ /^hpux/    ? "-lm" :
	$^O =~ /^linux/ && $Config{'archname'} =~ /^i\d86/ ? "-lm -lcrypt" :
	$^O =~ /^linux/ && $Config{'archname'} =~ /^ia64/ ? "-lm -lcrypt" :
	$^O =~ /^solaris/ ? "-lm -lsocket -lnsl" :
	$^O =~ /^svr4/ && $Config{'archname'} =~ /^RM/ ? 
		"-lm -lsocket -lnsl" :
	"";	
}

sub get_strip {
	$^O =~ /^mswin/i ? "touch" : "strip";
}

sub get_mkdir {
	$^O =~ /^mswin/i ? "mkdir" : "-mkdir -p";
}

sub get_rm {
	$^O =~ /^mswin/i ? "del" : "-rm";
}

sub get_cp {
	$^O =~ /^mswin/i ? "copy" : "cp";
}

sub get_coredir {
	my $dir = defined ($Config{'archlibexp'}) ?
	$Config{'archlibexp'}.'/CORE' :
	$Config{'privlibexp'}.'/CORE';

	$dir =~ s/\//\\/ if $^O =~ /^mswin/i;
	return ($dir);
}

sub get_xsubpp {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ".$Config{'privlibexp'}."\\ExtUtils\\xsubpp" :
	"\$(PERL) ".$Config{'privlibexp'}."/ExtUtils/xsubpp";
}

sub get_makesym {
	$^O =~ /^mswin/i  ? 
	"\$(PERL) ..\\..\\tool\\makesym" :
	"\$(PERL) ../../tool/makesym";
}

sub get_ldflags {
	$^O =~ /mswin/i ? 
	"-nodefaultlib -libpath:\$(PERL_COREDIR) \$(PERL_PERLLIB)" :
	$^O =~ /hpux/ ? "+DA1.1 +DS2.0 -L\$(PERL_COREDIR) -lperl" :
	$^O =~ /svr4/ && $Config{'archname'} =~ /^RM/ ? 
		"-Bstatic -L\$(PERL_COREDIR) -lperl -Bdynamic" :
	"-L\$(PERL_COREDIR) -lperl";
}

sub get_defs {
	"-DHAS_BYTEORDER=".$Config{'byteorder'};
}

main (@ARGV);

__DATA__
PERL = $(PERL_PERL)
RM = $(PERL_RM)
LD = $(PERL_LD)
CP = $(PERL_CP)
CC = $(PERL_CC)
MKDIR = $(PERL_MKDIR)
STRIP = $(PERL_STRIP)
INST_PREFIX = $(PERL_INST_PREFIX)
MAKESYM = $(PERL_MAKESYM)
XSUBPP = $(PERL_XSUBPP)
CFLAGS = $(PERL_CFLAGS)
DLD = $(PERL_DLD)
DLD_FLAGS = $(PERL_DLDFLAGS)
LIBS = $(PERL_EXTLIBS)
LD_FLAGS = $(PERL_LDFLAGS)
DLEXT = $(PERL_DLEXT)
OBJEXT = $(PERL_OBJEXT)

.xs.c:
	$(XSUBPP) $*.xs > $*.c
 
.c$(OBJEXT):
	$(CC) -c $(CFLAGS) $<

GnuTar_OBJ = GnuTar$(OBJEXT)

ZLIB_OBJ = ../../lib/zlib/zlib.ro

ALL_PL_H = GnuTar.h

ALL_XS_C = GnuTar.c

ALL_XS_OBJ = $(GnuTar_OBJ)

ALL_XS_DLL = GnuTar.$(DLEXT)

all: $(ALL_XS_DLL)

GnuTar.$(DLEXT): $(GnuTar_OBJ) $(ZLIB_OBJ)
	$(MAKESYM) GnuTar.xs
	$(DLD) $(DLD_FLAGS) -o GnuTar.$(DLEXT) $(GnuTar_OBJ) $(LIBS) $(ZLIB_OBJ)

GnuTar$(OBJEXT): GnuTar.c
	$(CC) -c $(CFLAGS) -I../../lib/zlib GnuTar.c

GnuTar.c: GnuTar.xs
	$(XSUBPP) $*.xs > $*.c 

install: all
#	$(CP) *.$(DLEXT) $(INST_PREFIX)

clean:
	$(RM) $(ALL_PL_H)
	$(RM) $(ALL_XS_C) 
	$(RM) $(ALL_XS_OBJ) 
	$(RM) $(ALL_XS_DLL)
	$(RM) Makefile
