/*
  -----------------------------------------------------------------------------
  module: vls31_catalogextract.cpp
  -----------------------------------------------------------------------------

  responsible:  d033893 and SteffenS

  special area: SAP DB LOADER

  description:  Implementation of the CATALOGEXTRACT/CATALOGLOAD functionality

  version:      7.6.
  -----------------------------------------------------------------------------


    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end


*/

#include "gls00.h"
#include "hls03.h" 
#include "hls15.h"
#include "hls30.h"
#include "hls99.h"
#include "hls31_odbc_Result.h"
#include "hls31_catalogextract.h"

/* -----------------------------------------------------------------------------
   TransportPackageCatalog
/-------------------------------------------------------------------------------*/
SAPDB_Int4
TransportPackageCatalog(MetaDataDef    *&pMetaDataDef,
                        tls00_DBInfo   *pDBInfo,
                        tsp00_Addr     ErrText)
{
    ROUTINE_DBG_MEO00 ("TransportPackageCatalog");
    LoaderMessages *pMsgInstance = LoaderMessages::Instance();

    SAPDB_Long rc = errOK_els00;

    TransformationDef       * pTransformationDef  = pMetaDataDef->getTransformationModel();
    tls00_TransformationCmd * pTransformationCmd  = pTransformationDef->getTransformationCmd();

    StudioOAL_String  ErrorText;
    StudioTD_String   username;
    tls00_FileInfo    streamInfo;
    // interpret <catalogextract statement>::=
    streamInfo.fileSpec = pTransformationCmd->trCatalogFile_ls00;

    if ( errOK_els00 == rc  )
    {
        //*
        //*  initialize stream
        //*
        streamInfo.vFile.vfFileType_ls00   = streamInfo.fileSpec.fsDeviceType_ls00;

        if ( dtypeTape_ls00 == streamInfo.fileSpec.fsDeviceType_ls00 && rc == errOK_els00)
        {
            streamInfo.vFile.vfBackupTool_ls00 = pDBInfo->dbiSession_ls00.siBackupTool_ls00;
            streamInfo.vFile.vfNodeName_ls00.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer,
                                                                     pDBInfo->dbiSession_ls00.si_pDBNode_ls00));
            streamInfo.vFile.vfDBName_ls00.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer,
                                                                   pDBInfo->dbiSession_ls00.siDBName_ls00.asCharp()));
        }
        //*
        //* export catalog ddl, xml
        //*
        if ( tsExport_ls00 == pTransformationCmd->trTransSpec_ls00 )
        {
            //*
            //*  import transformationpackage 
            //*
            //transformationpackage = "IMPORT    PACKAGE ";
            rc = pTransformationDef->ImportPackage ( pMetaDataDef, ErrText );

            ls30VFInitialize(streamInfo.vFile, pTransformationCmd->trCatalogFile_ls00.fsFileName_ls00, 
                             sp5vf_text, sp5vf_write, streamInfo.fileSpec.fsFormat_ls00.ffoCodeType_ls00);

            rc = ls30VFOpen (streamInfo.vFile,ErrText);
            if ( rc != errOK_els00 )
                return rc;

            pMetaDataDef->setMedium( streamInfo );
            //*
            //*  execute transformationpackage
            //*
            //transformationpackage = "EXECUTE   PACKAGE TO TRANSFORM CATALOG";
            pMsgInstance->ls98PrintMessage(layCatalogExtract_els98, msgCommandExecuting_els98,
                                                                    "EXECUTE   PACKAGE TO TRANSFORM CATALOG");

            if ( ptTable_els00  < pTransformationCmd->trPartSpec_ls00.psPartType_ls00 &&
                 SAPDB_FALSE   == pTransformationDef->ExistPackageData(pTransformationCmd->trPackageid_ls00) )
            {
                pMsgInstance->ls98PrintMessage(layCatalogExtract_els98, msgNumTabToTransform_els98    ,pTransformationDef->m_nTableCount);
                pMsgInstance->ls98PrintMessage(layCatalogExtract_els98, msgNumViewToTransform_els98   ,pTransformationDef->m_nViewCount);
                pMsgInstance->ls98PrintMessage(layCatalogExtract_els98, msgNumSynonymToTransform_els98,pTransformationDef->m_nSynonymCount);
            }

            username.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer,
                                             pDBInfo->dbiSession_ls00.siUserName_ls00.asCharp()));
            //*
            //*  update  transformationpackage
            //*
            RelationalDef* pRelationalDef = pMetaDataDef->getRelationalModel();

            if (ptAll_els00    == pTransformationCmd->trPartSpec_ls00.psPartType_ls00 && rc == errOK_els00)
                rc = pRelationalDef->ExportPackageDb(username, ErrText );
            if (ptUser_els00   == pTransformationCmd->trPartSpec_ls00.psPartType_ls00 && rc == errOK_els00 ||
                ptSchema_els00 == pTransformationCmd->trPartSpec_ls00.psPartType_ls00 && rc == errOK_els00)
                rc = pRelationalDef->ExportPackageUser (username, ErrText );
            if (ptTable_els00  == pTransformationCmd->trPartSpec_ls00.psPartType_ls00 && rc == errOK_els00)
                rc = pRelationalDef->ExportPackageTable  ( );

            SAPDB_Int4 rcFile = ls30VFClose(streamInfo.vFile,ErrText);
            if ( (errOK_els00 == rc) && (errOK_els00 != rcFile) )
                 rc = rcFile;
            //*
            //*  storagedevice, servernode, serverdb, username, codetype, codepage, transformation, part, streamformat
            //*
            if ( (dtypeTape_ls00 == pTransformationCmd->trCatalogFile_ls00.fsDeviceType_ls00 ) &&
                 (ptTable_els00   < pTransformationCmd->trPartSpec_ls00.psPartType_ls00 )   )
            {
                pTransformationCmd->trCatalogFile_ls00.fsBackupId_ls00.bBackupIdName_ls00 =
                    CONST_CAST(SAPDB_Char*, streamInfo.vFile.vfBackupId_ls00.CharPtr());
                rc = pTransformationDef->updateCatalogCmd();
            }
            //*
            //*  export transformationpackage
            //*
            //transformationpackage = "EXPORT    PACKAGE x'";
            //transformationpackage += pTransformationCmd->trPackageid_ls00;
            //transformationpackage += "'";
            if ( errOK_els00 == rc && NULL == pTransformationCmd->trDataFile_ls00.fsFileName_ls00 )
                 rc = pTransformationDef->ExportPackage( pMetaDataDef, ErrText );
        }//if ( tsExport == pTransformationCmd->trTransSpec_ls00  && rc == errOK_els00 )

        //*
        //* import catalog ddl
        //*
        else if ( tsImport_ls00 == pTransformationCmd->trTransSpec_ls00  )
        {
            if ( dtypeTape_ls00 == pTransformationCmd->trCatalogFile_ls00.fsDeviceType_ls00 )
            {
                streamInfo.vFile.vfBackupId_ls00.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer,
                                                        streamInfo.fileSpec.fsBackupId_ls00.bBackupIdName_ls00));
            }

            ls30VFInitialize(streamInfo.vFile, pTransformationCmd->trCatalogFile_ls00.fsFileName_ls00,
                             sp5vf_text, sp5vf_read, streamInfo.fileSpec.fsFormat_ls00.ffoCodeType_ls00);
            rc = ls30VFOpen (streamInfo.vFile, ErrText);
            if (errOK_els00 == rc)
            {
                pMetaDataDef->setMedium( streamInfo );

                username.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer,
                                                 pDBInfo->dbiSession_ls00.siUserName_ls00.asCharp()));
                //*
                //*  execute transformationpackage
                //*
                RelationalDef* pRelationalDef = pMetaDataDef->getRelationalModel();

                //transformationpackage = "EXECUTE   PACKAGE TO TRANSFORM CATALOG";
                pMsgInstance->ls98PrintMessage(layCatalogLoad_els98, msgCommandExecuting_els98,
                                                                     "EXECUTE   PACKAGE TO TRANSFORM CATALOG");

                if (ptAll_els00    == pTransformationCmd->trPartSpec_ls00.psPartType_ls00)
                    rc = pRelationalDef->ImportPackageDb(username);
                if (ptUser_els00   == pTransformationCmd->trPartSpec_ls00.psPartType_ls00 ||
                    ptSchema_els00 == pTransformationCmd->trPartSpec_ls00.psPartType_ls00)
                    rc = pRelationalDef->ImportPackageUser(username);
                if (ptTable_els00  == pTransformationCmd->trPartSpec_ls00.psPartType_ls00)
                    rc = pRelationalDef->ImportPackageTable( pTransformationCmd->trPartSpec_ls00, ErrText );

                SAPDB_Int4 rcFile = ls30VFClose(streamInfo.vFile,ErrText);
                if ( (errOK_els00 == rc) && (errOK_els00 != rcFile) )
                    rc = rcFile;

                if (errOK_els00 == rc)
                {
                    //*
                    //*  import transformationpackage 
                    //*
                    //transformationpackage = "IMPORT    PACKAGE ";
                    rc = pTransformationDef->ImportPackage ( pMetaDataDef, ErrText );
                }
                if (errOK_els00 != rc && ptTable_els00 < pTransformationCmd->trPartSpec_ls00.psPartType_ls00)
                {
                    ErrorHndl *pError = pMetaDataDef->getError();
                    pMsgInstance->ls98MsgToString(layTableExtract_els98, ErrText, STATIC_CAST(ErrorNumbers, rc), pError->getErrorText().StrPtr());
                }
                if ( ptTable_els00  < pTransformationCmd->trPartSpec_ls00.psPartType_ls00 &&
                     SAPDB_FALSE   == pTransformationDef->ExistPackageData(pTransformationCmd->trPackageid_ls00) )
                     pMsgInstance->ls98PrintMessage(layCatalogExtract_els98, msgNumTabTransformed_els98, pTransformationDef->m_nTableCount);
                //*
                //*  export  transformationpackage
                //*
                //transformationpackage = "EXPORT    PACKAGE x'";
                //transformationpackage += pTransformationCmd->trPackageid_ls00;
                //transformationpackage += "'";
                if ( errOK_els00 == rc && NULL == pTransformationCmd->trDataFile_ls00.fsFileName_ls00 )
                     rc = pTransformationDef->ExportPackage( pMetaDataDef, ErrText );
            }
        } // if ( pTransformationCmd->trTransSpec_ls00 == tsImport )
    }
    return rc;
}// TransportPackageCatalog

/*===========================================================================*
 *     END
 *===========================================================================*/
