#    coding: utf-8
#
#    module:       transport.py
#
#    version:      7.6.00.01
#
#    responsible : d033893
#
#    special area: loader
#
#    description : transport, export or import sdb data/metadata using the 
#                  transformation service provided by sdb loader 
#                  imports, executes and validates default or userdefined
#                  transformation packages via generation and execution of  
#                  sdb loader statements from the relational model of 
#                  sdb database instances.
#
#    prerequisite: install sdb and std python libraries
#
#    ========== licence begin  GPL
#    Copyright (c) 2000-2004 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
#    ========== licence end
#

#system
import  os
import  sys
import  stat
import  time
import  socket
import  signal
import  shutil
import  thread
mutex = thread.allocate_lock()
import  getpass
#environment
import  ConfigParser
#string
import  string
import  codecs
import  re
#csv parser
import  csv
#xml parser
from    xml             import sax
#dom
from    xml.dom.minidom import parse
#archive
import  zlib
import  zipfile
import  tarfile
#sdb
import  optlib
from    sdb             import dbm,loader,sql

"""
|-------------------------------------------------------------------------------------------------------------------------------------|
|                                                                                                                                     |
|                                             Loader Transport Parameter                                                              |
|                                                                                                                                     |
|-------------------------------------------------------------------------------------------------------------------------------------|
|                                                                                                                                     |
| ExportID ImportID    Transport Type      LoggingPosition       Package-Part    Medium         Format Export-Import  CharEncoding Lob|
|                                                                                                                                     |
|-------------------------------------------------------------------------------------------------------------------------------------|
|                                                                                                                                     |
|                                            Loader Session Parameter                                                                 |
|                                                                                                                                     |
|-------------------------------------------------------------------------------------------------------------------------------------|
|                                                                                                                                     |
|  Session Attribute       Parameter Value                                 Default Value                                              |
|                                                                                                                                     |
|-------------------------------------------------------------------------------------------------------------------------------------|
|                                                                                                                                     | 
|  SET CONFIGURATION       [EXCLUDE|INCLUDE]                               EXCLUDE                                                    | 
|  SET COMPRESSED          \'/<s>/<d>/\'                                   \'/,/"/\'                                                  |
|  SET SQLMODE             [INTERNAL|ANSI|DB2|ORACLE]                      INTERNAL                                                   | 
|  SET LANGUAGE            [ENG|DEU]                                       \'ENG\'                                                    | 
|  SET CODETYPE            [ASCII|UCS2|UTF8]                               ASCII                                                      |
|  SET CODEPAGE            [http://www.unicode.org/Public/MAPPINGS/]       ISO-8859-1                                                 |
|  SET DATE                [INTERNAL|ISO|USA|EUR|JIS]                      ISO                                                        |
|  SET TIME                [INTERNAL|ISO|USA|EUR|JIS]                      ISO                                                        |
|  SET TIMESTAMP           [INTERNAL|ISO|USA|EUR|JIS]                      ISO                                                        |
|  SET DECIMAL             \'/[<t>]/<d>/\'                                 \'//./\'                                                   |
|  SET BOOLEAN                                                             \'TRUE/FALSE\'                                             |
|  SET NULL                                                                \'?                   \'                                   |
|  SET TRANSACTION SIZE                                                    1000                                                       |
|  SET ISOLATION LEVEL                                                     3                                                          |
|  SET MAXERRORCOUNT                                                       100                                                        |
|  SET PACKAGETIMEOUT                                                      1000                                                       |
|  SET BACKUPTOOL          [TSM|NETWORKER|BACKINT]                         NETWORKER                                                  | 
|  SET ODBCTRACE           [ON|OFF]                                        OFF                                                        |
|                                                                                                                                     |
|-------------------------------------------------------------------------------------------------------------------------------------|
|                                                                                                                                     |
|                                             Loader Transformation Parameter Values                                                  |
|                                                                                                                                     |
|-------------------------------------------------------------------------------------------------------------------------------------|
|                    |                    |                 |                |               |                       |                |
| What should be     | Transformation of  |  Logging of     | Package of     | Medium of     | Format of             | Mapping of     |
| exported/imported ?| Models und Data    |  Transformation | Transformation | Staging Area  | Models and Data       | Char Encodings |
|                    |                    |                 |                |               |                       |                |
|-------------------------------------------------------------------------------------------------------------------------------------|
|Catalog, Data and   |EXPORT|IMPORT       |[RESTORE]        | DB             |               |                       |                |
|Package of a        |CONFIGURATION       |                 |                | FILE          | DTL, XMI              | UCS2           |
|Database            |CATALOG             |                 |                | PIPE          | DDL, XMI              | UTF8           |
|                    |DATA                |                 |                | TAPE          | PAGES, RECORDS        | ASCII          |
|                    |PACKAGE             |[RESTART]        |                |               | DTL, XMI              | DYNAMIC        |
|-------------------------------------------------------------------------------------------------------------------------------------|
|Catalog, Data and   |EXPORT|IMPORT       |[RESTORE]        | USER           |               |                       |                |
|Package of a        |CONFIGURATION       |                 |                | FILE          | DTL, XMI              | UCS2           |
|Database User       |CATALOG             |                 |                | PIPE          | DDL, XMI              | UTF8           |
|                    |DATA                |                 |                | TAPE          | PAGES, RECORDS        | ASCII          |
|                    |PACKAGE             |[RESTART]        |                |               | DTL, XMI              | DYNAMIC        |
|-------------------------------------------------------------------------------------------------------------------------------------|
|Catalog, Data and   |EXPORT|IMPORT       |[RESTORE]        | TABLE          |               |                       |                |
|Package of a        |CONFIGURATION       |                 |                | FILE          | DTL, XMI              | UCS2           |
|Database Table      |CATALOG             |                 |                | PIPE          | DDL, XMI              | UTF8           |
|                    |DATA                |                 |                | TAPE          | PAGES, RECORDS,       | ASCII          |
|                    |                    |                 |                |               | FORMATTED, COMPRESSED | DYNAMIC        |
|                    |PACKAGE             |[RESTART]        |                |               | DTL, XMI              |                |
|-------------------------------------------------------------------------------------------------------------------------------------|
|Data and Package    |EXPORT|IMPORT       |                 | COLUMNS FIELDS |               |                       |                |
|of a qualified      |                    |                 |                | FILE          |                       | UCS2           |
|Database Columnset  |                    |                 |                | PIPE          |                       | UTF8           |
|                    |DATA                |                 |                | TAPE          | FORMATTED, COMPRESSED | ASCII          |
|                    |PACKAGE             |[RESTART]        |                |               | DTL, XMI              | DYNAMIC        |
|                    |                    |                 |                |               |                       |                |
|-------------------------------------------------------------------------------------------------------------------------------------|
"""
TESTLOADER              = 'FALSE'
#TRANFORMATION LOADER RESOURCES
INDEPPROGPATH           = ''
INDEPDATAPATH           = ''
EXPORTER_VERSION        = ''
IMPORTER_VERSION        = ''
#TRANFORMATION DB RESOURCES
DB_INSTROOT_SOURCE      = ''
DB_INSTROOT_TARGET      = ''
DB_VERSION_SOURCE       = ''
DB_VERSION_TARGET       = ''
DB_INDEPDATAPATH_SOURCE = ''
DB_INDEPDATAPATH_TARGET = ''
DB_DEFAULTCODE_SOURCE   = 'ASCII'
DB_CODE_SOURCE          = 'ASCII'
DB_DEFAULTCODE_TARGET   = 'ASCII'
DB_CODE_TARGET          = 'ASCII'
DBA_SOURCE              = ''
DBA_TARGET              = ''
DB_SCHEMAS_SOURCE       = ''
DB_SCHEMAS_TARGET       = ''
#TRANSFORMATION CONFIGURATION
LOADERCONFIGURATION     = None
LOADERCONFIGFILE        = None
LOADERPROGPATH          = None
LOADERDATAPATH          = None
#TRANSFORMATION RUNTIME
PACKAGETIMEOUT          = 0
TRANSACTIONSIZE         = 1000
ISOLATIONLEVEL          = 3
MAXERRORCOUNT           = 150
AUTOCOMMIT              = 'ON'
#TRANSFORMATION PARAMETERS
PACKAGEEXECUTION        = 'EXECUTE'
PACKAGETIMESTAMP        = ''
PACKAGEARCHIVE          = 'TAR'
PACKAGEPATH             = 'HOMEDIR'
PACKAGENAME             = 'TIMESTAMP'
VERBOSE                 = 'ON'
MONITOR                 = 'OFF'
TRANSFORMATIONSYNTAX    = 'UNIFIED' 
SYSTEMATTRIBUTE         = 'OFF'
FIELDCOMPRESSED         = '/,/\"/'
FIELDBOOLVALUE          = 'TRUE/FALSE'
FIELDDECIMALVALUE       = '///'
FIELDNULLVALUE          = '?                   '
FIELDATEVALUE           = 'ISO'
FIELDTIMEVALUE          = 'ISO' 
FIELDTIMESTAMPVALUE     = 'ISO' 
FIELDLOBMASK            = '####'

#TRANSFORMATION PACKAGE
EXISTSDBA               = "SELECT DISTINCT OWNER    FROM DOMAIN.USERS   WHERE USERMODE     LIKE 'SYSDBA'" 
MAXUSERTASKS            = ''
USEDUSERTASKS           = ''
DEFAULTCODE             = ''
CODE                    = ''
DISTINCTCODE            = "SELECT DISTINCT CODETYPE FROM DOMAIN.COLUMNS WHERE OWNER = '%s' AND TABLENAME = '%s' AND CODETYPE <> ''"
PACKAGEIMPORT           = ''
PACKAGEMAXSTEP_SOURCE   = ''
PACKAGEMAXSTEP_TARGET   = ''
PACKAGEROWCOUNT_SOURCE  = ''
PACKAGEROWCOUNT_TARGET  = ''
STEPIDROWCOUNT_SOURCE   = ''
STEPIDROWCOUNT_TARGET   = ''
STEPIDSROWCOUNT_SOURCE  = ''
STEPIDSROWCOUNT_TARGET  = ''
PACKAGEVALIDATE         = ''

#TRANSFORMATION CONSTANTS
DATE                    = 10
DATEINTERNAL            =  8
TIME                    =  8
TIMEINTERNAL            =  8
TIMESTAMP               = 26
TIMESTAMPINTERNAL       = 20
SMALLINT                =  5
INTEGER                 = 10
HEXLENGTH               =  2
SIGN                    =  2
FLOAT                   = 16
FLOATMASK               =  6  
LONGPOSITION            = 21
STEPID                  =  0

def _options ():
     return [
     #Mode
     ('mode','mode'                 , ':', ''                    , 'specifies mode'  ),                         # PARAM,INTER,DEBUG
     ('T', 'transporttype'          , ':', ''                    , 'type         of transport'),                # EXECUTE, GENERATE
     #Transformation Attributes
     ('t', 'transformationtype'     , ':', ''                    , 'type         of transformation '),          # IMPORT , EXPORT, TRANSPORT
     ('p', 'transformationpackage'  , ':', ''                    , 'package      of transformation '),          # DB     , USER,   TABLE
     ('',  'packagename'            , ':', ''                    , 'packagename  of transformation '),          # DB     , USER,   TABLE
     ('P', 'transformationpart'     , ':', ''                    , 'packagepart  of transformation '),          # DB     , USER,   TABLE
     ('m', 'streammedium'           , ':', ''                    , 'streammedium of transformation '),          # PIPE   , FILE,   TAPE
     ('f', 'streamformat'           , ':', ''                    , 'streamformat of transformation '),          # RECORDS, PAGES,  COMPRESSED, FORMATTED, DDL, XML
     ('F', 'streamformat_target'    , ':', ''                    , 'streamformat of transformation '),          # PAGE   , ROW  ,  DDL, DTL
     ('c', 'streamcharencoding'     , ':', ''                    , 'streamcoding of transformation '),          # ASCII  , UCS2 ,  UTF8, DYNAMIC
     ('l', 'streamlob'              , ':', ''                    , 'lobmask      of transformation '),          # TABLE  , COLUMN, COLUMNVALUE
     ('i', 'initposition'           , ':', 'START'               , 'initposition of transformation '),          # START  , RESTART
     #Transformation Resources 
     ('nSRC', 'sourcedbnode'        , ':', ''                    , 'specifies source serverdb node'),
     ('dSRC', 'sourcedbname'        , ':', ''                    , 'specifies source serverdb to create'),
     ('uSRC', 'sourcedbuser'        , ':', ''                    , 'specifies source user'),
     ('sSRC', 'sourcedbschema'      , ':', ''                    , 'specifies source schema'),
     ('nTRG', 'targetdbnode'        , ':', ''                    , 'specifies target serverdb node '),
     ('dTRG', 'targetdbname'        , ':', ''                    , 'specifies target serverdb to create'),
     ('uTRG', 'targetdbuser'        , ':', ''                    , 'specifies target user'),
     ('sTRG', 'targetdbschema'      , ':', ''                    , 'specifies target schema'),
     #Transformation Loader Resources
     ('R', 'loaderinstroot'         , ':', ''                    , 'specifies loader instroot '),                # 
     #Transformation Resources Attributes
     ('X', 'migration'              , ':', 'sdb'                 , 'specifies source database system'),          # SDB, ADABAS
     ('j', 'sourceindepdatapath'    , ':', ''                    , 'specifies source loader independent path'),  # 
     ('J', 'targetindepdatapath'    , ':', ''                    , 'specifies target loader independent path'),  # 
]

def signal_handler(sig):
    if( sig == signal.SIGINT   or
        sig == signal.SIGTER   or
        sig == signal.SIGABR   or
        sig == signal.SIGBREAK   ):
        try:
           os._exit(0)
        except OSError, err:
           print err
           
def main (options, args):
    input   = None
    rc      = True
    #print "\nLoader transport module:\n'%s'" % os.path.abspath(sys.argv[0])
    if import_package(options) == True :
       try:
          # Transformation Package Design
          design_package(options) 
          export_package(options)
          # Transformation Package Transport
          if options.mode == 'PARAM' :
             input = '1'
          if options.mode != 'PARAM' :
             while input not in ['1','2'] :
                   print '\nStep %i: Execute transformation package:'   % STEPID
                   input = raw_input(' [1] OK\n [2] CANCEL\n\nStep %i: ' %STEPID )
          if input in ['1']:     
             package = TransformationPackage(options)
             # Transformation Package Execution
             # Transformation Package Validation
             rc = package.cmd_execute()
             # Transformation Package Release
             package.release()
             # Transformation Package Archive
             if rc == True:
                package.cmd_archive()
       except EOFError:
          thread.interrupt_main()
       except EnvironmentError:
          thread.interrupt_main()
       except KeyboardInterrupt:
          thread.interrupt_main()
       except:
          sys.exc_traceback
       
class TransformationPackage (sax.ContentHandler):
      # transformationpackage model2xml
      def __init__(self, options = None ):
          self.options                 = options
          self.sourcepackagename       = os.path.split(self.options.sourceindepdatapath)[1]
          self.targetpackagename       = os.path.split(self.options.targetindepdatapath)[1]
          self.starttime               = None
          self.stoptime                = None
          #package   
          self.transformation          = None
          self.packagereader           = None
          self.packagedom              = None
          self.sourceconfiguration     = os.path.join(self.options.sourceindepdatapath,  '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1])
          self.targetconfiguration     = os.path.join(self.options.targetindepdatapath,  '%s.CONFIGURATION' % os.path.split(options.targetindepdatapath)[1])
          self.transformationpackage   = None
          self.transformationparameter = None
          self.transformation_step     = []
          self.maxerrorcount           = 0
          #db                        
          self.sapdbnode               = None
          self.sapdb                   = None
          #user  
          self.maxstepid               = 0
          self.stepid                  = None                   
          self.user                    = None
          self.pwd                     = None
          self.schema                  = ''
          self.usercount               = 0
          #table                     
          self.table                   = None
          self.tablecount              = 0
          self.index                   = None
          self.indexStatement          = ''
          self.indexStatements         = []
          self.trigger                 = None
          self.triggerstatement        = None
          #column                    
          self.column                  = ''
          self.columnIsNullable        = ''
          self.columnInitialValue      = ''
          self.columnEncoding          = ''
          self.columnLength            = 0
          self.columncount             = 0
          self.keycolumn               = None
          self.column_long             = []
          #field 
          self.fieldCompressed         = FIELDCOMPRESSED
          self.fieldDecimalValue       = FIELDDECIMALVALUE
          self.fieldNullValue          = FIELDNULLVALUE
          self.fieldBoolValue          = FIELDBOOLVALUE
          self.fieldDateValue          = FIELDATEVALUE
          self.fieldTimeValue          = FIELDTIMEVALUE
          self.fieldTimeStampValue     = FIELDTIMESTAMPVALUE                    
          self.field_startpos          = 1
          self.field_endpos            = 1
          #transformation            
          self.column_field            = ''
          self.column_field_long       = ''
          self.statement               = ''
          if self.options.streamcharencoding == 'DYNAMIC':
             self.options.streamcharencoding  = ''
      
      def release(self):
          self.transformation.release()
      
      # model parser
      def startElement (self, name, attrs):
          #Datasource
          if name == 'dbm:DataSource':
             self.startDataSource()
          #Connection
          if name == 'sdbm:Connection':
             connectString =  map(string.strip, string.split(attrs.get('dbm:ConnectString', None),";"))
             pos           = 1
             for i in connectString:
                 if pos == 1 :
                    pass
                 if pos == 2 :
                    self.sapdbnode  =  ''
                 if pos == 3 :
                    self.sapdb      =  self.options.sourcedbname
                 if pos == 4 :
                    self.user       =  re.split(',',self.options.sourcedbuser)[0]
                 if pos == 5 :
                    self.pwd        = 'initial'
                 pos = pos + 1
          #Catalog
          if name == 'dbm:DeployedCatalog':
             if(self.options.transformationpart == 'DB'):
                self.startCatalog()
          #Schema
          if name == 'sdbm:DeployedSchema' :
             if(self.options.transformationpart     == 'USER' or
                self.options.transformationpart     == 'TABLE' ):
                if  self.options.transformationtype == 'EXPORT' and self.options.transformationpart == 'TABLE' :
                    if self.options.sourcedbschema  == '':
                       self.schema           = attrs.get('oim:name',None )
                    if self.options.sourcedbschema  != '':
                       self.schema  = self.options.sourcedbschema
                if  self.options.transformationtype == 'IMPORT':
                    if self.options.targetdbschema  != '':
                       self.schema    = self.options.targetdbschema
                self.user             = attrs.get('sdbm:Username',None )
                if self.options.transformationpart != 'DB' and TESTLOADER == 'TRUE':
                   self.options.sourcedbuser     = '%s,%s' % ( self.user, self.pwd )
                   self.options.sourcedbschema   = '%s'    % ( self.schema )
                   self.options.targetdbuser     = '%s,%s' % ( self.user, self.pwd )
                   self.options.targetdbschema   = '%s'    % ( self.schema )
                self.startSchema()
          #Table
          if(self.options.transformationpart == 'TABLE' and (
             name == 'sdbm:DeployedTable' )) :
             self.table            = attrs.get('oim:name', None)
             self.startTable()
          #Index   
          if(self.options.transformationpart == 'TABLE' and (
             name == 'dbm:Index')):
             self.index            = attrs.get('dbm:name', None)
             self.indexStatement   = 'CREATE '
             if attrs.get('dbm:IsUnique',None) == '1':
                self.indexStatement= self.indexStatement + ' UNIQUE '
             self.indexStatement = self.indexStatement   + ' INDEX '   
             self.indexStatement = self.indexStatement   + ' "%s" ON "%s" ' % (self.index, self.table )
             self.indexStatement = self.indexStatement   + ' ( '
          if(self.options.transformationpart == 'TABLE' and (
             name == 'dbm:IndexColumn' )) :
             self.indexStatement   = self.indexStatement + ' "%s" ' % attrs.get('dbm:name', None)
             if attrs.get('dbm:IsAscending',None) == '0':
                self.indexStatement= self.indexStatement + ' DESC,'
             if attrs.get('dbm:IsAscending',None) == '1':
                self.indexStatement= self.indexStatement + ' ASC,'
          if(self.options.transformationpart == 'TABLE' and 
             name == 'dbm:DeployedTrigger'):
             trigger               = attrs.get('oim:id', None)
             self.triggerstatement = attrs.get('dbm:Statements', None )
             string.replace( trigger, '_%s.' % self.user  , '' )
             string.replace( trigger, '.%s'  % self.table , '' )
             self.trigger = trigger
          #Columns
          if(self.options.transformationpart == 'TABLE' and (
             name == 'dbm:DeployedColumn' )): 
             self.column            = attrs.get('dbm:name', None)
             self.columnIsNullable  = attrs.get('dbm:IsNullable'  ,None)
             self.columnInitialValue= attrs.get('dbm:initialValue',None)
             self.columnLength      = attrs.get('dbm:Length',None)
             self.startColumn()
          if self.options.transformationpart == 'TABLE' and name == 'dbm:KeyColumns' :
             self.keycolumn = 'true'
          #Column Types   
          if(self.options.transformationpart == 'TABLE' and (
             name == 'sdbm:BOOLEAN')):
             if self.keycolumn != 'true':
                self.columnTypeBool(name)
          if(self.options.transformationpart == 'TABLE' and (
             name == 'sdbm:CHAR'   or 
             name == 'sdbm:VARCHAR'or
             name == 'sdbm:BINARY' )):
             if self.keycolumn != 'true':
                if(name == 'sdbm:CHAR'    or 
                   name == 'sdbm:VARCHAR'   ):
                   self.columnEncoding = attrs.get('sdbm:Encoding', None) 
                self.columnTypeCharacter(name)
          if(self.options.transformationpart == 'TABLE' and (
             name == 'sdbm:SMALLINT'or
             name == 'sdbm:INTEGER' or
             name == 'sdbm:DECIMAL' or
             name == 'sdbm:FIXED'   or
             name == 'sdbm:FLOAT'   or
             name == 'sdbm:REAL'    or
             name == 'sdbm:DOUBLE_PRECISION' or
             name == 'sdbm:NUMERIC' )):
             if self.keycolumn != 'true':
                self.columnTypeNumber(name)
          if(self.options.transformationpart == 'TABLE' and (
             name == 'sdbm:DATE' or
             name == 'sdbm:TIME' or
             name == 'sdbm:TIMESTAMP' )):
             if self.keycolumn != 'true':
                self.columnTypeDateTime(name)
          if(self.options.transformationpart == 'TABLE' and (                
             name == 'sdbm:LONG' or
             name == 'sdbm:LONG_BYTE'  )):
             if self.keycolumn != 'true':
                self.columnTypeLong(name)

      def endElement(self, name ):
          #DataSource
          if name == 'dbm:DataSource':
             self.endDataSource()
          #Catalog
          if name == 'dbm:DeployedCatalog' :
             self.endCatalog()
          #Schema
          if name == 'sdbm:DeployedSchema':
             self.endSchema()
          #Table
          if(self.options.transformationpart == 'TABLE' and 
             name == 'sdbm:DeployedTable') :
             self.endTable ()
          #Index 
          if(self.options.transformationpart == 'TABLE' and 
             name == 'dbm:Index'):
             self.endIndex()
          #Trigger
          if(self.options.transformationpart == 'TABLE' and 
             name == 'dbm:DeployedTrigger'):
             self.endTrigger()
          #Columns
          if(self.options.transformationpart == 'TABLE' and 
             name == 'dbm:DeployedColumn'):
             self.endColumn()
          if(self.options.transformationpart == 'TABLE' and 
             name == 'dbm:KeyColumns') :
             self.endKeyColumns()
      
      # model export
      def cmd_export(self):
          rc = True
          if self.options.transformationtype == 'TRANSPORT' or self.options.transformationtype == 'EXPORT' :
             if(self.options.transformationpackage  == 'DB'     or
                self.options.transformationpackage  == 'USER'   or
                self.options.transformationpackage  == 'SCHEMA' or
                self.options.transformationpackage  == 'TABLE'     ):
                
                print '\nExport    transformation package'
                protocol =  self.transformation.exporter.protocol
                self.fieldCompressed     = self.transformation.exporter.fieldCompressed     
                self.fieldDecimalValue   = self.transformation.exporter.fieldDecimalValue     
                self.fieldNullValue      = self.transformation.exporter.fieldNullValue
                self.fieldBoolValue      = self.transformation.exporter.fieldBoolValue
                self.fieldDateValue      = self.transformation.exporter.fieldDateValue     
                self.fieldTimeValue      = self.transformation.exporter.fieldTimeValue     
                self.fieldTimeStampValue = self.transformation.exporter.fieldTimeStampValue 
                
                if self.options.sourcedbschema != '':
                   packageuser = 'USE USER %s %s SCHEMA %s SERVERDB %s ON %s \n' % ( re.split(',',self.options.sourcedbuser)[0], re.split(',',self.options.sourcedbuser)[1], self.options.sourcedbschema, self.options.sourcedbname, self.options.sourcedbnode )
                else:
                   packageuser = 'USE USER %s %s           SERVERDB %s ON %s \n' % ( re.split(',',self.options.sourcedbuser)[0], re.split(',',self.options.sourcedbuser)[1],                              self.options.sourcedbname, self.options.sourcedbnode )                
                   
                self.transformation.exporter.cmd  = packageuser
                self.transformation.exporter.cmd_execute()
                  
                # Transport Package and Transport Part
                configuration    = "SET CONFIGURATION EXCLUDE \n" 
                if self.options.transformationpackage == 'DB' :        
                   package       = "EXPORT DB \n"
                   package       = package + " CONFIGURATION INSTREAM FILE '%s'            %s\n"  % (os.path.join(self.options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(self.options.sourceindepdatapath)[1]) , 'ASCII' ) 
                   package       = package + " CATALOG      OUTSTREAM FILE '%s' XML          \n"  %  os.path.join(self.options.sourceindepdatapath, '%s.xml'           % os.path.split(self.options.sourceindepdatapath)[1])
                   package       = package + " PACKAGE      OUTSTREAM FILE '%s' COMPRESSED %s\n"  % (os.path.join(self.options.sourceindepdatapath, '%s.PACKAGE'       % os.path.split(self.options.sourceindepdatapath)[1]) , 'UTF8' )
                if self.options.transformationpackage == 'USER' :   
                   package       = "EXPORT USER \n"
                   package       = package + " CONFIGURATION INSTREAM FILE '%s'            %s \n" % (os.path.join(self.options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(self.options.sourceindepdatapath)[1]) , 'ASCII' )
                   package       = package + " CATALOG      OUTSTREAM FILE '%s' XML           \n" %  os.path.join(self.options.sourceindepdatapath, '%s.xml'           % os.path.split(self.options.sourceindepdatapath)[1])
                   package       = package + " PACKAGE      OUTSTREAM FILE '%s' COMPRESSED % s\n" % (os.path.join(self.options.sourceindepdatapath, '%s.PACKAGE'       % os.path.split(self.options.sourceindepdatapath)[1]) , 'UTF8' )
                if self.options.transformationpackage == 'SCHEMA' :   
                   package       = "EXPORT USER \n"
                   package       = package + " CONFIGURATION INSTREAM FILE '%s'            %s \n" % (os.path.join(self.options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(self.options.sourceindepdatapath)[1]) , 'ASCII' )
                   package       = package + " CATALOG      OUTSTREAM FILE '%s' XML           \n" %  os.path.join(self.options.sourceindepdatapath, '%s.xml'           % os.path.split(self.options.sourceindepdatapath)[1])
                   package       = package + " PACKAGE      OUTSTREAM FILE '%s' COMPRESSED % s\n" % (os.path.join(self.options.sourceindepdatapath, '%s.PACKAGE'       % os.path.split(self.options.sourceindepdatapath)[1]) , 'UTF8' )                   
                if self.options.transformationpackage == 'TABLE' and self.options.transformationpart == 'TABLE':
                   configuration = "SET CONFIGURATION INCLUDE \n" 
                   package       = "EXPORT USER \n" 
                   package       = package + " CONFIGURATION INSTREAM FILE '%s'            %s \n" % (os.path.join(self.options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(self.options.sourceindepdatapath)[1]) , 'ASCII' )
                   package       = package + " CATALOG      OUTSTREAM FILE '%s' XML           \n" %  os.path.join(self.options.sourceindepdatapath, '%s.xml'           % os.path.split(self.options.sourceindepdatapath)[1])
                   package       = package + " PACKAGE      OUTSTREAM FILE '%s' COMPRESSED %s \n" % (os.path.join(self.options.sourceindepdatapath, '%s.PACKAGE'       % os.path.split(self.options.sourceindepdatapath)[1]) , 'UTF8' )
 
                self.transformation.exporter.cmd =  configuration
                self.transformation.exporter.cmd_execute()
                self.transformation.exporter.cmd =  package
                self.transformation.exporter.cmd_execute()

                if os.path.isfile( os.path.join(self.options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(self.options.sourceindepdatapath)[1] )) == False:
                   pass
                if os.path.isfile( os.path.join(self.options.sourceindepdatapath, '%s.xml'           % os.path.split(self.options.sourceindepdatapath)[1] )) == False:
                   print '%s.xml does not exist ' % os.path.split(self.options.sourceindepdatapath)[1]
                   rc = False
                else:
                   self.transformationpackage = os.path.join(self.options.sourceindepdatapath, '%s.xml' % os.path.split(self.options.sourceindepdatapath)[1])                       
             else:
                self.transformationpackage    = os.path.join( os.path.split(self.options.sourceindepdatapath)[0], self.options.transformationpackage,  '%s.xml' % self.options.transformationpackage )
          if self.options.transformationtype == 'IMPORT' :
             if os.path.isfile             ( os.path.join(self.options.sourceindepdatapath, '%s.xml'% os.path.split(self.options.sourceindepdatapath)[1] )) == False:
                print '%s.xml does not exist ' % os.path.split(self.options.sourceindepdatapath)[1]
                rc = False
             else:
                self.transformationpackage = os.path.join(self.options.sourceindepdatapath, '%s.xml' % os.path.split(self.options.sourceindepdatapath)[1])                         
          return rc

      # model import
      def cmd_import (self): 
          rc = True
          droptables = []
          if  os.path.isfile(self.transformationpackage) and  os.stat(self.transformationpackage)[stat.ST_SIZE] <= 2000000 :
              exportmonitor = None
              importmonitor = None
              if self.transformation.exporter != None:
                 exportmonitor = self.transformation.exporter.monitor
              if self.transformation.importer != None:
                 importmonitor = self.transformation.importer.monitor
              print  '\nImport    transformation package'
              packagedom = parse( self.transformationpackage )
              schemas    = packagedom.getElementsByTagName('sdbm:DeployedSchema')
              for schema in schemas:
                  print 'User      %s' % (schema.getAttribute ('sdbm:Username'))
                  print 'Schema    %s' % (schema.getAttribute ('oim:name'))
                  if( self.options.transformationpart == 'USER' or
                      self.options.transformationpart == 'TABLE'  ):
                      schemaname =  '"' + schema.getAttribute ('oim:name') + '"'
                      tables     =  schema.getElementsByTagName('sdbm:DeployedTable')
                      for table in tables:
                          tablename = '"' + table.getAttribute('oim:name') + '"'
                          rc = exist_table(self.options,exportmonitor,importmonitor,schemaname,tablename)
                          if rc == False:
                             rc  = False
                             droptables.append(tablename)
                          else:
                             print ' Table    %s' % tablename
                          if(self.options.transformationtype == 'TRANSPORT' or self.options.transformationtype == 'IMPORT') and rc == True:
                             varchars = table.getElementsByTagName('sdbm:VARCHAR')
                             for varchar in varchars : 
                                 if rc == True:
                                    columnEncoding = varchar.getAttribute('sdbm:Encoding')
                                    if columnEncoding== 'UNICODE' and  DB_CODE_TARGET == 'ASCII':
                                       print 'Table     %s contains UNICODE Character Columns that can not be imported in an ASCII Database' % tablename
                                       rc = False
                             chars = table.getElementsByTagName('sdbm:CHAR')
                             for char in chars : 
                                 if rc == True:
                                    columnEncoding = char.getAttribute('sdbm:Encoding')
                                    if columnEncoding== 'UNICODE' and  DB_CODE_TARGET == 'ASCII':
                                       print 'Table     %s contains UNICODE Character Columns that can not be imported in an ASCII Database' % tablename
                                       rc = False
              packagedom.unlink()
          return rc  
                 
      # model execution
      def cmd_execute  (self):
          global   STEPID
          global   DB_DEFAULTCODE_SOURCE
          global   DB_DEFAULTCODE_TARGET
          rc = True
          try:
             input   = None
             self.transformation = Transformation( self.options, self )
             # Transformation Package Init
             if self.cmd_export() == True and self.cmd_import() == True:
                self.starttime     = time.time()
                self.transformation.cmd_stamp_package() 
                # Transformation Package Execution
                self.options.mode  = 'PARAM' 
                packageparser = sax.make_parser()
                packageparser.setContentHandler( self ) 
                packageparser.parse (self.transformationpackage )
                # Transformation Package Validation
                if self.options.transporttype == 'EXECUTE' or self.options.transporttype == 'GENERATE':
                   if  self.options.transformationtype    == 'TRANSPORT':
                       transformationpackage_dir  = os.path.split(self.options.sourceindepdatapath)[0]
                       transformationpackage_name = os.path.split(self.options.sourceindepdatapath)[1]
                   elif  self.options.transformationtype  == 'EXPORT':
                       transformationpackage_dir  = os.path.split(self.options.sourceindepdatapath)[0]
                       transformationpackage_name = os.path.split(self.options.sourceindepdatapath)[1]
                   elif  self.options.transformationtype  == 'IMPORT':
                       transformationpackage_dir  = os.path.split(self.options.targetindepdatapath)[0]
                       transformationpackage_name = os.path.split(self.options.targetindepdatapath)[1]
                   #elif self.options.transformationpackage != '':
                   #    transformationpackage_name = self.options.transformationpackage 
                   #PACKAGE CONTENT
                   shutil.copy(os.path.join(LOADERPROGPATH,'sdb','loader','config','loader.cfg'), os.path.join(transformationpackage_dir,transformationpackage_name,transformationpackage_name + os.extsep + 'cfg'))
                   transformation_source_connect = ''
                   transformation_target_connect = ''
                   if self.options.streamcharencoding == '':
                      streamcharencoding = 'DYNAMIC'
                   if self.options.streamcharencoding != '':
                      streamcharencoding = self.options.streamcharencoding
                   if self.options.streamcharencoding == '':
                      self.options.streamcharencoding  = 'DYNAMIC'
                   if self.options.transformationtype == 'IMPORT':
                      self.options.transformationpackage =  os.path.split(self.options.sourceindepdatapath)[1]
                   if self.options.transformationtype ==  'TRANSPORT' or self.options.transformationtype == 'EXPORT' :
                      transformation_source_connect = " -nSRC %s -dSRC %s -uSRC %s -sSRC %s " % ( self.options.sourcedbnode, self.options.sourcedbname, re.split(',',self.options.sourcedbuser)[0]+',*', self.options.sourcedbschema)
                   if self.options.transformationtype ==  'TRANSPORT' or self.options.transformationtype == 'IMPORT':
                      transformation_target_connect = " -nTRG %s -dTRG %s -uTRG %s -sTRG %s " % ( self.options.targetdbnode, self.options.targetdbname, re.split(',',self.options.targetdbuser)[0]+',*', self.options.targetdbschema)   
                   transformationparameter          = " -T %s -t %s -p %s:%s -P %s -i %s -m %s -f %s -F %s -c %s -l %s "  % ( self.options.transporttype, self.options.transformationtype, self.options.transformationpackage, self.options.packagename, self.options.transformationpart, self.options.initposition, self.options.streammedium,self.options.streamformat,self.options.streamformat_target,streamcharencoding, self.options.streamlob)
                   transformationparameter          = transformationparameter + transformation_source_connect + transformation_target_connect
                   transformationparameters         = file(os.path.join(transformationpackage_dir,transformationpackage_name,transformationpackage_name+os.extsep+'par'),'w+')
                   transformationparameters.write( transformationparameter )                   
                   transformationparameters.close()
                   #PACKAGE HEADER
                   transformation_source_connect    = "|%-20s |%-20s |%-20s |%-20s"  % ( '', '', '', '' )
                   transformation_target_connect    = "|%-20s |%-20s |%-20s |%-20s |"% ( '', '', '', '' )
                   if os.path.isfile(os.path.join(LOADERDATAPATH,'sdb','loader','packages','transport'+os.extsep+'log')) == False:
                      transformation_id             = "|%-48s |%-16s |%-16s |%-15s |%-15s "   % ( 'PACKAGEGUID'   , 'EXPORTER_VERSION' , 'IMPORTER_VERSION', 'SOURCE_USERKEY', 'TARGET_USERKEY' )
                      transformation_source_connect = "|%-20s |%-20s |%-20s |%-20s " %( 'SOURCE_NODE -nSRC', 'SOURCE_DB -dSRC'  , 'SOURCE_USER -uSRC', 'SOURCE_SCHEMA -sSRC')
                      transformation_target_connect = "|%-20s |%-20s |%-20s |%-20s "% ( 'TARGET_NODE -nTRG', 'TARGET_DB -dTRG'  , 'TARGET_USER -uTRG', 'TARGET_SCHEMA -sTRG')
                      source_target_versions        = "|%-15s |%-15s "        % ( 'SOURCE_VERSION', 'TARGET_VERSION')
                      transformationparameter_1     =  "%-20s |%-15s |%-10s |%-10s |%-15s |%-10s |%-10s |%-10s |%-10s  %-10s |%-10s |%-11s  |%-10s "  % ( 'PACKAGETIMESTAMP', 'PACKAGENAME' , 'TYPE -T','TYPE -t' , 'PACKAGE -p', 'PART -P', 'INITPOS -i', 'MEDIUM -m','FORMAT -f','-F ','CODING -c', 'LOB -l','ARCHIVE')
                      transformationparameter_1     = transformationparameter_1 + transformation_id +  transformation_source_connect + transformation_target_connect + source_target_versions + '\n'
                      transformation_id             = "|%-48s |%-16s |%-16s |%-15s |%-15s " % ( '13', '14', '15', '16', '17')
                      transformation_source_connect = "|%-20s |%-20s |%-20s |%-20s "        % ( '18', '19', '20', '21' )
                      transformation_target_connect = "|%-20s |%-20s |%-20s |%-20s "        % ( '22', '23', '24', '25' )
                      source_target_versions        = "|%-15s |%-15s "                      % ( '26', '27')
                      transformationparameter_2     = "%-20s |%-15s |%-10s |%-10s |%-15s |%-10s |%-10s |%-10s |%-10s |%-10s |%-10s |%-11s  |%-10s "  % ( '0' , '1' , '2', '3', '4', '5','6','7','8','9','10','11','12')
                      transformationparameter_2     = transformationparameter_2 + transformation_id + transformation_source_connect + transformation_target_connect + source_target_versions + '\n'
                      transformationparameter       = transformationparameter_1 + transformationparameter_2 
                      transformationparameters      = file(os.path.join(LOADERDATAPATH,'sdb','loader','packages','transport'+os.extsep+'log'),'a+')
                      transformationparameters.write( transformationparameter )
                      transformationparameters.close()
                   #PACKAGE BODY
                   transformationpackage_guid = ''
                   if self.options.transformationtype == 'TRANSPORT' or self.options.transformationtype == 'EXPORT':
                      transformationpackage_guid = '' #self.transformation.exporter.packageguid 
                   if self.options.transformationtype == 'IMPORT' :
                      transformationpackage_guid = '' #self.transformation.importer.packageguid 
                   transformation_id                = "|%-48s |%-16s |%-16s |%-15s |%-15s " % ( transformationpackage_guid, EXPORTER_VERSION, IMPORTER_VERSION, '', '' )
                   if self.options.transformationtype    == 'TRANSPORT' or self.options.transformationtype == 'EXPORT' or self.options.transformationtype == 'IMPORT' :
                      transformation_source_connect      = "|%-20s |%-20s |%-20s |%-20s " % ( self.options.sourcedbnode, self.options.sourcedbname, re.split(',',self.options.sourcedbuser)[0]+',*', self.options.sourcedbschema)
                      if self.options.transformationtype == 'EXPORT':
                          transformation_target_connect  = "|%-20s |%-20s |%-20s |%-20s " % ('','','', '')
                   if self.options.transformationtype    == 'TRANSPORT' or self.options.transformationtype == 'IMPORT' :                      
                      transformation_target_connect = "|%-20s |%-20s |%-20s |%-20s" % ( self.options.targetdbnode, self.options.targetdbname, re.split(',',self.options.targetdbuser)[0]+',*' , self.options.targetdbschema)
                   source_target_versions           = "|%-15s |%-15s  "        % ( DB_VERSION_SOURCE, DB_VERSION_TARGET)
                   transformationparameter          =  "%-20s |%-15s |%-10s |%-10s |%-15s |%-10s |%-10s |%-10s |%-10s |%-10s |%-10s |%-11s  |%-10s "  % ( PACKAGETIMESTAMP, transformationpackage_name, self.options.transporttype, self.options.transformationtype, self.options.transformationpackage, self.options.transformationpart, self.options.initposition, self.options.streammedium,self.options.streamformat,self.options.streamformat_target,self.options.streamcharencoding, self.options.streamlob, PACKAGEARCHIVE)
                   transformationparameter          = transformationparameter + transformation_id + transformation_source_connect + transformation_target_connect + source_target_versions + '\n'
                   transformationparameters         = file(os.path.join(LOADERDATAPATH,'sdb','loader','packages','transport'+os.extsep+'log'),'a+')
                   transformationparameters.write( transformationparameter )                   
                   transformationparameters.close()
                   self.stoptime  = time.time()  
                   print 'Transformation package name   : %s '    % transformationpackage_name
                   print 'Transformation package time   : %.3fs ' % float( self.stoptime - self.starttime )
                   print 'Transformation package errors : %i '    % self.maxerrorcount
             else:
                 rc = False
          except IOError , (errno,err):
            rc = False
            print '\nError parsing the package information \n  %s : %s ' % ( err, self.transformationpackage )
          except:
            rc = False
            kind, val, traceback = sys.exc_info()
            raise kind, val, traceback
          return rc
      
      # model package validation
      def cmd_verify(self):
          self.transformation.cmd_verify_package()
      
      # model package statistic
      def cmd_statistic(self):
          print 'Transformation package time   : %.3fs ' % float( self.stoptime - self.starttime )
          print 'Transformation package errors : %i '    % self.maxerrorcount
      
      # model package archive    
      def cmd_archive(self):
          if PACKAGEARCHIVE == 'ZIP':
             if( self.options.transformationtype   == 'TRANSPORT' or
                 self.options.transformationtype   == 'EXPORT'   ):
                 try:
                     zipdirname  = os.path.split(self.options.sourceindepdatapath)[0]
                     zipfilename = os.path.split(self.options.sourceindepdatapath)[1]+os.extsep+'zip'
                     zip =  zipfile.ZipFile(os.path.join (zipdirname,zipfilename), 'w', zipfile.ZIP_DEFLATED )  
                     print 'Transformation package archive: %s ' % zipfilename
                     os.chdir(self.options.sourceindepdatapath)
                     for filename in os.listdir(os.getcwd()):
                         zip.write(filename)
                     zip.close()
                     #VALIDATE PACKAGE
                     zip =  zipfile.ZipFile(os.path.join (zipdirname,zipfilename), 'r', zipfile.ZIP_DEFLATED )  
                     if zip.testzip() == None:
                        zip.close()
                        os.chdir(self.options.sourceindepdatapath)
                        for filename in os.listdir(os.getcwd()):
                            os.unlink(filename)
                        os.chdir (os.path.split(self.options.sourceindepdatapath)[0])
                        time.sleep(1)
                        os.removedirs(self.options.sourceindepdatapath)
                     else:
                        print 'Not ok'
                 except RuntimeError, err:
                     print err
                 except OSError, (errno, err ):
                     if errno != 13:
                        print err
                 except IOError, (errno, err ):
                     print err
                 except:
                     kind, val, traceback = sys.exc_info()
                     raise kind, val, traceback
             if( self.options.transformationtype  == 'TRANSPORT' or
                 self.options.transformationtype  == 'IMPORT'  ):
                 try:
                     zipdirname  = os.path.split(self.options.targetindepdatapath)[0]
                     zipfilename = os.path.split(self.options.targetindepdatapath)[1]+os.extsep+'zip'
                     zip =  zipfile.ZipFile(os.path.join (zipdirname,zipfilename), 'w', zipfile.ZIP_DEFLATED )  
                     if  self.options.transformationtype  == 'IMPORT':
                         print 'Transformation package archive: %s ' % zipfilename
                     os.chdir(self.options.targetindepdatapath)
                     for filename in os.listdir(os.getcwd()):
                         zip.write(filename)
                     zip.close()
                     #VALIDATE PACKAGE
                     zip =  zipfile.ZipFile(os.path.join (zipdirname,zipfilename), 'r', zipfile.ZIP_DEFLATED )  
                     if zip.testzip() == None:
                        zip.close()
                        os.chdir(self.options.targetindepdatapath)
                        for filename in os.listdir(os.getcwd()):
                            os.unlink(filename)
                        os.chdir (os.path.split(self.options.targetindepdatapath)[0])
                        time.sleep(1)
                        os.removedirs(self.options.targetindepdatapath)
                     else:
                        print 'Not ok'
                 except RuntimeError, err:
                     print err
                 except OSError, (errno, err ):
                     if errno != 13:
                        print err
                 except IOError, (errno, err ):
                     print err
                 except:
                     kind, val, traceback = sys.exc_info()
                     raise kind, val, traceback
          if PACKAGEARCHIVE == 'TAR':
             if( self.options.transformationtype   == 'TRANSPORT' or
                 self.options.transformationtype   == 'EXPORT' ):
                 try:
                     zipdirname  = os.path.split(self.options.sourceindepdatapath)[0]
                     zipfilename = os.path.split(self.options.sourceindepdatapath)[1]+os.extsep+'tgz'
                     zip =  tarfile.TarFile.open(os.path.join (zipdirname,zipfilename), 'w:gz' )  
                     print 'Transformation package archive: %s ' % zipfilename
                     os.chdir(self.options.sourceindepdatapath)
                     for filename in os.listdir(os.getcwd()):
                         zip.add(filename)
                     zip.close()
                 except RuntimeError, err:
                     print err
                 except OSError, (errno, err ):
                     if errno != 13:
                        print err
                 except IOError, (errno, err ):
                     print err
                 except:
                     kind, val, traceback = sys.exc_info()
                     raise kind, val, traceback      
             if( self.options.transformationtype  == 'TRANSPORT' or
                 self.options.transformationtype  == 'IMPORT'  ):
                 try:
                     zipdirname  = os.path.split(self.options.targetindepdatapath)[0]
                     zipfilename = os.path.split(self.options.targetindepdatapath)[1]+os.extsep+'tgz'
                     zip =  tarfile.TarFile.open(os.path.join (zipdirname,zipfilename), 'w:gz' ) 
                     if  self.options.transformationtype  == 'IMPORT':
                         print 'Transformation package archive: %s ' % zipfilename 
                     os.chdir(self.options.targetindepdatapath)
                     for filename in os.listdir(os.getcwd()):
                         zip.add(filename)
                     zip.close()
                     #VERIFY PACKAGE
                     if tarfile.is_tarfile(os.path.join (zipdirname,zipfilename)) == True:
                        zip.close()
                        os.chdir(self.options.targetindepdatapath)
                        for filename in os.listdir(os.getcwd()):
                            os.unlink(filename)
                        os.chdir (os.path.split(self.options.targetindepdatapath)[0])
                        time.sleep(1)
                        os.removedirs(self.options.targetindepdatapath)
                     else:
                        print 'Not ok'
                 except RuntimeError, err:
                     print err
                 except OSError, (errno, err ):
                     if errno != 13:
                        print err
                 except IOError, (errno, err ):
                     print err
                 except:
                     kind, val, traceback = sys.exc_info()
                     raise kind, val, traceback
                
      def cmd_dearchive(self):
          if PACKAGEARCHIVE == 'ZIP':
             if((self.options.transformationtype   == 'TRANSPORT' or
                 self.options.transformationtype   == 'IMPORT')   and
                 self.options.streammedium         == 'FILE'):         
                 try:
                     zipdirname  = os.path.split(self.options.sourceindepdatapath)[0]
                     zipfilename = os.path.split(self.options.sourceindepdatapath)[1]+os.extsep+'zip'
                     zip =  zipfile.ZipFile(os.path.join (zipdirname,zipfilename), 'r', zipfile.ZIP_DEFLATED )
                     if zip.testzip() == None:
                        print 'Transformation package archive: %s Ok' % zipfilename
                        os.chdir(self.options.sourceindepdatapath)
                        for filename in zip.namelist():
                            unzip        = file (filename,'wb')
                            unzip.write(zip.read(filename))
                            unzip.close()
                        zip.close()
                        os.chdir(os.path.split(self.options.sourceindepdatapath)[0])
                     else:
                        print 'Not ok'
                 except IOError, err:
                     print err
                 except:
                     kind, val, traceback = sys.exc_info()
                     raise kind, val, traceback

      # model element callback functions      
      def len (self, name):
          length = 0
          if self.options.streamcharencoding == 'ASCII' :
             length = len(name)
          if self.options.streamcharencoding == 'UTF8' :
             length = len(name)
          if self.options.streamcharencoding == 'UCS2' :
             length = 2*len(name)
          return length   
          
      def int (self, name): 
          length = 0
          if self.options.streamcharencoding == 'ASCII' :
             length = int(name,10)
          if self.options.streamcharencoding == 'UTF8' :
             length = int(name,10)   
          if self.options.streamcharencoding == 'UCS2' :
             length = 2*int(name,10)
          return length
          
      def transformation_statement (self ):
          return self.transformation.cmd_statement()    
          
      def startDataSource(self):
          pass    
          
      def startCatalog(self):
          if( self.options.transformationtype  == 'TRANSPORT' or
              self.options.transformationtype  == 'EXPORT'      ):
              if self.options.sourcedbschema   != '':
                 sourcedbuser = 'USE USER %s %s SCHEMA %s SERVERDB %s ON %s \n' % ( re.split(',',self.options.sourcedbuser)[0], re.split(',',self.options.sourcedbuser)[1], self.options.sourcedbschema, self.options.sourcedbname, self.options.sourcedbnode )
              else:
                 sourcedbuser = 'USE USER %s %s           SERVERDB %s ON %s \n' % ( re.split(',',self.options.sourcedbuser)[0], re.split(',',self.options.sourcedbuser)[1],                              self.options.sourcedbname, self.options.sourcedbnode )                 
              try:
                  self.transformation.exporter.cmd  = sourcedbuser
                  self.transformation.exporter.cmd_execute ()
                  self.transformation.exporter.user = re.split(',',self.options.sourcedbuser)[0]
                  self.transformation.exporter.schema = self.options.sourcedbschema
              except loader.LoaderError, err:
                  print 'command failed:' , err

          if( self.options.transformationtype  == 'TRANSPORT' or
              self.options.transformationtype  == 'IMPORT'      ):
              if self.options.targetdbschema   != '':
                 targetdbuser = 'USE USER %s %s SCHEMA %s SERVERDB %s ON %s \n' % ( re.split(',',self.options.targetdbuser)[0],re.split(',',self.options.targetdbuser)[1], self.options.targetdbschema, self.options.targetdbname, self.options.targetdbnode )
              else:
                 targetdbuser = 'USE USER %s %s           SERVERDB %s ON %s \n' % ( re.split(',',self.options.targetdbuser)[0],re.split(',',self.options.targetdbuser)[1],                              self.options.targetdbname, self.options.targetdbnode )                 
              try :
                  self.transformation.importer.cmd  = targetdbuser
                  self.transformation.importer.cmd_execute ()
                  self.transformation.importer.user = re.split(',',self.options.targetdbuser)[0]
                  self.transformation.importer.schema = self.options.targetdbschema
              except loader.LoaderError, err:
                  print 'command failed:' , err  
              
      def startSchema(self):
          #Source
          if( self.options.transformationtype == 'TRANSPORT' or
              self.options.transformationtype == 'EXPORT'      ):
              self.transformation.exporter.cmd = ''
              if( self.options.transformationpart == 'USER'   or
                  self.options.transformationpart == 'TABLE'):
                  #Source
                  if self.schema != '':
                     cmd = 'USE USER %s %s SCHEMA %s  SERVERDB %s ON %s \n' % ( re.split(',',self.options.sourcedbuser)[0], re.split(',',self.options.sourcedbuser)[1] , self.schema , self.options.sourcedbname, self.options.sourcedbnode)
                  else:
                     cmd = 'USE USER %s %s            SERVERDB %s ON %s \n' % ( re.split(',',self.options.sourcedbuser)[0], re.split(',',self.options.sourcedbuser)[1] ,               self.options.sourcedbname, self.options.sourcedbnode)
                  try :
                      self.transformation.exporter.cmd  = cmd
                      self.transformation.exporter.cmd_execute ()
                      self.transformation.exporter.user = re.split(',',self.options.sourcedbuser)[0]
                  except loader.LoaderError, err:
                      print 'command failed:' , err
          #Target
          if( self.options.transformationtype == 'TRANSPORT' or
              self.options.transformationtype == 'IMPORT'      ):
              self.transformation.importer.cmd = ''
              if( self.options.transformationpart == 'USER'   or
                  self.options.transformationpart == 'TABLE'):
                  if self.schema != '':
                      cmd = 'USE USER %s %s SCHEMA %s SERVERDB %s ON %s \n' % ( re.split(',',self.options.targetdbuser)[0], re.split(',',self.options.targetdbuser)[1], self.schema , self.options.targetdbname, self.options.targetdbnode )
                  else:
                      cmd = 'USE USER %s %s           SERVERDB %s ON %s \n' % ( re.split(',',self.options.targetdbuser)[0], re.split(',',self.options.targetdbuser)[1],               self.options.targetdbname, self.options.targetdbnode )                                            
                  try :
                      self.transformation.importer.cmd  = cmd
                      self.transformation.importer.cmd_execute ()
                      self.transformation.importer.user = re.split(',',self.options.targetdbuser)[0]
                  except loader.LoaderError, err:
                      print 'command failed:' , err
          time.sleep(1)          
      
      def startTable(self):
          self.statement         = ''
          self.indexStatement    = ''
          self.indexStatements   = []
          self.tablecount        = self.tablecount +1
          self.columncount       = 0
          self.statement_column  = ''
          self.field_startpos    = 1
          self.field_endpos      = 1
          self.column            = ''
          self.column_field      = ''
          self.column_field_long = ''
          self.column_long       = []
          if self.transformation.exporter != None:
             self.transformation.exporter.cmd = ''
          if self.transformation.importer != None:
             self.transformation.importer.cmd = ''
          
      def startColumn(self): 
          self.columncount       = self.columncount + 1   
          self.columnEncoding    = ''            
          
      def endDataSource(self):
          if self.options.transformationpart  == 'DB':
             mutex.acquire()                                           #acquire
             self.cmd_verify()                                         #release
          self.transformation.release()
           
      def endCatalog(self):
          if self.options.transformationpart  == 'DB'   :
             mutex.acquire()                                           #acquire
             self.transformation_statement()
             input  = None
             #Format
             if(self.options.streamformat        == 'XML'     or
                self.options.streamformat        == 'DDL'     or
                self.options.streamformat        == 'PAGES'   or
                self.options.streamformat        == 'RECORDS'  ):
                #Type
                if self.options.transformationtype   == 'TRANSPORT':
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Transport DB %s ? [y]/[n] : ' % ( self.sapdb ) )
                          if self.options.mode == 'PARAM':
                             print             'Transport DB %s\n' % ( self.sapdb )
                             input  = "y"
                          if input == "y":
                             self.transformation.cmd_transport_catalog_package()
                          if input == "n":
                             if mutex.locked() == True:
                                mutex.release()
                if(self.options.transformationtype   == 'EXPORT'  and
                    self.options.streammedium        == 'FILE'     ):
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Export    DB %s ? [y]/[n] : ' % ( self.sapdb ))
                          if self.options.mode == 'PARAM':
                             print             'Export    DB %s\n' % ( self.sapdb )
                             input = "y"
                          if input == "y":
                             self.transformation.cmd_export_catalog_package()
                          if input == "n":
                             if mutex.locked() == True:
                                mutex.release()
                if(self.options.transformationtype   == 'IMPORT' and
                    self.options.streammedium        == 'FILE'    ):
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Import    DB %s ? [y]/[n] : ' % (self.sapdb ))
                          if self.options.mode == 'PARAM':
                             print             'Import    DB %s\n' % (self.sapdb )
                             input = "y"
                          if input == "y":
                             self.transformation.cmd_import_catalog_package()
                          if input == "n":
                             if mutex.locked() == True:
                                mutex.release()                        #release
             self.user = '' 
             
      def endSchema(self):
          if(self.options.transformationpart  == 'USER'    and
            self.user                         <> 'DBA'       ) :
            mutex.acquire()                                           #acquire
            self.transformation_statement()
            input  = None
            #Format
            if( self.options.streamformat        == 'XML'     or
                self.options.streamformat        == 'DDL'     or
                self.options.streamformat        == 'PAGES'   or
                self.options.streamformat        == 'RECORDS'  ):
                #Type
                if self.options.transformationtype    == 'TRANSPORT':
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Transport User %s ? [y]/[n] : ' % ( self.user ) )
                          if self.options.mode == 'PARAM':
                             print             '\nTransport User %s' % ( self.user )
                             input = "y"
                          if input == "y":
                             self.transformation.cmd_transport_schema_package()
                          if input == "n":
                             if mutex.locked() == True:
                                mutex.release()
                if( self.options.transformationtype    == 'EXPORT'  and
                    self.options.streammedium          == 'FILE'     ):
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Export    User   %s ? [y]/[n] :  ' % ( self.user ))
                          if self.options.mode == 'PARAM':
                             print             '\nExport    User   %s' % ( self.user )
                             input = "y"
                          if input == "y":
                             self.transformation.cmd_export_schema_package()
                          if input == "n":
                             if mutex.locked() == True:
                                mutex.release()
                if( self.options.transformationtype     == 'IMPORT' and
                    self.options.streammedium           == 'FILE'    ):
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Import    User   %s ? [y]/[n] :  ' % (self.user ))
                          if self.options.mode == 'PARAM':
                             print             '\nImport    User   %s' % (self.user )
                             input = "y"
                          if input == "y":
                             self.transformation.cmd_import_schema_package()
                          if input == "n":
                             if mutex.locked() == True :               #release
                                mutex.release()
            self.user = '' 
            
      def endTable(self):
          if self.options.transformationpart   == 'TABLE' :
             mutex.acquire()                                          #acquire
             if self.options.streamformat         != 'SQL':
                self.transformation_statement()
             input  = None
             #Format
             if(self.options.streamformat         == 'XML'       or
                self.options.streamformat         == 'DDL'       or
                self.options.streamformat         == 'PAGES'     or
                self.options.streamformat         == 'RECORDS'   or
                self.options.streamformat[0:9]    == 'FORMATTED' or
                self.options.streamformat         == 'COMPRESSED'or
                self.options.streamformat         == 'SQL'          ):
                #Type
                if self.options.transformationtype    == 'TRANSPORT':
                   while input not in ["y","n"]:
                         if self.options.mode == 'INTER':
                            input = raw_input('Transport Table %s ? [y]/[n] : ' % ( self.table ) )
                         if self.options.mode == 'PARAM':
                            print             'Transport Table %s\n' % ( self.table )
                            input  = "y"
                         if input == "y":
                            self.transformation.cmd_transport_table_package()
                         if input == "n":
                            if mutex.locked() == True:
                               mutex.release()
                if(self.options.transformationtype    == 'EXPORT'  and
                    self.options.streammedium         == 'FILE'     ):
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Export    Table %s ? [y]/[n] : ' % ( self.table ))
                          if self.options.mode == 'PARAM':
                             print             'Export    Table %s :\n' % ( self.table )
                             input  = "y"
                          if input == "y":
                             self.transformation.cmd_export_table_package()
                          if input == "n":
                             if mutex.locked() == True:
                                mutex.release()
                if(self.options.transformationtype     == 'IMPORT' and
                    self.options.streammedium          == 'FILE'    ):
                    while input not in ["y","n"]:
                          if self.options.mode == 'INTER':
                             input = raw_input('Import    Table %s ? [y]/[n] : ' % (self.table ))
                          if self.options.mode == 'PARAM':
                             print             'Import    Table %s\n' % (self.table )
                             input  = "y"
                          if input == "y":
                             self.transformation.cmd_import_table_package()
                          if input == "n":
                             if mutex.locked() == True:
                                mutex.release()                       #release
                              
      def endIndex(self):
          self.indexStatement = self.indexStatement   + ') '
          self.indexStatement = string.replace(self.indexStatement,',)',')\n')
          self.indexStatements.append(self.indexStatement)
          if(self.options.transformationpart   == 'TABLE' and self.options.streamformat != 'PAGES' and ( self.options.streamformat_target == 'PAGE' or self.options.streamformat_target == 'DTL' ) and  self.column_field_long == '' and self.transformation.importer != None) :
             self.transformation.importer.cmd  = 'DROP INDEX   \"%s\" ON \"%s\".\"%s\" \n' % (self.index, self.user, self.table)
             self.transformation.importer.cmd_execute()
             self.transformation.importer.cmd  =  ''  
             
      def endTrigger(self):     
          if self.options.transformationpart   == 'TABLE' and self.options.streamformat_target == 'PAGE' and self.transformation.importer != None:
             self.transformation.importer.cmd  = 'DROP TRIGGER \"%s\" OF \"%s\".\"%s\" \n' % (self.trigger, self.user, self.table)
             self.transformation.importer.cmd_execute()
             self.transformation.importer.cmd  =  ''          

      def endColumn(self):
          column_field = ''
          if self.options.streamformat[0:9]   == 'FORMATTED' :
             if self.keycolumn == None :
                column_field      = "%+16s%-15s  %3i - %3i CHAR " % (' ','\"%s\"' % self.column,self.field_startpos, self.field_endpos)
                self.column_field = self.column_field  + column_field 
                if self.columnIsNullable == '0':
                   self.column_field = self.column_field + "\n"
                if self.columnIsNullable != '0':
                   self.column_field = self.column_field + "DEFAULT NULL \n"
             self.field_startpos = self.field_endpos + 1
          if self.options.streamformat        == 'COMPRESSED':   
             if self.keycolumn == None :
                column_field    = "%+16s%-15s  %3i  CHAR "      % (' ','\"%s\"' % self.column, self.columncount )
                self.column_field = self.column_field  + column_field 
                if self.columnIsNullable == '0':
                   self.column_field = self.column_field + "\n"
                if self.columnIsNullable != '0':
                   self.column_field = self.column_field + "DEFAULT NULL \n"

      def endKeyColumns(self):
          self.keycolumn = None                   
          
      def columnTypeBool (self, name):
          if name == 'sdbm:BOOLEAN':
             fieldBoolLength = '1'
             if self.columnIsNullable == '0':
                self.field_endpos      = self.field_startpos + self.int('%s' %self.columnInitialValue) -1
             if self.columnIsNullable == '1':   
                if len(self.fieldNullValue) >  len('%s' %self.columnInitialValue) :
                   self.field_endpos   = self.field_startpos + len(self.fieldNullValue) -1
                if len(self.fieldNullValue) <= len('%s' %self.columnInitialValue) :
                   self.field_endpos   = self.field_startpos + self.int('%s' %self.columnInitialValue) -1
      
      def columnTypeCharacter(self, name):
          if(name == 'sdbm:CHAR'    or 
             name == 'sdbm:VARCHAR'   ):
             if self.columnIsNullable == '0': 
                self.field_endpos      = self.field_startpos + self.int('%s' %self.columnLength) -1
             if self.columnIsNullable == '1':   
                if len(self.fieldNullValue)  > self.int('%s' % self.columnLength):
                   self.field_endpos   = self.field_startpos + len(self.fieldNullValue)-1
                if len(self.fieldNullValue) <= self.int('%s' %self.columnLength):   
                   self.field_endpos   = self.field_startpos + self.int('%s' %self.columnLength) -1
          if name == 'sdbm:BINARY':  
             self.columnEncoding = 'HEX'
             if self.columnIsNullable == '0': 
                self.field_endpos      = self.field_startpos + HEXLENGTH*self.int('%s' %self.columnLength) -1
             if self.columnIsNullable == '1':   
                if len(self.fieldNullValue)  > self.int('%s' % self.columnLength):
                   self.field_endpos   = self.field_startpos + HEXLENGTH*len(self.fieldNullValue)-1
                if len(self.fieldNullValue) <= self.int('%s' %self.columnLength):   
                   self.field_endpos   = self.field_startpos + HEXLENGTH*self.int('%s' %self.columnLength) -1       
      
      def columnTypeDateTime(self, name ):
          if name == 'sdbm:DATE': 
             fieldDateLength    = 0
             if self.fieldDateValue == 'INTERNAL':
                fieldDateLength = DATEINTERNAL
             if self.fieldDateValue != 'INTERNAL':
                fieldDateLength = DATE
             if self.columnIsNullable == '0': 
                self.field_endpos      = self.field_startpos + self.int( '%i' % fieldDateLength) -1 
             if self.columnIsNullable == '1':   
                if len(self.fieldNullValue) >  self.int( '%i' % fieldDateLength): 
                   self.field_endpos   = self.field_startpos + len(self.fieldNullValue) -1
                if len(self.fieldNullValue) <= self.int( '%i' % fieldDateLength):
                   self.field_endpos   = self.field_startpos + self.int( '%i' % fieldDateLength) -1
          if name == 'sdbm:TIME': 
             fieldTimeLength    = 0
             if self.fieldTimeValue == 'INTERNAL':
                fieldTimeLength = TIMEINTERNAL
             if self.fieldTimeValue != 'INTERNAL':
                fieldTimeLength = TIME
             if self.columnIsNullable == '0': 
                self.field_endpos      = self.field_startpos + self.int( '%i' % fieldTimeLength) -1
             if self.columnIsNullable == '1':   
                if len(self.fieldNullValue) >  len(self.fieldTimeValue) : 
                   self.field_endpos   = self.field_startpos + len(self.fieldNullValue) -1
                if len(self.fieldNullValue) <= len(self.fieldTimeValue) :
                   self.field_endpos   = self.field_startpos + self.int(fieldTimeLength) -1
          if name == 'sdbm:TIMESTAMP':
             fieldTimeStampLength    = 0
             if self.fieldTimeStampValue == 'INTERNAL':
                fieldTimeStampLength = TIMESTAMPINTERNAL
             if self.fieldTimeStampValue != 'INTERNAL':
                fieldTimeStampLength = TIMESTAMP
             if self.columnIsNullable == '0': 
                self.field_endpos      = self.field_startpos + self.int( '%i' % fieldTimeStampLength) -1
             if self.columnIsNullable == '1':   
                if len(self.fieldNullValue) >  self.int( '%i' % fieldTimeStampLength) :
                   self.field_endpos   = self.field_startpos + len(self.fieldNullValue) -1 
                if len(self.fieldNullValue) <= self.int( '%i' % fieldTimeStampLength) :
                   self.field_endpos   = self.field_startpos + self.int( '%i' % fieldTimeStampLength) -1
      
      def columnTypeNumber(self, name ):    
          fieldIntegerLength = 0
          signSymbolLength  = SIGN
          ThsdCharNum       = 0
          FracPart          = 0
          IntPart           = 0
          if name == 'sdbm:SMALLINT':
             fieldIntegerLength = SMALLINT
          if(name == 'sdbm:INTEGER' or
             name == 'sdbm:DECIMAL'):
             fieldIntegerLength = INTEGER
          if name == 'sdbm:FIXED'  :
             fieldIntegerLength = int(string.split(self.columnLength,',')[0])
             FracPart           = int(string.split(self.columnLength,',')[1])
          if name == 'sdbm:FLOAT' :
             fieldIntegerLength = int(self.columnLength)
             field_length       = fieldIntegerLength + FLOATMASK
             if self.columnIsNullable == '0':
                self.field_endpos      = self.field_startpos + self.int('%s' % field_length) -1
             if self.columnIsNullable == '1': 
                if len(self.fieldNullValue) >  self.int('%s' % field_length): 
                   self.field_endpos   = self.field_startpos + len(self.fieldNullValue) - 1
                if len(self.fieldNullValue) <= self.int('%s' % field_length):
                   self.field_endpos   = self.field_startpos + self.int('%s' % field_length) -1
          if name != 'sdbm:FLOAT':
             IntPart          = fieldIntegerLength - FracPart
             field_length     = 0  
             if string.split(self.fieldDecimalValue,'/')[1] <> '':
                ThsdCharNum = ( IntPart - 1) / 3
             field_length = IntPart + FracPart+ signSymbolLength + ThsdCharNum
             if   FracPart == 0:
                  field_length = field_length - 1
             elif IntPart  == 0: 
                  field_length = field_length - ( ThsdCharNum - 1 )
             if self.columnIsNullable == '0':
                self.field_endpos      = self.field_startpos + self.int('%s' % field_length) -1
             if self.columnIsNullable == '1': 
                if len(self.fieldNullValue) >  self.int('%s' % field_length): 
                   self.field_endpos   = self.field_startpos + len(self.fieldNullValue) -1
                if len(self.fieldNullValue) <= self.int('%s' % field_length):
                   self.field_endpos   = self.field_startpos + self.int('%s' % field_length) -1
      def columnTypeLong (self, name ):#
          if name == 'sdbm:LONG'      and self.options.streamformat   != 'RECORDS' :
             if self.column != ''     and self.options.streammedium   == 'FILE'    :
                if  self.options.streamlob == 'COLUMN':
                    self.column_field_long = self.column_field_long + "%+15s LONGSTREAM FILE \"%s\" '%s' %s \n" % ( ' ', self.column, os.path.join(self.options.sourceindepdatapath, '%s_%s.DATA%s' % (self.table,self.column,'')),self.options.streamcharencoding)
                    self.field_endpos      = self.field_startpos + LONGPOSITION -1
                if  self.options.streamlob == 'COLUMNVALUE':
                    self.column_field_long = self.column_field_long + "%+15s LONGSTREAM FILE \"%s\" '%s' %s \n" % ( ' ', self.column, os.path.join(self.options.sourceindepdatapath, '%s_%s.DATA%s' % (self.table,self.column,FIELDLOBMASK)),self.options.streamcharencoding)
                    self.field_endpos      = self.field_startpos + self.len(os.path.join(self.options.sourceindepdatapath,  '%s_%s.DATA%s' % (self.table,self.column, FIELDLOBMASK))) -1 
             if self.column != ''       and ( self.options.streammedium == 'PIPE' or  self.options.streammedium  == 'PIPE2FILE' ) :
                self.column_field_long = self.column_field_long + "%+15s LONGSTREAM FILE %s '%s' %s \n" % (' ', self.column, '\\\\.\\PIPE\\%s_%s.DATA' % (self.table,self.column),self.options.streamcharencoding)
                self.column_long.append(self.column)
          if name == 'sdbm:LONG_BYTE' and self.options.streamformat   != 'RECORDS' :
             if self.column != ''     and self.options.streammedium   == 'FILE'    :
                if  self.options.streamlob == 'COLUMN':
                    self.column_field_long  = self.column_field_long + "%+15s LONGSTREAM FILE \"%s\" '%s' \n" % ( ' ',self.column, os.path.join(self.options.sourceindepdatapath,   '%s_%s.DATA%s' % (self.table,self.column, '')))
                    self.field_endpos       = self.field_startpos + LONGPOSITION -1 
                if  self.options.streamlob == 'COLUMNVALUE':
                    self.column_field_long  = self.column_field_long + "%+15s LONGSTREAM FILE \"%s\" '%s' \n" % ( ' ',self.column, os.path.join(self.options.sourceindepdatapath,   '%s_%s.DATA%s' % (self.table,self.column, FIELDLOBMASK)))
                    self.field_endpos       = self.field_startpos + self.len(os.path.join(self.options.sourceindepdatapath,   '%s_%s.DATA%s' % (self.table,self.column,FIELDLOBMASK))) -1
             if self.column != ''       and ( self.options.streammedium == 'PIPE' or  self.options.streammedium  == 'PIPE2FILE' ) :
                self.column_field_long = self.column_field_long + "%+15s LONGSTREAM FILE \"%s\" '%s' \n" % ( ' ',self.column, '\\\\.\\PIPE\\%s_%s.DATA' % (self.table,self.column))
                self.column_long.append(self.column)
                   
class Transformation :
      def __init__( self, options = None, package = None ) :
          self.options                = options
          self.package                = package
          self.transformation         = None
          self.exportstatement        = None
          self.importstatement        = None
          self.exporter           = None
          self.importer           = None
          # stream loader source
          if( options.transformationtype == 'TRANSPORT' or
              options.transformationtype == 'EXPORT'      ):
              self.exporter = StreamTransformator( self.options, self.package, 'EXPORT' )
              try:
                 if options.mode == 'DEBUG':
                    input('\nStarted   Exporter')
              except:
                 pass                                              
          # stream loader target
          if( options.transformationtype == 'TRANSPORT' or
              options.transformationtype == 'IMPORT'      ):
              self.importer = StreamTransformator( self.options, self.package, 'IMPORT' )
              try:
                 if options.mode == 'DEBUG':
                    input('\nStarted   Importer')
              except:
                 pass  
      
      def release(self):
          if mutex.locked() == True:
             mutex.release()
          if self.exporter != None:
             self.exporter.release()
          if self.importer != None:
             self.importer.release()
      
      def cmd_release_execute_package(self):
          # trigger package and release StreamTransformator
          release        = 'FALSE'
          while release == 'TRUE':
                try:
                    cmd  = "SELECT COUNT(*) FROM SYSDBA.TRANSFORMATIONMODEL WHERE RESTART = FALSE "
                    self.sourcesession.cmd = cmd
                    self.sourcesession.cmd_execute()
                    if self.sourcesession.result[0] == 1 :
                       print '\nStop   Package Execution'
                       self.exporter.loadersession.release()  # python->cancelbyte
                       self.importer.loadersession.release()  # python->cancelbyte
                       release = 'TRUE'
                except sql.SQLError,  err :
                    print 'sql command failed:', err
                except loader.LoaderError, err :
                    print 'loader command failed:', err
                except loader.CommunicationError, err :
                    print 'loader communication failed:', err
                    
      def cmd_statement(self):
          #transformationstreamencoding
          if self.options.streamcharencoding == 'DYNAMIC':
             self.options.streamcharencoding  = ''
          #transformationstreams
          datastream_format = self.options.streamformat
          datastream_field  = ''
          if(datastream_format  == 'COMPRESSED' or datastream_format  == 'FORMATTED' ) and self.options.streamformat_target == 'PAGE' and self.package.column_field_long <> '' :
             datastream_format   = 'RECORDS'
          if(datastream_format  == 'PAGES'   or
             datastream_format  == 'RECORDS' or
             datastream_format  == 'DDL'     or
             datastream_format  == 'XML'   ):
             configurationstream_name_mask = "'%s.CONFIGURATION'" + "%-0s"
             catalogstream_name_mask       = "'%s.CATALOG'"       + "%-6s"
             catalogstream_encoding_mask   = "%-6s %s"
             datastream_name_mask          = "'%s.DATA'"          + "%-9s"
             datastream_encoding_mask      = "%-2s %s"
             packagestream_name_mask       = "'%s.PACKAGE'"       + "%-6s"
             packagestream_format_mask     = "%-0s" + "COMPRESSED"
             packagestream_encoding_mask   = "%-0s %s"
          if(datastream_format  == 'COMPRESSED' or
             datastream_format  == 'FORMATTED'    ) :
             configurationstream_name_mask = "'%s.CONFIGURATION'" + "%-0s"
             catalogstream_name_mask       = "'%s.CATALOG'"       + "%-1s"
             catalogstream_encoding_mask   = "%-6s %s"
             datastream_name_mask          = "'%s.DATA'"          + "%-4s"
             packagestream_name_mask       = "'%s.PACKAGE'"       + "%-1s"
             datastream_encoding_mask      = "%-0s %s"
             packagestream_format_mask     = "%-0s" + "DTL"
             packagestream_encoding_mask   = "%-0s %s"
          
          #transformationstream_names
          export_configurationstream_name    = os.path.join(self.options.sourceindepdatapath)
          import_configurationstream_name    = os.path.join(self.options.targetindepdatapath)
          export_packagestream_name          = os.path.join(self.options.sourceindepdatapath)
          import_packagestream_name          = os.path.join(self.options.targetindepdatapath)
          if self.options.streammedium      =='FILE':
             datastream_name                 = os.path.join(self.options.sourceindepdatapath)
             catalogstream_name              = os.path.join(self.options.sourceindepdatapath)
          if self.options.streammedium      =='PIPE' or self.options.streammedium== 'TAPE':  
             datastream_name                 = os.path.join('\\\\','.','PIPE')
             catalogstream_name              = os.path.join('\\\\','.','PIPE')
          if self.options.transformationpart == 'DB'   :
             export_configurationstream_name = os.path.join(export_configurationstream_name,self.options.sourcedbname)
             import_configurationstream_name = os.path.join(import_configurationstream_name,self.options.targetdbname)
             if self.options.streammedium   == 'PIPE':
                export_catalogstream_name    = os.path.join(catalogstream_name       ,self.options.sourcedbname)
                import_catalogstream_name    = os.path.join(catalogstream_name       ,self.options.sourcedbname)
             if self.options.streammedium   != 'PIPE': 
                export_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.sapdb)
                import_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.sapdb)        
             datastream_name                 = os.path.join(datastream_name          ,self.package.sapdb)
             export_packagestream_name       = os.path.join(export_packagestream_name,self.options.sourcedbname)
             import_packagestream_name       = os.path.join(import_packagestream_name,self.options.targetdbname)
          if self.options.transformationpart == 'USER' :
             export_configurationstream_name = os.path.join(export_configurationstream_name,self.package.user)
             import_configurationstream_name = os.path.join(import_configurationstream_name,self.package.user)
             if self.options.streammedium   == 'PIPE':
                export_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.user)
                import_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.user)
             if self.options.streammedium   != 'PIPE': 
                export_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.user)
                import_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.user)     
             datastream_name                 = os.path.join(datastream_name          ,self.package.user)
             export_packagestream_name       = os.path.join(export_packagestream_name,self.package.user)
             import_packagestream_name       = os.path.join(import_packagestream_name,self.package.user)
          if self.options.transformationpart == 'TABLE':
             export_configurationstream_name = os.path.join(export_configurationstream_name,self.package.table)
             import_configurationstream_name = os.path.join(import_configurationstream_name,self.package.table)
             if self.options.streammedium   == 'PIPE':
                export_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.table)
                import_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.table)
             if self.options.streammedium   != 'PIPE': 
                export_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.table)
                import_catalogstream_name    = os.path.join(catalogstream_name       ,self.package.table)  
             datastream_name                 = os.path.join(datastream_name          ,self.package.table)
             export_packagestream_name       = os.path.join(export_packagestream_name,self.package.table)
             import_packagestream_name       = os.path.join(import_packagestream_name,self.package.table)

          #configurationstream
          export_configurationstream_name = configurationstream_name_mask %( export_configurationstream_name,'' )
          import_configurationstream_name = configurationstream_name_mask %( import_configurationstream_name,'' )
          configurationstream_medium      = 'FILE'
          configurationstream_format      = 'COMPRESSED'
          configurationstream_encoding    = ''
          
          #catalogstream
          export_catalogstream_name      = catalogstream_name_mask %( export_catalogstream_name, '' )
          import_catalogstream_name      = catalogstream_name_mask %( import_catalogstream_name, '' )
          catalogstream_medium           = self.options.streammedium     
          if catalogstream_medium       == 'TAPE' and catalogstream_direction    == 'INSTREAM':
             catalogstream_medium        = catalogstream_medium + ' EBID ' + self.importer.ddlpipe_ebid
          if self.options.streamformat  == 'DDL'  or  self.options.streamformat  == 'XML':
             catalogstream_format        = self.options.streamformat
          else:
             catalogstream_format        = 'DDL'
          catalogstream_encoding         = catalogstream_encoding_mask %( '', '' )
          
          #datastream
          datastream_name                = datastream_name_mask % (datastream_name, '')
          datastream_medium              = self.options.streammedium
          if datastream_medium          == 'TAPE' and datastream_direction    == 'INSTREAM':
             datastream_medium           = datastream_medium + 'EBID' + self.importer.mappipe_ebid
          datastream_format              = self.options.streamformat
          if(datastream_format          == 'COMPRESSED' or datastream_format  == 'FORMATTED' ) and self.package.column_field_long <> '' and ( self.options.streammedium == 'PIPE' or self.options.streamformat_target == 'PAGE' ):
             datastream_format           = 'RECORDS'
          if datastream_format          == 'PAGES':
             datastream_encoding         = ''
          if datastream_format          == 'RECORDS' and self.options.streamcharencoding == 'UTF8':
             datastream_encoding         = ''
          if datastream_format          == 'RECORDS' and self.options.streamcharencoding != 'UTF8':
             datastream_encoding         = datastream_encoding_mask  % ('', self.options.streamcharencoding )   
          if datastream_format          == 'COMPRESSED' or datastream_format == 'FORMATTED' or datastream_format  == 'DDL' or datastream_format  == 'XML' :   
             datastream_encoding         = datastream_encoding_mask  % ('', self.options.streamcharencoding )
          if datastream_format          == 'COMPRESSED' or datastream_format == 'FORMATTED' :
             if datastream_format       == 'COMPRESSED' :
                datastream_field         = datastream_field + "%-15s SEPARATOR  '%s' \n" % ( '', re.split('/',self.package.fieldCompressed)[1] )
                datastream_field         = datastream_field + "%-15s DELIMITER  '%s' \n" % ( '', re.split('/',self.package.fieldCompressed)[2] )
             datastream_field            = datastream_field + "%-15s NULL       '%s' \n" % ( '', self.package.fieldNullValue  )
             datastream_field            = datastream_field + "%-15s DECIMAL    '%s' \n" % ( '', self.package.fieldDecimalValue) 
             datastream_field            = datastream_field + "%-15s BOOLEAN    '%s' \n" % ( '', self.package.fieldBoolValue) 
             datastream_field            = datastream_field + "%-15s TIMESTAMP   %s  \n" % ( '', self.package.fieldTimeStampValue)
             datastream_field            = datastream_field + "%-15s DATE        %s  \n" % ( '', self.package.fieldDateValue)
             datastream_field            = datastream_field + "%-15s TIME        %s  \n" % ( '', self.package.fieldTimeValue)     
          
          #packagestream
          export_packagestream_name      = packagestream_name_mask %( export_packagestream_name, '') 
          import_packagestream_name      = packagestream_name_mask %( import_packagestream_name, '') 
          packagestream_medium           = 'FILE' 
          packagestream_format           = packagestream_format_mask   %('') 
          packagestream_encoding         = packagestream_encoding_mask %('', '' ) 
          
          #transformationstreams
          export_configurationstream     = "  CONFIGURATION %s %s %s %s %s \n"   % ( 'INSTREAM ', configurationstream_medium, export_configurationstream_name, configurationstream_format, configurationstream_encoding)        
          export_catalogstream           = "  CATALOG       %s %s %s %s %s \n"   % ( 'OUTSTREAM', catalogstream_medium      , export_catalogstream_name      , catalogstream_format      , catalogstream_encoding)
          export_datastream              = "  DATA          %s %s %s %s %s \n"   % ( 'OUTSTREAM', datastream_medium         , datastream_name                , datastream_format         , datastream_encoding )
          import_configurationstream     = "  CONFIGURATION %s %s %s %s %s \n"   % ( 'INSTREAM ', configurationstream_medium, import_configurationstream_name, configurationstream_format, configurationstream_encoding)        
          import_catalogstream           = "  CATALOG       %s %s %s %s %s \n"   % ( 'INSTREAM ', catalogstream_medium      , import_catalogstream_name      , catalogstream_format      , catalogstream_encoding)
          import_datastream              = "  DATA          %s %s %s %s %s \n"   % ( 'INSTREAM ', datastream_medium         , datastream_name                , datastream_format         , datastream_encoding )
          
          export_datastream              = export_datastream + datastream_field
          import_datastream              = import_datastream + datastream_field

          if datastream_format          == 'COMPRESSED' or datastream_format  == 'FORMATTED':
             export_datastream           = export_datastream + '%-15s START  %-4s 1   MAXROWS    \n'  %('','')
             import_datastream           = import_datastream + '%-15s START  %-4s 1   MAXRECORDS \n'  %('','') 
             export_datastream           = export_datastream + ''
             import_datastream           = import_datastream + self.package.column_field
             if self.options.streamlob == 'COLUMN':
                export_datastream        = export_datastream + self.package.column_field_long
                import_datastream        = import_datastream + self.package.column_field_long
             if self.options.streamlob == 'COLUMNVALUE': 
                export_datastream        = export_datastream + self.package.column_field_long  
                import_datastream        = import_datastream + ''
          if datastream_format          == 'RECORDS' and self.options.transformationpart == 'TABLE' : 
             export_datastream           = export_datastream + '%-15s START  %-4s 1   MAXROWS    \n' %('','')
             import_datastream           = import_datastream + '%-15s START  %-4s 1   MAXRECORDS \n' %('','') 
          export_packagestream           = "  PACKAGE       %s %s %s %s %s  \n"   % ('OUTSTREAM', packagestream_medium      , export_packagestream_name      , packagestream_format      , packagestream_encoding )      
          import_packagestream           = "  PACKAGE       %s %s %s %s %s  \n"   % ('OUTSTREAM', packagestream_medium      , import_packagestream_name      , packagestream_format      , packagestream_encoding )      
          
          if((self.options.transformationpart == 'DB'    or  
              self.options.transformationpart == 'USER') and
              self.options.streamformat_target== 'DTL' ):
              export_datastream = ''
              import_datastream = ''
          if(self.options.transformationpart  == 'TABLE'   and
             self.options.streamformat        != 'PAGES'   and
             self.options.streamformat        != 'RECORDS' and 
             self.options.streamformat_target == 'ROW'   ):
             import_datastream           = import_datastream    + '%-15s ROW USAGE \n' %(' ')
             export_packagestream        = export_packagestream + '%-15s ROW USAGE \n'   %(' ')
             import_packagestream        = import_packagestream + '%-15s ROW USAGE \n'   %(' ')
          if(self.options.transformationpart  == 'TABLE'   and
             self.options.streamformat        != 'PAGES'   and   
             self.options.streamformat        != 'RECORDS' and  
             self.options.streamformat_target == 'PAGE'    and 
             self.package.column_field_long   == ''     ):
             import_datastream           = import_datastream    + '%-15s PAGE WITH 25 ROWS USAGE \n' %(' ')
             export_packagestream        = export_packagestream + '%-15s PAGE WITH 25 ROWS USAGE \n' %(' ')
             import_packagestream        = import_packagestream + '%-15s PAGE WITH 25 ROWS USAGE \n' %(' ')
          if(self.options.transformationpart  == 'TABLE' and  
             self.options.streamformat_target == 'DTL'   and
            (datastream_format                == 'COMPRESSED' or 
             datastream_format                == 'FORMATTED') ):
             if self.package.column_field_long != '' :
                import_datastream           = import_datastream    + '%-15s ROW USAGE \n' %(' ')
                export_packagestream        = export_packagestream + '%-15s ROW USAGE \n'   %(' ')
                import_packagestream        = import_packagestream + '%-15s ROW USAGE \n'   %(' ')
             if self.package.column_field_long == '' :
                import_datastream           = import_datastream    + '%-15s PAGE WITH 25 ROWS USAGE \n' %(' ')
                export_packagestream        = export_packagestream + '%-15s PAGE WITH 25 ROWS USAGE \n' %(' ')
                import_packagestream        = import_packagestream + '%-15s PAGE WITH 25 ROWS USAGE \n' %(' ')
          if(self.options.transformationpart  == 'TABLE' ):
             packagestream_format           = 'COMPRESSED'
             import_packagestream           = "  PACKAGE       %s %s %s %s %s  \n"   % ('OUTSTREAM', packagestream_medium      , import_packagestream_name      , packagestream_format      , packagestream_encoding )      
             
          #transformation
          
          #transformationtype export 
          export_stmt  = 'EXPORT %s ' % self.options.transformationpart
          #transformationpackage
          if(self.options.transformationpart  == 'DB' or 
             self.options.transformationpart  == 'USER'):
             export_stmt = export_stmt  + '\n'
          if(self.options.transformationpart  == 'TABLE'):
             export_stmt = export_stmt + '"%s" ' % self.package.table
             if( self.options.streamformat_target== 'DTL' ):
                 export_stmt = export_stmt + " FOR IMPORT TABLE "
             export_stmt = export_stmt + '\n'    
          #configurationstream   
          if(datastream_format  <> 'DDL' and
            (self.options.transformationpart  == 'DB' or 
             self.options.transformationpart  == 'USER'  )):    
             export_stmt = export_stmt + export_configurationstream 
          #catalogstream 
          export_stmt    = export_stmt + export_catalogstream
          #datastream 
          if(datastream_format == 'PAGES'      or   
             datastream_format == 'RECORDS'    or 
             datastream_format == 'COMPRESSED' or 
             datastream_format == 'FORMATTED'  ):
             export_stmt = export_stmt + export_datastream  
          #packagestream
          if datastream_format     <> 'DDL' :   
             export_stmt = export_stmt + export_packagestream 
             
          #transformationtype import
          import_stmt  = 'IMPORT %s ' % self.options.transformationpart
          #transformationpackage
          if(self.options.transformationpart  == 'DB' or 
             self.options.transformationpart  == 'USER'):
             import_stmt = import_stmt  + '\n'
          if(self.options.transformationpart  == 'TABLE'):
             import_stmt = import_stmt + '"%s" \n' % self.package.table
          #configurationstream   
          if(datastream_format  <> 'DDL' and
            (self.options.transformationpart  == 'DB' or 
             self.options.transformationpart  == 'USER'  )):    
             import_stmt = import_stmt + import_configurationstream 
          #catalogstream 
          import_stmt    = import_stmt + import_catalogstream
          #datastream 
          if(datastream_format == 'PAGES'      or   
             datastream_format == 'RECORDS'    or 
             datastream_format == 'COMPRESSED' or 
             datastream_format == 'FORMATTED'  ):
             import_stmt = import_stmt + import_datastream  
          #packagestream
          if  datastream_format    <> 'DDL':
              import_stmt = import_stmt + import_packagestream    
              
          if self.options.transformationtype == 'EXPORT':
             self.exporter.catalogstream_name = export_catalogstream_name
             self.exporter.datastream_name    = datastream_name
             self.exporter.packagestream_name = export_packagestream_name
             self.exporter.cmd = export_stmt
          if self.options.transformationtype == 'IMPORT':
             self.importer.catalogstream_name = import_catalogstream_name
             self.importer.datastream_name    = datastream_name
             self.importer.packagestream_name = import_packagestream_name
             self.importer.cmd = import_stmt
          if self.options.transformationtype == 'TRANSPORT': 
             self.exporter.catalogstream_name = export_catalogstream_name
             self.exporter.datastream_name    = datastream_name
             self.exporter.packagestream_name = export_packagestream_name
             self.exporter.cmd = export_stmt
             self.importer.catalogstream_name = import_catalogstream_name
             self.importer.datastream_name    = datastream_name
             self.importer.packagestream_name = import_packagestream_name
             self.importer.cmd = import_stmt

          self.exportstatement  = export_stmt
          self.importstatement  = import_stmt
          
          if VERBOSE == 'ON':
             self.cmd_monitor_statement()
          
      def cmd_monitor_statement(self): 
          export_stmt = self.exportstatement 
          import_stmt = self.importstatement
          export_stmt = string.replace(export_stmt,'\\\\.\\PIPE\\','')
          export_stmt = string.replace(export_stmt,self.options.sourceindepdatapath+os.sep, '')
          import_stmt = string.replace(import_stmt,'\\\\.\\PIPE\\','')
          import_stmt = string.replace(import_stmt,self.options.sourceindepdatapath+os.sep, '')
          import_stmt = string.replace(import_stmt,self.options.targetindepdatapath+os.sep, '')
                    
          if self.options.transformationtype == 'TRANSPORT':
             print '\n%s;'  % export_stmt
             print '\n%s;'  % import_stmt
          if self.options.transformationtype == 'EXPORT':
             print '\n%s;'  % export_stmt
          if self.options.transformationtype == 'IMPORT':   
             print '\n%s;'  % import_stmt 
             
      def cmd_stamp_package(self):
          if self.exporter != None:
             self.exporter.cmd = 'REMARK "TRANSFORMATIONPACKAGE  %s" \n' % self.package.sourcepackagename
             self.exporter.cmd_execute()
          if self.importer != None:
             self.importer.cmd = 'REMARK "TRANSFORMATIONPACKAGE  %s" \n' % self.package.targetpackagename
             self.importer.cmd_execute()
             
      def cmd_transport_catalog_package(self):
          input = None
          # stream connector and stream filter
          self.streamconnector =  StreamConnector(self.exporter, self.importer )
          self.streamfilter    =  StreamFilter   (self.options , self )

          #stream package
          if self.options.initposition == 'RESTART':
             try:
                 if options.mode == 'INTER':
                    input('Start init Transformationpackages')
                 if options.mode == 'PARAM':
                    print 'Start init Transformationpackages'
                    i = 1/0
             except:
                 restartRecord = self.importer.cmd_restart_targetdb_package()
                 self.exporter.cmd_restart_sourcedb_package ( restartRecord )

          #stream export
          while input not in ["y"]:
                if self.options.mode == 'INTER':
                   input = raw_input('Export    DB %s ? [y]     : ' % ( self.package.sapdb ))
                if self.options.mode == 'PARAM':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.exporter.cmd_use_serverdb_user      ,())
                   time.sleep(1)
                   thread.start_new_thread( self.exporter.cmd_export_catalog_package ,()) #export
                   time.sleep(1)
                   #stream package
                   if self.options.streammedium == 'FILE' and self.options.streamformat <> 'XML':
                      while self.exporter.packageload  == 'FALSE' :
                            thread.start_new_thread( self.exporter.cmd_import_package   ,())    
                            time.sleep(1) 
                      while self.exporter.packageunload == 'FALSE' :
                            thread.start_new_thread( self.exporter.cmd_monitor_package,())
                            thread.start_new_thread( self.exporter.cmd_export_package ,())#release
                            time.sleep(1)
                      mutex.acquire()                                                         #acquire      
                   if self.options.streammedium == 'TAPE' :
                      while self.exporter.packageunload == 'FALSE' :
                            thread.start_new_thread( self.exporter.cmd_export_package,()) #release
                            time.sleep(1)
                      mutex.acquire()                                                         #acquire
                      self.importer.ddlpipe_ebid = self.exporter.ddlpipe_ebid
                      self.importer.datpipe_ebid = self.exporter.datpipe_ebid
                      self.importer.mappipe_ebid = self.exporter.mappipe_ebid

          input = None

          #stream import
          while input not in ["y"] :
                if self.options.mode == 'INTER':
                    input = raw_input('Import    DB %s ? [y]     : ' % ( self.package.sapdb ))
                if self.options.mode == 'PARAM':
                    input  = "y"
                if input  == "y":
                    #stream loader
                    thread.start_new_thread( self.importer.cmd_use_serverdb_user      ,())
                    time.sleep(1)
                    thread.start_new_thread( self.importer.cmd_import_catalog_package ,())  #import
                    time.sleep(1)
                    #stream package
                    while self.importer.packageload  == 'FALSE' :
                          thread.start_new_thread( self.importer.cmd_import_package   ,())    
                          time.sleep(1) 
                    while self.importer.packageunload == 'FALSE' :
                          thread.start_new_thread( self.importer.cmd_monitor_package,())
                          thread.start_new_thread( self.importer.cmd_export_package ,())   #release
                          time.sleep(1)
          self.exporter.packageload   = 'FALSE'
          self.exporter.packageunload = 'FALSE'
          self.importer.packageload   = 'FALSE'
          self.importer.packageunload = 'FALSE'
			    
      def cmd_export_catalog_package(self):
          if MONITOR == 'OFF':
             #stream packages
             self.exporter.cmd_export_catalog_package()
             if  mutex.locked()  == True :
                 mutex.release()           #release
          if MONITOR == 'ON':
             # stream loader
             thread.start_new_thread( self.exporter.cmd_export_catalog_package,())    #export
             # stream package
             while self.exporter.packageload  == 'FALSE' :
                   thread.start_new_thread( self.exporter.cmd_import_package   ,())    
                   time.sleep(1) 
             while self.exporter.packageunload == 'FALSE' :
                   thread.start_new_thread( self.exporter.cmd_monitor_package,())
                   thread.start_new_thread( self.exporter.cmd_export_package ,())    #release
                   time.sleep(1)
             self.exporter.packageload    = 'FALSE'
             self.exporter.packageunload  = 'FALSE'

      def cmd_import_catalog_package(self):
          if MONITOR == 'OFF':
             #stream packages
             self.importer.cmd_import_catalog_package()
             if  mutex.locked()  == True :
                 mutex.release()           #release
          if MONITOR == 'ON':
             # stream loader
             thread.start_new_thread( self.importer.cmd_import_catalog_package,())    #export
             # stream package
             while self.importer.packageload  == 'FALSE' :
                   thread.start_new_thread( self.importer.cmd_import_package   ,())    
                   time.sleep(1) 
             while self.importer.packageunload == 'FALSE' :
                   thread.start_new_thread( self.importer.cmd_monitor_package,())
                   thread.start_new_thread( self.importer.cmd_export_package ,())    #release
                   time.sleep(1)
             self.importer.packageload    = 'FALSE'
             self.importer.packageunload  = 'FALSE'

      def cmd_transport_schema_package(self):
          input = None
          # stream connector and stream filter
          self.streamconnector =  StreamConnector(self.exporter, self.importer )
          self.streamfilter    =  StreamFilter   (self.options , self )
          #stream packages
          if self.options.initposition == 'RESTART':
             try:
                 if options.mode == 'INTER':
                    input('Start init Transformationpackages')
             except:
                 restartRecord = self.importer.cmd_restart_targetdb_package()
                 self.exporter.cmd_restart_sourcedb_package( restartRecord )
          #stream export
          while input not in ["y"]:
                if self.options.mode == 'INTER':
                   input = raw_input('Export    Schema %s ? [y] : ' % ( self.package.user ))
                if self.options.mode == 'PARAM':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.exporter.cmd_export_schema_package  ,()) #export
                   #stream package
                   if self.options.streammedium == 'FILE' and self.options.streamformat <> 'XML':
                      while self.exporter.packageload  == 'FALSE' :
                            thread.start_new_thread( self.exporter.cmd_import_package   ,())    
                            time.sleep(1) 
                      while self.exporter.packageunload == 'FALSE' :
                            thread.start_new_thread( self.exporter.cmd_monitor_package,())
                            thread.start_new_thread( self.exporter.cmd_export_package ,())#release
                            time.sleep(1)
                      mutex.acquire()                                                         #acquire      
                   if self.options.streammedium == 'TAPE' :
                      mutex.acquire()
                      self.importer.ddlpipe_ebid = self.exporter.ddlpipe_ebid
                      self.importer.datpipe_ebid = self.exporter.datpipe_ebid
                      self.importer.mappipe_ebid = self.exporter.mappipe_ebid

          input = None
          # stream import
          while input not in ["y"]:
                if self.options.mode == 'INTER':
                   input = raw_input('Import    Schema %s ? [y] : ' % ( self.package.user ))
                if self.options.mode == 'PARAM':
                   input   = "y"
                if input  == "y":
                    #stream loader
                    thread.start_new_thread( self.importer.cmd_import_schema_package ,())  #import
                    time.sleep(1)
                    #stream package
                    while self.importer.packageload  == 'FALSE' :
                          thread.start_new_thread( self.importer.cmd_import_package   ,())    
                          time.sleep(1) 
                    while self.importer.packageunload == 'FALSE' :
                          thread.start_new_thread( self.importer.cmd_monitor_package,())
                          thread.start_new_thread( self.importer.cmd_export_package ,())   #release
                          time.sleep(1)
          self.exporter.packageload   = 'FALSE'
          self.exporter.packageunload = 'FALSE'
          self.importer.packageload   = 'FALSE'
          self.importer.packageunload = 'FALSE'

      def cmd_export_schema_package(self):
          if MONITOR == 'OFF':
             #stream package
             self.exporter.cmd_export_schema_package()
             if  mutex.locked()  == True :
                 mutex.release()           #release
          if MONITOR == 'ON':
             # stream loader
             thread.start_new_thread( self.exporter.cmd_export_schema_package,())    #export
             # stream package
             while self.exporter.packageload  == 'FALSE' :
                   thread.start_new_thread( self.exporter.cmd_import_package   ,())    
                   time.sleep(1) 
             while self.exporter.packageunload == 'FALSE' :
                   thread.start_new_thread( self.exporter.cmd_monitor_package,())
                   thread.start_new_thread( self.exporter.cmd_export_package ,())    #release
                   time.sleep(1)
             self.exporter.packageload    = 'FALSE'
             self.exporter.packageunload  = 'FALSE'

      def cmd_import_schema_package(self):
          if MONITOR == 'OFF':
             #stream package
             self.importer.cmd_import_schema_package()
             if  mutex.locked()  == True :
                 mutex.release()           #release
          if MONITOR == 'ON':
             # stream loader
             thread.start_new_thread( self.importer.cmd_export_schema_package,())    #export
             # stream package
             while self.importer.packageload  == 'FALSE' :
                   thread.start_new_thread( self.importer.cmd_import_package   ,())    
                   time.sleep(1) 
             while self.importer.packageunload == 'FALSE' :
                   thread.start_new_thread( self.importer.cmd_monitor_package,())
                   thread.start_new_thread( self.importer.cmd_export_package ,())    #release
                   time.sleep(1)
             self.importer.packageload    = 'FALSE'
             self.importer.packageunload  = 'FALSE'

      def cmd_transport_table_package(self):
          input = None
          if( (self.options.sourcedbnode == self.options.targetdbnode)   and
              (self.options.sourcedbname == self.options.targetdbname) and 
               self.options.streamformat == 'SQL' ):
               granted = True
               self.exporter.cmd     = "SELECT PRIVILEGE FROM DOMAIN.TABLEPRIVILEGES WHERE GRANTEE = '%s' AND TABLENAME = '%s' AND PRIVILEGE LIKE 'SELECT' " %(re.split(',',self.options.targetdbuser)[0],self.package.table)    
               self.exporter.cmd_execute()
               if self.exporter.sqlCode == 100: 
                  self.exporter.cmd  = "GRANT SELECT ON %s.%s TO     %s"     %(re.split(',',self.options.sourcedbuser)[0],self.package.table, re.split(',',self.options.targetdbuser)[0])
                  self.exporter.cmd_execute()
                  self.exporter.cmd  = "COMMIT WORK"
                  self.exporter.cmd_execute()
                  granted = False
               self.importer.cmd = "CREATE TABLE %s.%s LIKE       %s.%s"     %(re.split(',',self.options.targetdbuser)[0],self.package.table,re.split(',',self.options.sourcedbuser)[0],self.package.table)   
               self.importer.cmd_execute()
               self.importer.cmd = "COMMIT WORK"
               self.importer.cmd_execute()
               self.importer.cmd = "INSERT INTO  %s.%s SELECT * FROM %s.%s"  %(re.split(',',self.options.targetdbuser)[0],self.package.table,re.split(',',self.options.sourcedbuser)[0],self.package.table)
               self.importer.cmd_execute()
               self.importer.cmd = "COMMIT WORK"
               self.importer.cmd_execute()
               if  granted == False:
                   self.exporter.cmd = "REVOKE SELECT ON %s.%s  FROM %s"     %(re.split(',',self.options.sourcedbuser)[0],self.package.table, re.split(',',self.options.targetdbuser)[0])
                   self.exporter.cmd_execute()
                   self.exporter.cmd = "COMMIT WORK"
                   self.exporter.cmd_execute()
               if mutex.locked() == True:
                  mutex.release()

          if self.options.streamformat != 'SQL' : 
             #stream filter
             self.streamfilter   = StreamFilter   (self.options , self )
             #Medium
             streamformat        = self.options.streamformat
             streamcharencoding  = self.options.streamcharencoding
             #Format
             streamformat_target = self.options.streamformat_target
          
             if self.package.column_field_long <> '' :
                if((self.options.streamformat     == 'FORMATTED' or
                    self.options.streamformat     == 'COMPRESSED'  )):
                    if self.options.streammedium  == 'PIPE' :
                       self.options.streamformat   = 'RECORDS'
                    if self.options.streamformat_target == 'PAGE' :
                       self.options.streamformat_target  = 'ROW'
                if self.options.streamformat      == 'RECORDS':
                   if self.options.streamcharencoding   == 'UTF8':
                      self.options.streamcharencoding    = 'UCS2'
             #stream export
             while input not in ["y"]:
                   if self.options.mode == 'INTER':
                      input = raw_input('Export    Table %s ? [y] : ' % ( self.package.table ))
                   if self.options.mode == 'PARAM':
                      input   = "y"
                   if input  == "y":
                      #stream loader
                      thread.start_new_thread( self.exporter.cmd_export_table_package,   ())    #export
                      if  self.options.streammedium == 'FILE'  :
                          #stream package
                          while  self.exporter.packageload  == 'FALSE' :
                                 thread.start_new_thread( self.exporter.cmd_import_package,())   
                                 time.sleep(1) 
                          while  self.exporter.packageunload == 'FALSE' :
                                 thread.start_new_thread( self.exporter.cmd_monitor_package,())
                                 thread.start_new_thread( self.exporter.cmd_export_package ,()) #release
                                 time.sleep(1)
             input = None
             #stream import
             while input not in ["y"]:
                   if self.options.mode == 'INTER':
                      input = raw_input('Import    Table %s ? [y] : ' % ( self.package.table ))
                   if self.options.mode == 'PARAM':
                      input   = "y"
                   if input  == "y":
                      #stream loader
                      thread.start_new_thread( self.importer.cmd_import_table_package, ())    #import
                      #stream package
                      while self.importer.packageload  == 'FALSE' :
                             thread.start_new_thread( self.importer.cmd_import_package  ,())    
                             time.sleep(1) 
                      while  self.importer.packageunload == 'FALSE':
                             thread.start_new_thread( self.importer.cmd_monitor_package,())
                             thread.start_new_thread( self.importer.cmd_export_package ,())   #release
                             time.sleep(1)
             #Medium
             if streamformat != self.options.streamformat :
                self.options.streamformat = streamformat
             if streamcharencoding != self.options.streamcharencoding :
                self.options.streamcharencoding = streamcharencoding
             #Format
             if streamformat_target != self.options.streamformat_target :
                self.options.streamformat_target = streamformat_target
             self.exporter.packageload   = 'FALSE'
             self.exporter.packageunload = 'FALSE'
             self.importer.packageload   = 'FALSE'
             self.importer.packageunload = 'FALSE'

      def cmd_export_table_package(self):
          if MONITOR == 'OFF':
             #stream package
             self.exporter.cmd_export_table_package()
             if  mutex.locked()  == True :
                 mutex.release()           #release
          if MONITOR == 'ON':
             # stream loader
             thread.start_new_thread( self.exporter.cmd_export_table_package, ())    #export
             # stream package
             while self.exporter.packageload  == 'FALSE' :
                   thread.start_new_thread( self.exporter.cmd_import_package   ,())    
                   time.sleep(1) 
             while self.exporter.packageunload == 'FALSE' :
                   thread.start_new_thread( self.exporter.cmd_monitor_package,())
                   thread.start_new_thread( self.exporter.cmd_export_package ,())    #release
                   time.sleep(1)
             self.exporter.packageload    = 'FALSE'
             self.exporter.packageunload  = 'FALSE' 
		  
      def cmd_import_table_package(self):
          if MONITOR == 'OFF':
             #stream package
             self.importer.cmd_import_table_package()
             if  mutex.locked()  == True :
                 mutex.release()           #release
          if MONITOR == 'ON':
             # stream connector
             self.streamconnector =  StreamConnector(self.exporter, self.importer )
             self.streamfilter    =  StreamFilter   (self.options , self )
             #stream filter
             self.streamfilter.cmd_dtlsource2dtltarget()
             #stream loader
             thread.start_new_thread(self.importer.cmd_import_table_package, ())    #import
             # stream package
             while self.importer.packageload  == 'FALSE' :
                   thread.start_new_thread( self.importer.cmd_import_package   ,())    
                   time.sleep(1) 
             while self.importer.packageunload == 'FALSE' :
                   thread.start_new_thread( self.importer.cmd_monitor_package,())
                   thread.start_new_thread( self.importer.cmd_export_package ,())   #release
                   time.sleep(1)
             self.importer.packageload    = 'FALSE'
             self.importer.packageunload  = 'FALSE'
                         
      def cmd_import_table_package_index(self):
          for cmd in self.package.indexStatements :
              self.importer.cmd = cmd
              self.importer.cmd_execute()

      def cmd_verify_package(self):
          input         = None
          sourcepackage = None
          targetpackage = None
          while input not in ["y"]:
                if self.options.mode == 'INTER':
                   input = raw_input('\nVerify    transformation package ? [y] : ')
                if self.options.mode == 'PARAM':
                   print '\nVerify    transformation package'
                   input   = "y"
                if input  == "y":
                   if  self.options.transformationpart  == 'DB'   :
                       if  self.options.transformationtype == 'TRANSPORT'  :
                           if os.path.isfile(os.path.join(self.options.sourceindepdatapath,'%s.package' % self.options.sourcedbname )):
                              sourcepackage =  PACKAGEIMPORT % ( os.path.join(self.options.sourceindepdatapath,'%s.package' % self.options.sourcedbname ),'')
                              if self.exporter.packageguid != None:
                                 print 'Import    transformation package of EXPORT '   
                              if self.exporter.packageguid == None:   
                                 print 'Import    transformation package of EXPORT ' 
                              try :
                                     self.importer.loadersession.cmd ("%s" %  sourcepackage  )
                              except loader.CommunicationError, err :
                                     raise loader.CommunicationError, err, sys.exc_traceback
                              except loader.LoaderError, err:
                                     raise loader.LoaderError, err, sys.exc_traceback

                       time.sleep(1)               
                       if  self.options.transformationtype == 'TRANSPORT'  :
                           if os.path.isfile(os.path.join(self.options.targetindepdatapath,  '%s.package' % self.options.targetdbname )) :
                               if self.importer.catalog == 'UNICODE' :
                                  self.importer.package  = 'UCS2'
                               if self.importer.catalog == 'ASCII' :
                                  self.importer.package  = 'ASCII'
                               targetpackage =  PACKAGEIMPORT % ( os.path.join(self.options.targetindepdatapath,'%s.package' % self.options.targetdbname ), '') 
                               if self.importer.packageguid != None:
                                  print 'Import    transformation package of IMPORT ' 
                               if self.importer.packageguid == None:
                                  print 'Import    transformation package of IMPORT ' 
                               try :
                                      self.importer.loadersession.cmd ("%s" %  targetpackage )
                               except loader.CommunicationError, err :
                                      raise loader.CommunicationError, err, sys.exc_traceback
                               except loader.LoaderError, err:
                                      raise loader.LoaderError, err, sys.exc_traceback

                       if  self.options.transformationtype == 'TRANSPORT'  :
                           time.sleep(1)
                           cmd = PACKAGEVALIDATE
                           cmd = cmd + '   WHERE  ( T1.start  <> T2.start  or T1.datastreamheaderpos <>  T2.datastreamheaderpos or  T1.datastreamdatapos <> T2.datastreamdatapos  ) '
                           cmd = cmd + '   AND T1.owner = T2.owner and T1.tablename = T2.tablename '
                           self.cmd = cmd
                           try :
                               self.importer.loadersession.cmd ("%s" % self.cmd  )
                           except loader.LoaderError, err:
                               if err.sqlCode == 100:
                                  print 'OK'
                               else:
                                  raise loader.LoaderError, err, sys.exc_traceback
                              
          if mutex.locked () == 1:
             mutex.release()  #release                           
             
      def cmd_purify_package(self):
          if  socket.gethostname() == 'p30882' :  #p30882
              self.exporter.loadersession.release()
              self.importer.loadersession.release()
              try :
                   os.chdir (os.path.join (os.environ ['INSTROOT'], 'wrk'))
              except OSError, (errno, err ):
                   print 'command failed: ', err

              #source
              if  os.path.isfile      ( self.exporter.protocol ) :
                  transformationstep = []
                  protocol      = file( self.exporter.protocol ,'r')
                  protocolline  = protocol.readline()
                  while protocolline :
                        match_1 = re.search ('// R', protocolline )
                        if match_1 != None :
                           cmd  = string.replace( protocolline , "// R\t" , '//\n')
                           transformationstep.append(cmd)
                        protocolline  = protocol.readline()
                  protocol.close()
                  time.sleep(1)
                  protocol      = file( self.exporter.protocol ,'a+')
                  protocol.writelines(transformationstep)
                  protocol.close()

                  sourceloader_batch  = string.replace(self.exporter.protocol, 'prt', 'bat' )
                  sourceloader_purify = string.replace(self.exporter.protocol, 'prt', 'pur' )
                  try:
                     os.rename(self.exporter.protocol, sourceloader_batch )
                  except OSError, err:
                     print 'command failed:', err
                  purify_loadercli  = '\n%s -b %s -E 10\n'   % (os.path.join(self.options.sourceindepdatapath,'bin','loadercli'), sourceloader_batch)
                  #purify_loadercli  = 'purify /SaveTextData=%s /ExcludeSystemDir %s.exe -b %s'   % (sourceloader_purify,os.path.join( self.options.sourceindepdatapath,'bin','loadercli'), sourceloader_batch)
                  print purify_loadercli
                  try:
                      os.system (purify_loadercli)
                  except OSError, err:
                      print 'command failed:', err

              #target
              if  os.path.isfile      ( self.importer.protocol ) :
                  transformationstep = []
                  protocol      = file( self.importer.protocol ,'r')
                  protocolline  = protocol.readline()
                  while protocolline :
                        match_1 = re.search ('// R', protocolline )
                        if match_1 != None :
                           cmd  = string.replace( protocolline , "// R\t" , '//\n')
                           transformationstep.append(cmd)
                        protocolline  = protocol.readline()
                  protocol.close()
                  time.sleep(1)
                  protocol      = file( self.importer.protocol ,'a+')
                  protocol.writelines(transformationstep)
                  protocol.close()

                  targetloader_batch  = string.replace(self.importer.protocol, 'prt', 'bat' )
                  targetloader_purify = string.replace(self.importer.protocol, 'prt', 'pur' )
                  try:
                     os.rename(self.importer.protocol, targetloader_batch )
                  except OSError, err:
                     print 'command failed:', err
                  purify_loadercli  = '\n%s -b %s -E 10\n'   % (os.path.join(self.options.targetindepdatapath,'bin','loadercli'), targetloader_batch)
                  #purify_loadercli  = 'purify /SaveTextData=%s /ExcludeSystemDir %s.exe -b %s'   % (targetloader_purify,os.path.join(self.options.targetindepdatapath,'bin','loadercli'),targetloader_batch)
                  print purify_loadercli
                  try:
                      os.system (purify_loadercli)
                  except OSError, err:
                      print 'command failed:', err

class StreamTransformator:
      def __init__(self, options = None, package = None, transformationtype = None):
          self.options                = options
          self.package                = package
          self.transformationtype     = transformationtype
          if self.transformationtype== 'EXPORT':
             self.sapdbnode = options.sourcedbnode
             self.sapdb     = options.sourcedbname
             username = re.split(',',options.sourcedbuser)[0]
             password = re.split(',',options.sourcedbuser)[1]
             if username != '' and username [0] != '"' and username[len(username)-1] != '"':
                username = string.upper(username)
             elif username != '':
                username = string.replace(username,'"','') 
             self.user      = username
             self.pwd       = password
             self.schema    = self.options.sourcedbschema
             self.catalog   = DB_DEFAULTCODE_SOURCE
          if self.transformationtype== 'IMPORT':
             self.sapdbnode = options.targetdbnode
             self.sapdb     = options.targetdbname
             username = re.split(',',options.targetdbuser)[0]
             password = re.split(',',options.targetdbuser)[1]
             if username != '' and username [0] != '"' and username[len(username)-1] != '"':
                username = string.upper(username)
             elif username != '':
                username = string.replace(username,'"','') 
             self.user      = username
             self.pwd       = password
             self.schema    = self.options.targetdbschema
             self.catalog   = DB_DEFAULTCODE_TARGET
          self.version                = ''
          self.streammedium           = options.streammedium
          self.streamformat           = options.streamformat
          self.streamcharencoding     = options.streamcharencoding
          self.initposition          = options.initposition
          self.owner                  = None
          self.table                  = None
          self.packagecode            = 'ASCII'
          self.packageguid            = None
          self.packagestart           = None
          self.packageend             = PACKAGETIMEOUT
          self.packageload            = 'FALSE'
          self.packageunload          = 'FALSE'
          self.protocol               = None
          self.cmd                    = None
          self.loaderpackage          = None
          #stream_name
          self.catalogstream_name     = None
          self.datastream_name        = None
          self.packagestream_name     = None 
          # medium pipe
          self.ddlpipe                = "\\\\.\PIPE\\catalog"
          self.datpipe                = "\\\\.\PIPE\\data"
          self.datpipe_source         = "\\\\.\PIPE\\source.data"
          self.datpipe_target         = "\\\\.\PIPE\\target.data"
          self.ddlpipe_source         = os.path.join('\\\\','.','PIPE','%s.CATALOG' % self.options.sourcedbname)
          self.ddlpipe_target         = os.path.join('\\\\','.','PIPE','%s.CATALOG' % self.options.targetdbname)
          # medium tape
          self.ddlpipe_ebid           = None
          self.datpipe_ebid           = None
          self.mappipe_ebid           = None
          # statement
          self.cmd
          self.monitor                = None
          self.sqlCode                = 0
          self.loadersession          = None
          self.errorCode              = 0
          # protocol session
          try:
              if self.package != None:
                 if self.transformationtype == 'EXPORT' or self.transformationtype == 'TRANSPORT' :
                    self.loaderpackage  = file(os.path.join(self.options.sourceindepdatapath,os.path.split(self.options.sourceindepdatapath)[1] +os.extsep+"prt"),'w+')
                 if self.transformationtype == 'IMPORT' or self.transformationtype == 'TRANSPORT' :
                    self.loaderpackage  = file(os.path.join(self.options.targetindepdatapath,os.path.split(self.options.targetindepdatapath)[1] +os.extsep+"prt"),'w+')
          except IOError, err:
              print err
          
          # sql session
          try:
              self.monitor = sql.connect ( self.user, self.pwd, self.sapdb, self.sapdbnode )
          except sql.CommunicationError, err :
              print err
              raise sql.CommunicationError, err, sys.exc_traceback
          except sql.SQLError, err :
              print err
              raise sql.SQLError, err, sys.exc_traceback
              
          # loader session
          try:
              global EXPORTER_VERSION
              global IMPORTER_VERSION
              self.loadersession =  loader.Loader('', '', '%s' % self.options.loaderinstroot)
              #input = raw_input('Loader started  ' )
              #Module Name
              self.cmd = 'REMARK "Loader Extension Module: %s"'% os.path.abspath(sys.argv[0])
              remark = self.cmd_execute()
              #Configuration Directory
              self.cmd = 'REMARK "Loader Config Directory: %s"'% os.path.join (LOADERPROGPATH,'sdb','loader','config')
              remark = self.cmd_execute()
              #version
              self.cmd = 'VERSION\n'
              version = self.cmd_execute()
              if self.transformationtype== 'EXPORT':
                 EXPORTER_VERSION = string.replace(re.split(',',re.split('\n',version)[1])[0],'"','')
              if self.transformationtype== 'IMPORT':
                 IMPORTER_VERSION = string.replace(re.split(',',re.split('\n',version)[1])[0],'"','')
              #protocol odbc
              self.cmd = 'SET ODBCTRACE        OFF\n'
              self.cmd_execute()
              #protocol loader
              self.cmd = 'SET\n'
              configuration = self.cmd_execute()
              if configuration != 0:
                 self.protocol = string.strip(string.replace(string.replace(configuration,'Loader protocol:',''),'\'',''))
              self.cmd  = "SET CONFIGURATION   EXCLUDE \n" 
              self.cmd_execute()   
              #sqlmode
              self.cmd = 'SQLMODE INTERNAL\n'
              self.cmd_execute()
              self.cmd = 'SET SYSTEMATTRIBUTE  %s\n' % SYSTEMATTRIBUTE
              self.cmd_execute()
              #transaction
              self.cmd = 'AUTOCOMMIT           %s\n' % AUTOCOMMIT
              self.cmd_execute()
              self.cmd = 'SET TRANSACTION SIZE %s\n' % TRANSACTIONSIZE
              self.cmd_execute()
              self.cmd = 'SET ISOLATION LEVEL  %s\n' % ISOLATIONLEVEL
              self.cmd_execute()              
              self.cmd = 'SET MAXERRORCOUNT    %s\n' % MAXERRORCOUNT
              self.cmd_execute()
              #print      'SET PACKAGETIMEOUT  %s\n' % PACKAGETIMEOUT
              #format
              self.cmd = "SET COMPRESSED       '%s'\n" % FIELDCOMPRESSED
              self.fieldCompressed            = FIELDCOMPRESSED
              self.cmd_execute()
              self.cmd = "SET LANGUAGE         'ENG'\n"
              self.cmd_execute()
              #datatypes
              #self.cmd = 'SET CODETYPE         ASCII\n'
              #self.cmd_execute()
              self.cmd = 'SET CODEPAGE         ISO-8859-1\n'
              self.cmd_execute()
              self.cmd = "SET BOOLEAN          '%s'\n" % FIELDBOOLVALUE
              self.fieldBoolValue            =  FIELDBOOLVALUE
              self.cmd_execute()
              #self.cmd = "SET DECIMAL         '%s'\n" % FIELDDECIMALVALUE
              #self.fieldDecimalValue        =  FIELDDECIMALVALUE
              #self.cmd_execute()
              self.fieldDecimalValue         =  FIELDDECIMALVALUE
              self.cmd = "SET NULL              '%s'\n" % FIELDNULLVALUE
              self.fieldNullValue            =  FIELDNULLVALUE
              self.cmd_execute()
              self.cmd = 'SET DATE             %s\n' % FIELDATEVALUE
              self.fieldDateValue            = FIELDATEVALUE
              self.cmd_execute()
              self.cmd = 'SET TIME             %s\n' % FIELDTIMEVALUE
              self.fieldTimeValue            = FIELDTIMEVALUE
              self.cmd_execute()
              self.cmd = 'SET TIMESTAMP        %s\n' % FIELDTIMESTAMPVALUE
              self.fieldTimeStampValue       = FIELDTIMESTAMPVALUE
              self.cmd_execute()
              #medium
              self.cmd = 'SET BACKUPTOOL       NETWORKER\n'
              self.cmd_execute()
          except loader.CommunicationError, err :
                 raise loader.CommunicationError, err, sys.exc_traceback
          except loader.LoaderError, err:
                 raise loader.LoaderError, err, sys.exc_traceback

      def release(self):
          # sql session
          if self.monitor != None:
             try:
                self.monitor.release()
             except sql.CommunicationError, err :
                pass
             except:
                pass
          # loader session
          if self.loadersession != None:
             try:
                 self.loadersession.release()
             except loader.CommunicationError, err :
                    raise loader.CommunicationError, err, sys.exc_traceback
             except loader.LoaderError, err:
                    raise loader.LoaderError, err, sys.exc_traceback
          # protocol session
          if self.loaderpackage != None:
             try:
                 time.sleep(0.5)
                 self.loaderpackage.close()
             except IOError, err:
                 print err
          
      def cmd_execute(self):
          self.packagestart = time.time()
          rc  = 0
          cmd = self.cmd
          cmd = string.replace(cmd,self.options.sourceindepdatapath + os.sep , '')
          cmd = string.replace(cmd,self.options.targetindepdatapath + os.sep , '')
          cmd = '\n%s//' % cmd
          if self.package != None and self.options.transporttype != 'EXECUTE':  
             self.packageload   = 'TRUE'
             self.packageunload = 'TRUE'
             try :
                  if self.loaderpackage != None and self.cmd[0:8] == 'USE USER':
                     if self.package.schema != '':
                        cmd  = '\nUSE USER %s %s SCHEMA %s SERVERDB %s ON %s \n//'% ( self.user,'*', self.package.schema, self.sapdb, self.sapdbnode)
                     else:
                        cmd  = '\nUSE USER %s %s           SERVERDB %s ON %s \n//'% ( self.user,'*',                      self.sapdb, self.sapdbnode)
                  if self.loaderpackage != None and self.cmd[0:6] != 'REMARK' :
                     self.loaderpackage.write(cmd)
             except IOError, err:
                   print err
             if mutex.locked() == True:
                mutex.release()    #release
          if self.package == None or self.package.transformationpackage == None or self.options.transporttype == 'EXECUTE':         
             try :
                 rc = self.loadersession.cmd ("%s" % self.cmd )
             except loader.LoaderError, err:
                 if self.package != None and self.package.transformationpackage != None:
                    self.cmd_error_handler(err)
                 if self.package.transformationpackage == None:
                    print err  
                    self.package.transformation.release()
                    os._exit(1) 
             except loader.CommunicationError, err :
                 print err
                 print 'Received cancel request; stopping execution'
                 self.package.transformation.release()
                 os._exit(1)
             try :
                 if self.loaderpackage != None and self.cmd[0:8] == 'USE USER':
                    if self.schema != '':
                       cmd  = '\nUSE USER %s %s SCHEMA %s SERVERDB %s ON %s \n//'% ( self.user,'*', self.schema, self.sapdb, self.sapdbnode)
                    else:
                       cmd  = '\nUSE USER %s %s           SERVERDB %s ON %s \n//'% ( self.user,'*',              self.sapdb, self.sapdbnode)
                 if self.loaderpackage != None and self.cmd[0:6] != 'REMARK' :
                    self.loaderpackage.write(cmd)
             except IOError, err:
                   print err
          self.cmd = ''
          return rc
      
      def cmd_error_handler(self,err):
          self.sqlCode          = err.sqlCode
          self.errorCode        = err.errorCode
          if( err.sqlCode       ==  100   or
              err.sqlCode       ==  200   or
              err.sqlCode       == -4004  or
              err.sqlCode       == -4011  or
              err.sqlCode       == -5001  or
              err.sqlCode       == -6000  or
              err.sqlCode       == -6008  or
              err.sqlCode       == -7017  or
              err.errorCode     == -25010 or
              err.errorCode     == -25362 or
              err.errorCode     == -25498 or
              err.errorCode     == -25500 or
              err.errorCode     == -25501  ):
              if( err.sqlCode   ==  200   or
                  err.errorCode == -25010 or
                  err.errorCode == -25362 or
                  err.errorCode == -25498 or
                  err.errorCode == -25500 or
                  err.errorCode == -25501 ):
                  if err.sqlCode   ==  200  and self.options.streamformat != 'SQL' :
                     print '\n%s error(s) during execution of package: \n---> See loader server logfile for more information.' % MAXERRORCOUNT
                     self.package.transformation.release()
                     os._exit(1)
                  if err.errorCode == -25010 :
                     pass
                  if err.errorCode == -25362 :
                     pass
                  if err.errorCode == -25498 :
                     pass
                  if err.errorCode == -25500 :
                     print err
                     self.package.transformation.release()
                     os._exit(1)
                  if err.errorCode == -25501 :
                     print '\n%s error(s) during execution of package: \n---> See loader server logfile for more information.' % MAXERRORCOUNT
                     pass
              else:
                 self.package.maxerrorcount = self.package.maxerrorcount + 1
              if self.package.maxerrorcount == 1 :
                 print '\nAn error during execution of package: \n---> See loader server logfile for more information.'
              if self.package.maxerrorcount == MAXERRORCOUNT :
                 self.package.transformation.release()
                 os._exit(1)
          else:
             raise loader.LoaderError, err, sys.exc_traceback

      def cmd_use_serverdb_user(self):
          if self.schema != '':
              cmd     = 'USE USER %s %s SCHEMA %s SERVERDB %s' % ( self.user, self.pwd, self.schema, self.sapdb )
          else:
              cmd     = 'USE USER %s %s           SERVERDB %s' % ( self.user, self.pwd,              self.sapdb )
          if self.sapdbnode != '':
             cmd  = cmd + ' ON %s\n'             % ( self.sapdbnode )
          self.cmd     = cmd
          self.cmd_execute()

      def cmd_export_catalog_package(self):
          #Medium
          if   self.streammedium[0:4] == 'PIPE' :
               streammedium = 'PIPE'
          else :
               streammedium = self.streammedium

          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'    ):
              #Medium
              if self.options.streammedium == 'FILE' :
                 cmd = "CATALOGEXTRACT ALL"
                 cmd = cmd +   " OUTSTREAM FILE '%s' %s          "   % (os.path.join(self.options.sourceindepdatapath,  '%s.catalog' % (self.options.sourcedbname )),self.options.streamformat,)
              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = 'CATALOGEXTRACT ALL \n'
                 cmd = cmd +   " OUTSTREAM  %s '%s'  %s    %s    "   % (streammedium, self.ddlpipe_source, self.options.streamformat, self.streamcharencoding )

              if self.streamformat  == 'XML':
                 self.packageload    = 'TRUE'
                 self.packageunload  = 'TRUE'
                 mutex.release()

          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              restartRecord = self.cmd_restart_package()
              #Medium
              if self.options.streammedium == 'FILE' :
                 cmd = 'DBEXTRACT \n'
                 cmd = cmd +   " CONFIGURATION  INSTREAM  FILE '%s' COMPRESSED  %s     \n"  % (os.path.join(self.options.sourceindepdatapath,  '%s.configuration' % self.sapdb ), self.options.streamcharencoding )
                 cmd = cmd +   " CATALOG        OUTSTREAM FILE '%s' DDL         %s     \n"  % (os.path.join(self.options.sourceindepdatapath,  '%s.catalog'       % self.sapdb ), self.streamcharencoding )
                 cmd = cmd +   " DATA           OUTSTREAM FILE '%s' %s          %s     \n"  % (os.path.join(self.options.sourceindepdatapath,  '%s.data'          % self.sapdb ), self.options.streamformat, self.streamcharencoding )
                 cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s' COMPRESSED  %s       "  % (os.path.join(self.options.sourceindepdatapath,  '%s.package'       % self.sapdb ), self.streamcharencoding )

              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = 'DBEXTRACT \n'
                 cmd = cmd +   " CONFIGURATION  INSTREAM  FILE '%s'  COMPRESSED %s     \n"  % (os.path.join(self.options.sourceindepdatapath,  '%s.configuration' % self.sapdb ), self.streamcharencoding )
                 cmd = cmd +   " CATALOG        OUTSTREAM %s   '%s'  DDL        %s     \n"  % (streammedium, self.ddlpipe_source                     , self.streamcharencoding )
                 cmd = cmd +   " DATA           OUTSTREAM %s   '%s'  %s         %s     \n"  % (streammedium, self.datpipe_source , self.streamformat , self.streamcharencoding )
                 cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s'  COMPRESSED %s       "  % (os.path.join(self.options.sourceindepdatapath,  '%s.package'       % self.sapdb ), self.streamcharencoding )
              #Start
              if restartRecord <> [] :
                 cmd = cmd + " RESTART '%s' " % (restartRecord[0])

          if TRANSFORMATIONSYNTAX == 'STRATIFIED':
             self.cmd = cmd
          if TRANSFORMATIONSYNTAX == 'UNIFIED':
             self.cmd = self.package.transformation.exportstatement
          self.cmd_execute()

      def cmd_import_catalog_package(self):
          #Medium
          if  self.streammedium      == 'TAPE' :
              self.ddlpipe_target =  self.ddlpipe_source
              self.datpipe_source =  self.datpipe_source
          if  self.streammedium[0:4] == 'PIPE' :
              streammedium = 'PIPE'
          else:
              streammedium = self.streammedium

          #Encoding
          if self.catalog == 'UNICODE' :
             self.packagecode  = 'UCS2'
          if self.catalog == 'ASCII' :
             self.packagecode  = 'ASCII'

          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'   ):
              cmd = 'CATALOGLOAD ALL '
              #Medium
              if self.streammedium      == 'FILE':
                 cmd = cmd + "                   INSTREAM FILE '%s'             %s          " % (os.path.join(self.options.sourceindepdatapath,   '%s.catalog' % (self.options.sourcedbname )),self.streamformat)
              #Medium
              if self.streammedium[0:4] == 'PIPE' :
                 cmd = cmd + "                   INSTREAM  %s  '%s'             %s  %s      " % (streammedium, self.ddlpipe_target                       , self.streamformat, self.streamcharencoding   )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd = cmd + "                   INSTREAM  %s  EBID %s  '%s'    %s  %s      " % (streammedium, self.ddlpipe_ebid , self.ddlpipe_target   , self.streamformat, self.streamcharencoding   )

              if self.streamformat  == 'XML':
                 self.packageload    = 'TRUE'
                 self.packageunload  = 'TRUE'

          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              restartRecord = self.cmd_restart_package()

              #Medium
              if self.streammedium[0:4] == 'FILE' :
                 cmd = 'DBLOAD \n'
                 cmd = cmd +    " CONFIGURATION  INSTREAM FILE  '%s' COMPRESSED  %s   \n" % (os.path.join(self.options.sourceindepdatapath,  '%s.configuration' % self.sapdb), self.streamcharencoding )
                 cmd = cmd +    " CATALOG        INSTREAM FILE  '%s' DDL         %s   \n" % (os.path.join(self.options.sourceindepdatapath,  '%s.catalog'       % self.options.sourcedbname), self.streamcharencoding)
                 cmd = cmd +    " DATA           INSTREAM FILE  '%s' %s          %s   \n" % (os.path.join(self.options.sourceindepdatapath,  '%s.data'          % self.options.sourcedbname), self.options.streamformat, self.streamcharencoding)
              #Medium
              if self.streammedium[0:4] == 'PIPE' :
                 cmd = 'DBLOAD \n'
                 cmd = cmd +    " CONFIGURATION  INSTREAM FILE  '%s' COMPRESSED  %s   \n" % (os.path.join(self.options.sourceindepdatapath,  '%s.configuration' % self.sapdb), self.streamcharencoding )
                 cmd = cmd +    " CATALOG        INSTREAM  %s   '%s' DDL         %s   \n" % (streammedium, self.ddlpipe_target                     , self.streamcharencoding   )
                 cmd = cmd +    " DATA           INSTREAM  %s   '%s' %s          %s   \n" % (streammedium, self.datpipe_source , self.streamformat , self.streamcharencoding   )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd = 'DBLOAD \n'
                 cmd = cmd +    " CONFIGURATION  INSTREAM FILE  '%s' COMPRESSED  %s        \n" % (os.path.join(self.options.sourceindepdatapath,  '%s.configuration' % self.sapdb), self.streamcharencoding )
                 cmd = cmd +    " CATALOG        INSTREAM  %s   EBID %s  '%s'   DDL   %s   \n" % (streammedium, self.ddlpipe_ebid , self.ddlpipe_target ,                       self.streamcharencoding   )
                 cmd = cmd +    " DATA           INSTREAM  %s   EBID %s  '%s'    %s   %s   \n" % (streammedium, self.mappipe_ebid , self.datpipe_source , self.streamformat   , self.streamcharencoding   )
              cmd = cmd +       " PACKAGE        OUTSTREAM FILE '%s' COMPRESSED  %s          " % (os.path.join(self.options.targetindepdatapath,  '%s.package' % (self.sapdb)), self.packagecode )
              #Start
              if restartRecord <> [] :
                 cmd = cmd +    " RESTART '%s' " % (restartRecord[0])

          if self.streamformat  <> 'XML':
             if TRANSFORMATIONSYNTAX == 'STRATIFIED':
                self.cmd = cmd
             if TRANSFORMATIONSYNTAX == 'UNIFIED':
                self.cmd = self.package.transformation.importstatement
             self.cmd_execute()

      def cmd_export_schema_package(self):
          #Medium
          if   self.streammedium[0:4] == 'PIPE' :
               streammedium = 'PIPE'
          else :
               streammedium = self.streammedium
          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              cmd = 'TABLEEXTRACT USER \n'
              cmd = cmd +      " CONFIGURATION  INSTREAM  FILE '%s'    COMPRESSED %s \n" % (os.path.join(self.options.sourceindepdatapath,   '%s.configuration' % (self.sapdb )), self.streamcharencoding )
              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = cmd +   " DATA           OUTSTREAM %s   '%s'    %s         %s \n" % (streammedium, self.datpipe_source , self.streamformat  , self.streamcharencoding )
                 cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s'    COMPRESSED %s   " % (os.path.join (self.options.sourceindepdatapath,   '%s.package'       % (self.sapdb )), self.streamcharencoding )
              #Medium
              if(self.options.streammedium == 'FILE'     ):
                 cmd = cmd +   " DATA           OUTSTREAM FILE '%s'    %s         %s \n" % (os.path.join(self.options.sourceindepdatapath,   '%s_%s.data'       % (self.options.sourcedbname,self.user )),self.options.streamformat,self.streamcharencoding )
                 cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s'    COMPRESSED %s   " % (os.path.join(self.options.sourceindepdatapath,   '%s_%s.package'    % (self.options.sourcedbname,self.user)), self.streamcharencoding )
          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'      ):
              cmd = 'CATALOGEXTRACT USER \n'
              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = cmd +   " OUTSTREAM %s   '%s'  %s   %s  "  % ( streammedium, self.ddlpipe_source, self.options.streamformat, self.streamcharencoding )
              #Medium
              if self.options.streammedium == 'FILE'  :
                 cmd = cmd +   " OUTSTREAM FILE '%s'  %s   %s  "    % (os.path.join(self.options.sourceindepdatapath,   '%s_%s.dtl'  % (self.sapdb,self.user )),self.options.streamformat, self.streamcharencoding)

          if TRANSFORMATIONSYNTAX == 'STRATIFIED':
             self.cmd = cmd
          if TRANSFORMATIONSYNTAX == 'UNIFIED':
             self.cmd = self.package.transformation.exportstatement
          self.cmd_execute()

      def cmd_import_schema_package(self):
          #Medium
          if  self.streammedium      == 'TAPE' :
              self.ddlpipe_target =  self.ddlpipe_source
              self.datpipe_source =  self.datpipe_source
          if  self.streammedium[0:4] == 'PIPE' :
              streammedium = 'PIPE'
          else:
              streammedium = self.streammedium

          if self.catalog == 'UNICODE' :
             self.packagecode  = 'UCS2'
          if self.catalog == 'ASCII' :
             self.packagecode  = 'ASCII'

          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              cmd = 'TABLELOAD    USER \n'
              cmd =       cmd + " CONFIGURATION INSTREAM  FILE '%s' COMPRESSED %s \n" % (os.path.join(self.options.sourceindepdatapath,  '%s.configuration' % (self.sapdb )), self.streamcharencoding )
              #Medium
              if self.streammedium[0:4] == 'PIPE':
                 cmd =    cmd + " DATA          INSTREAM  %s   '%s' %s         %s \n" % (streammedium, self.datpipe_source , self.streamformat , self.streamcharencoding   )
                 cmd =    cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath,  '%s.package' % (self.sapdb)), self.packagecode )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd =    cmd + " DATA          INSTREAM  %s   '%s' EBID %s %s %s \n" % (streammedium, self.datpipe_source , self.mappipe_ebid , self.streamformat , self.streamcharencoding   )
                 cmd =    cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath,  '%s.package' % (self.sapdb)), self.packagecode )
              #Medium
              if self.streammedium      == 'FILE':
                 if self.options.streamformat   == 'PAGES'   :
                    cmd = cmd + " DATA          INSTREAM  FILE '%s' %s            \n" % (os.path.join(self.options.sourceindepdatapath,  '%s_%s.data'       % (self.options.sourcedbname,self.user )),self.options.streamformat)
                    cmd = cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath,  '%s_%s.package'    % (self.options.targetdbname,self.user)), self.packagecode)
                 if self.options.streamformat   == 'RECORDS' :
                    cmd = cmd + " DATA          INSTREAM  FILE '%s' %s         %s \n" % (os.path.join(self.options.sourceindepdatapath,  '%s_%s.data'       % (self.options.sourcedbname,self.user )),self.options.streamformat,self.streamcharencoding)
                    cmd = cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath,  '%s_%s.package'    % (self.options.targetdbname,self.user)), self.packagecode)
          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'      ):
              cmd = 'CATALOGLOAD USER\n'
              #Medium
              if self.streammedium[0:4] == 'PIPE' :
                 cmd = cmd + "                INSTREAM  %s  '%s'          %s  %s  \n" % (streammedium, self.ddlpipe_target                     , self.options.streamformat  , self.streamcharencoding  )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd = cmd + "                INSTREAM  %s  '%s' EBID %s  %s  %s  \n" % (streammedium, self.ddlpipe_target , self.ddlpipe_ebid , self.options.streamformat , self.streamcharencoding   )
              #Medium
              if self.streammedium      == 'FILE':
                 cmd = cmd + "                INSTREAM FILE '%s'          %s   %s   " % (os.path.join(self.options.sourceindepdatapath,  '%s_%s.dtl'  % (self.options.sourcedbname,self.user )), self.options.streamformat, self.streamcharencoding )

              if self.streamformat  == 'XML':
                 self.packageload    = 'TRUE'
                 self.packageunload  = 'TRUE'

          if self.streamformat  <> 'XML':
             if TRANSFORMATIONSYNTAX == 'STRATIFIED':
                self.cmd = cmd
             if TRANSFORMATIONSYNTAX == 'UNIFIED':
                self.cmd = self.package.transformation.importstatement
             self.cmd_execute()

      def cmd_export_table_package(self):
          sapdb        = self.package.transformation.exporter.sapdb
          user         = self.package.user
          table        = self.package.table

          if TRANSFORMATIONSYNTAX == 'STRATIFIED':
             #Medium
             if self.options.streammedium    == 'FILE'  :
                #Format
               if(self.options.streamformat == 'XML' or
                  self.options.streamformat == 'DDL' ) :
                  cmd = 'CATALOGEXTRACT TABLE '
                  cmd = cmd + " \"%s\".\"%s\" \n"                 % (user, table)
                  cmd = cmd + "      OUTSTREAM FILE '%s' %s   \n" % (os.path.join(self.options.sourceindepdatapath, '%s_%s_%s.dtl'  % (sapdb,user,table )),self.options.streamformat)
               #Format
               if(self.options.streamformat == 'PAGES' or
                  self.options.streamformat == 'RECORDS'  ):
                  cmd = 'TABLEEXTRACT TABLE '
                  cmd = cmd + " \"%s\".\"%s\" \n"                 % ( user, table)
                  cmd = cmd + " DATA OUTSTREAM FILE '%s' %s %s\n" % (os.path.join(self.options.sourceindepdatapath, '%s_%s_%s.data' % (sapdb,user,table )),self.options.streamformat,self.options.streamcharencoding)
               #Format
               if(self.options.streamformat[0:9] == 'FORMATTED'  or
                  self.options.streamformat      == 'COMPRESSED'    ):
                  cmd = "DATAEXTRACT "
                  if self.options.streamformat_target == 'ROW':
                     cmd  = cmd + " FOR DATALOAD TABLE "
                  if self.options.streamformat_target == 'PAGE':
                     cmd  = cmd + " FOR FASTLOAD TABLE "
                  cmd = cmd + " \"%s\".\"%s\" \n"                 % (user, table)
                  cmd = cmd + "      OUTSTREAM FILE '%s'      \n" % (os.path.join(self.options.sourceindepdatapath, '%s_%s_%s.dtl'  % (sapdb,user,table )))
                  cmd = cmd + "      OUTSTREAM FILE '%s' %s %s\n" % (os.path.join(self.options.sourceindepdatapath, '%s_%s_%s.data' % (sapdb,user,table )),self.options.streamformat,self.options.streamcharencoding)
                  cmd = cmd + self.package.column_field_long
             #Medium
             if(self.options.streammedium[0:4] == 'PIPE' ):
                #Format
                if( self.options.streamformat == 'PAGES' or
                    self.options.streamformat == 'RECORDS'  ) :
                    cmd  = "TABLEEXTRACT TABLE "
                    cmd  = cmd + " \"%s\".\"%s\" \n"                 % ( user, table )
                    cmd  = cmd + " DATA OUTSTREAM PIPE '%s' %s %s\n" % ('\\\\.\\PIPE\\%s_%s_%s.dat' % ( sapdb, user, table ), self.options.streamformat, self.options.streamcharencoding )
                #Format
                if( self.options.streamformat == 'FORMATTED'  or
                    self.options.streamformat == 'COMPRESSED'      ):
                    cmd  = "DATAEXTRACT "
                    if self.options.streamformat_target == 'ROW':
                       cmd  = cmd + " FOR DATALOAD TABLE "
                    if self.options.streamformat_target == 'PAGE':
                       cmd  = cmd + " FOR FASTLOAD TABLE "
                    cmd  = cmd + " \"%s\".\"%s\" \n"               % (user, table)
                    cmd  = cmd + " OUTSTREAM PIPE '%s'         \n" % ('\\\\.\\PIPE\\%s_%s_%s.dtl' % ( sapdb, user, table ) )
                    cmd  = cmd + " OUTSTREAM PIPE '%s' %s   %s   " % ('\\\\.\\PIPE\\%s_%s_%s.dat' % ( sapdb, user, table ), self.options.streamformat, self.options.streamcharencoding )
                    cmd  = cmd + self.package.column_field_long
             self.cmd = cmd
             self.cmd_execute()       
          
          if TRANSFORMATIONSYNTAX == 'UNIFIED':
             self.cmd = self.package.transformation.exportstatement
             self.cmd_execute()       

      def cmd_import_table_package(self):
          sapdb  = self.options.sourcedbname
          user   = self.package.user
          table  = self.package.table

          if TRANSFORMATIONSYNTAX == 'STRATIFIED':
             #Medium
             if self.options.streammedium  == 'FILE':
                #Format
                if( self.options.streamformat == 'PAGES'   or
                    self.options.streamformat == 'RECORDS'    ) :
                    cmd = 'TABLELOAD TABLE '
                    cmd = cmd + " \"%s\".\"%s\" \n"                % (user, table)
                    cmd = cmd + " DATA INSTREAM FILE '%s' %s %s\n" % (os.path.join(self.options.sourceindepdatapath,   '%s_%s_%s.data' % (sapdb,user,table )),self.options.streamformat,self.options.streamcharencoding)
                    self.cmd = cmd
                    self.cmd_execute()
                #Format
                if( self.options.streamformat == 'DDL'        or
                    self.options.streamformat == 'FORMATTED'  or
                    self.options.streamformat == 'COMPRESSED'   ):
                    for cmd in self.package.transformation_step :
                        self.cmd = cmd
                        self.cmd_execute()
                        #grant and execute
                        if string.find(cmd,'Create') == 0 and string.find(cmd,'Table')  == 0 :
                           self.cmd= 'GRANT ALL ON %s TO PUBLIC' % table
             #Medium
             if(self.options.streammedium[0:4] == 'PIPE' ):
                #Format
                if( self.options.streamformat == 'PAGES'   or
                    self.options.streamformat == 'RECORDS') :
                    cmd      = 'TABLELOAD TABLE '
                    cmd      = cmd + " \"%s\".\"%s\" \n"               % ( user, table )
                    cmd      = cmd + " DATA INSTREAM PIPE '%s' %s %s " % ('\\\\.\\PIPE\\%s_%s_%s.dat' % ( sapdb, user, table ), self.options.streamformat, self.options.streamcharencoding)
                    self.cmd = cmd
                    self.cmd_execute()
                #Format
                if( self.options.streamformat == 'DDL'       or
                    self.options.streamformat == 'FORMATTED' or
                    self.options.streamformat == 'COMPRESSED'   ) :
                    for cmd in self.package.transformation_step :
                        self.cmd = cmd
                        self.cmd_execute()
                        #grant and execute
                        if string.find(cmd,'Create') == 0 and string.find(cmd,'Table')  == 0 :
                           self.cmd= 'GRANT ALL ON %s TO PUBLIC' % table
             self.package.transformation_step = []

          if TRANSFORMATIONSYNTAX == 'UNIFIED':
             self.cmd = self.package.transformation.importstatement
             self.cmd_execute()    
             
          if self.options.streamformat != 'PAGES' and self.options.streamformat_target == 'PAGE' :                 
             self.package.transformation.cmd_import_table_package_index()         

      def cmd_import_package(self):
          if PACKAGEEXECUTION == 'EXECUTE':
             try:
                 self.packagestart = time.time()
                 match_0     = None
                 match_1     = None
                 match_2     = None
                 match_3     = None
                 user        = None
                 packageguid = None
                 if  self.transformationtype == 'EXPORT' or self.transformationtype == 'IMPORT' :
                     protocolfile = os.path.join(LOADERPROGPATH,'sdb','loader','log',self.protocol )
                 if  os.path.isfile      ( protocolfile ) :
                     protocol      = file( protocolfile ,'r')
                     protocolline  = protocol.readline()
                     while protocolline and self.packageload  == 'FALSE' :
                           #timestamp user
                           if match_0 == None:
                              if  'TRANSPORT' == self.transformationtype or 'EXPORT' == self.transformationtype:
                                   if TESTLOADER == 'TRUE' and self.options.transformationpackage != self.options.transformationpart:
                                      user = self.package.user
                                   else:
                                      user = re.split(',',self.options.sourcedbuser)[0]
                                   match_0  = re.search ('TRANSFORMATIONPACKAGE  %s' % self.package.sourcepackagename, protocolline )
                              if  'TRANSPORT' == self.transformationtype or 'IMPORT' == self.transformationtype :
                                   if TESTLOADER == 'TRUE' and self.options.transformationpackage != self.options.transformationpart:
                                      user = self.package.user
                                   else:
                                      user = re.split(',',self.options.targetdbuser)[0]
                                   match_0  = re.search ('TRANSFORMATIONPACKAGE  %s' % self.package.targetpackagename, protocolline )
                           #user
                           if match_0 != None and match_1 == None:
                              match_1  = re.search ('USE USER %s' % user, protocolline )
                           #table
                           if match_1 != None and match_2 == None:
                              if self.options.transformationpart == 'DB' :
                                 match_2  = re.search ('PORT DB'  ,protocolline)
                              if self.options.transformationpart == 'USER' :
                                 match_2  = re.search ('PORT USER',protocolline)
                              if self.options.transformationpart == 'TABLE' :   
                                 match_2  = re.search ('PORT TABLE "%s"' % self.package.table, protocolline) 
                           if match_2 != None and match_3 == None:
                              match_3  = re.search ('IMPORT    PACKAGE x', protocolline)
                           #package
                           if match_3 != None :
                              packageguid       = re.split ( '\'', protocolline)
                              packageguid       = string.replace( packageguid[1] , "\n" , '')
                              self.packageguid  = packageguid
                              self.packageload  = 'TRUE'
                              if self.transformationtype== 'EXPORT' or self.transformationtype== 'TRANSPORT':
                                 resultset = self.monitor.sql(PACKAGEMAXSTEP_SOURCE % self.packageguid )
                              if self.transformationtype== 'IMPORT':
                                 resultset = self.monitor.sql(PACKAGEMAXSTEP_TARGET % self.packageguid )
                              if resultset != None:
                                 row = resultset.first()
                                 if row != None:
                                    self.package.maxstepid = row[0] 
                              if self.options.mode == 'INTER':
                                 print "Load      Transformation Package x'%s'" % self.packageguid
                           protocolline = protocol.readline()
                     protocol.close()
             except sql.CommunicationError, err:
                 pass
             except sql.SQLError,  err :
                 pass
             except IndexError,err:
                 pass
           
      def cmd_monitor_package(self):
          if MONITOR == 'ON':
             try:
                 if self.options.streamformat == 'PAGES' or  self.options.streamformat_target == 'PAGE':
                    pass
                 if self.options.streamformat != 'PAGES' and self.options.streamformat_target != 'PAGE' :
                    if self.options.transformationpart == 'TABLE':
                       self.table = self.package.table
                    if self.package.stepid == None:
                       if self.transformationtype== 'EXPORT' or self.transformationtype== 'TRANSPORT':
                          resultset  = self.monitor.sql(PACKAGEROWCOUNT_SOURCE  % self.packageguid )
                       if self.transformationtype== 'IMPORT':
                          resultset  = self.monitor.sql(PACKAGEROWCOUNT_TARGET  % self.packageguid )
                       if resultset != None:
                          row = resultset.first()
                          if row != None:
                             self.package.stepid = row[0] 
                             self.owner          = row[1] 
                             if self.options.transformationpart != 'TABLE':
                                self.table       = row[2]
                    if self.package.stepid != None:
                       if self.transformationtype== 'EXPORT' or self.transformationtype== 'TRANSPORT':
                          resultset  = self.monitor.sql(STEPIDROWCOUNT_SOURCE  % (self.packageguid, self.package.stepid ) )
                       if self.transformationtype== 'IMPORT':
                          resultset  = self.monitor.sql(STEPIDROWCOUNT_TARGET  % (self.packageguid, self.package.stepid ) )
                       row = resultset.first()
                       if row != None:
                          if self.package.stepid == row[0]:
                             print '\rTable     %-10s : %s  ' %( self.table, row[3]),
                       if row == None and self.package.maxstepid != self.package.stepid:
                          if self.transformationtype== 'EXPORT' or self.transformationtype== 'TRANSPORT':
                             resultset  = self.monitor.sql(STEPIDSROWCOUNT_SOURCE  % (self.packageguid, self.package.stepid ) )
                          if self.transformationtype== 'IMPORT':
                             resultset  = self.monitor.sql(STEPIDSROWCOUNT_TARGET  % (self.packageguid, self.package.stepid ) )
                          if resultset != None:
                             for row in resultset:
                                 if row != None and self.package.stepid == row[0]:
                                    print '\rTable     %-10s : %s ' %( row[2], row[3])  
                                 if row != None and self.package.stepid != row[0]:
                                    print 'Table     %-10s : %s '   %( row[2], row[3])
                          if self.package.stepid    < self.package.maxstepid:
                             self.package.maxstepid = self.package.stepid
                             self.package.stepid    = None
             except sql.CommunicationError, err:
                 self.package.tranformation.release()
                 os._exit(1)
             except sql.SQLError,  err :
                 pass
             except :
                 pass
          if MONITOR == 'OFF':
             pass
     
      def cmd_export_package(self):
          TRANSFORMATIONSYNTAX = 'STRATIFIED' 
          
          if self.packageend > 0 and ( time.time() - self.packagestart > self.packageend ):
             print 'Processing Time exceeded; stopping execution'
             self.package.tranformation.release()
             os._exit(1)
             
          if TRANSFORMATIONSYNTAX == 'UNIFIED' and PACKAGEEXECUTION == 'EXECUTE' :
             catalogstream_name = None
             datastream_name    = None
             packagestream_name = None
             catalogstream_lock = None
             datastream_lock    = None
             packagestream_lock = None
             if  'EXPORT' == self.transformationtype :
                 catalogstream_name    = string.replace(self.catalogstream_name,"'","")
                 datastream_name       = string.replace(self.datastream_name   ,"'","")
                 packagestream_name    = string.replace(self.packagestream_name,"'","")
             if  'IMPORT' == self.transformationtype :
                 catalogstream_name    = string.replace(self.catalogstream_name,"'","")
                 datastream_name       = string.replace(self.datastream_name   ,"'","")
                 packagestream_name    = string.replace(self.packagestream_name,"'","")
                 catalogstream_name    = string.replace(catalogstream_name, self.options.sourcedbname, self.options.targetdbname)
                 datastream_name       = string.replace(datastream_name   , self.options.sourcedbname, self.options.targetdbname)
                 packagestream_name    = string.replace(packagestream_name, self.options.sourcedbname, self.options.targetdbname)
             try:
                if self.options.streamformat == 'DDL' or self.options.streamformat == 'XML':
                   if os.path.isfile           (catalogstream_name) :
                      catalogstream = os.rename(catalogstream_name, catalogstream_name)
                      catalogstream_lock = 'release'
                      datastream_lock    = 'release' 
                      packagestream_lock = 'release'
                if(self.options.streamformat == 'COMPRESSED' or self.options.streamformat == 'FORMATTED' or
                   self.options.streamformat == 'RECORDS'    or self.options.streamformat == 'PAGES'    ):
                   if os.path.isfile           (catalogstream_name) :
                      catalogstream = os.rename(catalogstream_name, catalogstream_name) 
                      catalogstream_lock = 'release'
                   if os.path.isfile        (datastream_name) : 
                      datastream = os.rename(datastream_name, datastream_name)
                      catalogstream_lock = 'release' 
                      datastream_lock    = 'release' 
                   if os.path.isfile        ( string.replace(datastream_name,'.DATA','.DATA0000')) : 
                      datastream = os.rename( string.replace(datastream_name,'.DATA','.DATA0000') ,string.replace(datastream_name,'.DATA','.DATA0000'))
                      catalogstream_lock = 'release' 
                      datastream_lock    = 'release'
                   if os.path.isfile           (packagestream_name) :      
                      packagestream = os.rename(packagestream_name, packagestream_name)
                      catalogstream_lock = 'release' 
                      datastream_lock    = 'release' 
                      packagestream_lock = 'release'
                if( catalogstream_lock == 'release' and 
                    datastream_lock    == 'release' and 
                    packagestream_lock == 'release'    ):
                    self.packageunload  = 'TRUE'
                    if  DB_DEFAULTCODE_SOURCE == 'ASCII' and DB_DEFAULTCODE_TARGET == 'ASCII' and MONITOR == 'ON':
                        try:
                           packagefilehandle = file (packagestream_name )
                           resultset   = csv.reader(packagefilehandle, delimiter  = '|')
                           if resultset != None:
                              print 'Transformation package rows   :'
                              for row in resultset:
                                  if MONITOR == 'ON':
                                     if self.package.maxstepid < string.atoi( row[12], 10 ) :
                                        print 'Table     %-10s : %s ' % (row[1], row[33])
                                  if MONITOR == 'OFF':
                                     print    'Table     %-10s : %s ' % (row[1], row[33])
                        except:
                          pass
                    if not ( 'EXPORT'  == self.transformationtype and 'PIPE'  == self.streammedium ):
                       if  mutex.locked()  == True :
                           mutex.release()           #release
             except OSError, (errno, err ):
                catalogstream_lock = None
                datastream_lock    = None
                packagestream_lock = None
                pass
                
          if TRANSFORMATIONSYNTAX == 'STRATIFIED' and PACKAGEEXECUTION == 'EXECUTE':
             match_0      = None
             match_1      = None
             match_2      = None
             match_3      = None
             match_4      = None
             packageguid  = None
             user         = None
             try:
                 if  'EXPORT' == self.transformationtype  or 'IMPORT' == self.transformationtype :
                     protocolfile = os.path.join(LOADERPROGPATH,'sdb','loader','log',self.protocol )
                 if  os.path.isfile  ( protocolfile ) :
                     protocol  = file( protocolfile ,'r')
                     protocolline  = protocol.readline()
                     counter       = 0
                     while protocolline and self.packageunload == 'FALSE':
                           #timestamp
                           if match_0 == None:
                              if  'TRANSPORT' == self.transformationtype or 'EXPORT' == self.transformationtype:
                                  if TESTLOADER == 'TRUE' and self.options.transformationpackage != self.options.transformationpart:
                                     user = self.package.user
                                  else:
                                     user = re.split(',',self.options.sourcedbuser)[0]
                                  match_0  = re.search ('TRANSFORMATIONPACKAGE  %s' % self.package.sourcepackagename, protocolline )
                              if  'TRANSPORT' == self.transformationtype or 'IMPORT' == self.transformationtype :
                                  if TESTLOADER == 'TRUE' and self.options.transformationpackage != self.options.transformationpart:
                                     user = self.package.user
                                  else:
                                     user = re.split(',',self.options.targetdbuser)[0]
                                  match_0  = re.search ('TRANSFORMATIONPACKAGE  %s' % self.package.targetpackagename, protocolline )
                           #user                                  
                           if match_0 != None and match_1 == None:
                              match_1  = re.search ('USE USER %s' % user, protocolline )
                           #part
                           if match_1 != None and match_2 == None :
                              match_2  = re.search ("IMPORT    PACKAGE x'%s'" % self.packageguid , protocolline)
                           if match_2 != None and match_3 == None :
                              #monitor
                              if self.options.transformationpart != 'TABLE' :
                                 match_3 = 'ok'
                              if self.options.transformationpart == 'TABLE' :
                                 if self.options.streamformat    == 'PAGES':
                                    match_3 = 'ok'
                                 if self.options.streamformat    == 'RECORDS' or self.options.streamformat == 'COMPRESSED' or self.options.streamformat == 'FORMATTED' :
                                    match_3  = re.search ("Sum of " , protocolline)
                                    if match_3 != None:
                                       packagerows        = re.split (":",protocolline)[1]
                                       packagerows        = re.split (",",packagerows )[0]
                                       print '\rTable     %-10s : %s  \n' %( self.package.table, packagerows),
                                 if self.options.streamformat    == 'DDL' or self.options.streamformat  == 'XML' or self.options.streamformat_target == 'DTL':
                                    match_3 = 'ok'
                           if match_3 != None and match_4 == None :
                              #package
                              if 'EXPORT'  == self.transformationtype and 'TAPE'  == self.streammedium :
                                  #format
                                  if self.streamformat    == 'DDL' :
                                     if protocolline[0:8] == '// M  NST':
                                          if counter == 0:
                                             ddlpipe_ebid = re.split  ('NST',protocolline)
                                             ddlpipe_ebid = string.replace(ddlpipe_ebid[1], "\n" , '')
                                             self.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                                  if self.streamformat    ==  'RECORDS' or self.streamformat ==  'PAGES':
                                     if protocolline[0:8] == '// M  NST':
                                        if counter == 0:
                                           ddlpipe_ebid = re.split  ('NST',protocolline)
                                           ddlpipe_ebid = string.replace(ddlpipe_ebid[1], "\n" , '')
                                           self.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                                        if counter == 1:
                                           datpipe_ebid = re.split  ('NST',protocolline)
                                           datpipe_ebid = string.replace(datpipe_ebid[1], "\n" , '')
                                           self.datpipe_ebid = "\'NST%s\'" % datpipe_ebid
                                        if counter == 2:
                                           mappipe_ebid = re.split  ('NST',protocolline)
                                           mappipe_ebid = string.replace(mappipe_ebid[1], "\n" , '')
                                           self.mappipe_ebid = "\'NST%s\'" % mappipe_ebid
                                        counter  = counter + 1
                              match_4  = re.search ("EXPORT    PACKAGE x'%s'" % self.packageguid , protocolline)
                           #rows
                           if match_4 != None :
                              self.packageunload = 'TRUE'
                              #monitor
                              package = None
                              if self.options.transformationtype == 'EXPORT' or self.options.transformationtype == 'TRANSPORT':
                                 if self.options.transformationpart == 'DB' :
                                    package = os.path.join(self.options.sourceindepdatapath,'%s.package' % self.options.sourcedbname)
                                 if self.options.transformationpart == 'USER' :
                                    package = os.path.join(self.options.sourceindepdatapath,'%s.package' % re.split(',', self.options.sourcedbuser)[0])
                              if self.options.transformationtype == 'IMPORT' or self.options.transformationtype == 'TRANSPORT':
                                 if self.options.transformationpart == 'DB' :
                                    package = os.path.join(self.options.targetindepdatapath,'%s.package' % self.options.targetdbname)
                                 if self.options.transformationpart == 'USER' :
                                    package = os.path.join(self.options.targetindepdatapath,'%s.package' % re.split(',', self.options.targetdbuser)[0])
                              if((self.options.transformationpart == 'DB'    or  self.options.transformationpart == 'USER') and self.options.streamformat != 'PAGES' and
                                  DB_DEFAULTCODE_SOURCE == 'ASCII' and DB_DEFAULTCODE_TARGET == 'ASCII' and MONITOR == 'ON' ) :
                                 if os.path.isfile(package) :
                                    packagefilehandle = file (package )
                                    resultset   = csv.reader(packagefilehandle, delimiter  = '|')
                                    if resultset != None:
                                       print 'Transformation package rows   :'
                                       for row in resultset:
                                           if MONITOR == 'ON':
                                              if self.package.maxstepid < string.atoi( row[12], 10 ) :
                                                 print 'Table     %-10s : %s ' % (row[1], row[33])
                                           if MONITOR == 'OFF':
                                              print    'Table     %-10s : %s ' % (row[1], row[33])
                              #mutex
                              if not ( 'EXPORT'  == self.transformationtype and 'PIPE'  == self.streammedium ):
                                 if  mutex.locked()  == True :
                                     mutex.release()           #release
          
                           protocolline = protocol.readline()
                     protocol.close()
             except IOError,    (errno, err ):
                pass
                
             TRANSFORMATIONSYNTAX = 'UNIFIED'

      def cmd_restart_package(self):
          restartRecord = []
          if self.initposition == 'RESTART' :
             statement     =  " SELECT DISTINCT PACKAGEGUID, OWNER, TABLENAME, \n DATASTREAMHEADERBLOCK, DATASTREAMHEADERPOS, \n DATASTREAMDATABLOCK, DATASTREAMDATAPOS, ROWCOUNT \n"
             statement     =  statement + "FROM SYSDBA.TRANSFORMATIONMODEL "
             statement     =  statement + "WHERE RESTART = TRUE "

             result = self.monitor.sql (statement )
             row    = result.next ()
             if row :
                packageguid = ('%02X' * 24) % tuple (map (ord, row[0] ))
                restartRecord.append (packageguid)
                restartRecord.append (row[1])
                restartRecord.append (row[2])
                restartRecord.append (row[3])
                restartRecord.append (row[4])
                restartRecord.append (row[5])
                restartRecord.append (row[6])
                restartRecord.append (row[7])

          return restartRecord

      def cmd_restart_sourcedb_package(self, targetRecord ):
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'RECORDS' and targetRecord <> [] :
              sourceRecord  = self.cmd_restart_package()
              statement     = " UPDATE SYSDBA.TRANSFORMATIONMODEL SET \n"
              statement     = statement + " DATASTREAMHEADERBLOCK = %s, DATASTREAMHEADERPOS=%s, \n DATASTREAMDATABLOCK=%s,DATASTREAMDATAPOS=%s,\n ROWCOUNT=%s " % ( targetRecord[3] , targetRecord[4], targetRecord[5], targetRecord[6], targetRecord[7] )
              statement     = statement + " WHERE PACKAGEGUID= x'%s' \n AND OWNER = '%s' AND TABLENAME = '%s' AND RESTART = TRUE \n"                            % ( sourceRecord[0], targetRecord[1], targetRecord[2])
              print  statement
              self.monitor.sql (statement )
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'PAGES'   and targetRecord <> [] :
              pass
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'RECORDS' and targetRecord <> [] :
              pass
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'PAGES'   and targetRecord <> [] :
              pass

      def cmd_restart_targetdb_package(self):
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'RECORDS':
              restartRecord = self.cmd_restart_package()
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'PAGES'  :
              restartRecord = self.cmd_restart_package()

              statement     = " UPDATE SYSDBA.TRANSFORMATIONMODEL SET \n"
              statement     = statement + " DATASTREAMHEADERBLOCK = 0, DATASTREAMHEADERPOS=0, \n DATASTREAMDATABLOCK=0,DATASTREAMDATAPOS=0,\n ROWCOUNT=0, RESTART = TRUE \n"
              statement     = statement + " WHERE PACKAGEGUID= x'%s' \n AND OWNER = '%s' AND TABLENAME = '%s' \n"  % (restartRecord[0], restartRecord[1], restartRecord[2])
              print statement
              result        = self.monitor.sql (statement)
              statement     = " DELETE %s.%s \n"  % (restartRecord[1], restartRecord[2] )
              print statement
              result        = self.monitor.sql (statement)
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'RECORDS':
              restartRecord = self.cmd_restart_package()
          #Medium Format Start
          if  self.initposition == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'PAGES'  :
              restartRecord = self.cmd_restart_package()

          return restartRecord
          
#class StreamTransformator 

class StreamConnector :
      def __init__(self, source_loader = None, target_loader = None ) :
          self.exporter   = source_loader
          self.importer   = target_loader
          if self.exporter != None:
             self.user           = self.exporter.package.user
             self.table          = self.exporter.package.table
          if self.importer != None:
             self.user           = self.importer.package.user
             self.table          = self.importer.package.table
          self.longcolumn     = ''
          self.sourceddltime  = None
          self.sourcedattime  = None
          self.ddlpipe_exist  = 1
          self.datpipe_exist  = 1

      def cmd_datpipe2datfile(self):
          datfile = os.path.join (self.exporter.sapdbroot, '%s_%s.dat'         % ( self.exporter.package.user, self.exporter.package.table))
          datpipe2file    = "pipe2file -d pipe2file -p %s           -f %s"         % ( self.exporter.datpipe, datfile )
          try:
             print 'Start     StreamConnector to Connect \n FILE %s to\n PIPE %s ' % ( self.exporter.datpipe, datfile )
             os.system( datpipe2file )
          except OSError, err:
             print 'command failed:', err

      def cmd_longpipe2longfiles(self):
          for longcolumn in self.exporter.package.column_long:
              self.longcolumn = longcolumn
              thread.start_new_thread( self.cmd_longpipe2longfile,())
              time.sleep(1)

      def cmd_longpipe2longfile(self):
          longpipe  = '\\\\.\\PIPE\\%s_%s_%s_%s.long'                                                                                            % (self.exporter.package.sapdb, self.user ,self.table, self.longcolumn )
          longfile  = os.path.join (self.exporter.sapdbroot, '%s_%s_%s.long' % (self.user, self.table, self.longcolumn ))
          longpipe2file = "pipe2file -d pipe2file -p %s -f %s" % ( longpipe , longfile )
          try:
              print 'Start     StreamConnector to Connect \n FILE %s to\n PIPE %s ' % ( longfile, longpipe )
              os.system( longpipe2file )
          except OSError, err:
              print 'command failed:', err

      def cmd_datfile2datpipe(self):
          print '\nStart     StreamConnector to Connect\n FILE %s to\n PIPE %s '    % ( self.exporter.datfile, self.exporter.datpipe )
          datfile2pipe    = "pipe2file -d file2pipe -p %s           -f %s"          % ( self.exporter.datpipe, self.exporter.datfile )
          try:
             os.system( datfile2pipe )
          except OSError, err:
             print 'command failed:', err

      def cmd_pipe2tape(self):
          counter = 0
          protocolfile = os.path.join(self.exporter.sapdbroot, 'wrk',self.exporter.protocol)
          if  os.path.isfile     ( protocolfile ) :
              protocol     = file( protocolfile ,'r')

              if self.exporter.streamformat <> 'DDL' :
                 protocolline = protocol.readline()
                 while protocolline :
                     if ( protocolline[0:8] == '// M    NST'):
                          if counter == 0:
                             ddlpipe_ebid = re.split  ('NST',protocolline)
                             ddlpipe_ebid = string.replace(ddlpipe_ebid[1] , "\n" , '')
                             self.importer.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                          if counter == 1:
                             datpipe_ebid = re.split  ('NST',protocolline)
                             datpipe_ebid = string.replace(datpipe_ebid[1] , "\n" , '')
                             self.importer.datpipe_ebid = "\'NST%s\'" % datpipe_ebid
                          counter  = counter + 1
                     protocolline = protocol.readline()
                 if counter == 2:
                    if mutex.locked() == True:
                       mutex.release() #release

              if self.exporter.streamformat == 'DDL' :
                 protocolline = protocol.readline()
                 while protocolline :
                     if ( protocolline[0:8] == '// M    NST'):
                          if counter == 0:
                             ddlpipe_ebid = re.split  ('NST',protocolline)
                             ddlpipe_ebid = string.replace(ddlpipe_ebid[1] , "\n" , '')
                             self.importer.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                          counter  = counter + 1
                     protocolline = protocol.readline()
                 if counter == 1:
                    if mutex.locked() == True:
                       mutex.release() #release

      def cmd_ddlpipe2ddltape(self):
          print '\nStart     StreamConnector to Connect \n TAPE to\n PIPE %s ' % ( self.exporter.ddlpipe_source )
          ddltapesave  = self.exporter.sapdbroot + "\\wrk\\" + self.exporter.sapdb + ".ddltape_save"
          ddlpipe2tape = "\nsave    -s %s -N %s -l full -LL %s 2> %s"          % ( self.exporter.sapdbnode, self.exporter.sapdb,
                                                                                   self.exporter.ddlpipe_source, ddltapesave )
          try:
             os.system( ddlpipe2tape )
          except OSError, err:
             print 'command failed:', err
          if  os.path.isfile  ( ddltapesave ) :
              tapefile  = file( ddltapesave ,'r')
              tapeline  = tapefile.readline()
              while tapeline  :
                  if ( tapeline[0:19] == 'completed savetime='):
                       savetime = re.split ('=',tapeline)
                       savetime = string.replace( savetime[1] , "\n" , '')
                       self.sourceddltime = savetime
                  tapeline = tapefile.readline()
              tapefile.close()

      def cmd_ddltape2ddlpipe(self):
          print '\nStart     StreamConnector to Connect \n TAPE %s to\n PIPE %s' % ( self.sourceddltime, self.exporter.ddlpipe_source )
          ddltape2pipe = "\nrecover -q -s %s -a -t %s -c %s -iY %s "             % ( self.exporter.sapdbnode, self.sourceddltime,
                                                                                     self.exporter.sapdbnode, self.exporter.ddlpipe_source )
          try:
              os.system( ddltape2pipe )
          except OSError, err:
              print 'command failed:', err

      def cmd_datpipe2dattape(self):
          print 'Start     StreamConnector to Connect \n TAPE to\n PIPE %s ' %  self.exporter.datpipe_source
          dattapesave  = self.exporter.sapdbroot + "\\wrk\\" + self.exporter.sapdb + ".dattape_save"
          datpipe2tape = "save    -s %s -N %s -l full -LL %s           2> %s"% ( self.exporter.sapdbnode, self.exporter.sapdb,
                                                                                 self.exporter.datpipe_source , dattapesave  )
          try:
             os.system( datpipe2tape )
          except OSError, err:
             print 'command failed:', err
          if  os.path.isfile  ( dattapesave ) :
              tapefile  = file( dattapesave ,'r')
              tapeline  = tapefile.readline()
              while tapeline :
                  if ( tapeline[0:19] == 'completed savetime='):
                       savetime = re.split ('=',tapeline)
                       savetime = string.replace( savetime[1] , "\n" , '')
                       self.sourcedattime = savetime
                  tapeline = tapefile.readline()
              tapefile.close()

      def cmd_dattape2datpipe(self):
          print '\nStart     StreamConnector to Connect\n TAPE %s to\n PIPE %s ' % ( self.sourcedattime, self.exporter.datpipe_source )
          dattape2pipe = "\nrecover -q -s %s -a -t %s -c %s -iY %s       "       % ( self.exporter.sapdbnode, self.sourcedattime,
                                                                                     self.exporter.sapdbnode, self.exporter.datpipe_source )
          try:
              os.system( dattape2pipe )
          except OSError, err:
              print 'command failed:', err

class StreamFilter :
      def __init__(self, options = None, transformation = None ) :
          self.options        = options
          self.transformation = transformation
          self.ddlerror       = 1
          self.dtlerror       = 1
          self.daterror       = 1

      def cmd_dtlsource2dtltarget(self):
          dtlsource    = None
          if self.options.streammedium == 'FILE':
             dtlsource = os.path.join(self.options.sourceindepdatapath,  '%s_%s_%s.dtl'  % (self.options.sourcedbname,re.split(',',self.options.sourcedbuser)[0],self.transformation.package.table ))
          if self.options.streammedium == 'PIPE':
             dtlsource = '\\\\.\\PIPE\\%s_%s_%s.dtl'  % (self.exporter.sapdb,self.exporter.user,self.exporter.package.table )
          try :
             self.dtlerror   = 0
             step            = ''
             dtlsourcestream = file(dtlsource, 'r' )
             sourcedtl       = dtlsourcestream.readline()
             self.importer.package.transformation_step = []
             while sourcedtl :
                   targetdtl = string.strip(sourcedtl)
                   if ( targetdtl[0:2] <> "//" ) and targetdtl <> '' :
                        string.replace(targetdtl, 'ASCII', self.options.streamcharencoding)
                        step = step + targetdtl + "\n"
                   if ( targetdtl[0:2] == "//" or targetdtl == '' )  and step <> '' and step <> ' ':
                        self.importer.package.transformation_step.append(step)
                        step= ''
                   sourcedtl = dtlsourcestream.readline()
             dtlsourcestream.close()
          except IOError, (errno, err ):
             self.dtlerror  = 1

      def cmd_ddlsource2ddltarget(self):
          try :
              self.ddlerror   = 0
              ddlpipesource   = file(self.transformation.exporter.ddlpipe_source, 'rt' )
              ddlpipetarget   = file(self.transformation.importer.ddlpipe_target, 'wt' )
              sourceddl       = ddlpipesource.readline()
              while sourceddl :
                    targetddl = string.replace(sourceddl, '%s' % string.upper(self.transformation.exporter.catalog),
                                                          '%s' % string.upper(self.transformation.importer.catalog) )
                    ddlpipetarget.write(targetddl)
                    sourceddl = ddlpipesource.readline()
              ddlpipesource.close()
              ddlpipetarget.close()
          except IOError, (errno, err ):
              self.ddlerror  = 1
              print 'command failed:', errno, err

      def cmd_datsource2dattarget_bin(self):
          try :
             self.daterror   = 0
             datpipesource   = file(self.exporter.datpipe_source, 'rb')
             datpipetarget   = file(self.exporter.datpipe_target, 'wb')
             sourcedat       = datpipesource.read(400)
             while sourcedat :
                   targetdat = sourcedat
                   datpipetarget.write(targetdat)
             sourcedat = datpipesource.read(400)
             datpipesource.close()
             datpipetarget.close()
          except IOError, (errno, err ):
             print 'command failed:', errno, err
             self.daterror  = 1

      def cmd_datsource2dattarget_txt(self):
          try :
             self.daterror   = 0
             datpipesource   = file(self.exporter.datpipe_source, 'rt')
             datpipetarget   = file(self.exporter.datpipe_target, 'wt')
             sourcedat       = datpipesource.readline()
             while sourcedat :
                   targetdat = sourcedat
                   datpipetarget.write(targetdat)
             sourcedat = datpipesource.read()
             datpipesource.close()
             datpipetarget.close()
          except IOError, (errno, err ):
             print 'command failed: %s, %s \n' %( errno, err )
             self.daterror  = 1
             
def  design_package(options):
     global LOADERPROGPATH
     global LOADERDATAPATH
     global MONITOR
     global PACKAGEARCHIVE
     global DB_SCHEMAS_SOURCE
     global DB_SCHEMAS_TARGET
     rc   = True
     # Transformation Package Version
     
     loaderdatapath = ''
     exportmonitor  = None
     importmonitor  = None
     username       = ''
     userpassword   = ''
     table          = None
     input          = None 
     try:
        global STEPID
        input = '' 
        if options.transformationtype in ['']:
           STEPID = STEPID+1
           print "\nStep %i: What do you want to do" % STEPID
        while options.transformationtype in [ '' ]:
              input = raw_input(""" [1] Transport from a source database into a target database\n"""
                                """ [2] Export    from a source database\n"""
                                """ [3] Import    into a target database\n """
                                """ \nStep %i: """ %STEPID )
              if input == '1':
                 options.transformationtype = 'TRANSPORT'
              if input == '2':
                 options.transformationtype = 'EXPORT'
              if input == '3':
                 options.transformationtype = 'IMPORT'
        input = '' 
        if options.mode == 'INTER' and PACKAGEPATH != 'HOMEDIR' :
           STEPID = STEPID+1
           print "\nStep %i: Choose the transformation package path: " % STEPID
           print " Transformation package path: %s " % LOADERDATAPATH
           while loaderdatapath in ['']:
                 input = raw_input(' [1] DEFAULT \n [2] CHANGE DEFAULT \n\nStep %i: ' % STEPID)
                 if input == '1':
                    loaderdatapath = LOADERDATAPATH
                 if input == '2':
                    input = None
                    while input in [None]:
                          input = raw_input(' transformation package path: ' )
                          if input != ''    and os.path.isdir  (input):
                             LOADERDATAPATH   = os.path.abspath(input)    
                             loaderdatapath   = LOADERDATAPATH
        else:
           loaderdatapath = LOADERDATAPATH
        input = ''
        if PACKAGENAME != 'TIMESTAMP' and options.mode == 'INTER' :
           if options.packagename in ['']:                  
              STEPID = STEPID+1
              print "\nStep %i: Enter the transformation package name: " % STEPID
              print " Transformation package name: %s " % PACKAGETIMESTAMP
           while options.packagename in ['']:
                 input = raw_input(' [1] DEFAULT \n [2] CHANGE DEFAULT \n\nStep %i: ' % STEPID)
                 if input == '1':
                    options.packagename = PACKAGETIMESTAMP
                 if input == '2':
                    input = ''
                    while input in ['']:
                          input = raw_input(' Transformation package name: ' )
                          if input != '' :
                             options.packagename = input
        elif options.packagename != '' and options.mode == 'PARAM' :
             options.packagename = options.packagename
        elif options.packagename == '':
             options.packagename = PACKAGETIMESTAMP
        input = '' 
        username     = ''
        userpassword = ''
        if options.sourcedbuser not in [''] :
           try:
              username     = re.split(',',options.sourcedbuser)[0]
              userpassword = re.split(',',options.sourcedbuser)[1]
              if userpassword == '*':
                 userpassword  = ''
           except:
              pass
        #DATA SOURCE  DATABASE
        if( options.transformationtype == 'TRANSPORT' or
            options.transformationtype == 'EXPORT'      ):
            if(options.sourcedbnode   in [''] or 
               options.sourcedbname   in [''] or 
               options.sourcedbschema in [''] or 
               username               in [''] or
               userpassword           in ['']   ):
               STEPID = STEPID+1
               print "\nStep %i: Enter the source database connection " % STEPID 
               if options.sourcedbnode   not in [''] :
                  print 'Database computer: %s' % options.sourcedbnode
               if options.sourcedbname   not in [''] :
                  print 'Database name    : %s' % options.sourcedbname
               if username               not in [''] :
                  print 'Database user    : %s' % username
               if options.sourcedbschema not in ['']:
                  print 'Database schema  : %s' % options.sourcedbschema 
            while exportmonitor == None:
                  while options.sourcedbnode in ['']:   
                        input    = raw_input('Database computer: ')
                        if input == '':
                           options.sourcedbnode = socket.gethostname()
                        if input != '':
                           options.sourcedbnode = input
                  while options.sourcedbname in ['']:   
                        input    = raw_input('Database name    : ') 
                        options.sourcedbname = string.upper(input)
                  while username in ['']:  
                        username = raw_input('Database user    : ')   
                  while userpassword in ['']:   
                        userpassword   = getpass.getpass()
                  options.sourcedbuser = '%s,%s' %( username, userpassword )
                  if options.sourcedbuser not in ['']:
                     username     = re.split(',',options.sourcedbuser)[0]
                     userpassword = re.split(',',options.sourcedbuser)[1]
                  if username != '' and username [0] != '"' and username[len(username)-1] != '"':
                     username  = string.upper(username)
                  elif username != '' :
                     username = string.replace(username,'"','')  
                  options.sourcedbuser = '%s,%s' %( username, userpassword )  
                  try:
                      exportmonitor = sql.connect( username, userpassword, options.sourcedbname, options.sourcedbnode )
                  except (sql.CommunicationError ) , err:
                      print 'Connect failed, try again:' , err
                      exportmonitor    = None 
                  except (sql.SQLError ) , err:  
                      print 'Connect failed, try again:' , err    
                      exportmonitor    = None 
                  if exportmonitor    == None:
                     options.sourcedbnode = ''
                     options.sourcedbname = ''
                     options.sourcedbuser = ''
                     username     = ''
                     userpassword = ''
            #DATA PATH
            try:
                options.sourceindepdatapath = os.path.join(LOADERDATAPATH, 'sdb','loader', 'packages', 'export', options.sourcedbnode, options.sourcedbname, options.packagename )
                if os.path.isdir(options.sourceindepdatapath) == False:
                   os.makedirs  (options.sourceindepdatapath)
            except EnvironmentError, err:
                print err
        #DATA SOURCE  PACKAGE
        if( options.transformationtype == 'IMPORT' ):
            if(options.sourcedbnode    == '' or  
               options.sourcedbname    == '' ):
               STEPID = STEPID+1
               print "\nStep %i: Enter the computer and database name of the exported data" % STEPID 
            count = 0
            exportpackage = None
            while exportpackage == None and count < 4:
                  input = ''
                  while options.sourcedbnode in ['']:   
                        input = raw_input('Database computer: ')
                        if input == '':
                           input  = socket.gethostname()
                        if os.path.isdir(os.path.join(LOADERDATAPATH, 'sdb','loader', 'packages', 'export', input)) == True:
                           options.sourcedbnode = input
                        else:
                           print '%s does not exist as datasource' % ( input )
                           options.sourcedbnode = ''
                           input = '' 
                  while options.sourcedbname in ['']:   
                        input = raw_input('Database name    : ') 
                        if os.path.isdir(os.path.join(LOADERDATAPATH, 'sdb','loader', 'packages', 'export', options.sourcedbnode, input)) == True:
                           options.sourcedbname = string.upper(input) 
                        else:
                           print '%s does not exist as datasource' % ( input )
                           options.sourcedbname = ''
                           input = ''
                  if os.path.isdir(os.path.join(LOADERDATAPATH, 'sdb','loader', 'packages', 'export', options.sourcedbnode, options.sourcedbname)) == True:
                     options.sourceindepdatapath = os.path.join(LOADERDATAPATH, 'sdb','loader', 'packages', 'export', options.sourcedbnode, options.sourcedbname)
                  #DATA PATH
                  if options.transformationpackage  == '':
                     STEPID = STEPID+1
                     print "\nStep %i: Enter the packagename of the exported data" % STEPID
                     packages = []
                     # PACKAGECOLLECTION 
                     packagefile        = os.path.join(LOADERDATAPATH,'sdb', 'loader', 'packages', 'transport' + os.extsep + 'log')    
                     packagefilehandle  = file(packagefile,'r')  
                     packagecollection  = csv.reader(packagefilehandle, delimiter  = '|'  )
                     for package in packagecollection :
                         if( 'EXECUTE'  ==  string.strip(package[2]) and
                             'IMPORT'   !=  string.strip(package[3]) and
                             'FILE'     ==  string.strip(package[7]) ):
                            packages.append(string.strip(package[1]))
                     input = ''
                     # PACKAGE
                     packagedict  = dict([(packages.index(package)+1,package) for package in packages])  
                     index = 0
                     for package in packagedict:
                         index = index + 1
                         print ' [%i]:  %s ' % (index, packagedict[package])
                     while input == '' or string.atoi(input,10) < 0 or string.atoi(input,10) > len (packagedict):
                           input  =  raw_input("\nStep %i: "  % STEPID )
                     if   PACKAGEARCHIVE == 'ZIP':
                          options.sourceindepdatapath = string.replace(os.path.join(options.sourceindepdatapath,packagedict[string.atoi(input,10)]),os.extsep+'zip','')
                     elif PACKAGEARCHIVE == 'TAR':
                          options.sourceindepdatapath = string.replace(os.path.join(options.sourceindepdatapath,packagedict[string.atoi(input,10)]),os.extsep+'tgz','')
                     else:
                          options.sourceindepdatapath = os.path.join(options.sourceindepdatapath,packagedict[string.atoi(input,10)])
                  elif options.transformationpackage  != '':
                     if PACKAGEARCHIVE == 'ZIP':
                        options.sourceindepdatapath = os.path.join(options.sourceindepdatapath, options.transformationpackage)
                     elif PACKAGEARCHIVE == 'TAR':
                        options.sourceindepdatapath = os.path.join(options.sourceindepdatapath, options.transformationpackage)
                     else:
                        options.sourceindepdatapath = os.path.join(options.sourceindepdatapath, options.transformationpackage)
                  try:
                     if os.path.isdir(options.sourceindepdatapath) == False:
                        print options.sourceindepdatapath
                        os.makedirs  (options.sourceindepdatapath)
                  except OSError, err:
                     print err
                  # PACKAGE PACKAGEARCHIVE
                  if PACKAGEARCHIVE == 'ZIP':
                     try:
                         zipdirname  = os.path.split(options.sourceindepdatapath)[0]
                         zipfilename = os.path.split(options.sourceindepdatapath)[1]+ os.extsep + 'zip'
                         zip =  zipfile.ZipFile(os.path.join (zipdirname,zipfilename), 'r', zipfile.ZIP_DEFLATED )
                         if zip.testzip() == None:
                            os.chdir(options.sourceindepdatapath)
                            for filename in zip.namelist():
                                unzip        = file (filename,'wb')
                                unzip.write(zip.read(filename))
                                unzip.close()
                            zip.close()
                            os.chdir(os.path.split(options.sourceindepdatapath)[0])
                         else:
                            print 'Export Package: %s not Ok' % os.path.split(options.sourceindepdatapath)[1]
                     except IOError, err:
                        print 'Error extracting the package zip archive : \n %s, %s ' % ( err, zipfilename )
                     except:
                        kind, val, traceback = sys.exc_info()
                        raise kind, val, traceback
                  elif PACKAGEARCHIVE == 'TAR':
                     try:
                         zipdirname  = os.path.split(options.sourceindepdatapath)[0]
                         zipfilename = os.path.split(options.sourceindepdatapath)[1]+ os.extsep + 'tgz'
                         zip =  tarfile.TarFile.open(os.path.join (zipdirname,zipfilename), 'r:gz' )
                         if tarfile.is_tarfile(os.path.join (zipdirname,zipfilename)) == True:
                            os.chdir(options.sourceindepdatapath)
                            for filename in zip.getnames():
                                zip.extract(filename,options.sourceindepdatapath)
                            zip.close()
                            os.chdir(os.path.split(options.sourceindepdatapath)[0])
                         else:
                            print 'Export Package: %s not Ok' % os.path.split(options.sourceindepdatapath)[1]
                     except IOError, err:
                        print 'Error extracting the package tar archive : \n %s, %s ' % ( err, zipfilename )
                     except:
                        kind, val, traceback = sys.exc_info()
                        raise kind, val, traceback
                  else:
                     os.chdir(os.path.split(options.sourceindepdatapath)[0])
                  # PACKAGE PARAMETER
                  found = False
                  for   package in os.listdir(options.sourceindepdatapath) :
                        if os.path.isfile(os.path.join(options.sourceindepdatapath,package)) and (string.find(package,'.par') > 0 ) and found == False:
                           global DB_DEFAULTCODE_SOURCE
                           global DB_DEFAULTCODE_TARGET
                           found = True
                           parameters = codecs.open ( os.path.join(options.sourceindepdatapath, '%s' % package ) , 'r')
                           parameter  = parameters.readline()
                           #Transformation Attributes
                           options.transporttype          = string.replace(string.replace(re.split('-',parameter) [1] ,'T ',''),' ','')
                           transformationpackage          = string.replace(string.replace(re.split('-',parameter) [3] ,'p ',''),' ','')
                           options.transformationpackage  = string.strip(re.split(':', transformationpackage) [0])
                           options.transformationpart     = string.replace(string.replace(re.split('-',parameter) [4] ,'P ',''),' ','')
                           options.initposition           = string.replace(string.replace(re.split('-',parameter) [5] ,'i ',''),' ','')
                           options.streammedium           = string.replace(string.replace(re.split('-',parameter) [6] ,'m ',''),' ','')
                           options.streamformat           = string.replace(string.replace(re.split('-',parameter) [7] ,'f ',''),' ','')
                           options.streamformat_target    = string.replace(string.replace(re.split('-',parameter) [8] ,'F ',''),' ','')
                           options.streamcharencoding     = string.replace(string.replace(re.split('-',parameter) [9], 'c ',''),' ','')
                           options.streamlob              = string.replace(string.replace(re.split('-',parameter) [10],'l ',''),' ','')
                           #Transformation Resources
                           options.sourcedbnode           = string.replace(string.replace(re.split('-',parameter) [11],'nSRC ',''),' ','')
                           options.sourcedbname           = string.replace(string.replace(re.split('-',parameter) [12],'dSRC ',''),' ','')
                           options.sourcedbuser           = string.replace(string.replace(re.split('-',parameter) [13],'uSRC ',''),' ','')
                           options.sourcedbschema         = string.replace(string.replace(re.split('-',parameter) [14],'sSRC ',''),' ','')
                        if found == True:                              
                           exportpackage = os.path.join(options.sourceindepdatapath,package)
                  if found == False:
                     count  = count + 1
                     exportpackage = None
                     print ' Package % s not found' % options.transformationpackage
        #DATA TARGET DATABASE
        username     = ''
        userpassword = ''
        if options.targetdbuser not in [''] :
           try:
              username     = re.split(',',options.targetdbuser)[0]
              userpassword = re.split(',',options.targetdbuser)[1]
              if userpassword == '*':
                 userpassword  = ''
           except:
              pass
        if( options.transformationtype == 'TRANSPORT' or
            options.transformationtype == 'IMPORT'      ): 
            if(options.targetdbnode    in [''] or 
               options.targetdbname    in [''] or 
               username                in [''] or
               userpassword            in [''] or
               options.targetdbschema  in ['']      ):
               STEPID = STEPID+1
               print "\nStep %i: Enter the target database connection " % STEPID 
               if options.targetdbnode not in [''] :
                  print 'Database computer: %s' % options.targetdbnode
               if options.targetdbname not in [''] :
                  print 'Database name    : %s' % options.targetdbname
               if username             not in [''] :
                  print 'Database user    : %s' % username
               if options.targetdbschema not in ['']:
                  print 'Database schema  : %s' % options.targetdbschema
            while importmonitor == None:
                  while options.targetdbnode in ['']:   
                        input = raw_input('Database computer: ')
                        if input == '':
                           options.targetdbnode = socket.gethostname()
                        if input != '':
                           options.targetdbnode = input
                  while options.targetdbname in ['']:   
                        input = raw_input('Database name    : ') 
                        options.targetdbname = string.upper(input)
                  while username in ['']: 
                        username = raw_input('Database user    : ')  
                  while userpassword in ['']:   
                        userpassword   = getpass.getpass()
                  options.targetdbuser = '%s,%s' %( username, userpassword )
                  if options.targetdbuser not in ['']:
                     username     = re.split(',',options.targetdbuser)[0]
                     userpassword = re.split(',',options.targetdbuser)[1]
                  if username != '' and username [0] != '"' and username[len(username)-1] != '"':
                     username  = string.upper(username)
                  elif username != '' :
                     username = string.replace(username,'"','')  
                  options.targetdbuser = '%s,%s' %( username, userpassword )  
                  try:
                      importmonitor = sql.connect( username, userpassword, options.targetdbname, options.targetdbnode )
                  except (sql.CommunicationError ) , err:
                      print 'Connect failed, try again:' , err
                      importmonitor    = None 
                  except (sql.SQLError ) , err:  
                      print 'Connect failed, try again:' , err    
                      importmonitor    = None 
                  if importmonitor    == None:
                     options.targetdbnode = ''
                     options.targetdbname = ''
                     options.targetdbuser = ''
                     username     = ''
                     userpassword = ''
            exist_db (options, exportmonitor, importmonitor)
            if(DB_SCHEMAS_TARGET != '' and options.targetdbschema == '' ):
               STEPID = STEPID+1
               print "\nStep %i: Enter the target database schema " % STEPID 
               while options.targetdbschema in ['']:
                     schemacollection     = importmonitor.sql(DB_SCHEMAS_TARGET % ( username ) )
                     schemas = []
                     if schemacollection != None:
                        for schema in schemacollection :
                            schemas.append(schema[0])
                        schemadict  = dict([(schemas.index(schema)+1,schema) for schema in schemas]) 
                        if len(schemadict) == 1:
                           options.targetdbschema = schemadict[1] 
                        if len(schemadict)  > 1:
                           index = 0
                           for schema in schemadict:
                               index = index + 1
                               print ' [%i]:  %s ' % (index, schemadict[schema])
                           input = '-1'
                           while string.atoi(input,10) < 0 or string.atoi(input,10) > len (schemadict):
                                 input  =  raw_input("\nStep %i: "  % STEPID )
                                 options.targetdbschema = schemadict[string.atoi(input,10)]
            elif options.targetdbschema != '' :
                 options.targetdbschema = options.targetdbschema
            #DATA PATH
            try:
               options.targetindepdatapath = os.path.join(LOADERDATAPATH, 'sdb','loader', 'packages', 'import', options.targetdbnode, options.targetdbname, options.packagename)
               if os.path.isdir(options.targetindepdatapath) == False:
                  os.makedirs  (options.targetindepdatapath)
            except EnvironmentError, err:
                print err
        input =  ''
        if options.streamformat_target == '':
           STEPID = STEPID+1 
           print "\nStep %i: What do you want to transport" %STEPID                   
           while input not in ['1','2'] :  
                 input = raw_input(' [1] Catalog and application data\n [2] Catalog and package metadata \n\nStep %i: ' %STEPID)  
                 if input == '1':
                    pass
                 if input == '2':
                    if options.streamformat       == 'DDL':
                       options.streamformat_target = 'DDL'
                    elif options.streamformat     == 'XML':
                       options.streamformat_target = 'XML'
                    else:
                       options.streamformat_target = 'DTL'
        input = ''
        if((options.transformationtype == 'TRANSPORT' or
            options.transformationtype == 'EXPORT'  ) and
            exist_db   (options, exportmonitor, importmonitor) == True):
           if(exist_dba(options, exportmonitor, importmonitor) == True):
              if options.transformationpackage in ['']:
                 STEPID = STEPID+1 
                 print "\nStep %i: What do you want to include into the package" % STEPID
                 CASE_0 = """ [1] The complete database; one stream per database\n
                              [2] The complete database; one stream per database, excluding some tables\n
                              [3] The complete user; one stream per user  \n
                              [4] The complete user; one stream per user , excluding some tables\n
                              [5] The complete user; one stream per table \n
                              [6] The complete user; one stream per table, excluding some tables\n 
                              [7] Set of tables\n\nStep %i: """ % STEPID 
                 CASE_1 = " [1] The complete database; one stream per database"
                 CASE_1 = CASE_1 + """ [2] The complete database; one stream per database, excluding some tables \n\nStep %i: """ % STEPID 
              while options.transformationpackage in [''] :
                    input = raw_input( CASE_1 )
                    if input == '1':
                       options.transformationpackage = 'DB'
                       if TESTLOADER == 'FALSE':
                          options.transformationpart = 'DB'
                       MONITOR = 'OFF'
                       options.transformationpart    = 'DB'
                    if input == '2':
                       options.transformationpackage = 'DB'
                       if TESTLOADER == 'FALSE':
                          options.transformationpart = 'DB'
                       MONITOR = 'OFF'
                       STEPID = STEPID+1 
                       print '\nStep %i: Enter the table names you want to exclude from the package ' % STEPID
                       configuration = codecs.open(os.path.join(options.sourceindepdatapath,  '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1]) , "wb+","utf-8")
                       count = 1
                       username   = ''
                       tablename  = ''
                       table = raw_input(' [%i] ' %count )
                       while table not in [''] :
                             if string.find( table,'.' ) != -1 :
                                username  = re.split('\.',table)[0]
                                tablename = re.split('\.',table)[1]
                                count = count + 1
                                if tablename [0] == '"' and tablename[len(tablename)-1] == '"':
                                   tablename = string.replace(tablename,'"','')
                                else:
                                   tablename = string.upper(tablename)
                                if username [0] == '"' and username[len(username)-1] == '"':
                                   username = string.replace(username,'"','')
                                else:
                                   username = string.upper(username)
                                configuration.write(u'"%s","%s"\n' % (username,tablename) )
                             table = raw_input(' [%i] ' %count )
                       configuration.close()
                    if input == '3':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'USER'
                    if input == '4':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'USER'
                       STEPID = STEPID+1 
                       print '\nStep %i: Enter the table names you want to exclude from the package ' % STEPID
                       configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1]), "wb+","utf-8")
                       count = 1
                       schemaname = re.split(',',options.sourcedbuser)[0]
                       table      = raw_input(' [%i] ' %count )
                       while table not in [''] :
                             if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                                count = count + 1
                                if table [0] == '"' and table[len(table)-1] == '"':
                                   table = string.replace(table,'"','')
                                else:
                                   table = string.upper(table)
                                configuration.write(u'"%s","%s"\n' % (schemaname,table) )
                             table = raw_input(' [%i] ' %count )
                       configuration.close()
                    if input == '5':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'TABLE'
                    if input == '6':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'TABLE'
                       STEPID = STEPID+1 
                       print '\nStep %i: Enter the table names you want to exclude from the package ' % STEPID
                       configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1]), "wb+","utf-8")
                       count = 1
                       schemaname = re.split(',',options.sourcedbuser)[0]
                       table      = raw_input(' [%i] ' %count )
                       while table not in [''] :
                             if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                                count = count + 1
                                if table [0] == '"' and table[len(table)-1] == '"':
                                   table = string.replace(table,'"','')
                                else:
                                   table = string.upper(table)
                                configuration.write(u'"%s","%s"\n' % (schemaname,table) )
                             table = raw_input(' [%i] ' %count )
                       configuration.close()
                    if input == '7':
                       options.transformationpackage = 'TABLE'    
                       options.transformationpart    = 'TABLE' 
                       STEPID = STEPID+1 
                       print '\nStep %i: Enter the table names you want to include into the package ' % STEPID
                       configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1] ), "wb+","utf-8")
                       count = 1
                       schemaname = options.sourcedbschema
                       table      = raw_input(' [%i] ' %count )
                       while table not in [''] :
                             if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                                count = count + 1
                                if table [0] == '"' and table[len(table)-1] == '"':
                                   table = string.replace(table,'"','')
                                else:
                                   table = string.upper(table)
                                configuration.write(u'"%s","%s"\n' % (schemaname, table) )
                             table = raw_input(' [%i] ' %count )
                       configuration.close()
           #if exist_dba(options, exportmonitor, importmonitor ) == False:
           elif options.transformationpackage in ['']:
              STEPID = STEPID+1 
              print "\nStep %i: What do you want to include into the package" % STEPID
              while options.transformationpackage in [''] :
                    input = raw_input(""" [1] The complete user; one stream per user \n"""
                                      """ [2] The complete user; one stream per user , excluding some tables \n"""
                                      """ [3] The complete user; one stream per table \n"""
                                      """ [4] The complete user; one stream per table, excluding some tables \n"""
                                      """ [5] A complete schema; one stream per table \n"""
                                      """ [6] A complete schema; one stream per table, excluding some tables \n"""
                                      """ [7] A set of tables\n\nStep %i: """ % STEPID )
                    if input == '1':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'USER'
                    if input == '2':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'USER'
                       STEPID = STEPID+1 
                       print '\nStep %i: Enter the table names you want to exclude from the package ' % STEPID
                       configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1]), "wb+","utf-8")
                       count = 1
                       schemaname = re.split(',',options.sourcedbuser)[0]
                       table      = raw_input(' [%i] ' %count )
                       while table not in [''] :
                             if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                                count = count + 1
                                if table [0] == '"' and table[len(table)-1] == '"':
                                   table = string.replace(table,'"','')
                                else:
                                   table = string.upper(table)
                                configuration.write(u'"%s","%s"\n' % (schemaname,table) )
                             table = raw_input(' [%i] ' %count )
                       configuration.close()
                    if input == '3':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'TABLE'
                    if input == '4':
                       options.transformationpackage = 'USER'
                       options.transformationpart    = 'TABLE'
                       STEPID = STEPID+1 
                       print '\nStep %i: Enter the table names you want to exclude from the package ' % STEPID
                       configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1]), "wb+","utf-8")
                       count = 1
                       schemaname = re.split(',',options.sourcedbuser)[0]
                       table      = raw_input(' [%i] ' %count )
                       while table not in [''] :
                             if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                                count = count + 1
                                if table [0] == '"' and table[len(table)-1] == '"':
                                   table = string.replace(table,'"','')
                                else:
                                   table = string.upper(table)
                                configuration.write(u'"%s","%s"\n' % (schemaname,table) )
                             table = raw_input(' [%i] ' %count )
                       configuration.close()
                    if input == '5' or  input == '6':
                       options.transformationpackage = 'SCHEMA'
                       options.transformationpart    = 'TABLE'
                       if((options.mode  == 'INTER' and DB_SCHEMAS_SOURCE != '') or
                          (options.mode  == 'PARAM' and DB_SCHEMAS_SOURCE != '' and options.transformationpackage == 'SCHEMA' and options.sourcedbschema == '' ) ):
                          STEPID = STEPID+1
                          print "\nStep %i: Enter the source database schema " % STEPID 
                          while options.sourcedbschema in ['']:
                                schemacollection     = exportmonitor.sql(DB_SCHEMAS_SOURCE % ( re.split(',',options.sourcedbuser)[0] ))
                                schemas = []
                                if schemacollection != None:
                                   for schema in schemacollection :
                                       schemas.append(schema[0])
                                schemadict  = dict([(schemas.index(schema)+1,schema) for schema in schemas])  
                                if len(schemadict) == 1:
                                    options.sourcedbschema = schemadict[1]
                                if len(schemadict)  > 1: 
                                   index = 0
                                   for schema in schemadict:
                                       index = index + 1
                                       print ' [%i] %s ' % (index, schemadict[schema])
                                   input_schema = '-1'
                                   while string.atoi(input_schema,10) < 0 or string.atoi(input_schema,10) > len (schemadict):
                                         input_schema  =  raw_input("\nStep %i: "  % STEPID )
                                         options.sourcedbschema = schemadict[string.atoi(input_schema,10)]
                       if input == '6':
                          STEPID = STEPID+1 
                          print '\nStep %i: Enter the table names you want to exclude from the package ' % STEPID
                          configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1]), "wb+","utf-8")
                          count = 1
                          schemaname = options.sourcedbschema
                          table      = raw_input(' [%i] ' %count )
                          while table not in [''] :
                                if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                                   count = count + 1
                                   if table [0] == '"' and table[len(table)-1] == '"':
                                      table = string.replace(table,'"','')
                                   else:
                                      table = string.upper(table)
                                   configuration.write(u'"%s","%s"\n' % (schemaname,table) )
                                table = raw_input(' [%i] ' %count )
                          configuration.close()
                    if input == '7':
                       options.transformationpackage = 'TABLE'    
                       options.transformationpart    = 'TABLE' 
                       STEPID = STEPID+1 
                       print '\nStep %i: Enter the table names you want to include into the package ' % STEPID
                       configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1] ), "wb+","utf-8")
                       count = 1
                       schemaname = options.sourcedbschema
                       table      = raw_input(' [%i] ' %count )
                       while table not in [''] :
                             if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                                count = count + 1
                                if table [0] == '"' and table[len(table)-1] == '"':
                                   table = string.replace(table,'"','')
                                else:
                                   table = string.upper(table)
                                configuration.write(u'"%s","%s"\n' % (schemaname,table) )
                             table = raw_input(' [%i] ' %count )
                       configuration.close()        
           # command line parameters
           elif(options.transformationpackage in ['TABLE'] and 
                options.transformationpart    in ['TABLE'] ):
                STEPID = STEPID+1 
                print "\nStep %i: What do you want to include into the package" % STEPID
                configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1] ), "wb+","utf-8")
                count = 1
                schemaname = re.split(',',options.sourcedbuser)[0]
                table      = raw_input(' [%i] ' %count )
                while table not in [''] :
                      if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                         count = count + 1
                         if table [0] == '"' and table[len(table)-1] == '"':
                            table = string.replace(table,'"','')
                         else:
                            table = string.upper(table)
                         configuration.write(u'"%s","%s"\n' % (schemaname,table) )
                      table = raw_input(' [%i] ' %count )
                configuration.close()  
           elif(options.transformationpackage in ['USER']  and ( 
                options.transformationpart    in ['USER']  or 
                options.transformationpart    in ['TABLE'])and
                options.mode                  in ['INTER']     ):
                STEPID = STEPID+1 
                print '\nStep %i: Enter the table names you want to exclude from the package ' % STEPID
                configuration = codecs.open(os.path.join(options.sourceindepdatapath, '%s.CONFIGURATION' % os.path.split(options.sourceindepdatapath)[1]), "wb+","utf-8")
                count = 1
                schemaname = re.split(',',options.sourcedbuser)[0]
                table      = raw_input(' [%i] ' %count )
                while table not in [''] :
                      if exist_table( options, exportmonitor, importmonitor, schemaname, table) :
                         count = count + 1
                         if table [0] == '"' and table[len(table)-1] == '"':
                            table = string.replace(table,'"','')
                         else:
                            table = string.upper(table)
                         configuration.write(u'"%s","%s"\n' % ( schemaname, table) )
                      table = raw_input(' [%i] ' %count )
                configuration.close()      
           input = ''
           if options.streammedium in ['']:
              if options.transformationpackage == 'TABLE' :    
                 options.transformationpart     = 'TABLE'                      
              if options.transformationtype    == 'TRANSPORT' :#and sys.platform[:3] == 'win'
                 streammedium                   = 'PIPE'
              else:
                 streammedium                   = 'FILE'
           else:
              streammedium = options.streammedium
           if options.streamformat in ['']:      
              if  (streammedium                 == 'PIPE' ) :
                   streamformat                  = 'RECORDS'
              elif(options.transformationpackage!= 'TABLE' or
                   options.transformationpart   != 'TABLE') :
                   streamformat                  = 'RECORDS'
              elif(options.sourcedbnode == options.targetdbnode and
                   options.sourcedbname == options.targetdbname and
                   options.transformationpart  == 'TABLE'):
                   streamformat                 = 'SQL'
              else:
                   streamformat                 = 'COMPRESSED'
           else:
              streamformat = options.streamformat
           if options.streamformat_target in ['']:
              streamformat_target               = 'ROW'
           else:
              streamformat_target = options.streamformat_target
           if options.streamcharencoding  in ['']:
              streamcharencoding                = 'DYNAMIC'
           else:
              streamcharencoding = options.streamcharencoding
           if options.streamlob in ['']:
              if(options.transformationpackage != 'TABLE' and
                 options.transformationpart    != 'TABLE') :               
                 streamlob                      = 'TABLE'
              else:
                 streamlob                      = 'COLUMNVALUE'
           else:
               streamlob = options.streamlob
           input = ''
           STEPID = STEPID+1  
           if options.transporttype in ['']:
              while options.transporttype in [''] or input in ['1']:
                    print '\nStep %i: The package can be designed, generated or executed now. ' % STEPID
                    input = ''
                    while input not in ['1','2','3']:
                          input = raw_input(""" [1] Design   package to generate  it later \n"""
                                            """ [2] Generate package to execute   it later \n"""
                                            """ [3] Execute  package to transport data now \n\nStep %i: """  % STEPID )
                          if input == '3':
                             options.transporttype           = 'EXECUTE'
                             if options.streammedium        == '' :
                                options.streammedium         = streammedium
                             if options.streamformat        == '':
                                options.streamformat         = streamformat
                             if options.streamformat_target == '':
                                options.streamformat_target  = streamformat_target
                             if options.streamcharencoding  == '':
                                options.streamcharencoding   = streamcharencoding
                             if options.streamlob           == '':
                                options.streamlob            = streamlob 
                          if input == '2':
                             options.transporttype           = 'GENERATE'  
                             if options.streammedium        == '' :
                                options.streammedium         = streammedium
                             if options.streamformat        == '':
                                options.streamformat         = streamformat
                             if options.streamformat_target == '':
                                options.streamformat_target  = streamformat_target
                             if options.streamcharencoding  == '':
                                options.streamcharencoding   = streamcharencoding
                             if options.streamlob           == '':
                                options.streamlob            = streamlob 
                          if input == '1':
                             options.streammedium            = streammedium
                             input  = ''
                             STEPID = STEPID+1    
                             if(options.transformationtype == 'TRANSPORT'  and DB_DEFAULTCODE_SOURCE == DB_DEFAULTCODE_TARGET) or options.transformationtype == 'EXPORT':
                                CASE_1 =(""" [1] Platform independent loader records format\n"""       
                                         """ [2] Platform independent database page  format (without logging, backup necessary)\n \nStep %i: """) %STEPID
                                CASE_2 =(""" [1] Comma separated values; one stream per lob column\n"""
                                         """ [2] Comma separated values; one stream per lob value \n"""
                                         """ [3] Fixed width values;     one stream per lob column\n"""
                                         """ [4] Fixed width values;     one stream per lob value \n"""
                                         """ [5] Platform independent loader records format		  \n"""
                                         """ [6] Platform independent database page  format (without logging, backup necessary)\n"""
                                         """ [7] No format\nStep %i: """) %STEPID
                                CASE_3 =(""" [1] Comma separated values; one stream per lob column\n"""
                                         """ [2] Comma separated values; one stream per lob value \n"""
                                         """ [3] Fixed width values;     one stream per lob column\n"""
                                         """ [4] Fixed width values;     one stream per lob value \n"""
                                         """ [5] Platform independent loader records format       \n"""
                                         """ [6] Platform independent database page  format (without logging, backup necessary)\n              \nStep %i: """) %STEPID
                             if options.transformationtype == 'TRANSPORT'  and DB_DEFAULTCODE_SOURCE != DB_DEFAULTCODE_TARGET:
                                CASE_1   = ' [1] Platform independent loader records format       \nStep %i: ' %STEPID
                                CASE_2 =(""" [1] Comma separated values; one stream per lob column\n"""
                                         """ [2] Comma separated values; one stream per lob value \n"""
                                         """ [3] Fixed width values;     one stream per lob column\n"""
                                         """ [4] Fixed width values;     one stream per lob value \n"""
                                         """ [5] Platform independent loader records format		  \n"""
                                         """ [6] No format\n   \nStep %i: """) %STEPID
                                CASE_3 =(""" [1] Comma separated values; one stream per lob column\n"""
                                         """ [2] Comma separated values; one stream per lob value \n"""
                                         """ [3] Fixed width values;     one stream per lob column\n"""
                                         """ [4] Fixed width values;     one stream per lob value \n"""
                                         """ [5] Platform independent loader records format\n     \nStep %i: """) %STEPID
                                
                             print "\nStep %i: How do you want to export the data" %STEPID       
                             if options.transformationpart == 'DB' or  options.transformationpart == 'USER' or options.transformationpart == 'SCHEMA':
                                while options.streamformat in [''] :
                                      input = raw_input(CASE_1)
                                      if input == '1':
                                         options.streamformat = 'RECORDS'
                                      if input == '2':
                                         options.streamformat = 'PAGES'
                                         if options.transformationtype == 'TRANSPORT' and DB_DEFAULTCODE_SOURCE != DB_DEFAULTCODE_TARGET:
                                            options.streamformat = ''
                             input = ''
                             if options.transformationpart == 'TABLE':
                                if( options.sourcedbnode   == options.targetdbnode   and
                                    options.sourcedbname   == options.targetdbname ):
                                    while options.streamformat in [''] :
                                          input = raw_input(CASE_2)
                                          if input == '1':
                                             options.streamformat             = 'COMPRESSED'
                                             options.streamlob                = 'COLUMN'
                                             options.streamcharencoding       = 'DYNAMIC'
                                          if input == '2':				  
                                             options.streamformat             = 'COMPRESSED'
                                             options.streamlob                = 'COLUMNVALUE'
                                             options.streamcharencoding       = 'DYNAMIC'
                                          if input == '3':
                                             options.streamformat             = 'FORMATTED'
                                             options.streamlob                = 'COLUMN'
                                             if options.transformationtype   == 'TRANSPORT':
                                                if DB_DEFAULTCODE_TARGET     == 'UNICODE':
                                                   options.streamcharencoding = 'UTF8'
                                                if DB_DEFAULTCODE_TARGET     == 'ASCII':
                                                   options.streamcharencoding = 'ASCII'
                                          if input == '4':
                                             options.streamformat             = 'FORMATTED'
                                             options.streamlob                = 'COLUMNVALUE'
                                             if options.transformationtype   == 'TRANSPORT':
                                                if DB_DEFAULTCODE_TARGET     == 'UNICODE':
                                                   options.streamcharencoding = 'UTF8'
                                                if DB_DEFAULTCODE_TARGET     == 'ASCII':
                                                   options.streamcharencoding = 'ASCII'
                                          if input == '5':				   
                                             options.streamformat             = 'RECORDS'
                                             options.streamcharencoding       = 'DYNAMIC'
                                          if input == '6':				     
                                             options.streamformat             = 'PAGES'
                                             if options.transformationtype   == 'TRANSPORT' and DB_DEFAULTCODE_SOURCE != DB_DEFAULTCODE_TARGET:
                                                options.streamformat = ''
                                          if input == '7':
                                             options.streamformat             = 'SQL'
                                else:
                                     while options.streamformat in [''] :
                                           input = raw_input(CASE_3)
                                           if input == '1':
                                              options.streamformat             = 'COMPRESSED'
                                              options.streamlob                = 'COLUMN'
                                              options.streamcharencoding       = 'DYNAMIC'
                                           if input == '2':
                                              options.streamformat             = 'COMPRESSED'
                                              options.streamlob                = 'COLUMNVALUE'
                                              options.streamcharencoding       = 'DYNAMIC'
                                           if input == '3':
                                              options.streamformat             = 'FORMATTED'
                                              options.streamlob                = 'COLUMN'
                                              if options.transformationtype   == 'TRANSPORT':
                                                 if DB_DEFAULTCODE_TARGET     == 'UNICODE':
                                                    options.streamcharencoding = 'UTF8'
                                                 if DB_DEFAULTCODE_TARGET     == 'ASCII':
                                                    options.streamcharencoding = 'ASCII'
                                           if input == '4':
                                              options.streamformat             = 'FORMATTED'
                                              options.streamlob                = 'COLUMNVALUE'
                                              if options.transformationtype   == 'TRANSPORT':
                                                 if DB_DEFAULTCODE_TARGET     == 'UNICODE':
                                                    options.streamcharencoding = 'UTF8'
                                                 if DB_DEFAULTCODE_TARGET     == 'ASCII':
                                                    options.streamcharencoding = 'ASCII'
                                           if input == '5':
                                              options.streamformat             = 'RECORDS'
                                              options.streamcharencoding       = 'DYNAMIC'
                                           if input == '6':
                                              options.streamformat             = 'PAGES'
                                              if options.transformationtype   == 'TRANSPORT' and DB_DEFAULTCODE_SOURCE != DB_DEFAULTCODE_TARGET:
                                                 options.streamformat = ''
                             input = ''
                             if options.streamformat_target != 'DTL':
                                STEPID = STEPID+1                            
                                while options.streamformat_target in [''] and options.streamformat != 'PAGES' and options.streamformat != 'RECORDS' :
                                      print "\nStep %i: How do you want to import the data" %STEPID   
                                      input = raw_input(""" [1] Row by page (without logging, backup necessary)\n"""
                                                        """ [2] Row by row  (with    logging) \n\nStep %i: """ % STEPID)
                                      if input == '1':
                                         options.streamformat_target = 'PAGE'
                                      if input == '2':
                                         options.streamformat_target = 'ROW'
           else:
               if options.streammedium        == '' :
                  options.streammedium         = streammedium
               if options.streamformat        == '':
                  options.streamformat         = streamformat
               if options.streamformat_target == '':
                  options.streamformat_target  = streamformat_target
               if options.streamcharencoding  == '':
                  options.streamcharencoding   = streamcharencoding
               if options.streamlob           == '':
                  options.streamlob            = streamlob 
           try:
              if exportmonitor != None:
                 exportmonitor.release()
              if importmonitor != None:   
                 importmonitor.release()
           except loader.CommunicationError, err :
              raise loader.CommunicationError, err, sys.exc_traceback
     except EOFError:
         os._exit(1)
     return rc
     
def exist_packageloader (options): 
    bVersion = True
    try:
       session       = loader.Loader('','')
       versionstring = session.cmd('VERSION')
       match         = re.search ('"(\d+)[.](\d+)[.](\d+)[.](\d+)"', versionstring)
       version       = tuple (map (int, match.group (1, 2, 3, 4)))
       if version < (7,5,0,16):
          print versionstring
          print 'Loader Version 7.5.00.16 or higher required'
          bVersion =  False
          if sys.platform[:3] == 'win':
             print 'Try to start Loader from %s ' % INDEPPROGPATH
       return bVersion
    except loader.LoaderError, err:
           raise loader.LoaderError, err, sys.exc_traceback
    except loader.CommunicationError, err :
           raise loader.CommunicationError, err, sys.exc_traceback
         
def exist_db (options, exportmonitor, importmonitor):
    rc = True
    maxusertasks  = 0
    usedusertasks = 0
    global DB_INSTROOT_SOURCE
    global DB_INSTROOT_TARGET
    global DB_VERSION_SOURCE
    global DB_VERSION_TARGET
    global DB_INDEPDATAPATH_SOURCE
    global DB_INDEPDATAPATH_TARGET
    global DB_CODE_SOURCE
    global DB_CODE_TARGET
    global DB_DEFAULTCODE_SOURCE
    global DB_DEFAULTCODE_TARGET
    global DB_SCHEMAS_SOURCE
    global DB_SCHEMAS_TARGET
    global PACKAGEIMPORT
    global PACKAGEMAXSTEP_SOURCE
    global PACKAGEMAXSTEP_TARGET  
    global PACKAGEROWCOUNT_SOURCE
    global PACKAGEROWCOUNT_TARGET 
    global STEPIDROWCOUNT_SOURCE
    global STEPIDROWCOUNT_TARGET  
    global STEPIDSROWCOUNT_SOURCE
    global STEPIDSROWCOUNT_TARGET 
    global PACKAGEVALIDATE
    #Source DBM
    try:
        if exportmonitor != None: 
           dbmsource     = dbm.DBM (options.sourcedbnode, options.sourcedbname, '','')
           indepdatapath = dbmsource.cmd('dbm_getpath     INDEPDATAPATH')
           if DB_INDEPDATAPATH_SOURCE == '':
              DB_INDEPDATAPATH_SOURCE = re.split('\n',indepdatapath)[0]
           instroot = dbmsource.cmd('dbm_version  INSTROOT')
           DB_INSTROOT_SOURCE = re.split('\n',instroot)[0]
           version  = dbmsource.cmd('dbm_version BUILD')
           version  = string.replace(version,'DBMServer ','')
           DB_VERSION_SOURCE = '%s.%s' %( re.split('Build',version)[0][0:6],re.split('Build',version)[1][2:4])
           match = re.search ('(\d+)[.](\d+)[.](\d+)', DB_VERSION_SOURCE)
           version = tuple (map (int, match.group (1, 2, 3)))
           if version      >=  (7, 5, 0) :
              MAXUSERTASKS    = "SELECT VALUE    FROM ACTIVECONFIGURATION  WHERE PARAMETERNAME LIKE 'MAXUSERTASKS'"
              CODE            = "SELECT VALUE    FROM ACTIVECONFIGURATION  WHERE PARAMETERNAME LIKE '_UNICODE'"
              DEFAULTCODE     = "SELECT VALUE    FROM ACTIVECONFIGURATION  WHERE PARAMETERNAME LIKE 'DEFAULT_CODE'"
              USEDUSERTASKS   = "SELECT COUNT(*) FROM SESSIONS"
           if version      <   (7, 5, 0):
              MAXUSERTASKS    = "SELECT VALUE    FROM DBPARAMETERS         WHERE DESCRIPTION   LIKE 'MAXUSERTASKS'"
              CODE            = "SELECT VALUE    FROM DBPARAMETERS         WHERE PARAMETERNAME LIKE '_UNICODE'"
              DEFAULTCODE     = "SELECT VALUE    FROM DBPARAMETERS         WHERE DESCRIPTION   LIKE 'DEFAULT_CODE'"
              USEDUSERTASKS   = "SELECT COUNT(*) FROM CONNECTEDUSERS"
           if version      >= (7,6,0) :
              PACKAGEMAXSTEP_SOURCE  = "SELECT MAX(STEPID)                              FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE "
              PACKAGEROWCOUNT_SOURCE = "SELECT STEPID, SCHEMANAME, TABLENAME, ROWCOUNT  FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE "
              STEPIDROWCOUNT_SOURCE  = "SELECT STEPID, SCHEMANAME, TABLENAME, ROWCOUNT  FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE  AND STEPID  = %s "
              STEPIDSROWCOUNT_SOURCE = "SELECT STEPID, SCHEMANAME, TABLENAME, ROWCOUNT  FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = FALSE AND STEPID >= %s "
              DB_SCHEMAS_SOURCE    =("""SELECT  SCHEMANAME                              FROM DOMAIN.SCHEMAS WHERE OWNER LIKE '%s' """
                                     """AND SCHEMANAME NOT LIKE 'OMS' """
                                     """AND SCHEMANAME NOT LIKE 'DOMAIN' """
                                     """AND SCHEMANAME NOT LIKE 'SYSINFO'""")
           if version      <  (7, 6, 0):
              PACKAGEMAXSTEP_SOURCE  = "SELECT MAX(STEPID)                         FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE "
              PACKAGEROWCOUNT_SOURCE = "SELECT STEPID, OWNER, TABLENAME, ROWCOUNT  FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE "
              STEPIDROWCOUNT_SOURCE  = "SELECT STEPID, OWNER, TABLENAME, ROWCOUNT  FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE  AND STEPID  = %s "
              STEPIDSROWCOUNT_SOURCE = "SELECT STEPID, OWNER, TABLENAME, ROWCOUNT  FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = FALSE AND STEPID >= %s "
        if exportmonitor != None: 
           resultset    = exportmonitor.sql(MAXUSERTASKS)
           for row in resultset:
               maxusertasks = row[0]
           resultset    = exportmonitor.sql(USEDUSERTASKS)
           for row in resultset:
               sessions = row[0]    
           if int(maxusertasks) - int(usedusertasks) + 1  < 2:
              print 'At least 2 free USERTASKS for Data Source Needed'
              rc = False
           resultset    = exportmonitor.sql(CODE)
           for row in resultset:
               code = row[0]
               if code[0:2]            == 'NO':
                  DB_CODE_SOURCE        = 'ASCII'
               else:
                  DB_CODE_SOURCE        = 'UNICODE'
           resultset    = exportmonitor.sql(DEFAULTCODE)
           for row in resultset:
               defaultcode = row[0]
               if defaultcode[0:5]     == 'ASCII':
                  DB_DEFAULTCODE_SOURCE = 'ASCII'
               if defaultcode[0:7]     == 'UNICODE':
                  DB_DEFAULTCODE_SOURCE = 'UNICODE'
    except dbm.CommunicationError, err :
           pass
    except dbm.DBMServError, err:
           pass
    except (sql.CommunicationError ), err:   
           print 'Connect failed, try again:' , err
    except (sql.SQLError ), err:
           print err
        
    #Target DBM
    try:
        if importmonitor != None: 
           dbmtarget     = dbm.DBM (options.targetdbnode, options.targetdbname, '','')
           indepdatapath = dbmtarget.cmd('dbm_getpath     INDEPDATAPATH')
           if DB_INDEPDATAPATH_TARGET == '':
              DB_INDEPDATAPATH_TARGET = re.split('\n',indepdatapath)[0]
           instroot = dbmtarget.cmd('dbm_version  INSTROOT')
           DB_INSTROOT_TARGET = re.split('\n',instroot)[0]
           version = dbmtarget.cmd('dbm_version BUILD')
           version = string.replace(version,'DBMServer ','')
           DB_VERSION_TARGET = '%s.%s' %( re.split('Build',version)[0][0:6],re.split('Build',version)[1][2:4])
           match   = re.search ('(\d+)[.](\d+)[.](\d+)', DB_VERSION_TARGET)
           version = tuple (map (int, match.group (1, 2, 3)))
           if version      >=  (7, 5, 0) :
              MAXUSERTASKS    = "SELECT VALUE    FROM ACTIVECONFIGURATION  WHERE PARAMETERNAME LIKE 'MAXUSERTASKS'"
              CODE            = "SELECT VALUE    FROM ACTIVECONFIGURATION  WHERE PARAMETERNAME LIKE '_UNICODE'"
              DEFAULTCODE     = "SELECT VALUE    FROM ACTIVECONFIGURATION  WHERE PARAMETERNAME LIKE 'DEFAULT_CODE'"
              USEDUSERTASKS   = "SELECT COUNT(*) FROM SESSIONS"
           if version      <   (7, 5, 0):
              MAXUSERTASKS    = "SELECT VALUE    FROM DBPARAMETERS         WHERE DESCRIPTION   LIKE 'MAXUSERTASKS'"
              CODE            = "SELECT VALUE    FROM DBPARAMETERS         WHERE PARAMETERNAME LIKE '_UNICODE'"
              DEFAULTCODE     = "SELECT VALUE    FROM DBPARAMETERS         WHERE DESCRIPTION   LIKE 'DEFAULT_CODE'"
              USEDUSERTASKS   = "SELECT COUNT(*) FROM CONNECTEDUSERS"
           if version      >= (7,6,0) :
              PACKAGEMAXSTEP_TARGET  = "SELECT MAX(STEPID)                              FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE "
              PACKAGEROWCOUNT_TARGET = "SELECT STEPID, SCHEMANAME, TABLENAME, ROWCOUNT  FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE "
              STEPIDROWCOUNT_TARGET  = "SELECT STEPID, SCHEMANAME, TABLENAME, ROWCOUNT  FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE  AND STEPID  = %s "
              STEPIDSROWCOUNT_TARGET = "SELECT STEPID, SCHEMANAME, TABLENAME, ROWCOUNT  FROM SYSLOADER.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = FALSE AND STEPID >= %s "
              packageimport          = "LOAD DATA TABLE SYSLOADER.TRANSFORMATIONMODEL \n"
              packageimport          =  packageimport + 'OWNER                 1 DEFAULT      NULL SCHEMANAME           2 DEFAULT      NULL TABLENAME                 3 DEFAULT      NULL \nDATASTREAMNAME      4 DEFAULT      NULL \n'
              PACKAGEVALIDATE        = "SELECT   T1.PACKAGEGUID , T1.TRANSFORMATION, T1.OWNER, T1.TABLENAME , T1.DATASTREAMHEADERBLOCK, T1.DATASTREAMHEADERPOS , T1.DATASTREAMDATABLOCK, T1.DATASTREAMDATAPOS , T1.START  FROM   SYSLOADER.TRANSFORMATIONSTEPS T1, SYSLOADER.TRANSFORMATIONSTEPS T2 "
              DB_SCHEMAS_TARGET    =("""SELECT  SCHEMANAME                              FROM DOMAIN.SCHEMAS WHERE OWNER LIKE '%s' """
                                     """AND SCHEMANAME NOT LIKE 'OMS' """
                                     """AND SCHEMANAME NOT LIKE 'DOMAIN' """
                                     """AND SCHEMANAME NOT LIKE 'SYSINFO'""")
           if version      <  (7, 6, 0):
              PACKAGEMAXSTEP_TARGET  = "SELECT MAX(STEPID)                         FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE "
              PACKAGEROWCOUNT_TARGET = "SELECT STEPID, OWNER, TABLENAME, ROWCOUNT  FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE "
              STEPIDROWCOUNT_TARGET  = "SELECT STEPID, OWNER, TABLENAME, ROWCOUNT  FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = TRUE  AND STEPID  = %s "
              STEPIDSROWCOUNT_TARGET = "SELECT STEPID, OWNER, TABLENAME, ROWCOUNT  FROM SYSDBA.TRANSFORMATIONMODEL WHERE PACKAGEGUID  = x'%s' AND EXCLUDE = FALSE AND RESTART = FALSE AND STEPID >= %s "
              packageimport          = "LOAD DATA TABLE SYSDBA.TRANSFORMATIONMODEL \n"
              packageimport          =  packageimport + 'OWNER                 1 DEFAULT      NULL                                          TABLENAME                 3 DEFAULT      NULL \nDATASTREAMNAME      4 DEFAULT      NULL \n'
              PACKAGEVALIDATE        = "SELECT   T1.PACKAGEGUID , T1.TRANSFORMATION, T1.OWNER, T1.TABLENAME , T1.DATASTREAMHEADERBLOCK, T1.DATASTREAMHEADERPOS , T1.DATASTREAMDATABLOCK, T1.DATASTREAMDATAPOS , T1.START  FROM   SYSDBA.TRANSFORMATIONSTEPS T1, SYSDBA.TRANSFORMATIONSTEPS T2 "
           '''
           packageimport             =  packageimport + "SEPARATOR  '%s'  \n" % re.split('/',FIELDCOMPRESSED)[1]
           packageimport             =  packageimport + "DELIMITER  '%s'  \n" % re.split('/',FIELDCOMPRESSED)[2]
           packageimport             =  packageimport + "NULL       '%s'  \n" % FIELDNULLVALUE
           packageimport             =  packageimport + "DECIMAL    '%s'  \n" % FIELDDECIMALVALUE
           packageimport             =  packageimport + "BOOLEAN    '%s'  \n" % FIELDBOOLVALUE
           packageimport             =  packageimport + "TIMESTAMP   %s   \n" % FIELDTIMESTAMPVALUE
           packageimport             =  packageimport + "DATE        %s   \n" % FIELDATEVALUE
           packageimport             =  packageimport + "TIME        %s   \n" % FIELDTIMEVALUE
           '''           
           packageimport             =  packageimport + 'DATASTREAMHEADERBLOCK 5 DEFAULT      NULL DATASTREAMHEADERPOS  6 DEFAULT      NULL \nDATASTREAMTRAILERBLOCK  7 DEFAULT      NULL DATASTREAMTRAILERPOS  8 DEFAULT      NULL  DATASTREAMDATABLOCK  9 DEFAULT      NULL DATASTREAMDATAPOS   10  DEFAULT    NULL \n'
           packageimport             =  packageimport + '\"EXCLUDE\"          11 DEFAULT      NULL \n'
           packageimport             =  packageimport + 'PACKAGEGUID          12 DEFAULT      NULL TASKID              13 DEFAULT      NULL \nSTEPID                 14 DEFAULT      NULL SERVERNODE           15 DEFAULT      NULL \n'
           packageimport             =  packageimport + '\"SERVERDB\"         16 DEFAULT      NULL USERNAME            17 DEFAULT      NULL \n\"CODETYPE\"           18 DEFAULT      NULL CODEPAGEID           19 DEFAULT      NULL \n'
           packageimport             =  packageimport + '\"STORAGEDEVICE\"    20 DEFAULT      NULL TRANSACTIONSIZE     21 DEFAULT      NULL TRANSFORMATION           22 DEFAULT      NULL \nPART               23 DEFAULT      NULL TABLETYPE            24 DEFAULT      NULL \n'
           packageimport             =  packageimport + 'CATALOGSTREAMNAME    25 DEFAULT      NULL CATALOGSTREAMTYPE   26 DEFAULT      NULL \nCATALOGSTREAMEBID      27 DEFAULT      NULL CATALOGSTREAMFORMAT  28 DEFAULT      NULL DATASTREAMTYPE       29 DEFAULT      NULL  DATASTREAMEBID     30  DEFAULT     NULL \n'
           packageimport             =  packageimport + 'DATASTREAMFORMAT     31 DEFAULT      NULL \"RESTART\"         32 DEFAULT      NULL \nRESTORE                33 DEFAULT      NULL PAGECOUNT            34 DEFAULT      NULL \n'
           packageimport             =  packageimport + 'ROWCOUNT             35 DEFAULT      NULL STARTTIME           36 DEFAULT      NULL \nENDTIME                37 DEFAULT      NULL REQUESTTIME          38 DEFAULT      NULL \n'
           packageimport             =  packageimport + 'RECEIVETIME          39 DEFAULT      NULL IOTIME              40 DEFAULT      NULL \nLOGTIME                41 DEFAULT      NULL '
           packageimport             =  packageimport + " INSTREAM FILE '%s'  COMPRESSED %s \n"
           PACKAGEIMPORT             =  packageimport
        if importmonitor != None: 
           resultset      = importmonitor.sql(MAXUSERTASKS)
           for row in resultset:
               maxusertasks = row[0]
           resultset    = importmonitor.sql(USEDUSERTASKS)
           for row in resultset:
               sessions = row[0]    
           if int(maxusertasks) - int(usedusertasks) + 1  < 2:
              print 'At least 2 free USERTASKS for Data Source Needed'
              rc = False
           resultset    = importmonitor.sql(CODE)
           for row in resultset:
               code = row[0]
               if code[0:2]            == 'NO':
                  DB_CODE_TARGET        = 'ASCII'
               else:
                  DB_CODE_TARGET        = 'UNICODE'
           resultset    = importmonitor.sql(DEFAULTCODE)
           for row in resultset:
               defaultcode = row[0]
               if defaultcode[0:5]     == 'ASCII':
                  DB_DEFAULTCODE_SOURCE = 'ASCII'
               if defaultcode[0:7]     == 'UNICODE':
                  DB_DEFAULTCODE_SOURCE = 'UNICODE'
    except dbm.CommunicationError, err :
           pass
    except dbm.DBMServError, err:
           pass
    except (sql.CommunicationError ), err:   
           print 'Connect failed, try again:' , err
    except (sql.SQLError ), err:
           print err
    return rc

def  exist_dba(options, exportmonitor, importmonitor ):
     global DBA_SOURCE
     global DBA_TARGET
     exist_sourcedba = False
     exist_targetdba = False
     if exportmonitor == None:  
        exist_sourcedba = True
     if exportmonitor != None:  
        try:   
           resultset = exportmonitor.sql(EXISTSDBA)
           for row in resultset:
               if re.split(',',options.sourcedbuser)[0] == row[0] :
                  exist_sourcedba = True
        except (sql.CommunicationError ), err:   
            print 'Connect failed, try again:' , err
        except (sql.SQLError ), err:
            exist_sourcedba = False
     if importmonitor == None:
        exist_targetdba = True
     if importmonitor != None:         
        try:   
            resultset = importmonitor.sql(EXISTSDBA)
            for row in resultset:
                if re.split(',',options.targetdbuser)[0] == row[0] :
                   exist_targetdba = True
        except (sql.CommunicationError ), err:   
            print 'Connect failed, try again:' , err
        except (sql.SQLError ), err:
            exist_targetdba = False
     if exist_sourcedba == True and exist_targetdba == True:
        rc = True
        if options.transformationpackage == 'DB':
           DBA_SOURCE = options.sourcedbuser
           DBA_TARGET = options.targetdbuser
     else:
        rc = False      
     return rc    

def  exist_table(options, exportmonitor, importmonitor, schemaname, table):
     exist_sourcetable = True
     exist_targettable = True
     if len(re.split('\.',table)) == 1:
        if table [0] == '"' and table[len(table)-1] == '"':
           table = string.replace(table,'"','')
        else:
           table = string.upper(table)
        schema = schemaname
     if len(re.split('\.',table)) == 2: 
        schema     = re.split('\.',table)[0]
        tablename  = re.split('\.',table)[1]
        if schema [0] == '"' and schema[len(schema)-1] == '"':
           schema = string.replace(schema,'"','')
        else:
           schema = string.upper(schema)
        if tablename [0] == '"' and tablename[len(tablename)-1] == '"':
           tablename = string.replace(tablename,'"','')
        else:
           tablename = string.upper(tablename)
        table = tablename
     if exportmonitor == None:
        exist_sourcetable = True    
     if exportmonitor != None and TESTLOADER == 'FALSE':  
        try:   
           cmd = 'EXISTS TABLE "%s"."%s" ' % (schema,table)
           exportmonitor.sql(cmd)
           exist_sourcetable = True
        except (sql.CommunicationError ), err:   
            print 'Connect failed, try again:' , err
        except (sql.SQLError ), err:
            if  err.errorCode == -4004: 
                exist_sourcetable = False
                print 'Table     %-18s does not exist in source database' %(table)
     if importmonitor == None:
        exist_targettable = True           
     if options.transporttype == 'GENERATE':
        exist_targettable = False
     if importmonitor != None and options.transporttype != 'GENERATE':         
        try: 
            cmd = 'EXISTS TABLE "%s" ' % (table)
            importmonitor.sql(cmd)
            exist_targettable = True
            print 'Table     %-18s exists in target database'        %(table )  
        except (sql.CommunicationError ), err:   
            print 'Connect failed, try again:' , err
        except (sql.SQLError ), err:
            if  err.errorCode == -4004: 
                exist_targettable = False
     
     if options.transformationtype == 'TRANSPORT':
        if exist_sourcetable == True and exist_targettable == False:
           rc = True
        else:
           rc = False  
     if options.transformationtype == 'EXPORT' :
        if exist_sourcetable == True:
           rc = True
        else:
           rc = False  
     if options.transformationtype == 'IMPORT' :     
        if exist_targettable == False:
           rc = True
        else:
           rc = False     
     return rc  
     
def import_package(options):
    rc = True
    global TESTLOADER
    global PACKAGEPATH
    global PACKAGENAME
    global PACKAGETIMESTAMP
    global PACKAGEEXECUTION
    global PACKAGEARCHIVE
    #LOADER ENVIRONMENT
    global INDEPPROGPATH    
    global INDEPDATAPATH        
    
    #LOADER CONFIGURATION
    global LOADERCONFIGURATION
    global LOADERCONFIGFILE
    global LOADERPROGPATH
    global LOADERDATAPATH
    
    # TRANSFORMATION RUNTIME
    global PACKAGETIMEOUT       
    global MAXERRORCOUNT        
    global TRANSACTIONSIZE 
    global ISOLATIONLEVEL     
    global AUTOCOMMIT           
    
    # TRANSFORMATION PARAMETER
    global VERBOSE              
    global MONITOR              
    global TRANSFORMATIONSYNTAX 
    global SYSTEMATTRIBUTE
    global FIELDLOBMASK
    
    TESTLOADER  = os.getenv('TESTLOADER')
    if TESTLOADER == None:
       TESTLOADER = 'FALSE'
    INSTROOT = os.getenv('INSTROOT')   
    if INSTROOT == None:
       INSTROOT  = ''
    try:
        session = dbm.DBM ('','',INSTROOT)
        INDEPPROGPATH = string.split(session.cmd('dbm_getpath INDEPPROGPATH'),'\n')[0]
        INDEPDATAPATH = string.split(session.cmd('dbm_getpath INDEPDATAPATH'),'\n')[0]
        if options.loaderinstroot == '':
           options.loaderinstroot  = INDEPPROGPATH
    except dbm.CommunicationError, err :
        print err
    except dbm.DBMServError, err:
        print err       
    #LOADER ENVIRONMENT PARAMETER
    if sys.platform[:3] == 'win':
       import _winreg
       try: 
          LOADERPROGPATH  = _winreg.QueryValueEx(_winreg.OpenKey(_winreg.HKEY_CURRENT_USER ,"Volatile Environment"),"APPDATA")[0] 
          LOADERDATAPATH  = _winreg.QueryValueEx(_winreg.OpenKey(_winreg.HKEY_CURRENT_USER ,"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders"),"Personal")[0]
       except EnvironmentError:
          print "Error reading Registry"
    if sys.platform[:3] != 'win':
       try:
          LOADERPROGPATH = os.getenv('HOME')
          LOADERDATAPATH = os.getenv('HOME')
       except:
          pass
    #TRANSPORT  PARAMETER   
    
    #LOADER COMMANDLINE PARAMETER
    options.transporttype               = string.upper(options.transporttype)
    options.transformationtype          = string.upper(options.transformationtype)
    transformationpackage               = options.transformationpackage
    if len(re.split(':',transformationpackage)) == 1:
       options.transformationpackage    = string.strip(re.split(':',transformationpackage)[0])
       if string.lower(options.transformationpackage) not in ['db','user','table']:
          options.packagename           = options.transformationpackage
    if len(re.split(':',transformationpackage)) == 2:
       options.transformationpackage    = string.strip(re.split(':',transformationpackage)[0])
       options.packagename              = string.strip(re.split(':',transformationpackage)[1])
    if string.lower(options.transformationpackage) in ['db','user','table']:
       options.transformationpackage    = string.upper(options.transformationpackage)
    options.transformationpart          = string.upper(options.transformationpart)
    options.streammedium                = string.upper(options.streammedium)
    options.streamformat                = string.upper(options.streamformat)
    options.streamformat_target         = string.upper(options.streamformat_target)
    options.streamcharencoding          = string.upper(options.streamcharencoding)
    options.streamlob                   = string.upper(options.streamlob)
    options.initposition                = string.upper(options.initposition)
    options.migration                   = string.upper(options.migration)
    #LOADER CONFIGURATION PARAMETER
    try:
       LOADERCONFIGURATION  = ConfigParser.ConfigParser()
       configfile = None
       if sys.platform[:3] == 'win':
          if(os.path.isfile(os.path.join (LOADERPROGPATH, 'sdb'   ,'loader','config','loader.cfg')) == False and
             os.path.isfile(os.path.join (INDEPPROGPATH , 'config','loader.cfg')) == True ):
             shutil.copy( os.path.join   (INDEPPROGPATH , 'config','loader.cfg'),
                          os.path.join   (LOADERPROGPATH, 'sdb'   ,'loader','config','loader.cfg'))
          LOADERCONFIGFILE = os.path.join(LOADERPROGPATH, 'sdb'   ,'loader','config','loader.cfg')
       if sys.platform[:3] != 'win':
          if(os.path.isfile(os.path.join (LOADERPROGPATH, '.sdb'  ,'loader','config','loader.cfg')) == False and
             os.path.isfile(os.path.join (INDEPPROGPATH , 'config','loader.cfg')) == True ):
             shutil.copy(os.path.join    (INDEPPROGPATH , 'config','loader.cfg'),
                         os.path.join    (LOADERPROGPATH, '.sdb'  ,'loader','config','loader.cfg'))
          LOADERCONFIGFILE = os.path.join(LOADERPROGPATH, '.sdb'  ,'loader','config','loader.cfg')
       configfile          = file(LOADERCONFIGFILE,'r')
       LOADERCONFIGURATION.readfp(configfile)
       # PACKAGE
       PACKAGETIMESTAMP      = time.strftime("%Y%m%d%H%M%S",time.localtime())
       PACKAGEPATH           = LOADERCONFIGURATION.get    ('PACKAGE','PACKAGEPATH')
       PACKAGENAME           = LOADERCONFIGURATION.get    ('PACKAGE','PACKAGENAME')
       PACKAGEARCHIVE        = LOADERCONFIGURATION.get    ('PACKAGE','PACKAGEARCHIVE')
       PACKAGEEXECUTION      = LOADERCONFIGURATION.get    ('PACKAGE','PACKAGEEXECUTION')
       PACKAGETIMEOUT        = LOADERCONFIGURATION.getint ('PACKAGE','TIMEOUT')
       VERBOSE               = LOADERCONFIGURATION.get    ('PACKAGE','VERBOSE')
       MONITOR               = LOADERCONFIGURATION.get    ('PACKAGE','MONITOR')
       # DATABASE
       SYSTEMATTRIBUTE       = LOADERCONFIGURATION.get    ('DATABASE_DESIGNTIME','SYSTEMATTRIBUTE')
       AUTOCOMMIT            = LOADERCONFIGURATION.get    ('DATABASE_CONNECTION','AUTOCOMMIT')
       ISOLATIONLEVEL        = LOADERCONFIGURATION.getint ('DATABASE_CONNECTION','ISOLATIONLEVEL')
       TRANSACTIONSIZE       = LOADERCONFIGURATION.getint ('DATABASE_RUNTIME'   ,'TRANSACTIONSIZE')
       # LOADER
       MAXERRORCOUNT         = LOADERCONFIGURATION.getint ('LOADER','MAXERRORCOUNT')
       TRANSFORMATIONSYNTAX  = LOADERCONFIGURATION.get    ('STREAM','TRANSFORMATIONSYNTAX')
       configfile.close()
       transportparameterfile = None
       if options.transformationtype not in ['']:
          options.transformationtype = string.upper(options.transformationtype)
       if options.sourcedbnode not in ['']:
          if string.find(string.upper(options.sourcedbnode),'LOCALHOST') == 0:
             options.sourcedbnode    = socket.gethostname()
          else:
             options.sourcedbnode    = options.sourcedbnode
       if options.sourcedbname not in ['']:
          options.sourcedbname       = options.sourcedbname
       if options.sourcedbuser not in ['']:
          options.sourcedbuser       = options.sourcedbuser
       if options.sourcedbschema not in ['']:
          options.sourcedbschema     = options.sourcedbschema
       if options.targetdbnode not in ['']:
          if string.find(string.upper(options.targetdbnode),'LOCALHOST') == 0:
             options.targetdbnode    = socket.gethostname()
          else:
             options.targetdbnode    = options.targetdbnode
       if options.targetdbname not in ['']:
          options.targetdbname       = options.targetdbname
       if options.targetdbuser not in ['']:
          options.targetdbuser       = options.targetdbuser 
       if options.targetdbschema not in ['']:
          options.targetdbschema     = options.targetdbschema
       if(options.transformationpackage not in ['','DB','USER','TABLE'] and options.packagename != '' ):
          # PACKAGECOLLECTION 
          packages          = []
          packagefile       = os.path.join(LOADERDATAPATH,'sdb', 'loader', 'packages', 'transport' + os.extsep + 'log')
          if os.path.isfile( packagefile ) :
             packagefilehandle = file(packagefile,'r')  
             packagecollection = csv.reader(packagefilehandle, delimiter  = '|'  )
             for package in packagecollection :
                 if((options.transformationpackage  == string.strip(package[1]) and 
                     'EXECUTE'                      == string.strip(package[2]) and
                     'FILE'                         == string.strip(package[7])) or
                    (options.transformationpackage  == string.strip(package[1]) and 
                     'GENERATE'                     == string.strip(package[2]) and
                     'IMPORT'                       != string.strip(package[3]))  ):
                    packages.append(package)
             # PACKAGE
             packagedict  = dict([(packages.index(package)+1,package) for package in packages])  
             if len(packagedict) >= 1:
                transformationpackage         = string.strip(packages[0][1] )
                if options.transporttype      in ['']:
                   options.transporttype      = 'EXECUTE'
                if options.transformationtype in ['']:
                   options.transformationtype = string.strip(packages[0][3] )
                options.transformationpart    = string.strip(packages[0][5] )
                options.initposition          = string.strip(packages[0][6] )
                options.streammedium          = string.strip(packages[0][7] )
                options.streamformat          = string.strip(packages[0][8] )
                options.streamformat_target   = string.strip(packages[0][9] )
                options.streamcharencoding    = string.strip(packages[0][10])
                options.streamlob             = string.strip(packages[0][11])
                PACKAGEARCHIVE                = string.strip(packages[0][12])
                if options.sourcedbnode in ['']:
                   options.sourcedbnode       = string.strip(packages[0][18])
                if options.sourcedbname in ['']:
                   options.sourcedbname       = string.strip(packages[0][19])
                if options.sourcedbuser in ['']:   
                   options.sourcedbuser       = string.strip(packages[0][20])
                if options.sourcedbschema in ['']:   
                   options.sourcedbschema     = string.strip(packages[0][21])   
                if options.targetdbnode in ['']:
                   options.targetdbnode       = string.strip(packages[0][22])
                if options.targetdbname in ['']:
                   options.targetdbname       = string.strip(packages[0][23])
                if options.targetdbuser in ['']:
                   options.targetdbuser       = string.strip(packages[0][24])
                if options.targetdbschema in ['']:
                   options.targetdbschema     = string.strip(packages[0][25])  
          else:
              print 'Transport historyfile %s not found' % os.path.split(packagefile)[1]
              rc = False
    except EnvironmentError, err:
       print 'Error using the package archive: \n %s ' % err
       kind, val, traceback = sys.exc_info()
       raise kind, val, traceback
    except csv.Error , err :
       print 'Error using the package archive: \n %s ' % err
    except IOError , (errno,err):
       print 'Error using the package archive: \n %s ' % err
    except ConfigParser.Error ,err:
       print 'Error using the package archive: \n %s ' % err
    except:
       kind, val, traceback = sys.exc_info()
       raise kind, val, traceback
       
    if rc == True:
       if options.streamlob == 'TABLE':
          FIELDLOBMASK       = ''
       if options.streamlob == 'COLUMN':
          FIELDLOBMASK       = ''
       if options.streamlob == 'COLUMNVALUE':
          FIELDLOBMASK       = '####'
       
       if options.transformationtype       == 'TRANSPORT':   
          if options.transporttype         == '':
             options.transporttype          = PACKAGEEXECUTION
          if options.sourcedbnode	       == ''   :
             options.sourcedbnode           = socket.gethostname()
          if options.targetdbnode	       == ''   :
             options.targetdbnode           = socket.gethostname()
          if options.transformationpackage == 'DB' :
             if TESTLOADER == 'FALSE':
                options.transformationpart  = 'DB'
          if(options.transformationpackage == 'USER' and 
             options.transformationpart    == '' ):
             options.transformationpart     = 'USER'
          if options.streammedium          == '':
             options.streammedium           = 'PIPE'
          if options.streamformat          == '':
             options.streamformat           = 'RECORDS'
          if options.streamformat          == 'PAGES':
             options.streamformat_target    = 'PAGE'
             options.streamlob              = 'TABLE'
          if options.streamformat          == 'RECORDS':
             options.streamformat_target    = 'ROW'
             options.streamlob              = 'TABLE'
          if options.initposition          == '' :
             options.initposition           = 'START' 
       if options.transformationtype       == 'EXPORT' :
          if options.transporttype         == '':
             options.transporttype          = PACKAGEEXECUTION
          if options.sourcedbnode	       == ''   :
             options.sourcedbnode           = '' #socket.gethostname()
          if options.transformationpackage == 'DB' :
             if TESTLOADER == 'FALSE':
                options.transformationpart  = 'DB'
          if(options.transformationpackage == 'USER' and 
             options.transformationpart    == '' ):
             options.transformationpart     = 'USER'
          if options.streammedium          == '':
             options.streammedium           = 'FILE'
          if options.streamformat          == '':
             options.streamformat           = 'RECORDS'
          if options.streamformat          == 'PAGES':
             options.streamformat_target    = 'PAGE'
             options.streamlob              = 'TABLE'
          if options.streamformat          == 'RECORDS':
             options.streamformat_target    = 'ROW'
             options.streamlob              = 'TABLE'
          if options.initposition          == '' :
             options.initposition           = 'START'
       if options.transformationtype       == 'IMPORT':
          if options.sourcedbnode	       == ''   :
             options.sourcedbnode           = '' #socket.gethostname()
          if options.targetdbnode	       == ''   :
             options.targetdbnode           = '' #socket.gethostname()
          
       #INTER
       if options.transformationtype       ==  '' :
          options.mode = 'INTER'
       #PARAM
       if(options.transformationtype       != '' and
          options.sourcedbnode	           != '' and
          options.sourcedbname	           != '' and
          options.sourcedbuser	           != '' and
          options.transformationpackage    != '' and
          options.transformationpart       != '' and
          options.streammedium             != '' and
          options.streamformat             != '' and
          options.streamformat_target      != '' and
          options.streamcharencoding       != '' and
          options.streamlob                != '' and
          options.initposition             != '' and
          options.mode                     != 'INTER'):
          options.mode = 'PARAM'
       #INTER OR PARAM
       if(options.transformationtype       == 'TRANSPORT' and (
          options.sourcedbname	           == '' or
          options.sourcedbuser	           == '' or
          options.targetdbname	           == '' or
          options.targetdbuser             == '' or
          options.mode                     == 'INTER')):
          options.mode = 'INTER'
       elif (options.transformationtype    == 'TRANSPORT'):
          options.mode = 'PARAM'
       if(options.transformationtype       == 'EXPORT' and (
          options.sourcedbname	           == '' or
          options.sourcedbuser	           == '' or
          options.mode                     == 'INTER')):
          options.mode = 'INTER'
       elif (options.transformationtype    == 'EXPORT'):
          options.mode = 'PARAM'
       if(options.transformationtype       == 'IMPORT' and (
          options.transformationpackage    == '' or
          options.targetdbname	           == '' or
          options.targetdbuser             == '' or
          options.mode                     == 'INTER')) :
          options.mode = 'INTER'
       elif (options.transformationtype    == 'IMPORT'):
          options.mode = 'PARAM'
       rc     = check_environment(options) 
       
    return rc
    
def check_environment(options):
    rc = True
    if( options.transformationpackage in ['DB'] and 
        options.transformationpart    in ['DB'] and
        options.streamformat          in ['COMPRESSED','FORMATTED']):
        rc = False
        print 'Format %s not supported for package %s and packagepart %s ' %(options.streamformat, options.transformationpackage, options.transformationpart )
    if( options.transformationpackage in ['USER'] and 
        options.transformationpart    in ['USER'] and
        options.streamformat          in ['COMPRESSED','FORMATTED']): 
        options.transformationpart    = 'TABLE'    
    return rc
       
def export_package(options):
    # Transformation Resources

    if options.streamformat_target != 'DTL':
       #print 'Stream           : %s ' % "CATALOG, PACKAGE, DATA "
       pass
    if options.streamformat_target == 'DTL':
       #print 'Stream           : %s ' % "CATALOG, PACKAGE "
       pass
    streamcharencoding = ''
    if options.streamcharencoding == '':
       streamcharencoding = 'DYNAMIC'
    if options.streamcharencoding != '':
       streamcharencoding = options.streamcharencoding
    sourceusername = ''
    sourcepassword = ''
    if options.sourcedbuser != '':
       sourceusername = re.split(',', options.sourcedbuser)[0]
       sourcepassword = re.split(',', options.sourcedbuser)[1]
    if sourceusername != '' and sourceusername [0] != '"' and sourceusername[len(sourceusername)-1] != '"':
       sourceusername = string.upper(sourceusername)
    elif sourceusername != '':
       sourceusername = string.replace(sourceusername,'"','')    
    targetusername = ''
    targetpassword = ''
    if options.targetdbuser != '':
       targetusername = re.split(',', options.targetdbuser)[0]
       targetpassword = re.split(',', options.targetdbuser)[1]
    if targetusername != '' and targetusername [0] != '"' and targetusername[len(targetusername)-1] != '"':
       targetusername = string.upper(targetusername)
    elif targetusername != '':
       targetusername = string.replace(targetusername,'"','') 

    print '\nTransformation package parameters'
    #print 'Transformation package path: %s ' % ''
    if options.transformationtype == 'TRANSPORT' or options.transformationtype == 'EXPORT':
       print ' Transformation package name : %s ' % os.path.split(options.sourceindepdatapath)[1]
    if options.transformationtype == 'IMPORT':
       print ' Transformation package name : %s ' % os.path.split(options.targetindepdatapath)[1]
    print ' Transformation package name : %s ' % options.packagename
    print ' Transport action        [-T]: %s ' % options.transporttype
    print ' Transport direction     [-t]: %s ' % options.transformationtype
    print ' Transformation package  [-p]: %s ' % options.transformationpackage
    print ' Package part            [-P]: %s ' % options.transformationpart
    print ' Stream  medium          [-m]: %s ' % options.streammedium
    print ' Stream  format          [-f]: %s ' % options.streamformat
    print ' Stream  format internal [-F]: %s ' % options.streamformat_target
    print ' Character encoding      [-c]: %s ' % options.streamcharencoding
    print ' Large Objects           [-l]: %s ' % options.streamlob 
    print '\nTransformation resource parameters'
    if options.transformationtype == 'TRANSPORT' or options.transformationtype == 'EXPORT':
       print ' Source database computer[-nSRC]: %s ' % options.sourcedbnode     
       print ' Source database name    [-dSRC]: %s ' % options.sourcedbname
       print ' Source database user    [-uSRC]: %s ' % sourceusername
       print ' Source database schema  [-sSRC]: %s ' % options.sourcedbschema
    if options.transformationtype == 'TRANSPORT' or options.transformationtype == 'IMPORT':  
       print ' Target database computer[-nTRG]: %s ' % options.targetdbnode     
       print ' Target database name    [-dTRG]: %s ' % options.targetdbname
       print ' Target database user    [-uTRG]: %s ' % targetusername
       print ' Target database schema  [-sTRG]: %s ' % options.targetdbschema
    print '\nTransformation configuration parameters'
    print ' PACKAGEEXECUTION        : %s ' % PACKAGEEXECUTION
    print ' PACKAGEARCHIVE          : %s ' % PACKAGEARCHIVE
    print ' VERBOSE                 : %s ' % VERBOSE
    print ' MONITOR                 : %s ' % MONITOR
    print ' MAXERRORCOUNT           : %s ' % MAXERRORCOUNT  
    
    
if __name__ == '__main__':
    optlib.optMain2 (main, _options ())
    
       
 
